package com.ibm.etools.beaninfo.adapters;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.eclipse.jdt.core.JavaCore;
/**
 * Entry of search path for beaninfo.
 * *package* because it is for old format and should not be used outside of this package.
 */

class BeaninfoSearchPathEntry {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected String fPackageName;	// Name of package that this entry references	

	/**
	 * packageName.
	 */	
	public BeaninfoSearchPathEntry(String packageName) {
		fPackageName = packageName;
	}
	
	public String getPackageName() {
		return fPackageName;
	}
		
	/**
	 * package protected so that only BeaninfoNature can call it. This is the
	 * old format which is no longer being used. It is here to allow conversion
	 * from Beta to GA.
	 */		 
	private static final String sSearchPathEntryElementName = "pathEntry"; //$NON-NLS-1$
	private static final String sPackageElementName = "package";	 //$NON-NLS-1$
	static BeaninfoSearchPathEntry readOldEntry(Node node) {
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			Element beElement= (Element) node;
			if (beElement.getNodeName().equalsIgnoreCase(sSearchPathEntryElementName)) {
				String pkgName = beElement.getAttribute(sPackageElementName);
				return new BeaninfoSearchPathEntry(pkgName);
			}
		}
		return null;		
	}
	
	public static BeaninfoSearchPathEntry readEntry(Node node) {
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			Element beElement= (Element) node;
			if (beElement.getNodeName().equalsIgnoreCase(sSearchPathEntryElementName)) {
				String pkgName = beElement.getAttribute(sPackageElementName);
				return new BeaninfoSearchPathEntry(pkgName);
			}
		}
		return null;
	}
	
	public Element writeEntry(Document doc) {
		Element entry = doc.createElement(sSearchPathEntryElementName);	// Create entry
		entry.setAttribute(sPackageElementName, getPackageName());	// Set the package name
		return entry;
	}	
		
	public boolean equals(Object other) {
		if (this == other)
			return true;
			
		if (!(other instanceof BeaninfoSearchPathEntry))
			return false;
			
		BeaninfoSearchPathEntry otherEntry = (BeaninfoSearchPathEntry) other;
		return (fPackageName.equals(otherEntry.fPackageName));
	}
}