package com.ibm.etools.beaninfo.adapters;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.beaninfo.BeaninfoPackage;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.resource.Context;
import com.ibm.etools.proxy.*;
import com.ibm.etools.java.beaninfo.IIntrospectionAdapter;

/**
 * Static initializer class to initialize the beaninfo stuff.
 * It is disconnected from the desktop and so doesn't require it.
 */

public class Init {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	// So it can't be instantiated.
	private Init() {};		
		
	/**
	 * Initialize the context with an IBeaninfoSupplier, used to set up the
	 * introspection process. 
	 *
	 * The beaninfo supplier is responsible for setting up the bean info search path,
	 * including removing the sun beaninfos, if desired.
	 */
	public static void initialize(Context context, final IBeaninfoSupplier supplier) {
		RefRegister.getPackage(BeaninfoPackage.packageURI);	// Make sure the beaninfo package is initialized
		context.addAdapterFactory(new BeaninfoAdapterFactory(supplier));
	}
	
	/**
	 * Initialize the registry now that it is available.
	 */
	public static void initialize(ProxyFactoryRegistry registry) {
		// Remove the "sun.beans.info" from the beaninfo search path because
		// we completely override the sun bean infos.
		Utilities.removeBeanInfoPath(registry, "sun.beans.infos"); //$NON-NLS-1$
	}
	
	/**
	 * Cleanup from the context because we are being removed.
	 * If clearResults is true, then the factory should clear the results of introspection
	 * from the everything because the context will still be around.
	 */
	public static void cleanup(Context context, boolean clearResults) {
		BeaninfoAdapterFactory factory = (BeaninfoAdapterFactory) context.getAdapterFactory(IIntrospectionAdapter.ADAPTER_KEY);
		context.removeAdapterFactory(IIntrospectionAdapter.ADAPTER_KEY);
		if (factory != null)
			factory.closeAll(clearResults);
	}
		
}