package com.ibm.etools.beaninfo.export;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.boot.IPlatformRunnable;
import org.eclipse.core.resources.IProject;

import com.ibm.etools.beaninfo.adapters.BeaninfoNature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.*;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.proxy.test.JavaProjectUtil;

/**
 * Export entire beaninfos to diff file format.
 * 
 * parms: root-directory-for-files project-name fully-qualified-classname ...
 * 
 * @version 	1.0
 * @author
 */
public class ExportBeaninfos implements IPlatformRunnable {

	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	/*
	 * @see IPlatformRunnable#run(Object)
	 */
	public Object run(Object args) throws Exception {
		String rootDir = null;
		String projectName = null;
		List classNames = new ArrayList();
		if (args instanceof String[]) {
			String[] argStrings = (String[]) args;
			for (int i = 0; i < argStrings.length; i++) {
				// -dev also comes through as an argument, if it does, then skip the next parm.
				if ("-dev".equalsIgnoreCase(argStrings[i])) { //$NON-NLS-1$
					i++;
					continue;
				}
				if (rootDir == null)
					rootDir = argStrings[i];
				else if (projectName == null)
					projectName = argStrings[i];
				else
					classNames.add(argStrings[i]);
			}

			if (rootDir == null) {
				System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Err.NoRoot_EXC_")); //$NON-NLS-1$
				return null;
			}
			if (projectName == null) {
				System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Err.NoProject_EXC_")); //$NON-NLS-1$
				return null;
			}
			if (classNames.isEmpty()) {
				System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Err.NoClassName_EXC_")); //$NON-NLS-1$
				return null;
			}
		}

		IProject project = JavaProjectUtil.getProject(projectName);
		if (project == null) {
			System.out.println(MessageFormat.format(BeanInfoExportMessages.getString("ExportBeanInfo.Err.ProjectNotFound_EXC_"), new Object[]{projectName})); //$NON-NLS-1$
			return null;
		}

		System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Processing_INFO_")); //$NON-NLS-1$
		BeaninfoNature nature = BeaninfoNature.getRuntime(project);
		ResourceSet rset = nature.getContext().getResourceSet();
		for (int i = 0; i < classNames.size(); i++) {
			JavaHelpers o = JavaClassImpl.reflect((String) classNames.get(i), rset);
			System.out.print(o.getQualifiedName()+": "); //$NON-NLS-1$
			if (o.isPrimitive()) 
				System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Err.IsAPrimitive_INFO_")); //$NON-NLS-1$
			else if (o.isArray())
				System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Err.IsAnArray_INFO_")); //$NON-NLS-1$
			else if (((JavaClass) o).getValueKind() == TypeKind.UNDEFINED)
				System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Err.DoesntExist_INFO_")); //$NON-NLS-1$
			else {
				ExportManager export = new ExportManager((JavaClass) o);
				export.forceAllOverride();
				export.createDiff(rootDir);
				System.out.println(BeanInfoExportMessages.getString("ExportBeanInfo.Finished_INFO_")); //$NON-NLS-1$
			}
		}

		return null;
	}

}