package com.ibm.etools.beaninfo.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.boot.IPlatformRunnable;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.*;

import com.ibm.etools.beaninfo.*;
import com.ibm.etools.beaninfo.adapters.*;
import com.ibm.etools.emf.ecore.EAttribute;
import com.ibm.etools.emf.ecore.EBehavioralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.*;
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.test.JavaProjectUtil;
;

/**
 * Do some tests of Java Beaninfo Modeling.
 *
 * This is only a test class to see if things are set up correctly.
 * Because this should be run only under the supervision or request
 * of an IBM tech support, it is not NLSed.
 */
public class TestClassBeaninfoModeling implements IPlatformRunnable {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected String version = null;
	protected int xtraAttributes = 0;	// If java instantiation is also installed, then there are extra attributes that it adds. Need to take into account.
	
	public Object run(Object p1) throws CoreException {

		boolean preTestUI = true;
		if (p1 instanceof String[]) {
			String[] args = (String[]) p1;
			for (int i = 0; i < args.length; i++)
				if (args[i].equalsIgnoreCase("noui")) { //$NON-NLS-1$
					preTestUI = false;
					break;
				}
		}

		final boolean testUI = preTestUI;
		System.out.println("-- Initializing the test data --"); //$NON-NLS-1$

		final IProject[] project =
			new IProject[] {
				JavaProjectUtil.getProject("Test Beaninfo"), //$NON-NLS-1$
				JavaProjectUtil.getProject("Test Beaninfo Beaninfos"), //$NON-NLS-1$
				JavaProjectUtil.getProject("Test Beaninfo Prereq")}; //$NON-NLS-1$
		try {
			final IWorkspace workspace = ResourcesPlugin.getWorkspace();
			IWorkspaceDescription description = workspace.getDescription();
			boolean autoBuild = description.isAutoBuilding();
			if (!autoBuild) {
				description.setAutoBuilding(true);
				workspace.setDescription(description);
			}

			try {
				// Do all within a runnable so that the build is done only after all changes have been made.
				ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
					public void run(IProgressMonitor monitor) throws CoreException {
						// First delete the project.
						for (int i = 0; i < project.length; i++) {
							if (project[i] != null)
								project[i].delete(true, true, monitor); // Get rid of it, we want a clean one for testing.
						}

						try {
							// Now import the requested files and create the projects.
							IPath projectPathBeaninfo = new Path("/Test Beaninfo");							
							if (testUI)
								if (!JavaProjectUtil.importZipURLs(
									projectPathBeaninfo,
									new URL[] {
										new URL(
											BeaninfoPlugin.getPlugin().getDescriptor().getInstallURL(),
											"testdata/testbeaninfo-src.jar"), //$NON-NLS-1$
										new URL(
											BeaninfoPlugin.getPlugin().getDescriptor().getInstallURL(),
											"testdata/testuibeaninfo-src.jar")}, //$NON-NLS-1$
									monitor))
									throw new RuntimeException("Couldn't import for some reason.");
							else
								if (!JavaProjectUtil.importZipURLs(
									projectPathBeaninfo,
									new URL[] {
										 new URL(
											BeaninfoPlugin.getPlugin().getDescriptor().getInstallURL(),
											"testdata/testbeaninfo-src.jar")}, //$NON-NLS-1$
									monitor))
									throw new RuntimeException("Couldn't import for some reason.");									
							
							if (!JavaProjectUtil.importZipURLs(
								new Path("/Test Beaninfo Beaninfos"),
								new URL[] {
									 new URL(
										BeaninfoPlugin.getPlugin().getDescriptor().getInstallURL(),
										"testdata/testbeaninfoproject-src.jar")}, //$NON-NLS-1$
								monitor))
								throw new RuntimeException("Couldn't import for some reason.");								
								
							if (!JavaProjectUtil.importZipURLs(
								new Path("/Test Beaninfo Prereq"),
								new URL[] {
									 new URL(
										BeaninfoPlugin.getPlugin().getDescriptor().getInstallURL(),
										"testdata/testbeaninfopreq-src.jar")}, //$NON-NLS-1$
								monitor))
								throw new RuntimeException("Couldn't import for some reason.");								

						} catch (MalformedURLException e) {
						}
						
						// Now get the new projects.
						project[0] = JavaProjectUtil.getProject("Test Beaninfo"); //$NON-NLS-1$
						project[1] = JavaProjectUtil.getProject("Test Beaninfo Beaninfos"); //$NON-NLS-1$
						project[2] = JavaProjectUtil.getProject("Test Beaninfo Prereq"); //$NON-NLS-1$						
					}
				}, null);
			} finally {
				if (!autoBuild) {
					description.setAutoBuilding(autoBuild);
					workspace.setDescription(description);
				}
			}

			boolean hadMarkers = false;
			for (int i = 0; i < project.length; i++) {
				if (project[i] == null)
					continue;
				IMarker[] markers = project[i].findMarkers(null, true, IResource.DEPTH_INFINITE);
				if (markers.length > 0) {
					hadMarkers = true;
					System.out.println("---There were errors compiling the test code for project \"" + project[i].getName()+"\"---"); //$NON-NLS-1$ //$NON-NLS-2$
					int max = Math.min(markers.length, 10);
					for (int i1 = 0; i1 < max; i1++) {
						IMarker iMarker = markers[i1];
						System.out.println(
							"  *" //$NON-NLS-1$
								+ iMarker.getType()
								+ ":" //$NON-NLS-1$
								+ iMarker.getAttribute(IMarker.MESSAGE)
								+ "/" //$NON-NLS-1$
								+ iMarker.getAttribute(IMarker.LOCATION));
					}
					if (markers.length != max)
						System.out.println(" ... And there were more..."); //$NON-NLS-1$
				}
			}
			
			if (hadMarkers)
				return null;

			System.out.println("-- Data initialized --"); //$NON-NLS-1$

			BeaninfoNature nature = BeaninfoNature.getRuntime(project[0]);
			// This will initialize the nature, i.e. connect with the java modeling and start the remote vm.
			// Fill in the required paths
			// This path is one that is in the same project.
			nature.setSearchPath(
				new BeaninfosDoc(
					new IBeaninfosDocEntry[] {
						new SearchpathEntry(
							IClasspathEntry.CPE_SOURCE,
							new Path(""), //$NON-NLS-1$
							"com.ibm.etools.beaninfo.testing.infos"), //$NON-NLS-1$
						new BeaninfoEntry(
							BeaninfoEntry.createEntry(
								IClasspathEntry.CPE_SOURCE,
								new Path("/Test Beaninfo Beaninfos"), //$NON-NLS-1$
								project[0],
								true),
							new SearchpathEntry[] { new SearchpathEntry("com.ibm.etools.beaninfo.testing.infos.project")}, //$NON-NLS-1$
							true),
						}));

			if (nature.getRegistry() != null) {
				// See if java instantiation is installed. This affects the attribute counts.
				if (Platform.getPluginRegistry().getPluginDescriptor("com.ibm.etools.java.instantiation") != null) //$NON-NLS-1$
					xtraAttributes = 4;	// It adds four extra attributes to Object.
				
				ProxyFactoryRegistry registry = nature.getRegistry();
				IBeanTypeProxy system = registry.getBeanTypeProxyFactory().getBeanTypeProxy("java.lang.System"); //$NON-NLS-1$
				IMethodProxy props = system.getMethodProxy("getProperty", "java.lang.String"); //$NON-NLS-1$ //$NON-NLS-2$
				IStringBeanProxy vers =
					(IStringBeanProxy) props.invokeCatchThrowableExceptions(
						null,
						registry.getBeanProxyFactory().createBeanProxyWith("java.version")); //$NON-NLS-1$
				version = vers.stringValue();
				test(nature);
				if (testUI)
					testUI(nature);
				testChanges(nature);
			} else
				System.out.println("** No remote vm was created for some reason. **"); //$NON-NLS-1$
		} catch (Exception e) {
			e.printStackTrace();	// Finally screws up the stack trace often.
		}
		
		for (int i = 0; i < project.length; i++) {
			if (project[i] != null)
				project[i].delete(true, false, null); // Get rid of the project and the files themselves.
		}
		ResourcesPlugin.getWorkspace().save(true, null); // Save the current state of the workspace.

		return null;

	}

	/**
	 * Starts the application.
	 * @param args an array of command-line arguments
	 */
	public void test(BeaninfoNature nature) {
		System.out.println("-------- Starting Standard Beaninfo Modeling Test --------"); //$NON-NLS-1$

		ResourceSet rset = nature.getContext().getResourceSet();
		ProxyFactoryRegistry registry = nature.getRegistry();

		// Test one without a beaninfo. This will do reflection.
		JavaClass test1Class =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class"); //$NON-NLS-1$

		// Test that the reflection key can find features that aren't yet loaded.
		Object foo = rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class/foo"); //$NON-NLS-1$
		if (foo == null)
			System.out.println("Failed 0.5. foo feature was not found."); //$NON-NLS-1$

		// Test that the reflection key can find behaviors that aren't yet loaded.
		Object getFoo =
			rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class/behavior/getFoo"); //$NON-NLS-1$
		if (foo == null)
			System.out.println("Failed 0.6. getFoo behavior was not found."); //$NON-NLS-1$

		BeanDecorator bd = Utilities.getBeanDecorator(test1Class);
		if (!"Test1Class".equals(bd.getName())) //$NON-NLS-1$
			System.out.println("Failed 1. Name not \"Test1Class\": " + bd.getName()); //$NON-NLS-1$
		if (bd.getCustomizerClass() != null)
			System.out.println(
				"Failed 1.1. CustomizerClass should be null: " + bd.getCustomizerClass().getQualifiedName()); //$NON-NLS-1$
		if (test1Class.getEAttributes().size() != 2)
			System.out.println(
				"Failed 1.2. Attributes size should be 2: " + test1Class.getEAttributes().size()); //$NON-NLS-1$
		Iterator itr = Utilities.getPropertiesIterator(test1Class.getEAttributes());
		boolean found = false;
		while (itr.hasNext()) {
			PropertyDecorator pd = (PropertyDecorator) itr.next();
			if (pd.getName().equals("foo")) { //$NON-NLS-1$
				found = true;
				if (pd instanceof IndexedPropertyDecorator) {
					IndexedPropertyDecorator ip = (IndexedPropertyDecorator) pd;
					JavaHelpers type = (JavaHelpers) ip.getPropertyType();
					if (!type.getQualifiedName().equals("java.lang.String[]")) //$NON-NLS-1$
						System.out.println("Failed 1.3: Type is not String[]: " + type.getQualifiedName()); //$NON-NLS-1$
					Method mthd = ip.getReadMethod();
					if (mthd != rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class.getFoo(")) //$NON-NLS-1$
						System.out.println(
							"Failed 1.4. ReadMethod should be \"getFoo()\": " + (mthd != null ? mthd.getName() : "null")); //$NON-NLS-1$ //$NON-NLS-2$
					mthd = ip.getWriteMethod();
					if (mthd
						!= rset.getObjectAndLoad(
							"java:/com.ibm.etools.beaninfo.testing#Test1Class.setFoo(java.lang.String[]")) //$NON-NLS-1$
						System.out.println(
							"Failed 1.5. WriteMethod should be \"setFoo(String[])\": " //$NON-NLS-1$
								+ (mthd != null ? mthd.getName() : "null")); //$NON-NLS-1$
					mthd = ip.getIndexedReadMethod();
					if (mthd != rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class.getFoo(int")) //$NON-NLS-1$
						System.out.println(
							"Failed 1.6. Indexed ReadMethod should be \"getFoo(int)\": " //$NON-NLS-1$
								+ (mthd != null ? mthd.getName() : "null")); //$NON-NLS-1$
					mthd = ip.getIndexedWriteMethod();
					if (mthd
						!= rset.getObjectAndLoad(
							"java:/com.ibm.etools.beaninfo.testing#Test1Class.setFoo(int,java.lang.String")) //$NON-NLS-1$
						System.out.println(
							"Failed 1.6. Indexed ReadMethod should be \"setFoo(int,String)\": " //$NON-NLS-1$
								+ (mthd != null ? mthd.getName() : "null")); //$NON-NLS-1$
				} else
					System.out.println("Failed 1.9: Should of been an indexed property."); //$NON-NLS-1$
				break;
			}
		}
		if (!found)
			System.out.println("Failed 1.10: Property foo not found."); //$NON-NLS-1$

		if (test1Class.getEBehaviors().size() != 10)
			System.out.println("Failed 1.11. Behaviors size should be 10: " + test1Class.getEBehaviors().size()); //$NON-NLS-1$
		itr = test1Class.getEBehaviors().iterator();
		Method desired =
			(Method) rset.getObjectAndLoad(
				"java:/com.ibm.etools.beaninfo.testing#Test1Class.setFoo(int,java.lang.String"); //$NON-NLS-1$
		found = false;
		while (itr.hasNext()) {
			MethodProxy bhav = (MethodProxy) itr.next();
			if (bhav.getMethod() == desired) {
				found = true;
				List parms = Utilities.getMethodDecorator(bhav).getParameterDescriptors();
				if (parms.size() != 2)
					System.out.println("Failed 1.12: Number of parms should be 2: " + parms.size()); //$NON-NLS-1$
				else {
					ParameterDecorator pmd = (ParameterDecorator) parms.get(0);
					if (!pmd.getName().equals("index")) //$NON-NLS-1$
						System.out.println("Failed 1.13: Parm name should be \"index\": " + pmd.getName()); //$NON-NLS-1$
				}
				break;
			}
		}
		if (!found)
			System.out.println("Failed 1.14: Behavior setFoo not found."); //$NON-NLS-1$
			
		// Test event set reflected.
		List events = test1Class.getEvents();
		if (events.size() != 2)
			System.out.println("Failed 1.15: Events size should be 2: " + events.size()); //$NON-NLS-1$
		itr = events.iterator();
		boolean foundevt1 = false, foundevt2 = false;
		while (itr.hasNext()) {
			JavaEvent event = (JavaEvent) itr.next();
			if ("test1ClassEvent".equals(event.getName())) { //$NON-NLS-1$
				foundevt1 = true;
				EventSetDecorator edec = Utilities.getEventSetDecorator(event);
				List mlist = edec.getListenerMethods();
				if (mlist.size() != 2)
					System.out.println("Failed 1.16: ListenerMethods size should be 2: " + mlist.size()); //$NON-NLS-1$
			} else if ("test1ClassUnicastEvent".equals(event.getName())) { //$NON-NLS-1$
				foundevt2 = true;
				EventSetDecorator edec = Utilities.getEventSetDecorator(event);
				if (!edec.isUnicast())
					System.out.println("Failed 1.17: test1ClassUnicastEvent should be set to isUnicast."); //$NON-NLS-1$
			}
		}
		if (!foundevt1)
			System.out.println("Failed 1.18: Did not find event test1ClassEvent"); //$NON-NLS-1$
		if (!foundevt2)
			System.out.println("Failed 1.18: Did not find event test1ClassUnicastEvent"); //$NON-NLS-1$


		// Test one with a beaninfo.
		test1Class = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test2ClassB"); //$NON-NLS-1$
		bd = Utilities.getBeanDecorator(test1Class);
		if (!"Test2ClassB".equals(bd.getName())) //$NON-NLS-1$
			System.out.println("Failed 2. Name not \"Test2ClassB\": " + bd.getName()); //$NON-NLS-1$
		// The following will test if search across beaninfo search path worked. It would fail if it didn't work.
		if (!"A name for this class.".equals(bd.getDisplayName())) //$NON-NLS-1$
			System.out.println(
				"Failed 2.1. DisplayName not \"A name for this class.\": " + bd.getDisplayName()); //$NON-NLS-1$
		if (bd.isExpert())
			System.out.println("Failed 2.2. IsExpert should be false."); //$NON-NLS-1$
		if (bd.getCustomizerClass() != test1Class)
			System.out.println(
				"Failed 2.3. CustomizerClass should be Test2ClassB: " + bd.getCustomizerClass().getQualifiedName()); //$NON-NLS-1$
		if (test1Class.getEAttributes().size() != 2)
			System.out.println(
				"Failed 2.4. Attributes size should be 2: " + test1Class.getEAttributes().size()); //$NON-NLS-1$
		itr = Utilities.getPropertiesIterator(test1Class.getEAttributes());
		boolean gotSet = false, gotFoo = false;
		while (itr.hasNext()) {
			PropertyDecorator pd = (PropertyDecorator) itr.next();
			Method mthd = null;
			if (pd.getName().equals("setA")) { //$NON-NLS-1$
				gotSet = true;
				mthd = pd.getReadMethod();
				if (mthd != rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test2ClassB.isSetA(")) //$NON-NLS-1$
					System.out.println("Failed 2.6. ReadMethod should be \"isSetA\": " + mthd.getName()); //$NON-NLS-1$
				if (!pd.isExpert())
					System.out.println("Failed 2.7. isExpert should be true."); //$NON-NLS-1$
			} else if (pd.getName().equals("foo")) { //$NON-NLS-1$
				gotFoo = true;
				IndexedPropertyDecorator ipd = (IndexedPropertyDecorator) pd;
				mthd = ipd.getIndexedReadMethod();
				if (mthd != rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test2ClassB.getFoo(int")) //$NON-NLS-1$
					System.out.println("Failed 2.8. ReadMethod should be \"getFoo(int)\": " + mthd.getName()); //$NON-NLS-1$
				if (test1Class.getEBehaviors().size() != 2)
					System.out.println("Failed 2.9. Behaviors size should be 2: " + test1Class.getEBehaviors().size()); //$NON-NLS-1$
			}
		}
		if (!gotSet)
			System.out.println("Failed 2.1. Didn't find \"setA\" property."); //$NON-NLS-1$
		if (!gotFoo)
			System.out.println("Failed 2.1a. Didn't find \"foo\" property."); //$NON-NLS-1$

		found = false;
		itr = test1Class.getEBehaviors().iterator();
		while (itr.hasNext()) {
			EBehavioralFeature bhav = (EBehavioralFeature) itr.next();
			if (bhav.getName().equals("setSetA")) { //$NON-NLS-1$
				List parms = Utilities.getMethodDecorator(bhav).getParameterDescriptors();
				if (parms.size() != 1)
					System.out.println("Failed 2.10: Number of parms should be 1: " + parms.size()); //$NON-NLS-1$
				else {
					ParameterDecorator pmd = (ParameterDecorator) parms.get(0);
					if (!pmd.getName().equals("aBooleanSetting")) //$NON-NLS-1$
						System.out.println("Failed 2.11: Parm name should be \"aBooleanSetting\": " + pmd.getName()); //$NON-NLS-1$
				}
				found = true;
				break;
			}
		}
		if (!found)
			System.out.println("Failed 2.12. The behavior \"setSetA\" not found."); //$NON-NLS-1$
			
		// Test event set introspected.
		events = test1Class.getEvents();
		if (events.size() != 1)
			System.out.println("Failed 2.13: Events size should be 1: " + events.size()); //$NON-NLS-1$
		else {
			JavaEvent event = (JavaEvent) events.get(0);
			if ("overrideName".equals(event.getName())) { //$NON-NLS-1$
				EventSetDecorator edec = Utilities.getEventSetDecorator(event);
				List mlist = edec.getListenerMethods();
				if (mlist.size() != 1)
					System.out.println("Failed 2.14: ListenerMethods size should be 1: " + mlist.size()); //$NON-NLS-1$
				else {
					MethodProxy mproxy = (MethodProxy) mlist.get(0);
					MethodDecorator mdec = Utilities.getMethodDecorator(mproxy);
					if (!"event1".equals(mdec.getName())) //$NON-NLS-1$
						System.out.println("Failed 2.15: Event method not named correctly. \""+mdec.getName()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
					if (!"Event 1".equals(mdec.getDisplayName())) //$NON-NLS-1$
						System.out.println("Failed 2.16: Event method display name not correct. \""+mdec.getDisplayName()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
				}
			} else 
				System.out.println("Failed 2.17: Event not named correctly. \""+event.getName()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
		}

		// Now do the testing of merging with super types.
		JavaClass testClass =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 3+xtraAttributes)
			System.out.println("Failed 3. All Attributes should be "+(3+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1ClassA"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 4+xtraAttributes)
			System.out.println(
				"Failed 3.1. All Attributes should be "+(4+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
				
		// Test merging supertypes event set.
		events = testClass.getAllEvents();
		if (events.size() != 2)
			System.out.println("Failed 3.1a: Events size should be 2: " + events.size()); //$NON-NLS-1$
		itr = events.iterator();
		foundevt1 = false;
		foundevt2 = false;
		while (itr.hasNext()) {
			JavaEvent event = (JavaEvent) itr.next();
			if ("test1ClassEvent".equals(event.getName())) { //$NON-NLS-1$
				foundevt1 = true;
				EventSetDecorator edec = Utilities.getEventSetDecorator(event);
				List mlist = edec.getListenerMethods();
				if (mlist.size() != 2)
					System.out.println("Failed 3.1b: ListenerMethods size should be 2: " + mlist.size()); //$NON-NLS-1$
			} else if ("test1ClassUnicastEvent".equals(event.getName())) { //$NON-NLS-1$
				foundevt2 = true;
				EventSetDecorator edec = Utilities.getEventSetDecorator(event);
				if (!edec.isUnicast())
					System.out.println("Failed 3.1c: test1ClassUnicastEvent should be set to isUnicast."); //$NON-NLS-1$
			}
		}
		if (!foundevt1)
			System.out.println("Failed 3.1d: Did not find event test1ClassEvent"); //$NON-NLS-1$
		if (!foundevt2)
			System.out.println("Failed 3.1e: Did not find event test1ClassUnicastEvent");				 //$NON-NLS-1$
			
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test2Class"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 1+xtraAttributes)
			System.out.println(
				"Failed 3.2. All Attributes should be "+(1+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test2ClassA"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 2+xtraAttributes)
			System.out.println(
				"Failed 3.3. All Attributes should be "+(2+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test2ClassB"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 3+xtraAttributes)
			System.out.println(
				"Failed 3.4. All Attributes should be "+(3+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		// This will test that we could find Test2ClassC beaninfo in other project.
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test2ClassC"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 3+xtraAttributes)
			System.out.println(
				"Failed 3.5. All Attributes should be "+(3+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
				
		// Test finding beaninfos for classes in another project (pre-reqed project).
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing.prereq#Test2ClassPreq"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 1+xtraAttributes)
			System.out.println(
				"Failed 3.6. All Attributes should be "+(1+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		// Test removing a property directly from a super class causes the subclasses to re-build their eAllAttributes.
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class"); //$NON-NLS-1$
		testClass.getEAttributes().remove(0); // Remove the first one.
		if (testClass.getEAllAttributes().size() != 2+xtraAttributes)
			System.out.println("Failed 4. All Attributes should be "+(2+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1ClassA"); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 3+xtraAttributes)
			System.out.println(
				"Failed 4.1. All Attributes should be "+(3+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		// Test removing an event directly from a super class causes the subclasses to re-build their allEvents.
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class"); //$NON-NLS-1$
		testClass.getEvents().remove(0); // Remove the first one.
		if (testClass.getEvents().size() != 1)
			System.out.println("Failed 5. All Events should be "+(1)+": " + testClass.getEvents().size()); //$NON-NLS-1$ //$NON-NLS-2$
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1ClassA"); //$NON-NLS-1$
		if (testClass.getAllEvents().size() != 1)
			System.out.println(
				"Failed 5.1. All Events should be "+(1)+": " + testClass.getAllEvents().size()); //$NON-NLS-1$ //$NON-NLS-2$
				
		// Now test customer to make sure that it introspects correctly, plus the inner class of it does too.
		testClass = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Customer"); //$NON-NLS-1$
		if (testClass.getEAttributes().size() != 2)
			System.out.println("Failed 6. Attributes should be "+(2)+": " + testClass.getEAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$	
		else {
			itr = testClass.getEAttributes().iterator();
			found = false;
			while (itr.hasNext()) {
				EAttribute ea = (EAttribute) itr.next();
				if ("savings".equals(ea.getName())) {
					found = true;
					JavaClass innerClass = (JavaClass) ea.getETypeClassifier();
					if (innerClass.getEAttributes().size() != 2)
						System.out.println("Failed 6.1. Attributes should be "+(2)+": " + innerClass.getEAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$						
				}
			}
			
			if (!found)
				System.out.println("Failed 6.2. Did not find \"savings\" property"); //$NON-NLS-1$
		}

		System.out.println("-------- Ending the Standard Beaninfo Modeling Test --------"); //$NON-NLS-1$

	}

	private void testUI(BeaninfoNature nature) throws CoreException {

		ResourceSet rset = nature.getContext().getResourceSet();
		ProxyFactoryRegistry registry = nature.getRegistry();

		// Now let's do a big test of AWT/Swing type.	
		Utilities.insertBeanInfoSearchPath(
			nature,
			new SearchpathEntry(
				IClasspathEntry.CPE_VARIABLE,
				new Path("JRE_LIB"), //$NON-NLS-1$
				"com.ibm.etools.beaninfo.testing.java.awt"), //$NON-NLS-1$
			-1);
		Utilities.insertBeanInfoSearchPath(
			nature,
			new SearchpathEntry(
				IClasspathEntry.CPE_VARIABLE,
				new Path("JRE_LIB"), //$NON-NLS-1$
				"com.ibm.etools.beaninfo.testing.javax.swing"), //$NON-NLS-1$
			-1);
			try {
				// This guy has awt.Button beaninfo in it. Test case where beaninfos are in external libraries.
				Utilities.insertBeanInfoSearchPath(
					nature,
					new BeaninfoEntry(
						BeaninfoEntry.createEntry(
							IClasspathEntry.CPE_LIBRARY,
							new Path(
								Platform
									.asLocalURL(
										new URL(
											BeaninfoPlugin.getPlugin().getDescriptor().getInstallURL(),
											"testdata/testbeaninfoui1.jar")) //$NON-NLS-1$
									.getFile()),
							nature.getProject(),
							true),
						new SearchpathEntry[] { new SearchpathEntry("com.ibm.etools.beaninfo.testing.java.awt"), }, //$NON-NLS-1$
						true),
					-1);
			} catch (MalformedURLException e) {
			} catch (IOException e) {
			}
		System.out.println("-------- Starting the UserInterface (AWT/Swing)Test. --------"); //$NON-NLS-1$

		JavaClass testClass = (JavaClass) rset.getObjectAndLoad("java:/java.awt#Button"); //$NON-NLS-1$
		JavaClass loopClass = testClass;
		while (loopClass != null) {
			System.out.println("Reflecting for " + loopClass.getQualifiedName()); //$NON-NLS-1$
			loopClass.getIsFinal(); // This causes reflection.
			loopClass = loopClass.getSupertype();
		}
		System.out.println("About to get all."); //$NON-NLS-1$
		// This tests finding beaninfo in external jar.
		if (testClass.getEAllAttributes().size() != 28+xtraAttributes)
			System.out.println(
				"Failed 1. All Attributes should be "+(28+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
		System.out.println("Got all attributes of Button."); //$NON-NLS-1$
		if (testClass.getEBehaviors().size() != 4)
			System.out.println("Failed 1.2. Behaviors should be 4: " + testClass.getEAllAttributes().size()); //$NON-NLS-1$
		testClass = (JavaClass) rset.getObjectAndLoad("java:/javax.swing#JLabel"); //$NON-NLS-1$
		loopClass = testClass;
		while (loopClass != null) {
			System.out.println("Reflecting for " + loopClass.getQualifiedName()); //$NON-NLS-1$
			loopClass.getIsFinal(); // This causes reflection.
			loopClass = loopClass.getSupertype();
		}
		System.out.println("About to get all."); //$NON-NLS-1$
		if (testClass.getEAllAttributes().size() != 60+xtraAttributes)
			System.out.println(
				"Failed 1.3. All Attributes should be "+(60+xtraAttributes)+": " + testClass.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$
		System.out.println("Got all attributes of JLabel."); //$NON-NLS-1$

		// Testing getting all behaviors
		int numBehav = version.startsWith("1.4") ? 174 : 173; // Pre-1.4 has only 173. //$NON-NLS-1$
		if (testClass.getEAllBehaviors().size() != numBehav)
			System.out.println(
				"Failed 2. All Behaviors should be " + numBehav + ": " + testClass.getEAllBehaviors().size()); //$NON-NLS-1$ //$NON-NLS-2$
		System.out.println("Got all behaviors of JLabel."); //$NON-NLS-1$

		System.out.println("-------- Ending the UserInterface (AWT/Swing)Test. --------"); //$NON-NLS-1$
	}

	private void testChanges(BeaninfoNature nature) throws CoreException {
		ResourceSet rset = nature.getContext().getResourceSet();

		IProject project = JavaProjectUtil.getProject("Test Beaninfo"); //$NON-NLS-1$
		final IJavaProject jProject =
			JavaCore.create(project.getWorkspace().getRoot()).getJavaProject(project.getName());

		// This will test that changes to classes will cause the nature to recycle and we can get
		// the update changes.
		System.out.println("-------- Starting the Class Changes Test. --------"); //$NON-NLS-1$

		JavaClass testChangesClass =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#TestChangesClass"); //$NON-NLS-1$
		BeanDecorator bd = Utilities.getBeanDecorator(testChangesClass);
		if (!"TestChangesClass".equals(bd.getName())) //$NON-NLS-1$
			System.out.println("Failed 1. Name not \"TestChangesClass\": " + bd.getName()); //$NON-NLS-1$
		if (testChangesClass.getEAttributes().size() != 1)
			System.out.println(
				"Failed 1.2. Attributes size should be 1: " + testChangesClass.getEAttributes().size()); //$NON-NLS-1$
		if (testChangesClass.getEBehaviors().size() != 2)
			System.out.println(
				"Failed 1.3. Behaviors size should be 2: " + testChangesClass.getEBehaviors().size()); //$NON-NLS-1$

		ProxyFactoryRegistry registry = nature.getRegistry(); // This is the registry before changes.		

		// First we'll add a method. This should then show up in the behaviors.
		// This will be against a class that has a beaninfo that returns null for methods so that reflection on the remote vm will be performed.
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				ICompilationUnit cu =
					(ICompilationUnit) jProject.findElement(
						new Path("com/ibm/etools/beaninfo/testing/TestChangesClass.java")); //$NON-NLS-1$
				IType type = cu.getType("TestChangesClass"); //$NON-NLS-1$
				type.createMethod("public boolean isNew() {}", null, true, monitor); //$NON-NLS-1$
			}
		}, null);

		// Redo the tests, except there should now be 3 behaviors.
		if (testChangesClass.getEBehaviors().size() != 3)
			System.out.println(
				"Failed 2.1. Behaviors size should be 3: " + testChangesClass.getEBehaviors().size()); //$NON-NLS-1$
		if (registry == nature.getRegistry())
			System.out.println("Failed 2.2. Did not get a new registry as we should of."); //$NON-NLS-1$


		registry = nature.getRegistry();

		// Now run the test against one that was already done once but make a change. Since it is already stale
		// from the above recycle it shouldn't recycle again.
		JavaClass test1Class =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class"); //$NON-NLS-1$
		if (test1Class.getEAttributes().size() != 2)
			System.out.println(
				"Failed 3.1. Attributes size should be 2: " + test1Class.getEAttributes().size()); //$NON-NLS-1$
		if (test1Class.getEBehaviors().size() != 10)
			System.out.println("Failed 3.2. Behaviors size should be 10: " + test1Class.getEBehaviors().size()); //$NON-NLS-1$
		if (registry != nature.getRegistry())
			System.out.println("Failed 3.3. Got a new registry. Should not of gotton a new registry"); //$NON-NLS-1$
		if (test1Class.getEAllAttributes().size() != 3+xtraAttributes)
			System.out.println(
				"Failed 3.4. All attributes size should be "+(3+xtraAttributes)+": " + test1Class.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		registry = nature.getRegistry();

		// First we'll add a two methods that form a property . This should then show up in the behaviors and properties.
		// This will be against a class that has no beaninfo so that reflection locally will be performed.
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				ICompilationUnit cu =
					(ICompilationUnit) jProject.findElement(
						new Path("com/ibm/etools/beaninfo/testing/Test1Class.java")); //$NON-NLS-1$
				IType type = cu.getType("Test1Class"); //$NON-NLS-1$
				type.createMethod("public boolean isNew() {}", null, true, monitor); //$NON-NLS-1$
				type.createMethod("public void setNew(boolean isNew) {}", null, true, monitor); //$NON-NLS-1$
			}
		}, null);

		// Redo the tests, except there should now be 8 behaviors and 3 properties.
		if (test1Class.getEAttributes().size() != 3)
			System.out.println(
				"Failed 4.1. Attributes size should be 3: " + test1Class.getEAttributes().size()); //$NON-NLS-1$
		if (test1Class.getEBehaviors().size() != 12)
			System.out.println("Failed 4.2. Behaviors size should be 12: " + test1Class.getEBehaviors().size()); //$NON-NLS-1$
		if (registry == nature.getRegistry())
			System.out.println("Failed 4.3. Did not get a new registry as we should of."); //$NON-NLS-1$
		if (test1Class.getEAllAttributes().size() != 4+xtraAttributes)
			System.out.println(
				"Failed 4.4. All attributes size should be "+(4+xtraAttributes)+": " + test1Class.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		JavaClass test1ClassA =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1ClassA"); //$NON-NLS-1$
		if (test1ClassA.getEAllAttributes().size() != 5+xtraAttributes)
			System.out.println(
				"Failed 4.5. All Attributes should be "+(5+xtraAttributes)+": " + test1ClassA.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		registry = nature.getRegistry();

		// Test deleting the methods and properties we just added.	
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				ICompilationUnit cu =
					(ICompilationUnit) jProject.findElement(
						new Path("com/ibm/etools/beaninfo/testing/Test1Class.java")); //$NON-NLS-1$
				IType type = cu.getType("Test1Class"); //$NON-NLS-1$
				IMethod[] methods = type.getMethods();
				type.getMethod("isNew", null).delete(true, monitor); //$NON-NLS-1$
				type.getMethod("setNew", new String[] { "Z" }).delete(true, monitor); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}, null);

		if (test1Class.getEAttributes().size() != 2)
			System.out.println(
				"Failed 5.1. Attributes size should be 2: " + test1Class.getEAttributes().size()); //$NON-NLS-1$
		if (test1Class.getEBehaviors().size() != 10)
			System.out.println("Failed 5.2. Behaviors size should be 10: " + test1Class.getEBehaviors().size()); //$NON-NLS-1$
		if (registry == nature.getRegistry())
			System.out.println("Failed 5.3. Did not get a new registry as we should of."); //$NON-NLS-1$
		if (test1Class.getEAllAttributes().size() != 3+xtraAttributes)
			System.out.println(
				"Failed 5.4. All attributes size should be "+(3+xtraAttributes)+": " + test1Class.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		// Test that changing a super class (e.g. removing a property but not doing the introspection on it, will 
		// cause subclass to reget the super all property).
		if (test1ClassA.getEAllAttributes().size() != 4+xtraAttributes)
			System.out.println(
				"Failed 6.1. All Attributes should be "+(4+xtraAttributes)+": " + test1ClassA.getEAllAttributes().size()); //$NON-NLS-1$ //$NON-NLS-2$

		// Test deleting the removeMethod for a reflected event causes appropriate action.
		test1Class.getAllEvents();	// Just to make sure it is filled in before we go on.
		test1ClassA.getAllEvents();
		registry = nature.getRegistry();			
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				ICompilationUnit cu =
					(ICompilationUnit) jProject.findElement(
						new Path("com/ibm/etools/beaninfo/testing/Test1Class.java")); //$NON-NLS-1$
				IType type = cu.getType("Test1Class"); //$NON-NLS-1$
				IMethod[] methods = type.getMethods();
				type.getMethod("addTest1ClassEventListener", new String[] {"QITest1ClassEventListener;"}).delete(true, monitor); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}, null);
		
		if (test1Class.getEvents().size() != 1)
			System.out.println(
				"Failed 7.1. Events size should be 1: " + test1Class.getEvents().size()); //$NON-NLS-1$
		if (registry == nature.getRegistry())
			System.out.println("Failed 7.2. Did not get a new registry as we should of."); //$NON-NLS-1$
				
		// Test that changing a super class (e.g. removing an event but not doing the introspection on it, will 
		// cause subclass to reget the super all events).
		if (test1ClassA.getAllEvents().size() != 1)
			System.out.println(
				"Failed 7.3. All Events should be "+(1)+": " + test1ClassA.getAllEvents().size()); //$NON-NLS-1$ //$NON-NLS-2$

		// Test removing a class.
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				ICompilationUnit cu =
					(ICompilationUnit) jProject.findElement(
						new Path("com/ibm/etools/beaninfo/testing/Test1ClassA.java")); //$NON-NLS-1$
				cu.delete(true, monitor);
			}
		}, null);

		if (test1ClassA.getValueKind() != TypeKind.UNDEFINED)
			System.out.println("Failed 8.1. Test1ClassA was not marked as undefined."); //$NON-NLS-1$
			
		// Test changing a class in a pre-req package will cause the reqistry to recycle.
		JavaClass testChangesClassPreq =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing.prereq#TestChangesClassPreq"); //$NON-NLS-1$
		testChangesClassPreq.getEBehaviors();
		
		registry = nature.getRegistry();		
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				ICompilationUnit cu =
					(ICompilationUnit) jProject.findElement(
						new Path("com/ibm/etools/beaninfo/testing/prereq/TestChangesClassPreq.java")); //$NON-NLS-1$
				IType type = cu.getType("TestChangesClassPreq"); //$NON-NLS-1$
				type.createMethod("public boolean isNew() {}", null, true, monitor); //$NON-NLS-1$
			}
		}, null);

		// Redo the tests, except there should now be 1 behavior.
		if (testChangesClassPreq.getEBehaviors().size() != 1)
			System.out.println(
				"Failed 9.1. Behaviors size should be 1: " + testChangesClassPreq.getEBehaviors().size()); //$NON-NLS-1$
		if (registry == nature.getRegistry())
			System.out.println("Failed 9.2. Did not get a new registry as we should of."); //$NON-NLS-1$


		// Run test changes on inner classes to make sure they work right.
		JavaClass testChangesClassInner =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#TestChangesClass$InnerTestChangesClass"); //$NON-NLS-1$
		bd = Utilities.getBeanDecorator(testChangesClassInner);
		if (!"TestChangesClass$InnerTestChangesClass".equals(bd.getName())) //$NON-NLS-1$
			System.out.println("Failed 10.1 Name not \"TestChangesClass$InnerTestChangesClass\": " + bd.getName()); //$NON-NLS-1$
		if (testChangesClassInner.getEAttributes().size() != 1)
			System.out.println(
				"Failed 10.2. Attributes size should be 1: " + testChangesClassInner.getEAttributes().size()); //$NON-NLS-1$
		if (testChangesClassInner.getEBehaviors().size() != 2)
			System.out.println(
				"Failed 10.3. Behaviors size should be 2: " + testChangesClassInner.getEBehaviors().size()); //$NON-NLS-1$

		registry = nature.getRegistry(); // This is the registry before changes.		

		// First we'll add a method. This should then show up in the behaviors.
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				IType type =
					jProject.findType("com.ibm.etools.beaninfo.testing.TestChangesClass.InnerTestChangesClass"); //$NON-NLS-1$
				type.createMethod("public boolean isNew() {}", null, true, monitor); //$NON-NLS-1$
			}
		}, null);

		// Redo the tests, except there should now be 3 behaviors.
		if (testChangesClassInner.getEBehaviors().size() != 3)
			System.out.println(
				"Failed 11.1. Behaviors size should be 3: " + testChangesClassInner.getEBehaviors().size()); //$NON-NLS-1$
		if (registry == nature.getRegistry())
			System.out.println("Failed 11.2. Did not get a new registry as we should of."); //$NON-NLS-1$
	
		// Test the getting rid of an inner class in a pre-req package is noticed.
		testChangesClassInner =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing.prereq#TestChangesClassPreq$InnerTestChangesClassPreq"); //$NON-NLS-1$		
		testChangesClassInner.getEBehaviors();
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				IType type = jProject.findType("com.ibm.etools.beaninfo.testing.prereq.TestChangesClassPreq.InnerTestChangesClassPreq");
				type.delete(true, monitor);
			}
		}, null);

		if (testChangesClassInner.getValueKind() != TypeKind.UNDEFINED)
			System.out.println("Failed 12.1. InnerTestChangesClassPreq was not marked as undefined."); //$NON-NLS-1$		

		// Test getting rid of an outer class will get rid of both the inner and the outer class.
		testChangesClass =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#TestChangesClass"); //$NON-NLS-1$
		testChangesClass.getEBehaviors();
		testChangesClassInner =
			(JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#TestChangesClass$InnerTestChangesClass"); //$NON-NLS-1$
		testChangesClassInner.getEBehaviors();
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				ICompilationUnit cu =
					(ICompilationUnit) jProject.findElement(
						new Path("com/ibm/etools/beaninfo/testing/TestChangesClass.java")); //$NON-NLS-1$
				cu.delete(true, monitor);
			}
		}, null);

		if (testChangesClass.getValueKind() != TypeKind.UNDEFINED)
			System.out.println("Failed 13.1. TestChangesClass was not marked as undefined."); //$NON-NLS-1$		
		if (testChangesClassInner.getValueKind() != TypeKind.UNDEFINED)
			System.out.println("Failed 12.1. InnerTestChangesClass was not marked as undefined."); //$NON-NLS-1$		
		
		
		System.out.println("-------- Ending the Class Changes Test. --------"); //$NON-NLS-1$

	}
}