package com.ibm.etools.beaninfo.ui;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.text.Collator;

import com.ibm.etools.beaninfo.adapters.BeaninfoEntry;
import com.ibm.etools.beaninfo.adapters.SearchpathEntry;

import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

/**
 * Sorter for sorting BPBeaninfoListElements within the main list.
 * 
 * @version 	1.0
 * @author
 */
public class BIListElementSorter extends ViewerSorter {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	/*
	 * @see ViewerSorter#category(Object)
	 */
	public int category(Object obj) {
		if (obj instanceof BPBeaninfoListElement) {
			BPBeaninfoListElement element = (BPBeaninfoListElement) obj;
			
			switch (((BeaninfoEntry) element.getEntry()).getKind()) {
			case IClasspathEntry.CPE_LIBRARY:
				return 3;
			case IClasspathEntry.CPE_PROJECT:
				return 1;
			case IClasspathEntry.CPE_SOURCE:
				return 0;
			case IClasspathEntry.CPE_VARIABLE:
				return 2;
			case BeaninfoEntry.BIE_PLUGIN:
				return 4;
			}
		}
		return super.category(obj);
	}

	/*
	 * @see ViewerSorter#compare()
	 */	
	public int compare(Viewer viewer, Object e1, Object e2) {
		int cat1 = category(e1);
		int cat2 = category(e2);
	
		if (cat1 != cat2)
			return cat1 - cat2;
			
		if ((e1 instanceof BPBeaninfoListElement) && (e2 instanceof BPBeaninfoListElement)) {
			BeaninfoEntry p1 = (BeaninfoEntry) ((BPBeaninfoListElement) e1).getEntry();
			BeaninfoEntry p2 = (BeaninfoEntry) ((BPBeaninfoListElement) e2).getEntry();
			
			// Compare paths
			return p1.getPath().toString().compareTo(p2.getPath().toString());
		}
		return super.compare(viewer, e1, e2);
	}	



}
