package com.ibm.etools.beaninfo.vm;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.beans.*;
import java.util.*;
import java.lang.reflect.Method;
/**
 * Equality tester for EventSetDescriptors
 */
public class EventSetDescriptorEquality extends FeatureDescriptorEquality {
	
	static {
		try {
			MAP_EQUALITY.put(EventSetDescriptor.class, (EventSetDescriptorEquality.class).getConstructor(new Class[] {EventSetDescriptor.class}));
		} catch (NoSuchMethodException e) {
		}
	}
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	private ArrayList fListenerMethodDescriptors;	// Array of MethodDescriptorEquality's.
	
	public EventSetDescriptorEquality(EventSetDescriptor descr) {
		super(descr);
	}
	
	/**
	 * A new feature is being set into this object,
	 * clear any cache members so that they can be reconstructed.
	 *
	 * NOTE: Subclasses - remember to call super.clearFeature();
	 */
	protected void clearFeature() {
		super.clearFeature();
		fListenerMethodDescriptors = null;
	}
		
	protected ArrayList listenerMethodDescriptors() {
		if (fListenerMethodDescriptors == null) {
			MethodDescriptor[] mds = ((EventSetDescriptor) fFeature).getListenerMethodDescriptors();
			fListenerMethodDescriptors = new ArrayList(mds.length);
			for (int i=0; i<mds.length; i++)
				fListenerMethodDescriptors.add(new MethodDescriptorEquality(mds[i]));
		}
		return fListenerMethodDescriptors;
	}	
	
	/**
	 * Calculate the hashcode for the current feature, add this
	 * to the hashcode received from super.calculateHashCode
	 * and return the new value.
	 *
	 * NOTE: for subclasses, it is MANDITORY that the first line be:
	 *         int hashcode = super.calculateHashCode();
	 *       and the last line be:
	 *         return hashcode*31 + (your calculated hashcode for just this subclass);
	 */
	protected int calculateHashCode() {
		int hashcode = super.calculateHashCode();
		EventSetDescriptor bd = (EventSetDescriptor) fFeature;
		int hc = bd.getAddListenerMethod().hashCode();
		Method[] methods = bd.getListenerMethods();
		int mhc = 0;
		for (int i=0; i<methods.length; i++)
			mhc = mhc*31 + methods[i].hashCode();
		hc += mhc;
		hc += listenerMethodDescriptors().hashCode();
		hc += bd.getListenerType().hashCode();
		hc += bd.getRemoveListenerMethod().hashCode();
		hc += (bd.isInDefaultEventSet() ? Boolean.TRUE : Boolean.FALSE).hashCode();
		hc += (bd.isUnicast() ? Boolean.TRUE : Boolean.FALSE).hashCode();		

		return hashcode*31 + hc;
	}

	/**
	 * equals: See if this is equal semantically.
	 *
	 * NOTE: Every subclass needs to implement this and
	 *       the first line should be:
	 *         if (!super.equals(obj))
	 *           return false;
	 */
	 
	public boolean equals(Object obj) {
		if (!super.equals(obj))
 			return false;
 	
		EventSetDescriptor oe = (EventSetDescriptor) ((FeatureDescriptorEquality) obj).fFeature;
		EventSetDescriptor fe = (EventSetDescriptor) fFeature;
 		
		if (!oe.getAddListenerMethod().equals(fe.getAddListenerMethod()))
			return false;
		if (!java.util.Arrays.equals(oe.getListenerMethods(), fe.getListenerMethods()))
			return false;	
		if (!((EventSetDescriptorEquality) obj).listenerMethodDescriptors().equals(listenerMethodDescriptors()))
			return false;
		if (oe.getListenerType() != fe.getListenerType())
			return false;
		if (oe.getRemoveListenerMethod() != fe.getRemoveListenerMethod())
			return false;
		if (oe.isInDefaultEventSet() != fe.isInDefaultEventSet())
			return false;
		if (oe.isUnicast() != oe.isUnicast())
			return false;
			
		return true;			
	}
}