package com.ibm.etools.beaninfo.vm;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.beans.*;
/**
 * PropertyDescriptor equality tester
 */
public class PropertyDescriptorEquality extends FeatureDescriptorEquality {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	static {
		try {
			MAP_EQUALITY.put(PropertyDescriptor.class, (PropertyDescriptorEquality.class).getConstructor(new Class[] {PropertyDescriptor.class}));
		} catch (NoSuchMethodException e) {
		}
	}	
	
	public PropertyDescriptorEquality(PropertyDescriptor descr) {
		super(descr);
	}	
	/**
	 * Calculate the hashcode for the current feature, add this
	 * to the hashcode received from super.calculateHashCode
	 * and return the new value.
	 *
	 * NOTE: for subclasses, it is MANDITORY that the first line be:
	 *         int hashcode = super.calculateHashCode();
	 *       and the last line be:
	 *         return hashcode*31 + (your calculated hashcode for just this subclass);
	 */
	protected int calculateHashCode() {
		int hashcode = super.calculateHashCode();
		PropertyDescriptor pd = (PropertyDescriptor) fFeature;
		int hc = 0;
		if (pd.getPropertyEditorClass() != null)
			hc += pd.getPropertyEditorClass().hashCode();
		if (pd.getPropertyType() != null)
			hc += pd.getPropertyType().hashCode();
		if (pd.getReadMethod() != null)
			hc += pd.getReadMethod().hashCode();
		if (pd.getWriteMethod() != null)
			hc += pd.getWriteMethod().hashCode();
			
		hc += (pd.isBound() ? Boolean.TRUE : Boolean.FALSE).hashCode();
		hc += (pd.isConstrained() ? Boolean.TRUE : Boolean.FALSE).hashCode();		
						
		return hashcode*31 + hc;
	}

	/**
	 * equals: See if this is equal semantically.
	 *
	 * NOTE: Every subclass needs to implement this and
	 *       the first line should be:
	 *         if (!super.equals(obj))
	 *           return false;
	 */
	 
	public boolean equals(Object obj) {
		if (!super.equals(obj))
			return false;
 	
		PropertyDescriptor op = (PropertyDescriptor) ((FeatureDescriptorEquality) obj).fFeature;
		PropertyDescriptor fp = (PropertyDescriptor) fFeature;
 		
		if (op.getPropertyEditorClass() != fp.getPropertyEditorClass())
			return false;
		if (op.getReadMethod() != fp.getReadMethod())
			return false;
		if (op.getWriteMethod() != fp.getWriteMethod())
			return false;
		if (op.isBound() != fp.isBound())
			return false;				
		if (op.isConstrained() != fp.isConstrained())
			return false;														
			
		return true;
	}


}