package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.gef.requests.CreateRequest.Factory;
import com.ibm.etools.cde.emf.IDomainedFactory;
import com.ibm.etools.cdm.*;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefObject;
import java.util.*;
import com.ibm.etools.emf.ecore.utilities.copy.EtoolsCopyUtility;

/**
 * This factory creates the annotation for the object that will be created from
 * the factory passed to it.
 */
public class AnnotationCreationFactory extends Object implements Factory, IDomainedFactory {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Factory objectFactory;
	protected List keyedValues;
	protected AnnotationLinkagePolicy policy;

public AnnotationCreationFactory(List keyedValues, Factory objectFactory) {
	this.keyedValues = keyedValues;
	this.objectFactory = objectFactory;
}

public AnnotationCreationFactory(Factory objectFactory) {
	this(null, objectFactory);
}

public Object getObjectType() {
	return objectFactory.getObjectType();
}

/**
 * getNewObject method comment. 
 */
public Object getNewObject() {
	Object newObject = objectFactory.getNewObject();
	if (newObject instanceof RefObject) {
		CDMFactory fact = CDEUtilities.CDM_PACKAGE.getCDMFactory();
		AnnotationEMF annotation = fact.createAnnotationEMF();
		if (keyedValues != null) {
			Iterator itr = keyedValues.iterator();
			while (itr.hasNext()) {
				KeyedValue kv = (KeyedValue) itr.next();
				annotation.setKeyedValue(kv.getKey(), (KeyedValue) EtoolsCopyUtility.createCopy(kv));
			}
		}
		annotation.setAnnotates((RefObject) newObject);
	} else {
		CDMFactory fact = CDEUtilities.CDM_PACKAGE.getCDMFactory();
		AnnotationGeneric annotation = fact.createAnnotationGeneric();
		if (keyedValues != null) {
			Iterator itr = keyedValues.iterator();
			while (itr.hasNext()) {
				KeyedValue kv = (KeyedValue) itr.next();
				annotation.setKeyedValue(kv.getKey(), (KeyedValue) EtoolsCopyUtility.createCopy(kv));
			}
		}
		policy.setModelOnAnnotation(newObject, annotation);
	}

	return newObject;
}
	/**
	 * @see IDomainedFactory#setEditDomain(EditDomain)
	 */
	public void setEditDomain(EditDomain domain) {
		policy = domain.getAnnotationLinkagePolicy();
		if (objectFactory instanceof IDomainedFactory)
			((IDomainedFactory) objectFactory).setEditDomain(domain);
	}

}