package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPartViewer;
import com.ibm.etools.cdm.CDMPackage;
import com.ibm.etools.cde.decorators.DecoratorsPackage;
import com.ibm.etools.cde.utility.UtilityPackage;
import com.ibm.etools.emf.ref.RefRegister;
/**
 * Utilities for CDE functions that are not found in a more logical place.
 */
public class CDEUtilities {
	
	/**
	 * Return the EditPartFactory for the given editpart.
	 */
	public static IEditPartFactory getEditPartFactory(EditPart editpart) {
		return (IEditPartFactory) EditDomain.getEditDomain(editpart).getViewerData(editpart.getRoot().getViewer(), IEditPartFactory.EDITPART_FACTORY_KEY);
	}
	
	/**
	 * Set the EditPartFactory for the given viewer.
	 */
	public static void setEditPartFactory(EditPartViewer epv, IEditPartFactory factory) {
		EditDomain dom = (EditDomain) epv.getEditDomain();
		dom.setViewerData(epv, IEditPartFactory.EDITPART_FACTORY_KEY, factory);
	}
	
	/**
	 * Return the ModelAdapterFactory for the given domain.
	 */
	public static IModelAdapterFactory getModelAdapterFactory(EditDomain domain) {
		return (IModelAdapterFactory) domain.getData(IModelAdapterFactory.MODEL_FACTORY_KEY);
	}

	
	/**
	 * Return the ModelAdapterFactory for the given editpart (only used to find the domain).
	 */
	public static IModelAdapterFactory getModelAdapterFactory(EditPart editpart) {
		return getModelAdapterFactory(EditDomain.getEditDomain(editpart));
	}
	
	/**
	 * Set the ModelAdapterFactory for the given domain.
	 */
	public static void setModelAdapterFactory(EditDomain domain, IModelAdapterFactory factory) {
		domain.setData(IModelAdapterFactory.MODEL_FACTORY_KEY, factory);
	}
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final CDMPackage CDM_PACKAGE;
	public static final DecoratorsPackage DECORATORS_PACKAGE;
	public static final UtilityPackage UTILITY_PACKAGE;
	static {
		CDM_PACKAGE = (CDMPackage) RefRegister.getPackage(CDMPackage.packageURI);
		DECORATORS_PACKAGE = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
		UTILITY_PACKAGE = (UtilityPackage) RefRegister.getPackage(UtilityPackage.packageURI);
	}
	
}