package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.editparts.AbstractGraphicalEditPart;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.Insets;
import com.ibm.etools.gef.LayerConstants;
import java.util.List;
import com.ibm.etools.gef.*;
/**
 * CDE Contents (Freeform surface) Graphical Edit Part.
 * Override to perform domain specific functions. 
 * This provides the necessary functions for maintaining the
 * freeform surface, such as positioning, zooming, ...
 */

public abstract class ContentsGraphicalEditPart extends AbstractGraphicalEditPart {
		
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
/**
 * Set the figure's layout manager to be an XYFlowLayout.
 */
protected IFigure createFigure() {
	IFigure contentPane = new Figure();
	contentPane.setBorder(new MarginBorder(new Insets(10, 10, 100, 100)));
	contentPane.setBackgroundColor(ColorConstants.white);
	contentPane.setLayoutManager(new XYFlowLayout());
	contentPane.setOpaque(true);
	return contentPane;
}

/**
 * Apart from the EditPolicy updates and refreshes of super,
 * this also sets the layout of the CONNECTION_LAYER to
 * reflect the connection updates.
 */
protected void doInitialize() {
	// Get the connection layer from the drawing and set its connection 
	// layout manager.  This allows one instance of a connection layout
	// manager to be shared by all connections.
	ConnectionLayer cLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
	// The fan routre will resolve conflicts and provide auto-bending functionality
	// This occurs after manual bending ( handled by the BendableConnectionRouter )
	FanRouter chainRouter = new FanRouter();
	chainRouter.setNextRouter(new BendpointConnectionRouter());
	cLayer.setConnectionRouter(chainRouter);
	super.doInitialize();
}

}