package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.editpolicies.AbstractEditPolicy;
import com.ibm.etools.gef.Request;
import com.ibm.etools.gef.requests.DeleteRequest;
import com.ibm.etools.gef.RequestConstants;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.gef.EditPart;
import java.util.List;
/**
 * Default Container Edit Policy. It should be added to container editparts
 * so that delete and ancestor deleted requests are propagated properly.
 */
public class DefaultContainerEditPolicy extends AbstractEditPolicy {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
public Command getCommand(Request request){
	if (RequestConstants.REQ_ANCESTOR_DELETED.equals(request.getType())) {
		CompoundCommand cmd = new CompoundCommand();
		cmd.append(propagateAncestorDeletedCommands((DeleteRequest) request));
		cmd.append(getAncestorDeletedCommand((DeleteRequest)request));
		return cmd.isEmpty() ? null : cmd.unwrap();
	}
	
	if (RequestConstants.REQ_DELETE.equals(request.getType())) {
		return propagateAncestorDeletedCommands((DeleteRequest) request);
	}
	return null;
}

public EditPart getTargetEditPart(Request request){
	return null;
}

/**
 * An ancestor (grandparent) has been deleted. Return any command that
 * this container needs to do if its ancestor has been deleted. Return
 * null if there is nothing that needs to be added. (Null is valid in
 * the case). That is the default in this case. This method can be
 * overridden to provide other function.
 */ 
protected Command getAncestorDeletedCommand(DeleteRequest request) {
	return null;
}

/**
 * This policy's host, or an ancestor of it, is being deleted. Propagate the ancestor deleted
 * request to the children edit parts.
 *
 * This is not meant to be overridden.
 */
protected Command propagateAncestorDeletedCommands(DeleteRequest delete) {
	CompoundCommand compound = new CompoundCommand();
	List children = getHost().getChildren();
	DeleteRequest ancestorDeletedReq = new DeleteRequest(RequestConstants.REQ_ANCESTOR_DELETED);
	ancestorDeletedReq.setContributions(delete.getContributions());

	for (int i=0; i<children.size(); i++) {
		EditPart child = (EditPart)children.get(i);
		compound.append(child.getCommand(ancestorDeletedReq));
	}

	return compound.isEmpty() ? null : compound.unwrap();
}
}