package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.gef.*;
import java.util.*;
import com.ibm.etools.cdm.*;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.gef.editparts.AbstractTreeEditPart;
import com.ibm.etools.cde.emf.*;
/**
 * TreeEditPart for a DiagramFigure where DiagramFigures are its children.
 */
public abstract class DiagramFigureTreeEditPart extends AbstractTreeEditPart implements IEMFListener {
	
	/**
	 * Subclasses need to call super.createEditPolicies so that
	 * this method gets called.
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.PRIMARY_DRAG_ROLE, new TreePrimaryDragRoleEditPolicy());
	}
		
	protected List getModelChildren() {
		DiagramFigure diagramFigure = (DiagramFigure) getModel();
		return diagramFigure.getChildFigures();
	}

	public void activate() {
		super.activate();
		((DiagramFigure) getModel()).addAdapter(getModelAdapter());
	}
	
	public void deactivate() {
		super.deactivate();
		((DiagramFigure) getModel()).removeAdapter(getModelAdapter());
	}
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private EMFListenerAdapter diagramAdapter;
	
	protected Adapter getModelAdapter() {

		if (diagramAdapter == null) {
			// Create an adaptor that is used to listen to changes from the
			// MOF model object.  The MOF notifications are used by this edit part to stay updated.
			sfFigures = ((DiagramFigure) getModel()).ePackageCDM().getDiagramFigure_ChildFigures();
			diagramAdapter = new EMFListenerAdapter(this, true);
		}
		return diagramAdapter;
	
	}
	
	/**
	 * The Diagram has changed. So if it is the figures feature, refresh the children.
	 * Subclasses may override to add in their own interest, but they must call super.
	 */
	private RefStructuralFeature sfFigures;
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sfFigures)
			refreshChildren();
	}
	
	/**
	 * Create the child editpart for this model.
	 * A general approach that subclasses may use is using
	 * factories to create the edit part from the model. Or
	 * they may hard-code it since they may know exactly what
	 * they need.
	 */
	protected abstract EditPart createChild(Object model);
		

}


