package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.*;

/*
 * This figure is used to draw a boundary box around the selected editparts
 * so they can be distributed within it's surface area. It has selection 
 * handles (see DistributeHandle) in each corner and in between so allow
 * resizing the bounding area.
 */
public class DistributeFigure extends Polyline implements MouseMotionListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Rectangle fBounds = new Rectangle();

public DistributeFigure () {
	
	setBackgroundColor(ColorConstants.blue);
	setLineStyle(Graphics.LINE_DASH);
	setForegroundColor(ColorConstants.white);
	
	setPointsBasedOnRectangle();
}

/**
 * Change the bounding box.
 */
public void setBoundingBox(Rectangle rect) {
	fBounds.setBounds(rect);
	setPointsBasedOnRectangle();
}

/*
 * One of the handles is moving... adjust the size of this figure
 * according to the mouse location.
 */
protected void handleDrag(MouseEvent me) {
	int locType = ((DistributeHandle)me.getSource()).getLocatorType();
	Point p = me.getLocation();
	switch (locType) {
		case (PositionConstants.NORTH) :
			{
				fBounds.height += (fBounds.y - p.y);
				fBounds.y = p.y;
				break;
			}
		case (PositionConstants.SOUTH) :
			{
				fBounds.height = p.y - fBounds.y;
				break;
			}
		case (PositionConstants.EAST) :
			{
				fBounds.width = p.x - fBounds.x;
				break;
			}
		case (PositionConstants.WEST) :
			{
				fBounds.width += (fBounds.x - p.x);
				fBounds.x = p.x;
				break;
			}
		case (PositionConstants.NORTH_WEST) :
			{
				fBounds.width += (fBounds.x - p.x);
				fBounds.x = p.x;
				fBounds.height += (fBounds.y - p.y);
				fBounds.y = p.y;
				break;
			}
		case (PositionConstants.NORTH_EAST) :
			{
				fBounds.width = p.x - fBounds.x;
				fBounds.height += (fBounds.y - p.y);
				fBounds.y = p.y;
				break;
			}
		case (PositionConstants.SOUTH_WEST) :
			{
				fBounds.width += (fBounds.x - p.x);
				fBounds.x = p.x;
				fBounds.height = p.y - fBounds.y;
				break;
			}
		case (PositionConstants.SOUTH_EAST) :
			{
				fBounds.width = p.x - fBounds.x;
				fBounds.height = p.y - fBounds.y;
				break;
			}
		default :
			{
				// default is the NORTH location.
				fBounds.height += (fBounds.y - p.y);
				fBounds.y = p.y;
				break;
			}
	}
	setPointsBasedOnRectangle();
}
public void mouseHover(MouseEvent me){};
public void mouseMoved(MouseEvent me) {};
public void mouseEntered(MouseEvent me){};
public void mouseExited(MouseEvent me){};
public void mouseDragged(MouseEvent me){
	handleDrag(me);
}
protected void setPointsBasedOnRectangle() {
	PointList pl = new PointList(5);
	
	pl.addPoint(fBounds.getTopLeft());
	pl.addPoint(fBounds.getTopRight());
	pl.addPoint(fBounds.getBottomRight());
	pl.addPoint(fBounds.getBottomLeft());
	pl.addPoint(fBounds.getTopLeft());
	setPoints(pl);
	fireMoved();	// Need to do this because handles are listening for changes and setting points doesn't fire move.
}
}