package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.*;
import com.ibm.etools.gef.requests.*;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.gef.editpolicies.NonResizableEditPolicy;
import java.util.Collections;
/**
 * Common Diagram Layout Edit Policy that uses a CDE ContainerEditPolicy to do the
 * container functions. This allows the ContainerEditPolicy to also be used
 * as in the tree also.
 */
public class FlowLayoutEditPolicy extends com.ibm.etools.gef.editpolicies.FlowLayoutEditPolicy {
	
		
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ContainerPolicy containerPolicy;		// Handles the containment functions
	
protected EditPolicy createChildEditPolicy(EditPart child) {
	return new NonResizableEditPolicy();
}
	
/**
 * Create with the container policy for handling children.
 */
public FlowLayoutEditPolicy(ContainerPolicy containerPolicy) {
	this.containerPolicy = containerPolicy;
}

public void activate() {
	super.activate();
	containerPolicy.setContainer(getHost().getModel());
}

public void deactivate() {
	super.deactivate();
	containerPolicy.setContainer(null);
}

protected Command createAddCommand(EditPart childEP, EditPart beforeEP) {
	return  containerPolicy.getAddCommand(Collections.nCopies(1, childEP.getModel()), beforeEP != null ? beforeEP.getModel() : null);
}

protected Command getDeleteDependantCommand(Request aRequest) {
	Command cmd = containerPolicy.getCommand(aRequest);
	return cmd != null ? cmd : UnexecutableCommand.INSTANCE;
}

protected Command createMoveChildCommand(EditPart childEP, EditPart beforeEP) {
	return  containerPolicy.getMoveChildrenCommand(Collections.nCopies(1, childEP.getModel()), beforeEP != null ? beforeEP.getModel() : null);
}

protected Command getOrphanChildrenCommand(Request req) {
	// Now get the orphan command for the child.
	return containerPolicy.getCommand(req);
}

protected Command getCreateCommand(CreateRequest req){
	EditPart before = getInsertionReference(req);
	return containerPolicy.getCreateCommand(req.getNewObject(), before != null ? before.getModel() : null);
}

}