package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.gef.*;
import com.ibm.etools.gef.editpolicies.GraphicalEditPolicy;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.common.command.*;
import com.ibm.etools.draw2d.geometry.*;

public class HoverFeedbackEditPolicy extends GraphicalEditPolicy implements FigureListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Figure feedbackFigure = null;

public void deactivate(){
	super.deactivate();
	if (feedbackFigure != null)
		stopListening();
}
public void eraseTargetFeedback(Request request){
	if(feedbackFigure!=null)
		stopListening();
}
protected void followFigure(IFigure figure){
	Rectangle bounds = figure.getBounds();
	getSelectionFeedback().setBounds(bounds.getExpanded(1,1));
}
public Command getCommand(Request request){return null;}
public void figureMoved(IFigure figure){
	followFigure(figure);
}
protected IFigure getSelectionFeedback(){
	if (feedbackFigure == null){
		feedbackFigure = new Figure();
		feedbackFigure.setBorder(new LineBorder(ColorConstants.darkBlue,2));
		feedbackFigure.setOpaque(false);
		addFeedback(feedbackFigure);
		startListening();
	}
	return feedbackFigure;
}
public EditPart getTargetEditPart(Request request){
	return request.getType().equals(RequestConstants.REQ_SELECTION_HOVER) ?
		getHost() : null;
}
protected void showSelectionFeedback(){
	getSelectionFeedback();
}
public void showTargetFeedback(Request request){
	if(request.getType().equals(RequestConstants.REQ_SELECTION_HOVER))
		showSelectionFeedback();
}
protected void startListening(){
	IFigure figure = ((GraphicalEditPart)getHost()).getFigure();
	figure.addFigureListener(this);
	followFigure(figure);
}
protected void stopListening(){
	IFigure figure = ((GraphicalEditPart)getHost()).getFigure();
	figure.removeFigureListener(this);
	removeFeedback(feedbackFigure);
	feedbackFigure = null;
}
}