package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

/**
 * This interface is used whenever the model needs to
 * be changed. It is required that changes go through here
 * so that the model and other changes are synchronized correctly.
 * 
 * This interface is not meant to be implemented by users. 
 * It is stored in the domain for usage. The key stored in this
 * interface can be used to retrieve it from the domain.
 */
public interface IModelChangeController {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String MODEL_CHANGE_CONTROLLER_KEY = "com.ibm.etools.cde.IModelChangeController"; //$NON-NLS-1$
	
	/**
	 * Call this method with a runnable. The runnable will
	 * do the actual update of the model. This method will
	 * make sure that the updates are blocked correctly to
	 * the text editor, for instance.
	 * 
	 * RunExceptions will not be squelched, they will be returned,
	 * 
	 * It will return whether the runnable could be executed. For
	 * example if the file was read-only and could not be checked out,
	 * the update won't occur.
	 */
	public boolean run(Runnable runnable); 
	
	/**
	 * This method provides the ability to hold or resume changes to the model.
	 * @param flag when true, will hold (inhibit) changes.
	 * @param reasonMsg optional message denoting a reason for the hold request.
	 * 
	 * Note: as we go forwared, we should think of allowing multiple entities to
	 *       call setHoldChanges with some type of caller signiture, so that only when
	 *       all entities have clear the hold, would the controller keep on going.
	 */
	public void setHoldChanges(boolean flag, String reasonMsg) ;
	
	/**
	 * Return the status of the hold state
	 */
	public boolean isHoldChanges() ;
	
	/**
	 * Return the current hold msg.
	 */
	public String getHoldMsg() ;
	
}
