package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.graphics.Image;

import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.Rectangle;
/**
 * Figure that takes an image and will display it.
 * This figure will not own the image. It is the
 * responsibility of the caller to maintain the image.
 */
public class ImageFigure extends Figure {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Image image;
	protected boolean useParentImageFigure = false;
	
	{
		setPreferredSize(10,10);
	}
	
	/**
	 * Set the image this figure is to use.
	 * NOTE: This figure does not own the image. It shares it. The sender
	 * still maintains ownership of the image and must dispose of it when
	 * necessary.
	 */
	public void setImage(Image image) {
		if (image == this.image)
			return;
		this.image = image;
		if (image == null)
			setPreferredSize(10,10);
		else {
			org.eclipse.swt.graphics.Rectangle bounds = image.getBounds();
			setPreferredSize(bounds.width, bounds.height);
		}
		revalidate();
		repaint();	// In case didn't change size, still need to repaint.
	}
	
	/**
	 * Return the image currently being used.
	 */
	public Image getImage() {
		return image;
	}
	
	/**
	 * Set to use the parent's ImageFigure as the image to use.
	 * This figure must a child or grandchild of this
	 * ImageFigure. It will paint its portion of the image
	 * (depending upon its own bounds) of that ImageFigure.
	 * This is to allow the borders to be appropriately clipped
	 * while still using the one master image.
	 */
	public void setUseParentImageFigure(boolean aBool) {
		if (this.useParentImageFigure == aBool)
			return;
		useParentImageFigure = aBool;
		image = null;
		setPreferredSize(10, 10);
		revalidate();
		repaint();	// In case didn't change size, still need to repaint.		
	}
	
	protected void paintClientArea(Graphics g) {
		if (image != null) {
			Rectangle r = getClientArea();
			g.drawImage(image, r.x, r.y);
		} else if (useParentImageFigure) {
			// Get the parent up the chain that has the actual image to use.
			ImageFigure parentImageFigure = null;
			for (IFigure parent = getParent(); parent != null; parent = parent.getParent()) {
				if (parent instanceof ImageFigure && ((ImageFigure)parent).getImage() != null) {
					parentImageFigure = (ImageFigure) parent;
					break;
				}
			}
			if (parentImageFigure != null) {
				Image parentImage = parentImageFigure.getImage();
				if (parentImage != null) {
					Rectangle r = getClientArea();
					Rectangle parentr = parentImageFigure.getClientArea();
					// Need to get the intersection of the client area with the 
					// parent's client area in case the parent's boundary is smaller.
					r = r.intersect(parentr);
					g.drawImage(parentImageFigure.getImage(), r.x-parentr.x, r.y-parentr.y, r.width, r.height, r.x, r.y, r.width, r.height);
				}
			}
		}
		super.paintClientArea(g);
	}	
}