package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import org.eclipse.swt.graphics.ImageData;
/**
 * This is an image notifier support class. It maintains
 * the listeners for an image notifier and does the firing.
 * Implementers of IImageNotifier only need an instance
 * of this class to manage the listener lists and can forward
 * them to here. Use an VisualComponentSupport class for 
 * the IVisualComponent non-image portion.
 */
public class ImageNotifierSupport {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected List imageListeners = null;	// Listeners for IComponentNotification.

	/**
	 * addImageListener.
	 */
	public synchronized void addImageListener(IImageListener aListener) {
		if (imageListeners == null)
			imageListeners = new ArrayList(2);
		imageListeners.add(aListener);
	}
	
	/**
	 * Fire image changed notification.
	 */
	public void fireImageChanged(ImageData imageData) {
		// Probably should make a copy of the notification list to prevent
		// modifications while firing, but we'll see if this gives any problems.
		if (imageListeners != null) {
			for (int i=0; i<imageListeners.size(); i++) {
				IImageListener listener = (IImageListener) imageListeners.get(i);
				listener.imageChanged(imageData);
			}
		}
	}
	
	/**
	 * hasListeners method..
	 */
	public boolean hasImageListeners() {
		return imageListeners != null && !imageListeners.isEmpty();
	}
	
	/**
	 * removeImageListener method.
	 */
	public synchronized void removeImageListener(IImageListener aListener) {
		imageListeners.remove(aListener);
	}

}