package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.draw2d.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.SWT;
import com.ibm.etools.draw2d.geometry.Insets;
import com.ibm.etools.draw2d.geometry.Rectangle;
/**
 * This border outlines the figure. It will draw the outline
 * on top of the figure, not outside the client area. This is so that
 * the client area won't be reduced by the border.
 */
public class OutlineBorder extends AbstractBorder {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Color 
		foreground = ColorConstants.black, 
		background;
		
	private static final Insets insets = new Insets(0,0,0,0);
	protected int lineStyle = SWT.LINE_SOLID;

	public OutlineBorder() {
	}
	
	public OutlineBorder(Color foreground, Color background) {
		this.foreground = foreground;
		this.background = background;
	}
	
	public OutlineBorder(Color foreground, Color background, int lineStype) {
		this(foreground, background);
		this.lineStyle = lineStyle;
	}

	
	public void paint(IFigure aFigure, Graphics g, Insets insets) {
		Rectangle r = getPaintRectangle(aFigure, insets);
		r.resize(-1,-1);	// Make room for the outline.
		g.setForegroundColor(foreground);
		if (lineStyle != SWT.LINE_SOLID) {
			// Non-solid lines need a background color to be set. If we have one use it, else compute it.
			if (background != null)
				g.setBackgroundColor(background);
			else {
				// If no background is set then make the background black
				// and set it to XOR true.  This means the line will dash over
				// the background.  The foreground will also XOR
				// so it only works well if the foreground is Black or Gray.  Colors
				// don't work well because they only paint true on black
				// areas
				g.setBackgroundColor(ColorConstants.black);
				g.setXORMode(true);
			}
		}
		g.setLineStyle(lineStyle);
		g.drawRectangle(r);
	}
	
	public void setColors(Color foreground, Color background) {
		this.foreground = foreground;
		this.background = background;
	}
	
	public void setLineStyle(int aStyle) {
		lineStyle = aStyle;
	}
	
	public Insets getInsets(IFigure aFigure) {
		return insets;
	}
	
	public boolean isOpaque() {
		return true;
	}	
}