/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



package com.ibm.etools.cde;

import java.util.*;
import org.eclipse.jface.action.*;
import com.ibm.etools.cde.palette.*;
import com.ibm.etools.gef.palette.*;

public class PaletteCategoriesToolEntryAction extends PaletteToolEntryAction {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected PaletteRoot fPaletteRoot;
	protected IMenuCreator fMenuCreator;
	protected MenuManager fToolbarMenuManager;
	protected MenuManager fDropdownMenuManager;	// The menu manager cannot be shared.  There is some kind of bug
	// that means that if you get the action used as a menu ( by making the IDE so small it shows up on the right and
	// must be dropped down ) then you get errors when you go back to the toolbar
	protected List fChildActions = new ArrayList();
	
public PaletteCategoriesToolEntryAction(AbstractToolEntry entry,PaletteRoot paletteRoot){
	super(entry);
	setId(entry.refID());
	fPaletteRoot = paletteRoot;
	// Build a menu manager by traversing the palette
	fToolbarMenuManager = new MenuManager(entry.getLabel());
	fDropdownMenuManager = new MenuManager(entry.getLabel());
	Iterator iter = fPaletteRoot.getChildren().iterator();
	while(iter.hasNext()){
		Object paletteChild = iter.next();
		// The child could be a palette container or an entry
		if(paletteChild instanceof PaletteContainer){
			// Do not add the control group back into the menu
			if(paletteChild instanceof Group && "control".equals(((Group)paletteChild).refID())){ //$NON-NLS-1$
				continue;
			}
			// Recurse through the items on the container
			addContainerToMenu((PaletteContainer)paletteChild,fToolbarMenuManager);
			addContainerToMenu((PaletteContainer)paletteChild,fDropdownMenuManager);			
		} else if (paletteChild instanceof AbstractToolEntry){
			// The first set of children are the things like Selection and Marquee
			// don't add these to the menu
			// We need a better way to do this to ignore the control group but this will 
			// do for now
//			addToolEntryToMenu((AbstractToolEntry)paletteChild,fMenuManager);
		}		
	}
}
/** 
 * Create the PaletteToolEntry action and add it to the menu manager as well as record
 * it in the fChildActions so it can be found later
 */
protected void addToolEntryToMenu(AbstractToolEntry toolEntry,IMenuManager aMenuManager){
	PaletteToolEntryAction entryAction = new PaletteToolEntryAction(this,toolEntry);
	fChildActions.add(entryAction);
	aMenuManager.add(entryAction);
}
protected void addContainerToMenu(PaletteContainer container,IMenuManager parentMenuManager){
	// Create a new MenuManager which will appear as a sub menu	
	MenuManager menuManager = new MenuManager(container.getLabel());
	parentMenuManager.add(menuManager);
	Iterator entries = container.getChildren().iterator();
	while(entries.hasNext()){
		Object entry = entries.next();
		if ( entry instanceof AbstractToolEntry ){
			addToolEntryToMenu((AbstractToolEntry)entry,menuManager);
		} else if ( entry instanceof PaletteContainer){
			// We fold containers of containers into the same menu manager
			Iterator iter = ((PaletteContainer)entry).getChildren().iterator();
			while(iter.hasNext()){
				Object child = iter.next();
				if(child instanceof AbstractToolEntry){
					addToolEntryToMenu((AbstractToolEntry)child,menuManager);
				}		
			}
		}
	}
}
public int getStyle(){
	return AS_DROP_DOWN_MENU;
}
public IMenuCreator getMenuCreator() {
	if ( fMenuCreator == null){
		fMenuCreator = new PaletteMenuCreator(fToolbarMenuManager,fDropdownMenuManager);
	}
	return fMenuCreator;
}
/* We contain entries so we need to recurse through these to see if any of them
 * match the argument
 */
public boolean selectIfContainsId(String anID){
	Iterator actions = fChildActions.iterator();
	while (actions.hasNext()){
		Action action = (Action)actions.next();
		if (anID.equals(action.getId())){
			action.setChecked(true);
			return true;			
		}
	}
	return false;
}

public void dispose() {
	if (fMenuCreator != null) {
		fMenuCreator.dispose();
		fMenuCreator = null;
	}
}
public Object clone(){
	return new PaletteCategoriesToolEntryAction(fToolEntry,fPaletteRoot);
}
}
