package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.net.MalformedURLException;
import java.net.URL;

import com.ibm.etools.gef.Tool;
import com.ibm.etools.gef.palette.PaletteEvent;
import com.ibm.etools.gef.palette.PaletteListener;
import com.ibm.etools.gef.palette.PaletteRoot;
import com.ibm.etools.gef.tools.SelectionTool;
import com.ibm.etools.gef.ui.palette.PaletteViewer;
import com.ibm.etools.gef.ui.palette.PaletteViewerImpl;

import org.eclipse.swt.widgets.*;
import org.eclipse.ui.actions.RetargetAction;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import com.ibm.etools.cde.palette.AbstractToolEntry;

public class PaletteToolEntryAction extends Action implements IPaletteContributionItem {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected PaletteCategoriesToolEntryAction fParent;	// Optional parent if we are on a menu and not a top level entry
	protected AbstractToolEntry fToolEntry;
	protected EditDomain fEditDomain;
	protected Button fButton;
	protected Image fImage;
	protected Tool fTool;
	
public PaletteToolEntryAction(PaletteCategoriesToolEntryAction aParent, AbstractToolEntry aToolEntry){
	this(aToolEntry);
	setId(aToolEntry.refID());
	fParent = aParent;
}
	
public PaletteToolEntryAction(AbstractToolEntry aToolEntry){
	try {
		setImageDescriptor(ImageDescriptor.createFromURL(new URL(aToolEntry.getIcon16Name())));
	} catch ( MalformedURLException exc ) {
	}
	setId(aToolEntry.refID());
	setText(aToolEntry.getLabel());
	setToolTipText(aToolEntry.getLabel());
	fToolEntry = aToolEntry;
	fTool = aToolEntry.getTool();
}
public void run(){

	// if we have an edit domain we are on the toolbar of the first level set of menus
	// and we can use it
	if ( getEditDomain() != null){
		// See whether there is a PaletteViewer
		PaletteViewerImpl paletteViewer = (PaletteViewerImpl)getEditDomain().getData(PaletteViewer.class);
		if ( paletteViewer != null){
			paletteViewer.setSelection(fToolEntry);
		} else {
			getEditDomain().setTool(fTool);
		}
	}
}
/**
 * Return the local edit domain or else ask the parent
 * This means that only the top level set of config items need to explicitly store
 * the edit domain, and the children can just cascade up
 * This saves time when switching editors and notifying of a new edit domain
 * as only the top level need to be informed
 */
EditDomain getEditDomain(){
	if ( fEditDomain != null ){
		return fEditDomain;
	} else if ( fParent != null ){
		return fParent.getEditDomain();
	} else {
		return null;
	}
}
public void setEditDomain(EditDomain anEditDomain){
	setEnabled(anEditDomain != null);
	fEditDomain = anEditDomain;
}	
/**
 * Make us appear in a selected state if the argument matches our ID
 * and return true if it did match, false otherwise
 */
public boolean selectIfContainsId(String anID){
	if ( getId().equals(anID) ) {
		setChecked(true);
		return true;
	}
	return false;
}
public Object clone(){
	if ( fParent == null){
		return new PaletteToolEntryAction(fToolEntry);
	} else {
		return new PaletteToolEntryAction(fParent,fToolEntry);
	}
}
}