package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IEditorPart;

import com.ibm.etools.common.command.*;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.RequestConstants;
import com.ibm.etools.gef.requests.ChangeBoundsRequest;
import com.ibm.etools.gef.ui.actions.SelectionAction;

public class SnapToGridAction extends SelectionAction implements IPropertyChangeListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String ACTION_ID = "cde.SNAPTOGRID"; //$NON-NLS-1$

	public SnapToGridAction(IEditorPart part, ShowGridAction showGridAction) {
		super(part);
		setText(CDEMessages.getString("SnapToGridAction.label")); //$NON-NLS-1$
		setToolTipText(CDEMessages.getString("SnapToGridAction.tooltip")); //$NON-NLS-1$
		setId(ACTION_ID);
		setImageDescriptor(
			ImageDescriptor.createFromFile(getClass(),  CDEMessages.getString("SnapToGridAction.image"))); //$NON-NLS-1$
		setEnabled(false);
		showGridAction.addPropertyChangeListener(this);
		// It is assumed that both show grid action and snaptogrid action go away at the same time so no need to have a removeListener.
	}
	/**
	 * Create a command to move the selected objects to the nearest coordinates in the grid.
	 */
	protected Command createMoveCommand(List objects) {

		ChangeBoundsRequest request = new ChangeBoundsRequest(RequestConstants.REQ_MOVE);
		request.setEditParts(objects);
		request.setMoveDelta(new Point(0, 0));
		request.setLocation(new Point(0, 0));

		// Need to verify that the editparts are all EditParts, if they aren't then we can't align.
		Iterator itr = objects.iterator();
		while (itr.hasNext())
			if (!(itr.next() instanceof EditPart))
				return UnexecutableCommand.INSTANCE;
			
		CompoundCommand compoundCmd =
			new CompoundCommand(""); //$NON-NLS-1$
		for (int i = 0; i < objects.size(); i++) {
			EditPart object = (EditPart) objects.get(i);
			Command cmd = object.getCommand(request);
			if (cmd != null)
				compoundCmd.append(cmd);
		}
		return compoundCmd.isEmpty() ? null : compoundCmd.unwrap();
		// It is ok to not have something move (i.e. no commands)
	}
	/**
	 * @see EditorPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		List editParts = getSelectedObjects();
		boolean enabled = false;
		if (editParts.size() > 0) {
			GridController gridController = GridController.getGridController(getEditorPart());
			if (gridController.isGridShowing()) {
				Iterator itr = editParts.iterator();
				while (itr.hasNext()) {
					Object ep = itr.next();
					if (!(ep instanceof EditPart) ||
						!gridController.isEditPartRegistered(((EditPart)ep).getParent())) {
						enabled = false;
						break;
					}
					enabled = true;
				}
			}
		}

		return enabled;
	}
	/*
	 * Updates this action when the grid visibility changes
	 */
	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().equals(ShowGridAction.SHOW_GRID)) {
			refresh();
		}
	}

	/**
	 * Creates and executes the move command on the selected objects.
	 */
	public void run() {
		execute(createMoveCommand(getSelectedObjects()));
	}
}