package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.draw2d.geometry.Dimension;
import java.util.*;
/**
 * This is a visual component support class. It maintains
 * the listeners for a component and does the firing.
 * Implementers of IVisualComponent only need an instance
 * of this class to manage the listener lists and can forward
 * them to here. Use an ImageNotifierSupport class for 
 * the image notification portion.
 */
public class VisualComponentSupport {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected List componentListeners = null;	// Listeners for IComponentNotification.

	/**
	 * addComponentListener.
	 */
	public synchronized void addComponentListener(IVisualComponentListener aListener) {
		if (componentListeners == null)
			componentListeners = new ArrayList(2);
		componentListeners.add(aListener);
	}
	
	/**
	 * Fire component hidden notification.
	 */
	public void fireComponentHidden() {
		// Probably should make a copy of the notification list to prevent
		// modifications while firing, but we'll see if this gives any problems.
		if (componentListeners != null) {
			for (int i=0; i<componentListeners.size(); i++) {
				IVisualComponentListener listener = (IVisualComponentListener) componentListeners.get(i);
				listener.componentHidden();
			}
		}
	}
	
	/**
	 * Fire component moved notification.
	 */
	public void fireComponentMoved(int x, int y) {
		// Probably should make a copy of the notification list to prevent
		// modifications while firing, but we'll see if this gives any problems.
		if (componentListeners != null) {
			for (int i=0; i<componentListeners.size(); i++) {
				IVisualComponentListener listener = (IVisualComponentListener) componentListeners.get(i);
				listener.componentMoved(x , y);
			}
		}
	}
	/**
	 * Fire component refreshed notification.
	 */
	public void fireComponentRefreshed() {
		// Probably should make a copy of the notification list to prevent
		// modifications while firing, but we'll see if this gives any problems.
		if (componentListeners != null) {
			for (int i=0; i<componentListeners.size(); i++) {
				IVisualComponentListener listener = (IVisualComponentListener) componentListeners.get(i);
				listener.componentRefreshed();
			}
		}
	}
	/**
	 * Fire component resized notification.
	 */
	public void fireComponentResized(int width, int height) {
		// Probably should make a copy of the notification list to prevent
		// modifications while firing, but we'll see if this gives any problems.
		if (componentListeners != null) {
			for (int i=0; i<componentListeners.size(); i++) {
				IVisualComponentListener listener = (IVisualComponentListener) componentListeners.get(i);
				listener.componentResized(width , height);
			}
		}
	}
	/**
	 * Fire component shown notification.
	 */
	public void fireComponentShown() {
		// Probably should make a copy of the notification list to prevent
		// modifications while firing, but we'll see if this gives any problems.
		if (componentListeners != null) {
			for (int i=0; i<componentListeners.size(); i++) {
				IVisualComponentListener listener = (IVisualComponentListener) componentListeners.get(i);
				listener.componentShown();
			}
		}
	}
	
	/**
	 * removeComponentListener method..
	 */
	public synchronized void removeComponentListener(IVisualComponentListener aListener) {
		componentListeners.remove(aListener);
	}

}
