package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CommandWrapper;
import com.ibm.etools.emf.ref.RefStructuralFeature;

/**
 * This is used to add a set of annotations to the diagram data. Subclasses
 * may be used to add additional commands. For example, if there is a nameInComposition,
 * it would be desired that the name be unique, and this can't be known until the
 * time the set of annotations are actually added. Then the subclass (AddAnnotatationsWithNames)
 * would be used in that case, and it would look at the set of annotations being added and return
 * command to make them unique in the current editdomain.
 * @version 	1.0
 * @author
 */
public class AddAnnotationsCommand extends CommandWrapper {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain domain;
	protected List annotations;
	
	private static final RefStructuralFeature SF_ANNOTATIONS;
	
	static {
		SF_ANNOTATIONS = CDEUtilities.CDM_PACKAGE.getDiagramData_Annotations();	
	}
	
	/**
	 * Constructor for AddAnnotationsCommand.
	 */
	public AddAnnotationsCommand() {
	}
	
	public void setDomain(EditDomain domain) {
		this.domain = domain;
	}
	
	public void setAnnotations(List annotations) {
		this.annotations = annotations;
	}
	
	protected boolean prepare() {
		// Need to override prepare because prepare expects to have a command
		// create, and at the time of prepare being called, we don't have a command yet.
		return domain != null && annotations != null;
	}	
	
	public void execute() {
		if (command != null)
			super.execute();
		if (annotations.isEmpty())
			return;
			
		CommandBuilder cb = new CommandBuilder(null);
	
		cb.append(getAdditionalCommands(annotations, domain));
		cb.applyAttributeSettings(domain.getDiagramData(), SF_ANNOTATIONS, annotations);
		
		command = cb.getCommand();
		command.execute();
	}
	
	/**
	 * Override this to return additional commands to modify the annotations, if needed,
	 * for the current environment. This would be used if the actual modifications can't
	 * be known until the moment the annotations are to be added.
	 * 
	 * Note: The command to add the annotations to the diagramData will be append AFTER
	 * these commands.
	 */
    protected Command getAdditionalCommands(List annotations, EditDomain domain) {
    	return null;
    }
}
