package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.gef.commands.AbstractCommand;
import java.util.*;
/**
 * Command to replace an entire multi-valued attribute setting list for a many-valued
 * feature with a new list of values.
 */

public class ApplyAttributeSettingListCommand extends AbstractApplyAttributeCommand {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected List oldValues;
	
	public ApplyAttributeSettingListCommand(String name) {
		super(name);
	}	
	
	public ApplyAttributeSettingListCommand() {
		super();
	}		
	
	public boolean canExecute() {
		return super.canExecute() && feature.refIsMany();
	}	

	public void execute() {
		List values = (List) getTarget().refValue(feature);
		oldValues = new ArrayList(values);	// Save the old values
		try {
			values.clear();
			values.addAll(getAttributeSettingValues());
		} catch (RuntimeException e) {
			primUndo();
			throw e;
		}
	}
	
	protected void primUndo() {
		try {
			List values = (List) getTarget().refValue(feature);
			values.clear();
			if (!oldValues.isEmpty())
				values.addAll(oldValues);
			oldValues = null;	// Get rid of it so we don't hold onto it.
		} catch (RuntimeException e) {
			getTarget().refSetValue(feature, getAttributeSettingValues());
			throw e;
		}
	}
		
		
}