package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.common.command.AbstractCommand;
import com.ibm.etools.cdm.*;
/**
 * This command applies a keyed value to the keyed value holder.\
 */

public class ApplyKeyedValueCommand extends AbstractCommand {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected KeyedValueHolder target;
	protected String key;
	protected KeyedValue newValue;
	protected KeyedValue oldValue;
	
	public ApplyKeyedValueCommand(String name) {
		super(name);
	}
	
	public ApplyKeyedValueCommand() {
	}
	
	public void setTarget(KeyedValueHolder target) {
		this.target = target;
	}	
	
	public void setValue(KeyedValue value) {
		newValue = value;
	}
	
	public boolean canExecute() {
		return target != null && newValue != null && newValue.getKey() != null;
	}
	
	public boolean canUndo() {
		return true;
	}
	
	public void execute() {
		key = newValue.getKey();
		oldValue = target.setKeyedValue(key, newValue);
	}
	
	public void redo() {
		execute();
	}
	
	public void undo() {
		if (oldValue != null)
			target.setKeyedValue(key, oldValue);
		else
			target.unsetKeyedValue(key);
		oldValue = null;	// So that we don't hold onto it.
	}

}