package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.common.command.AbstractCommand;
import com.ibm.etools.cdm.*;
import com.ibm.etools.cde.*;
/**
 * This command will apply the model to the annotation.
 * It will not change the connection state of the annotation (it
 * won't make the annotation alive if not already alive).
 *
 * NOTE: the modelObject needs to be live so that its ID can be found.
 */

public class ApplyModelToAnnotationCommand extends AbstractCommand {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected AnnotationLinkagePolicy linkagePolicy;
	protected Object modelObject;
	protected Annotation annotation;
	protected Object oldModelObject;
	
	/**
	 * Construct the command. A null modelObject means to unset the linkage.
	 */
	public ApplyModelToAnnotationCommand(Object modelObject, Annotation annotation, AnnotationLinkagePolicy linkagePolicy) {
		this.modelObject = modelObject;
		this.annotation = annotation;
		this.linkagePolicy = linkagePolicy;
	}
	
	protected boolean prepare() {
		return annotation != null && linkagePolicy != null;
	}
	
	public void execute() {
		if (annotation instanceof AnnotationEMF) {
			oldModelObject = ((AnnotationEMF) annotation).getAnnotates();
		} else {
			oldModelObject = ((AnnotationGeneric) annotation).getAnnotatesID();
		}
		
		linkagePolicy.setModelOnAnnotation(modelObject, annotation);
	}
	
	public void redo() {
		execute();
	}
	
	public void undo() {
		if (annotation instanceof AnnotationEMF)
			linkagePolicy.setModelOnAnnotation(oldModelObject, annotation);
		else
			((GenericAnnotationLinkagePolicy) linkagePolicy).setAnnotationID((AnnotationGeneric) annotation, (String) oldModelObject);
			
		oldModelObject = null;	// So that we don't hold onto it.
	}
	
}