package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.common.command.*;
import com.ibm.etools.emf.ref.RefRegister;
import java.util.*;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cdm.model.*;
import com.ibm.etools.cdm.*;

/**
 * Apply either the new constraint ,the new size, or new location to the Visual constraint keyed value.
 * The three are mutually exclusive, the last set in the command will be what's applied.
 * Creation date: (06/05/00 3:04:51 PM)
 * @author: Administrator
 */
public class ApplyVisualConstraintCommand extends CommandWrapper {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Object fNewConstraint;
	protected KeyedValueHolder fTarget;
	protected static final Rectangle SDEFAULT_RECT = new Rectangle(Integer.MIN_VALUE, Integer.MIN_VALUE, -1, -1);
/**
 * ApplyVisualLocationCommand constructor comment.
 * @param desc java.lang.String
 */
public ApplyVisualConstraintCommand(String desc) {
	super(desc);
}
public void setTarget(KeyedValueHolder target) {
	fTarget = target;
}	
protected boolean prepare() {
	// Need to override prepare because prepare expects to have a command
	// create, and at the time of prepare being called, we don't have a command yet.
	return fTarget != null && fNewConstraint != null;
}

public void execute() {
	if (command != null)
		super.execute();
		
	CDMPackage cPkg = (CDMPackage) RefRegister.getPackage(CDMPackage.packageURI);
	CDMFactory cFact = cPkg.getCDMFactory();
	KeyedConstraint c = cFact.createKeyedConstraint();
	c.setKey(CDMModelConstants.VISUAL_CONSTRAINT_KEY);	
	if (fNewConstraint instanceof Rectangle) {
		c.setValue((Rectangle) fNewConstraint);
	} else if (fNewConstraint instanceof Point) {
		// Setting size instead of entire constraint.
		KeyedValue oldV = fTarget.getKeyedValue(CDMModelConstants.VISUAL_CONSTRAINT_KEY);
		// to be on the safe side, in case old is not a KeyedConstraint, or doesn't exist.
		Rectangle old = oldV instanceof KeyedConstraint ? ((KeyedConstraint) oldV).getValue() : SDEFAULT_RECT;
		// OldV must exist, can't set size if no old v of type constraint.
		c.setValue(new Rectangle(((Point) fNewConstraint).x, ((Point) fNewConstraint).y, old.width, old.height));
	} else if (fNewConstraint instanceof Dimension) {
		// Setting size instead of entire constraint.
		KeyedValue oldV = fTarget.getKeyedValue(CDMModelConstants.VISUAL_CONSTRAINT_KEY);
		// to be on the safe side, in case old is not a KeyedConstraint, or doesn't exist.
		Rectangle old = oldV instanceof KeyedConstraint ? ((KeyedConstraint) oldV).getValue() : SDEFAULT_RECT;
		// OldV must exist, can't set size if no old v of type constraint.
		c.setValue(new Rectangle(old.x, old.y, ((Dimension) fNewConstraint).width, ((Dimension) fNewConstraint).height));
	}
	
	CommandBuilder cbld = new CommandBuilder(getLabel());
	cbld.applyAttributeSetting(fTarget, c);
	command = cbld.getCommand();
	command.execute();
}

public void setConstraint(Object constraint) {
	if (constraint instanceof Rectangle || constraint instanceof Point || constraint instanceof Dimension)
		fNewConstraint = constraint;
}

/**
 * Set the constraint to apply.
 * Creation date: (06/05/00 3:06:38 PM)
 */
public void setRectangle(Rectangle newConstraint) {
	fNewConstraint = newConstraint;
}
/**
 * Set the point to apply.
 * Creation date: (06/05/00 3:06:38 PM)
 */
public void setLocation(Point newPoint) {
	fNewConstraint = newPoint;
}
/**
 * Set the size to apply.
 * Creation date: (06/05/00 3:06:38 PM)
 */
public void setSize(Dimension newSize) {
	fNewConstraint = newSize;
}

}
