package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.gef.commands.AbstractCommand;
import java.util.*;
/**
 * Command to clear out an entire attribute setting list for a many-valued
 * feature.
 */

public class ClearAttributeSettingListCommand extends AbstractAttributeCommand {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public ClearAttributeSettingListCommand(String name) {
		super(name);
	}	
	
	public ClearAttributeSettingListCommand() {
		super();
	}		
	
	public boolean canExecute() {
		return super.canExecute() && feature.refIsMany();
	}	

	public void execute() {
		List values = (List) getTarget().refValue(feature);
		setAttributeSettingValue(new ArrayList(values));	// Save the old values, the ArrayList ctor makes a copy of the list
		values.clear();
	}
	
	
	public void undo() {
		try {
			((List) getTarget().refValue(feature)).addAll(getAttributeSettingValues());
			setAttributeSettingValue(null);	// Get rid of it so we don't hold onto it.
		} catch (RuntimeException e) {
			((List) getTarget().refValue(feature)).clear();	// Clear it out to back out undo.
			throw e;
		}
	}
		
		
}