package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.emf.ref.*;
import java.util.*;
import com.ibm.etools.emf.notify.Notifier;

/**
 * Command to replace MOF attributes at indexed location.
 * It will replace the one already at that location.
 */
public class ReplaceAttributeSettingCommand extends AbstractApplyAttributeCommand {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Object oldValue;
	protected int fReplaceIndex = -1;
	protected Object fReplaceValue;
	
public ReplaceAttributeSettingCommand(String name) {
	super(name);
}	

public ReplaceAttributeSettingCommand() {
	super();
}		

public boolean canExecute() {
	return (fReplaceIndex != -1 || fReplaceValue != null) && feature.refIsMany() && getAttributeSettingValues().size() == 1 && super.canExecute();
}


public void execute() {		
	RefObject target = getTarget();
	List featureList = (List) target.refValue(feature);
	if (fReplaceValue != null) {
		// We want to replace a value. Find the position of that value.
		fReplaceIndex = featureList.indexOf(fReplaceValue);
	}

	oldValue = featureList.set(fReplaceIndex, getAttributeSettingValues().get(0));
}

public void undo() {	
	RefObject target = getTarget();
	
	((List) target.refValue(feature)).set(fReplaceIndex, oldValue);
	
	oldValue = null;	// Don't hold onto it.	
}

/**
 * Set the replacement index.
 */
public void setReplaceIndex(int newIndex) {
	fReplaceIndex = newIndex;
	fReplaceValue = null;	// Reset it to indicate index type requested.
}
/**
 * Set the insert before value. This is mutually exclusive with the
 * setInsertionIndex() method.
 */
public void setReplaceValue(Object replaceValue) {
	fReplaceIndex = -1;	// Reset it to indicate replace value requested.
	fReplaceValue = replaceValue;
}

}


