package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.decorators.ClassDescriptorDecorator;
import com.ibm.etools.cdm.KeyedString;
import com.ibm.etools.gef.requests.CreateRequest;
import com.ibm.etools.gef.requests.CreateRequest.Factory;
import com.ibm.etools.gef.tools.CreationTool;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.emf.ecore.*;

/**
 * CDE Creation Tool. It exposes the EditDomain and
 * if a factory implements IDomainedFactory, it will set
 * the domain into the factory when activated.
 * @version 	1.0
 * @author
 */
public class CDECreationTool extends CreationTool {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static String CREATION_POLICY_KEY = "com.ibm.etools.cde.creationtool.policy"; //$NON-NLS-1$
	protected CreationPolicy fCreationPolicy;
	protected boolean fHasLookedupCreationPolicy = false;

	/**
	 * Constructor for CDECreationTool.
	 */
	public CDECreationTool() {
	}

	/**
	 * Constructor for CDECreationTool.
	 * @param arg0
	 */
	public CDECreationTool(Factory factory) {
		super(factory);
	}

	/**
	 * Return the edit domain.
	 */
	public com.ibm.etools.gef.EditDomain getDomain() {
		return super.getDomain();
	}

	/**
	 * @see Tool#activate()
	 */
	public void activate() {
		super.activate();
		
		//Frog
		
		Factory fact = getFactory();
		if (fact instanceof IDomainedFactory)
			((IDomainedFactory) fact).setEditDomain((EditDomain) getDomain());
	}
	
	protected Command getCommand(){
		Command result = super.getCommand();
		Object type = getFactory().getObjectType();
		// There is an API where an implementor of CDECreationTool.CreationPolicy can register themselves
		// on a decorator with a key
		// The purpose of this is so that people can decorate their classes with CreationPolicy
		// that can put commands in front of and after the set of commands that are called on creation
		// This is cach'd for speed of lookup
		if ( fCreationPolicy == null && !fHasLookedupCreationPolicy) {
			if ( type instanceof EClassifier ) {
				ClassDescriptorDecorator decorator = (ClassDescriptorDecorator)ClassDecoratorFeatureAccess.getDecoratorWithKeyedFeature((EClassifier)type, ClassDescriptorDecorator.class, CREATION_POLICY_KEY);			
				if ( decorator != null ) {
					String creationPolicyClassName = ((KeyedString)decorator.getKeyedValue(CREATION_POLICY_KEY)).getValue();
					// The class name may be from another plugin so we must instantiate it correctly
					try {
						fCreationPolicy = (CreationPolicy)CDEPlugin.createInstance(null, creationPolicyClassName);
					} catch ( Exception exc ) {
						// If the class can't be created then just drop down and let the regular command be returned
					}
				}
			}
			fHasLookedupCreationPolicy = true;
		}
		if ( fCreationPolicy != null ) {
			return fCreationPolicy.getCommand(result,getCreateRequest());
		} else {
			return result;
		}
	}
	
	public interface CreationPolicy{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
        public Command getCommand(Command aCommand,CreateRequest createRequest);
        /**
         *  Optionally overides the default null constructor super string.
         *  e.g., super(new Arg1()) 
         *  @return null if no overide is needed, String if an overide exists
         */
        public String getDefaultSuperString(EClass superClass) ;
	}

}
