package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.ecore.EDecorator;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.NotImplementedException;
import com.ibm.etools.cdm.KeyedValueHolder;
import java.util.List;
/**
 * This class is a utility class to access decorators on the class
 * hierarchy.
 */

public class ClassDecoratorFeatureAccess {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private ClassDecoratorFeatureAccess() {
		// Never instantiated.
	}
	
	/**
	 * This will return the decorator that has the requested feature set in it.
	 * If the feature is single, then isSet is used. If the feature is many, then
	 * if the list is not empty, then it is considered to be set.
	 * It will return null if no decorator with that feature set was found.
	 */
	public static EDecorator getDecoratorWithFeature(EClassifier aClass, Class decoratorType, RefStructuralFeature feature) {
		if (aClass == null)
			return null;
			
		ClassDecoratorIterator classItr = new ClassDecoratorIterator(aClass, decoratorType);
		
		// Now for each decorator, determine if the feature is set, return it if it is set.
		while (classItr.hasNext()) {
			EDecorator dec = (EDecorator) classItr.next();
			try {
				if (feature.refIsMany()) 
					if (!((List) dec.refValue(feature)).isEmpty())
						return dec;
					else
						;
				else
					if (dec.refIsSet(feature))
						return dec;
			} catch (NotImplementedException e) {
				// This isn't a valid feature for this decorator, try next one.
			}
		}
		
		// None with a set feature was found.
		return null;
	}
	
	/**
	 * This will return the decorator that has the requested KeyedValue key set in it.
	 * It will return null if no decorator with that feature set was found.
	 *
	 */
	public static EDecorator getDecoratorWithKeyedFeature(EClassifier aClass, Class decoratorType, String key) {
		if (aClass == null)
			return null;
		ClassDecoratorIterator classItr = new ClassDecoratorIterator(aClass, decoratorType);
		
		// Now for each decorator, determine if the feature is set, return it if it is set.
		while (classItr.hasNext()) {
			EDecorator dec = (EDecorator) classItr.next();
			if (dec instanceof KeyedValueHolder)
				if (((KeyedValueHolder) dec).isSetKeyedValue(key))
					return dec;
		}
		
		// None with a set keyedValue was found.
		return null;
	}
	
}