package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import java.util.*;
import com.ibm.etools.emf.ecore.*;
import java.text.MessageFormat;

/**
 * This is an iterator to walk over the decorators of a specific kind.
 * It walks through the supertype hierarchy starting at the
 * given EModelElement.
 */
public class ClassDecoratorIterator implements Iterator {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Iterator fSuperItr;
	protected Object fNext;	// We always go one forward to simplify the hasNext().
	protected Class fDecoratorType;
	
	public ClassDecoratorIterator(EClassifier startWith, Class decoratorType) {
		fDecoratorType = decoratorType;
		if (startWith instanceof EGeneralizableElement)
			fSuperItr = ((EGeneralizableElement) startWith).getAllSupertypes().iterator();
		fNext = findDecorator(startWith);
		if (fNext == null) {
			fNext = findNext();	// Find the next one from the superType iterator
		}
	}
	
	public boolean hasNext() {
		return fNext != null;
	}
	
	public Object next() {
		Object result = fNext;
		fNext = findNext();
		return result;
	}
	
	public void remove() {
		throw new UnsupportedOperationException(MessageFormat.format(CDEEmfMessages.getString("ClassDecoratorIterator.UnsupportedRemove(Object)_EXC_"), new Object[] {getClass().getName()})); //$NON-NLS-1$
	}
	
	protected Object findDecorator(EModelElement eClass) {
		Iterator decorItr = eClass.getEDecorators().iterator();
		while (decorItr.hasNext()) {
			Object decor = decorItr.next();
			if (fDecoratorType.isInstance(decor))
				return decor;
		}
		return null;
	}
	
	/**
	 * From the current supertype iterator, find the next matching decorator, if any.
	 */
	protected Object findNext() {
		if (fSuperItr == null)
			return null;	// This wasn't a EGeneralizableElement, so there aren't any supertypes.
		if (fSuperItr.hasNext()) {
			Object next = findDecorator((EModelElement) fSuperItr.next());
			if (next == null)
				return findNext();	// This class didn't have one, go on.
			return next;
		}
		return null;
	}
}