package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.editparts.AbstractGraphicalEditPart;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.cde.DefaultComponentEditPolicy;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.emf.ref.RefObject;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;
import java.net.*;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ecore.EClassifier;
import org.eclipse.jface.viewers.ILabelProvider;
/**
 * A simple graphical editpart for a EMF object. It is simply
 * an icon (somepart.gif) with a label which is the EMF ID of the
 * object. This can be used as a default for any object that don't
 * know what to do with. This is marked in ClassDescriptorDecoratorPolicy
 * as the default graph edit part.
 */

public class DefaultGraphicalEditPart extends AbstractGraphicalEditPart {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected DefaultLabelProviderNotifier labelProviderNotifier;
	
	public void activate() {
		super.activate();
		labelProviderNotifier = new DefaultLabelProviderNotifier();
		labelProviderNotifier.setModel((RefObject) getModel(), EditDomain.getEditDomain(this), new DefaultLabelProviderNotifier.IDefaultLabelProviderListener() {
			public void refreshLabel(ILabelProvider provider) {
				DefaultGraphicalEditPart.this.refreshVisuals(provider);
			}
			
		}, ClassDescriptorDecoratorPolicy.getPolicy(this).getLabelProvider(
				(EClassifier) ((RefObject) getModel()).refMetaObject()));
		refreshVisuals();	// Because refreshVisuals is called normally BEFORE activation.
	}
	
	public void deactivate() {
		if (labelProviderNotifier != null) {
			ILabelProvider provider = labelProviderNotifier.getLabelProvider();
			labelProviderNotifier.setModel(null, null, null, null);
			labelProviderNotifier = null;
			if (provider != null)
				provider.dispose();
		}
		super.deactivate();
	}
	
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new DefaultComponentEditPolicy());
	}
	
	protected IFigure createFigure() {
		Label label = new Label("?");	 //$NON-NLS-1$
		label.setTextPlacement(PositionConstants.SOUTH);	
		return label;
	}
	
	protected void refreshVisuals() {
		if (labelProviderNotifier != null)
			refreshVisuals(labelProviderNotifier.getLabelProvider());
	}
	
	private void refreshVisuals(ILabelProvider provider) {
		Label fig = (Label) getFigure();
		if (provider != null) {
			setFigureImage(fig,provider.getImage(getModel()));
			setFigureLabel(fig,provider.getText(getModel()));
		} else {
			fig.setText("?"); //$NON-NLS-1$
		}
	}
	
	protected void setFigureLabel(Label aLabel,String aString){
		aLabel.setText(aString);
	}
	
	protected void setFigureImage(Label aLabel,Image anImage){
		aLabel.setIcon(anImage);
	}
	
	public Object getAdapter(Class key) {
		if (key == IPropertySource.class)
			return ((RefObject) getModel()).getAdapter(IPropertySource.class);
		else
			return super.getAdapter(key);
	}

}