package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.RefObject;
import org.eclipse.jface.viewers.LabelProvider;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.propertysheet.INeedData;
import org.eclipse.swt.graphics.Image;
/**
 * A default label provider that uses the ID of the EMF object
 * as the string and the image comes from the ClassDescriptorDecorator.
 */
public class DefaultLabelProvider extends LabelProvider implements INeedData {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain domain;
		
	public void setData(Object data) {
		if (data instanceof EditDomain)
			domain = (EditDomain) data;
	}
	
	/**
	 * The <code>LabelProvider</code> implementation of this 
	 * <code>ILabelProvider</code> method returns the element's <code>toString</code>
	 * string. Subclasses may override.
	 */
	public String getText(Object element) {
		if (!(element instanceof RefObject))
			return super.getText(element);
		
		RefObject model = (RefObject) element;
		String id =  model.refID();
		if (id == null) {
			// We need to get one assigned
			model.refSetID();
			id = model.refID();
		}
		
		return id;
	}
	
	/**
	 * The default image comes from the ClassDescriptorDecorator.
	 */
	public Image getImage(Object element) {
		if (!(element instanceof RefObject))
			return super.getImage(element);
			
		return ClassDescriptorDecoratorPolicy.getPolicy(domain).getIcon((EClassifier) ((RefObject) element).refMetaObject()).getImage();
	}
}