package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jface.viewers.*;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.AnnotationPolicy.AnnotationListener;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;

/**
 * This can be used by EMF editparts to encapsulate handling a label provider,
 * listening for changes in the label provider, listening for changes to the
 * EMF model object, and notify when the label needs to be updated.
 * 
 * When no longer needed, call with setModel(null, null, null) to remove all listening.
 * 
 * @version 	1.0
 * @author
 */
public class DefaultLabelProviderNotifier {

	public interface IDefaultLabelProviderListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		/**
		 * Notify to refresh the visuals.
		 * @param provider - The provider to use
		 */
		public void refreshLabel(ILabelProvider provider);
	}

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected RefObject model;
	protected ILabelProvider labelProvider;
	protected IDefaultLabelProviderListener listener;
	protected AnnotationListener annotationListener;
	protected ILabelProviderListener labelProviderListener;
	protected EditDomain domain;
	{
		labelProviderListener = new ILabelProviderListener() {
			public void labelProviderChanged(LabelProviderChangedEvent event) {
				// We'll be conservative and refresh for any change.
				listener.refreshLabel(labelProvider);
			}
		};
	}

	/**
	 * Constructor for DefaultLabelProviderNotifier.
	 */
	public DefaultLabelProviderNotifier() {
	}

	/**
	 * Return the label provider for ad-hoc requests.
	 */
	public ILabelProvider getLabelProvider() {
		return labelProvider;
	}
	
	/**
	 * Set a new provider. It should not be null.
	 */
	public void setLabelProvider(ILabelProvider provider) {
		if (model != null) {
			if (this.labelProvider != null)
				this.labelProvider.removeListener(labelProviderListener);
			this.labelProvider = provider;
			if (this.labelProvider != null) {
				this.labelProvider.addListener(labelProviderListener);
				
				if (annotationListener == null) {
					annotationListener = new AnnotationListener(model, domain, new IEMFListener() {
						public void notifyChanged(
							Notifier notifier,
							int eventType,
							RefObject sf,
							Object oldValue,
							Object newValue,
							int pos) {
							if (DefaultLabelProviderNotifier.this.labelProvider == null)
								return;
							if (sf instanceof RefStructuralFeature) {
								// It's supposed to be, but it may be null
								if (DefaultLabelProviderNotifier.this.labelProvider
									.isLabelProperty(DefaultLabelProviderNotifier.this.model, ((RefStructuralFeature) sf).refName()))
									DefaultLabelProviderNotifier.this.listener.refreshLabel(labelProvider);
							}
						}
					});
				}
			}
		}
	}
		
	
	/**
	 * Set up and start listening.
	 * To stop listening, send in null for everything.
	 */
	public void setModel(RefObject model, EditDomain domain, IDefaultLabelProviderListener listener, ILabelProvider provider) {
		if (this.model != null) {
			// release old
			if (this.labelProvider != null)
				this.labelProvider.removeListener(labelProviderListener);
			this.labelProvider = null;
			if (annotationListener != null)
				annotationListener.removeListening();
			annotationListener = null;
			this.model = null;
			this.domain = null;
		}

		if (model != null) {
			this.model = model;
			this.domain = domain;
			this.listener = listener;
			if (provider != null)
				setLabelProvider(provider);
		}
	}

}