package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.editparts.AbstractTreeEditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.emf.ref.RefObject;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.swt.graphics.Image;
import java.net.*;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.DefaultComponentEditPolicy;
import com.ibm.etools.cde.TreePrimaryDragRoleEditPolicy;
import com.ibm.etools.emf.ecore.EClassifier;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
/**
 * A simple tree editpart for a EMF object. It is simply
 * an icon (somepart.gif) with a label which is the EMF ID of the
 * object. This can be used as a default for any object that don't
 * know what to do with. This is marked in ClassDescriptorDecoratorPolicy
 * as the default graph edit part.
 */
public class DefaultTreeEditPart extends AbstractTreeEditPart {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ILabelDecorator labelDecorator;
	protected DefaultLabelProviderNotifier labelProviderNotifier;
	
	public void activate() {
		super.activate();
		ILabelProvider provider = getDecoratedLabelProvider();
		labelProviderNotifier = new DefaultLabelProviderNotifier();
		labelProviderNotifier.setModel(
				(RefObject) getModel(),
				EditDomain.getEditDomain(this),
				new DefaultLabelProviderNotifier.IDefaultLabelProviderListener() {
					public void refreshLabel(ILabelProvider provider) {
						DefaultTreeEditPart.this.refreshVisuals(provider);
					}
				},
				provider);
		refreshVisuals(provider);	// Because refreshVisuals is usually called BEFORE activation.
	}

	private ILabelProvider getDecoratedLabelProvider() {
		ILabelProvider provider =
			ClassDescriptorDecoratorPolicy.getPolicy(this).getLabelProvider(
				(EClassifier) ((RefObject) getModel()).refMetaObject());
		if (labelDecorator != null) {
			provider = new DecoratingLabelProvider(provider, labelDecorator);
		}
		return provider;
	}
	
	public void deactivate() {
		if (labelProviderNotifier != null) {
			ILabelProvider provider = labelProviderNotifier.getLabelProvider();
			labelProviderNotifier.setModel(null, null, null, null);
			labelProviderNotifier = null;
			if (provider instanceof DecoratingLabelProvider)
				provider = ((DecoratingLabelProvider) provider).getLabelProvider();	// Really only want to dispose provider, not label decorator, that will be reused.
			if (provider != null)
				provider.dispose();		
		}
		super.deactivate();
	}	
	
	public DefaultTreeEditPart(Object model) {
		setModel(model);
	}
	
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new DefaultComponentEditPolicy());
		installEditPolicy(EditPolicy.PRIMARY_DRAG_ROLE, new TreePrimaryDragRoleEditPolicy());
	}
	
		
	protected void refreshVisuals() {
		super.refreshVisuals();
		if (labelProviderNotifier != null)
			refreshVisuals(labelProviderNotifier.getLabelProvider());
	}
	
	private void refreshVisuals(ILabelProvider provider) {
		if (provider != null) {
			setWidgetImage(provider.getImage(getModel()));
			setWidgetText(provider.getText(getModel()));
		} else {
			setWidgetText("?"); //$NON-NLS-1$
		}	
	}
	
	/**
	 * This is used to add a label decorator to the label provider.
	 * The decorator is not owned by this edit part. If it needs to
	 * be disposed, that is the responsibility of the caller.
	 */
	public void setLabelDecorator(ILabelDecorator decorator) {
		labelDecorator = decorator;
		if (isActive()) {
			ILabelProvider provider = labelProviderNotifier.getLabelProvider();
			if (provider instanceof DecoratingLabelProvider)
				((DecoratingLabelProvider) provider).setLabelDecorator(labelDecorator);
			else {
				ILabelProvider newProvider = new DecoratingLabelProvider(
					provider,
					labelDecorator);
				labelProviderNotifier.setLabelProvider(newProvider);
			}
			refreshVisuals(labelProviderNotifier.getLabelProvider());
		}
	}
	
	public Object getAdapter(Class key) {
		if (key == IPropertySource.class)
			return ((RefObject) getModel()).getAdapter(IPropertySource.class);
		else
			return super.getAdapter(key);
	}
	
}


