package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.WrappedException;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.logging.util.Level;

/**
 * This is a creation factory that takes a string referencing
 * the EMF class to create. The string is in the form of an
 * href (i.e. "xyz.xmi#XYZClass"). This is called the URI form.
 * @version 	1.0
 * @author
 */
public class EMFClassCreationFactory extends EMFCreationFactory implements IDomainedFactory {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected String className;
	
	/**
	 * Constructor for EMFCreationStringFactory.
	 * @param aClassName
	 */
	public EMFClassCreationFactory(String aClassName) {
		super(null);
		
		setObjectTypeName(aClassName);
	}
	
	public void setObjectTypeName(String aClassName) {
		className = aClassName;
	}
	
	

	/**
	 * @see IDomainedFactory#setEditDomain(EditDomain)
	 */
	public void setEditDomain(EditDomain domain) {
		// Now that we have a domain, find and set the actual class.
		// We are being activated, so this is a good time to do it.
		
		ResourceSet rset = EMFEditDomainHelper.getResourceSet(domain);
		try {
			setObjectType(rset.getObjectAndLoad(className));
		} catch (WrappedException e) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, e);
		}
	}

}
