package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.emf.ecore.EMetaObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
/**
 * This is an implementation of EMF containment that is for a single
 * structural feature. This will be the case the majority of the time.
 */
public abstract class EMFContainerPolicy extends AbstractEMFContainerPolicy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected RefStructuralFeature containmentSF;
	
	public EMFContainerPolicy(RefStructuralFeature containmentSF, EditDomain domain) {
		super(domain);
		this.containmentSF = containmentSF;
	}
	
	public Command getCreateCommand(Object child, Object positionBeforeChild) {
		return getCreateCommand(child, positionBeforeChild, containmentSF);
	}
	
	
	public Command getAddCommand(List children, Object positionBeforeChild) {
		return getAddCommand(children, positionBeforeChild, containmentSF);
	}
	
	public Command getDeleteDependentCommand(Object child) {
		return getDeleteDependentCommand(child, containmentSF);
	}
	
	public Command getMoveChildrenCommand(List children, Object positionBeforeChild) {
		return getMoveChildrenCommand(children, positionBeforeChild, containmentSF);
	}
	
	public Command getOrphanChildrenCommand(List children) {
		return getOrphanChildrenCommand(children, containmentSF);
	}

	protected boolean isValidChild(Object child) {
		// By default it is the type of the structural feature. 
		return ((EMetaObject) containmentSF.refType()).isInstance(child);
	}

}