package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.cde.AlignmentAction;
import com.ibm.etools.emf.resource.*;
import org.eclipse.ui.part.*;
import com.ibm.etools.gef.ui.actions.*;
import com.ibm.etools.gef.*;
import com.ibm.etools.common.command.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.*;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import com.ibm.etools.gef.internal.GEFMessages;
import com.ibm.etools.gef.palette.PaletteRoot;
import com.ibm.etools.gef.ui.palette.PaletteViewerImpl;
import org.xml.sax.SAXException;
import com.ibm.xmi.base.*;
import com.ibm.etools.gef.ui.palette.PaletteViewer;
import java.util.*;
import com.ibm.etools.gef.ui.parts.*;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import java.io.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.resource.impl.*;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.internal.IWorkbenchGraphicConstants;
import org.eclipse.ui.internal.WorkbenchImages;
import org.eclipse.ui.*;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import java.lang.reflect.InvocationTargetException;
import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.dialogs.ContainerGenerator;

import java.text.MessageFormat;
import org.eclipse.jface.action.*;
import org.eclipse.ui.views.properties.*;

import com.ibm.etools.logging.util.Level;
import com.ibm.etools.propertysheet.*;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.EditDomain;

import org.eclipse.swt.events.*;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionListener;

import com.ibm.etools.cdm.DiagramData;
import com.ibm.etools.cdm.Diagram;
import com.ibm.etools.cdm.CDMFactory;
import com.ibm.etools.cdm.CDMPackage;
/**
 * Base class to use for MOG Graphical Editors.
 * It is abstract and must be specialized.
 *
 * This will be changed in the future. IT needs to be
 * redesigned and may go away.
 */
public abstract class EMFGraphicalEditorPart
	extends EditorPart
{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected boolean wantResourceListener = true;
// This class listens to changes to the file system in the workspace, and 
// makes changes accordingly.
// 1) An open, saved file gets deleted -> close the editor
// 2) An open file gets renamed or moved -> change the editor's input accordingly	
class ResourceTracker implements IResourceChangeListener, IResourceDeltaVisitor {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected boolean resourcesChanged = false;
	public void resourceChanged(IResourceChangeEvent e) {
		IResourceDelta delta = e.getDelta();
		try {
			if (delta != null) {
				// The accept will execute visit(IResourceDelta delta) and
				// visit all the children of the delta resource if visit 
				// returns true.
				delta.accept(this);
				// If any resources were changed during the visit calls,
				// make the calls to handle this by notifying the editparts, etc.
				if (resourcesChanged) {
					// run this separately since Markers may be involved which
					// can't be changed while were in the middle of a resourceChanged().
					Display display = getSite().getShell().getDisplay();
					display.asyncExec(new Runnable() {
						public void run() {
							// Calling setInput() would have the undesired effect of flushing
							// the command stack.  To avoid that, superSetInput() method is called.
							handleResourceChanged();
						}
					});
					resourcesChanged = false;
				}
			}
		} catch (CoreException x) {
			// What should be done here?
		}
	}
	public boolean visit(IResourceDelta delta) {
		if (delta == null)
			return true;
		IResource dres = delta.getResource();
		if (delta.getResource().equals(((IFileEditorInput) getEditorInput()).getFile())) {
			if (delta.getKind() == IResourceDelta.REMOVED) {
				if ((IResourceDelta.MOVED_TO & delta.getFlags()) == 0) { // if the file was deleted
					// NOTE: The case where an open, unsaved file is deleted is being handled by the 
					// PartListener added to the Workbench in the initialize() method.
					if (!isDirty())
						closeEditor(false);
				} else { // else if it was moved or renamed
					final IFile newFile = ResourcesPlugin.getWorkspace().getRoot().getFile(delta.getMovedToPath());
					Display display = getSite().getShell().getDisplay();
					display.asyncExec(new Runnable() {
						public void run() {
							// Calling setInput() would have the undesired effect of flushing
							// the command stack.  To avoid that, superSetInput() method is called.
							superSetInput(new FileEditorInput(newFile));
						}
					});
				}
			}
		} else {
			// If the resource is a file, see if it's part of our resource set.
			// If so, remove it from our resource set and let the subclasses
			// handle the rest.
			// Note: ignore marker changes.
			if (delta.getKind() == IResourceDelta.CHANGED
				&& ((IResourceDelta.MARKERS & delta.getFlags()) == 0)
				&& delta.getResource().getType() == IResource.FILE) {
				String filename = delta.getResource().getFullPath().makeRelative().toString();
				ResourceSet rset = getResourceSet();
				if (rset != null) {
					Iterator iter = getResourceSet().getResources().iterator();
					while (iter.hasNext()) {
						Resource res = (Resource) iter.next();
						if (res.getURI().getFile().equals(filename)) {
							CDEHack.fixMe(); // MOF has a major bug that has to be fixed here before we remove this
							// resource from the resource set. Otherwise  we can't save the this file
							// because we throw a NullPointerException later on trying to resolve 
							// this resource's workspace.
							// !!!! UNCOMMENT THE NEXT LINE when the bug gets fixed. !!!!
							//			getResourceSet().remove(res);
							resourcesChanged = true;
							// Remove the following line when the code is fixed.
							createWarningMessage(filename);
							break;
						}
					}
				}
			}
		}
		return true;
	}
}

private ActionRegistry actionRegistry;
private DefaultEditDomain domain;
protected EToolsPropertySheetPage fPropertySheetPage;
protected ListenerList fPropertySheetSelectionListeners = new ListenerList(1);

protected EditPartViewer primaryViewer;
protected PaletteViewer paletteViewer;

protected Resource fResource;
protected ResourceSet fResourceSet;

protected SelectionSynchronizer selectionSynchronizer;

private boolean fDirtyState = false;

// Remove the following 2 fields after a MOF bug is fixed. See resourceTracker for details.
private boolean needToCloseEditor = false; 
private String filenameOfChangedResource = ""; //$NON-NLS-1$

protected CommandStackListener fDirtyListener = new CommandStackListener() {
	public void commandStackChanged(EventObject event){
		setDirty(true);
	}
};

protected IResourceChangeListener resourceListener;

public EMFGraphicalEditorPart() {
}

private KeyHandler outlineKeyHandler;
private KeyHandler canvasKeyHandler;
/**
 * Returns the KeyHandler for the outline view
 */
protected KeyHandler getOutlineKeyHandler(){
	if (outlineKeyHandler == null){
		outlineKeyHandler = new KeyHandler();
		outlineKeyHandler.put(KeyStroke.getPressed(SWT.DEL, 0),getOutlineDeleteAction());
	}
	return outlineKeyHandler;
}
/**
 * Returns the KeyHandler for the canvas view
 */
protected KeyHandler getCanvasKeyHandler(){
	if (canvasKeyHandler == null){
		canvasKeyHandler = new KeyHandler();
		canvasKeyHandler.put(KeyStroke.getPressed(SWT.DEL, 0),getCanvasDeleteAction());
	}
	return canvasKeyHandler;
}
protected EditorPartAction fOutlineDeleteAction;
protected EditorPartAction fCanvasDeleteAction;

/* The delete action always goes to the current selection to see who should be deleted
 * This is to avoid the problem where the user selects the editor part with a right mouse click
 * and no GEF selection event is fired if the selection hasn't changed
 * The current selection comes from the selection provider argument, thus allowing the action on the outline
 * and GEF canvas to be different.  Attempts to try and get the canvas edit part for both fail because of JavaBeans
 * like TableColumn that are only on the tree, and attempts to use the outline are doomed for the same reason plus the
 * fact there is no guarantee that the JavaBeans viewer not been closed by the user
 * This we use a model where the selection provider is used for actions and we create different instances of the 
 * same action for the tree and canvas pop-up menu
 */
public static class DeleteAction extends EditorPartAction{
	private ISelectionProvider fSelectionProvider;
	public DeleteAction(EditorPart editorPart, ISelectionProvider selectionProvider){
		super(editorPart);
		fSelectionProvider = selectionProvider;	
	}
	protected void init(){
		super.init();
		setText(getGEFDeleteLabel());
		setToolTipText(getGEFDeleteTooltip());
		setId(org.eclipse.ui.IWorkbenchActionConstants.DELETE);
		setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_DELETE_EDIT_HOVER));
		setImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_DELETE_EDIT));
		setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_DELETE_EDIT_DISABLED));
		setEnabled(false);
		}
		public boolean isEnabled(){
			Command deleteCommand = com.ibm.etools.gef.ui.actions.DeleteAction.createDeleteCommand(getSelectedEditParts());
			return deleteCommand != null && deleteCommand.canExecute();
		}
		public void run(){
			execute(com.ibm.etools.gef.ui.actions.DeleteAction.createDeleteCommand(getSelectedEditParts()));
		}
		protected java.util.List getSelectedEditParts(){
			ISelection selection = fSelectionProvider.getSelection();
			if ( selection != null ) {
				return ((IStructuredSelection)selection).toList();
			}
			return Collections.EMPTY_LIST;
		}
	}

protected EditorPartAction getOutlineDeleteAction(){
	if (fOutlineDeleteAction == null){
		fOutlineDeleteAction = new EMFGraphicalEditorPart.DeleteAction(this,getContentOutlinePage());
	}
	return fOutlineDeleteAction;
}
protected EditorPartAction getCanvasDeleteAction(){
	if (fCanvasDeleteAction == null){
		fCanvasDeleteAction = new EMFGraphicalEditorPart.DeleteAction(this,getPrimaryViewer());
	}
	return fCanvasDeleteAction;
}
// The storing and caching of these fields is because GEFMessages is an internal package that is explicitly not exported in its plugin manifest
public static String GEF_DELETE_LABEL;
public static String getGEFDeleteLabel(){
	if ( GEF_DELETE_LABEL == null ) {
		org.eclipse.core.runtime.IPluginDescriptor desc = org.eclipse.core.runtime.Platform.getPluginRegistry().getPluginDescriptor("com.ibm.etools.gef");//$NON-NLS-1$
		try{
			GEF_DELETE_LABEL = desc.getResourceString("%DeleteAction.ActionLabelText_UI_"); //$NON-NLS-1$
		}catch(MissingResourceException e){
			GEF_DELETE_LABEL = "%DeleteAction.ActionLabelText_UI_"; //$NON-NLS-1$
		}
	}
	return GEF_DELETE_LABEL;
}
public static String GEF_DELETE_TOOLTIP;
public static String getGEFDeleteTooltip(){
	if ( GEF_DELETE_TOOLTIP == null ) {
		org.eclipse.core.runtime.IPluginDescriptor desc = org.eclipse.core.runtime.Platform.getPluginRegistry().getPluginDescriptor("com.ibm.etools.gef");//$NON-NLS-1$
		try{
			GEF_DELETE_TOOLTIP = desc.getResourceString("%DeleteAction.ActionToolTipText_UI_");	//$NON-NLS-1$
		}catch(MissingResourceException e){
			GEF_DELETE_TOOLTIP = "%DeleteAction.ActionLabelText_UI_"; //$NON-NLS-1$
		}
	}
	return GEF_DELETE_TOOLTIP;
}
/** Closes this editor. */
protected void closeEditor( final boolean save ){
	Display display = getSite().getShell().getDisplay();
	display.asyncExec( new Runnable(){
		public void run(){
			getSite().getPage().closeEditor( EMFGraphicalEditorPart.this, save );
		}
	} );
}

protected ActionRegistry createActionRegistry(){
	return new ActionRegistry();
}

/**
 * This may be overridden to supply a subclass of 
 * DefaultEditDomain.
 */
protected DefaultEditDomain createEditDomain(){
	com.ibm.etools.cde.EditDomain dom = new com.ibm.etools.cde.EditDomain(this);
	dom.setAnnotationLinkagePolicy(createLinkagePolicy());
	// Give it a default do nothing special model controller.
	dom.setData(IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY, new IModelChangeController() {
		private boolean fHoldChanges = false ;
		public synchronized void setHoldChanges(boolean flag, String msg) {
			fHoldChanges = flag ;			
		}	
	    public synchronized boolean isHoldChanges() {
	    	return fHoldChanges ;
	    }
		public boolean run(Runnable runnable) {
		  if (!fHoldChanges) {
			runnable.run();
			return true;
		  }
		  else
		    return false ;
		}
		public String getHoldMsg() {
			return null ;
		}
	});
	return dom;
}

/**
 * This should be overridden if a different linkage policy is needed.
 */
protected AnnotationLinkagePolicy createLinkagePolicy() {
	return new EMFAnnotationLinkagePolicy();
}

protected ContextMenuProvider canvasContextMenuProvider;
protected ContextMenuProvider getCanvasContextMenuProvider(){
	if (canvasContextMenuProvider == null) {
		createCanvasContextMenuProvider();
	}
	return canvasContextMenuProvider;
}
protected ContextMenuProvider outlineContextMenuProvider;
protected ContextMenuProvider getOutlineContextMenuProvider(){
	if (outlineContextMenuProvider == null) {
		createOutlineContextMenuProvider();
	}
	return outlineContextMenuProvider;
}
protected void createCanvasContextMenuProvider(){
	canvasContextMenuProvider = new EMFContextMenuProvider(this,getCanvasDeleteAction());
}
protected void createOutlineContextMenuProvider(){
	outlineContextMenuProvider = new EMFContextMenuProvider(this,getOutlineDeleteAction());
}
protected void createPrimaryViewer(){
	primaryViewer = new GraphicalViewerImpl();
	primaryViewer.setRootEditPart( new GraphicalRootEditPartWithOverview() );
	primaryViewer.setEditDomain(getDomain());
	((com.ibm.etools.cde.EditDomain) getDomain()).setViewerData(primaryViewer, com.ibm.etools.cde.ZoomController.ZOOM_KEY, new com.ibm.etools.cde.ZoomController());
	((com.ibm.etools.cde.EditDomain) getDomain()).setViewerData(primaryViewer, com.ibm.etools.cde.GridController.GRID_KEY, new com.ibm.etools.cde.GridController());
	((com.ibm.etools.cde.EditDomain) getDomain()).setViewerData(primaryViewer, com.ibm.etools.cde.DistributeController.DISTRIBUTE_KEY, new com.ibm.etools.cde.DistributeController((GraphicalViewer) primaryViewer));
	primaryViewer.setKeyHandler(new GraphicalViewerKeyHandler((GraphicalViewer)primaryViewer).setParent(getCanvasKeyHandler()));
	primaryViewer.setContextMenuProvider(getCanvasContextMenuProvider());
	// The registry action holds a delete action because this is what the toolbar uses
	actionRegistry.registerAction(getCanvasDeleteAction());	
	getDomain().addViewer(primaryViewer);
}

public void createPrimaryViewerControl(Composite parent) {
	Canvas canvas = new Canvas(parent, SWT.BORDER);
	getPrimaryViewer().setControl(canvas);
}
public void createPaletteControl(Composite parent){
	primCreatePaletteControl(parent);
}
protected void primCreatePaletteControl(Composite parent) {
	Canvas canvas = new Canvas(parent, SWT.NONE);
//	canvas.setLayoutData(new Splitter.Weight(100,0,0));
	if (paletteViewer == null){
		paletteViewer = new PaletteViewerImpl();
	}
	paletteViewer.setControl(canvas);
	getDomain().setPaletteViewer(paletteViewer);
	// Subclasses can supply their own palette, otherwise it is read from the
	// plugin extension point palette="nnn" where nnn is an href to the palette
	// in the plugin.xml file that defined this editor.
	setPaletteRoot();	
}

/**
 * By now we have parsed the document (see readDocument()) and any severe errors
 * will have been logged as markers via handleException(Exception, IFile) and 
 * through validateModel(RefObject).
 * If there are no severe errors, create the palette and primary viewer control,
 * otherwise, create a part control to show the errors in case the task list is
 * not showing.
 */
public void createPartControl(Composite c) {
	SashForm splitter = new SashForm(c, SWT.HORIZONTAL);
	createPaletteControl(splitter);
	createPrimaryViewerControl(splitter);
	splitter.setWeights(new int []{1,4});
}
/**
 * Create controls to display the errors (markers).
 */
protected void createPartControlWithErrors(Composite parent, IMarker[] markers) {
	// First consolidate the markers with messages into a String array.
	String[] errorMessages = new String [markers.length];
	for (int i=0; i<markers.length; i++) {
		try {
			if (markers[i].exists() || 
				markers[i].getAttribute(IMarker.MESSAGE) != null) 
			{
				String msg = (String)markers[i].getAttribute(IMarker.MESSAGE);
				// Include line number in the message if there is one
				if (markers[i].getAttribute(IMarker.LINE_NUMBER) != null) {
					Integer linenum = (Integer) markers[i].getAttribute(IMarker.LINE_NUMBER);
					msg = MessageFormat.format(CDEEmfMessages.getString("Line_number"), new Object[] {msg, linenum.toString()}); //$NON-NLS-1$
				}
				errorMessages[i] = msg;
			}
		} catch ( CoreException exc ) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, exc.getStatus());
		}
	}
	// Now create the SWT controls to show the messages in a list.
	Composite composite= new Composite(parent, SWT.NONE);
	composite.setLayout(new GridLayout());
	composite.setLayoutData(new GridData());
//	composite.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL));
	
	Text text = new Text(composite,SWT.MULTI|SWT.READ_ONLY|SWT.WRAP);
	text.setForeground(text.getDisplay().getSystemColor(SWT.COLOR_RED));
	text.setBackground(text.getDisplay().getSystemColor(SWT.COLOR_GRAY));
	text.setText(getErrorTextMessage());
	text.setLayoutData(new GridData());
	
	text = new Text(composite,SWT.SINGLE|SWT.READ_ONLY);
	text.setForeground(text.getDisplay().getSystemColor(SWT.COLOR_BLUE));
	text.setBackground(text.getDisplay().getSystemColor(SWT.COLOR_GRAY));
	text.setText(CDEEmfMessages.getString("List_of_errors")); //$NON-NLS-1$
	text.setLayoutData(new GridData());

	List list= new List(composite, SWT.BORDER|SWT.V_SCROLL|SWT.H_SCROLL);
	list.setItems(errorMessages);
	list.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL|GridData.GRAB_HORIZONTAL));
}

public void dispose() {
	try {
		if (getEditorInput() != null && resourceListener != null) {
			IFile file = ((IFileEditorInput) getEditorInput()).getFile();
			file.getWorkspace().removeResourceChangeListener(resourceListener);
		}
		if (partListener != null)
			getSite().getWorkbenchWindow().getPartService().removePartListener(partListener);		
	} catch (Throwable t) {
		CDEPlugin.getPlugin().getLog().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", t)); //$NON-NLS-1$
	}

	try {
		getDomain().setTool(null);
	} catch (Throwable t) {
		CDEPlugin.getPlugin().getLog().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", t)); //$NON-NLS-1$
	}
	try {
		getActionRegistry().dispose();
		if (getPrimaryViewer() != null)
			getPrimaryViewer().dispose();
	} catch (Throwable t) {
		CDEPlugin.getPlugin().getLog().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", t)); //$NON-NLS-1$
	}
	try {
		if (getPaletteViewer() != null)
			getPaletteViewer().dispose();
	} catch (Throwable t) {
		CDEPlugin.getPlugin().getLog().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", t)); //$NON-NLS-1$
	}
	try {
		((com.ibm.etools.cde.EditDomain) getDomain()).dispose();
	} catch (Throwable t) {
		//		JBCFPlugin.log(t) ;
	}
	super.dispose();
}

protected ActionRegistry getActionRegistry() {
	if (actionRegistry == null)
		actionRegistry = createActionRegistry();
	return actionRegistry;
}

public Object getAdapter(Class type) {
	if (type == IPropertySheetPage.class)
		return getPropertySheetPage();
	if (type == GraphicalViewer.class)
		return getPrimaryViewer();
	if (type == IContentOutlinePage.class)
		return getContentOutlinePage();
	if (type == CommandStack.class)
		return getDomain().getCommandStack();
	if (type == ActionRegistry.class)
		return getActionRegistry();
	if (type == EditDomain.class)
		return (EditDomain)getDomain();
	if (type == PaletteRoot.class)
		return getPaletteRoot();
	return super.getAdapter(type);
}
/**
 * Do not force creation of the property sheet page
 * This is because the property sheet viewer may not have been created yet (i.e. user didn't want it), so if it has not been created
 * yet then store the listener for when the page may get created
 */
public void addPropertySheetSelectionChangeListener(ISelectionChangedListener aSelectionListener){
	if ( fPropertySheetPage != null ) {
		fPropertySheetPage.addSelectionChangedListener(aSelectionListener);
	}	
	fPropertySheetSelectionListeners.add(aSelectionListener);	// Always add because if property sheet needs to be recreated we need all of them again.
}

public void removePropertySheetSelectionChangelListener(ISelectionChangedListener aSelectionListener){
	if ( fPropertySheetPage != null ) {
		fPropertySheetPage.removeSelectionChangedListener(aSelectionListener);
	}
	fPropertySheetSelectionListeners.remove(aSelectionListener);
}

/**
 * So that users can know if the property sheet has been created.
 * If it hasn't been created (or has been disposed) then they
 * know and they won't call getPropertySheetPage to create it.
 */
public final boolean isPropertySheetCreated() {
	return fPropertySheetPage != null;
}

public final EToolsPropertySheetPage getPropertySheetPage(){
	if ( fPropertySheetPage == null ){
		fPropertySheetPage = createPropertySheetPage();
		if (!fPropertySheetSelectionListeners.isEmpty()){
			Object[] listeners = fPropertySheetSelectionListeners.getListeners();
			for (int i = 0; i < listeners.length; i++) {
				fPropertySheetPage.addSelectionChangedListener((ISelectionChangedListener) listeners[i]);
			}
		}
		
		fPropertySheetPage.setRootEntry(createRootPropertySheetEntry());		
		
		fPropertySheetPage.addListener(new EToolsPropertySheetPage.Listener() {
			/**
			 * @see com.ibm.etools.propertysheet.EToolsPropertySheetPage.Listener#controlCreated()
			 */
			public void controlCreated(Control control) {
				control.addDisposeListener(new DisposeListener() {
					/**
					 * @see org.eclipse.swt.events.DisposeListener#widgetDisposed(DisposeEvent)
					 */
					public void widgetDisposed(DisposeEvent e) {
						fPropertySheetPage = null;	// We no longer have one.
					}
				});
			}
	
		});
		
	}
	return fPropertySheetPage;
}

protected EToolsPropertySheetPage createPropertySheetPage() {
	return new EToolsPropertySheetPage(){
		public void setActionBars(IActionBars actionBars) {
			super.setActionBars(actionBars);
			// The menu and toolbars have RetargetActions for UNDO and REDO
			// Set an action handler to redirect these to the action registry's actions so they work when the property sheet is enabled
			ActionRegistry registry = getActionRegistry();			
			actionBars.setGlobalActionHandler(IWorkbenchActionConstants.UNDO,registry.getAction(IWorkbenchActionConstants.UNDO));
			actionBars.setGlobalActionHandler(IWorkbenchActionConstants.REDO,registry.getAction(IWorkbenchActionConstants.REDO));
		}		
	};
}

protected AbstractPropertySheetEntry createRootPropertySheetEntry(){
	return new CommandStackPropertySheetEntry(getCommandStack(), null, null);
}

protected CommandStack getCommandStack(){
	return getDomain().getCommandStack();
}

public final DefaultEditDomain getDomain(){
	if (domain == null) {
		domain = createEditDomain();
		// Listen to the command stack so that the first time a command is pushed we make ourselves
		// dirty
		getDomain().getCommandStack().addCommandStackListener(fDirtyListener);
	}
	return domain;
}

public EditPartViewer getPrimaryViewer() {
	return primaryViewer;
}

public EditPartViewer getPaletteViewer() {
	return paletteViewer;
}


public class EMFContentOutlinePage extends ContentOutlinePage {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public EMFContentOutlinePage(EditPartViewer viewer ){
		super(viewer);
	}
		
	public void setActionBars(IActionBars actionBars) {
		ActionRegistry registry = getActionRegistry();
	
		super.setActionBars(actionBars);

		// The menu and toolbars have RetargetActions for DELETE, UNDO and REDO
		// Set an action handler to redirect these to the action registry's actions so they work
		// with the content outline without having to separately contribute these
		// to the outline page's toolbar
		actionBars.setGlobalActionHandler(IWorkbenchActionConstants.DELETE,getOutlineDeleteAction());
		actionBars.setGlobalActionHandler(IWorkbenchActionConstants.UNDO,registry.getAction(IWorkbenchActionConstants.UNDO));
		actionBars.setGlobalActionHandler(IWorkbenchActionConstants.REDO,registry.getAction(IWorkbenchActionConstants.REDO));		
	}
	
	/*
	 * override for further configuration
	 */
	protected void configureOutlineViewer(){
		getViewer().setEditDomain(getDomain());
		getDomain().addViewer(getViewer());
		getViewer().setContextMenuProvider(getOutlineContextMenuProvider());
		getViewer().setKeyHandler(getOutlineKeyHandler());
	}
	
	public void createControl(Composite parent){
		super.createControl(parent);
		configureOutlineViewer();
		hookOutlineViewer();
		initializeOutlineViewer();
	}
	
	public void dispose(){
		unhookOutlineViewer();
		getDomain().removeViewer(getViewer());		
		super.dispose();
	}
	
	protected void hookOutlineViewer(){
		getSelectionSynchronizer().addViewer(getViewer());
	}

	protected void initializeOutlineViewer(){
		getViewer().setContents(getRootTreeEditPart());
	}
	
	protected void unhookOutlineViewer(){
		getSelectionSynchronizer().removeViewer(getViewer());
	}	
}

/*
 * Cache and return the outline page
 */
protected EMFContentOutlinePage outlinePage;

protected EMFContentOutlinePage getContentOutlinePage(){
	if ( outlinePage == null ) {
		outlinePage = createOutlinePage();
	}
	return outlinePage;
}

protected EMFContentOutlinePage createOutlinePage(){
	return new EMFContentOutlinePage(new TreeViewer());
}

/**
 * This will get the palette root, or the default, and set it into the EditDomain.
 */
protected void setPaletteRoot() {
	PaletteRoot palette = getPaletteRoot();
	if ( palette == null ) {
		// Use default CDE pallete.
		try {
			palette = (PaletteRoot) getResourceSet().getObjectAndLoad("plugin.com.ibm.etools.cde/cde_palette.xmi#cde_palette"); //$NON-NLS-1$
		} catch (RuntimeException e) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, e);
		}
	}
	getDomain().setPaletteRoot(palette);
}
protected String getPaletteHRef(){
	return getConfigurationElement().getAttributeAsIs("palette"); //$NON-NLS-1$
}
/**
 * Return the root of the palette to use for this
 * EditorPart. It may be changed at later time by
 * getting the domain and setting it in there.
 */
protected PaletteRoot getPaletteRoot() {
	String paletteHRef = getPaletteHRef();
	if ( paletteHRef != null ) {
		try {
			return (PaletteRoot) getResourceSet().getObjectAndLoad(paletteHRef);
		} catch (RuntimeException e) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING,e);
		}
	}	
	
	return null;
}

/**
 * Starting up with an input.
 */
public void init(IEditorSite site, IEditorInput input) 
	throws PartInitException 
{
	// Check input.
	if (!(input instanceof IFileEditorInput))
		throw new PartInitException(MessageFormat.format(CDEMessages.getString(CDEMessages.NOT_FILE_INPUT), new Object[] {input.getName()}));

	// Save input.
	setSite(site);
	initialize();
	setInput(input);
}

private IPartListener partListener;
protected void initialize() {
	initActionRegistry();
	initViewers();
	if (wantResourceListener) {
		// If an open, unsaved file was deleted, query the user to either do a "Save As"
		// or close the editor.		
		partListener = new IPartListener() {
			public void partActivated(IWorkbenchPart part) {
				if (part != EMFGraphicalEditorPart.this)
					return;
				if (!((IFileEditorInput) getEditorInput()).getFile().exists()) {
					Shell shell = getSite().getShell();
					String title = CDEEmfMessages.getString("FILE_DELETED_TITLE_UI_"); //$NON-NLS-1$
					String message = CDEEmfMessages.getString("FILE_DELETED_WITHOUT_SAVE_INFO_"); //$NON-NLS-1$
					String[] buttons = { CDEEmfMessages.getString("SAVE_BUTTON_UI_"), CDEEmfMessages.getString("CLOSE_BUTTON_UI")}; //$NON-NLS-1$ //$NON-NLS-2$
					MessageDialog dialog = new MessageDialog(shell, title, null, message, MessageDialog.QUESTION, buttons, 0);
					if (dialog.open() == 0) {
						if (!performSaveAs())
							partActivated(part);
					} else {
						closeEditor(false);
					}
				}
			}
			public void partBroughtToTop(IWorkbenchPart part) {
			}
			public void partClosed(IWorkbenchPart part) {
			}
			public void partDeactivated(IWorkbenchPart part) {
			}
			public void partOpened(IWorkbenchPart part) {
			}
		};
		getSite().getWorkbenchWindow().getPartService().addPartListener(partListener);
	}
}

public boolean isDirty() {
	return fDirtyState;
}

public boolean isSaveAsAllowed() {
	return true;
}

protected void initActionRegistry(){
	getActionRegistry(); // Force lazy init of the ActionRegistry
	actionRegistry.registerAction(new UndoAction(this));
	actionRegistry.registerAction(new RedoAction(this));
	actionRegistry.registerAction(new SaveAction(this));

	actionRegistry.registerAction(new com.ibm.etools.cde.ZoomAction(this));
	actionRegistry.registerAction(new com.ibm.etools.cde.ZoomInAction(this));
	actionRegistry.registerAction(new com.ibm.etools.cde.ZoomOutAction(this));
	
	com.ibm.etools.cde.ShowGridAction sgAction = new com.ibm.etools.cde.ShowGridAction(this);
	actionRegistry.registerAction(sgAction);
	actionRegistry.registerAction(new com.ibm.etools.cde.GridPropertiesAction(this, sgAction));
	actionRegistry.registerAction(new com.ibm.etools.cde.SnapToGridAction(this, sgAction));
	
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.LEFT_ALIGN));
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.CENTER_ALIGN));
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.RIGHT_ALIGN));
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.TOP_ALIGN));
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.MIDDLE_ALIGN));
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.BOTTOM_ALIGN));
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.MATCH_WIDTH));
	actionRegistry.registerAction(new AlignmentAction(this, AlignmentCommandRequest.MATCH_HEIGHT));

	actionRegistry.registerAction(new ShowDistributeBoxAction(this));
	actionRegistry.registerAction(new DistributeAction(this, DistributeCommandRequest.HORIZONTAL));
	actionRegistry.registerAction(new DistributeAction(this, DistributeCommandRequest.VERTICAL));
	
	//registry.registerAction(new OutlineToggleViewAction(this));
	
	
}

protected void initViewers(){
	createPrimaryViewer();
	getSite().setSelectionProvider(getPrimaryViewer());
	// Keep the graph view and tree view in sync
	getSelectionSynchronizer().addViewer(getPrimaryViewer());
}
protected SelectionSynchronizer getSelectionSynchronizer(){
	if (selectionSynchronizer == null)
		selectionSynchronizer = new SelectionSynchronizer();
	return selectionSynchronizer;
}

public void setActionRegistry(ActionRegistry registry) {
	actionRegistry = registry;
}

public void setFocus(){
	Control c = getPrimaryViewer().getControl();
	if (c != null && !c.isDisposed()) {
CDEHack.fixMe();  // Remove the following "if block" after a MOF bug is fixed. See resourceListener for details.
		if (needToCloseEditor) {
			String msg = java.text.MessageFormat.format(CDEEmfMessages.getString("Resource_changed_please_close_editor_message_WARN_"),  //$NON-NLS-1$
						new Object[] {filenameOfChangedResource});
			Shell shell= getSite().getShell();
			MessageDialog.openWarning(shell, CDEEmfMessages.getString("Warning_message_dialog_title_WARN_"), msg); //$NON-NLS-1$
			needToCloseEditor = false;
			return;
		}
		c.setFocus();
	}
}

/**
 * Set the dirty state. Only signal the first time it goes dirty until
 * it becomes clean.
 */
protected void setDirty(boolean dirty) {
	// If being set clean, signal. Else if not already dirty, then signal.
	if (!dirty || !fDirtyState) {
		fDirtyState = dirty;
		firePropertyChange(PROP_DIRTY);
	}
}

/**
 * Create and return an empty document.  This 
 * method should be implemented to create a new document
 * and a new root within the document. It should be
 * created within the given ResourceSet.
 * 
 * NOTE: There really shouldn't be empty resources, they
 * should be created correctly by wizards, but as a fallback
 * this method is available.
 */
protected abstract Resource createEmptyResource(String filename, ResourceSet rset);


/**
 * Get the base Model object of the resource being edited. This will open
 * the file.
 */
protected RefObject getModel(IFile file) throws CoreException, IOException, Exception {

	RefObject rootModel = null;
	InputStream is = null;
	try {
		is = file.getContents();
	} catch (CoreException exc) {
		// look for the specific case where the file is not in sync
		if (exc.getStatus().getCode() == IResourceStatus.OUT_OF_SYNC_LOCAL) {
			// refresh from local and try it again
			file.refreshLocal(IResource.DEPTH_ZERO, null);
			is = file.getContents();
		} else {
			throw exc;
		}
	}
	try {
		if (is.available() == 0) {  // New file
			fResource = createEmptyResource(file.getLocation().toString(), getResourceSet());
		} else {
			fResource = readDocument(file.getFullPath().toString(), is);
		}
		if (fResource != null) {
			rootModel = getModel(fResource);
		}
	} finally {
		try {
			is.close();	// Make sure the stream is closed
		} catch (IOException e) {
		}
	}
	
	return rootModel;
}

/**
 * Subclasses need to call this when they get the DiagramData.
 */
protected void setupDiagramData(DiagramData dd) {
	((com.ibm.etools.cde.EditDomain) getDomain()).setDiagramData(dd);
	Diagram d = (Diagram) dd.refResource().getObject(URIFactoryRegister.getFactory().makeURI(" #"+Diagram.PRIMARY_DIAGRAM_ID)); //$NON-NLS-1$
	if (d != null)
		((com.ibm.etools.cde.EditDomain) getDomain()).setViewerData(getPrimaryViewer(), com.ibm.etools.cde.EditDomain.DIAGRAM_KEY, d);	// Put out the diagram for the viewer.

}

/**
 * The File has been parsed into a document which is the argument
 * Return the RefObject you want to be the model of the base EditPart
 */
protected abstract RefObject getModel(Resource doc);

/**
 * The RefObject argument is the base object returned by
 * getModel(Resource).  Return the edit part you want to be used
 * as the base ( top level ) edit part by the graph viewer
 */
protected abstract EditPart getEditPart(RefObject rootModel);

/**
 * The RefObject argument is the base object returned by
 * getModel(Resource).  Return the tree edit part you want to be used
 * as the base ( top level ) tree edit part by the outline viewer
 */
protected abstract EditPart getTreeEditPart(RefObject rootModel);


/**
 * Return the resourceSet. This is public so that it can be retrieved
 * by others.
 */
public final ResourceSet getResourceSet() {
	if (fResourceSet == null) {
		fResourceSet = createResourceSet();
		initializeResourceSet(fResourceSet);
	}
	return fResourceSet;
}

/**
 * Return the resource. This is public so that it can be retrievd by others.
 */
public final Resource getResource() {
	return fResource;
}



/**
 * Create the ResourceSet to use and initialize it
 */
protected ResourceSet createResourceSet() {
//	ResourceSet rs = new HierarchicalResourceSet();
	ResourceSet rs = new ResourceSetImpl();
	Context context = new ContextImpl();
//	context.setURIConverter(new EMFWorkbenchURIConverterImpl());
	context.setURIConverter(new WorkbenchURIConverterImpl());
	rs.setContext(context);
	return rs;
}

protected void initializeResourceSet(ResourceSet rs){
	rs.getContext().setParent(getGlobalResourceSet().getContext());
	EMFEditDomainHelper.setResourceSet(rs,(EditDomain)getDomain());	
}	
	
/**
 * The global resourceSet is where we store resources that are to be available
 * to all MOF Editors.
 */
private static ResourceSet sGlobalResourceSet;
public final static ResourceSet getGlobalResourceSet(){
	if (sGlobalResourceSet == null) {
		sGlobalResourceSet = new ResourceSetImpl();
		sGlobalResourceSet.setContext(new ContextImpl());
	}
	return sGlobalResourceSet;
}


protected Resource  readDocument(String uri, InputStream is) throws Exception { 
	Resource res = null;
	// If an error occurs reading or loading the xmi file,
	// the handle the exception and create markers for the errors.
	try {
		res = getResourceSet().load(uri, is);
		return res;
		// Catch parse exceptions
	} catch ( SAXException exc ) {
		// What to do here ???/
	}
	return null;
}
/**
 * Return the error message to be displayed in the viewer when there are
 * errors found parsing the document.
 * Subclasses can override with their own specific error message.
 */
protected String getErrorTextMessage() {
	return CDEEmfMessages.getString("Error_text_message_ERROR_"); //$NON-NLS-1$
}
protected String getErrorMessageFrom(WarningException exc){
	String msg = ""; //$NON-NLS-1$
	if ( exc instanceof BadTypeException ){
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_bad_type_EXC_"),  //$NON-NLS-1$
				new Object[] {((BadTypeException)exc).getValue()});
	} else if ( exc instanceof BadValueException ) {
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_bad_value_EXC_"),  //$NON-NLS-1$
				new Object[] {((BadValueException)exc).getValue()});
	} else if ( exc instanceof com.ibm.xmi.base.ClassNotFoundException ) {
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_missing_class_EXC_"),  //$NON-NLS-1$
				new Object[] {((com.ibm.xmi.base.ClassNotFoundException)exc).getName()});
	} else if ( exc instanceof FeatureNotFoundException ) {
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_missing_feature_EXC_"),  //$NON-NLS-1$
				new Object[] {((FeatureNotFoundException)exc).getName()});
	} else if ( exc instanceof NamespaceNotFoundException ) {
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_missing_namespace_EXC_"),  //$NON-NLS-1$
				new Object[] {((NamespaceNotFoundException)exc).getName()});
	} else if ( exc instanceof PackageNotFoundException ) {
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_missing_package_EXC_"),  //$NON-NLS-1$
				new Object[] {((PackageNotFoundException)exc).getPrefix()});
	} else if ( exc instanceof UnresolvedReferenceException ) {
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_bad_reference_EXC_"),  //$NON-NLS-1$
				new Object[] {((UnresolvedReferenceException)exc).getId()});
	} else {
		msg = MessageFormat.format(CDEEmfMessages.getString("Warning_exception_error_EXC_"),  //$NON-NLS-1$
				new Object[] {exc.getLocalizedMessage()});
	}
	return msg;
}
/**
 * Subclasses need to handle when a marker is selected from the task list.
 * Basically, find the edit part for this model and select it so the user
 * can correct the problem.
 */
public abstract void gotoMarker(IMarker marker);

/**
 * Subclasses need to handle when a resource changes.
 */
protected abstract void handleResourceChanged();

/**
 * Save the current resource back out.
 */
public void doSave(IProgressMonitor monitor) {
	// If the underlying editor input is null, or the resource file does not exist, do a 
	// "Save As" instead.
	if( getEditorInput() == null || ((IFileEditorInput)getEditorInput()).getFile() == null ||
			!((IFileEditorInput)getEditorInput()).getFile().exists() ){
		if( isSaveAsAllowed() )
			doSaveAs();
		// What should be done if "Save As" is not allowed?
		// Pop up an error message?
		return;
	}
	
	save(monitor);
}	

/**
 * Perform the save.
 */
protected void save(IProgressMonitor monitor) {
	
	com.ibm.etools.cde.EditDomain dom = (com.ibm.etools.cde.EditDomain) getDomain();	
	com.ibm.etools.common.command.Command cmd = dom.getAnnotationLinkagePolicy().cleanupDiagramData(dom.getDiagramData());
	if (cmd != null)
		cmd.execute();
		
	WorkspaceModifyOperation op= new WorkspaceModifyOperation() {	
		public void execute(final IProgressMonitor monitor) throws CoreException {		
			monitor.beginTask(CDEMessages.getString(CDEMessages.SAVING), 2000);
			try {		
				IFile file = ((IFileEditorInput) getEditorInput()).getFile();
				ByteArrayOutputStream os = new ByteArrayOutputStream();
				try {
					RefObject rootmodel = getModel(fResource);
					if (rootmodel != null) {
						validateModel(rootmodel);
					}
					fResource.save(os);
				} catch (Exception e) {
					throw new CoreException(new Status(IStatus.ERROR, CDEPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, MessageFormat.format(CDEMessages.getString(CDEMessages.SAVE_ERROR), new Object[] {file.getFullPath().toString()}), e));		
				}
				monitor.worked(1000);
				InputStream stream= new ByteArrayInputStream(os.toByteArray());
				file.setContents(stream, false, true, new SubProgressMonitor(monitor, 1000));				
			} catch (Exception e) {
				throw new CoreException(new Status(IStatus.ERROR, CDEPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, MessageFormat.format(CDEMessages.getString(CDEMessages.SAVE_ERROR), new Object[] {fResource.getURI().toString()}), e));
			}
			setDirty(false);
			monitor.done();
		}
	};
	
	try {
		op.run(monitor);
	} catch (InterruptedException e) {
	} catch (InvocationTargetException e) {
		Throwable eReal = e.getTargetException();
		IStatus status = null;
		if (eReal instanceof CoreException) {
			status = ((CoreException) eReal).getStatus();
		} else {
			status = new Status(IStatus.ERROR, CDEPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, MessageFormat.format(CDEMessages.getString(CDEMessages.SAVE_ERROR), new Object[] {fResource.getURI().toString()}), eReal);
		}
		CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, status);
		showErrorMessage(status.getMessage());
	}
	
	

}

/**
 * Select a new name to save it as, and then save it.
 */
public void doSaveAs() {
	performSaveAs();
}

/**
 * Returns a boolean indicating whether the user decided to save, or to
 * cancel.  True, if saved; false otherwise.
 */
protected boolean performSaveAs(){
	Shell shell= getSite().getShell();

	SaveAsDialog dialog= new SaveAsDialog(shell);
	dialog.setOriginalFile(((IFileEditorInput) getEditorInput()).getFile());
	dialog.open();
	IPath path= dialog.getResult();
	if (path == null)
		return false;

	final IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(path);
	
	com.ibm.etools.cde.EditDomain dom = (com.ibm.etools.cde.EditDomain) getDomain();
	com.ibm.etools.common.command.Command cmd = dom.getAnnotationLinkagePolicy().cleanupDiagramData(dom.getDiagramData());
	if (cmd != null)
		cmd.execute();

	WorkspaceModifyOperation op= new WorkspaceModifyOperation() {
		public void execute(final IProgressMonitor monitor) throws CoreException {
			monitor.beginTask(CDEMessages.getString(CDEMessages.SAVING_AS), 2000);
			saveAs(new SubProgressMonitor(monitor,1000), file, fResource);
			setInput(new FileEditorInput(file));
			monitor.done();			
		}
	};

	try {
		ProgressMonitorDialog monitorDialog = new ProgressMonitorDialog(shell);
		monitorDialog.run(false, false, op); // Not cancelable.
	} catch (InterruptedException e) {
	} catch (InvocationTargetException e) {
		Throwable eReal = e.getTargetException();
		IStatus status = null;
		if (eReal instanceof CoreException) {
			status = ((CoreException) eReal).getStatus();
		} else {
			status = new Status(IStatus.ERROR, CDEPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, MessageFormat.format(CDEMessages.getString(CDEMessages.SAVE_ERROR), new Object[] {path.toString()}), eReal);
		}
		CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, status);
		showErrorMessage(status.getMessage());
	}
	return true;
}

protected void saveAs(IProgressMonitor monitor, IFile file, Resource resource) throws CoreException {
	monitor.beginTask("", 3000); //$NON-NLS-1$
	ByteArrayOutputStream os = new ByteArrayOutputStream();
	try {
		resource.save(os);
	} catch (Exception e) {
		throw new CoreException(new Status(IStatus.ERROR, CDEPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, MessageFormat.format(CDEMessages.getString(CDEMessages.SAVE_ERROR), new Object[] {file.getFullPath().toString()}), e));		
	}
	
	InputStream stream= new ByteArrayInputStream(os.toByteArray());
	if (file.exists()) {
		file.setContents(stream, false, true, new SubProgressMonitor(monitor, 2000));
	} else {
		ContainerGenerator locator= new ContainerGenerator(file.getParent().getFullPath());
		locator.generateContainer(new SubProgressMonitor(monitor, 1000));
		file.create(stream, false, new SubProgressMonitor(monitor, 1000));
	}
	
	monitor.done();
}

/*
 * Called by outline page to set the contents
 */
protected EditPart getRootTreeEditPart() {
	return getTreeEditPart(getRootModel());	
}

private RefObject rootModel;
protected RefObject getRootModel() {
	return rootModel;
}

/**
 * A new input has been entered for this editpart. It can be assumed that it is
 * an IFileEditorInput because that must be tested for within the init(...) method
 * override.
 */
protected void setInput(IEditorInput input) {
	if (!input.equals(getEditorInput())) {
		superSetInput(input);
		IFileEditorInput fileInput = (IFileEditorInput) input;
		IFile file = fileInput.getFile();
		try { 
			Resource oldResource = fResource;	// Need to save it so that it can be unloaded after new file put into the viewer.
			// Get the model. If any severe problems occur, markers are created 
			// via the exceptions thrown (see handleException(Exception, IFile)).
			
			com.ibm.etools.cde.EditDomain dom = (com.ibm.etools.cde.EditDomain) getDomain();
			// Need to release the old diagram data.
			dom.setDiagramData(null);
			
			rootModel = getModel(file);
			// It is important that if any exceptions are thrown that this method continues through to get a root edit part and then
			// sets it as the contents on the edit part.  If not the user sees a grey screen.
			// The kind of exceptions that might be thrown are generally found in some kind of adapter that comes across a code pattern it 
			// can't recognize.  Rather than give the user a grey screen give them one a white one that may be recoverable
			try { 
				initializeWithNewRoot(rootModel);
			} catch (Exception exc){
				CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, exc);				
			}
			if (rootModel != null) {
				EditPart baseEditPart = getEditPart(rootModel);
				getPrimaryViewer().setContents(baseEditPart);
				// Validate the model. It's here that the editorpart 
				// should create any errors as markers to be shown
				// in the task list.
				validateModel(rootModel);
				getCommandStack().flush();	// Clear the command stack
				if (oldResource != null)
					getResourceSet().remove(oldResource);
			}
			
		} catch (Exception exc) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, exc);
		}
	}
	// This has been moved out of the if statement so as to handle the case
	// when the user saves a file as itself.
	setDirty(false);	// Mark not dirty
}

/*
 * A new root model has been retrieved. This is called just before
 * the model is wrappered within edit parts.
 */
protected void initializeWithNewRoot(RefObject newModel){
}

protected void showErrorMessage( String message ) {
	Shell shell= getSite().getShell();
	MessageDialog.openError(shell, CDEMessages.getString(CDEMessages.ERROR_TITLE), message);
}

// This method is mainly a result of code-factoring.  Also, there was a need to call
// super.setInput() from one of the inner classes (ResourceTracker).
protected void superSetInput( IEditorInput input ){
	
	// The workspace never changes for an editor.  So, removing and re-adding the 
	// resourceListener is not necessary.  But it is being done here for the sake
	// of proper implementation.  Plus, the resourceListener needs to be added 
	// to the workspace the first time around.
	if( getEditorInput() != null && resourceListener != null ){
		IFile file = ((IFileEditorInput)getEditorInput()).getFile();
		file.getWorkspace().removeResourceChangeListener( resourceListener );
	}
	
	super.setInput( input );
	
	if( getEditorInput() != null ){
		IFile file = ((IFileEditorInput)getEditorInput()).getFile();
		if (wantResourceListener) {
			if (resourceListener == null)
				resourceListener =  new ResourceTracker();
			file.getWorkspace().addResourceChangeListener( resourceListener );
		}
		setTitle( file.getName() );
	}
}
/**
 * Validate the model. It's here that subclasses should create any soft (or severe)
 * errors as markers to be shown in the task list. 
 *
 * Note: Plugin specific markers can be used here to allow editor parts to create 
 *       and manage only those markers (errors) that are relevent to them.
 */
protected abstract void validateModel(RefObject model);
/**
 * This code is temporary and should be removed once a MOF bug is fixed.
 * See the resourceTracker for more information.
 */
protected void createWarningMessage(String filename) {
CDEHack.fixMe();  // Remove this method after a MOF bug is fixed. See resourceListener for details.
	needToCloseEditor = true;
	filenameOfChangedResource = filename;
}
}
