package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.notify.Notification;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.cdm.*;
import com.ibm.etools.cde.KeyedValueNotificationHelper;
/**
 * This is a simple adapter to listen for notifications on an EMF object and then
 * to notify listener that implements IEMFListener.
 * IEMFListener is an very light-weight interface that only has
 * the notification method. This adapter is a simple notifier. It will only
 * notify one object, the listener. This listener pattern
 * would be used when just simple notification is all that
 * is required so that the listener doesn't need to implement
 * all of the other requirements of an adapter.
 *
 * It will also handle any KeyedValueHolder notifications if
 * this option is turned on.
 */
public class EMFListenerAdapter extends AdapterImpl {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IEMFListener listener = null;
	protected boolean listenForKeyValues = false;
	
	public EMFListenerAdapter() {
	}
	
	public EMFListenerAdapter(IEMFListener listener) {
		this.listener = listener;
	}
	
	public EMFListenerAdapter(boolean listenForKeyValues) {
		this.listenForKeyValues = listenForKeyValues;
	}
	
	public EMFListenerAdapter(IEMFListener listener, boolean listenForKeyValues) {
		this.listener = listener;		
		this.listenForKeyValues = listenForKeyValues;
	}
			
	public void setListener(IEMFListener listener) {
		this.listener = listener;
	}
	
	public void setKeyedValueListening(boolean listenForKeyValues) {
		this.listenForKeyValues = listenForKeyValues;
	}
	
	
	public void notifyChanged(Notification msg) {			
		if (listenForKeyValues && KeyedValueNotificationHelper.notifyChanged(this, msg.getNotifier(), msg.getEventType(), msg.getStructuralFeature(), msg.getOldValue(), msg.getNewValue(), msg.getPosition()))
			return;	
		listener.notifyChanged(msg.getNotifier(), msg.getEventType(), msg.getStructuralFeature(), msg.getOldValue(), msg.getNewValue(), msg.getPosition());
	}
}