package com.ibm.etools.cde.palette;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.emf.ref.RefBaseObject;
import com.ibm.etools.gef.EditDomain;
import com.ibm.etools.gef.tools.CreationTool;

import java.util.List;




import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.cde.decorators.ClassDescriptorDecorator;
import com.ibm.etools.cde.emf.CDECreationTool;
import com.ibm.etools.cde.emf.ClassDecoratorFeatureAccess;
import com.ibm.etools.cdm.KeyedString;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.emf.ecore.*;
/**
 * A creation tool entry where the actual object created is determined by the selectionCreation class. This allows the actual object created to be determined at selection time rather than statically defined in the palette entry itself.
 */
public interface SelectionCreationToolEntry extends CreationToolEntry {


	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * This is the interface that this tool entry will use to select the
	 * object at activation time on the tool. It will return a new object
	 * when called. It is passed this entry so that it can access further
	 * info, such as the ResourceSet for creating the entry.
	 */
	public interface ISelector {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		/**
		 * Return a two element array where element [0] is the new object and
		 * element [1] is the type of the object (e.g. java.lang.Class if a
		 * java object, or an EClassifier if the object is an instance of a
		 * EMF EClassifier).
		 *
		 * Note: Return null if new object can't be created.
		 */
		public Object[] getNewObjectAndType(SelectionCreationTool selectionCreationTool);
	}
	
	/**
	 * SelectionCreationTool
	 */
	public abstract static class SelectionCreationTool extends CreationTool {
		
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		protected boolean fHasLookedupCreationPolicy = false;
		protected CDECreationTool.CreationPolicy fCreationPolicy;
		/**
		 * Return the edit domain.
		 */
		public com.ibm.etools.gef.EditDomain getDomain(){
			return super.getDomain();
		}
		
		/**
		 * Return the SelectionCreationToolEntry
		 */
		public abstract SelectionCreationToolEntry getSelectionToolEntry();
		
		protected Command getCommand(){
			Command result = super.getCommand();			
		// There is an API where an implementor of CDECreationTool.CreationPolicy can register themselves
		// on a decorator with a key
		// The purpose of this is so that people can decorate their classes with CreationPolicy
		// that can put commands in front of and after the set of commands that are called on creation
		// This is cach'd for speed of lookup
		Object type = getFactory().getObjectType();
		if ( fCreationPolicy == null && !fHasLookedupCreationPolicy) {
			if ( type instanceof EClassifier ) {
				ClassDescriptorDecorator decorator = (ClassDescriptorDecorator)ClassDecoratorFeatureAccess.getDecoratorWithKeyedFeature((EClassifier)type, ClassDescriptorDecorator.class, CDECreationTool.CREATION_POLICY_KEY);			
				if ( decorator != null ) {
					String creationPolicyClassName = ((KeyedString)decorator.getKeyedValue(CDECreationTool.CREATION_POLICY_KEY)).getValue();
					// The class name may be from another plugin so we must instantiate it correctly
					try {
						fCreationPolicy = (CDECreationTool.CreationPolicy)CDEPlugin.createInstance(null, creationPolicyClassName);
					} catch ( Exception exc ) {
						// If the class can't be created then just drop down and let the regular command be returned
					}
				}
			}
			fHasLookedupCreationPolicy = true;
		}
		if ( fCreationPolicy != null ) {
			return fCreationPolicy.getCommand(result,getCreateRequest());			
		} else {
			return result;
		}		
	}
	}
		
	/**
	 * Return the selection history list.
	 */
	public List getSelectionHistory();
	
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	PalettePackage ePackagePalette();
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	EClass eClassSelectionCreationToolEntry();
	/**
	 * @generated This field/method will be replaced during code generation 
	 * @return The value of the SelectorClassName attribute
	 * This is the classname of the selector class. It must implement the ISelector interface.
	 */
	String getSelectorClassName();
	/**
	 * @generated This field/method will be replaced during code generation 
	 * @param value The new value of the SelectorClassName attribute
	 */
	void setSelectorClassName(String value);
	/**
	 * @generated This field/method will be replaced during code generation 
	 * Unsets the SelectorClassName attribute
	 */
	void unsetSelectorClassName();
	/**
	 * @generated This field/method will be replaced during code generation 
	 * @return <b>true</b> if the SelectorClassName attribute has been set
	 */
	boolean isSetSelectorClassName();
}
