package com.ibm.etools.cde.palette.impl;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.cde.palette.*;
import org.eclipse.swt.graphics.Image;


import com.ibm.etools.emf.ref.RefBaseObject;
import com.ibm.etools.emf.ref.impl.RefBaseObjectImpl;
import java.util.*;


import com.ibm.etools.gef.palette.PaletteEntry;
import com.ibm.etools.emf.ref.EList;


import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.cde.utility.*;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.InternalProxy;
import com.ibm.etools.emf.ref.Internals;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.gef.palette.PaletteRoot;
import java.lang.Object;
import java.lang.ClassCastException;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.Notification;
/**
 * This is the abstract base palette. There are two standard implementations of it:
 * 
 * PaletteCmp: This is a palette where the categories and control group are composite aggregations. (i.e. they are contained within the  PaletteCmp).
 * 
 * PaletteRef: This is a palette where the categories and control group are shared aggregations. (i.e. they are actually contained somewhere else and only referenced here). This type of palette allows references to pieces of other palettes to form together a new palette.
 */
/**
 * This is the abstract base palette. There are two standard implementations of it:
 * 
 * PaletteCmp: This is a palette where the categories and control group are composite aggregations. (i.e. they are contained within the  PaletteCmp).
 * 
 * PaletteRef: This is a palette where the categories and control group are shared aggregations. (i.e. they are actually contained somewhere else and only referenced here). This type of palette allows references to pieces of other palettes to form together a new palette.
 */
/**
 * @lastgen class PaletteImpl extends ContainerImpl implements Palette, Container, PaletteRoot, RefObject {}
 */


public class PaletteImpl extends ContainerImpl implements Palette, Container, PaletteRoot, RefObject {






	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected AbstractString paletteLabel = null;
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected boolean setPaletteLabel = false;
	protected PaletteImpl() {
		super();
	}
	/**
	 * Return the Control Group.
	 */
	public Group getControlGroup(){
		return getPaletteControlGroup();
	}

	/**
	 * Return the categories
	 */	
	public List getCategories() {
		return getPaletteCategories();
	}
	
	
	/**
	 * getChildren for a standard palette is to merge the controlGroup with the children
	 * until. If something different is desired then subclass the palette.
	 */
	public List getChildren() {
		List result = new ArrayList(getPaletteCategories().size() + 1);
		Object cGroup = getControlGroup();
		if (cGroup != null)
			result.add(getControlGroup());	// There may not be a control group.
			
		result.addAll(getPaletteCategories());
		return result;
	}
	
	
	protected Group getPaletteControlGroup() {
		throw new IllegalStateException("Must be implemented by subclass."); //$NON-NLS-1$
	}
	
	
	protected List getPaletteCategories() {
		throw new IllegalStateException("Must be implemented by subclass."); //$NON-NLS-1$
	}	
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public RefObject initInstance() {
		refSetMetaObject(eClassPalette());
		initInstanceDelegates();
		return this;
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public PalettePackage ePackagePalette() {
		return (PalettePackage)RefRegister.getPackage(PalettePackage.packageURI);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public EClass eClassPalette() {
		return ((PalettePackage)RefRegister.getPackage(PalettePackage.packageURI)).getPalette();
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 */
	public AbstractString getPaletteLabel() {
		try {
			if (this.setPaletteLabel) {
				if (this.paletteLabel==null) return null;
				this.paletteLabel = (AbstractString) ((InternalProxy)this.paletteLabel).resolve(this);
				if (this.paletteLabel==null) this.setPaletteLabel = false;
				return this.paletteLabel;
			}
			return (AbstractString)ePackagePalette().getPalette_PaletteLabel().refGetDefaultValue();
		} catch (Exception e) {
			return null;
		}
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public void setPaletteLabel(AbstractString value) {
		refSetValueForRefObjectSF(ePackagePalette().getPalette_PaletteLabel(),this.paletteLabel,value);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public void unsetPaletteLabel() {
		if (this.paletteLabel != null)
			notify(((Internals)this.paletteLabel).refSetContainer(null, null));
		else
			notify(refBasicUnsetValue(ePackagePalette().getPalette_PaletteLabel()));
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public boolean isSetPaletteLabel() {
		return setPaletteLabel;
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Object refValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refValue(feature);
		}
		switch (eClassPalette().getEFeatureId(eFeature)) {
			case PalettePackage.PALETTE__PALETTE_LABEL: return getPaletteLabel();


		}
		return super.refValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Object refBasicValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refBasicValue(feature);
		}
		switch (eClassPalette().getEFeatureId(eFeature)) {
			case PalettePackage.PALETTE__PALETTE_LABEL:
				if (!this.setPaletteLabel||this.paletteLabel==null) return null;
				if (((InternalProxy)this.paletteLabel).refIsDeleted()) {this.paletteLabel=null; this.setPaletteLabel=false;}
				return this.paletteLabel;


		}
		return super.refBasicValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public boolean refIsSet(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refIsSet(feature);
		}
		switch (eClassPalette().getEFeatureId(eFeature)) {
			case PalettePackage.PALETTE__PALETTE_LABEL: return isSetPaletteLabel();


		}
		return super.refIsSet(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public void refSetValue(RefStructuralFeature feature, Object newValue) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			super.refSetValue(feature, newValue);
		}
		switch (eClassPalette().getEFeatureId(eFeature)) {
			case PalettePackage.PALETTE__PALETTE_LABEL: {
				setPaletteLabel((AbstractString)newValue);
				return;
			}


		}
		super.refSetValue(feature, newValue);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Notification refBasicSetValue(RefStructuralFeature feature, Object newValue) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refBasicSetValue(feature, newValue);
		}
		switch (eClassPalette().getEFeatureId(eFeature)) {
			case PalettePackage.PALETTE__PALETTE_LABEL: {
				Object oldValue = this.paletteLabel;
				this.paletteLabel = (AbstractString)newValue;
				this.setPaletteLabel = true;
				return new NotificationImpl(refDelegateOwner(),Notifier.SET,ePackagePalette().getPalette_PaletteLabel(),oldValue,newValue);
			}


		}
		return super.refBasicSetValue(feature, newValue);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public void refUnsetValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			super.refUnsetValue(feature);
		}
		switch (eClassPalette().getEFeatureId(eFeature)) {
			case PalettePackage.PALETTE__PALETTE_LABEL: {
				unsetPaletteLabel();
				return;
			}


		}
		super.refUnsetValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Notification refBasicUnsetValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refBasicUnsetValue(feature);
		}
		switch (eClassPalette().getEFeatureId(eFeature)) {
			case PalettePackage.PALETTE__PALETTE_LABEL: {
				Object oldValue = this.paletteLabel;
				this.paletteLabel = null;
				this.setPaletteLabel = false;
				return new NotificationImpl(refDelegateOwner(),Notifier.UNSET,ePackagePalette().getPalette_PaletteLabel(),oldValue,getPaletteLabel());
			}

		}
		return super.refBasicUnsetValue(feature);
	}
}
