package com.ibm.etools.cde.properties;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.propertysheet.EToolsPropertyDescriptor;
import com.ibm.etools.propertysheet.ISourcedPropertyDescriptor;
import com.ibm.etools.propertysheet.ICommandPropertyDescriptor;
import com.ibm.etools.propertysheet.INeedData;
import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.*;
import com.ibm.etools.cdm.Annotation;
import com.ibm.etools.cdm.KeyedValue;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.emf.ref.RefStructuralFeature;
/**
 * This is a default base class for handling property descriptors
 * on the keys of an annotation.
 */
public abstract class AbstractAnnotationPropertyDescriptor extends EToolsPropertyDescriptor implements ISourcedPropertyDescriptor, ICommandPropertyDescriptor, INeedData {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain domain;
	
	/**
	 * The ID should be the key of the KeyedValue that this descriptor describes.
	 */
	public AbstractAnnotationPropertyDescriptor(String key, String displayName) {
		super(KeyedValueNotificationHelper.getFeature(key), displayName);
	}
	
	public void setData(Object data) {
		domain = (EditDomain) data;
	}
	
	public Object getValue(IPropertySource ps) {
		Annotation annotation = domain.getAnnotationLinkagePolicy().getAnnotation(ps.getEditableValue());
		if (annotation != null) {
			KeyedValue kv = annotation.getKeyedValue(((RefStructuralFeature) getId()).refName());
			if (kv != null)
				return getKeyedValue(kv);
		}
		return null;
	}
	
	/**
	 * Return the value (as it getValue requires, i.e. turn into IPropertySource if necessary),
	 * to be edited/displayed in the property sheet.
	 */
	protected abstract Object getKeyedValue(KeyedValue kv);


	public boolean isSet(IPropertySource ps) {
		Annotation annotation = domain.getAnnotationLinkagePolicy().getAnnotation(ps.getEditableValue());
		if (annotation != null) {
			KeyedValue kv = annotation.getKeyedValue(((RefStructuralFeature) getId()).refName());
			if (kv != null)
				return isSetKeyedValue(kv);
		}
		return false;	
	}
	
	/**
	 * Return whether the value in the KeyedValue is set.
	 */
	protected abstract boolean isSetKeyedValue(KeyedValue kv);
	
	public Command setValue(IPropertySource ps, Object setValue) {
		Object modelObject = ps.getEditableValue();
		Annotation annotation = domain.getAnnotationLinkagePolicy().getAnnotation(ps.getEditableValue());
		KeyedValue oldkv = null;
		if (annotation != null) 
			oldkv = annotation.getKeyedValue(((RefStructuralFeature) getId()).refName());
		KeyedValue kv = getSetKeyedValue(oldkv, setValue);	// Get the keyed value to apply
		return AnnotationPolicy.applyAnnotationSetting(modelObject, kv, domain);
	}
	
	/**
	 * Return the KeyedValue to apply to the annotation for the given value from the property sheet.
	 * It can return the oldkv if the setValue is the same. This can occur in nested cases.
	 */
	protected abstract KeyedValue getSetKeyedValue(KeyedValue oldkv, Object setValue);

	public Command resetValue(IPropertySource ps) {
		Object modelObject = ps.getEditableValue();
		
		// For things to work correctly, we must always have a KeyedValue for this key.
		// So for reset, we need one that has the KeyedValue, but the value field isn't set.
		// We can't do this generically, so we need subclass to return one.
		KeyedValue kv = getUnsetKeyedValue();	// Get the unset keyed value
		return AnnotationPolicy.applyAnnotationSetting(modelObject, kv, domain);
	}
	
	/**
	 * Return the KeyedValue that represents the value as not being set.
	 */
	protected abstract KeyedValue getUnsetKeyedValue();
}