package com.ibm.etools.cde.properties;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.emf.notify.impl.*;
import org.eclipse.ui.views.properties.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.core.runtime.*;
import java.lang.reflect.*;
import com.ibm.etools.cde.*;

import com.ibm.etools.emf.ref.*;
import com.ibm.etools.logging.util.Level;
import com.ibm.etools.emf.ecore.*;

import java.text.MessageFormat;
/**
 * This is the base property descriptor adapter. There will be specific subclasses
 * to provide the required information for this adapter. An example is the DecoratedPropertyDescriptorAdapter,
 * which knows how to get the information from the CDE EMF PropertyDecorators.
 */

public abstract class AbstractPropertyDescriptorAdapter extends AdapterImpl implements IPropertyDescriptor {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static Class IPROPERTYDESCRIPTOR_TYPE = IPropertyDescriptor.class;	// Key for adapter for property descriptors.

	
/**
 * instantiate the  class passed in. If it has a ctor that
 * takes a IPropertyDescriptor, use that one.
 *
 * A new one is created each time. It is the responsibility of
 * the caller to dispose of it when no longer needed.
 */
public static ILabelProvider createLabelProviderInstance(Class clazz, String data, Object initData, IPropertyDescriptor pd) {
	Constructor ctor = null;
	try {
		ctor = clazz.getConstructor(new Class[] {IPropertyDescriptor.class});
	} catch (NoSuchMethodException e) {
	};
	try {
		ILabelProvider provider = null;
		if (ctor != null) {
			provider = (ILabelProvider) ctor.newInstance(new Object[] {pd});
		} else {
			provider = (ILabelProvider) clazz.newInstance();
		}
		
		CDEPlugin.setInitializationData(provider, data, initData);
		return provider;
	} catch (Exception exc) {
		String msg = MessageFormat.format(CDEMessages.getString(CDEMessages.OBJECT_NOINSTANTIATE), new Object[] {clazz});		
		CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, msg, exc));
	}
	return null;
}

/**
 * instantiate the  class passed in. If it has an ctor that
 * takes a IPropertyDescriptor, use that one.
 * A CellEditor must be constructed with a Composite
 */
protected CellEditor createCellEditorInstance(Class clazz, Composite aComposite, String data, Object initData) {
	Constructor ctor = null;
	CellEditor editor = null;
	try {
		ctor = clazz.getConstructor(new Class[] {IPropertyDescriptor.class,Composite.class});
	} catch (NoSuchMethodException e) {
	};
	try {
		if (ctor != null) {
			editor = (CellEditor) ctor.newInstance(new Object[] {this,aComposite});
		} else {
			ctor = clazz.getConstructor(new Class[] {Composite.class});
			editor = (CellEditor) ctor.newInstance(new Object[] {aComposite});
		}
		
		CDEPlugin.setInitializationData(editor, data, initData);
		ICellEditorValidator validator = getValidator();
		if (validator != null)
			editor.setValidator(validator);
	} catch (Exception exc) {
		String msg = MessageFormat.format(CDEMessages.getString(CDEMessages.OBJECT_NOINSTANTIATE), new Object[] {clazz, target});		
		CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, msg, exc));
	}
	return editor;
}

/**
 * Subclasses should return the validator to use, if any. Null if none.
 */
protected abstract ICellEditorValidator getValidator();

/**
 * A helper method to find a decorator of a specific instance.
 */
protected EDecorator findDecorator(EList decorators, Class decoratorType) {
	if (decorators == null) return null;
	for (int i=0; i < decorators.size(); i++) {
		EDecorator o = (EDecorator) decorators.get(i);
		if (decoratorType.isInstance(o))
			return o;
	}
	return null;
}

public String getDisplayName() {
	String dn = primGetDisplayName();
	
	return dn != null ? dn : ((RefObject) target).refName();
}

/**
 * Subclasses need to return the displayName, if none return null.
 */
protected abstract String primGetDisplayName();


/**
 * Return the feature as the ID of the property descriptor
 */
public Object getId() {
	return target;
}

public boolean isAdapterForType(Object type) {
	return IPROPERTYDESCRIPTOR_TYPE.equals(type);
}

public boolean isCompatibleWith(IPropertyDescriptor anotherProperty) {
	return this.equals(anotherProperty);
}
}

