package com.ibm.etools.cde.properties;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
import java.util.*;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.commands.AddAnnotationsCommand;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cdm.*;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.emf.ref.RefRegister;

/**
 * This checks the name in composition and makes sure that
 * they are valid, and will change them if they are not.
 * @version 	1.0
 * @author
 */
public class AddAnnotationsWithName extends AddAnnotationsCommand {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * Constructor for AddAnnotationsWithName.
	 */
	public AddAnnotationsWithName() {
		super();
	}
	
	

	/**
	 * @see AddAnnotationsCommand#getAdditionalCommands(List, EditDomain)
	 */
	protected Command getAdditionalCommands(List annotations, EditDomain domain) {
		CommandBuilder cb = new CommandBuilder();
		
		HashSet addedNames = new HashSet(annotations.size());
		Iterator anns = annotations.iterator();
		while (anns.hasNext()) {
			Annotation ann = (Annotation) anns.next();
			KeyedString ks = (KeyedString) ann.getKeyedValue(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY);
			if (ks != null) {
				String name = ks.getValue();
				if (name != null) {
					String newName = NameInCompositionPropertyDescriptor.getUniqueNameInComposition(domain, name, addedNames);
					if (!newName.equals(name)) {
						// We need to apply the new unique name
						addedNames.add(newName);	// So that it won't be used again.
						// We always apply an entire new keyvalue, not just modify the setting.
						KeyedString newKs = ((CDMPackage) RefRegister.getPackage(CDMPackage.packageURI)).getCDMFactory().createKeyedString();
						newKs.setKey(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY);
						newKs.setValue(newName);
						cb.applyAttributeSetting(ann, newKs);
					}
				}
			} 		
		}
		
		CompoundCommand cmd = cb.getCompoundCommand();
		if (cmd.isEmpty())
			return null;	// Nothing changed
		else
			return cmd.unwrap();
	}

}
