package com.ibm.etools.cde.properties;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.emf.notify.impl.AdapterFactoryImpl;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.emf.ClassDecoratorFeatureAccess;
import java.text.MessageFormat;
import org.eclipse.core.runtime.*;
import com.ibm.etools.cde.decorators.*;
import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.logging.util.Level;
import com.ibm.etools.propertysheet.INeedData;
/**
 * Adapter factory for PropertySource adapters.
 *
 * The key for PropertySources is IPropertySource.class
 *
 * Note: This will give the EditDomain to those adapters that implement INeedData.
 * Creation date: (10/28/99 9:27:57 AM)
 * @author: Joe Winchester
 */
public class PropertySourceAdapterFactory extends AdapterFactoryImpl {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Class fDefaultPropertySourceClass;
	protected EditDomain domain;
	
	protected static final RefStructuralFeature sPropertySourceAdapterClassnameFeature;
	static {
		DecoratorsPackage dp = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
		sPropertySourceAdapterClassnameFeature = dp.getPropertySourceAdapterInformation_PropertySourceAdapterClassname();
	}

public PropertySourceAdapterFactory(EditDomain domain) {
	super(IPropertySource.class, PropertySourceAdapter.class);
	this.domain = domain;
}

public PropertySourceAdapterFactory(EditDomain domain, Class defaultPropertySourceClass) {
	this(domain);
	fDefaultPropertySourceClass = defaultPropertySourceClass;
}


/**
 * Adapt us based upon the decorator. The decorator will be found on the metaObject, or one
 * of its super types. If none found, then the default class, if any will be used.
 * Otherwise use a default adapter.
 *
 * If the propertySourceAdapterClass value is null for a decorator, the next decorator
 * will be found and used.
 */
public Adapter createAdapter(Notifier target) {
	if (!(target instanceof RefObject))
		return null;	// Can't adapt something that is not a RefObject as a PropertySource. It is probably a datatype then, and those aren't property sources. Property sources can have properties, only RefObjects can have properties.
		
	PropertySourceAdapterInformation decr = (PropertySourceAdapterInformation) ClassDecoratorFeatureAccess.getDecoratorWithFeature((EClassifier) ((RefObject) target).refMetaObject(), PropertySourceAdapterInformation.class, sPropertySourceAdapterClassnameFeature);
	if (decr != null && decr.isSetPropertySourceAdapterClassname()) {
		try {
			Adapter a = (Adapter) CDEPlugin.createInstance(null, decr.getPropertySourceAdapterClassname());
			if (a instanceof INeedData)
				((INeedData) a).setData(domain);
			return a;
		} catch (Exception exc) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", exc)); //$NON-NLS-1$
		}
	}
		
	if (fDefaultPropertySourceClass != null) {
		try {
			Adapter a = (Adapter) fDefaultPropertySourceClass.newInstance();
			if (a instanceof INeedData)
				((INeedData) a).setData(domain);
			return a;
		} catch (Exception e) {
			String msg = MessageFormat.format(CDEMessages.getString(CDEMessages.OBJECT_NOINSTANTIATE), new Object[] {adapterClass});		
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, msg, e));
		}
	}		
	
	return super.createAdapter(target); // Can't find one, so do default.
}
}
