package com.ibm.etools.cde.utility.impl;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.net.*;
import java.util.Locale;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.MissingResourceException;

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.model.PluginDescriptorModel;
import org.eclipse.core.runtime.model.PluginFragmentModel;

import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.cde.utility.*;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notification;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.logging.util.Level;

/**
 * A bundle url is of the form:
 * 
 * ...directory...
 * 
 * For example:
 * 
 * platform:/plugin/pluginname/nls/
 * 
 * This will look in the plugin directory "nls/" in the plugin "pluginname".
 * 
 * NOTE: If this is a directory then the URL MUST end with '/' or it won't work, it will think "nls" is a file and not a directory.
 * 
 * There can be more than one if they are to be searched in order.
 * 
 * NOTE: To handle nls within plugins, the url plugin protocol doesn't know to add in any fragments. To get this to work so that
 * nls fragments can be added at a later time for additional languages, this class will add in all of the fragments for the plugin
 * and will use all that follows the plugin name to look in the fragment. For example, if there is a fragment called "fragmentname" of
 * the plugin "pluginname", then the resulting urls to be searched will be:
 *   platform:/plugin/pluginname/nls/
 *   platform:/fragment/fragmentname/nls/
 * 
 */
/**
 * @lastgen class URLResourceBundleImpl extends ResourceBundleImpl implements URLResourceBundle, ResourceBundle {}
 */


public class URLResourceBundleImpl extends ResourceBundleImpl implements URLResourceBundle, ResourceBundle {






	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private java.util.ResourceBundle fBundle = null;
	private boolean fLoaded = false;
	
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected String bundleName = null;
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected EList bundleURLs = null;
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected boolean setBundleName = false;
	protected URLResourceBundleImpl() {
		super();
	}
	public java.util.ResourceBundle getBundle() {
		if (!fLoaded) {
			fLoaded = true;
			if (isSetBundleName() && bundleURLs != null && bundleURLs.size() > 0) {
				ArrayList urls = new ArrayList(bundleURLs.size()*2);
				Iterator itr = bundleURLs.iterator();
				int i=0;
				while (itr.hasNext()) {
					try {
						String urlString = (String) itr.next();
						urls.add(new URL(urlString));
						if (urlString.startsWith("platform:/plugin/")) { //$NON-NLS-1$
							// Special, need to get the fragments too.
							int begPluginName = "platform:/plugin/".length(); //$NON-NLS-1$
							int endPluginName = urlString.indexOf('/', begPluginName);
							String pluginName = urlString.substring(begPluginName, endPluginName);
							String pid = getId(pluginName);
							String vid = getVersion(pluginName);
							String rest = urlString.substring(endPluginName);
							IPluginRegistry registry = Platform.getPluginRegistry();
							IPluginDescriptor desc = (vid==null || vid.equals("")) ? registry.getPluginDescriptor(pid) : registry.getPluginDescriptor(pid,new PluginVersionIdentifier(vid)); //$NON-NLS-1$
							if (desc != null) {
								PluginFragmentModel[] fragments = ((PluginDescriptorModel) desc).getFragments();	// See if there are any fragments
								if (fragments != null)
									for (int j = 0; j < fragments.length; j++) {
										urls.add(new URL("platform:/fragment/"+ fragments[j].toString() + rest)); //$NON-NLS-1$
									}
							}
						}
					} catch (MalformedURLException e) {
						return null;	// An error, don't go on.
					}
				}
				URLClassLoader cl = new URLClassLoader((URL[]) urls.toArray(new URL[urls.size()]), null);
				try {
					fBundle = java.util.ResourceBundle.getBundle(getBundleName(), Locale.getDefault(), cl);
				} catch (MissingResourceException e) {
					CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e)); //$NON-NLS-1$
				}
			}
		}
		return fBundle;
	}
	
	protected String getId(String spec) {
		int i = spec.lastIndexOf('_'); //$NON-NLS-1$
		return i >= 0 ? spec.substring(0, i) : spec;
	}
	
	protected String getVersion(String spec) {
		int i = spec.lastIndexOf('_');
		return i >= 0 ? spec.substring(i + 1, spec.length()) : ""; //$NON-NLS-1$
	}
	
	
	public void setBundleName(String value) {
		// Clear cache
		fLoaded = false;
		fBundle = null;
		this.setBundleNameGen(value);
	}
	public void unsetBundleName() {
		// Clear cache
		fLoaded = false;
		fBundle = null;
		this.unsetBundleNameGen();
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public RefObject initInstance() {
		refSetMetaObject(eClassURLResourceBundle());
		initInstanceDelegates();
		return this;
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public UtilityPackage ePackageUtility() {
		return (UtilityPackage)RefRegister.getPackage(UtilityPackage.packageURI);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public EClass eClassURLResourceBundle() {
		return ((UtilityPackage)RefRegister.getPackage(UtilityPackage.packageURI)).getURLResourceBundle();
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 */
	public String getBundleName() {
		if (this.setBundleName) return this.bundleName;
		else return (String)ePackageUtility().getURLResourceBundle_BundleName().refGetDefaultValue();
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public boolean isSetBundleName() {
		return setBundleName;
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 * The URL's to use to search for the bundle. If there is more, then they will be searched in order for the bundle (or bundles if this is locale specific override type bundle).
	 */
	public EList getBundleURLs() {
		if (this.bundleURLs==null) {
			this.bundleURLs=newCollection(this,ePackageUtility().getURLResourceBundle_BundleURLs());
		}
		return this.bundleURLs;
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Object refValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refValue(feature);
		}
		switch (eClassURLResourceBundle().getEFeatureId(eFeature)) {
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_NAME: return getBundleName();
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_UR_LS: return getBundleURLs();


		}
		return super.refValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Object refBasicValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refBasicValue(feature);
		}
		switch (eClassURLResourceBundle().getEFeatureId(eFeature)) {
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_NAME:
				return this.setBundleName? this.bundleName : null;
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_UR_LS:
				return this.bundleURLs;


		}
		return super.refBasicValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public boolean refIsSet(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refIsSet(feature);
		}
		switch (eClassURLResourceBundle().getEFeatureId(eFeature)) {
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_NAME: return isSetBundleName();


		}
		return super.refIsSet(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public void refSetValue(RefStructuralFeature feature, Object newValue) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			super.refSetValue(feature, newValue);
		}
		switch (eClassURLResourceBundle().getEFeatureId(eFeature)) {
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_NAME: {
				setBundleName((String)newValue);
				return;
			}


		}
		super.refSetValue(feature, newValue);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Notification refBasicSetValue(RefStructuralFeature feature, Object newValue) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refBasicSetValue(feature, newValue);
		}
		switch (eClassURLResourceBundle().getEFeatureId(eFeature)) {
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_NAME: {
				Object oldValue = this.bundleName;
				this.bundleName = (String)newValue;
				this.setBundleName = true;
				return new NotificationImpl(refDelegateOwner(),Notifier.SET,ePackageUtility().getURLResourceBundle_BundleName(),oldValue,newValue);
			}


		}
		return super.refBasicSetValue(feature, newValue);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public void refUnsetValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			super.refUnsetValue(feature);
		}
		switch (eClassURLResourceBundle().getEFeatureId(eFeature)) {
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_NAME: {
				unsetBundleName();
				return;
			}


		}
		super.refUnsetValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Notification refBasicUnsetValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refBasicUnsetValue(feature);
		}
		switch (eClassURLResourceBundle().getEFeatureId(eFeature)) {
			case UtilityPackage.URL_RESOURCE_BUNDLE__BUNDLE_NAME: {
				Object oldValue = this.bundleName;
				this.bundleName = null;
				this.setBundleName = false;
				return new NotificationImpl(refDelegateOwner(),Notifier.UNSET,ePackageUtility().getURLResourceBundle_BundleName(),oldValue,getBundleName());
			}


		}
		return super.refBasicUnsetValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public String toString() {
		String out="(";  //$NON-NLS-1$
		boolean first=true;
		boolean emptyList=true;
		if(isSetBundleName()) {
			if(!emptyList) out+=", "; //$NON-NLS-1$
			out+="bundleName: "+this.bundleName; //$NON-NLS-1$
			first=false;
			emptyList=false;
		}
		if(!getBundleURLs().isEmpty()) {
			if(!emptyList) out+=", "; //$NON-NLS-1$
			out+="bundleURLs: "+this.bundleURLs; //$NON-NLS-1$
			first=false;
			emptyList=false;
		}
		out+=")"; //$NON-NLS-1$
		if (!first) return super.toString() + " " +out; //$NON-NLS-1$
		return super.toString();
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected void setBundleNameGen(String value) {
		refSetValueForSimpleSF(ePackageUtility().getURLResourceBundle_BundleName(),this.bundleName,value);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected void unsetBundleNameGen() {
		notify(refBasicUnsetValue(ePackageUtility().getURLResourceBundle_BundleName()));
	}
}
