package com.ibm.etools.cdm.impl;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.cdm.*;
import com.ibm.etools.emf.ref.RefBaseObject;
import com.ibm.etools.emf.ref.impl.RefBaseObjectImpl;
import java.util.List;
import java.util.Iterator;
import java.util.ListIterator;


import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.EList;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.impl.RefObjectImpl;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import java.lang.Object;
import java.lang.ClassCastException;
import com.ibm.etools.emf.notify.Notifier;
/**
 * This class holds KeyedValues. It is used by mixing it in with multiple inheritance into where it is needed.
 * 
 * The operations allow treating the KeyedValues as properties. They should be used instead of the generic keyedValues relationship. It allows easier usage of the keyed values.
 * 
 * Note: KeyedValues should be treated as dataTypes, in other words, individual KeyedValues shouldn't be updated, they should be replaced instead. This is because typically users area listening for new keyed values, not subvalues within individual keyed values being changed. If the users understands this and still listens on individual KeyedValues, that is OK as long thier model handles unlistening when the keyedvalue is removed.
 */
public class KeyedValueHolderImpl extends RefObjectImpl implements KeyedValueHolder, RefObject{


	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	protected EList keyedValues = null;
	protected KeyedValueHolderImpl() {
		super();
	}
	public KeyedValue getKeyedValue(String key) {
		// The assumption is that the list of keyed values won't be more than a few entries. So in
		// that case a simple direct linear search will be faster.
		Iterator itr = getKeyedValues().iterator();
		while (itr.hasNext()) {
			KeyedValue v = (KeyedValue) itr.next();
			if (v.isSetKey() && v.getKey().equals(key))
				return v;
		}
		return null;
	}
	
	public KeyedValue setKeyedValue(String key, KeyedValue keyValue) {
		keyValue.setKey(key);	// Make sure key matches.
		ListIterator itr = getKeyedValues().listIterator();
		while (itr.hasNext()) {
			KeyedValue v = (KeyedValue) itr.next();
			if (v.isSetKey() && v.getKey().equals(key)) {
				itr.set(keyValue);
				return v;
			}
		}
		itr.add(keyValue);
		return null;
	}
	
	public boolean isSetKeyedValue(String key) {
		Iterator itr = getKeyedValues().iterator();
		while (itr.hasNext()) {
			KeyedValue v = (KeyedValue) itr.next();
			if (v.isSetKey() && v.getKey().equals(key))
				return true;
		}
		return false;
	}


	public KeyedValue unsetKeyedValue(String key) {
		ListIterator itr = getKeyedValues().listIterator();
		while (itr.hasNext()) {
			KeyedValue v = (KeyedValue) itr.next();
			if (v.isSetKey() && v.getKey().equals(key)) {
				itr.remove();
				return v;
			}
		}
		
		return null;
	}
		
 
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public RefObject initInstance() {
		refSetMetaObject(eClassKeyedValueHolder());
		initInstanceDelegates();
		return this;
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public CDMPackage ePackageCDM() {
		CDMPackage pkg = null;
		if (eClassKeyedValueHolder() != null)
			pkg =  (CDMPackage)eClassKeyedValueHolder().refContainer();
		if (pkg == null)
			return (CDMPackage)RefRegister.getPackage(CDMPackage.packageURI);
		return pkg;
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public EClass eClassKeyedValueHolder() {
		return ((CDMPackage)RefRegister.getPackage(CDMPackage.packageURI)).getKeyedValueHolder();
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 */
	public EList getKeyedValues() {
		if (this.keyedValues==null) {
			this.keyedValues=newCollection(refDelegateOwner(),ePackageCDM().getKeyedValueHolder_KeyedValues());
		}
		return this.keyedValues;
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Object refValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refValue(feature);
		}
		switch (eClassKeyedValueHolder().getEFeatureId(eFeature)) {
			case CDMPackage.KEYED_VALUE_HOLDER__KEYED_VALUES: return getKeyedValues();


		}
		return super.refValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation.
	 */
	public Object refBasicValue(RefStructuralFeature feature) {
		EStructuralFeature eFeature=null;
		try {
			eFeature=(EStructuralFeature)feature;
		} catch (ClassCastException e) {
			return super.refBasicValue(feature);
		}
		switch (eClassKeyedValueHolder().getEFeatureId(eFeature)) {
			case CDMPackage.KEYED_VALUE_HOLDER__KEYED_VALUES:
				return this.keyedValues;


		}
		return super.refBasicValue(feature);
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 * Get the KeyedValue for a specific key.
	 */
	protected KeyedValue getKeyedValueGen(String key) {
		return null;
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 * Set the KeyValue for the given key. Replace a KeyValue that is already there of that key. Return the previous setting, if any.
	 */
	protected KeyedValue setKeyedValueGen(String key, KeyedValue keyValue) {
		return null;
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 * Is there a KeyedValue for given key?
	 */
	protected boolean isSetKeyedValueGen(String key) {
		return false;
	}
	/**
	 * @generated This field/method will be replaced during code generation 
	 * Unset the KeyedValue for the given key, if any. Return the setting that was there, if any.
	 */
	protected KeyedValue unsetKeyedValueGen(String key) {
		return null;
	}
}
