package com.ibm.etools.java.instantiation.base;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.notify.impl.NotifierImpl;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.JavaDataType;

/**
 * This is the default instance of a Java Model Datatype (i.e. primitive).
 * It can be created from a string, which becomes the initialization string
 * for the instance. It's toString will be the initialization string.
 *
 * It should not be referenced directly, the IJavaDataTypeInstance interface should be
 * used instead. It is public so that it can be subclassed. 
 */
public class JavaDataTypeInstance extends NotifierImpl implements IJavaDataTypeInstance {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected final String fInitString;
	protected final JavaDataType fType;
	
	public JavaDataTypeInstance(String initString, JavaDataType type) {
		fInitString = initString;
		fType = type;
	}
	
	public JavaDataTypeInstance(JavaDataType type) {
		this(null, type);
	}
	
	
	public JavaHelpers getJavaType() {
		return fType;
	}
	
	public String getInitializationString() {
		return fInitString;
	}
	
	public boolean isSetInitializationString() {
		return fInitString != null;
	}
	
	public String toString() {
		return fInitString != null ? fInitString : "null"; //$NON-NLS-1$
	}
	public boolean isPrimitive(){
		return true;
	}
}
