package com.ibm.etools.java.instantiation.base;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ecore.EAttribute;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.resource.*;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.IInstantiationAdapter;

/**
 * This class is used to initialize java model instantiation code and to access dynamic features.
 */
public class JavaInstantiation {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String ID_INITSTRING = "Object/initializationString"; //$NON-NLS-1$
	public static final String ID_IMPLICIT = "Object/implicit"; //$NON-NLS-1$
	public static final String ID_INSTANTIATEUSING = "Object/instantiateUsing"; //$NON-NLS-1$
	public static final String ID_SERIALIZEDATA = "Object/serializeData";	 //$NON-NLS-1$
	
	public static final URI URI_INITSTRING = URIFactoryRegister.getFactory().makeURI("java:/java.lang#"+ID_INITSTRING); //$NON-NLS-1$
	public static final URI URI_IMPLICIT = URIFactoryRegister.getFactory().makeURI("java:/java.lang#"+ID_IMPLICIT); //$NON-NLS-1$
	public static final URI URI_INSTANTIATEUSING = URIFactoryRegister.getFactory().makeURI("java:/java.lang#"+ID_INSTANTIATEUSING); //$NON-NLS-1$
	public static final URI URI_SERIALIZEDATA = URIFactoryRegister.getFactory().makeURI("java:/java.lang#"+ID_SERIALIZEDATA);	 //$NON-NLS-1$
	
	
	/**
	 * Get a structural feature from an instance of an IJavaObjectInstance, where the feature
	 * is specified by its URI (e.g. "java:/java.lang#Object/class").
	 */
	public static EStructuralFeature getSFeature(IJavaObjectInstance jobject, URI sfURI) {
		return getSFeature((JavaClass) jobject.refMetaObject(), sfURI);
	}

	/**
	 * Get a structural feature from a JavaClass, where the feature
	 * is specified by its URI (e.g. "java:/java.lang#Object/class").
	 */
	public static EStructuralFeature getSFeature(JavaClass jclass, URI sfURI) {
		return (EStructuralFeature) getSFeature(jclass.getResourceSet(), sfURI);
	}
	
	/**
	 * Get a structural feature from a ResourceSet, where the feature
	 * is specified by its URI (e.g. "java:/java.lang#Object/class").
	 */
	public static EStructuralFeature getSFeature(ResourceSet rset, URI sfURI) {
		return (EStructuralFeature) rset.getObjectAndLoad(sfURI);
	}	
	
	/**
	 * Get the init string Feature for this object. Since it depends
	 * on what resource that the metaclass is defined in, we must look for it.
	 */
	public static EAttribute getInitializationStringFeature(IJavaObjectInstance jobject) {
		return (EAttribute) getSFeature(jobject, URI_INITSTRING);
	}
	
	/**
	 * Get the implicit Feature for this object. Since it depends
	 * on what resource that the metaclass is defined in, we must look for it.
	 */
	public static EAttribute getImplicitFeature(IJavaObjectInstance jobject) {
		return (EAttribute) getSFeature(jobject, URI_IMPLICIT);
	}
		
	/**
	 * Get the instantiate using Feature for this object. Since it depends
	 * on what resource that the metaclass is defined in, we must look for it.
	 */
	public static EAttribute getInstantiateUsingFeature(IJavaObjectInstance jobject) {
		return (EAttribute) getSFeature(jobject, URI_INSTANTIATEUSING);
	}

	/**
	 * Get the serialize data Feature for this object. Since it depends
	 * on what resource that the metaclass is defined in, we must look for it.
	 */
	public static EAttribute getSerializeDataFeature(IJavaObjectInstance jobject) {
		return (EAttribute) getSFeature(jobject, URI_SERIALIZEDATA);
	}
		
	public static void initialize(Context context) {
			if (context.getAdapterFactory(IInstantiationAdapter.ADAPTER_KEY) == null)
				context.addAdapterFactory(new DefaultJavaInstantiationAdapterFactory(true));
	}
}
