package com.ibm.etools.java.instantiation.base;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.notify.Notification;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.java.*;
import com.ibm.etools.java.instantiation.IInstantiationAdapter;

/**
 * Adapter for instantiating java model objects. It doesn't save any state so there
 * is no need to have more than one. It's an internal adapter that other users won't
 * need to have access to.
 */
public class JavaObjectAdapter extends AdapterImpl implements IInstantiationAdapter {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public void notifyChanged(Notification msg){
	}
	
	public Notifier getTarget() {
		return null;
	}
	
	public boolean isAdapterForType(Object type) {
		return IInstantiationAdapter.ADAPTER_KEY.equals(type);
	}
	
	public void setTarget(Notifier target) {
		// We won't save the target, but we will set the correct instantiation class on it so that
		// that class will be created by default.
		JavaClass jc = (JavaClass) target;
		if (!jc.isSetInstanceClass())
			jc.setInstanceClass(JavaObjectInstance.class);
	}
	
	public boolean adapterCreatesInstance() {
		return false;
	}

	public RefObject eCreateInstance(JavaClass javaClass) {
		return null;	// Shouldn't be called
	}
		
	public boolean adapterCreatesDataType() {
		return false;
	}
		
	public Object createFromString(String s, JavaDataType dataType) {
		return null;	// Shouldn't be called
	}
			
	public boolean adapterConvertsToString() {
		return false;
	}
	
	public String convertToString(Object o, JavaDataType type) {
		return null;	// Shouldn't be called
	}
	
	public boolean isInstance(Object o, EClassifier type) {
		return o instanceof IJavaObjectInstance && ((JavaHelpers) type).isAssignableFrom((EClassifier) ((IJavaObjectInstance) o).getJavaType());
	}

}
