package com.ibm.etools.java.instantiation.base;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.NotImplementedException;
import com.ibm.etools.emf.ref.impl.RefObjectImpl;
import com.ibm.etools.java.JavaHelpers;

/**
 * This is the default instance for java model objects.
 * It should not be referenced directly, the IJavaObjectInstance interface should be
 * used instead. It is public so that it can be subclassed.
 */
public class JavaObjectInstance extends RefObjectImpl implements IJavaObjectInstance {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public JavaHelpers getJavaType(){
		return (JavaHelpers) refMetaObject();
	}
	
	public boolean isImplicit() {
		return isSetImplicit() ? ((Boolean) refValue(JavaInstantiation.getImplicitFeature(this))).booleanValue() : false;
	}
	
	public boolean isSetImplicit() {
		return refIsSet(JavaInstantiation.getImplicitFeature(this));
	}
	
	public void setImplicit(boolean implicit) {
		refSetValue(JavaInstantiation.getImplicitFeature(this), implicit ? Boolean.TRUE : Boolean.FALSE);
	}	
	
	public String getInitializationString() {
		if (isSetInitializationString())
			return (String) refValue(JavaInstantiation.getInitializationStringFeature(this));
		else {
			// Return construct with default ctor when not explicitly set.
			JavaHelpers jc = getJavaType();
			String qn = jc.getQualifiedName();
			return "new " + qn + "()"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public boolean isSetInitializationString() {
		return refIsSet(JavaInstantiation.getInitializationStringFeature(this));
	}
	
	public void setInitializationString(String initString) {
		refSetValue(JavaInstantiation.getInitializationStringFeature(this), initString);
	}
	
	public String getInstantiateUsing() {
		if (isSetInstantiateUsing())
			return (String) refValue(JavaInstantiation.getInstantiateUsingFeature(this));
		else
			return null;
	}
	
	public boolean isSetInstantiateUsing() {
		return refIsSet(JavaInstantiation.getInstantiateUsingFeature(this));
	}
	
	public void setInstantiateUsing(String aClassname) {
		refSetValue(JavaInstantiation.getInstantiateUsingFeature(this), aClassname);
	}
	
	public String getSerializeData(){
		if(isSetSerializeData())
			return (String) refValue(JavaInstantiation.getSerializeDataFeature(this));
		else
			return null;
	}
	
	public boolean isSetSerializeData(){
		return refIsSet(JavaInstantiation.getSerializeDataFeature(this));
	}
	
	public void setSerializeData(String serializableData){
		refSetValue(JavaInstantiation.getSerializeDataFeature(this), serializableData);
	}
	
	public boolean isPrimitive(){
		return false;
	}
	
	public String toString() {
		try {
			String initString = getInitializationString();
			if (initString != null)
				return super.toString() + ":" + initString; //$NON-NLS-1$
			else
				return super.toString();
		} catch (NotImplementedException e) {
			return super.toString();
		}
	}
	
}
