package com.ibm.etools.java.instantiation.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


 
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.core.resources.*;
import org.eclipse.core.boot.IPlatformRunnable;
import com.ibm.etools.proxy.test.JavaProjectUtil;
import com.ibm.etools.beaninfo.adapters.*;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;

import java.net.*;
import java.util.Iterator;

import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.emf.resource.ResourceFactoryRegister;
import com.ibm.etools.emf.resource.Resource;
import java.io.*;

import com.ibm.xmi.xmi2.XMIContentHelper;

/**
 * Do some tests of Java Modeling Instantiation.
 *
 * This is only a test class to see if things are set up correctly.
 * Because this should be run only under the supervision or request
 * of an IBM tech support, it is not NLSed.
 */
public class TestClassJavaModelingInstantiation implements IPlatformRunnable {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
public Object run(Object p1) throws CoreException {
	
	System.out.println("-- Initializing the test data --");
	final IProject[] project = new IProject[] {JavaProjectUtil.getProject("Test Beaninfo")};
	try {
		// Delete/create/populate within a runnable so that the build is done only after all changes have been made.
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				// First create/recreate the project.
				if (project[0] != null)
					project[0].delete(true, true, null);	// Get rid of it, we want a clean one for testing.
				
				try {
					// Now import the requested files.				
					JavaProjectUtil.importZipURLs(new Path("/Test Beaninfo"), new URL[] {new URL (BeaninfoPlugin.getPlugin().getDescriptor().getInstallURL(), "testdata/testbeaninfo-src.jar")}, monitor);			
				} catch(MalformedURLException e) {
				}
				project[0] = JavaProjectUtil.getProject("Test Beaninfo");
			}
		}, null);	
		System.out.println("-- Data initialized --");	
	
		BeaninfoNature nature = BeaninfoNature.getRuntime(project[0]);	// This will initialize the nature, i.e. connect with the java modeling and start the remote vm.
		// Fill in the required paths, this is picked up to be the same as from the test in Beaninfo plugin itself.
		// This path is one that is in the same project.
		nature.setSearchPath(
			new BeaninfosDoc(
				new IBeaninfosDocEntry[] {
					new SearchpathEntry(
						IClasspathEntry.CPE_SOURCE,
						new Path(""),
						"com.ibm.etools.beaninfo.testing.infos"),
					new BeaninfoEntry(
						BeaninfoEntry.createEntry(
							IClasspathEntry.CPE_SOURCE,
							new Path("/Test Beaninfo Beaninfos"),
							project[0],
							true),
						new SearchpathEntry[] { new SearchpathEntry("com.ibm.etools.beaninfo.testing.infos.project")},
						true),
					}));		
		
		com.ibm.etools.java.instantiation.base.JavaInstantiation.initialize(nature.getContext());
		
		if (nature.getRegistry() != null) {
			test(nature);		
		} else
			System.out.println("** No remote vm was created for some reason. **");
	} finally {
		if (project[0] != null)
			project[0].delete(true, false, null);	// Get rid of the project and the files themselves.
		ResourcesPlugin.getWorkspace().save(true, null);	// Save the current state of the workspace.
	}
	return null;
	
}

/**
 * Starts the application.
 * @param args an array of command-line arguments
 */
public void test(BeaninfoNature nature) {
	System.out.println("-------- Starting Standard Java Modeling Instantiation Test --------");

	ResourceSet rset = nature.getContext().getResourceSet();

	// Test one without a beaninfo. This will do reflection.
	Object bool = rset.getObjectAndLoad("java:/#boolean");
	JavaClass test1Class = (JavaClass) rset.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class");
	IJavaObjectInstance ji1 = (IJavaObjectInstance) test1Class.ePackageJavaRef().getJavaRefFactory().create(test1Class);
	ji1.setInitializationString("new Test1Class()");
	if (!ji1.isSetInitializationString())
		System.out.println("** Error: initstring not set. It should be.");
	if (!"new Test1Class()".equals(ji1.getInitializationString()))
		System.out.println("** Error: initstring not \"new Test1Class()\" :\"" + ji1.getInitializationString() + "\"");
		
	RefStructuralFeature sf = null;
	Iterator itr = test1Class.getEAttributes().iterator();
	while (itr.hasNext()) {
		RefStructuralFeature isf = (RefStructuralFeature) itr.next();
		if ("set".equals(isf.refName())) {
			sf = isf;
			break;
		}
	}
	Object v = null;
	if (sf == null)
		System.out.println("** Error: no property \"set\" for Test1Class.");
	else {
		XMIContentHelper xmi = (XMIContentHelper) ji1.getAdapter(XMIContentHelper.BY_CONTENT);
		xmi.setXMIValue(ji1, sf, "true");
	
		v = ji1.refValue(sf);
		if (!(v instanceof IJavaInstance))
			System.out.println("** Error: Value not a IJavaInstance. : \"" + v.getClass().toString());
		else
			if (((IJavaInstance) v).getJavaType() != bool)
				System.out.println("Error: Invalid java type for value: \"" + ((IJavaInstance) v).getJavaType().toString());
			else
				if (!("true".equals(((IJavaInstance) v).getInitializationString())))
					System.out.println("Error: Invalid initstring, not \"true\" : \"" + ((IJavaInstance) v).getInitializationString());
	}
	
	// Write it out, see if it is what it should be, then read it in and see if it loads correctly.
	ResourceSet rswork1 = ResourceFactoryRegister.getResourceSetFactory().makeResourceSet();
	rswork1.setContext(ResourceFactoryRegister.getContextFactory().makeContext());
	rswork1.getContext().setParent(rset.getContext());
	
	ResourceSet rswork2 = ResourceFactoryRegister.getResourceSetFactory().makeResourceSet();
	rswork2.setContext(ResourceFactoryRegister.getContextFactory().makeContext());
	rswork2.getContext().setParent(rset.getContext());

	
	Resource r = ResourceFactoryRegister.getFactory("f.xmi", rswork1).makeResource("f.xmi");
	r.getExtent().add(ji1);
	
	Resource rIn = null;

	try {	
		ByteArrayOutputStream bo = new ByteArrayOutputStream();
		r.save(bo);
		String master = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n<com.ibm.etools.beaninfo.testing:Test1Class xmi:version=\"2.0\" xmlns:xmi=\"http://www.omg.org/XMI\" xmlns:com.ibm.etools.beaninfo.testing=\"java:/com.ibm.etools.beaninfo.testing\" xmi:id=\"Test1Class_1\" initializationString=\"new Test1Class()\" set=\"true\"/>\r\n";
		if (!bo.toString().equals(master)) {
			System.out.println("Error: Serialized file doesn't match what it should. It follows:");
			System.out.println(bo.toString());
			return;
		}
		
		// Now read it back in and see if it loads correctly.
		rIn = rswork2.load("f1.xmi", new ByteArrayInputStream(bo.toByteArray()));
	} catch (Exception e) {
		e.printStackTrace();
		return;
	}
	
	// Now rerun the tests to see if rIn is identical to the original.
	test1Class = (JavaClass) rswork2.getObjectAndLoad("java:/com.ibm.etools.beaninfo.testing#Test1Class");
	ji1 = (IJavaObjectInstance) rIn.getExtent().getObjectByType(test1Class);
	if (!ji1.isSetInitializationString())
		System.out.println("** Error: initstring not set. It should be.");
	if (!"new Test1Class()".equals(ji1.getInitializationString()))
		System.out.println("** Error: initstring not \"new Test1Class()\" :\"" + ji1.getInitializationString() + "\"");
		
	sf = null;
	itr = test1Class.getEAttributes().iterator();
	while (itr.hasNext()) {
		RefStructuralFeature isf = (RefStructuralFeature) itr.next();
		if ("set".equals(isf.refName())) {
			sf = isf;
			break;
		}
	}
	v = null;
	if (sf == null)
		System.out.println("** Error: no property \"set\" for Test1Class.");
	else {
		XMIContentHelper xmi = (XMIContentHelper) ji1.getAdapter(XMIContentHelper.BY_CONTENT);
		xmi.setXMIValue(ji1, sf, "true");
	
		v = ji1.refValue(sf);
		if (!(v instanceof IJavaInstance))
			System.out.println("** Error: Value not a IJavaInstance. : \"" + v.getClass().toString());
		else
			if (((IJavaInstance) v).getJavaType() != bool)
				System.out.println("Error: Invalid java type for value: \"" + ((IJavaInstance) v).getJavaType().toString());
			else
				if (!("true".equals(((IJavaInstance) v).getInitializationString())))
					System.out.println("Error: Invalid initstring, not \"true\" : \"" + ((IJavaInstance) v).getInitializationString());
	}	
				
	System.out.println("-------- Ending the Standard Java Modeling Instantiation Tes --------");

}

}
