package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jface.viewers.LabelProvider;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.properties.AbstractPropertyDescriptorAdapter;
import com.ibm.etools.emf.ecore.EModelElement;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.beaninfo.PropertyDecorator;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.propertysheet.ISourced;
import com.ibm.etools.proxy.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.jface.resource.ImageDescriptor;
import java.net.*;
import org.eclipse.core.runtime.*;

public class BeanCellRenderer extends LabelProvider implements IExecutableExtension, INeedData, ISourced {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected static Image BEAN_IMAGE;
	protected String fJavaBeansPropertyEditorClassName;
	protected PropertyEditorBeanProxyWrapper fPropertyEditorProxyWrapper;
	protected EditDomain editDomain;
	protected IJavaObjectInstance source;
	protected boolean rebuildWrapper = true;

	/**
	 * This is called when this is on the BasePropertyDecorator for a type or if none is specified
	 * at all. If none specified, then just toBeanString will be used. It will later be
	 * called with the property editor class name in the initialization data. This is when
	 * there is no specific label provider or property editor on the feature.
	 */
	public BeanCellRenderer() {
	}

	public BeanCellRenderer(String aPropertyEditorClassName) {
		fJavaBeansPropertyEditorClassName = aPropertyEditorClassName;
	}
	/**
	 * By default use the toString() method on the bean
	 */
	public String getText(Object element) {
		if (element == null)
			return ""; //$NON-NLS-1$
		if (!(element instanceof IJavaInstance))
			return element.toString();

		IBeanProxy elementProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance) element, JBCFEditDomainHelper.getResourceSet(editDomain));
		if (elementProxy == null)
			return ""; //$NON-NLS-1$
	
		calculateJavaBeanPropertyEditor();
		
		if (fPropertyEditorProxyWrapper == null) {
			// The property editor class not given, or could not instantiate, so just do bean string.
			return elementProxy.toBeanString();
		}
		// The PropertyEditorBeanProxyWrapper wraps the java.beans.PropertyEditor on the remote VM
		// Use its setValue(Object) and getAsText() API to get a string		
		fPropertyEditorProxyWrapper.setValue(elementProxy);
		// For debug purposes puts a * in front of the string so we can tell it came from a java property editor
		return fPropertyEditorProxyWrapper.getAsText();
	}
	
	
	protected void calculateJavaBeanPropertyEditor() {
		if (fPropertyEditorProxyWrapper != null && !rebuildWrapper)
			return;	// We got one and no rebuild requested.

		rebuildWrapper = false;
		// We should of been given a property editor class name, if not, we don't set one up.
		if (fJavaBeansPropertyEditorClassName != null) {
			IConstructorProxy ctor = null;
			IBeanTypeProxy fPropertyEditorTypeProxy =
				JBCFEditDomainHelper.getBeanProxyDomain(editDomain).getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy(
					fJavaBeansPropertyEditorClassName);
			try {
				// if there is an ISourced object, see if the property editor has a ctor that takes an object and pass it in.
				if (source != null) {
					ctor = fPropertyEditorTypeProxy.getConstructorProxy(new String[] { "java.lang.Object" }); //$NON-NLS-1$
					if (ctor != null) {
						// got a ctor? let's instantiate the property editor
						fPropertyEditorProxyWrapper =
							new PropertyEditorBeanProxyWrapper(
								ctor.newInstance(new IBeanProxy[] { BeanProxyUtilities.getBeanProxy(source, JBCFEditDomainHelper.getResourceSet(editDomain))}));
					} else {
						// just instantiate with a null ctor
						fPropertyEditorProxyWrapper =
							new PropertyEditorBeanProxyWrapper(fPropertyEditorTypeProxy.newInstance());
					}
				} else {
					// no ISourced object... just instantiate with a null ctor
					fPropertyEditorProxyWrapper =
						new PropertyEditorBeanProxyWrapper(fPropertyEditorTypeProxy.newInstance());
				}
			} catch (ThrowableProxy exc) {
				// The ctor that takes an object may have failed because we gave it an object it didn't expect,
				// so let's try with the default ctor. But don't try again if the default ctor is the one that threw the exception.
				if (ctor != null) {
					try {
						fPropertyEditorProxyWrapper =
							new PropertyEditorBeanProxyWrapper(fPropertyEditorTypeProxy.newInstance());
					} catch (ThrowableProxy e) {
						// Still failed, so log it.
						JBCFPlugin.log(e);
					}
				} else {
					// It failed and we had used the default ctor, so log it.
					JBCFPlugin.log(exc);
				}
					
			}
		}
	}

	
	
	/**
	 * This will only expect initData to be a string that contains the qualified name
	 * of the java.beans.PropertyEditor class that is used to edit the feature
	 */
	public void setInitializationData(IConfigurationElement ce, String pName, Object initData) {
		if (initData instanceof String) {
			fJavaBeansPropertyEditorClassName = ((String) initData).trim();
		}
	}
	/**
	 * @see INeedData#setData(Object)
	 */
	public void setData(Object data) {
		editDomain = (EditDomain) data;
	}
	
	public void setSources(Object[] sources, IPropertySource[] pos, IPropertyDescriptor[] des) {
		if (sources[0] instanceof IJavaObjectInstance) {
			rebuildWrapper = !(sources[0].equals(source));
			source = (IJavaObjectInstance) sources[0];
		}
	}	

}