package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.properties.AbstractPropertyDescriptorAdapter;
import com.ibm.etools.propertysheet.IEToolsPropertyDescriptor;
import com.ibm.etools.cde.decorators.PropertyDescriptorDecorator;
import com.ibm.etools.cde.decorators.BasePropertyDecorator;
import com.ibm.etools.beaninfo.PropertyDecorator;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.java.*;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.cde.decorators.DecoratorsPackage;
import com.ibm.etools.beaninfo.FeatureAttributeValue;
import com.ibm.etools.cde.emf.ClassDecoratorFeatureAccess;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.proxy.ProxyFactoryRegistry;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ecore.EModelElement;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.jface.viewers.CellEditor;
import java.util.*;
import com.ibm.etools.proxy.IArrayBeanProxy;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.CDEPlugin;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.ILabelProvider;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.proxy.IStringBeanProxy;
import java.lang.reflect.Constructor;
import java.text.MessageFormat;

/**
 * The creation method for BeanProperties is different than for regular MOF properties
 * The order is
 * 	1	-	Look for a CellEditor described directly on the EMF feature
 *	2	-	Look for a java.beans.PropertyEditor described on the Beans PropertyDescriptor
 *	3	-	Look for any enumerated values on the BeanInfo.  If so use a special property editor that can handle these
 *	4	-	Look for an CellEditor described on the EMF type
 */
public class BeanPropertyDescriptorAdapter extends AbstractPropertyDescriptorAdapter implements IEToolsPropertyDescriptor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final Class BEAN_PROPERTY_DESCRIPTOR_TYPE = BeanPropertyDescriptorAdapter.class;
	public static final String ATTRIBUTE_NAME_ENUMERATIONVALUES = "enumerationValues"; //$NON-NLS-1$
	
public CellEditor createPropertyEditor(Composite parent){
	
	if (!((RefStructuralFeature) getTarget()).refIsChangeable())
		return null;	// The feature is a read-only, so no editor to allow changes
	
	Class editorClass = null;
	String classNameAndData = ""; //$NON-NLS-1$
		
	// Step 1 - See if the emf feature decorator has it.
	BasePropertyDecorator decorator = getBaseDecorator();
	if (decorator != null && decorator.isSetCellEditorClassname()) {
		try {
			classNameAndData = decorator.getCellEditorClassname();
			editorClass = CDEPlugin.getClassFromString(classNameAndData);
		} catch (ClassNotFoundException e) {
			// One specified, but incorrect, log it, but continue and see if we can get another way.
			JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e), JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
		}
	}
	
	//	2 - Look for a java.beans.PropertyEditor on the Bean PropertyDescriptor
	if ( editorClass == null ) {
		PropertyDecorator propertyDecorator = Utilities.getPropertyDecorator((EModelElement)target);
		if ( propertyDecorator != null ) {
			JavaClass propertyEditorClass = propertyDecorator.getPropertyEditorClass();
			// If we have a property editor class explicitly defined on the feature then instantiate it
			if ( propertyEditorClass != null ) {				
				return new BeanFeatureEditor(
				(JavaHelpers) ((RefStructuralFeature)getTarget()).refType(),
				parent,
				propertyEditorClass.getQualifiedNameForReflection());
			}
			// 3	-	VAJava had a wierd feature ( that we have to support for backward compatiliby )
			// where you could get enumerated values code generated for you
			// These were stored with a key of ENUMERATIONVALUES and were an array
			// that was arranged in a tiplicate repeating pattern of displayName, value, initString
			// e.g. "On" , Boolean.TRUE , "Boolean.TRUE" , "Off" , Boolean.FALSE , "Boolean.FALSE"
			// or	 "Vertical" , new Integer(0) , "java.awt.Scrollbar.HORIZONTAL" , "Horizontal" , new Integer(1) , "java.awt.Scrollbar.VERTICAL"
			Iterator propertyAttributes = propertyDecorator.getAttributes().iterator();
			while ( propertyAttributes.hasNext() ) {
				FeatureAttributeValue featureValue = (FeatureAttributeValue)propertyAttributes.next();
				if (featureValue.getName().equals(ATTRIBUTE_NAME_ENUMERATIONVALUES)){
					IArrayBeanProxy enums = null;
					if (featureValue.isSetValueProxy())
						enums = (IArrayBeanProxy) featureValue.getValueProxy();
					else if (featureValue.isSetValue()) {
						IJavaObjectInstance enumsValue = (IJavaObjectInstance) featureValue.getValue();
						if (enumsValue != null) {
							CDEHack.fixMe();	// Need a way of creating the proxy from the value . PRoblem
												// is that this value is in a resourceset that has no access
												// to BeanProxyHost, nor do we want it too because it would
												// put the enums proxy into the wrong registry. Problem 
												// is at this point in time we don't know what registry
												// to use. Maybe if we could come up with a way of creating
												// a proxy host without the adapter factory and using
												// the registry of the beaninfo.
						}
					}
					
					if (enums != null) {		
						return new EnumeratedCellEditor(
							parent,
							enums,
							(JavaHelpers) ((RefStructuralFeature)target).refType());
					}
				}
			}			
		}
	}
		
	// Step 4 - If not on feature, then get it from the type, look for BasePropertyDecorator.
		if ( editorClass == null ) {
			DecoratorsPackage dp = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
			BasePropertyDecorator bdec = (BasePropertyDecorator) ClassDecoratorFeatureAccess.getDecoratorWithFeature((EClassifier) ((RefStructuralFeature) target).refType(), BasePropertyDecorator.class, dp.getBasePropertyDecorator_CellEditorClassname());
			if (bdec != null) {
				try {
					classNameAndData = bdec.getCellEditorClassname();
					editorClass = CDEPlugin.getPlugin().getClassFromString(classNameAndData);
				} catch (ClassNotFoundException e) {
					// One specified, but incorrect, log it, but continue and see if we can get another way.
					JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e), JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
				}
			}
				
		}
		
	if ( editorClass != null ) {
		return createCellEditorInstance(editorClass,parent, classNameAndData, null);
	} else
		return null;
}
	
public String[] getFilterFlags(){
	return null;
}
public ICellEditorValidator getValidator(){
	return null;
}
public Object getHelpContextIds(){
	return null;
}
/**
 * 1	-	If the feature has a LabelProvider then we should use this.
 * 2	- 	See whether a java.beans.PropertyEditor exists for the feature, if so
 * 			BeanCellRenderer which is able to wrapper the PropertyEditor to get the string
 * 3	-	See whether there are any enumeration values on from the BeanInfo.  If so use
 * 			a special label provider that knows how to handle these
 * 4	-	See if a LabelProvider exists for the JavaHelpers that can handle this particular type
 *			such as an editor that can handle IStringBeanProxy or IDimensionBeanProxy without 
 * 			going over to the target VM
 * 5	-	Just a BeanCellRenderer. It will do a toBeanString.
 */
public ILabelProvider getLabelProvider(){
	
	Class labelProviderClass = null;
	String classNameAndData = ""; //$NON-NLS-1$
		
	// Step 1 - See if the emf feature decorator has a label provider.
	BasePropertyDecorator decorator = getBaseDecorator();
	if (decorator != null && decorator.isSetLabelProviderClassname()) {
		try {
			classNameAndData = decorator.getLabelProviderClassname();
			labelProviderClass = CDEPlugin.getClassFromString(classNameAndData);
		} catch (ClassNotFoundException e) {
			// One specified, but incorrect, log it, but continue and see if we can get another way.
			JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e), JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
		}
	}
	
	// Step 2 - See if there is a java.beans.PropertyEditor on the decorator that holds information
	//			about the introspected java.beans.PropertyDescriptor
	if ( labelProviderClass == null ) {
		PropertyDecorator propertyDecorator = Utilities.getPropertyDecorator((EModelElement)target);
		if ( propertyDecorator != null ) {
			JavaClass propertyEditorClass = propertyDecorator.getPropertyEditorClass();
			// If we have a property editor class explicitly defined on the feature then instantiate it
			if ( propertyEditorClass != null ) {
				// java.beans.PropertyEditorManager does not allow property editors to be de-registered
				// Sun provide a number of defaults that we de-register by placing a placeholder 
				// DummyPropertyEditor there that we must detect here
				return new BeanCellRenderer(propertyEditorClass.getQualifiedNameForReflection());
			}
			// 3	-	Look for the enumeration values
			Iterator propertyAttributes = propertyDecorator.getAttributes().iterator();			
			while ( propertyAttributes.hasNext() ) {
				FeatureAttributeValue featureValue = (FeatureAttributeValue)propertyAttributes.next();
				if (featureValue.getName().equals(ATTRIBUTE_NAME_ENUMERATIONVALUES)){
					return new EnumeratedLabelProvider(
						(IArrayBeanProxy)featureValue.getValueProxy(),
						(JavaHelpers) ((RefStructuralFeature)target).refType());
				}
			}			
		}			
	}
	
	// Step 4 - See if there is a label provider on the decorator for the class on its BasePropertyDecorator
	if ( labelProviderClass == null ) {
		DecoratorsPackage dp = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
		BasePropertyDecorator bdec = (BasePropertyDecorator) ClassDecoratorFeatureAccess.getDecoratorWithFeature((EClassifier) ((RefStructuralFeature) target).refType(), BasePropertyDecorator.class, dp.getBasePropertyDecorator_LabelProviderClassname());
		if (bdec != null) {
			try {
				classNameAndData = bdec.getLabelProviderClassname();
				labelProviderClass = CDEPlugin.getPlugin().getClassFromString(classNameAndData);
			} catch (ClassNotFoundException e) {
				// One specified, but incorrect, log it, but continue and see if we can get another way.
				JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e), JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
			}
		}
	}	
	
	if ( labelProviderClass != null ) {
		return createLabelProviderInstance(labelProviderClass, classNameAndData, null, this);
	} else {
		// Step 5 - BeanCellRenderer will simply do toBeanString.
		return new BeanCellRenderer();
	}
}
public boolean isExpandable() {
	// Step 1 - See if the feature decorator has it.
	BasePropertyDecorator decorator = getBaseDecorator();
	if (decorator != null && decorator.isSetIsEntryExpandable())
		return decorator.isEntryExpandable();				
			
	// Step 2 - If not on feature, then get it from the type, look for BasePropertyDecorator.
	DecoratorsPackage dp = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
	BasePropertyDecorator bdec = (BasePropertyDecorator) ClassDecoratorFeatureAccess.getDecoratorWithFeature((EClassifier) ((RefStructuralFeature) target).refType(), BasePropertyDecorator.class, dp.getBasePropertyDecorator_IsEntryExpandable());
	if (bdec != null)
		return bdec.isEntryExpandable();				
				
	return true;	
}
	
public boolean areNullsInvalid() {

	// Step 1 - See if the feature decorator has it.
	BasePropertyDecorator decorator = getBaseDecorator();
	if (decorator != null && decorator.isSetIsNullInvalid())
		return decorator.isNullInvalid();				
			
	// Step 2 - If not on feature, then get it from the type, look for BasePropertyDecorator.
	DecoratorsPackage dp = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
	BasePropertyDecorator bdec = (BasePropertyDecorator) ClassDecoratorFeatureAccess.getDecoratorWithFeature((EClassifier) ((RefStructuralFeature) target).refType(), BasePropertyDecorator.class, dp.getBasePropertyDecorator_IsNullInvalid());
	if (bdec != null)
		return bdec.isNullInvalid();
				
	return false;	
}

public String getCategory(){
	PropertyDecorator propertyDecorator = Utilities.getPropertyDecorator((EModelElement)target);
	if ( propertyDecorator != null) {
		Iterator propertyAttributes = propertyDecorator.getAttributes().iterator();			
		while ( propertyAttributes.hasNext() ) {
			FeatureAttributeValue featureValue = (FeatureAttributeValue)propertyAttributes.next();
			if (featureValue.getName().equals("category")){ //$NON-NLS-1$
				Object value = featureValue.getValueProxy();		
				// value is an IStringBeanProxy 
				if (value instanceof IStringBeanProxy){
					IStringBeanProxy proxy = (IStringBeanProxy)value;
					return proxy.stringValue();
				}
			}
		}	
	}
	return null;

}

public String getDescription(){
	return null;
}

public boolean isReadOnly(){
	return !(((RefStructuralFeature) getTarget()).refIsChangeable());
}

public String primGetDisplayName(){
	//return getFeature().refName();
	PropertyDecorator propertyDecorator = Utilities.getPropertyDecorator((EModelElement)target);
	if ( propertyDecorator != null) {
		if (propertyDecorator.getDisplayName() != null){
			return propertyDecorator.getDisplayName();
		}
	}
	return null;

}
protected RefStructuralFeature getFeature(){
	return (RefStructuralFeature)getTarget();
}
protected BasePropertyDecorator getBaseDecorator() {
	// Return the BasePropertyDecorator for this feature.
	return (BasePropertyDecorator) findDecorator(((EModelElement) target).getEDecorators(), BasePropertyDecorator.class);
}
/**
 * instantiate the  class passed in. If it has an ctor that
 * takes a JavaHelpers, use that one.
 * A CellEditor must be constructed with a Composite
 */
protected CellEditor createCellEditorInstance(Class clazz,Composite aComposite, String data, Object initData) {
	Constructor ctor = null;
	CellEditor editor = null;
	try {
		ctor = clazz.getConstructor(new Class[] {JavaHelpers.class,Composite.class});
	} catch (NoSuchMethodException e) {
	};
	try {
		if (ctor != null) {
			editor = (CellEditor) ctor.newInstance(new Object[] {
				(JavaHelpers) ((RefStructuralFeature)getTarget()).refType(),
				aComposite});
		} else {
			ctor = clazz.getConstructor(new Class[] {Composite.class});
			editor = (CellEditor) ctor.newInstance(new Object[] {aComposite});
		}
		
		CDEPlugin.setInitializationData(editor, data, initData);
		ICellEditorValidator validator = getValidator();
		if (validator != null)
			editor.setValidator(validator);
	} catch (Exception exc) {
		String msg = MessageFormat.format(CDEMessages.getString(CDEMessages.OBJECT_NOINSTANTIATE), new Object[] {clazz});		
		JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, msg, exc), JBCFMsgLogger.LOG_WARNING);
	}
	return editor;
}
}