package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import com.ibm.etools.cde.properties.PropertySourceAdapter;
import org.eclipse.ui.views.properties.IPropertySource;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.beaninfo.*;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import com.ibm.etools.emf.ecore.EModelElement;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.resource.ResourceSet;
/**
 * Property Source adaptor for Beans.
 * Creation date: (1/19/00 1:43:16 PM)
 * @author: Joe Winchester
 */
public class BeanPropertySourceAdapter extends PropertySourceAdapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
/**
 * Compares two objects for equality. Returns a boolean that indicates
 * whether this object is equivalent to the specified object. This method
 * is used when an object is stored in a hashtable.
 * @param obj the Object to compare with
 * @return true if these Objects are equal; false otherwise.
 * @see java.util.Hashtable
 */

public boolean equals(Object obj) {
	if (super.equals(obj))
		return true;

	// Now also check to see if the bean proxies are equal. This could be if two different
	// mof targets, but the actual bean proxy on the VM is the same one. this occurs
	// because when we query property value, if not set in MOF, we then query the
	// live object. This can create a new bean proxy each time, but for the same bean.
	if (obj instanceof IPropertySource)
		obj = ((IPropertySource) obj).getEditableValue();
	if (obj instanceof RefObject) {
		IBeanProxyHost otherProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance)obj);
		if (otherProxyHost != null) {
			IBeanProxy otherProxy = otherProxyHost.getBeanProxy();
			if (otherProxy != null) {
				IBeanProxyHost myProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance)target);
				if (myProxyHost != null)
					return otherProxy.equals(myProxyHost.getBeanProxy());
			}
		}
	}
	return false;
}
/**
 * Beans are special because of the customizer support. The customizer can change
 * any property, so we need to be able to answer the hidden, expert, etc properties
 * so that the customizer can change them.
 */
public HashMap getCustomizableDescriptors() {
//	List descr = getTargetPropertyDescriptors(); // This gets them all.
com.ibm.etools.cde.CDEHack.fixMe(); // This was all commented out and I don't what it was trying to do, so for now do nothing.

	// Now copy to another hashtable, exclude non-bean descriptors (since
	// customizers couldn't handle them anyway) and descriptors explicitly
	// set to NOT design time. The others are ok.
	HashMap result = new HashMap(1 /* descr.size() */);
//	Iterator iter = descr.iterator();
//	while (iter.hasNext()) {
//		TypedElementExtn attr = (TypedElementExtn) enum.nextElement();
//		if (attr instanceof IBeanAttribute) {
//			IBeanAttribute beanProp = (IBeanAttribute) attr;
//			Object descriptor;
//			if (beanProp.isDesignTimeProperty() != Boolean.FALSE) {
//				if (!(beanProp instanceof ISourcedFeature))
//					descriptor = ((IAdaptable) beanProp).getAdaptor(PropertyDescriptorAdaptor.IPROPERTYDESCRIPTOR_TYPE);
//				else
//					descriptor = ((ISourcedFeature) beanProp).getDescriptor(target);
//				result.put(attr.getName(), descriptor);
//			}
//		}
//	}
	return result;
}
/**
 * Get The property value from the target and wrap as a property source if necessary.
 */
public Object getPropertyValue(Object structuralFeature) {
	
	Object value = null;
	if (!getRefTarget().refIsSet((RefStructuralFeature) structuralFeature)) {
		// The value was not explicitly set in the MOF Object, so we will get the Bean Proxy Adaptor
		// and ask it for the live value from the real java bean over in the VM.
		IBeanProxyHost sourceHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) getTarget());
		value = sourceHost.getBeanPropertyValue((RefStructuralFeature) structuralFeature);
		if (value != null) {
			// Put a propertySource adapter onto it because one will be needed but this guy isn't in a resource set to get it.
			// Set it an IPropertySource so that we don't loop it up again below.
			ResourceSet rset = ((RefObject) getTarget()).getResourceSet();
			IPropertySource ps = (IPropertySource) rset.getContext().getAdapterFactory(IPropertySource.class).adapt((Notifier) value, IPropertySource.class);
			if (ps != null)
				value = ps;	// It has a property source adapter.
		}
	} else
		value = getRefTarget().refValue((RefStructuralFeature)structuralFeature);	
		
	if (!(value instanceof IPropertySource) && value instanceof Notifier) {
		IPropertySource ps = (IPropertySource) ((Notifier) value).getAdapter(IPropertySource.class);
		return ps != null ? ps : value;
	} else
		return value;
}

/**
 * Bean method for filtering attributes.
 */
protected boolean includeFeature(RefStructuralFeature aFeature) {
	//first check if PropertyDecorator sets the feature to be hidden in property sheet
	PropertyDecorator propDecor = Utilities.getPropertyDecorator((EModelElement)aFeature);
	if (propDecor != null) {
		boolean propDecorFlag =  !propDecor.isHidden() &&
			(!propDecor.isSetIsDesignTime() || propDecor.isDesignTime());		
		if (! propDecorFlag) return false;
	}	
	
	//must call superclass method to check if the PropertyDescriptorAdatper sets the 
	//feature to be hidden in property sheet			
	return super.includeFeature(aFeature);
				
}

}
