package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.lang.reflect.Constructor;

import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.ClassDecoratorFeatureAccess;
import com.ibm.etools.emf.ecore.EAttribute;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.jbcf.BeanDecorator;
import com.ibm.etools.emf.jbcf.JBCFPackage;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.impl.AbstractAdapterFactoryImpl;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.proxy.ProxyFactoryRegistry;

/**
 * Factory for creating the BeanProxyAdaptor.
 * Creation date: (1/10/00 12:28:38 PM)
 * @author: Richard Lee Kulp
 */
public class BeanProxyAdapterFactory
	extends AbstractAdapterFactoryImpl
	implements IBeanProxyDomain {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ProxyFactoryRegistry fProxyFactoryRegistry;
	protected EditDomain domain;
	protected static EAttribute BEAN_PROXY_CLASS_NAME;

	/**
	 * BeanProxyAdaptorFactory constructor comment.
	 */
	public BeanProxyAdapterFactory(ProxyFactoryRegistry aProxyFactoryRegistry, EditDomain domain) {
		super(true);
		fProxyFactoryRegistry = aProxyFactoryRegistry;
		this.domain = domain;
		JBCFEditDomainHelper.setBeanProxyDomain(this, domain);
	}

	public ProxyFactoryRegistry getProxyFactoryRegistry() {
		return fProxyFactoryRegistry;
	}
	
	public void setProxyFactoryRegistry(ProxyFactoryRegistry registry) {
		fProxyFactoryRegistry = registry;
	}

	public EditDomain getEditDomain() {
		return domain;
	}

	protected static EAttribute getBeanProxyClassNameFeature() {
		// Lazy initialize the meta class for the BeanDecorator
		if (BEAN_PROXY_CLASS_NAME == null) {
			JBCFPackage jbcfPackage = (JBCFPackage) RefRegister.getPackage(JBCFPackage.packageURI);
			BEAN_PROXY_CLASS_NAME = jbcfPackage.getBeanDecorator_BeanProxyClassName();
		}
		return BEAN_PROXY_CLASS_NAME;
	}
	/**
	 * Adapt us based upon whether we have an attribute for "beanProxyAdaptor"
	 * Otherwise use a default adaptor
	 */
	protected Adapter createAdapter(Notifier adaptable, Object key) {
		if (!(adaptable instanceof IJavaInstance))
			return null; // Only Beans can be adapted to a beanProxyAdaptor.
		Adapter adapter = null;

		IJavaInstance bean = (IJavaInstance) adaptable;
		// The class of adapter to use comes from the BeanDecorator.  Get the one that has the property set
		BeanDecorator decr =
			(BeanDecorator) ClassDecoratorFeatureAccess.getDecoratorWithFeature(
				(EClassifier) bean.getJavaType(),
				BeanDecorator.class,
				getBeanProxyClassNameFeature());
		String adapterClassName = decr != null ? decr.getBeanProxyClassName() : null;

		if (adapterClassName != null) {
			Class adapterClass = null;
			try {
				// If the class is not the default we need to load it using the correct
				// class lodaed.  CDEPlugin can do this for us but if the name is BeanProxyAdapter or PrimitiveBeanProxyAdapter we can
				// just get the default class faster with a .class reference here
				if (adapterClassName.equals("com.ibm.etools.jbcf/com.ibm.etools.jbcf.BeanProxyAdapter")) //$NON-NLS-1$
					adapter = new BeanProxyAdapter(this);
				else if (adapterClassName.equals("com.ibm.etools.jbcf/com.ibm.etools.jbcf.PrimitiveProxyAdapter")) //$NON-NLS-1$
					adapter = new PrimitiveProxyAdapter(this);
				else
					adapterClass = CDEPlugin.getClassFromString(adapterClassName);
			} catch (ClassNotFoundException e) {
				adapterClass = null;
				JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
			}
			if (adapterClass != null)
				try {
					// There must be a constructor that takes an argument with the IBeanProxyDomain.
					Constructor constructor = adapterClass.getConstructor(new Class[] { IBeanProxyDomain.class });
					adapter = (Adapter) constructor.newInstance(new Object[] { this });
				} catch (Exception e) {
					JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
				}
		}

		return adapter != null ? adapter : new BeanProxyAdapter(this);
	}

	public boolean isFactoryForType(Object key) {
		return IBeanProxyHost.BEAN_PROXY_TYPE.equals(key);
	}

	public void adaptAllNew(Notifier notifier) {
		adaptNew(notifier, IBeanProxyHost.BEAN_PROXY_TYPE);
	}
}