package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.proxy.ThrowableProxy;
import com.ibm.etools.proxy.IMethodProxy;
import com.ibm.etools.proxy.IBeanTypeProxy;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.beaninfo.PropertyDecorator;
import com.ibm.etools.proxy.ProxyFactoryRegistry;
import com.ibm.etools.emf.ecore.EModelElement;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaDataType;
import com.ibm.etools.emf.jbcf.impl.JBCFPackageImpl;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.JavaClass;
import java.util.*;

public class BeanUtilities {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
/**
 * Create the object for the qualified class name.  The resourceSet is used only for reflection to look
 * for the class, and the object returned is not added to any resource set - the caller must do that if required
 * The class can be qualified class, e.g. java.lang.String
 * The java object has not been added to a resource set, however the calls of this method
 * may want to create a BeanProxyHost adaptor for it.  This is done as part of this method
 * because if not then it cannot occur because no adaptor factory can be found
 */
public static IJavaInstance createJavaObject(String qualifiedClassName, ResourceSet aResourceSet, String javaInitializationString ){	
	return createJavaObject(JavaClassImpl.reflect(qualifiedClassName,aResourceSet), aResourceSet, javaInitializationString);
}

/**
 * Create a java object given the class. If the resourceset is null, then it won't have
 * the beanproxyhost created. However, if a beanproxy host is needed before this gets set into
 * any resource, then the resource set must be supplied.
 */
public static IJavaInstance createJavaObject(JavaHelpers javaHelpers, ResourceSet aResourceSet, String javaInitializationString){
	IJavaInstance result = null;
	if ( javaHelpers.isPrimitive() ) {
		result = (IJavaInstance) ((JavaDataType)javaHelpers).createFromString(javaInitializationString);
	} else { 
		JavaClass javaClass = (JavaClass)javaHelpers;
	 	IJavaObjectInstance newInstance = (IJavaObjectInstance) javaClass.refPackage().getFactory().create(javaClass);
	 	if (javaInitializationString != null)
		 	newInstance.setInitializationString(javaInitializationString);
	 	result = newInstance;
	}
	
	if (aResourceSet != null) {
		// This method call on BeanProxyUtilities creates an adaptor in the argument resource set
		BeanProxyUtilities.getBeanProxyHost(result, aResourceSet);
	}

	return result;
}

public static IJavaObjectInstance createString(ResourceSet aResourceSet,String unquotedInitializationString){
		return (IJavaObjectInstance) createJavaObject("java.lang.String",aResourceSet, createStringInitString(unquotedInitializationString)); //$NON-NLS-1$
}

/**
 * Take a string and turn it into a init string for the String class. I.e. quote it and escape any imbedded quotes or escape chars.
 */
public static String createStringInitString(String value) {
	StringBuffer sb = new StringBuffer(value.length());
	sb.append('"');
	int sl = value.length();
	for (int i=0; i<sl; i++) {
		char c = value.charAt(i);
		if (c == '"' || c == '\\')
			sb.append('\\');	// We need to escape it.
		sb.append(c);
	}
	sb.append('"');
	return sb.toString();
}

public static String getUnqualifiedClassName(String aQualifiedClassName){

	int indexOfLastPeriod = aQualifiedClassName.lastIndexOf('.');
	if ( indexOfLastPeriod >= 0 ) {
		return aQualifiedClassName.substring(indexOfLastPeriod + 1 ,aQualifiedClassName.length());
	} else {
		return aQualifiedClassName;
	}
	
}
/**
 * Answer whether or not this is the class being composed
 */
public static boolean isThisPart(IJavaObjectInstance aBean){
	
	RefStructuralFeature containerSF = aBean.refContainerSF();
	return containerSF.refName().equals("thisPart"); //$NON-NLS-1$
				
}
}