package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.propertysheet.BooleanCellEditor;
import java.util.*;
import org.eclipse.core.runtime.*;
import org.eclipse.swt.widgets.Composite;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.emf.ecore.EMetaObject;
import com.ibm.etools.proxy.IBooleanBeanProxy;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.propertysheet.PropertysheetMessages;
import com.ibm.etools.cde.EditDomain;
/**
 * Boolean type property editor.
 * It will default to boolean unless the initData string is "class". In that case it will
 * be for java.lang.Boolean.
 */
public class BooleanJavaCellEditor extends BooleanCellEditor implements IExecutableExtension, INeedData , IJBCFCellEditor {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain fEditDomain;
	protected boolean fIsBooleanClass;	// Is this a boolean class editor or a boolean type editor.
	protected JavaHelpers fBooleanHelper;	// The appropriate classifer for the kind of boolean.
	
	public BooleanJavaCellEditor(Composite parent){
		super(parent);
	}
	
	public void setInitializationData(IConfigurationElement ce, String pName, Object initData) {
		fIsBooleanClass = initData instanceof String && "class".equalsIgnoreCase((String)initData);	// The initdata is "class", then this is for type class. //$NON-NLS-1$
	}

	public void setData(Object data) {
		fEditDomain = (EditDomain) data;
		if (!fIsBooleanClass) {
			fBooleanHelper = JavaClassImpl.reflect("boolean", JBCFEditDomainHelper.getResourceSet(fEditDomain)); //$NON-NLS-1$
			// Set into the items the data types for boolean.
			setItems(getDisplayStrings(),
				new Object[] {
					BeanUtilities.createJavaObject(fBooleanHelper, JBCFEditDomainHelper.getResourceSet(fEditDomain), "true"), //$NON-NLS-1$
					BeanUtilities.createJavaObject(fBooleanHelper, JBCFEditDomainHelper.getResourceSet(fEditDomain), "false") //$NON-NLS-1$
				});
		} else
			fBooleanHelper = JavaClassImpl.reflect("java.lang.Boolean", JBCFEditDomainHelper.getResourceSet(fEditDomain));		 //$NON-NLS-1$
	}
	
	protected String isCorrectObject(Object value) {
		if (value == null)
			if (!fIsBooleanClass)
				return PropertysheetMessages.getString(PropertysheetMessages.NULL_INVALID);	// Null is invalid for primitives.
			else
				return null;
				
		else if (((EMetaObject) fBooleanHelper).isInstance(value))
			return null;
		else
			return PropertysheetMessages.getString(PropertysheetMessages.NOT_BOOL);
	}
		
	public String getJavaInitializationString(){
		if (getSelectionIndex() == 0){
			return "true"; //$NON-NLS-1$
		} else {
			return "false"; //$NON-NLS-1$
		}
	}
	/**
	 * The object is being passed in, return
	 * the index to be used in the editor.
	 *
	 * It should return sNoSelection if the value can't be
	 * converted to a index. The errormsg will have
	 * already been set in this case.
	 */
	protected int doGetIndex(Object value){
		if (((EMetaObject) fBooleanHelper).isInstance(value))
			return ((IBooleanBeanProxy) BeanProxyUtilities.getBeanProxy((IJavaInstance) value, JBCFEditDomainHelper.getResourceSet(fEditDomain))).booleanValue() ? sTrueIndex : sFalseIndex;
		else
			return sNoSelection;
	}
	
	/**
	 * This is called when
	 * editing and a selection from the combobox is sent
	 * in and we need to send the object that it represents
	 * up to the validators. The index to convert will be
	 * passed in.
	 */
	protected Object doGetObject(int index) {
		if (!fIsBooleanClass)
			return super.doGetObject(index);	// Use the datatype instances passed in.
			
		// This is class boolean, so we need to new up a new one.
		switch (index) {
			case sNoSelection:
				return null;
			case sTrueIndex:
				return BeanUtilities.createJavaObject(fBooleanHelper, JBCFEditDomainHelper.getResourceSet(fEditDomain), "Boolean.TRUE"); //$NON-NLS-1$
			case sFalseIndex:
				return BeanUtilities.createJavaObject(fBooleanHelper, JBCFEditDomainHelper.getResourceSet(fEditDomain), "Boolean.FALSE");			 //$NON-NLS-1$
		}
		
		return null;
	}
			
}
