package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.widgets.*;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.*;
import com.ibm.etools.propertysheet.BooleanLabelProvider;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import java.net.*;
import org.eclipse.jface.resource.*;
import com.ibm.etools.proxy.IBooleanBeanProxy;
import com.ibm.etools.proxy.IBeanProxy;
import org.eclipse.swt.graphics.*;
/**
 * This is a generic cell renderer that can format boolean correctly
 * It goes to the bean proxy to get the boolean value, however because it casts to the
 * the specifc type it doesn't actually trip to the target VM as the value of the boolean
 * is stored in the IDE's implementation of IBeanProxy
 */
public class BooleanJavaLabelProvider extends BooleanLabelProvider implements INeedData {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private static Image TRUE_IMAGE;
	private static Image FALSE_IMAGE;	
	
	protected EditDomain editDomain;
	
public String getText(Object element){
	
	if (element instanceof IJavaInstance) {
		IBeanProxy proxy = BeanProxyUtilities.getBeanProxy((IJavaInstance)element, JBCFEditDomainHelper.getResourceSet(editDomain));
		if (proxy == null)
			return "";	// It shouldn't be null. //$NON-NLS-1$
		else if (proxy instanceof IBooleanBeanProxy)
			return super.getText(((IBooleanBeanProxy) proxy).getBooleanValue());
		else
			return proxy.toBeanString();
	} else
		return super.getText(element);
}
public Image getImage(Object element){
	// Too many problems with images now - just do nothing right now
	if(true)return null;
	if (element instanceof IJavaInstance) {
		IBooleanBeanProxy proxy = (IBooleanBeanProxy)BeanProxyUtilities.getBeanProxy((IJavaInstance)element, JBCFEditDomainHelper.getResourceSet(editDomain));
		if(proxy.getBooleanValue().booleanValue()){
			return getTrueImage();
		} else {
			return getFalseImage();
		}
	}	
	return null;
}
public static Image getTrueImage(){
	if ( TRUE_IMAGE == null ) {
		TRUE_IMAGE = getBooleanImage(true);
	}
	return TRUE_IMAGE;
}
public static Image getFalseImage(){
	if ( FALSE_IMAGE == null ) {
		FALSE_IMAGE = getBooleanImage(false);
	}
	return FALSE_IMAGE;
}
public static Image getBooleanImage(boolean aBool){

	String url = "platform:/plugin/com.ibm.etools.jbcf/icons/"; //$NON-NLS-1$
	if ( aBool ) {
		url = url + "booleantrue.gif"; //$NON-NLS-1$
	} else {
		url = url + "booleanfalse.gif"; //$NON-NLS-1$
	}
	try {
		return ImageDescriptor.createFromURL(new URL(url)).createImage();
	} catch (MalformedURLException e) {
		return null;
	}		
}
	/**
	 * @see INeedData#setData(Object)
	 */
	public void setData(Object data) {
		editDomain = (EditDomain) data;
	}

}