package com.ibm.etools.jbcf;

/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.text.MessageFormat;

import org.eclipse.swt.widgets.Composite;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaDataType;
import com.ibm.etools.java.instantiation.base.IJavaDataTypeInstance;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.propertysheet.ObjectCellEditor;

/**
 * Provides property sheet editor functions for char types.
 * 
 * @author Rolf Kalim
 */
public class CharJavaCellEditor extends ObjectCellEditor implements INeedData, IJBCFCellEditor {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$

	protected JavaDataType fCharType;
	protected ResourceSet fResourceSet;
	protected EditDomain fEditDomain;

	public CharJavaCellEditor(Composite parent) {
		super(parent);
	}

	/**
	 * Returns the object that the string represents.
	 * This is called when editing and a string from
	 * the text editor is sent in and we need to send
	 * the object that it represents up to the validators.
	 * The string to convert will be passed in.
	 */
	public Object doGetObject(String aString) {

		// The first character of the string, to be used as the character
		char charEntered = aString.charAt(0); //$NON-NLS-1$

		// Create a char so the user gets to see it as a user-visible string in their source
		return BeanUtilities.createJavaObject(fCharType, fResourceSet, "'" + new Character(charEntered).toString() + "'"); //$NON-NLS-1$ //$NON-NLS-2$

	}

	/**
	 * The object is being passed in,
	 * return the string to be used in the editor.
	 *
	 * It returns an empty string if the value can't be
	 * converted to a string.
	 */
	public String doGetString(Object anObject) {
		if (anObject == null) {
			return ""; //$NON-NLS-1$
		} else {
			return MnemonicLabelProvider.getCharacterLabel((IJavaDataTypeInstance) anObject, fEditDomain);
		}
	}

	/**
	 * Verifies the passed in string represents a good object.
	 * 
	 * This is usually from the text editor itself. Though it could be
	 * coming directly in from the setting, but we don't know this.
	 */
	public String isCorrectString(String aString) {

		// Check the string for invalid format
		if (aString.length() == 0) {
			return JBCFMessages.getString("CellEditor.CharJava.ErrMsg_ERROR_"); //$NON-NLS-1$
		} else if (aString == null || aString.length() != 1) {
			return JBCFMessages.getString("CellEditor.CharJava.InvalidMsg_ERROR_"); //$NON-NLS-1$
		}

		return null;
	}

	/**
	 * Implement this method to verify if this is a good
	 * object. This would be coming from a direct setting.
	 */
	public String isCorrectObject(Object anObject) {
		return null;
	}

	/**
	 * Set the data. (Used to perform validations?)
	 */
	public void setData(Object data) {
		fEditDomain = (EditDomain) data;
		fResourceSet = JBCFEditDomainHelper.getResourceSet(fEditDomain);
		fCharType = (JavaDataType) Utilities.getJavaClass("char", fResourceSet); //$NON-NLS-1$
	}

	/**
	 * The IJBCFCellEditor interface
	 */
	public String getJavaInitializationString() {
		if (getValue() == null) {
			return null;
		} else {
			// Make a label for a char, in which case the label has quotes, e.g. 'S' or 't'
			String label = MnemonicLabelProvider.getCharacterLabel((IJavaDataTypeInstance) getValue(), fEditDomain);
			return "'" + label + "'"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
}