package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.ContentsGraphicalEditPart;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.gef.*;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.proxy.ProxyFactoryRegistry;
import com.ibm.etools.cde.VisualInfoXYLayoutEditPolicy;
import java.util.*;
import com.ibm.etools.emf.jbcf.BeanComposition;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.emf.ref.RefObject;
/**
 * Composition Graphical Edit Part that instantiates and disposes bean proxies
 */
public class CompositionComponentsGraphicalEditPart extends ContentsGraphicalEditPart implements IEMFListener {


	public CompositionComponentsGraphicalEditPart(Object model) {
		setModel(model);
	}

	protected void createEditPolicies() {
		VisualInfoXYLayoutEditPolicy ep = new VisualInfoXYLayoutEditPolicy(new CompositionContainerPolicy(EditDomain.getEditDomain(this)));
		ep.setZoomable(true);
		installEditPolicy(EditPolicy.LAYOUT_ROLE, ep);
	}
	
	protected List getModelChildren() {
		BeanComposition comp = (BeanComposition) getModel();
		return comp != null ? comp.getComponents() : Collections.EMPTY_LIST;
	}
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter compositionAdapter;
	
	public void activate() {
		super.activate();
		if (getModel() != null)
			((BeanComposition) getModel()).addAdapter(getModelAdapter());
	}
	
	public void deactivate() {
		super.deactivate();
		if (getModel() != null)
			((BeanComposition) getModel()).removeAdapter(getModelAdapter());
	}
	
	protected Adapter getModelAdapter() {
		if (compositionAdapter == null) {
			// Create an adaptor that is used to listen to changes from the
			// MOF model object.  The MOF notifications are used by this edit part to stay updated.
			sfComponents = ((BeanComposition) getModel()).ePackageJBCF().getBeanComposition_Components();
			compositionAdapter = new EMFListenerAdapter(this, true);
		}
		return compositionAdapter;
	}
	
	private RefStructuralFeature sfComponents;
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sfComponents)
			refreshChildren();
	}
	
	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		return CDEUtilities.getEditPartFactory(this).createEditPart(model);
	}
}
