package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notification;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.emf.notify.impl.NotifierImpl;
import com.ibm.etools.emf.jbcf.BeanComposition;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import java.util.*;
/**
 * This is a special adaptor for BeanCompositions.
 * There really isn't a bean proxy, but we need to handle
 * the disposition of beanproxies. This will be where this is done.
 *
 * It is also a notifier. What will happen is it will renotify all
 * events before processing the event. This is useful because
 * other adapters that need to do something before this one cleans
 * up the proxies.
 *
 */
public class CompositionProxyAdapter extends NotifierImpl implements Adapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final Class BEAN_COMPOSITION_PROXY = CompositionProxyAdapter.class;
	
	Notifier target;
	
public Notifier getTarget() {
	return target;
}	

public void setTarget(Notifier target) {
	this.target = target;
}

public boolean isAdapterForType(Object type) {
	return BEAN_COMPOSITION_PROXY.equals(type);
}

public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
}

public void notifyChanged(Notification msg) {
	if (msg.getEventType()== Notifier.REMOVING_ADAPTER) {
		// Remove all of the adapters on this guy.
		removeAllAdapters();
	} else
		notify(new NotificationImpl(this, msg.getEventType(), msg.getStructuralFeature(), msg.getOldValue(), msg.getNewValue(), msg.getPosition()));	// Notify listeners of the change before we do anything with it.
	RefObject sf = msg.getStructuralFeature();
	if (sf == JBCFGraphicalEditorPart.SF_RELEASE_PROXIES) {
		releaseBeanProxy();
	} else if (sf == JBCFGraphicalEditorPart.SF_INSTANTIATE_PROXIES) {
		initBeanProxy();
	} else {
		// Now process it.
		switch (msg.getEventType()) {
			case Notifier.REMOVING_ADAPTER:
				releaseBeanProxy();
				break;
			case Notifier.ADD:
			case Notifier.SET:
				releaseSetting(msg.getOldValue());
				initSetting(msg.getNewValue());
				break;
			case Notifier.REMOVE:
			case Notifier.UNSET:
				releaseSetting(msg.getOldValue());
				break;
			case Notifier.ADD_MANY:
				Iterator itr = ((List) msg.getNewValue()).iterator();
				while (itr.hasNext())
					initSetting(itr.next());
				break;
			case Notifier.REMOVE_MANY:
				itr = ((List) msg.getOldValue()).iterator();
				while (itr.hasNext())
					releaseSetting(itr.next());
				break;
		}		
	}
}

public void initBeanProxy() {

	Iterator settings = ((InternalProxy)target).refBasicContains().iterator();	// Get only the attrs and composite refs.
	while (settings.hasNext()) {;
		initSetting(settings.next());
	}
}

/**
 * When we are explicitly disposed dispose all bean proxies on all values
 * of our composition.
 */
public void releaseBeanProxy() {

	// It will go through the attribute settings and dispose
	// of them too since they were instantiated by this object.
	Iterator settings = ((InternalProxy)target).refBasicContains().iterator();	// Get only the attrs and composite refs.
	while (settings.hasNext()) {
		releaseSetting(settings.next());
	}
}

protected void initSetting(Object v) {
	if (v instanceof IJavaInstance) {	
		IBeanProxyHost value = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) v);
		if (value != null && value.getBeanProxyDomain().getProxyFactoryRegistry().isValid()) {
			value.instantiateBeanProxy();
		}
	}
}

protected void releaseSetting(Object v) {
	if (v instanceof IJavaInstance) {	
		IBeanProxyHost value = (IBeanProxyHost) ((Notifier) v).getExistingAdapter(IBeanProxyHost.BEAN_PROXY_TYPE);
		if (value != null) {
			value.releaseBeanProxy();	// Dispose of a bean proxy automatically takes care of dispose any of the children of the proxy.
		}
	}
}



/**
 * About to be thrown away. Run disposeBeanProxy first to
 * make sure that any bean cleanup required is done.
 */
protected void finalize() throws Throwable {
	if (getTarget() != null)
		notifyChanged(new NotificationImpl(getTarget(), Notifier.REMOVING_ADAPTER, null, this, null, Notifier.NO_INDEX));
}
/**
 * isAdaptorForType method comment.
 */
public boolean isAdaptorForType(Object type) {
	return BEAN_COMPOSITION_PROXY.equals(type);
}
}
