package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import org.eclipse.ui.IEditorPart;

import com.ibm.etools.beaninfo.BeanDecorator;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.commands.CompoundCommand;
import com.ibm.etools.gef.ui.actions.SelectionAction;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.proxy.*;

public class CustomizeAction extends SelectionAction {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String ACTION_ID = "jbcf.CUSTOMIZE"; //$NON-NLS-1$

	public CustomizeAction(IEditorPart anEditorPart) {
		super(anEditorPart);
		setId(ACTION_ID);
		setText(JBCFMessages.getString("Action.Customize.Text")); //$NON-NLS-1$
	}
	/**
	 * @see EditorPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		List editParts = getSelectedObjects();
		JavaClass customizerClass = getCustomizerClass(editParts);
		return customizerClass != null;
	}
	/**
	 * Return the customizer class for the selected object.  Null if there is not a 
	 * customizer or more than one object is selected
	 */
	protected JavaClass getCustomizerClass(List editParts) {

		if (editParts.size() != 1)
			return null; // Only customizer a single bean at a time
		if (!(editParts.get(0) instanceof EditPart))
			return null; // Only work with an EditPart
		Object model = ((EditPart) editParts.get(0)).getModel();
		if (model instanceof IJavaObjectInstance) {
			CDEHack.fixMe("This is quick hack to not use thisPart for customization. Need to get a better way of doing this."); //$NON-NLS-1$
			IBeanProxyHost ba = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) model);
			if (ba instanceof BeanProxyAdapter && ((BeanProxyAdapter) ba).isThisPart())
				return null;
			JavaClass beanClass = (JavaClass) ((RefObject) model).refMetaObject();
			BeanDecorator beanDecor = Utilities.getBeanDecorator(beanClass);
			return beanDecor != null ? beanDecor.getCustomizerClass() : null;	// If class invalid, bean decor will be null.
		}
		return null;
	}
	protected String getCustomizerClassName() {
		JavaClass customizerClass = getCustomizerClass(getSelectedObjects());
		if (customizerClass != null) {
			return customizerClass.getQualifiedNameForReflection();
		}
		return null;
	}
	
	public void run() {
		try {
			EditPart editPart = (EditPart) getSelectedObjects().get(0);
			ProxyFactoryRegistry registry = BeanProxyUtilities.getProxyFactoryRegistry(editPart);
			IBeanTypeProxy customizerTypeProxy =
				registry.getBeanTypeProxyFactory().getBeanTypeProxy(getCustomizerClassName());

			IBeanProxy customizerProxy = customizerTypeProxy.newInstance();
			IBeanProxy beanProxy = BeanProxyUtilities.getBeanProxy((IJavaObjectInstance) editPart.getModel());

			// (1) Add the bean to the customizer
			// (2) Store the initial xml string, for undo.
			// (3) Launch the remote WindowLauncher
			// (1)
			IMethodProxy setObjectMethod =
				customizerProxy.getTypeProxy().getMethodProxy("setObject", new String[] { "java.lang.Object" }); //$NON-NLS-1$ //$NON-NLS-2$
			setObjectMethod.invoke(customizerProxy, new IBeanProxy[] { beanProxy });
			// (2)
			IBeanTypeProxy helperTypeProxy =
				registry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.remotevm.XMLHelper"); //$NON-NLS-1$
			IMethodProxy launcherMethodProxy =
				helperTypeProxy.getMethodProxy("getEncodedString", new String[] { "java.lang.Object" }); //$NON-NLS-1$ //$NON-NLS-2$
			IStringBeanProxy originalXMLRepresentaqtion =
				(IStringBeanProxy) launcherMethodProxy.invoke(helperTypeProxy, new IBeanProxy[] { beanProxy });
			// (3)
			IBeanTypeProxy windowLauncherType =
				registry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.remotevm.WindowLauncher"); //$NON-NLS-1$
			IConstructorProxy ctor =
				windowLauncherType.getConstructorProxy(new String[] { "java.awt.Component" }); //$NON-NLS-1$
			IBeanProxy windowLauncherProxy = ctor.newInstance(new IBeanProxy[] { customizerProxy });
			//

			IStringBeanProxy xmlOfCustomized = null;

			WindowLauncher launcher =
				new WindowLauncher(
					windowLauncherProxy,
					org.eclipse.swt.widgets.Display.getDefault().getActiveShell());
			launcher.waitUntilWindowCloses();
			switch (launcher.getWindowState()) {
				case Common.WIN_CLOSED :
				case Common.DLG_CANCEL :
					// Cancel the change.
					xmlOfCustomized = null;
					break;
				case Common.DLG_OK :
					// Make the change commands.
					launcherMethodProxy =
						helperTypeProxy.getMethodProxy("getEncodedString", new String[] { "java.lang.Object" }); //$NON-NLS-1$ //$NON-NLS-2$
					xmlOfCustomized =
						(IStringBeanProxy) launcherMethodProxy.invoke(helperTypeProxy, new IBeanProxy[] { beanProxy });
					break;
				default :
					xmlOfCustomized = null;
					break;
			};
			windowLauncherProxy = null;
			launcher = null;

			if (xmlOfCustomized == null)
				return;

			IJavaObjectInstance objectModified = (IJavaObjectInstance) editPart.getModel();
			CompoundCommand finalCommand =
				new CompoundCommand();
			if (objectModified.isSetInitializationString()) {
				ModifyAttributesCommand removeInitializationStringCommand =
					new ModifyAttributesCommand(""); //$NON-NLS-1$
				removeInitializationStringCommand.setFeature(
					JavaInstantiation.getInitializationStringFeature(objectModified));
				removeInitializationStringCommand.setOperation(removeInitializationStringCommand.FEATURE_REMOVE);
				removeInitializationStringCommand.setTarget(objectModified);
				finalCommand.add(removeInitializationStringCommand);
			}
			if (objectModified.isSetInstantiateUsing()) {
				ModifyAttributesCommand removeInstantiationStringCommand =
					new ModifyAttributesCommand(""); //$NON-NLS-1$
				removeInstantiationStringCommand.setFeature(
					JavaInstantiation.getInstantiateUsingFeature(objectModified));
				removeInstantiationStringCommand.setOperation(removeInstantiationStringCommand.FEATURE_REMOVE);
				removeInstantiationStringCommand.setTarget(objectModified);
				finalCommand.add(removeInstantiationStringCommand);
			}
			finalCommand.unwrap();

			ModifyAttributesCommand applySerializeCommand =
				new ModifyAttributesCommand(""); //$NON-NLS-1$
			applySerializeCommand.setData(xmlOfCustomized.stringValue());
			applySerializeCommand.setFeature(JavaInstantiation.getSerializeDataFeature(objectModified));
			applySerializeCommand.setTarget(objectModified);
			if (objectModified.isSetSerializeData()) // Modify Attribute
				applySerializeCommand.setOperation(applySerializeCommand.FEATURE_MODIFY);
			else // Add attribute;
				applySerializeCommand.setOperation(applySerializeCommand.FEATURE_ADD);

			if (finalCommand.isEmpty()) {
				execute(applySerializeCommand);
			} else {
				finalCommand.add(applySerializeCommand);
				execute(finalCommand);
			}
		} catch (Exception e) {
			JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
		}

	}

}