package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.commands.CancelAttributeSettingCommand;
import com.ibm.etools.cde.properties.AbstractPropertyDescriptorAdapter;
import com.ibm.etools.common.command.CompoundCommand;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.*;
import com.ibm.etools.beaninfo.adapters.*;
import org.eclipse.ui.views.properties.*;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;
import org.eclipse.ui.*;
import com.ibm.etools.java.*;
import com.ibm.etools.propertysheet.*;
import com.ibm.etools.gef.*;
import com.ibm.etools.proxy.*;
import com.ibm.etools.emf.ecore.EModelElement;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.beaninfo.*;

public class CustomizeJavaBeanAction extends CustomizeAction {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain fEditDomain;
	static final int NONE = 0;
	static final int NEWVALUE = 2;
	static final int CHANGED = 3;
	static final int CANCELED = 4;
	public static final String ACTION_ID = "jbcf.javabean.CUSTOMIZE"; //$NON-NLS-1$

public CustomizeJavaBeanAction(IEditorPart anEditorPart,EditDomain anEditDomain){
	super(anEditorPart);
	fEditDomain = anEditDomain;
	setId(ACTION_ID);
	setText(JBCFMessages.getString("Action.CustomizeJavaBean.Text")); //$NON-NLS-1$
	setToolTipText(JBCFMessages.getString("Action.CustomizeJavaBean.ToolTipText")); //$NON-NLS-1$
	try {
		URL customizeEnableImageURL = new URL("platform:/plugin/com.ibm.etools.jbcf/icons/full/etool16/customizebean_co.gif"); //$NON-NLS-1$
		setImageDescriptor(ImageDescriptor.createFromURL(customizeEnableImageURL));

		URL customizeDisableImageURL = new URL("platform:/plugin/com.ibm.etools.jbcf/icons/full/dtool16/customizebean_co.gif"); //$NON-NLS-1$
		setDisabledImageDescriptor(ImageDescriptor.createFromURL(customizeDisableImageURL));		
		
		URL customizeColorImageURL = new URL("platform:/plugin/com.ibm.etools.jbcf/icons/full/ctool16/customizebean_co.gif"); //$NON-NLS-1$
		setHoverImageDescriptor(ImageDescriptor.createFromURL(customizeColorImageURL));
		
	} catch ( MalformedURLException exc ){
	}
}
public void run(){
	try{
		EditPart editPart = (EditPart) getSelectedObjects().get(0);
		ProxyFactoryRegistry registry = BeanProxyUtilities.getProxyFactoryRegistry(editPart);
		IBeanTypeProxy customizerTypeProxy = registry.getBeanTypeProxyFactory().getBeanTypeProxy(getCustomizerClassName());
		
		IJavaObjectInstance bean = (IJavaObjectInstance)editPart.getModel();
		IBeanProxy customizerProxy = customizerTypeProxy.newInstance();
		final IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost(bean);
		IBeanProxy beanProxy = beanProxyHost.getBeanProxy();
		
		// (1) Add the bean to the customizer
		// (2) Store the initial property values, for undo.
		// (3) Launch the remote WindowLauncher
		// (4) Query the current properties against the stored ones to see what changed
		//	 and use the property editors for these to generate initialization strings and mof settings
		//     so we can code generate from them
		// (1)
		IMethodProxy setObjectMethod = customizerProxy.getTypeProxy().getMethodProxy("setObject", new String[]{"java.lang.Object"}); //$NON-NLS-1$ //$NON-NLS-2$
		setObjectMethod.invoke(customizerProxy, new IBeanProxy[]{beanProxy});
		// (2)
		// The only properties we are about are those that have get and set method pairs
		// Get the existing beanProxy object and put it in a map keyed by property descriptor
		Iterator propertyDecorators = Utilities.getPropertiesIterator(bean.refMetaObject().refAttributes());
		Map oldValues = new HashMap();
		while(propertyDecorators.hasNext()){
			PropertyDecorator propDecor = (PropertyDecorator)propertyDecorators.next();
			RefStructuralFeature sf = (RefStructuralFeature) propDecor.getEDecorates();
			if(sf.refIsChangeable()){
				// Query the bean object
				IBeanProxy existingValue = beanProxyHost.getBeanPropertyProxyValue(sf);
				oldValues.put(sf,existingValue);
			}
		}
		// (3)
		IBeanTypeProxy windowLauncherType = registry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.remotevm.WindowLauncher"); //$NON-NLS-1$
		IConstructorProxy ctor = windowLauncherType.getConstructorProxy(new String[] {"java.awt.Component"}); //$NON-NLS-1$
		IBeanProxy windowLauncherProxy = ctor.newInstance(new IBeanProxy[] {customizerProxy});
		//

		WindowLauncher launcher = new WindowLauncher(windowLauncherProxy, org.eclipse.swt.widgets.Display.getDefault().getActiveShell());
		launcher.addListener(new WindowLauncher.Listener(){
			public void propertyChange(String eventName){
				// To revalidate the bean being customized we need to get its BeanProxyAdapter
				// and revalidate this - this will cause a repaint on the VCE viewer if required
				beanProxyHost.revalidateBeanProxy();
			}
		});
		
		launcher.waitUntilWindowCloses();
		switch( launcher.getWindowState() ){
			case Common.WIN_CLOSED:
			case Common.DLG_CANCEL:
				// Cancel the change.
				customizerCancel(oldValues,beanProxyHost,bean);				
				;
				break;
			case Common.DLG_OK:
				// Make the change commands.
				customizerOK(oldValues,beanProxyHost,bean);
				break;
			default:
				break;
		};
		launcher.dispose();	// Deregister the callback and ensure cleanup occurs
		windowLauncherProxy = null;
		launcher = null;

	} catch ( Exception exc ){
		JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
	};
}
protected void customizerCancel(Map oldValues, IBeanProxyHost aBeanProxyHost, IJavaObjectInstance aBean){
	// When the customizer is cancelled the model will be fine, the only problem is that the customizer may have changed the target VM Java bean
	// We therefore need to iterate over all the values and see which are different
	Iterator keys = oldValues.keySet().iterator();
	
	while(keys.hasNext()){
		RefStructuralFeature sf = (RefStructuralFeature) keys.next();
		IBeanProxy oldValue = (IBeanProxy)oldValues.get(sf);
		IBeanProxy currentValue = aBeanProxyHost.getBeanPropertyProxyValue(sf);
		int changeType = NONE;
		if(oldValue == currentValue){
			changeType = NONE;
		} else if (oldValue == null || !oldValue.equals(currentValue)){
			changeType = CHANGED;
		}
		// If the value has been changed or is a new value then apply the old value directly to the bean.  Commands are not used
		// ( which they are for OK ) because we are just restoring the target VM to the model's state to undo the changes the customizer made
		// We could just apply all of the old values from the oldValues but it is cleaner code to just apply the ones
		// that we detect have changed because their .equals doesn't match
		switch(changeType){
			case CHANGED:
				aBeanProxyHost.applyBeanPropertyProxyValue(sf, oldValue);
		}
	}
	
	// Having directly called set methods on the target VM to reset the old values for the ones that changed
	// we now need to revalidate the bean so it can refresh its visual appearance ( if applicable )
	aBeanProxyHost.revalidateBeanProxy();
	
}
protected void customizerOK(Map oldValues,IBeanProxyHost aBeanProxyHost,IJavaObjectInstance aBean){
	// The bean has been modified by the customizer
	// and the user canceled the customizer.  We must restore the bean to its old values
	Iterator keys = oldValues.keySet().iterator();
	CompoundCommand cmd = new CompoundCommand();
	
	org.eclipse.swt.widgets.Shell shell = new org.eclipse.swt.widgets.Shell();	// We need a shell for the cell editors we will create to get the init strings.	
	IBeanProxy beanProxy = aBeanProxyHost.getBeanProxy();
	Map originalSettings = aBeanProxyHost.getOriginalSettingsTable();
	
	while(keys.hasNext()){
		RefStructuralFeature sf = (RefStructuralFeature) keys.next();
		IBeanProxy oldValue = (IBeanProxy)oldValues.get(sf);
		IBeanProxy currentValue = aBeanProxyHost.getBeanPropertyProxyValue(sf);
		int changeType = NONE;
		if(oldValue == currentValue){
			changeType = NONE;
		} else if (oldValue == null || !oldValue.equals(currentValue)){
			if (aBean.refIsSet(sf) && originalSettings.containsKey(sf)) {
				// It is set in EMF and we know what the original setting value was.
				IBeanProxy origValue = (IBeanProxy) originalSettings.get(sf);
				if (origValue == currentValue)
					changeType = CANCELED;
				else if (origValue == null || !origValue.equals(currentValue))
					changeType = CHANGED;
				else
					changeType = CANCELED;
			} else {
				changeType = CHANGED;	// Not currently set in EMF, or it is set, but we don't have an orig setting to verify against, so it is changed.
			}
		}

		switch(changeType){
			case CHANGED:
				// (1) Create a new java bean that wrappers either the new or original beanProxy based on the isNew argument
				// (2) Get the javaInitializationString for this beanProxy that involves going through the 
				//     cellEditor that is created in a non-visual manner
				// (3) create a command to apply this - This will actaully be picked up by the BeanProxyAdapter 
				//     and re-applied to the live bean but so be it - I don't want a hack that somehow stops the 
				//     BeanProxyAdapter from applying the change
				IJavaInstance newBean = BeanProxyUtilities.wrapperBeanProxy(currentValue,aBean.getResourceSet(),true); // (1)
				PropertyDecorator propDecor = Utilities.getPropertyDecorator((EModelElement) sf);
				IPropertyDescriptor propertySheetDescriptor = (IPropertyDescriptor)propDecor.getEDecorates().getAdapter(AbstractPropertyDescriptorAdapter.IPROPERTYDESCRIPTOR_TYPE);
				CellEditor cellEditor = propertySheetDescriptor.createPropertyEditor(shell);
				if ( cellEditor instanceof INeedData ) {
					((INeedData)cellEditor).setData(fEditDomain);
					// We must call doSetValue(...) with the new bean
					cellEditor.setValue(newBean);
					String initString = ((IJBCFCellEditor)cellEditor).getJavaInitializationString();					
					// If our bean is an IJavaObjectInstance we can set the string in the bean we just created
					if (newBean instanceof IJavaObjectInstance){
						((IJavaObjectInstance)newBean).setInitializationString(initString);
					} else {
						// The bean is primitive.  The init string can only be given on creation
						newBean = BeanProxyUtilities.wrapperBeanProxyAsPrimitiveType(
							currentValue,
							(JavaDataType)propDecor.getPropertyType(),							
							aBean.getResourceSet(),
							initString
						);
					};
				}
				// Now we have a newBean to set
				// If the oldBean exists and there is an explicit setting
				// then undo is straightforward - it just does an undo of the old value
				// however if there is no oldBean refValue then we need to tell the command
				// the oldBeanProxy so it can apply this when it does an undo.
				// This is all done in the ApplyCustomizedValueCommand
				
				ApplyCustomizedValueCommand applyCmd = new ApplyCustomizedValueCommand(); // (3)
				applyCmd.setTarget(aBean);
				applyCmd.setValue(newBean);
				applyCmd.setFeature(sf);
				applyCmd.setOldBeanProxy(oldValue);
				cmd.append(applyCmd);
				break;
			case CANCELED:
				CancelAttributeSettingCommand cancelCmd = new CancelAttributeSettingCommand();
				cancelCmd.setTarget(aBean);
				cancelCmd.setAttribute(sf);
				cmd.append(cancelCmd);
		}	
	}
	
	shell.dispose();	
		
	// Push the command on the stack to be executed
	execute(cmd);
}
}

