package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.ui.*;
import com.ibm.etools.gef.*;
import com.ibm.etools.proxy.*;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.common.command.CompoundCommand;

public class CustomizeXMLPersistenceAction extends CustomizeAction {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String ACTION_ID = "jbcf.CUSTOMIZE"; //$NON-NLS-1$

public CustomizeXMLPersistenceAction(IEditorPart anEditorPart){
	super(anEditorPart);
	setId(ACTION_ID);
	setText(JBCFMessages.getString("Action.CustomizeXMLPersistence.Text")); //$NON-NLS-1$
}
public void run(){
	try{
		EditPart editPart = (EditPart) getSelectedObjects().get(0);
		ProxyFactoryRegistry registry = BeanProxyUtilities.getProxyFactoryRegistry(editPart);
		IBeanTypeProxy customizerTypeProxy = registry.getBeanTypeProxyFactory().getBeanTypeProxy(getCustomizerClassName());
		
		IBeanProxy customizerProxy = customizerTypeProxy.newInstance();
		final IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance)editPart.getModel());
		IBeanProxy beanProxy = beanProxyHost.getBeanProxy();
		
		// (1) Add the bean to the customizer
		// (2) Store the initial xml string, for undo.
		// (3) Launch the remote WindowLauncher
		// (1)
		IMethodProxy setObjectMethod = customizerProxy.getTypeProxy().getMethodProxy("setObject", new String[]{"java.lang.Object"}); //$NON-NLS-1$ //$NON-NLS-2$
		setObjectMethod.invoke(customizerProxy, new IBeanProxy[]{beanProxy});
		// (2)
		IBeanTypeProxy helperTypeProxy = registry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.remotevm.XMLHelper"); //$NON-NLS-1$
		IMethodProxy launcherMethodProxy = helperTypeProxy.getMethodProxy("getEncodedString", new String[] {"java.lang.Object"}); //$NON-NLS-1$ //$NON-NLS-2$
		IStringBeanProxy originalXMLRepresentaqtion = (IStringBeanProxy) launcherMethodProxy.invoke(helperTypeProxy, new IBeanProxy[]{beanProxy});
		// (3)
		IBeanTypeProxy windowLauncherType = registry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.remotevm.WindowLauncher"); //$NON-NLS-1$
		IConstructorProxy ctor = windowLauncherType.getConstructorProxy(new String[] {"java.awt.Component"}); //$NON-NLS-1$
		IBeanProxy windowLauncherProxy = ctor.newInstance(new IBeanProxy[] {customizerProxy});
		//
				
		IStringBeanProxy xmlOfCustomized = null;
				
		WindowLauncher launcher = new WindowLauncher(windowLauncherProxy, org.eclipse.swt.widgets.Display.getDefault().getActiveShell());
		launcher.addListener(new WindowLauncher.Listener(){
			public void propertyChange(String eventName){
				// To revalidate the bean being customized we need to get its BeanProxyAdapter
				// and revalidate this - this will cause a repaint on the VCE viewer if required
				beanProxyHost.revalidateBeanProxy();
			}
		});
		
		
		launcher.waitUntilWindowCloses();
		switch( launcher.getWindowState() ){
			case Common.WIN_CLOSED:
			case Common.DLG_CANCEL:
				// Cancel the change.
				xmlOfCustomized = null;
				break;
			case Common.DLG_OK:
				// Make the change commands.
				launcherMethodProxy = helperTypeProxy.getMethodProxy("getEncodedString", new String[] {"java.lang.Object"}); //$NON-NLS-1$ //$NON-NLS-2$
				xmlOfCustomized = (IStringBeanProxy) launcherMethodProxy.invoke(helperTypeProxy, new IBeanProxy[]{beanProxy});
				break;
			default:
				xmlOfCustomized = null;
				break;
		};
		launcher.dispose();
		windowLauncherProxy = null;
		launcher = null;
		
		if(xmlOfCustomized==null)
			return;
		
		IJavaObjectInstance objectModified = (IJavaObjectInstance)editPart.getModel();
		CompoundCommand finalCommand = new CompoundCommand();
		if(objectModified.isSetInitializationString()){
			ModifyAttributesCommand removeInitializationStringCommand = new ModifyAttributesCommand(""); //$NON-NLS-1$
			removeInitializationStringCommand.setFeature(JavaInstantiation.getInitializationStringFeature(objectModified));
			removeInitializationStringCommand.setOperation(removeInitializationStringCommand.FEATURE_REMOVE);
			removeInitializationStringCommand.setTarget(objectModified);
			finalCommand.append(removeInitializationStringCommand);
		}
		if(objectModified.isSetInstantiateUsing()){
			ModifyAttributesCommand removeInstantiationStringCommand = new ModifyAttributesCommand(""); //$NON-NLS-1$
			removeInstantiationStringCommand.setFeature(JavaInstantiation.getInstantiateUsingFeature(objectModified));
			removeInstantiationStringCommand.setOperation(removeInstantiationStringCommand.FEATURE_REMOVE);
			removeInstantiationStringCommand.setTarget(objectModified);
			finalCommand.append(removeInstantiationStringCommand);
		}
		finalCommand.unwrap();

		ModifyAttributesCommand applySerializeCommand = new ModifyAttributesCommand(""); //$NON-NLS-1$
		applySerializeCommand.setData(xmlOfCustomized.stringValue());
		applySerializeCommand.setFeature(JavaInstantiation.getSerializeDataFeature(objectModified));
		applySerializeCommand.setTarget(objectModified);
		if(objectModified.isSetSerializeData()) 	// Modify Attribute
			applySerializeCommand.setOperation(applySerializeCommand.FEATURE_MODIFY);
		else							// Add attribute;
			applySerializeCommand.setOperation(applySerializeCommand.FEATURE_ADD);
		
		if(finalCommand.isEmpty()){
			execute(applySerializeCommand);
		}else{
			finalCommand.append(applySerializeCommand);
			execute(finalCommand);
		}
	}catch(Exception e){JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);}
		
}


}