package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.text.MessageFormat;

import org.eclipse.core.runtime.*;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.properties.DefaultLabelProviderWithName;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.proxy.IStringBeanProxy;
/**
 * This is label provider that decorates the default label with the
 * value of an attribute. The name of the attribute is passed in either
 * as a setter or as configuration data. The attribute must result in a string
 * or a an instance of java.lang.String (in Java Model, not JRE).
 */
public class DefaultLabelProviderWithNameAndAttribute extends DefaultLabelProviderWithName implements IExecutableExtension {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected String attributeName = ""; //$NON-NLS-1$
	
	public DefaultLabelProviderWithNameAndAttribute(String attributeName) {
		this.attributeName = attributeName;
	}
	
	public DefaultLabelProviderWithNameAndAttribute() {
	}
	
	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}

	/**
	 * @see IExecutableExtension#setInitializationData(IConfigurationElement, String, Object)
	 * 
	 * It is expected that the data be a string, and it is the attribute name.
	 */
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
		if (data instanceof String)
			attributeName = (String) data; 
	}

	/**
	 * @see ILabelProvider#getText(Object)
	 */
	public String getText(Object element) {
		if (element instanceof RefObject) {
			String mainPart = super.getText(element);
			RefObject ref = (RefObject) element;
			RefObject meta = ref.refMetaObject();
			if (meta instanceof EClass) {
				EStructuralFeature sf = (EStructuralFeature) ((EClass) meta).metaObject(attributeName);
				if (sf != null) {
					Object setting = ref.refValue(sf);
					if (setting instanceof IJavaInstance)
						setting = BeanProxyUtilities.getBeanProxy((IJavaInstance) setting, JBCFEditDomainHelper.getResourceSet(domain));
					if (setting instanceof IStringBeanProxy)
						setting = ((IStringBeanProxy) setting).stringValue();
					if (setting instanceof String) {
						String label = (String) setting;
						if (label.length() > 20)
							label = MessageFormat.format(JBCFMessages.getString("DefaultLabelProvider.Label.DottedVersion"), new Object[] {label.substring(0, 20)});	// Too large, so truncate it. //$NON-NLS-1$
						return MessageFormat.format(JBCFMessages.getString("DefaultLabelProvider.Label.FullVersion"), new Object[] {mainPart, label}); //$NON-NLS-1$
					}
				}
			}
			return mainPart;
		} else
			return super.getText(element);	
	}

	/**
	 * @see IBaseLabelProvider#isLabelProperty(Object, String)
	 */
	public boolean isLabelProperty(Object element, String property) {
		return attributeName.equals(property) || super.isLabelProperty(element, property);
	}

}