package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.propertysheet.*;
import com.ibm.etools.emf.resource.*;
import org.eclipse.swt.widgets.Composite;
import com.ibm.etools.proxy.*;
import com.ibm.etools.java.*;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.EditDomain;

public class EnumeratedCellEditor extends ObjectComboBoxCellEditor implements INeedData {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected String[] fDisplayNames;
	protected IBeanProxy[] fBeanProxies;
	protected String[] fInitStrings;
	protected EditDomain fEditDomain;
	protected JavaHelpers fFeatureType;		// This is the type of the object we are going to create
	protected ProxyFactoryRegistry fProxyFactoryRegistry;
	protected IBeanTypeProxy fBeanTypeProxy;

public EnumeratedCellEditor(Composite aComposite, IArrayBeanProxy arrayOfValues, JavaHelpers featureType){
	super(aComposite);
	fFeatureType = featureType;
	// Iterate over the array of values, these are stored in the format
	// displayName, object, initString
	int length = arrayOfValues.getLength();
	int j=0;
	fDisplayNames = new String[length/3];
	fInitStrings = new String[length/3];
	try {
		for ( int i=0;i<length;i+=3 ) {
			fDisplayNames[j] = ((IStringBeanProxy)arrayOfValues.get(i)).stringValue();
			fInitStrings[j] = ((IStringBeanProxy)arrayOfValues.get(i+2)).stringValue();
			// NOTE - We cannot in any way use the object that came in from the values array
			// This came from the VM that did introspection that is NOT the same as the
			// one the editor is necessarily running in so we must re-create the
			// bean proxies from the init string each time
			j++;
		}
		// Set the display names as the items the user sees in the combo box
		setItems(fDisplayNames);	
	} catch ( ThrowableProxy exc ){
		JBCFPlugin.log("Unable to determine enumeration values", JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
		JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
	}
}
protected String isCorrectObject(Object value) {
	return null;
}
protected Object doGetObject(int index){

	Object bean = null;
	if ( fFeatureType.isPrimitive() ) {
		bean = BeanProxyUtilities.wrapperBeanProxyAsPrimitiveType(
			fBeanProxies[index],
			(JavaDataType)fFeatureType,
			JBCFEditDomainHelper.getResourceSet(fEditDomain),
			fInitStrings[index]
		);
	} else {		
		bean = (IJavaObjectInstance)BeanProxyUtilities.wrapperBeanProxy(
			fBeanProxies[index],
			JBCFEditDomainHelper.getResourceSet(fEditDomain),
			true);	// Since we are getting a new value, we own it. (By definition, editors are supposed to return new instances).
		if (bean != null) {
			((IJavaObjectInstance)bean).setInitializationString(fInitStrings[index]);
		}
	}
	return bean;	
	
}
protected int doGetIndex(Object anObject){
	// The argument is an IJavaInstance.  Get its bean proxy and compare it against
	// the values stored by us
	IBeanProxy argumentProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance)anObject, JBCFEditDomainHelper.getResourceSet(fEditDomain));
	for (int i=0;i<fBeanProxies.length;i++){
		if(fBeanProxies[i].equals(argumentProxy)){
			return i;
		}
	}
	return sNoSelection;
}
public void setData(Object data){
	fEditDomain = (EditDomain) data;
	// Now that we have a VM we can create the proxies from the init strings
	fBeanTypeProxy = JBCFEditDomainHelper.getBeanProxyDomain(fEditDomain).getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy(fFeatureType instanceof JavaClass ? ((JavaClass) fFeatureType).getQualifiedNameForReflection() : fFeatureType.getQualifiedName());
	fBeanProxies = new IBeanProxy[fInitStrings.length];
	int index=0;
	try {
		for(int i=0;i<fInitStrings.length;i++){
			index = i;
			fBeanProxies[i] = fBeanTypeProxy.newInstance(fInitStrings[i]);
		}
	} catch ( ThrowableProxy exc ) {
		JBCFPlugin.log("Unable to create enumeration value for " + fInitStrings[index], JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
		JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
		com.ibm.etools.cde.CDEHack.fixMe("Rather than null, probably should do this in a validation step so the property sheet entry knows to not apply the bad value"); //$NON-NLS-1$
	} catch (InstantiationException exc) {			
		JBCFPlugin.log("Unable to create enumeration value for " + fInitStrings[index], JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
		JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
		com.ibm.etools.cde.CDEHack.fixMe("Rather than null, probably should do this in a validation step so the property sheet entry knows to not apply the bad value"); //$NON-NLS-1$
	}		
}
}