package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.proxy.*;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaHelpers;

public class EnumeratedLabelProvider extends org.eclipse.jface.viewers.LabelProvider implements INeedData {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected String[] fDisplayNames;
	protected String[] fInitStrings;
	protected IBeanProxy[] fBeanProxies;
	protected JavaHelpers fFeatureType;
	protected EditDomain editDomain;

public EnumeratedLabelProvider(IArrayBeanProxy aBeanInfoValuesArray, JavaHelpers aFeatureType){
	fFeatureType = aFeatureType;
	// Iterate over the array of values, these are stored in the format
	// displayName, object, initString
	// We only care about storing the displayName and the initStrings
	int length = aBeanInfoValuesArray.getLength();
	int j=0;
	fDisplayNames = new String[length/3];
	fInitStrings = new String[length/3];
	try {
		for ( int i=0;i<length;i+=3 ) {
			fDisplayNames[j] = ((IStringBeanProxy)aBeanInfoValuesArray.get(i)).stringValue();
			fInitStrings[j] = ((IStringBeanProxy)aBeanInfoValuesArray.get(i+2)).stringValue();
			// NOTE - We cannot in any way use the object that came in from the values array
			// This came from the VM that did introspection that is NOT the same as the
			// one the editor is necessarily running in so we must re-create the
			// bean proxies from the init string each time
			j++;
		}
	} catch ( ThrowableProxy exc ){
		JBCFPlugin.log("Unable to determine enumeration values", JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
		JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
	}	
}
public String getText(Object aJavaInstance){
	// Now that we have an object we have a remote VM.  If we have not turned the init
	// strings into objects do this now
	IBeanProxy valueProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance)aJavaInstance, JBCFEditDomainHelper.getResourceSet(editDomain));
	if ( fBeanProxies == null ) {
		// Mistake on my part, didn't get getQualifiedNameForReflection() added to JavaHelpers, so only available on JavaClass.
		String featureTypeName = null;
		if (fFeatureType instanceof JavaClass)
			featureTypeName = ((JavaClass) fFeatureType).getQualifiedNameForReflection();
		else
			featureTypeName = fFeatureType.getQualifiedName();			
		IBeanTypeProxy aBeanTypeProxy = valueProxy.getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy(featureTypeName);
		fBeanProxies = new IBeanProxy[fInitStrings.length];
		int index=0;
		try {
			for(int i=0;i<fInitStrings.length;i++){
				index = i;
				fBeanProxies[i] = aBeanTypeProxy.newInstance(fInitStrings[i]);
			}
		} catch ( ThrowableProxy exc ) {
			JBCFPlugin.log("Unable to create enumeration value for " + fInitStrings[index], JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
			JBCFPlugin.log(exc,JBCFMsgLogger.LOG_WARNING);
		} catch ( InstantiationException exc ) {
			JBCFPlugin.log("Unable to create enumeration value for " + fInitStrings[index], JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
			JBCFPlugin.log(exc,JBCFMsgLogger.LOG_WARNING);
		}
	}
	// This can occur following an introspection error
	if ( fBeanProxies == null ) {
		return JBCFMessages.getString("LabelProvider.Enumerated.getText_ERROR_"); //$NON-NLS-1$
	}
	
	// Now that we have an array of bean proxies compare the value against the entries
	for( int i=0;i<fBeanProxies.length;i++){
		if(fBeanProxies[i].equals(valueProxy)){
			return fDisplayNames[i];
		}
	}	
	com.ibm.etools.cde.CDEHack.fixMe("This needs more thought"); //$NON-NLS-1$
	return JBCFMessages.getString("LabelProvider.Enumerated.getText_ERROR_"); //$NON-NLS-1$
}
	/**
	 * @see INeedData#setData(Object)
	 */
	public void setData(Object data) {
		editDomain = (EditDomain) data;
	}

}