package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.propertysheet.ObjectComboBoxCellEditor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.proxy.IIntegerBeanProxy;
import com.ibm.etools.jbcf.*;
//import java.awt.GridBagConstraints;

public class GridBagConstraintsAnchorCellEditor extends ObjectComboBoxCellEditor implements INeedData {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain fEditDomain;
	public static int[] ANCHOR_VALUES = new int [] {
		GridBagConstraint.CENTER,
		GridBagConstraint.NORTH,
		GridBagConstraint.NORTHEAST,
		GridBagConstraint.EAST,
		GridBagConstraint.SOUTHEAST,
		GridBagConstraint.SOUTH,
		GridBagConstraint.SOUTHWEST,
		GridBagConstraint.WEST,
		GridBagConstraint.NORTHWEST
	};
	
	public static String[] ANCHOR_INITSTRINGS = new String[] {
		"java.awt.GridBagConstraints.CENTER", //$NON-NLS-1$
		"java.awt.GridBagConstraints.NORTH", //$NON-NLS-1$
		"java.awt.GridBagConstraints.NORTHEAST", //$NON-NLS-1$
		"java.awt.GridBagConstraints.EAST", //$NON-NLS-1$
		"java.awt.GridBagConstraints.SOUTHEAST", //$NON-NLS-1$
		"java.awt.GridBagConstraints.SOUTH", //$NON-NLS-1$
		"java.awt.GridBagConstraints.SOUTHWEST", //$NON-NLS-1$
		"java.awt.GridBagConstraints.WEST", //$NON-NLS-1$
		"java.awt.GridBagConstraints.NORTHWEST" //$NON-NLS-1$
	};	
	
public GridBagConstraintsAnchorCellEditor(Composite aComposite){
	// Create the combo editor with the list of possible anchor values
	super(aComposite, GridBagConstraintsAnchorLabelProvider.ANCHOR_VALUES);
}
/**
 * Return a MOF class that represents the constraint bean
 */
protected Object doGetObject(int index){
	String initString = ""; //$NON-NLS-1$
	if (index < ANCHOR_INITSTRINGS.length)
		initString = ANCHOR_INITSTRINGS[index];
	else
		initString = ANCHOR_INITSTRINGS[0];

	return BeanUtilities.createJavaObject(
		"int", //$NON-NLS-1$
		JBCFEditDomainHelper.getResourceSet(fEditDomain),
		initString
		);
}
protected int doGetIndex(Object anObject){
	// The argument is an IJavaInstance.  Get its bean proxy and compare it against
	// the values stored by us
	if (anObject instanceof IJavaInstance) {
		// get the init string from the real value and set the editor value
		IIntegerBeanProxy anchorValueProxy = (IIntegerBeanProxy)BeanProxyUtilities.getBeanProxy((IJavaInstance)anObject, JBCFEditDomainHelper.getResourceSet(fEditDomain));
		// The proxy is an int.  which represents one of the GridBagConstraints anchor values.
		// Loop the array of anchor values and return the index for the one found.
		int anchorValue = anchorValueProxy.intValue();
		for (int i = 0; i < ANCHOR_VALUES.length; i++) {
			if (anchorValue == ANCHOR_VALUES[i]) {
				return i;
			}
		}
	}
	return sNoSelection;
}
public String isCorrectObject(Object anObject){
	return null;
}
public void setData(Object data){
	fEditDomain = (EditDomain) data;
}
}