package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.propertysheet.ObjectComboBoxCellEditor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.proxy.IIntegerBeanProxy;
/**
 * Cell editor for the GridBagConstraints.fill  field
 */
public class GridBagConstraintsFillCellEditor extends ObjectComboBoxCellEditor implements com.ibm.etools.propertysheet.INeedData {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain fEditDomain;
	protected Object[] fSources; 	
	public static int[] FILL_VALUES = new int [] {
		GridBagConstraint.NONE,
		GridBagConstraint.HORIZONTAL,
		GridBagConstraint.VERTICAL,
		GridBagConstraint.BOTH
	};
	public static String[] FILL_INITSTRINGS = new String [] {
		"java.awt.GridBagConstraints.NONE", //$NON-NLS-1$
		"java.awt.GridBagConstraints.HORIZONTAL", //$NON-NLS-1$
		"java.awt.GridBagConstraints.VERTICAL", //$NON-NLS-1$
		"java.awt.GridBagConstraints.BOTH" //$NON-NLS-1$
	};	
	
public GridBagConstraintsFillCellEditor(Composite aComposite){
	// Create the combo editor with the list of possible fill values
	super(aComposite, GridBagConstraintsFillLabelProvider.FILL_VALUES);
}
/**
 * Return a MOF class that represents the constraint bean
 */
protected Object doGetObject(int index){
	String initString = ""; //$NON-NLS-1$
	if (index < FILL_INITSTRINGS.length)
		initString = FILL_INITSTRINGS[index];
	else
		initString = FILL_INITSTRINGS[0];
	
	return BeanUtilities.createJavaObject(
		"int", //$NON-NLS-1$
		JBCFEditDomainHelper.getResourceSet(fEditDomain),
		initString
		);
}
protected int doGetIndex(Object anObject){
	// The argument is an IJavaInstance.  Get its bean proxy and compare its int to what we have.
	if (anObject instanceof IJavaInstance) {
		// get the init string from the real value and set the editor value
		IIntegerBeanProxy fillValueProxy = (IIntegerBeanProxy)BeanProxyUtilities.getBeanProxy((IJavaInstance)anObject, JBCFEditDomainHelper.getResourceSet(fEditDomain));
		// The proxy is an int.  which represents one of the GridBagConstrains fill values.
		// Loop the array of fill values and return the index for the one found.
		int fillValue = fillValueProxy.intValue();
		for (int i = 0; i < FILL_VALUES.length; i++) {
			if (fillValue == FILL_VALUES[i]) {
				return i;
			}
		}
	}
	return sNoSelection;
}
public String isCorrectObject(Object anObject){
	return null;
}
public void setData(Object data){
	fEditDomain = (EditDomain) data;
}
}