package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
import com.ibm.etools.jbcf.IErrorHolder.ErrorType;

/**
 * Implementers of this interface can hold errors and notify when status has changed.
 * 
 * @author richkulp
 */
public interface IErrorNotifier extends IErrorHolder {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$	
	
	public static final Class ERROR_NOTIFIER_TYPE = IErrorNotifier.class;	// The type to use to ask for the adapter of type IErrorNotifier, in those case where it is being used.


	public interface ErrorListener {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		public void errorStatus(int errorStatus);	// Status has changed
		public void errorCleared(ErrorType error);	// A particular error has been cleared.
		public void errorAdded(ErrorType error);	// A particular error has been added.
	}

	public void addErrorListener(ErrorListener aListener);

	public void removeErrorListener(ErrorListener aListener);

	/**
	 * Default implementation of ErrorListener that can be subclassed to provide
	 * specific overrides and let the others do nothing.
	 * @author richkulp
	 */
	public static class ErrorListenerAdapter implements ErrorListener {

		/**
		 * @see com.ibm.etools.jbcf.IErrorNotifier.ErrorListener#errorStatus(int)
		 */
		public void errorStatus(int errorStatus) {
		}

		/**
		 * @see com.ibm.etools.jbcf.IErrorNotifier.ErrorListener#errorCleared(Error)
		 */
		public void errorCleared(ErrorType error) {
		}

		/**
		 * @see com.ibm.etools.jbcf.IErrorNotifier.ErrorListener#errorAdded(Error)
		 */
		public void errorAdded(ErrorType error) {
		}

	}

}
