package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.ui.views.properties.IPropertySourceProvider;

import com.ibm.etools.common.command.CommandStack;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ecore.utilities.copy.EtoolsCopyUtility;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.propertysheet.CommandStackPropertySheetEntry;
import com.ibm.etools.propertysheet.IDescriptorPropertySheetEntry;

/**
 * This is used in the JBCF to handle JBCF entries appropriately.
 * @version 	1.0
 * @author
 */
public class JBCFCommandStackPropertySheetEntry extends CommandStackPropertySheetEntry {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * Constructor for JBCFCommandStackPropertySheetEntr.
	 * @param stack
	 * @param parent
	 * @param provider
	 */
	public JBCFCommandStackPropertySheetEntry(
		CommandStack stack,
		CommandStackPropertySheetEntry parent,
		IPropertySourceProvider provider) {
		super(stack, parent, provider);
	}

	/**
	 * @see AbstractPropertySheetEntry#primFillValues(Object, Object[])
	 */
	protected void primFillValues(Object newEditValue, Object[] valuesArray) {
		// If the length is only one, don't bother we duplicating checks
		if (valuesArray.length > 1 && newEditValue instanceof RefObject) {
			if (fDescriptors[0].getId() instanceof EStructuralFeature) {
				if (((EStructuralFeature) fDescriptors[0].getId()).refIsComposite()) {
					// Composite RefObject settings must be copied. Shared ones should not be copied.
					valuesArray[0] = newEditValue;
					for (int i=1; i<valuesArray.length; i++) {
						valuesArray[i] = EtoolsCopyUtility.createCopy((RefObject) newEditValue);	// Create a copy, don't copy ids.
					}
					return;
				}
			}
		}
		
		// Datatypes (i.e. IJavaDataTypeInstance) do not need to be copied, even though they are 
		// "owned" by each, it should still work since further changes to datatypes are not permitted,
		// you should always create a new datatype for changes.
		//
		// Any other types are just used.
		super.primFillValues(newEditValue, valuesArray);	// Standard fill is appropriate
	}
	
	/**
	 * Create a property sheet entry of the desired type.
	 * Use the provider passed in.
	 *
	 * It is used to create children of this current entry.
	 */
	protected IDescriptorPropertySheetEntry createPropertySheetEntry(IPropertySourceProvider provider) {
		return new JBCFCommandStackPropertySheetEntry(fStack, this, provider);
	}	
	public Object getEditValue(){
		return super.getEditValue();
	}

}
