package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.gef.*;

import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.*;
import com.ibm.etools.emf.resource.Context;
import com.ibm.etools.emf.resource.impl.WorkbenchURIConverterImpl;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource ;
import org.eclipse.ui.IFileEditorInput;
import com.ibm.etools.proxy.*;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.emf.resource.Resource ;
import com.ibm.etools.emf.ecore.EMetaObject;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefObject ;
import com.ibm.etools.emf.ref.RefPackage;
import com.ibm.etools.emf.ref.impl.RefObjectImpl;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.NullProgressMonitor;
import com.ibm.etools.beaninfo.adapters.BeaninfoNature;

import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;

import java.util.Iterator;
import java.util.List;
import java.net.URL;
import java.io.IOException;
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.launching.VMRunnerConfiguration;
import org.eclipse.jface.action.*;
import org.eclipse.ui.*;

import com.ibm.etools.gef.palette.PaletteRoot;
import com.ibm.etools.gef.ui.actions.*;
import com.ibm.etools.gef.ui.parts.TreeViewer;
import com.ibm.etools.logging.util.Level;
import com.ibm.etools.propertysheet.AbstractPropertySheetEntry;
import com.ibm.etools.propertysheet.CommandStackPropertySheetEntry;
import com.ibm.etools.cde.emf.*;
import com.ibm.etools.cde.emf.ClassDescriptorDecoratorPolicy;
import com.ibm.etools.cde.emf.DefaultGraphicalEditPartFactory;
import com.ibm.etools.cde.emf.DefaultTreeEditPartFactory;
import com.ibm.etools.cde.emf.DefaultModelAdapterFactory;
import com.ibm.etools.cde.palette.*;
import com.ibm.etools.cde.palette.PaletteCmp;
import com.ibm.etools.cde.palette.PalettePackage;
import com.ibm.etools.cde.palette.impl.PalettePackageImpl;
import com.ibm.etools.cde.CDEUtilities;

public class JBCFGraphicalEditorPart extends OCMGraphicalEditorPart {

	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ProxyFactoryRegistry fProxyFactoryRegistry;
	
	public class JBCFOutlinePage extends EMFContentOutlinePage {
		
		/**
		 * Constructor for JBCFOutlinePage.
		 * @param viewer
		 */
		public JBCFOutlinePage(EditPartViewer viewer) {
			super(viewer);
		}

		/**
		 * @see com.ibm.etools.cde.emf.EMFGraphicalEditorPart.EMFContentOutlinePage#configureOutlineViewer()
		 */
		protected void configureOutlineViewer() {
			super.configureOutlineViewer();
			CDEUtilities.setEditPartFactory(getViewer(), new DefaultTreeEditPartFactory(ClassDescriptorDecoratorPolicy.getPolicy((EditDomain) getViewer().getEditDomain())));			
		}

	}

protected void createPrimaryViewer(){
	super.createPrimaryViewer();
	CDEUtilities.setEditPartFactory(primaryViewer, new DefaultGraphicalEditPartFactory(ClassDescriptorDecoratorPolicy.getPolicy((EditDomain) primaryViewer.getEditDomain())));
}


protected IConfigurationContributor getJBCFClassPath() throws CoreException {
	
	return (IConfigurationContributor) getConfigurationElement().createExecutableExtension("classpath"); //$NON-NLS-1$

}

protected DefaultEditDomain createEditDomain(){
	EditDomain dom = (EditDomain) super.createEditDomain();
	ClassDescriptorDecoratorPolicy policy = new ClassDescriptorDecoratorPolicy();
	ClassDescriptorDecoratorPolicy.setClassDescriptorDecorator(dom, policy);
	CDEUtilities.setModelAdapterFactory(dom, new DefaultModelAdapterFactory(policy));
	return dom;
}

protected EditPart getEditPart(RefObject rootModel) {
	return new CompositionComponentsGraphicalEditPart(rootModel);
}

protected EditPart getTreeEditPart(RefObject rootModel) {
	return new CompositionComponentsTreeEditPart(rootModel);
}

protected com.ibm.etools.cdm.DiagramData createEmptyComposition() {
	com.ibm.etools.cde.CDEHack.fixMe("This should probably be changed to a parm to the configuration element so that it can be parameterized"); //$NON-NLS-1$
	return ((com.ibm.etools.emf.jbcf.JBCFPackage) com.ibm.etools.emf.ref.RefRegister.getPackage(com.ibm.etools.emf.jbcf.JBCFPackage.packageURI)).getJBCFFactory().createBeanComposition();
}



/**
 * Check to see that we are part of a java project
 */
public void init(IEditorSite site, IEditorInput input) 
	throws PartInitException 
{ 	
	if (input instanceof IFileEditorInput) {
		IProject aProject = ((IFileEditorInput)input).getFile().getProject();
		try {
			if (!BeaninfoNature.isValidProject(aProject))
				throw new PartInitException(aProject.getName() + " is not a Java Project. Please move the file " + input.getName() + " to a Java Project"); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (CoreException exc ) {
			throw new PartInitException(exc.getStatus()); 
		}
	}
	super.init(site, input);
}

protected void initializeResourceSet(ResourceSet rs){
	Context parentContext = rs.getContext().getParent();	// super initialize wipes out parent, which screws us up. Save and restore it.
	super.initializeResourceSet(rs);
	rs.getContext().setParent(parentContext);

	initModelBuilder(rs);
}

protected void initializeWithNewRoot(RefObject newModel){
	super.initializeWithNewRoot(newModel);
	if (newModel != null && newModel.getExistingAdapter(CompositionProxyAdapter.BEAN_COMPOSITION_PROXY) == null) {
		 CompositionProxyAdapter a = new CompositionProxyAdapter();
		 newModel.addAdapter(a);
		 a.setTarget(newModel);
		 a.initBeanProxy();
	}
}

/**
 * Our cell editors that need the edit domain so they can get
 * stuff like the resource set, the proxy factory registry, etc...
 */
protected AbstractPropertySheetEntry createRootPropertySheetEntry() {
	JBCFCommandStackPropertySheetEntry result = new JBCFCommandStackPropertySheetEntry(getCommandStack(), null, null);
	result.setData(getDomain());
	return result;
}

protected void initActionRegistry(){
	ActionRegistry registry = getActionRegistry();
	super.initActionRegistry();
	registry.registerAction(new CustomizeJavaBeanAction(this,(EditDomain)getDomain()));
}
public static final RefObject SF_RELEASE_PROXIES = new RefObjectImpl();	// A BIG KLUDGE AT THE MOMENT TO NOTIFY THE COMPOSITION ADAPTERS TO RELEASE ALL PROXIES.
public static final RefObject SF_INSTANTIATE_PROXIES = new RefObjectImpl();	// A BIG KLUDGE AT THE MOMENT TO NOTIFY THE COMPOSITION ADAPTERS TO INSTANTIATE ALL PROXIES.
private BeanProxyAdapterFactory beanProxyAdapterFactory;
private ProxyFactoryRegistry.IRegistryListener registryListener = new ProxyFactoryRegistry.IRegistryListener() {
	/**
	 * @see com.ibm.etools.proxy.ProxyFactoryRegistry.IRegistryListener#registryTerminated(ProxyFactoryRegistry)
	 */
	public void registryTerminated(ProxyFactoryRegistry registry) {
		final RefObject root = getRootModel();
		// Need to deactivate all of the editparts because they don't work with no vm available.
		Display.getDefault().asyncExec(new Runnable() {
			/**
			 * @see java.lang.Runnable#run()
			 */
			public void run() {
				Iterator itr = ((EditDomain) getDomain()).getViewers().iterator();
				while (itr.hasNext()) {
					EditPartViewer viewer = (EditPartViewer) itr.next();
					EditPart rootEP = viewer.getContents();
					rootEP.deactivate();
					rootEP.setModel(null);
					rootEP.refresh();
				}
				root.notify(new NotificationImpl(root, Notifier.TOUCH, SF_RELEASE_PROXIES, null, null));
				if (++recycleCntr < 3) {
					// We went down prematurely, so recycle the vm. But we haven't gone down three times in a row prematurely.
					if (isActivePartAndShell()) {
						restartVM();
					} else
						restartVMNeeded = true;
				}				
			}
		});
	}
};
private int recycleCntr = 0;	// A counter to handle pre-mature terminations. We don't want to keep recycling if it keeps going down.
private boolean restartVMNeeded = false;
private void restartVM() {
	restartVMNeeded = false;		
	Display.getDefault().asyncExec(new Runnable() {
		/**
		 * @see java.lang.Runnable#run()
		 */
		public void run() {			
			try {
				IFile file = ((IFileEditorInput)getEditorInput()).getFile();
				fProxyFactoryRegistry = createProxyFactoryRegistry(file);
				if (fProxyFactoryRegistry != null) {
					beanProxyAdapterFactory.setProxyFactoryRegistry(fProxyFactoryRegistry);
					fProxyFactoryRegistry.addRegistryListener(registryListener);
					// Farm it off because we have some kludge somewhere that requires instantiation to be on the ui thread.
					// Next release look at this so that isn't necessary.
					RefObject root = getRootModel();
					root.notify(new NotificationImpl(root, Notifier.TOUCH, SF_INSTANTIATE_PROXIES, null, null));
					Iterator itr = ((EditDomain) getDomain()).getViewers().iterator();
					while (itr.hasNext()) {
						EditPartViewer viewer = (EditPartViewer) itr.next();
						EditPart rootEP = viewer.getContents();
						rootEP.setModel(getRootModel());
						rootEP.refresh();
						rootEP.activate();
					}
				}
			} catch (CoreException e) {
			}										
		}
	});
}
private ActivationListener activationListener = new ActivationListener();

private class ActivationListener extends ShellAdapter implements IPartListener {
	
	public IWorkbenchPart activePart;	
	private boolean isHandlingActivation= false;	
	public boolean shellActive = true;	// Assume that when we get added that the shell is active, or we wouldn't of been able to run the code to add it.
	
	/*
	 * @see IPartListener#partActivated(IWorkbenchPart)
	 */
	public void partActivated(IWorkbenchPart part) {
		activePart = part;
		handleActivation();
	}
	
	protected void handleActivation() {
		if (isHandlingActivation)
			return;
			
		if (activePart == realEditorPart && restartVMNeeded) {
			isHandlingActivation = true;
			try {
				restartVM();
			} finally {
				isHandlingActivation = false;
			}
		}
	}

	/*
	 * @see IPartListener#partBroughtToTop(IWorkbenchPart)
	 */
	public void partBroughtToTop(IWorkbenchPart part) {
	}

	/*
	 * @see IPartListener#partClosed(IWorkbenchPart)
	 */
	public void partClosed(IWorkbenchPart part) {
	}

	/*
	 * @see IPartListener#partDeactivated(IWorkbenchPart)
	 */
	public void partDeactivated(IWorkbenchPart part) {
		activePart= null;		
	}

	/*
	 * @see IPartListener#partOpened(IWorkbenchPart)
	 */
	public void partOpened(IWorkbenchPart part) {
	}
	
		
	/*
	 * @see org.eclipse.swt.events.ShellListener#shellActivated(ShellEvent)
	 */
	public void shellActivated(ShellEvent e) {
		shellActive = true;
		handleActivation();
	}

	/**
	 * @see org.eclipse.swt.events.ShellListener#shellDeactivated(ShellEvent)
	 */
	public void shellDeactivated(ShellEvent e) {
		shellActive = false;
	}
};
private IEditorPart realEditorPart = null;	// Kludge needed to know if we are the active editor or not. The JavaVisualEditorPart is the real one.
public void setRealEditorPart(IEditorPart realEditorPart) {
	this.realEditorPart = realEditorPart;
}
private boolean isActivePartAndShell() {
	// See if the active page is a JavaVisualEditor that contains us and that the shell is active.
	return activationListener.activePart ==  realEditorPart && activationListener.shellActive;
}

private JBCFModelSynchronizer modelSynchronizer = null;
protected void initModelBuilder(ResourceSet rs)
{

com.ibm.etools.cde.CDEHack.fixMe("may need to move some of this into the ModelBuilder init function");  //$NON-NLS-1$

	try { 
		IFile file = ((IFileEditorInput)getEditorInput()).getFile();
		// Initialize the nature for the project that creates a remote VM and registers the 
		// java:/ protocol in MOF
		IProject aProject = file.getProject();
		BeaninfoNature fNature = BeaninfoNature.getRuntime(aProject);
		com.ibm.etools.java.instantiation.base.JavaInstantiation.initialize(fNature.getContext());
		fNature.getContext().setParent(getGlobalResourceSet().getContext());
		// Create the target VM that is used for instances within the document we open
		fProxyFactoryRegistry = createProxyFactoryRegistry(file);
		// Make sure that there is an AdaptorFactory for BeanProxies installed
		beanProxyAdapterFactory = new BeanProxyAdapterFactory(fProxyFactoryRegistry, (EditDomain) getDomain());
		fProxyFactoryRegistry.addRegistryListener(registryListener);
		rs.getContext().addAdapterFactory(beanProxyAdapterFactory);	
				
		// Add listener to part activation so that we can handle recycle the vm when reactivated.
		IWorkbenchWindow window= getSite().getWorkbenchWindow();
		window.getPartService().addPartListener(activationListener);
		window.getShell().addShellListener(activationListener);
		
		// Add the model synchronizer.
		modelSynchronizer = new JBCFModelSynchronizer(beanProxyAdapterFactory, JavaCore.create(aProject), new Runnable() {
			/**
			 * @see java.lang.Runnable#run()
			 */
			public void run() {
				// Terminate registry
				recycleCntr = 0;	// So that this resets the cntr for premature terminates.
				fProxyFactoryRegistry.terminateRegistry();
			}
		});
	} catch ( CoreException exc ) {
		JBCFPlugin.getPlugin().getMsgLogger().log(exc.getStatus(), JBCFMsgLogger.LOG_WARNING);
	} catch ( Exception exc ) {
		JBCFPlugin.getPlugin().getMsgLogger().log(exc, JBCFMsgLogger.LOG_WARNING);
	}
}

protected List paletteCats = null;

protected ProxyFactoryRegistry createProxyFactoryRegistry(IFile aFile) throws CoreException {

	JBCFConfigurationContributor jbcfCont = new JBCFConfigurationContributor(aFile.getProject());
	IConfigurationContributor[] contribs = new IConfigurationContributor[] {BeaninfoNature.getRuntime(aFile.getProject()).getConfigurationContributor(), jbcfCont};
	
	ProxyFactoryRegistry registry = ProxyPlugin.getPlugin().startImplementation(
		aFile.getProject(), 
		"VM for " + aFile.getName(), //$NON-NLS-1$
		contribs,
		new NullProgressMonitor()
		);
	paletteCats = jbcfCont.paletteCats;
	return registry;
}
public void dispose(){
	super.dispose();
	// Now remove our listeners.
	if (modelSynchronizer != null) {
		modelSynchronizer.stopSynchronizer();
		modelSynchronizer = null;
	}
	
	IWorkbenchWindow window= getSite().getWorkbenchWindow();	
	window.getPartService().removePartListener(activationListener);
	Shell shell = window.getShell();
	if (shell != null && !shell.isDisposed())	
		window.getShell().removeShellListener(activationListener);
			
	if ( fProxyFactoryRegistry != null ) {
		fProxyFactoryRegistry.removeRegistryListener(registryListener);	// We're going away, don't let the listener come into play.
		fProxyFactoryRegistry.terminateRegistry();
	}
}
/**
 * This is required so the inner classes can see it so must explicitly declare the protected method
 * on this class even though it is in inherited.
 */
protected IConfigurationElement getConfigurationElement() {
	return super.getConfigurationElement();
}


	/**
	 * @see EMFGraphicalEditorPart#createResourceSet()
	 * We need to create one via the beaninfo nature.
	 */
	protected ResourceSet createResourceSet() {
		IProject aProject = ((IFileEditorInput) getEditorInput()).getFile().getProject();
		try {
			BeaninfoNature nature = BeaninfoNature.getRuntime(aProject);
			ResourceSet rset = nature.newResourceSet();
			rset.getContext().setURIConverter(new WorkbenchURIConverterImpl());
			return rset;
		} catch (CoreException e) {
		}
		
		return null;
	}

	/**
	 * @see com.ibm.etools.cde.emf.EMFGraphicalEditorPart#createOutlinePage()
	 */
	protected EMFContentOutlinePage createOutlinePage() {
		return new JBCFOutlinePage(new TreeViewer());
	}

/**
 * @see com.ibm.etools.cde.emf.EMFGraphicalEditorPart#getPaletteRoot()
 */
protected PaletteRoot getPaletteRoot() {
	PaletteRoot paletteRoot = super.getPaletteRoot();
	
	// This is a huge kludge for this release. We're not handling sharing of palette or modifications. We aren't cleaning up.
	if (paletteRoot instanceof Palette) {
		// Convert this into a PaletteRef so that we can modify it without screwing up the originals.
		PalettePackage palPkg = (PalettePackage) RefRegister.getPackage(PalettePackage.packageURI);
		PaletteRef newRoot = palPkg.getPaletteFactory().createPaletteRef();
		Palette oldRoot = (Palette) paletteRoot;
		newRoot.setRefControlGroup(oldRoot.getControlGroup());
		List newCats = newRoot.getRefCategories();
		newCats.addAll(oldRoot.getCategories());
		// Now get the extension palette cats and add them.
		if (paletteCats != null && !paletteCats.isEmpty()) {
			EMetaObject metaCats = palPkg.getCategory();
			for (Iterator iter = paletteCats.iterator(); iter.hasNext();) {
				String catxmi = (String) iter.next();
				Resource catres = null;
				try {
					catres = getResourceSet().getResourceAndLoad(catxmi);
					newCats.addAll(catres.getExtent().getObjectsByType(metaCats));
				} catch (RuntimeException e) {
					JBCFPlugin.getPlugin().getMsgLogger().log(e, Level.WARNING);
				}	
			}
		}
		paletteRoot = newRoot;
	}
	return paletteRoot;
}

}