package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.swt.graphics.Image;
import com.ibm.etools.cde.emf.DefaultGraphicalEditPart;
import com.ibm.etools.draw2d.Label;
import com.ibm.etools.java.instantiation.base.IJavaInstance;

public class JavaBeanGraphicalEditPart extends DefaultGraphicalEditPart {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IBeanProxyHost.ErrorListener fBeanProxyErrorListener;
	
	public void activate() {	
		super.activate();
		IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());		
		if(fBeanProxyErrorListener == null){		
			fBeanProxyErrorListener = new IErrorNotifier.ErrorListenerAdapter(){
				public void errorStatus(int severity){
					refreshVisuals();
				}
			};
		}
		beanProxyHost.addErrorListener(fBeanProxyErrorListener);
	}
	
	public void deactivate(){
		if(fBeanProxyErrorListener != null){
			IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());
			beanProxyHost.removeErrorListener(fBeanProxyErrorListener);
		}
	}
		
	protected void setFigureImage(Label aLabel,Image anImage){
		
		// See whether or not the JavaBean is in error
		IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());
		if (beanProxyHost.getErrorStatus() == IBeanProxyHost.ERROR_SEVERE){
			super.setFigureImage(aLabel, IErrorHolder.ErrorType.getSevereErrorImage());
		} else if (beanProxyHost.getErrorStatus() == IBeanProxyHost.ERROR_WARNING){
			super.setFigureImage(aLabel, IErrorHolder.ErrorType.getWarningErrorImage());			
		} else {
			super.setFigureImage(aLabel, anImage);
		}
	}	
	
	public Object getAdapter(Class aKey){
		// See if any of the MOF adapters on our target can return a value for the request		
		Object result = super.getAdapter(aKey);
		if ( result != null ) {
			return result;
		} else {
			Iterator mofAdapters = ((IJavaInstance)getModel()).getAdapters().iterator();
			while(mofAdapters.hasNext()){
				Object mofAdapter = mofAdapters.next();
				if ( mofAdapter instanceof IAdaptable ) {
					Object mofAdapterAdapter = ((IAdaptable)mofAdapter).getAdapter(aKey);
					if ( mofAdapterAdapter != null ) {
						return mofAdapterAdapter;
					}
				}
			}
		}
		return null;
	}
}
