package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.TreeItem;

import com.ibm.etools.cde.emf.DefaultTreeEditPart;
import com.ibm.etools.java.instantiation.base.IJavaInstance;

public class JavaBeanTreeEditPart extends DefaultTreeEditPart {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IErrorNotifier.ErrorListener fErrorListener;
	
	public JavaBeanTreeEditPart(Object aModel){
		super(aModel);
	}
	
	public void activate() {	
		super.activate();
		IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());		
		if(fErrorListener == null){		
			fErrorListener = new IErrorNotifier.ErrorListenerAdapter(){
				public void errorStatus(int severity){
					refreshVisuals();
				}
			};
		}
		beanProxyHost.addErrorListener(fErrorListener);

	}	
	
	public void deactivate(){
		if(fErrorListener != null){
			IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());
			beanProxyHost.removeErrorListener(fErrorListener);
		}
		super.deactivate();		
	}	
	
	protected void setWidgetImage(Image anImage){
		
		// See whether or not the JavaBean is in error
		IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());
		if (beanProxyHost.getErrorStatus() == IBeanProxyHost.ERROR_SEVERE){
			super.setWidgetImage(IErrorHolder.ErrorType.getSevereErrorImage());
		} else if (beanProxyHost.getErrorStatus() == IBeanProxyHost.ERROR_WARNING){
			super.setWidgetImage(IErrorHolder.ErrorType.getWarningErrorImage());
		} else {
			super.setWidgetImage(anImage);
		}
	}
	// For all the subclasses that have children we wish them to be expanded by default
	// so that the user doesn't have to explicitly expand it
	protected void doInitialize(){	
		super.doInitialize();
		((TreeItem)getWidget()).setExpanded(true);
	}		
	public Object getAdapter(Class aKey){
		Object result = super.getAdapter(aKey);
		if ( result != null ) {
			return result;
		} else {
			// See if any of the MOF adapters on our target can return a value for the request
			Iterator mofAdapters = ((IJavaInstance)getModel()).getAdapters().iterator();
			while(mofAdapters.hasNext()){
				Object mofAdapter = mofAdapters.next();
				if ( mofAdapter instanceof IAdaptable ) {
					Object mofAdapterAdapter = ((IAdaptable)mofAdapter).getAdapter(aKey);
					if ( mofAdapterAdapter != null ) {
						return mofAdapterAdapter;
					}
				}
			}
		}
		return null;
	}
}	