package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.text.MessageFormat;

import org.eclipse.swt.widgets.Composite;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaDataType;
import com.ibm.etools.java.instantiation.base.IJavaDataTypeInstance;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.propertysheet.ObjectCellEditor;
import com.ibm.etools.proxy.IIntegerBeanProxy;

public class MnemonicCellEditor extends ObjectCellEditor implements INeedData , IJBCFCellEditor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain fEditDomain;
	protected JavaDataType fIntType;
	protected ResourceSet fResourceSet;
	
public MnemonicCellEditor(Composite parent){
	super(parent);
}	
public Object doGetObject(String aString){
	// The user could have entered a char, e.g. S or t
	// or they could have entered the integer value itself, e.g. 83 or 116
	boolean isCharEntered = false;
	try {
		// If the string formats as a number treat it as such
		int number = Integer.parseInt(aString);
		// unless it is a single digit ( between 0 and 9 ) in which case it is more likely to be a char,
		// but if it is between 0 and 9 and more that one character, then treat it is an integer (e.g. 01, 02, etc.)
		if ( number >= 0 && number <= 9 && aString.length() == 1) {
			return BeanUtilities.createJavaObject(fIntType, fResourceSet , "'" + aString + "'"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			return BeanUtilities.createJavaObject(fIntType, fResourceSet , String.valueOf(number));
		}
	} catch ( NumberFormatException exc ) {
		// otherwise take the first character as the char
		char charEntered = aString.charAt(0); //$NON-NLS-1$
		
		// This Editor takes any single character, and packages it as such.
		// Including letters, digits, and symbols.
		if ( aString.length() == 1 && ( charEntered == '\\' || charEntered == '\'') ) { //$NON-NLS-1$ //$NON-NLS-2$
			return BeanUtilities.createJavaObject(fIntType,fResourceSet, "'\\" + new Character(charEntered).toString() + "'"); //$NON-NLS-1$ //$NON-NLS-2$			
		} else if ( aString.length() == 1 ) {
			return BeanUtilities.createJavaObject(fIntType,fResourceSet, "'" + new Character(charEntered).toString() + "'"); //$NON-NLS-1$ //$NON-NLS-2$			
		} else { 
			return BeanUtilities.createJavaObject(fIntType, fResourceSet , aString);
		}
	}
}
public String doGetString(Object anObject){
	if (anObject == null){
		return ""; //$NON-NLS-1$
	} else {
		return MnemonicLabelProvider.getCharacterLabel((IJavaDataTypeInstance)anObject, fEditDomain);
	}
}
public String isCorrectObject(Object anObject){
	return null;
}
/**
 * The user can either enter a char directly, e.g. S or t
 * or for non keyboard characters enter the value, e.g. 83 or 116
 * If the values 1 to 9 are entered these are assumed to be chars and not int values
 */
public String isCorrectString(String aString){
	if (aString == null ||aString.length() > 1){
		// Make sure the string is numeric
		try {
			int number = Integer.parseInt(aString);
			//  The number must be a valid character
			if (!Character.isDefined((char)number)){
				return JBCFMessages.getString("CellEditor.Mnemonic.InvalidRange_ERROR_"); //$NON-NLS-1$
			}
		} catch ( NumberFormatException exc ) {
			return JBCFMessages.getString("CellEditor.Mnemonic.InvalidMsg_ERROR_"); //$NON-NLS-1$			
		}
	} else if (aString.length() == 0){
		return JBCFMessages.getString("CellEditor.Mnemonic.ErrMsg_ERROR_"); //$NON-NLS-1$
	} 
	return null;
}
public void setData(Object data){
	fEditDomain = (EditDomain) data;
	fResourceSet = JBCFEditDomainHelper.getResourceSet(fEditDomain);
	fIntType = (JavaDataType)Utilities.getJavaClass("int",fResourceSet); //$NON-NLS-1$
}
public String getJavaInitializationString(){
	if ( getValue() == null ){
		return null;
	} else {
		// The value can either be an int, in which case the label doesn't have quotes around it, e.g. 116 or 83
		// or else it is a char in which case the label has quotes, e.g. 'S' or 't'
		String label = MnemonicLabelProvider.getCharacterLabel((IJavaDataTypeInstance)getValue(), fEditDomain);
		if ( label.length() == 1 ) {
			return "'" + label + "'"; //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			try {
				Integer.parseInt(label);
				return label;
			} catch ( NumberFormatException exc ) {
				// The label is not one character and is not a number.
				// This should not occur, however treat it as a char
				return "'" + label + "'"; //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}
}
}
