package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.instantiation.base.IJavaDataTypeInstance;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.proxy.*;
import org.eclipse.jface.viewers.*;

/**
 */
public class MnemonicLabelProvider extends LabelProvider implements INeedData {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain editDomain;
	
public String getText(Object element) {
	if ( element == null ) {
		return ""; //$NON-NLS-1$
	} else {
		return getCharacterLabel((IJavaDataTypeInstance)element, editDomain);
	}
}
public static String getCharacterLabel(IJavaDataTypeInstance intOrCharInstance, EditDomain editDomain){
	IBeanProxy intOrCharProxy = BeanProxyUtilities.getBeanProxy(intOrCharInstance, JBCFEditDomainHelper.getResourceSet(editDomain));
	if ( intOrCharProxy instanceof IIntegerBeanProxy ) {
		int intValue = ((IIntegerBeanProxy)intOrCharProxy).intValue();

		// See whether the int value can be displayed to the user 
		if ( Character.isLetterOrDigit((char)intValue)){
			// display any characters or digits
			return new Character((char)intValue).toString();
		} else if ( intValue <= 255 && !Character.isISOControl((char)intValue) ) {
			// display common non-letter characters (like symbols) within the standard ascii/Latin
			return new Character((char)intValue).toString();
		} else {
			// the character is not displayable as char

			// If the number is between 0 and 9, show it with a zero in front of it
			// to distinquish bewteen a number a character. (e.g. '3' versus 03)
			if (intValue >= 0 && intValue <= 9)
				return "0" + new Integer(intValue).toString(); //$NON-NLS-1$
			else
				// all remaining characters are displayed as their value
				return new Integer(intValue).toString();
		}

	} else if ( intOrCharProxy instanceof ICharacterBeanProxy ) {
		char charValue = ((ICharacterBeanProxy)intOrCharProxy).charValue();
		return new Character(charValue).toString();
	} else {
		return ""; //$NON-NLS-1$
	}
}
	/**
	 * @see INeedData#setData(Object)
	 */
	public void setData(Object data) {
		editDomain = (EditDomain) data;
	}

}
