package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ecore.EMetaObject;
import com.ibm.etools.propertysheet.NumberCellEditor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.JavaClass;
import java.text.*;
import java.util.*;
import org.eclipse.swt.widgets.Composite;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.JavaDataType;
import com.ibm.etools.proxy.INumberBeanProxy;
import org.eclipse.core.runtime.IConfigurationElement;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.propertysheet.PropertysheetMessages;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.impl.JavaClassImpl;
/**
 * Cell Editor for Integer beans
 */
public class NumberJavaCellEditor extends NumberCellEditor implements INeedData , IJBCFCellEditor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected JavaHelpers fNumberType;
	protected String fInitializationString;
	protected EditDomain fEditDomain;
	protected String fTypeName;
	
	/**
	 * NumberBeanTypeCellEditor constructor comment.
	 */
	public NumberJavaCellEditor(Composite aComposite) {
		super(aComposite);
	}
	
	public void setInitializationData(IConfigurationElement ce, String pName, Object initData) {
		if (initData instanceof String)
			fTypeName = (String) initData;
		else
			fTypeName = "void";	// Default to nothing allowed. //$NON-NLS-1$
			
		if (fTypeName.equals("java.lang.Integer")) { //$NON-NLS-1$
			fInitializationString = "new Integer({0})"; //$NON-NLS-1$
			super.setInitializationData(null, null, "integer"); //$NON-NLS-1$
		} else if (fTypeName.equals("int")) { //$NON-NLS-1$
			fInitializationString = "{0}"; //$NON-NLS-1$
			super.setInitializationData(null, null, "integer"); //$NON-NLS-1$
		} else if (fTypeName.equals("short")) { //$NON-NLS-1$
			fInitializationString = "(short) {0}"; //$NON-NLS-1$
			super.setInitializationData(null, null, "short"); //$NON-NLS-1$
		} else if (fTypeName.equals("java.lang.Short")) { //$NON-NLS-1$
			fInitializationString = "new Short((short) {0})"; //$NON-NLS-1$
			super.setInitializationData(null, null, "short");		 //$NON-NLS-1$
		} else if (fTypeName.equals("long")) { //$NON-NLS-1$
			fInitializationString = "{0}L"; //$NON-NLS-1$
			super.setInitializationData(null, null, "long");		 //$NON-NLS-1$
		} else if (fTypeName.equals("java.lang.Long")) { //$NON-NLS-1$
			fInitializationString = "new Long({0}L)"; //$NON-NLS-1$
			super.setInitializationData(null, null, "long"); //$NON-NLS-1$
		} else if (fTypeName.equals("byte")) { //$NON-NLS-1$
			fInitializationString = "(byte) {0}"; //$NON-NLS-1$
			super.setInitializationData(null, null, "byte"); //$NON-NLS-1$
		} else if (fTypeName.equals("java.lang.Byte")) { //$NON-NLS-1$
			fInitializationString = "new Byte((byte) {0})"; //$NON-NLS-1$
			super.setInitializationData(null, null, "byte"); //$NON-NLS-1$
		} else if (fTypeName.equals("java.lang.Float")) { //$NON-NLS-1$
			fInitializationString = "new Float({0}F)"; //$NON-NLS-1$
			super.setInitializationData(null, null, "float"); //$NON-NLS-1$
		} else if (fTypeName.equals("float")){ //$NON-NLS-1$
			fInitializationString = "{0}F"; //$NON-NLS-1$
			super.setInitializationData(null, null, "float"); //$NON-NLS-1$
		} else if (fTypeName.equals("double")){ //$NON-NLS-1$
			fInitializationString = "{0}D"; //$NON-NLS-1$
			super.setInitializationData(null, null, "double"); //$NON-NLS-1$
		} else if (fTypeName.equals("java.lang.Double")){ //$NON-NLS-1$
			fInitializationString = "new Double({0}D)"; //$NON-NLS-1$
			super.setInitializationData(null, null, "double"); //$NON-NLS-1$
		};
	}
	
	public void setData(Object data) {
		fEditDomain = (EditDomain) data;
		fNumberType = JavaClassImpl.reflect(fTypeName, JBCFEditDomainHelper.getResourceSet(fEditDomain));
	}
	

	/**
	 * Create an instance of the MOF IJavaInstance of the specified type
	 */
	protected Object doGetObject(String value){
		Number newNumber = (Number) super.doGetObject(value);
		return newNumber != null ? BeanUtilities.createJavaObject(fNumberType, JBCFEditDomainHelper.getResourceSet(fEditDomain), getJavaInitializationString(newNumber)) : null;
	}
	
	/**
	 * Returns the string for the value.
	 */
	protected String doGetString(Object value) {
		if (((EMetaObject) fNumberType).isInstance(value)) {
			return super.doGetString(((INumberBeanProxy) BeanProxyUtilities.getBeanProxy((IJavaInstance) value, JBCFEditDomainHelper.getResourceSet(fEditDomain))).numberValue());
		} else
			return null;
	}		
		
	protected String isCorrectObject(Object value) {
		if (value == null)
			if (fNumberType.isPrimitive())
				return PropertysheetMessages.getString(PropertysheetMessages.NULL_INVALID);	// null is invalid for primitives
			else
				return super.isCorrectObject(null);
		else if (((EMetaObject) fNumberType).isInstance(value))
			return super.isCorrectObject(((INumberBeanProxy) BeanProxyUtilities.getBeanProxy((IJavaInstance) value, JBCFEditDomainHelper.getResourceSet(fEditDomain))).numberValue());
		else
			return (fFormatter.isParseIntegerOnly() ? sNotIntegerError : sNotNumberError);
	}
	
	/**
	 * getJavaInitializationString method comment.
	 */
	public String getJavaInitializationString() {
		if (isValueValid()) {
			Object value = getSetValue();
			return (value != null) ? 
				getJavaInitializationString(((INumberBeanProxy) BeanProxyUtilities.getBeanProxy((IJavaInstance) value, JBCFEditDomainHelper.getResourceSet(fEditDomain))).numberValue()) : 
				null;
		}
		return null;
	}	
	
	
	/**
	 * getJavaInitializationString method comment.
	 */
	protected String getJavaInitializationString(Number aNumber) {
		// fInitializationString was set in the ctor based on the Mof type
		return MessageFormat.format(fInitializationString, new Object[] {aNumber.toString()});
	}
	
}
