package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */




import com.ibm.etools.cde.emf.EMFGraphicalEditorPart;
import com.ibm.etools.emf.resource.*;
import com.ibm.etools.emf.ref.Extent;
import com.ibm.etools.emf.ref.impl.ExtentImpl;


import com.ibm.etools.cde.properties.*;
import com.ibm.etools.cde.palette.*;
import java.util.Iterator;
import com.ibm.etools.common.command.*;
import com.ibm.etools.draw2d.ColorConstants;
import com.ibm.etools.draw2d.DelegatingLayout;
import com.ibm.etools.draw2d.IFigure;
import com.ibm.etools.draw2d.Label;
import com.ibm.etools.draw2d.Layer;
import com.ibm.etools.draw2d.LayeredPane;
import com.ibm.etools.draw2d.LineBorder;
import com.ibm.etools.draw2d.Locator;
import com.ibm.etools.draw2d.geometry.Dimension;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.draw2d.geometry.Rectangle;

import java.io.*;
import org.eclipse.core.resources.*;
import com.ibm.etools.gef.*;
import com.ibm.etools.gef.palette.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ecore.*;
import java.lang.reflect.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.*;
import org.eclipse.ui.part.FileEditorInput;
import com.ibm.etools.gef.ui.actions.ActionRegistry;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.views.properties.*;
import org.eclipse.core.runtime.*;
import com.ibm.etools.gef.ui.parts.TreeViewer;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import com.ibm.etools.gef.ui.parts.ContentOutlinePage;
import com.ibm.etools.cdm.*;
import com.ibm.etools.cde.*;

/**
 * Graphical Editor Part for Desktop for VCE models.
 */

public abstract class OCMGraphicalEditorPart extends EMFGraphicalEditorPart {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected XMLTextPage fXMLTextPage;
	// connection attribute name for storing the model name in an IMarker
	protected static String CONNECTION_MODEL_URL = "connection_model_url"; //$NON-NLS-1$
	protected static String CONNECTION_ERROR_TYPE = "connection_error_type"; //$NON-NLS-1$
	protected static String CONNECTION_SOURCE_TERMINAL_ERROR = "connection_source_terminal_error"; //$NON-NLS-1$
	protected static String CONNECTION_TARGET_TERMINAL_ERROR = "connection_target_terminal_error"; //$NON-NLS-1$
	
protected void createXMLTextViewerControl(Composite parent){
	//canvas = new Canvas(parent, SWT.BORDER);
	//canvas.setLayoutData(new sashform.Weight(100,0,0));
	getXMLTextPage().createControl(parent);
}


public void createPartControl(Composite c){
	IPreferenceStore store = CDEPlugin.getPlugin().getPreferenceStore();
	if (store.getBoolean(CDEPlugin.SHOW_XML)){
		// If the Pref store wants the XML page put it on a sash
		SashForm sashform = new SashForm(c, SWT.VERTICAL);
		super.createPartControl(sashform);
		createXMLTextViewerControl(sashform);
	} else {
		super.createPartControl(c);
   	}
}

protected void initializeResourceSet(ResourceSet rs){
	super.initializeResourceSet(rs);
	// Add the property source and property descriptor adapter factories
	Context context = rs.getContext();
	context.addAdapterFactory(new PropertySourceAdapterFactory((com.ibm.etools.cde.EditDomain) getDomain()));
	context.addAdapterFactory(new PropertyDescriptorAdapterFactory());		
}

protected void handleResourceChanged() {
}


/* Return the page for the XMLTextSource
 */ 
protected XMLTextPage getXMLTextPage(){
	
	if ( fXMLTextPage == null ) {
		fXMLTextPage = new XMLTextPage();
		// Add a command stack listener to refresh the text of the XML source page
		// each time something happens
		getDomain().getCommandStack().addCommandStackListener(new CommandStackListener() {
			public void commandStackChanged(java.util.EventObject anEventObject) {
				refreshTextPage();
			}
		});
		// Having created the page give it the initial source
		refreshTextPage();
	}
	
	return fXMLTextPage;	
	
}

public void refreshTextPage(){
	if ( getResource() != null ) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		try {
			getResource().save(os)	;
			fXMLTextPage.setText(os.toString());
		} catch (Exception e) {
			JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
			fXMLTextPage.setText(""); //$NON-NLS-1$
		}
	} else {
		fXMLTextPage.setText(""); //$NON-NLS-1$
	}
}	


/**
 * Create and return an empty document.  This 
 * method should be implemented to create a new document
 * and a new root within the document.
 */
protected Resource createEmptyResource(String filename, ResourceSet rs) {
	Resource res = ResourceFactoryRegister.getFactory(filename, rs).makeResource(filename);
	Extent ext = res.getExtent();
	rs.add(res);
	
	DiagramData composition = createEmptyComposition();
	ext.add(composition);
	return res;
}



/**
 * Create a default composition. This can be overridden to return
 * a different composition subclass.
 */
protected DiagramData createEmptyComposition() {
	CDEHack.fixMe("This should probably be changed to a parm to the configuration element so that it can be parameterized"); //$NON-NLS-1$
	return CDEUtilities.CDM_PACKAGE.getCDMFactory().createDiagramData();
}

/**
 * Find the first Composition in the document
 */
protected RefObject getModel(Resource res){
	CDEHack.fixMe("This should probably be changed to a parm to the configuration element so that it can be parameterized");	 //$NON-NLS-1$
	DiagramData rootComposition = (DiagramData) res.getExtent().getObjectByType(CDEUtilities.CDM_PACKAGE.getDiagramData());
	if (rootComposition != null)
		setupDiagramData(rootComposition);
	return rootComposition;
}


/**
 * If this is a problem marker for a connection, find the edit part for
 * this model and select it.
 */
public void gotoMarker(IMarker marker) {
	try {
		// If this marker launched the editor, it was probably deleted...
		// need to check that it still exists. Also check that it's a problem.
		// If so, get it's model and respective editpart and select it.
		if (marker.exists() && marker.getType().equals(IMarker.PROBLEM)) {
			String modelURL = (String) marker.getAttribute(CONNECTION_MODEL_URL);
			if (modelURL != null) {
				RefObject model = getResourceSet().getObjectAndLoad(modelURL);
				if (model != null) {
					EditPart connEP = (EditPart) getPrimaryViewer().getEditPartRegistry().get(model);
					if (connEP != null) {
						getPrimaryViewer().select(connEP);
					}
				}
			}
		}
	} catch (CoreException exc) {
		// How to handle?
		// handleException(exc);
	}
}


/**
 * Validate the model. It's here that we should create any soft (or severe)
 * errors as markers to be shown in the task list. 
 *
 * Note: Plugin specific markers can be used here to create and manage 
 *       only those markers (errors) that are relevent.
 */
protected void validateModel(RefObject model) {
}
/**
 * Create a label in the center of the free form that can be used to display status and other details
 * to the user as required
 */
protected Label fStatusLabel;
protected void updateStatusLabel(String aLabel){
	if ( aLabel == null ) {
		fStatusLabel.setVisible(false);
		fStatusLabel.setText("");		
	} else {
		fStatusLabel.setText(aLabel);
		fStatusLabel.setVisible(true);
	}
}
protected void createStatusLabel(){
	
	// A Figure is created on the graph viewer which sits in the center 
	// and can be used to display progress and other messages to the user when a reload or other task is occurring
	LayeredPane pane = (LayeredPane) ((GraphicalEditPart) getPrimaryViewer().getRootEditPart()).getFigure();
	Layer statusLabelLayer = new Layer();
	pane.add(statusLabelLayer);
	
	// This figure is a right pointing arrow on the canvas for the JVE
	final Label statusLabel = new Label("Loading ...");
	statusLabel.setBackgroundColor(ColorConstants.gray);
	statusLabel.setBorder(new LineBorder(ColorConstants.black,1));
	statusLabelLayer.setLayoutManager(new DelegatingLayout());
	// Use a locator that anchors to the middle of the left edge
	statusLabelLayer.add(statusLabel, new Locator() {
		public void relocate(IFigure fig) {
			IFigure parent = fig.getParent();
			Rectangle parentBounds = parent.getBounds();
			Dimension labelSize = statusLabel.getPreferredSize();
			fig.setSize(labelSize);
			fig.setLocation(new Point((parentBounds.width - statusLabel.getBounds().width) / 2 , parentBounds.height / 2 + 5));
			fig.repaint();
		}
	});	
}
}
