package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.util.StringTokenizer;
import org.eclipse.swt.widgets.Composite;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.cde.EditDomain;
/**
 * Cell Editor for Dimension Beans.
 */
public class PointJavaClassCellEditor extends DefaultJavaClassCellEditor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
public PointJavaClassCellEditor(Composite aComposite){
	super(aComposite);
}

/**
 * Returns the string for the value.
 */
protected String doGetString(Object value) {
	if (isInstance(value)) {
		return PointJavaClassLabelProvider.toString((IJavaInstance) value);
	} else
		return null;
}

protected String getJavaInitializationString(String dimString) {
	// want to make sure nicely formed (i.e. no extra spaces). This assumes the string is valid. This shouldn't be called if it isn't.
	StringTokenizer st = new StringTokenizer(dimString, ","); //$NON-NLS-1$
	StringBuffer sb = new StringBuffer(dimString.length());
	sb.append("new java.awt.Point("); //$NON-NLS-1$
	sb.append(st.nextToken().trim());
	while (st.hasMoreTokens()) {
		sb.append(',');
		sb.append(st.nextToken().trim());
	}
	sb.append(')');
	return (sb.toString());
}

/**
 * Parse the text string to see if it is a valid dimension, e.g. width,height
 */
protected String isCorrectString(String text) {
	StringTokenizer st = new StringTokenizer(text, ","); //$NON-NLS-1$
	
	String[] tokenMsgs = new String[] {JBCFMessages.getString("CellEditor.Point.XErrorMsg_ERROR_"), JBCFMessages.getString("CellEditor.Point.YErrorMsg_ERROR_")}; //$NON-NLS-1$ //$NON-NLS-2$
	if (st.countTokens() != tokenMsgs.length)
		return JBCFMessages.getString("CellEditor.Point.ErrorMsg_ERROR_"); //$NON-NLS-1$
	
	for (int i=0; i<tokenMsgs.length; i++) {
		String field = st.nextToken().trim();
		try {
			Integer.decode(field);
		} catch ( NumberFormatException exc ){
			return tokenMsgs[i];
		}
	}
	return null;
}
public void setData(Object data) {
	super.setData(data);
	setJavaType(JavaClassImpl.reflect("java.awt.Point", JBCFEditDomainHelper.getResourceSet(fEditDomain))); //$NON-NLS-1$
}

/**
 * Helper to return a well formed Java Initialization string for an width and height
 */
public static String getJavaInitializationString(int x, int y){
	StringBuffer buffer = new StringBuffer();
	buffer.append("new java.awt.Point("); //$NON-NLS-1$
	buffer.append(String.valueOf(x));
	buffer.append(',');
	buffer.append(String.valueOf(y));
	buffer.append(')');
	return buffer.toString();
}

}


