package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ecore.EMetaObject;

import org.eclipse.jdt.internal.corext.codemanipulation.GetterSetterUtil;
import org.eclipse.swt.widgets.Composite;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.propertysheet.PropertysheetMessages;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.proxy.IStringBeanProxy;
/**
 * Cell editor for Strings that are Beans.
 */
public class StringJavaClassCellEditor extends DefaultJavaClassCellEditor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
public StringJavaClassCellEditor(Composite aComposite){
	super(aComposite);
}

/**
 * getJavaInitializationString method comment.
 */
protected String getJavaInitializationString(String aString) {
	return BeanUtilities.createStringInitString(aString);
}

/**
 * Returns the string for the value.
 */
protected String doGetString(Object value) {
	if (isInstance(value)) {
		return ((IStringBeanProxy) BeanProxyUtilities.getBeanProxy((IJavaInstance) value, JBCFEditDomainHelper.getResourceSet(fEditDomain))).stringValue();
	} else if (value != null) {
		// Since this is not a string value, the text widget is disabled, 
		// just return the toString of this object.
		return BeanProxyUtilities.getBeanProxy((IJavaInstance) value, JBCFEditDomainHelper.getResourceSet(fEditDomain)).toBeanString();
	} else
		return null;
}

/**
 * Since we could have a value whose is bean is not a string, we need
 * to disable the test widget so it can't be edited since we don't know what it is.
 */
protected void doSetValue(Object value) {
	// Correct value?
	if (isInstance(value)) {
		// if the text widget was previously disabled, enable it
		if (!text.isEnabled())
			text.setEnabled(true);
	// value not a string? disable the text widget if not already
	} else {
		if (value != null && text.isEnabled())
			text.setEnabled(false);
	}
	super.doSetValue(value);
}

protected String getNotValidMsg(Object value) {
	return PropertysheetMessages.getString(PropertysheetMessages.NOT_STRING);
}

protected String isCorrectString(String value) {
	return null;	// Any string is correct string for string.
}

public void setData(Object data) {
	super.setData(data);
	setJavaType(JavaClassImpl.reflect("java.lang.String", JBCFEditDomainHelper.getResourceSet(fEditDomain))); //$NON-NLS-1$
}
}

