package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
import java.util.Iterator;

import com.ibm.etools.java.JavaParameter;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.proxy.*;

public class StringProxyAdapter extends BeanProxyAdapter {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected boolean isExternalized;
	
public StringProxyAdapter(IBeanProxyDomain aDomain){
	super(aDomain);
}
protected IBeanProxy instantiateWithString(IBeanTypeProxy targetClass, String initString) throws ThrowableProxy, InstantiationException {
	
	// If the initString is completely enclosed by a " at the beginning and end then it is a regular
	// String
	if ( initString.length() > 2 && initString.charAt(0) == '"' && initString.charAt(initString.length()-1) == '"'){
		isExternalized = false;
		return super.instantiateWithString(targetClass, initString);
	}
	// We have some kind of string that may be have come from a resource bundle
	// VAJava generated a style of string like
	// bundle.getString("String");
	// where bundle is a reference to a resource bundle field in the class
	// while WSAD can generate a reference like
	// MyMessages.getString("String");
	// where MyMessages is actually a class that re-directs the getString(String) method to the initilized bundle 
	// ( and therefore makes the loading of the bundle occur only once and is a bit more efficient )
	// Either way, the pattern is XXXXX.getString("YYYYY");
	// We should look for this and if so make the actual string be "{YYYYY}", with the angle brackets showing that the
	// string came from a bundle
	int indexOfGetString = initString.indexOf(".getString("); //$NON-NLS-1$
	if ( indexOfGetString != -1 && initString.charAt(initString.length()-1) == ')') {
		
		// We have two possibilites here, 1. VAJ style in which case no resolution is possible
		// on the target VM as is because it relies on a bundle instance.
		// 2. WSAD externalization pattern which relies on a static method, and can be
		//    resolved on the target VM.
		// Put a bandAid for the time being...
		boolean sGetString = false ;  // is there a static getString() method
		String className = initString.substring(0,indexOfGetString) ;
		com.ibm.etools.java.JavaHelpers h = JavaClassImpl.reflect(className,(com.ibm.etools.emf.ref.RefObject)target) ;
		if (h instanceof JavaClassImpl) {
			 JavaClassImpl clazz = (JavaClassImpl) h ;	
			 for (Iterator itr=clazz.getMethodsExtended().iterator(); itr.hasNext();) {			 	
			 	com.ibm.etools.java.Method m = (com.ibm.etools.java.Method) itr.next() ;			 	
			 	if (m.isStatic() && m.getName().equals("getString")) {
			 		com.ibm.etools.emf.ref.EList params = m.getParameters() ;
			 		if (params.size() == 2) {
			 			JavaParameter p = (JavaParameter) params.get(1) ;
			 			if (p.getJavaType().getJavaName().equals("java.lang.String")) {
			 				sGetString = true ;
			 				break ;
			 			}
			 		}
			 	}
			 }
		}		
		if (!sGetString) {
			String bundleKey = initString.substring(indexOfGetString + 12, initString.length() - 2);
			// Having got the string we need to put some brackets around it as well as some quotes
			// The brackets are designed to let the user know the string came from a bundle.
			// We also store this in a field just in case we want to reference it from the CellEditor later
			isExternalized = true;
			StringBuffer buffer = new StringBuffer();
			buffer.append('"');
			buffer.append('{');
			buffer.append(bundleKey);
			buffer.append('}');
			buffer.append('"');
			return instantiateWithString(targetClass, buffer.toString());
		}
	}
	isExternalized = false;
	return super.instantiateWithString(targetClass,initString);
}
}