package com.ibm.etools.jbcf.choosebean;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;
import org.eclipse.jface.resource.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import java.net.URL;
import java.util.*; 
import com.ibm.etools.emf.resource.*;
import com.ibm.etools.java.*;
import com.ibm.etools.jbcf.JBCFPlugin;

import org.eclipse.core.resources.*;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.internal.ui.dialogs.*;
import org.eclipse.jface.operation.*;
import org.eclipse.jdt.internal.ui.util.*;

public class ChooseBeanDialog 
	extends org.eclipse.ui.dialogs.SelectionDialog{

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String JBCF_CHBEAN_SELECTION_HISTORY_KEY = "CHOOSEBEAN_SELHIST";
	
protected Label statusLabel;
protected static Color 
	redColor = new Color(null,250,0,0),
	greenColor = new Color(null,0,100,0),
	yellowColor = new Color(null,200,200,0);

IProject eclipseProject;
ResourceSet resourceSet;
IJavaProject project;
IJavaElement file;
java.util.List selectionHistory;
protected ChooseBeanDialogSelectionAreaHelper classSelectionHelper;
protected ChooseBeanDialogSerializedFileHelper fileSelectionHelper;

public ChooseBeanDialog(Shell shell, IFile file, ResourceSet resourceSet, java.util.List selectionHistory){
	super(shell);
	setShellStyle(SWT.RESIZE|SWT.APPLICATION_MODAL|SWT.DIALOG_TRIM);
	this.file = JavaCore.getJavaCore().create(file);
	this.eclipseProject = file.getProject();
	this.project = JavaCore.getJavaCore().create(eclipseProject);
	this.resourceSet = resourceSet;
	this.selectionHistory = new ArrayList();
	setTitle(ChooseBeanMessages.getString("MainDialog.title")); //$NON-NLS-1$
	setMessage(ChooseBeanMessages.getString("MainDialog.message")); //$NON-NLS-1$
	try {
		IProject proj = file.getProject();
		String selhists = proj.getPersistentProperty(getQualifiedName());
		if(selhists!=null){
			StringTokenizer spacer = new StringTokenizer(selhists, " ", false);
			while(spacer.hasMoreTokens()){
				this.selectionHistory.add(spacer.nextToken());
			}
		}
	} catch (CoreException e) {
	}
} 

private Preferences getPreferences(){
	return JBCFPlugin.getPlugin().getPluginPreferences();
}

protected void createClassSelectionArea(Composite parentArea){
	classSelectionHelper = new ChooseBeanDialogSelectionAreaHelper(this);
	classSelectionHelper.fillArea(parentArea);
}

public Control createDialogArea(Composite parent){
	Composite dialogArea = (Composite)super.createDialogArea(parent);
	
	createClassSelectionArea(dialogArea);
	createFileSelectionArea(dialogArea);
	createStatusArea(dialogArea);

	classSelectionHelper.selectDefault();

	return dialogArea;
}

protected void createFileSelectionArea(Composite parentArea){
	fileSelectionHelper = new ChooseBeanDialogSerializedFileHelper(this);
	fileSelectionHelper.fillArea(parentArea);
}

protected void createStatusArea(Composite dialogArea){
	statusLabel = new Label(dialogArea, SWT.NONE);
	statusLabel.setForeground(yellowColor);
	statusLabel.setText(ChooseBeanMessages.getString("MainDialog.statusArea.initialText")); //$NON-NLS-1$
	GridData labelData = new GridData(GridData.FILL_HORIZONTAL);
	statusLabel.setLayoutData(labelData);
}

protected Point getInitialSize(){
	return new Point(450,550);
}

public Object[] getResult() {
	Object[] res = fileSelectionHelper.getResults(super.getResult());
	if(res==null)
		return classSelectionHelper.getResults(super.getResult());
	else
		return res;
}

void setResults(java.util.List results){
	super.setResult(results);
}
private QualifiedName getQualifiedName(){
	return new QualifiedName(JBCFPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), JBCF_CHBEAN_SELECTION_HISTORY_KEY);
}

void storeSelectionHistory(){
	try {
		StringBuffer buff = new StringBuffer();
		for(int i=0;i<selectionHistory.size() && i<10;i++){
			buff.append(selectionHistory.get(i)+" ");
		}
		eclipseProject.setPersistentProperty(getQualifiedName(), buff.toString());
	} catch (CoreException e) {
	}
}

void updateStatus(){
	Color messageColor=null;
	IChooseBeanHelper whichHelper = fileSelectionHelper;
	if(whichHelper==null || !whichHelper.isInstantiatable())
		whichHelper = classSelectionHelper;
	boolean instantiatable = whichHelper.isInstantiatable();
	if(instantiatable){
		java.util.List list = new ArrayList();
		list.add(whichHelper.getStatusMessage());
		setResult(list);
		messageColor=greenColor;
	}else{
		setResult(new ArrayList());
		messageColor=redColor;
	}
	if(getOkButton()!=null)
		getOkButton().setEnabled(instantiatable);
	if(statusLabel!=null){
		statusLabel.setForeground(messageColor);
		statusLabel.setText(whichHelper.getStatusMessage());
	}
}

protected void okPressed(){
	super.okPressed();
}

}
