package com.ibm.etools.jbcf.choosebean;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.jdt.internal.ui.util.*;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.ST;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jdt.internal.core.*;
import org.eclipse.jdt.internal.core.BinaryType;
import org.eclipse.jdt.internal.corext.util.TypeInfo;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.operation.IRunnableWithProgress;
import java.lang.reflect.InvocationTargetException;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;

import org.eclipse.core.resources.IProject;
import java.util.*;
import java.util.List;
import java.util.ArrayList;

import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.search.*;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.beaninfo.adapters.Utilities;

public class ChooseBeanDialogSelectionAreaHelper 
	implements IChooseBeanHelper {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected Table upperTable, lowerTable;
protected Button allCheck, swingCheck, awtCheck;
protected CCombo typeText;
protected TypeInfo[] elements;
protected ILabelProvider classLabelProvider = new TypeInfoLabelProvider(TypeInfoLabelProvider.SHOW_TYPE_ONLY);
protected ILabelProvider packageLabelProvider = new TypeInfoLabelProvider(TypeInfoLabelProvider.SHOW_TYPE_CONTAINER_ONLY | TypeInfoLabelProvider.SHOW_ROOT_POSTFIX);
private String[] renderedStrings;
private int[] elementMap;
private Integer[] packagerMap;
protected String message;
protected String secondaryMessage = ""; //$NON-NLS-1$
protected ChooseBeanDialog mainDialog;
protected ChooseBeanSearchEngine engine;
protected String previousTypeTextContent = null;

public ChooseBeanDialogSelectionAreaHelper(ChooseBeanDialog mainDialog){
	this.mainDialog = mainDialog;
	engine = new ChooseBeanSearchEngine(JavaPlugin.getWorkspace());
}

public void fillArea(Composite parentArea){
	Group classSelectionGroup = new Group(parentArea, SWT.SHADOW_ETCHED_IN);
	classSelectionGroup.setText(ChooseBeanMessages.getString("SelectionAreaHelper.GroupTitle")); //$NON-NLS-1$
	GridLayout groupLayout = new GridLayout();
	groupLayout.numColumns = 3;
	classSelectionGroup.setLayout(groupLayout);
	GridData groupLayoutData = new GridData(GridData.FILL_BOTH);
	classSelectionGroup.setLayoutData(groupLayoutData);
	fillClassSelectionArea(classSelectionGroup);
}

protected Object createInstance(String name){
	com.ibm.etools.emf.ecore.EClassifier classifier = 
		com.ibm.etools.beaninfo.adapters.Utilities.getJavaClass(name, mainDialog.resourceSet);
	return classifier.refPackage().getFactory().create(classifier);
}

protected void fillClassSelectionArea(Composite parentArea){
	typeText = new CCombo(parentArea, SWT.BORDER);
	GridData textLayoutData = new GridData(GridData.FILL_HORIZONTAL);
	textLayoutData.horizontalSpan=3;
	typeText.setLayoutData(textLayoutData);
	for(int i=0;i<mainDialog.selectionHistory.size();i++)
		typeText.add((String)mainDialog.selectionHistory.get(i));
	typeText.addModifyListener(new ModifyListener(){
		public void modifyText(ModifyEvent e){handleTextChange();}
	});
	// Hitting CR in the CComboBox will result in the firing
	// of a default selected - Hence we accept the dialog.
	// Since the list in CComboBox also fires the default
	// selected, we check if any items were selected.
	typeText.addSelectionListener(new SelectionListener() {
		public void widgetSelected(SelectionEvent e) {
		}
		public void widgetDefaultSelected(SelectionEvent e) {
			if(typeText.getSelectionIndex()<0) // no list selections - hence can be text only
				mainDialog.okPressed();
		}
	});
	// A CComboBox can handle a tab-traverse only when the
	// text portion of the CComboBox is focussed. The text
	// portion of the CComboBox is the only part which listens
	// to SWT.TRAVERSE events of the tab-kind.
	typeText.addFocusListener(new FocusListener() {
		public void focusGained(FocusEvent e) {
			typeText.setFocus();
		}
		public void focusLost(FocusEvent e) {
		}
	});
	
	upperTable = new Table(parentArea, SWT.BORDER);
	GridData upperTableData = new GridData(GridData.FILL_BOTH);
	upperTableData.horizontalSpan=3;
	upperTableData.heightHint=1;
	upperTable.setLayoutData(upperTableData);
	upperTable.addSelectionListener(new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent e){handleUpperTableSelectionChanged();}
		public void widgetSelected(SelectionEvent e){handleUpperTableSelectionChanged();}
	});
	
	lowerTable = new Table(parentArea, SWT.BORDER);
	GridData lowerTableData = new GridData(GridData.FILL_BOTH);
	lowerTableData.horizontalSpan=3;
	lowerTableData.heightHint=1;
	lowerTable.setLayoutData(lowerTableData);
	lowerTable.addSelectionListener(new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent e){handleLowerTableSelectionChanged();}
		public void widgetSelected(SelectionEvent e){handleLowerTableSelectionChanged();}
	});
	
	allCheck = new Button(parentArea, SWT.RADIO);
	GridData allData = new GridData(GridData.FILL_HORIZONTAL);
	allData.horizontalSpan=1;
	allCheck.setLayoutData(allData);
	allCheck.setText(ChooseBeanMessages.getString("SelectionAreaHelper.SelectType.ALL")); //$NON-NLS-1$
	allCheck.addSelectionListener(new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent e){handleAllCheckSelection();}
		public void widgetSelected(SelectionEvent e){handleAllCheckSelection();}
	});
	swingCheck = new Button(parentArea, SWT.RADIO);
	GridData swingData = new GridData(GridData.FILL_HORIZONTAL);
	swingData.horizontalSpan=1;
	swingCheck.setLayoutData(swingData);
	swingCheck.setText(ChooseBeanMessages.getString("SelectionAreaHelper.SelectType.SWING")); //$NON-NLS-1$
	swingCheck.addSelectionListener(new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent e){handleSwingCheckSelection();}
		public void widgetSelected(SelectionEvent e){handleSwingCheckSelection();}
	});
	awtCheck = new Button(parentArea, SWT.RADIO);
	GridData awtData = new GridData(GridData.FILL_HORIZONTAL);
	awtData.horizontalSpan=1;
	awtCheck.setLayoutData(awtData);
	awtCheck.setText(ChooseBeanMessages.getString("SelectionAreaHelper.SelectType.AWT")); //$NON-NLS-1$
	awtCheck.addSelectionListener(new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent e){handleAWTCheckSelection();}
		public void widgetSelected(SelectionEvent e){handleAWTCheckSelection();}
	});
	
	initialize(ChooseBeanSearchEngine.TYPE_ALL);
	
	allCheck.setSelection(true);
	
	//typeText.setText("");
	//typeText.selectAll();
}

protected IJavaSearchScope getJavaSearchScope(){
	return SearchEngine.createJavaSearchScope(new IJavaElement[] { ((IJavaProject)mainDialog.project) });
}

protected Table getLowerTable(){return lowerTable;}
protected CCombo getTypeText(){return typeText;}
protected Table getUpperTable(){return upperTable;}

public Object[] getResults(Object[] results){
	Object[] newResults = new Object[results.length*2];
	EClassifier classifier = Utilities.getJavaClass((String)results[0], mainDialog.resourceSet);
	newResults[0] = classifier.refPackage().getFactory().create(classifier);
	newResults[1] = classifier;
	if(mainDialog.selectionHistory!=null){
		if(!mainDialog.selectionHistory.contains(classifier.refName()))
			mainDialog.selectionHistory.add(classifier.refName());
		mainDialog.storeSelectionHistory();
	}
	return newResults;
}

protected void handleAllCheckSelection(){
	if(!allCheck.getSelection())
		return;
	initialize(ChooseBeanSearchEngine.TYPE_ALL);
	updateElements(true);
}

protected void handleSwingCheckSelection(){
	if(!swingCheck.getSelection())
		return;
	initialize(ChooseBeanSearchEngine.TYPE_SWING);
	updateElements(true);
}

protected void handleAWTCheckSelection(){
	if(!awtCheck.getSelection())
		return;
	initialize(ChooseBeanSearchEngine.TYPE_AWT);
	updateElements(true);
}

protected void handleTextChange(){
	updateElements(false);
}

protected void handleLowerTableSelectionChanged(){
	updateStatus();
}

protected void handleUpperTableSelectionChanged(){
	int selection= getUpperTable().getSelectionIndex();
	if (selection >= 0) {
		int i = elementMap[selection];
		int k= i;
		int length= renderedStrings.length;
		while (k < length && renderedStrings[k].equals(renderedStrings[i])) {
			k++;
		}
		updateLowerTable(i, k);
	} else 
		updateLowerTable(0, 0);
}

protected void initialize(int type){
	List typeList = engine.searchTypes(getJavaSearchScope(), type);
	if (typeList.isEmpty())
		return;
	TypeInfo[] typeRefs= (TypeInfo[])typeList.toArray(new TypeInfo[typeList.size()]);
	setElements(typeRefs);
	elementMap = new int[elements.length+1];
	renderedStrings = renderStrings(elements);
}

public boolean isInstantiatable(){
	//If any classes are valid, then they should appear in the
	//upper table.
	boolean isInstantiable = true;
	if(getUpperTable().getItemCount()<1){
		isInstantiable = false;
		secondaryMessage = " - " + ChooseBeanMessages.getString("SelectionAreaHelper.SecondaryMsg.NoSelectionMade"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	if(isInstantiable){
		TypeInfo typeInfo = elements[packagerMap[getLowerTable().getSelectionIndex()].intValue()];
		String presentPackageName = mainDialog.file==null || mainDialog.file.getParent()==null?null:mainDialog.file.getParent().getElementName();
		boolean isAnyConstructorPresent = false;
		boolean isNullConstructorPresent = false;
		boolean isTypePublic = false;
		boolean isTypeAbstract = true;
		boolean isInPresentPackage = false;
		try{
			IType type = typeInfo.resolveType(getJavaSearchScope());
			isTypePublic = Flags.isPublic(type.getFlags());
			isTypeAbstract = Flags.isAbstract(type.getFlags());
			isInPresentPackage = type.getPackageFragment().getElementName().equals(presentPackageName);
			IMethod[] methods = type.getMethods();
			for(int m=0;m<methods.length;m++){
				if(methods[m].isConstructor() &&
				   methods[m].getParameterTypes().length<1 &&
				   Flags.isPublic(methods[m].getFlags())){
				   	isNullConstructorPresent = true;
				   	break;
				}
				if(methods[m].isConstructor())
					isAnyConstructorPresent=true;
			}
			secondaryMessage = ""; //$NON-NLS-1$
			if(!isNullConstructorPresent && isAnyConstructorPresent)
				secondaryMessage = secondaryMessage.concat(" - " + ChooseBeanMessages.getString("SelectionAreaHelper.SecondaryMsg.NoPublicNullConstructor")); //$NON-NLS-1$ //$NON-NLS-2$
			if(!isTypePublic)
				secondaryMessage = secondaryMessage.concat(" - " + ChooseBeanMessages.getString("SelectionAreaHelper.SecondaryMsg.TypeNonPublic")); //$NON-NLS-1$ //$NON-NLS-2$
			isInstantiable = ((!isAnyConstructorPresent) || isNullConstructorPresent) && (isTypePublic || isInPresentPackage) && (!isTypeAbstract);
		}catch(JavaModelException e){
			isInstantiable = false;
			secondaryMessage = " - " + ChooseBeanMessages.getString("SelectionAreaHelper.SecondaryMsg.UnknownError"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	if(isInstantiable)
		secondaryMessage = ""; //$NON-NLS-1$
	return isInstantiable;
}

protected void updateElements(boolean forceUpdate){
	String matchString = new String(typeText.getText());
	if((!forceUpdate) && previousTypeTextContent!=null && previousTypeTextContent.equalsIgnoreCase(matchString))
		return;
	boolean fIgnoreCase = true;
	int k= 0;
	String text = matchString;
	previousTypeTextContent = matchString;
	String suffix = (matchString.endsWith("*"))?"":"*"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	StringMatcher matcher = new StringMatcher(text+suffix, fIgnoreCase, false); //$NON-NLS-1$
	String lastString = null;
	int length = elements.length;
	for (int i = 0; i < length; i++) {
		while (i < length && renderedStrings[i].equals(lastString))
			i++;
		if (i < length) {
			lastString= renderedStrings[i];
			if (matcher.match(renderedStrings[i])) {
				elementMap[k]= i;
				k++;
			}
		}
	}
	elementMap[k] = -1;
	updateUpperTable(elementMap, k);
}

private String[] renderStrings(Object[] p) {
	boolean fIgnoreCase = true;
	Object[] strings= new Object[p.length];
	int size= strings.length;
	for (int i= 0; i < size; i++) {
		strings[i]= classLabelProvider.getText(p[i]);
	}
	new TwoArrayQuickSorter(fIgnoreCase).sort(strings, p);
	String[] sortedStrings = new String[strings.length];
	for(int i=0;i<strings.length;i++)
		sortedStrings[i] = (String)strings[i];
	return sortedStrings;
}

protected void setElements(TypeInfo[] elements){
	this.elements = elements;
}

public void selectDefault(){
	typeText.setFocus();
	typeText.setText(""); //$NON-NLS-1$
}

/**
 * Updates the lower table based on the range given in as input.
 */
protected void updateLowerTable(int from, int to){
	getLowerTable().removeAll();
	Display d= getLowerTable().getDisplay();
	packagerMap = new Integer[to-from];
	Object[] qualifiers = new Object[to-from];
	for (int i= from; i < to; i++) {
		qualifiers[i-from]= packageLabelProvider.getText(elements[i]);
		packagerMap[i-from]= new Integer(i);
	}
	new TwoArrayQuickSorter(true).sort(qualifiers, packagerMap);
	for (int i= 0; i < to-from; i++) {
		TableItem ti= new TableItem(getLowerTable(), i);
		ti.setText((String)qualifiers[i]);
		Image img= packageLabelProvider.getImage(elements[from+i]);
		if (img != null)
			ti.setImage(img);
	}
	if (getLowerTable().getItemCount() > 0) {
		getLowerTable().setSelection(0);
	}
	handleLowerTableSelectionChanged();
}

protected void updateUpperTable(int[] indices, int size){
	getUpperTable().setRedraw(false);
	int itemCount= getUpperTable().getItemCount();
	if (size < itemCount) {
		getUpperTable().remove(0, itemCount-size-1);
	}
	Display d= getUpperTable().getDisplay();
	TableItem[] items= getUpperTable().getItems();
	for (int i= 0; i < size; i++) {
		TableItem ti= null;
		if (i < itemCount) {
			ti= items[i];
		} else {
			ti= new TableItem(getUpperTable(), i);
		}
		ti.setText(renderedStrings[indices[i]]);
		Image img= classLabelProvider.getImage(elements[indices[i]]);
		if (img != null)
			ti.setImage(img);
	}
	if (getUpperTable().getItemCount() > 0)
		getUpperTable().setSelection(0);
	getUpperTable().setRedraw(true);
	handleUpperTableSelectionChanged();
}

	/*
	 * elm - for level of 0, the input SHOULD be an IType
	 * got from CodegenTypeResolver.getFQNforType()
	 */
	public static String getFQNforType(IJavaElement elm, int level){
		if(elm instanceof IClassFile){
			IClassFile cf = (IClassFile) elm;
			String className = elm.getElementName();
			className = className.substring(0,className.lastIndexOf(".class")); //$NON-NLS-1$
			return getFQNforType(elm.getParent(),level+1) + className;
		}else if(elm instanceof ICompilationUnit){
			return getFQNforType(elm.getParent(),level+1);
		}else if(elm instanceof IPackageFragment){
			return elm.getElementName()+"."; //$NON-NLS-1$
		}else if(elm instanceof IPackageFragmentRoot){
			return ""; //$NON-NLS-1$
		}else if(elm instanceof BinaryType){
			return getFQNforType(elm.getParent(), level+1);
		}else if(elm instanceof SourceType){
			if(level==0)
				return getFQNforType(elm.getParent(), level+1) + elm.getElementName();
			else
				return getFQNforType(elm.getParent(), level+1) + elm.getElementName()+"$"; //$NON-NLS-1$
		}else if(elm instanceof IJavaProject){
			return ""; //$NON-NLS-1$
		}
		return ""; //$NON-NLS-1$
	}

protected String getFullSelectionName(){
	if(getLowerTable().getSelectionIndex()<0 || packagerMap[getLowerTable().getSelectionIndex()].intValue()<0)
		return null;
	TypeInfo packageInfo = elements[packagerMap[getLowerTable().getSelectionIndex()].intValue()];
	String packageName = packageInfo.getPackageName();
	String falseFQN = packageInfo.getFullyQualifiedName();
	String correctResolve = null;
	if(mainDialog.file!=null && mainDialog.file instanceof CompilationUnit){
		String[][] ret = null;
		try{
			IType[] types =((CompilationUnit)mainDialog.file).getTypes();
			if(types!=null && types.length>0)
				ret = types[0].resolveType(falseFQN);
		}catch(JavaModelException e){}
		if(ret!=null && ret.length>0){
			correctResolve = ret[0][0]+"."+ret[0][1].replace('.','$'); //$NON-NLS-1$
		}
	}
	if(correctResolve==null){
		IType type = null;
		try{
			type = mainDialog.project.findType(falseFQN);
		}catch(JavaModelException e){
			type = null;
		}
		if(type!=null)
			correctResolve = getFQNforType(type,0);
	}
	if(correctResolve==null){
			String className = new TypeInfoLabelProvider(TypeInfoLabelProvider.SHOW_TYPE_ONLY).getText(packageInfo);
			boolean isContainerPackage = packageInfo.getPackageName().equals(packageInfo.getTypeContainerName());
			String fullName = null;
			if(isContainerPackage){
				if(packageName.length()==0) 
					fullName = className;
				else
					fullName = packageName + "." + className; //$NON-NLS-1$
			}else{
				packageName = packageInfo.getTypeContainerName();
				fullName = packageName + "$" + className; //$NON-NLS-1$
			}
			correctResolve = fullName;
	}
	return correctResolve;
}

protected void updateStatus(){
	String fullName = getFullSelectionName();
	JBCFPlugin.log(ChooseBeanMessages.getString("SelectionAreaHelper.LogMessage.SelectedClassCreation")+fullName, JBCFMsgLogger.LOG_FINEST); //$NON-NLS-1$
	if(isInstantiatable()){
		Object createdObject = createInstance(fullName);
		message = fullName;
	}else{
		message = new String(ChooseBeanMessages.getString("SelectionAreaHelper.StatusMessage.ErrorInstantiating")); //$NON-NLS-1$
	}
	mainDialog.updateStatus();
}

public String getStatusMessage(){
	if(isInstantiatable())
		return message;
	else
		return message + secondaryMessage;
}

}