package com.ibm.etools.jbcf.choosebean;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import com.ibm.etools.java.*;
import com.ibm.etools.proxy.*;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.IBeanProxyDomain;
import com.ibm.etools.java.instantiation.base.*;

public class ChooseBeanDialogSerializedFileHelper
	implements IChooseBeanHelper {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected ChooseBeanDialog mainDialog = null;
protected Text typedText;
protected Button radioConstructor;
protected Button radioInstantiateUsing;
protected Text instantiateUsingText;
protected String message, mainText, iuText;

public ChooseBeanDialogSerializedFileHelper(ChooseBeanDialog mainDialog){
	this.mainDialog = mainDialog;
}

public void fillArea(Composite parentArea){
	Group fileSelectionGroup = new Group(parentArea, SWT.SHADOW_ETCHED_IN);
	fileSelectionGroup.setText(ChooseBeanMessages.getString("SerializedFileHelper.GroupTitle")); //$NON-NLS-1$
	GridLayout groupLayout = new GridLayout();
	groupLayout.numColumns = 3;
	fileSelectionGroup.setLayout(groupLayout);
	GridData groupLayoutData = new GridData(GridData.FILL_HORIZONTAL);
	fileSelectionGroup.setLayoutData(groupLayoutData);
	fillFileSelectionArea(fileSelectionGroup);
}

protected void fillFileSelectionArea(Composite parentArea){
	typedText = new Text(parentArea, SWT.SINGLE|SWT.BORDER);
	GridData typedData = new GridData(GridData.FILL_HORIZONTAL);
	typedData.horizontalSpan=3;
	typedText.setLayoutData(typedData);
	typedText.addModifyListener(new ModifyListener(){
		public void modifyText(ModifyEvent evt){handleTypedTextChange();}
	});
	
	// Disabled for the time being, as the code is not there to 
	// support the actions. 
	radioConstructor = new Button(parentArea, SWT.RADIO);
	radioConstructor.setText(ChooseBeanMessages.getString("SerializedFileHelper.SelectType.Constructor")); //$NON-NLS-1$
	////GridData consData = new GridData(GridData.FILL_HORIZONTAL);
	////consData.horizontalSpan=3;
	////radioConstructor.setLayoutData(consData);
	radioConstructor.addSelectionListener(new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent e){handleRadioConstructorChange();}
		public void widgetSelected(SelectionEvent e){handleRadioConstructorChange();}
	});
	
	radioInstantiateUsing = new Button(parentArea, SWT.RADIO);
	radioInstantiateUsing.setText(ChooseBeanMessages.getString("SerializedFileHelper.SelectType.BeansInstantiateUsing")); //$NON-NLS-1$
	////GridData instData = new GridData(GridData.FILL_HORIZONTAL);
	////instData.horizontalSpan=3;
	////radioInstantiateUsing.setLayoutData(instData);
	radioInstantiateUsing.addSelectionListener(new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent e){handleRadioInstantiateChange();}
		public void widgetSelected(SelectionEvent e){handleRadioInstantiateChange();}
	});
	
	instantiateUsingText = new Text(parentArea, SWT.SINGLE|SWT.BORDER);
	////GridData instTextData = new GridData(GridData.FILL_HORIZONTAL);
	//////instTextData.horizontalAlignment=GridData.END;
	////instTextData.horizontalSpan=2;
	////instantiateUsingText.setLayoutData(instTextData);
	instantiateUsingText.addModifyListener(new ModifyListener(){
		public void modifyText(ModifyEvent evt){handleInstantiateUsingTextChange();}
	});
	
	radioConstructor.setSelection(true);
	instantiateUsingText.setEnabled(false);
	
	//// Diabled for the time being, as there is no support for the actions.
	radioConstructor.setVisible(false);
	radioInstantiateUsing.setVisible(false);
	instantiateUsingText.setVisible(false);
}

public String getStatusMessage(){
	if(message==null)
		message = new String(""); //$NON-NLS-1$
	return message;
}

protected Object getInstanceOf(String className){
	// When we can get EditDomain, then this accessing of the adapter factory and to get resource set won't be needed. This is too delicate here.
	try{
	com.ibm.etools.emf.notify.AdapterFactory adFact = mainDialog.resourceSet.getContext().getAdapterFactory(com.ibm.etools.jbcf.IBeanProxyHost.BEAN_PROXY_TYPE);
		if(adFact instanceof IBeanProxyDomain){
			IBeanProxyDomain dom = (IBeanProxyDomain)adFact;
			IStandardBeanTypeProxyFactory typeFact = dom.getProxyFactoryRegistry().getBeanTypeProxyFactory();
			IBeanTypeProxy beansTypeProxy = typeFact.getBeanTypeProxy("java.beans.Beans"); //$NON-NLS-1$
			IMethodProxy methodProxy = beansTypeProxy.getMethodProxy("instantiate", new String[]{"java.lang.ClassLoader", "java.lang.String"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				IBeanProxy creationStringProxy = dom.getProxyFactoryRegistry().getBeanProxyFactory().createBeanProxyWith(className);
				IBeanProxy proxy = methodProxy.invoke(beansTypeProxy, new IBeanProxy[]{null, creationStringProxy});
				IJavaInstance inst = com.ibm.etools.jbcf.BeanProxyUtilities.wrapperBeanProxy(proxy, mainDialog.resourceSet, true);
				return inst;
		}
	}catch(Exception e){;}
	return null;
}

public Object[] getResults(Object[] results){
	if(mainText==null || !mainText.equalsIgnoreCase((String)results[0]))
		return null;
	Object[] newResults = new Object[2];
	if(iuText.equalsIgnoreCase("")){ //$NON-NLS-1$
		try{
			IJavaObjectInstance obj = (IJavaObjectInstance)getInstanceOf((String)results[0]);
			if(obj!=null){
				newResults[0]=obj;
				newResults[1]=obj.getJavaType();
			}
		}catch(Exception e){JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);}
	}
	if(!iuText.equalsIgnoreCase("")){ //$NON-NLS-1$
		EClassifier classifier = Utilities.getJavaClass(mainText, mainDialog.resourceSet);
		try{
			IJavaObjectInstance obj1 = (IJavaObjectInstance)getInstanceOf((String)results[0]);
			IJavaObjectInstance obj2 = (IJavaObjectInstance)getInstanceOf(iuText);
			newResults[0]=obj2;
			newResults[1]=obj1!=null?obj1.getJavaType():null;
		}catch(Exception e){JBCFPlugin.log(e,JBCFMsgLogger.LOG_WARNING);}
	}
	return newResults;		
}

protected void handleTypedTextChange(){
	updateStatus();
}

protected void handleRadioConstructorChange(){
	if(radioConstructor.getSelection())
		instantiateUsingText.setEnabled(false);
}

protected void handleRadioInstantiateChange(){
	if(radioInstantiateUsing.getSelection())
		instantiateUsingText.setEnabled(true);
}

protected void handleInstantiateUsingTextChange(){
	updateStatus();
}

public boolean isInstantiatable(){
	if(radioConstructor.getSelection())
		return !typedText.getText().equalsIgnoreCase(""); //$NON-NLS-1$
	if(radioInstantiateUsing.getSelection())
		return !typedText.getText().equalsIgnoreCase("") && !instantiateUsingText.getText().equalsIgnoreCase(""); //$NON-NLS-1$ //$NON-NLS-2$
	return false;
}

public void selectDefault(){
	typedText.setText(""); //$NON-NLS-1$
}

protected void updateStatus(){
	if(isInstantiatable()){
		mainText = typedText.getText();
		iuText = instantiateUsingText.getText();
		message = new String(typedText.getText());
	}
	mainDialog.updateStatus();
}

}