package com.ibm.etools.jbcf.choosebean;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jdt.core.search.*;
import org.eclipse.core.resources.IWorkspace;
import java.util.*;
import org.eclipse.jface.operation.*;
import org.eclipse.core.runtime.IProgressMonitor;
import java.lang.reflect.InvocationTargetException;

import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.internal.core.hierarchy.*;
import org.eclipse.jdt.internal.corext.util.TypeInfo;
import org.eclipse.jdt.ui.IJavaElementSearchConstants;

/*
 * ChooseBeanSearchEngine: Creating a new ChooseBeanSearchEngine will
 * create a snapshot of all the classes in the classpath. If the classpath
 * is updated and the search engine should reflect the changem a new 
 */
public class ChooseBeanSearchEngine { 

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected IWorkspace workspace;
public final static int 
	TYPE_ALL = 1, 
	TYPE_SWING = 2, 
	TYPE_AWT=4;
protected List allTypes = null;
protected List typeSWING = null;
protected List typeAWT = null;

public ChooseBeanSearchEngine(IWorkspace workspace){
	this.workspace = workspace;
}
	
private void doSearchTypes(final List typesFound, IJavaSearchScope scope, int style, IProgressMonitor pm) {
	ITypeNameRequestor requestor = new ITypeNameRequestor(){
		public void acceptClass(char[] packageName, char[] simpleTypeName, char[][] enclosingTypeNames, String path){
			typesFound.add(new TypeInfo(packageName, simpleTypeName, enclosingTypeNames, path, false));
		}
		public void acceptInterface(char[] packageName, char[] simpleTypeName, char[][] enclosingTypeNames, String path){
			typesFound.add(new TypeInfo(packageName, simpleTypeName, enclosingTypeNames, path, true));
		}
	};	
	try {
		new SearchEngine().searchAllTypeNames(
			workspace,
			null,
			null,
			IJavaSearchConstants.PATTERN_MATCH,
			IJavaSearchConstants.CASE_INSENSITIVE,
			computeKind(style),
			scope,
			requestor,
			IJavaSearchConstants.WAIT_UNTIL_READY_TO_SEARCH,
			pm);
	} catch(JavaModelException e){
		JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
	}
}

private static int computeKind(int style) {
	int kind= IJavaSearchConstants.INTERFACE;
	if ((style & IJavaElementSearchConstants.CONSIDER_INTERFACES) != 0 && (style & IJavaElementSearchConstants.CONSIDER_CLASSES) != 0) 
		kind= IJavaSearchConstants.TYPE;
	else if ((style & IJavaElementSearchConstants.CONSIDER_CLASSES) != 0)
		kind= IJavaSearchConstants.CLASS;
	return kind;
}

public List searchTypes(final IJavaSearchScope scope, final int style) {
	if(allTypes==null){
		// update only once.
		allTypes = new ArrayList();
		doSearchTypes(allTypes, scope, IJavaElementSearchConstants.CONSIDER_CLASSES, null);
	}
	List typesInterested = null;
	switch(style){
		case TYPE_SWING:
			if(typeSWING==null){
				typeSWING = updateSubClasses(allTypes, scope, "JComponent", "javax.swing", null, true); //$NON-NLS-1$ //$NON-NLS-2$
			}
			typesInterested = typeSWING;
			break;
		case TYPE_AWT:
			if(typeAWT==null){
				typeAWT = updateSubClasses(allTypes, scope, "Component", "java.awt", "javax.swing", true); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				typeAWT.addAll(updateSubClasses(typeAWT, scope, "JComponent", "javax.swing", null, false)); //$NON-NLS-1$ //$NON-NLS-2$
			}
			typesInterested = typeAWT;
			break;
		case TYPE_ALL:
			typesInterested = allTypes;
		default:
			break;
	};
	return typesInterested;
}

private List updateSubClasses(
	List typesFound, 
	IJavaSearchScope scope, 
	String baseClassName,
	String basePackageName,
	String excludingPackage,
	boolean retainSubclasses){
	
	List interestedTypes = new ArrayList();
	TypeInfo componentRef = null;
	for(int i=0;i<typesFound.size();i++){
		TypeInfo ref = (TypeInfo)typesFound.get(i);
		if(ref.getTypeName().equals(baseClassName) && ref.getPackageName().equals(basePackageName)){
			componentRef = ref;
			break;
		}
	}
	if(componentRef!=null){
		try{
			interestedTypes.addAll(typesFound);
			IType componentType = componentRef.resolveType(scope);
			TypeHierarchy hierarchy = new TypeHierarchy(componentType, scope, true);
			IndexBasedHierarchyBuilder builder = new IndexBasedHierarchyBuilder(hierarchy, scope);
			builder.build(true);
			IType[] subTypes = hierarchy.getAllSubtypes(componentType);
			Hashtable hash = new Hashtable();
			List marked = new ArrayList();
			for(int cmp=0;cmp<subTypes.length;cmp++)
				if(excludingPackage==null)
					hash.put(subTypes[cmp].getFullyQualifiedName(), new Boolean(true));
				else
					if(!subTypes[cmp].getFullyQualifiedName().startsWith(excludingPackage))
						hash.put(subTypes[cmp].getFullyQualifiedName(), new Boolean(true));
			for(int cmp=0;cmp<interestedTypes.size();cmp++)
				if(hash.containsKey(((TypeInfo)interestedTypes.get(cmp)).getFullyQualifiedName()))
					marked.add(interestedTypes.get(cmp));
			marked.add(componentRef);
			if(retainSubclasses)
				interestedTypes.retainAll(marked);
			else
				interestedTypes.removeAll(marked);
		}catch(Exception e){JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);}
	}
	return interestedTypes;
}

}