package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.resource.*;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.impl.ResourceSetImpl;
import com.ibm.etools.emf.resource.impl.ContextImpl;
import com.ibm.etools.emf.resource.impl.WorkbenchURIConverterImpl;
import com.ibm.etools.emf.ref.Extent;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import java.io.*;

import com.ibm.etools.jbcf.codegen.java.ISourceTranslatorListener;
import com.ibm.etools.jbcf.codegen.java.ISynchronizerListener;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import com.ibm.etools.cdm.*;
import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.jbcf.editorpart.IJVEStatus;
import com.ibm.etools.jbcf.emf.vce.*;


/**
 *  The AbstractModelBuilder class will provide the generic OCM handling.
 *  Subclasses can provide specific flavors for source type etc.
 */
public abstract class AbstractModelBuilder implements IDiagramModelBuilder { 
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	final static String defaultFileName = "NEWfile" ; //$NON-NLS-1$
	
	
	BeanSubclassComposition		 fRoot = null ;
	Resource		             fResource = null ;
	ResourceSet		             fResourceSet = null ;
	IDiagramSourceDecoder          fSourceDecoder = null ;	
    IJVEStatus                 fMsgRenderer = null ;
	
	IFile			 fFile ;
	String		 fUri ;


/**
 * Add Transient error listeners
 */
public void addTransientErrorListener(ITransientErrorListener listener){
	if(fSourceDecoder!=null && listener!=null)
		fSourceDecoder.addTransientErrorListener(listener);
}

/**
 *  Remove Transient error listeners
 */
public void removeTransientErrorListener(ITransientErrorListener listener){
	if(fSourceDecoder!=null && listener!=null)
		fSourceDecoder.removeTransientErrorListener(listener);
}

/**
 *   Get the current RS.
 */
public   ResourceSet  getModelResourceSet () {
	 return fResourceSet ;
}

/**
 * 
 */
 
public void setResourceSet(ResourceSet rs) {
	fResourceSet = rs ;
}

/**
 * Create a new ResourceSet, if one is not already created.
 */
protected void  createResourceSet()  throws Exception {

	if (fResourceSet == null) {
	  fResourceSet = new ResourceSetImpl();
	  Context context = new ContextImpl();
	  // Since we are working with IFiles,
	  context.setURIConverter(new WorkbenchURIConverterImpl());
	  fResourceSet.setContext(context);
	  
	  initializeResourceSet() ;	  
	}	  	
}

/**
 *   Implementors should hook up this document, to other models
 *   (e.g., java model).
 */
protected abstract void initializeResourceSet () throws Exception ;

protected abstract void resolveDecoder (String fileExt) throws Exception ;


/**
 * Will create the proper MOF structure, and root element
 */
public void	createEmptyComposition() throws Exception {
		
	if (fUri == null) throw new Exception ("Model URI is not set") ;	 //$NON-NLS-1$
	
	
	if (fResourceSet == null)
	    createResourceSet() ;
	Resource cr = fResourceSet.getResource(fUri) ;
	if (cr != null)
	  fResourceSet.remove(cr) ;
		
	fResource = ResourceFactoryRegister.getFactory(fUri, fResourceSet).makeResource(fUri);
	Extent ext = fResource.getExtent();
	fResourceSet.add(fResource);	
	// OCMModelConstants caches RefRegister.getPackage(OCMPackage.packageURI-"ocm.xmi");	
    BeanSubclassComposition composition = ((VcePackage) RefRegister.getPackage(VcePackage.packageURI)).getVceFactory().createBeanSubclassComposition();
    
    ext.add(composition);    
    addEmptyDiagram() ;      		
}


public void	addEmptyDiagram()  {
    
    BeanSubclassComposition composition = getModelRoot() ;    
    CDMFactory fact = ((CDMPackage) RefRegister.getPackage(CDMPackage.packageURI)).getCDMFactory();
	Diagram d = fact.createDiagram();
	d.refSetID(Diagram.PRIMARY_DIAGRAM_ID);
	composition.getDiagrams().add(d);    
}

public Diagram getDiagram() {
    return (Diagram) getModelRoot().getDiagrams().get(0) ;
}


/**
 * Get the composition root
 */
public BeanSubclassComposition getModelRoot(){
	if (fRoot != null) return fRoot ;	


		 
	if (fResource != null)  {
		fRoot = (BeanSubclassComposition) fResource.getExtent().getObjectByType(CDEUtilities.CDM_PACKAGE.getDiagramData()) ;	
	}

	return fRoot ;
}

/**
 * get the MOF Resource
 */
public Resource getModelResource(){
	return fResource ;
}


/**
 * Set a new model document file, this function will drive all the configuration
 * changes that are needed to deal with a new document
 */
public void setInputResource(IFile file) throws Exception {
	
	if (file == null) throw new Exception ("null Input File") ; //$NON-NLS-1$
	clearModel() ;
	
	fFile = file ;
    fUri = fFile.getFullPath().toString() ;
	
	
	createResourceSet() ;  // Generate a new ResourceSet ;
	
	resolveDecoder(fFile.getFileExtension()) ;
	      
	if (fSourceDecoder == null)  throw new Exception ("Invalid File Type") ; //$NON-NLS-1$
		   
}


/**
 * Erase the current Model
 */
public void clearModel(){

com.ibm.etools.cde.CDEHack.fixMe("May need to do more here") ; //$NON-NLS-1$
	fResource = null ;
	fRoot = null ;
	fResourceSet = null ;	
	fSourceDecoder = null ;
	fFile = null ;
	fUri = null ;
}

/**
 *
 */
public void saveModel(IProgressMonitor pm) throws Exception {
	fSourceDecoder.saveDocument(pm) ;			
}


/**
 * load the model from a file
 */
public void  loadModel(IProgressMonitor pm) throws Exception {
	
	if (fSourceDecoder == null) throw new Exception ("Invalid Source Decoder") ; //$NON-NLS-1$

    // Some decoders may need help in creating the Composition Element	
	if (fSourceDecoder.shouldCreateRoot() && fResource == null)
	   createEmptyComposition() ;
	   	        
    fResource = fSourceDecoder.decodeDocument (this , fUri, fFile,pm) ;	
    fSourceDecoder.addTransientErrorListener(new DefaultTransientErrorHandler(fMsgRenderer));
}
     


/**
 *  load the model from a given file file
 */
public void loadModel  (IFile file,IProgressMonitor pm) throws Exception {
	setInputResource(file) ;
	loadModel(pm) ;	
}


/**
 * Denote that a compound operation is completed.  This
 * will actually remove code for deleted components, if needed,
 * and will flush the changes to the Java Editor.
 */
public void commitAndFlush(boolean canWait) {   
    // Overide if needed 
}
/**
 * Async. version of commitAndFlush
 * 
 * @param listener implements the call back
 * @param marker is an optional designator of the call back.
 */
public void commitAndFlush(ISynchronizerListener listener, String marker) {
    // Overide if needed
}

/**
 * @param listener for Source Deocder events
 */	
public void addTranslatorListener (ISourceTranslatorListener listener) {
    // Overide if needed
}

public void removeTranslatorListener (ISourceTranslatorListener listener) {
    // Overide if needed
}

/**
 * @param delay will set up the delay time for JavaSourceSynchronizer
 *         value less than JavaSourceSynchronizer.DEFAULT_SYNC_DELAY will be ignored.
 */
public void setSynchronizerSyncDelay(int delay) {
   // Overide if needed
}

public void setMsgRenderer (IJVEStatus mr) {
    fMsgRenderer = mr ;
}


	/**s
	 * @see IDiagramModelBuilder#disconnect(boolean)
	 */
	public void disconnect(boolean clearModel) {
	}

	/**
	 * @see IDiagramModelBuilder#reconnect(IEditorInput)
	 */
	public void reconnect(IFileEditorInput input,IProgressMonitor pm) throws CodeGenException{
	}
	public void setEditDomain(EditDomain d) {
	}

}


