package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.io.IOException;
import java.util.*;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;
import org.eclipse.ui.part.EditorActionBarContributor;
import org.eclipse.ui.texteditor.IStatusField;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.IModelChangeController;
import com.ibm.etools.cde.OutlineBorder;
import com.ibm.etools.cde.emf.EMFContextMenuProvider;
import com.ibm.etools.common.command.CommandStack;
import com.ibm.etools.draw2d.ColorConstants;
import com.ibm.etools.draw2d.DelegatingLayout;
import com.ibm.etools.draw2d.FigureUtilities;
import com.ibm.etools.draw2d.Graphics;
import com.ibm.etools.draw2d.IFigure;
import com.ibm.etools.draw2d.Label;
import com.ibm.etools.draw2d.Layer;
import com.ibm.etools.draw2d.LayeredPane;
import com.ibm.etools.draw2d.Locator;
import com.ibm.etools.draw2d.SimpleEtchedBorder;
import com.ibm.etools.draw2d.geometry.Dimension;
import com.ibm.etools.draw2d.geometry.Insets;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.draw2d.geometry.Rectangle;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.gef.*;
import com.ibm.etools.gef.editparts.LayerManager;
import com.ibm.etools.gef.ui.palette.PaletteViewerImpl;
import com.ibm.etools.gef.ui.parts.SelectionSynchronizer;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.codegen.java.ISourceTranslatorListener;
import com.ibm.etools.jbcf.codegen.java.ISynchronizerListener;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.editorpart.*;
import com.ibm.etools.jbcf.vce.VCEPreferencePage;
import com.ibm.etools.jbcf.visual.VCEVisualGraphicalEditorPart;
import com.ibm.etools.draw2d.AbstractBorder;
import com.ibm.etools.gef.ui.actions.EditorPartAction;

public class CodeGenVisualGraphicalEditorPart extends VCEVisualGraphicalEditorPart {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IDiagramModelBuilder fDiagramBuilder = null;
	protected IActionBars fActionBars = null;
	protected CommandStack fCommandStack;
	protected StatusRenderer fStatus = new StatusRenderer();
	protected JVEStatusChangeListener statusChangeListener = new JVEStatusChangeListener();
	/** The map of the editor's status fields */
	protected Map fStatusFields;
	protected EditDomain fEditDomain = null;
	protected Boolean fisXMLTextView = null;
	protected JavaVisualEditorPart fJavaVisualEditorPart;
	/**
	 * This class is responsible to keep the status bar in sync. with the current editor
	 */
	class StatusRenderer implements IJVEStatus, MouseListener {

		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		int fState = JVE_CODEGEN_STATUS_OUTOFSYNC;
		Display fDisplay = null;
		int fPendingCounter = 0;
		int fPauseProcessing = 0;
		ArrayList fStatusListeners = new ArrayList();

		/**
		* @see IJVEStatus#showMsg(String, int)
		*/
		public void showMsg(final String msg, int kind) {

			if (fActionBars == null) {
				if (getEditorSite().getActionBarContributor() != null)
					fActionBars = ((EditorActionBarContributor) getEditorSite().getActionBarContributor()).getActionBars();
				if (fActionBars == null)
					return;
			}
			final org.eclipse.swt.graphics.Image image;
			if (kind == ERROR_MSG)
				image = IBeanProxyHost.ErrorType.getSevereErrorImage();
			else if (kind == WARNING_MSG)
				image = IBeanProxyHost.ErrorType.getWarningErrorImage();
			else
				image = null;

			if (msg != null)
				JBCFPlugin.log(msg, JBCFMsgLogger.LOG_FINE);
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					fActionBars.getStatusLineManager().setMessage(image, msg);
				}
			});

		}

		/**
		 * @see com.ibm.etools.jbcf.codegen.ICodeGenStatus#getStatus()
		 */
		public int getState() {
			return fState;
		}

		public void refreshStatus(final int oldState) {
			if (fStatusFields == null || fDisplay == null) {
				JBCFPlugin.log("CodeGenVisualGraphicalEditorPart$StatusRenderer: can not refresh status", JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$
				return;
			}
			if (fDisplay.equals(Display.getCurrent()))
				primRefreshStatus(oldState);
			else
				fDisplay.asyncExec(new Runnable() {
				public void run() {
					primRefreshStatus(oldState);
				}
			});
		}
		private void primRefreshStatus(int oldState) {

			/*
			 * 
			 * Need to clean up this renderer big time, and make the following code be a listener.
			 * 
			 */
			IStatusField textField = (IStatusField) fStatusFields.get(JavaVisualEditorActionContributor.STATUS_CATEGORY_SYNC_STATUS);
			IJVEActionField actionField = (IJVEActionField) fStatusFields.get(JavaVisualEditorActionContributor.STATUS_CATEGORY_SYNC_ACTION);

			if (textField != null) {
				String msg = null;
				if (isStatusSet(JVE_CODEGEN_STATUS_OUTOFSYNC))
					msg = JavaVisualEditorActionContributor.STATUS_MSG_SYNC_STATUS_OUTOFSYNC;
				else
					msg = JavaVisualEditorActionContributor.STATUS_MSG_SYNC_STATUS_INSYNC;

				if (isStatusSet(JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS) || isStatusSet(JVE_CODEGEN_STATUS_RELOAD_PENDING))
					msg = JavaVisualEditorActionContributor.STATUS_MSG_SYNC_STATUS_REALOAD;
				else if (isStatusSet(JVE_CODEGEN_STATUS_SYNCHING))
					msg = JavaVisualEditorActionContributor.STATUS_MSG_SYNC_STATUS_SYNCHING;
				else if (isStatusSet(JVE_CODEGEN_STATUS_PAUSE))
					msg = JavaVisualEditorActionContributor.STATUS_MSG_SYNC_STATUS_PAUSE;

				if (isStatusSet(JVE_CODEGEN_STATUS_PARSE_ERRROR))
					msg = JavaVisualEditorActionContributor.STATUS_MSG_SYNC_STATUS_PARSE_ERROR;

				textField.setText(msg);

				synchronized (this) {
					Iterator itr = fStatusListeners.iterator();
					while (itr.hasNext()) {
						IJVEStatusChangeListener element = (IJVEStatusChangeListener) itr.next();
						try {
							element.statusChanged(oldState, getState());
						} catch (Throwable t) {
							JBCFPlugin.log(t);
						}
					}
				}
			}

			if (actionField != null) {
				actionField.setPauseListener(this);

				// Update Arrows color
				if (isStatusSet(JVE_CODEGEN_STATUS_UPDATING_SOURCE)) {
					actionField.setModel2Src(true);
				} else {
					actionField.setModel2Src(false);
				}

				if (isStatusSet(JVE_CODEGEN_STATUS_UPDATING_JVE_MODEL)) {
					actionField.setSrc2Model(true);

				} else {
					actionField.setSrc2Model(false);
				}
				// Update Buttom's play/pause
				if (isStatusSet(JVE_CODEGEN_STATUS_PARSE_ERRROR))
					actionField.setError(true);
				else {
					if (isStatusSet(JVE_CODEGEN_STATUS_PAUSE)) {
						actionField.setPause(true);
					} else {
						actionField.setPause(false);
					}
				}
			}
		}

		public synchronized void refreshStatus(Display disp) {
			if (disp != null)
				fDisplay = disp;
			refreshStatus(0);
		}

		/**
		 * @see com.ibm.etools.jbcf.codegen.ICodeGenStatus#isStatusSet(int)
		 */
		public synchronized boolean isStatusSet(int state) {
			return ((getState() & state) == state);
		}

		/**
		 * @see com.ibm.etools.jbcf.codegen.ICodeGenStatus#setStatus(int, boolean)
		 */
		public synchronized void setStatus(int flag, boolean state) {
			int oldState = getState();
			if (state)
				fState = getState() | flag;
			else
				fState = getState() & (~flag);
			if (oldState != getState())
				refreshStatus(oldState);
		}

		/**
		 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(MouseEvent)
		 */
		public void mouseDoubleClick(MouseEvent e) {
		}
		public void mouseDown(MouseEvent e) {
		}
		public void mouseUp(MouseEvent e) {
			if (fPauseProcessing > 0)
				return;

			boolean flag;
			if (fStatus.isStatusSet(JVE_CODEGEN_STATUS_PARSE_ERRROR))
				return;
			if (fStatus.isStatusSet(JVE_CODEGEN_STATUS_PAUSE))
				flag = false;
			else
				flag = true;
			try {
				fPauseProcessing++;
				getDiagramBuilder().pauseRoundTripping(flag);
			} catch (Throwable t) {
				JBCFPlugin.log(t);
			} finally {
				fPauseProcessing--;
				if (fPauseProcessing < 0)
					fPauseProcessing = 0;
			}
		}

		/**
		 * @see com.ibm.etools.jbcf.codegen.ICodeGenStatus#setReloadPending(boolean)
		 */
		public synchronized boolean setReloadPending(boolean flag) {
			if (flag)
				fPendingCounter++;
			else if (fPendingCounter > 0)
				fPendingCounter--;
			if (fPendingCounter > 0) {
				setStatus(JVE_CODEGEN_STATUS_RELOAD_PENDING, true);
				return true;
			} else {
				setStatus(JVE_CODEGEN_STATUS_RELOAD_PENDING, false);
				return false;
			}
		}

		/**
		 * @see com.ibm.etools.jbcf.editorpart.IJVEStatus#addStatusListener(IJVEStatusChangeListener)
		 */
		public synchronized void addStatusListener(IJVEStatusChangeListener sl) {
			if (sl != null) {
				if (!fStatusListeners.contains(sl))
					fStatusListeners.add(sl);
			}
		}

		/**
		 * @see com.ibm.etools.jbcf.editorpart.IJVEStatus#removeStatusListener(IJVEStatusChangeListener)
		 */
		public synchronized void removeStatusListener(IJVEStatusChangeListener sl) {
			if (sl != null) {
				fStatusListeners.remove(sl);
			}
		}

	}

	/**
	 * Listener to listen for code status changes.
	 * Update the graph viewer with a "Loading..." message when
	 * reload from scratch is in progress.
	 */
	public class JVEStatusChangeListener implements IJVEStatusChangeListener {
		
		protected Layer loadingFigureLayer;
		protected Label loadingFigure = null;
		protected boolean showingLoadingFigure = false;
		
		class LoadingLabel extends Label {
			LoadingLabel(){
				super(CodegenMessages.getString("CodeGenVisualGraphicalEditorPart.StatusChangeListener.loading")); //$NON-NLS-1$				
				setBorder(new AbstractBorder() {
					public Insets getInsets(IFigure figure) {
						return new Insets(2);
					}
					public boolean isOpaque() {
						return false;
					}
					public void paint(IFigure figure,Graphics graphics,Insets insets) {
						graphics.setLineWidth(1);
						graphics.setLineStyle(Graphics.LINE_SOLID);
						graphics.setXORMode(false);
						Rectangle rect = getPaintRectangle(figure, insets);
						int top    = rect.y;
						int left   = rect.x;
						int bottom = rect.bottom()-1;
						int right  = rect.right()-1;
						// Draw a gray border
						graphics.setForegroundColor(ColorConstants.gray);
						graphics.drawLine(right, bottom, right,top);
						graphics.drawLine(right, bottom, left, bottom);
						graphics.drawLine(left, top, right, top);
						graphics.drawLine(left, top, left, bottom);
						// Draw a white border inside the grey one
						top ++;
						left ++;
						right--;
						bottom--;
						graphics.setForegroundColor(ColorConstants.white);
						graphics.drawLine(right, bottom, right,top);
						graphics.drawLine(right, bottom, left, bottom);
						graphics.drawLine(left, top, right, top);
						graphics.drawLine(left, top, left, bottom);
					}
				});
			}
			public void paint(Graphics graphics){			
				// Fill the background with a light grey color
				Rectangle bounds = getBounds();
				Color oldColor = graphics.getForegroundColor();
				// Fill the center in light gray
				graphics.setForegroundColor(ColorConstants.lightGray);
				graphics.fillRectangle(bounds);
				setForegroundColor(oldColor);
				super.paint(graphics);
			}
			public void validate(){
				// During the valid pass we can set our size, becuase only now do we can we make calculations
				// based on the current label and font metrics from the GC
				Dimension textSize = FigureUtilities.getTextExtents(getText(),getFont());
				// Allow for growth on the size of the label
				Dimension newSize = textSize.getExpanded(50,20);
				// If the figure isn't already at this size then this is the first pass through
				// We should set its size and also center it in the canvas
				if ( !newSize.equals(getSize()) ) {
					setSize(newSize);
				}
			}
		};

		JVEStatusChangeListener() {
			loadingFigure = new LoadingLabel();
		}
		public void statusChanged(int oldStatus, int newStatus) {
			if ((newStatus & (ICodeGenStatus.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS | ICodeGenStatus.JVE_CODEGEN_STATUS_RELOAD_PENDING)) > 0) {
				if (!showingLoadingFigure) {
					showLoadingFigure();
				}
			} else if (showingLoadingFigure)
				removeLoadingFigure();
		}
		protected void removeLoadingFigure() {
			showingLoadingFigure = false;
			if (!loadingFigureLayer.getChildren().isEmpty()) {
				java.util.List children = loadingFigureLayer.getChildren();
				if (children.contains(loadingFigure)) {
					EditPartViewer viewer = CodeGenVisualGraphicalEditorPart.this.getPrimaryViewer();
					loadingFigureLayer.remove(loadingFigure);
					loadingFigureLayer.repaint();
					viewer.flush();
				}
			}
		}
		protected void showLoadingFigure() {
			showingLoadingFigure = true;
			final EditPartViewer viewer = CodeGenVisualGraphicalEditorPart.this.getPrimaryViewer();
			if ( loadingFigureLayer == null ) {
				loadingFigureLayer = new Layer();
				loadingFigureLayer.setLayoutManager(new DelegatingLayout());
				LayeredPane layeredPane = (LayeredPane) ((GraphicalEditPart) viewer.getRootEditPart()).getFigure();
				layeredPane.add(loadingFigureLayer);
			}
			loadingFigureLayer.add(loadingFigure, new Locator() {
				public void relocate(IFigure target) {
					// Center the figure in the middle of the canvas
					int canvasWidth = viewer.getControl().getSize().x;
					int canvasHeight = viewer.getControl().getSize().y;
					int newX = ( canvasWidth - target.getSize().width) / 2;
					int newY = ( canvasHeight - target.getSize().height ) /2;
					target.setLocation(new Point(newX,newY));
				}
			});
			loadingFigureLayer.repaint();
			viewer.flush();
		}
	}
	public CodeGenVisualGraphicalEditorPart() {
		wantResourceListener = false; // We don't want a resource listener.
	}
	public CodeGenVisualGraphicalEditorPart(CommandStack aCommandStack, JavaVisualEditorPart aJavaVisualEditorPart) {
		this();
		fCommandStack = aCommandStack;
		fJavaVisualEditorPart = aJavaVisualEditorPart;
		fStatus.addStatusListener(statusChangeListener);
	}
	protected DefaultEditDomain createEditDomain() {
		DefaultEditDomain result = super.createEditDomain();
		if (fCommandStack != null) {
			result.setCommandStack(fCommandStack);
		}
		try {
			getDiagramBuilder().setEditDomain((EditDomain) result);
		} catch (CoreException e) {
		}
		return result;
	}
	/*-----------------------  Java Code Gen Slam -----------------------------*/

	/**
	 * Create it if not already there
	 */
	protected IDiagramModelBuilder getDiagramBuilder() throws CoreException {
		if (fDiagramBuilder != null)
			return fDiagramBuilder;

		fDiagramBuilder = new VCEModelBuilder();
		fDiagramBuilder.setMsgRenderer(fStatus);
		fEditDomain = (EditDomain) getAdapter(EditDomain.class);
		fDiagramBuilder.setEditDomain(fEditDomain);
		return fDiagramBuilder;
	}
	public void createPaletteViewerImpl() {
		if (paletteViewer == null) {
			paletteViewer = new PaletteViewerImpl();
			getDomain().setPaletteViewer(paletteViewer);
		}
	}

	/*
	 * Accessed only by VCECompilationUnitEditor for now.
	 * Need better way of setInput coming in for all cases.
	 */
	public void resetInput(IEditorInput input) {
		// If we have no input yet, they ignore this because it will come in appropriately later.
		if (getEditorInput() == null)
			return;

		try {
			if (input == null) {
				// Else we are all set up. If new input is null, then just disconnect.
				// This is necessary because for now (until OTI fix comes) we need to be able to disconnect
				// to remove our usage of the document so that their disconnect/reconnect will bring in a new copy.		
				getDiagramBuilder().disconnect(false);
			} else {
				// Now we want to reconnect to the new input.
				superSetInput(input);
				getDiagramBuilder().reconnect((IFileEditorInput) input, null);
			}
		} catch (CodeGenException e) {
			JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
		} catch (CoreException e) {
			JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
		}
	}
	/**
	 * 
	 * TEMP: Overide enable Java 
	 */
	protected RefObject getModel(IFile file) throws CoreException, IOException, Exception {

		if (!file.getFileExtension().equals("java")) //$NON-NLS-1$
			return super.getModel(file);

		RefObject modelRoot = null;
		try {
			getDiagramBuilder().setInputResource(file); // fCompositionModelBuilder is initialized now		  
			getDiagramBuilder().setMsgRenderer(fStatus);
			// Make sure to hook the Model with a beanProxyAdapterFactory.
			 ((VCEModelBuilder) fDiagramBuilder).setResourceSet(getResourceSet());

			if (!file.exists()) // New file
				fDiagramBuilder.createEmptyComposition();
			else {
				fDiagramBuilder.createEmptyComposition();
				// Let the editor set up the diagram, and hook up proxy adapters
				fResource = fDiagramBuilder.getModelResource();
				modelRoot = getModel(fResource);
				fDiagramBuilder.loadModel(null);
			}

		} catch (Exception exc) { // Any error deservese to be recorded
			fStatus.showMsg(exc.toString(), fStatus.ERROR_MSG);

			IModelChangeController controller = (IModelChangeController) fEditDomain.getData(IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY);
			controller.setHoldChanges(true, exc.toString());
			fStatus.setStatus(fStatus.JVE_CODEGEN_STATUS_PARSE_ERRROR, true);
			if (exc instanceof com.ibm.etools.jbcf.codegen.util.CodeGenSyntaxError)
				JBCFPlugin.log(exc.toString(), JBCFMsgLogger.LOG_FINE);
			else
				JBCFPlugin.log(exc, JBCFMsgLogger.LOG_FINE);
			//		handleException(exc);
		}
		return modelRoot;
	}
	/**
	 * Control of the GEF palette is done through an option,
	 * Even if we don't create one we do need to create a palette root
	 */
	public void createPartControl(Composite c) {

		if (((EditDomain) getDomain()).getPaletteRoot() == null) {
			setPaletteRoot();
		}
		if (VCEPreferencePage.isXMLTextOn()) {
			// If the Pref store wants the XML page put it on a sash
			SashForm sashform = new SashForm(c, SWT.VERTICAL);
			createPrimaryViewerControl(sashform);
			createXMLTextViewerControl(sashform);
			fisXMLTextView = Boolean.TRUE;
		} else {
			createPrimaryViewerControl(c);
			fisXMLTextView = Boolean.FALSE;
		}

	}

	public void refresh() {
		//	super.refresh() ;
		if (fisXMLTextView.booleanValue())
			refreshTextPage();
	}
	public void dispose() {
		try {
			getDiagramBuilder().dispose();
			fStatus.removeStatusListener(statusChangeListener);
		} catch (Exception e) {
		}
		super.dispose();
	}

	/**
	 * Initialize the project
	 * Extension point <context> defines a class that has a method that initializes stuff for the project
	 */
	//protected void initializeProject(IProject aProject){

	//	try {
	// Always try and initialize the base context for the project
	//		new JBCFBaseContext().initialize(aProject);
	//	   	new JBCFVisualContext().initialize(aProject);
	//	} catch ( CoreException exc ){
	//		JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
	//	}

	//}
	//protected IJBCFClassPath getJBCFClassPath() throws CoreException {
	//   	return new JBCFVisualClassPath();
	//}
	/**
	 * Initialize the proxy factory registry.  This is the VM that the beans are instantiated in
	 */
	//protected void initializeProxyFactoryRegistry(){
	//	new JBCFVisualEnvironment().initialize(fProxyFactoryRegistry);
	//}
	//protected String getPaletteHRef(){
	//	return "plugin.com.ibm.etools.jbcf.visual/javabeanspalette.xmi#palette";
	//	
	//}

	/**
	 * Denotes that a compound operation is completed.  This
	 * will actually remove source code of deleted components, if needed,
	 * and will flush the changes to the Java Editor.
	 * 
	 * This operation is synchroneous.
	 * @param canWait, if true will wait for the next synchronizer cycle.
	 */
	public void commitAndFlush(boolean canWait) {
		try {
			getDiagramBuilder().commitAndFlush(canWait);
		} catch (Exception e) {
		}
	}
	public void pauseRoundTripping(boolean pause) {
		try {
			getDiagramBuilder().pauseRoundTripping(pause);
		} catch (Exception e) {
		}
	}
	/**
	 * Async. version of commitAndFlush
	 * 
	 * @param listener implements the call back
	 * @param marker is an optional origin designator for the call back.
	 */
	public synchronized void commitAndFlush(ISynchronizerListener listener, String marker) {
		try {
			getDiagramBuilder().commitAndFlush(listener, marker);
		} catch (Exception e) {
		}
	}

	/**
	 * @param delay will set up the delay time for JavaSourceSynchronizer
	 *         value less than JavaSourceSynchronizer.DEFAULT_SYNC_DELAY will be ignored.
	 */
	public void setSynchronizerSyncDelay(int delay) {
		try {
			getDiagramBuilder().setSynchronizerSyncDelay(delay);
		} catch (Exception e) {
		}
	}

	public void addTranslatorListener(ISourceTranslatorListener listener) {
		try {
			getDiagramBuilder().addTranslatorListener(listener);
		} catch (CoreException e) {
		}
	}

	public void removeTranslatorListener(ISourceTranslatorListener listener) {
		try {
			getDiagramBuilder().removeTranslatorListener(listener);
		} catch (CoreException e) {
		}
	}
	protected SelectionSynchronizer getSelectionSynchronizer() {
		if (selectionSynchronizer == null)
			selectionSynchronizer = new SelectionNotifier();
		return selectionSynchronizer;
	}
	public SelectionNotifier getSelectionNotifier() {
		return (SelectionNotifier) getSelectionSynchronizer();
	}

	/*
	 *   The following will support the Status fields 
	 */

	/**
	 * This method will be called each time "this" editor becomes active.
	 * Potentialy, the Status Fields have been changed. Actually,
	 * The Controls are brand new.
	 */
	private void setStatusField(Object field, String category) {
		if (field != null && category != null) {
			if (fStatusFields == null)
				fStatusFields = new HashMap(2);

			fStatusFields.put(category, field);

			fStatus.refreshStatus(Display.getCurrent());

		} else if (fStatusFields != null)
			fStatusFields.remove(category);
	}
	public void setStatusField(IStatusField field, String category) {
		setStatusField((Object) field, category);
	}
	public void setStatusField(IJVEActionField field, String category) {
		setStatusField((Object) field, category);
	}
	public void refreshStatus() {
		fStatus.refreshStatus(fStatus.getState());
	}
	class CodeGenVisualContextMenuProvider extends EMFContextMenuProvider {
		protected ISelectionProvider fSelectionProvider;
		CodeGenVisualContextMenuProvider(CodeGenVisualGraphicalEditorPart aPart,EditorPartAction deleteAction, ISelectionProvider selectionProvider) {
			super(aPart,deleteAction);
			fSelectionProvider = selectionProvider;
		}
		public void buildContextMenu(IMenuManager menu, EditPartViewer viewer) {
			super.buildContextMenu(menu, viewer);
			RenameJavaBeanAction renameAction = new RenameJavaBeanAction(fJavaVisualEditorPart,fSelectionProvider);
			if (renameAction.isEnabled()) {
				menu.add(renameAction);
			}
		}
	}
	// Put a rename option that allows the field refactoring editor to be used directly from a selected edit part
	protected void createOutlineContextMenuProvider() {
		outlineContextMenuProvider = new CodeGenVisualContextMenuProvider(this,getOutlineDeleteAction(),getContentOutlinePage());
	}
	protected void createCanvasContextMenuProvider(){	
		canvasContextMenuProvider = new CodeGenVisualContextMenuProvider(this,getCanvasDeleteAction(),getPrimaryViewer());
	}
	/**
	 * Returns the status.
	 * @return StatusRenderer
	 */
	public StatusRenderer getStatus() {
		return fStatus;
	}

}
