package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.resource.*;
import com.ibm.etools.jbcf.codegen.java.ISourceTranslatorListener;
import com.ibm.etools.jbcf.codegen.java.ISynchronizerListener;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.editorpart.IJVEStatus;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor ;
import org.eclipse.ui.IFileEditorInput;

/**
 *   A VCE Compostion Model Builder provides an interface to an object
 *   that creates or store a composition model from/to an input file.  
 */
public interface IDiagramModelBuilder extends IDiagramModelInstance {
	
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	// Add Transient error listeners
	public void addTransientErrorListener(ITransientErrorListener listener);
	// Initialize a new Resource Set for a new file
	public void setInputResource(IFile file) throws Exception ;
	// Create a new Composition model from the input file
	public void     loadModel (IProgressMonitor pm) throws Exception ;  			
      // Save the model  
	public void	  saveModel  (IProgressMonitor pm) throws Exception ; 
	
	// Disconnect, and bring down all threads/resources used by the builder
	public void dispose() ;
	// Disconnect from external resources (e.g., Java Working Copy)
	// if clearModel is true, the VCE model will be cleared
	public void disconnect(boolean clearModel) ;  
	// Will re-load a previously disconnected model.
	public void reconnect(IFileEditorInput input, IProgressMonitor pm) throws CodeGenException ;
	
    /**
     * Denotes that a compound operation is completed.  This
     * will actually remove source code of deleted components, if needed,
     * and will flush the changes to the Java Editor.
     * 
     * This operation is synchroneous.
     */
    public void commitAndFlush(boolean canWait) ;
    /**
     * Async. version of commitAndFlush
     * 
     * @param listener implements the call back
     * @param marker is an optional origin designator for the call back.
     */
    public void commitAndFlush(ISynchronizerListener listener, String marker) ;
    
    /**
	 * @param delay will set up the delay time for JavaSourceSynchronizer
	 *         value less than JavaSourceSynchronizer.DEFAULT_SYNC_DELAY will be ignored.
	 */
	public void setSynchronizerSyncDelay(int delay) ;
			
	
	/**
	 * @param listener for Source Deocder events
	 */	
	public void addTranslatorListener (ISourceTranslatorListener listener) ;
	
	public void removeTranslatorListener (ISourceTranslatorListener listener) ;

	// Remove Transient error listeners
	public void removeTransientErrorListener(ITransientErrorListener listener);
    
    public void setMsgRenderer (IJVEStatus mr) ;
    public void setEditDomain (EditDomain d) ;
    public void pauseRoundTripping(boolean flag) throws CodeGenException ;
}


