package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.resource.*;
import java.io.InputStream;
import org.eclipse.core.resources.IFile;

import com.ibm.etools.jbcf.codegen.java.ISourceTranslatorListener;
import com.ibm.etools.jbcf.codegen.java.ISynchronizerListener;
import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.codegen.util.*;
import com.ibm.etools.jbcf.editorpart.IJVEStatus;

import org.eclipse.core.runtime.IProgressMonitor ;
import org.eclipse.ui.IFileEditorInput;

/**
 *   Implementors will build a Composition Model from a given source file. Also
 *   the model could be saved back.
 */

public interface IDiagramSourceDecoder {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public final static String XMIExt  =   "jbcf" ;     // XMI file //$NON-NLS-1$
	public final static String JAVAExt =   "java" ;     // JAVA file ; //$NON-NLS-1$
	
	// Add Transient error listeners
	public void addTransientErrorListener(ITransientErrorListener listener);
	
	// Remove Transient error listeners
	public void removeTransientErrorListener(ITransientErrorListener listener);

    // Decode the imput source
	public Resource	decodeDocument (IDiagramModelInstance cm, String uri, IFile sourceFile, IProgressMonitor pm) throws Exception ;
	// Save a document 
	public void		saveDocument (IProgressMonitor pm) throws Exception ;
	
	// Should an empty model created before calling the decode() ;
	public boolean shouldCreateRoot () ;
	
	// What file extention does this decoder works with
	public String  getFileExt() ;
	
	public void dispose() ;

	/**
	 * Denote that a compound operation is completed.  This
	 * will actually remove code for deleted components, if needed,
	 * and will flush the changes to the Java Editor.
	 * A call to this API is synchroneous.
	 * 
	 * @param canWait denotes if the call can wait for the next sync.
	 *         cycle.  if false, a flush will be instigated with no delay.
	 */
	public void commitAndFlush(boolean canWait) ;
	/**
	 * Async. version of commitAndFlush
	 *        The flush will occur on the next cycle of the synchronizer
	 * 
	 * @param listener implements the call back to be called when the flush occured.
	 * @param marker is an optional designator of the call back.
     */
	public void commitAndFlush(ISynchronizerListener listener, String marker) ;
	
	/**
	 * @param delay will set up the delay time for JavaSourceSynchronizer
	 *         value less than JavaSourceSynchronizer.DEFAULT_SYNC_DELAY will be ignored.
	 */
	public void setSynchronizerSyncDelay(int delay) ;
	
	/**
	 * @param listener for Source Deocder events
	 */	
	public void addTranslatorListener (ISourceTranslatorListener listener) ;
	
	public void removeTranslatorListener (ISourceTranslatorListener listener) ;
    
    public void setMsgRenderer (IJVEStatus mr)  ;
    
    // Disconnect from external resources (e.g., Java Working Copy)
	// if clearModel is true, the VCE model will be cleared
	public void disconnect(boolean clearModel) ;  
	// Will re-load a previously disconnected model.
	public void reconnect(IFileEditorInput input,IProgressMonitor pm) throws CodeGenException;
	public void setEditDomain (EditDomain d) ;
	public void pauseRoundTripping(boolean flag) throws CodeGenException ;
	
}


