package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import  com.ibm.etools.emf.resource.*;
import  com.ibm.etools.emf.ref.RefObject ;
import  com.ibm.etools.emf.ref.EList ;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cdm.Diagram;
import  com.ibm.etools.emf.ecore.EAttribute ;
import com.ibm.etools.emf.jbcf.BeanComposition;
import  com.ibm.etools.emf.ref.RefStructuralFeature ;
import  com.ibm.etools.jbcf.codegen.model.* ;
import  com.ibm.etools.jbcf.codegen.java.IExpressionDecoder ;
import  com.ibm.etools.jbcf.codegen.java.JavaBeanModelBuilder ;
import  com.ibm.etools.jbcf.codegen.java.* ;
import  com.ibm.etools.jbcf.codegen.util.* ;
import com.ibm.etools.jbcf.editorpart.IJVEStatus;
import com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition;

import  java.io.InputStream ;
import  org.eclipse.core.resources.IFile ;

import java.text.MessageFormat;
import  java.util.* ;
import  org.eclipse.jdt.internal.compiler.ast.* ;

import com.ibm.etools.java.JavaClass;
import  com.ibm.etools.java.impl.JavaClassImpl ;
import  com.ibm.etools.java.instantiation.base.IJavaInstance ;
import  java.io.OutputStream ;
import  org.eclipse.jdt.core.* ;

import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.JBCFPlugin;

import org.eclipse.core.runtime.IProgressMonitor ;
import org.eclipse.core.runtime.SubProgressMonitor ;
import com.ibm.etools.jbcf.codegen.java.rules.* ;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance ;
import org.eclipse.swt.widgets.Display ;
import org.eclipse.jface.text.IDocumentListener ;




public class JavaSourceTranslator implements IDiagramSourceDecoder { 
	
public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$

JavaBeanModelBuilder       fJavaModelBldr = null ;         
IBeanDeclModel             fBeanModel ;
IDiagramModelInstance      fCompositionModel = null ;
IWorkingCopyProvider       fWorkingCopy = null ;
JavaSourceSynchronizer     fSrcSync = null ;
int						fSrcSyncDelay = JavaSourceSynchronizer.DEFAULT_SYNC_DELAY ;
String                     fRSuri = null ;
IFile                      fFile = null ;
char[][]				   fPackageName = null ;
TransientErrorManager      fTransientErrorManager = new TransientErrorManager();
IJVEStatus                 fMsgRrenderer = null ;
EditDomain 			       fEDomain = null ;
final ArrayList		   fTranslatorListeners = new ArrayList() ;
private static boolean  fSnippetProcessingInProgress = false ;
boolean                   fdisconnected = true ;
boolean				   fpauseRoundTripping = false ;
boolean				   fFirstTimeDecode = true ;
public static String     fPauseSig = CodegenMessages.getString("JavaSourceTranslator.Round_Tripping_is_Paused_1") ;  //$NON-NLS-1$
public static int        fCommitAndFlushNestGuard = 0 ;





  /**
   *  This is the strategy for updating the shared Java document after
   *  a change in the local document.
   */	
  class  LocalToSharedUpdater implements IBackGroundWorkStrategy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
  	ICancelMonitor    fMonitor = null ;
  	IDocumentListener fDocListener = null ;

  	
  	public void run(Display disp, IDocumentListener docListener,
  	                Object[] items,
  	                ICancelMonitor monitor) {
  		fMonitor = monitor ;
  		fDocListener = docListener ;
  		
  		
  		if (items == null) return ;
  		
  		final List handles = new ArrayList() ;  		
  		for (int i=0 ; i<items.length; i++) {
  			if (((SynchronizerWorkItem)items[i]).isSharedToLocalUpdate()) throw new IllegalArgumentException("Shared To Local work Item") ; //$NON-NLS-1$
  			handles.add(((SynchronizerWorkItem)items[i]).getChangedElementHandle()) ;
  		}  			
  		if ((monitor!= null && monitor.isCanceled()) ||
  		    handles.isEmpty()) return ; 
  	
  		  disp.syncExec(new Runnable() {
			      public void run() {	
			      	  try {	
			      	    synchronized (fBeanModel.getDocumentLock()) {		
		                         fWorkingCopy.UpdateDeltaToShared(fMonitor,fDocListener,handles,false) ;
			      	    }
			      	  }
			      	  catch (CodeGenException e) {
		                      JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
		                    }
			      }
		  }) ;	
		  
		  if(fTransientErrorManager!=null)
		  	fTransientErrorManager.handleLocalToSharedChanged(fBeanModel,monitor);
  	}           			
  }
 
   /**
   *  This is the strategy for updating the shared Java document after
   *  a change in the local document.
   */	
  class  SharedToLocalUpdater implements IBackGroundWorkStrategy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
  	ICancelMonitor    fMonitor = null ;
  	IDocumentListener fDocListener = null ;
  	int               fCollisionCount = 0 ;
  	  	  	
  	boolean          fHold = false ;
  	String            fHoldMsg = null ;
  	int          	  fSetHold = 0 ;
  	
  	final int DELTA_SKIP 			= 1 ;
  	final int DELTA_PARSE_ERROR 	= 2 ;
  	final int DELTA_BDM_CHANGE 	= 3 ;
  	final int DELTA_COMPLEX_DELTA	= 4 ;
  	
  	
  	/**
	 * Check to see if the non method delta is important 
	 */
	private int getDeltaStatus (Object[] items) {

		if (items == null || items.length==0) return DELTA_SKIP ;
		
		// Skip changes to document areas that are not resolved - no need
		// to add them to the local document
		for (int i = 0; i < items.length; i++) {

			if (SynchronizerWorkItem.INSTANCE_ANNOTATION_HANDLE.equals(((SynchronizerWorkItem) items[i]).getChangedElementHandle())
				|| SynchronizerWorkItem.THIS_ANNOTATION_HANDLE.equals(((SynchronizerWorkItem) items[i]).getChangedElementHandle()))
				return DELTA_COMPLEX_DELTA;
			if (SynchronizerWorkItem.RELOAD_HANDLE.equals(((SynchronizerWorkItem) items[i]).getChangedElementHandle())
				/* ||
					     SynchronizerWorkItem.CLASS_IMPLEMENT_EXTENDS_HANDLE.equals(((SynchronizerWorkItem) items[i]).getChangedElementHandle())) */
				)
				// ** At this time changes to the extends/implements does not refelect in the JavaModel.. so
				//    no reason to reaload		     
				return DELTA_BDM_CHANGE;
		}

		// Take a closer look, and see if new fields, or new initMetod were added
		// Note that this will pick up changes to imports.
		JavaBeanModelBuilder modelBldr =
			new JavaBeanShadowModelBuilder(CodeGenUtil.getRefWorkingCopyProvider(fWorkingCopy.getSharedWorkingCopy()), fWorkingCopy.getFile().getLocation().toFile().toString(), null);
		modelBldr.setDiagram(fCompositionModel);
		IBeanDeclModel bdm = null;
		try {
			bdm = modelBldr.build();
		}
		catch (Exception e) {
			// Syntax error
			return DELTA_PARSE_ERROR;
		}
		if (fBeanModel == null || !fBeanModel.isStateSet(bdm.BDM_STATE_UP_AND_RUNNING) || fBeanModel.getBeans().size() != bdm.getBeans().size())
			return DELTA_BDM_CHANGE;

		ArrayList beanList = new ArrayList();
		for (int i = 0; i < fBeanModel.getBeans().size(); i++) {
			BeanPart b = (BeanPart) bdm.getBeans().elementAt(i);
			BeanPart oB = fBeanModel.getABean(b.getUniqueName());
			if (oB != null && oB.getFieldDeclHandle() != null) // Internal Beans do not have IField
				beanList.add(oB.getFieldDeclHandle());
			// If components have changed 
			if (oB == null || !oB.isEquivalent(b))
				return DELTA_BDM_CHANGE;
			// Has init method changed
			if ((b.getInitMethod() == null && oB.getInitMethod() != null) || (b.getInitMethod() != null && oB.getInitMethod() == null))
				return DELTA_BDM_CHANGE;
			if (b.getInitMethod() != null)
				if (!oB.getInitMethod().getMethodHandle().equals(b.getInitMethod().getMethodHandle()))
					return DELTA_BDM_CHANGE;
		}
		for (int i = 0; i < items.length; i++) {
			SynchronizerWorkItem elm = (SynchronizerWorkItem) items[i];
			if (elm.isField() && beanList.contains(elm.getChangedElementHandle()))
				return DELTA_BDM_CHANGE;
		}
		// Even though we compared BDMs, arguments (like Color) are not compared, and may need changes
		for (int i = 0; i < items.length; i++) {
			SynchronizerWorkItem elm = (SynchronizerWorkItem) items[i];
			if (elm.isImport()){
				ITypeResolver sharedResolver = fBeanModel;
				final CodegenTypeResolver localCodegenResolver = new CodegenTypeResolver(CodeGenUtil.getMainType(fWorkingCopy.getLocalWorkingCopy()));
				ITypeResolver localResolver = new ITypeResolver() {
					public String resolve(String unresolved) {
						return localCodegenResolver.resolveTypeComplex(unresolved);
					}
					public String resolveThis() {
						return localCodegenResolver.resolveTypeComplex(CodeGenUtil.getMainType(fWorkingCopy.getLocalWorkingCopy()).getElementName());
					}
				};
				Vector mainBeans = fBeanModel.getBeans();
				Vector deltaBeans = bdm.getBeans();
				if(mainBeans==null || deltaBeans==null || deltaBeans.size()!=mainBeans.size())
					continue;
				for(int bc=0;bc<deltaBeans.size();bc++){
					BeanPart delBean = (BeanPart) deltaBeans.get(bc);
					BeanPart mainBean = fBeanModel.getABean(delBean.getUniqueName());
					if(delBean==null || mainBean==null)
						continue; // maybe some bean was not instantiable - and hence was removed from the model.
					List delExps = new ArrayList();
					delExps.addAll(delBean.getRefExpressions());
					delExps.addAll(delBean.getBadExpressions());
					List mainExps = new ArrayList();
					mainExps.addAll(mainBean.getRefExpressions());
					mainExps.addAll(mainBean.getBadExpressions());
					if(delExps.size()!=mainExps.size())
						return DELTA_BDM_CHANGE;
					List nonCommonExpressions = new ArrayList();
					nonCommonExpressions.addAll(mainExps);
					nonCommonExpressions.addAll(delExps);
					for(int edc=0;edc<delExps.size();edc++){
						CodeExpressionRef de = (CodeExpressionRef) delExps.get(edc);
						for(int emc=0;emc<mainExps.size();emc++){
							CodeExpressionRef me = (CodeExpressionRef) mainExps.get(emc);
							try {
								if(me.isEquivalent(de)>-1){
									nonCommonExpressions.remove(me);
									nonCommonExpressions.remove(de);
									if(me.isEquivalentChanged(localResolver, de, sharedResolver))
										return DELTA_BDM_CHANGE;
								}
							} catch (CodeGenException e) {
								JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
							}
						}
					}
					if(nonCommonExpressions.size()>0)
						return DELTA_BDM_CHANGE;
				}
			}
		}
		return DELTA_SKIP;
	}  	/**
  	 *  ReLoad the BDM model from stratch
  	 */
  	private synchronized void Reload(Display disp,ICancelMonitor monitor) {
  	    try {
  			Object lock = fBeanModel== null ? new Object() : fBeanModel.getDocumentLock() ;  			  
  			fCollisionCount=0 ;
  			synchronized (lock) {
  					  if (fMsgRrenderer.setReloadPending(false) == false) // No more pending, go for it
			             reloadFromScratch(disp,monitor) ;	                               		           
			          else
			             JBCFPlugin.log("Reload: reload is pending, skipping",JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
  			}
  		}
  		catch (Throwable t) {
  				  JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING);                    
  		}      
 	}
  	
  	
  	public void run(Display disp, IDocumentListener docListener,
  	                Object[] items,
  	                ICancelMonitor monitor) {
  		fMonitor = monitor ;
  		fDocListener = docListener ;
  		
  		if (items == null || items.length == 0) return ;
  		  		
  		boolean deltaUpdate = items.length<1?false:true ;
  		
  		// Perform a delta update only if a BDM exists and the delta spans a 
  		// single method content change
  		for(int i=0;i<items.length;i++){
	  		if (((SynchronizerWorkItem)items[i]).isMethod() &&
	  		    ((SynchronizerWorkItem)items[i]).getChangedIndex()>=0 &&
	  		    ((SynchronizerWorkItem)items[i]).getSourceCode() != null &&
	  			fBeanModel!= null && 
	  			fBeanModel.isStateSet(fBeanModel.BDM_STATE_UP_AND_RUNNING)  &&
	  			!((SynchronizerWorkItem)items[i]).getChangedElementHandle().equals(SynchronizerWorkItem.THIS_ANNOTATION_HANDLE) &&
	  			!((SynchronizerWorkItem)items[i]).getChangedElementHandle().equals(SynchronizerWorkItem.RELOAD_HANDLE) 
	  			/*&&
	  			fCollisionCount == 0*/) {
	  		}else{
	  			deltaUpdate=false;
	  		}
  		}
  		  
		if (deltaUpdate) {
			// Delta Merge
			for (int i = 0; i < items.length; i++) {
				if (monitor != null && monitor.isCanceled())
					return;

				final SynchronizerWorkItem we = (SynchronizerWorkItem) items[i];
				
				IModelChangeController controller = (IModelChangeController) fEDomain.getData(IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY);				    
				try {
					we.refreshContents(fWorkingCopy.getSharedWorkingCopy());
					CodeSnippetTranslator translator =
						new CodeSnippetTranslator(
							we.getSourceCode(),
							we.getPackageName(),
							we.getExtends(),
							we.getImplements(),
							we.getImports(),
							we.getFields(),
							we.getMethods(),
							monitor,
							we.getCompilationUnit(),
							we.getMethodsHandles(),
							we.getMethodSkeletons(),
							we.getChangedIndex(),
							we.isMethod());
					synchronized (controller) {
						// It is possible that multi work elements stagger, and we do not want
						// a later work element, to cache, an UnRestored state.
						if (fSetHold++==0) {
						  fHoldMsg = controller.getHoldMsg();
						  fHold = controller.isHoldChanges();						  
						}
						controller.setHoldChanges(true, null);
				    }
					Object lock = fBeanModel == null ? new Object() : fBeanModel.getDocumentLock();
					synchronized (lock) {
						if (monitor != null && monitor.isCanceled())
							return;

						translator.setDiagram(fCompositionModel);
						final ICodeDelta delta =
							translator.generateCodeDelta(
								fBeanModel,
								we.getChangedElementHandle(),
								we.getMethodSources()[we.getChangedIndex()]);
						if (monitor != null && monitor.isCanceled())
							return;

						com.ibm.etools.cde.CDEHack.fixMe("Adapters will not react for GUI deltas !!!"); //$NON-NLS-1$
						fBeanModel.setState(fBeanModel.BDM_STATE_UPDATING_JVE_MODEL, true);
						if (delta == null || delta.getDeltaMethod() == null)
							JBCFPlugin.log("NOT Driving a delta merge: DeltaMethod==NULL", JBCFMsgLogger.LOG_FINEST); //$NON-NLS-1$
						else
							JBCFPlugin.log("Driving a delta merge: " + delta.getDeltaMethod(), JBCFMsgLogger.LOG_FINEST); //$NON-NLS-1$
						if (fBeanModel != null && delta != null && delta.getDeltaMethod() != null) {
							// Display thread
							disp.syncExec(new Runnable() {
								public void run() {
									try {
										CodeSnippetMergelet merglet =
											new CodeSnippetMergelet(
												delta,
												we.getChangedElementContent(),
												we.getChangedElementHandle(),
												we.isMethod());
										if (merglet.updateBDM(fBeanModel)) {
											fireUpdateNotification();
										}
									}
									catch (Throwable t) {
										JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING);
									}
								}
							});
						}
						if (fTransientErrorManager != null) {
							fTransientErrorManager.handleSharedToLocalChanges(fBeanModel, translator.getErrorsInCodeDelta(), monitor);
						}
						// We tried to apply the delta.  Check to see if we need
						// to reload from stratch - delta failure (e.g., method became an init, or not any more
						// and init method 		           		          
						if (fBeanModel.isStateSet(fBeanModel.BDM_STATE_DOWN)) {
							fTransientErrorManager.handleSharedToLocalChanges(fBeanModel, translator.getErrorsInCodeDelta(), monitor);
							// Note that we still hold the lock. We can cause a dead lock if we try to Reload here.
							// Reload(disp, monitor);							
						}
						else {
							// If expressions removed beans from their container
							// we need to add them to the FF
							refreshFreeFrom(disp);
							fBeanModel.setState(fBeanModel.BDM_STATE_UPDATING_JVE_MODEL, false);
						}
					}
					if (fBeanModel.isStateSet(fBeanModel.BDM_STATE_DOWN)) {
						Reload(disp, monitor);
						break ;
					}
				}
				catch (Throwable e) {
					JBCFPlugin.log(e, JBCFMsgLogger.LOG_FINE);
					// Reload from scratch will re-set the state of the BDM 		      	
					Reload(disp, monitor);
					break ;
				}
				finally {
					if (fBeanModel != null)
						fireSnippetProcessing(false);
					synchronized (controller) {
						   if (--fSetHold<=0) {
				             controller.setHoldChanges(fHold,fHoldMsg) ;
				             fSetHold = 0  ;
						   }
					}
				}
			}
		}
  		else {  // It is not a method delta
  		  try {  		  	
  			switch (getDeltaStatus(items)) {
  				case DELTA_SKIP: JBCFPlugin.log(":) Skipping Unresolved Work Element Handle",JBCFMsgLogger.LOG_FINEST) ;  //$NON-NLS-1$
  				                  break ;
  				case DELTA_COMPLEX_DELTA:
  				case DELTA_BDM_CHANGE:
  				case DELTA_PARSE_ERROR:  				   
  			                      Reload(disp,monitor) ;
  			}
  		  }
  		  catch (Throwable t) {
  		  	JBCFPlugin.log(t,JBCFMsgLogger.LOG_WARNING) ;
  		  }
          finally {
                 fireSnippetProcessing(false) ;        
          }  			
  		}
  	}	
  }
  
/**
 * Bean parts that needs to be on the FF and not, will be added
 * @param disp, if need to run async.
 */  
protected synchronized void refreshFreeFrom(Display disp) {
       
   BeanSubclassComposition ff = fCompositionModel.getModelRoot() ;   
   if (ff == null) return ;
   
   final EList ffBeans = ff.getComponents() ;
   final ArrayList     beansToAdd = new ArrayList() ;
   final ArrayList     beansToRemove = new ArrayList() ;
   
   Iterator itr = fBeanModel.getRootBeans().iterator() ;
   while (itr.hasNext()) {
       BeanPart bp = (BeanPart) itr.next() ;
       if (bp.getRefObject() != null && bp.isInstanceVar() &&
           (bp.getRefObject().refContainer() == null ||
           !bp.getRefObject().refContainer().equals(ff))
          )
          beansToAdd.add(bp.getRefObject()) ;       
   }   
   itr = ffBeans.iterator() ;
   while (itr.hasNext()) {
		RefObject c = (RefObject) itr.next();
		BeanPart bp = fBeanModel.getABean(c) ;
		if (bp.getContainer() != null) {
		   JBCFPlugin.log("JavaSourceTranslator.refreshFreeFrom(): Removing: "+bp.getSimpleName()) ;  //$NON-NLS-1$
		   beansToRemove.add(c) ;		
		}
	}
   
   if (beansToAdd.size() == 0 && beansToRemove.size()==0) return ;
   
   if (disp != null) {
       disp.syncExec(new Runnable() {
            public void run() {    
                  try {
                        Iterator itr = beansToAdd.iterator() ;
                        while (itr.hasNext()) {
                            RefObject obj = (RefObject)itr.next() ;
                            ffBeans.add(obj) ;
                        }
                        itr = beansToRemove.iterator() ;
                        while (itr.hasNext()) {
							RefObject obj = (RefObject) itr.next();
							ffBeans.remove(obj) ;							
						}
                  }catch (Throwable t) {
                    JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING);
                  }
            }
        }) ;
   }
   else {
       try {
         Iterator i = beansToAdd.iterator() ;
         while (i.hasNext()) {
            RefObject obj = (RefObject)i.next() ;
            ffBeans.add(obj) ;
         }
         i = beansToRemove.iterator() ;
         while (itr.hasNext()) {
			RefObject obj = (RefObject) itr.next();
			ffBeans.remove(obj) ;							
		 }
      }catch (Throwable t) {
         JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING);
      }
   }
   
    
}   
/**
 * Add Transient error listeners
 */
public void addTransientErrorListener(ITransientErrorListener listener){
	if(fTransientErrorManager==null)
		fTransientErrorManager = new TransientErrorManager();
	fTransientErrorManager.addTransientErrorListener(listener);
}

/**
 *  Remove Transient error listeners
 */
public void removeTransientErrorListener(ITransientErrorListener listener){
	if(fTransientErrorManager!=null)
		fTransientErrorManager.removeTransientErrorListener(listener);
}

/**
 *  Decode the expression (code) impact on the bean (part)
 */
boolean  decodeExpression(CodeExpressionRef code) throws CodeGenException {
	
    return code.decodeExpression() ;
}

/**
 *  Create MOF instances 
 */
void  createJavaInstances () throws CodeGenException {
	Iterator itr = fBeanModel.getBeans().iterator() ;
	ArrayList err = new ArrayList() ;
    BeanSubclassComposition comp = fCompositionModel.getModelRoot() ;
	while (itr.hasNext()) {
	   BeanPart bean = (BeanPart) itr.next() ;
	   
	   RefObject obj = CodeGenUtil.createInstance(bean.getType(),fCompositionModel) ;
       String annotatedName= bean.getSimpleName() ;
       
       if (!bean.getSimpleName().equals(bean.THIS_NAME)) {
    	   if (obj instanceof IJavaObjectInstance) {
// ModelBuilder has now a cleanModel() method that will clean up
// After BeanParts with no initializationExpressions
//    	   	
//    	       JavaClass c = (JavaClass) ((IJavaObjectInstance)obj).getJavaType() ;
//               if ((BeanPartGenerator.getInstanceInitializationExpr(bean) == null && (bean.getReturnedMethod()==null || bean.getRefExpressions().isEmpty())) 
//// Bean Proxy now handle repacing interfaces with the initStrings               
////                    ||
////                    c.isInterface() || 
////                    c.isAbstract()
//               ) {
//    	          JBCFPlugin.log("Discarting creation of an interface/abstract class "+c.getName(),JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
//    	          obj = null ;
//    	       }
    	   }
    	   else
    	      obj = null ;
       }
       else {  // a this part
          if (obj != null) {
             obj.refSetID(MessageFormat.format(bean.THIS_NAME+CodegenMessages.getString("CodegenMessages.ThisPart.uriID"), new Object[] {fRSuri})) ; //$NON-NLS-1$
             // If no annotation, the PS will not allow you to edit the name in composition
             annotatedName = null ;
          }
       }
       
       
	   if (obj == null) {
	    JBCFPlugin.log("Could not create a JavaObjectInstance for: "+bean.getType()+": "+bean.getUniqueName(),JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$ //$NON-NLS-2$
	    err.add(bean) ;
	    // Children will not be connected to the VCE model
	    Iterator bItr = bean.getChildren() ;
	    if (bItr != null)
	      while (bItr.hasNext()) 
	        err.add(bItr.next()) ;
	   }
	   else {	        	    	   
	     CodeGenUtil.addAnnotation(obj) ;	
         if (annotatedName != null)
           CodeGenUtil.addAnnotatedName((IJavaObjectInstance)obj, annotatedName, fCompositionModel)  ; 
         comp.getAnnotations().add(CodeGenUtil.getAnnotation(obj)) ;
	     bean.setRefObject(obj) ;
	     try {		     
	       BeanPartGenerator.updateInstanceInitString(bean) ;
	     }
	     catch (com.ibm.etools.emf.ref.NotImplementedException e) {
	     	JBCFPlugin.log(e,JBCFMsgLogger.LOG_FINE) ;
	     	if (!err.contains(bean)) {
	     	   err.add(bean) ;
	     	   // Children will not be connected to the VCE model
	           Iterator bItr = bean.getChildren() ;
	           if (bItr != null)
	             while (bItr.hasNext()) 
	                  err.add(bItr.next()) ;
	     	}
	     }
	   }
	}	
	for (int i = 0; i < err.size(); i++) {
        ((BeanPart)err.get(i)).dispose() ;
    }
}

/**
 *  Given the BeanDOM, build the Composition Model
 */
void	buildCompositionModel() throws CodeGenException {
	if (fBeanModel == null || fCompositionModel == null) throw new CodeGenException ("null Builder") ; //$NON-NLS-1$
	
	fBeanModel.setState(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL,true) ;
	
	try{
		// Before handle expressions, make sure all BeanPart s have instances
		createJavaInstances() ;
	
	    // Decode the relevant expressions	
		Iterator itr = fBeanModel.getBeans().iterator() ;
		Vector badExprssions = new Vector() ;
		while (itr.hasNext()) {
			if (isReloadPending()) return ;
			
		    BeanPart bean = (BeanPart) itr.next() ;
			
		    Iterator refs = bean.getRefExpressions().iterator() ;
		    // Process the expression referencing the bean, and build the 
		    // Composition.
		    while (refs.hasNext()) {
			  CodeExpressionRef codeRef = (CodeExpressionRef)refs.next() ;
		      
		      //if (getCorrespondingFeature(codeRef,obj) != null)
		      try {
			  if (!decodeExpression (codeRef)) {
				 JBCFPlugin.log ("JavaSourceTranslator.buildCompositionModel() : Did not Decoded: "+codeRef, JBCFMsgLogger.LOG_FINE) ;						 //$NON-NLS-1$
				 badExprssions.add(codeRef) ;			 
			  }
		      }
		      catch (Exception e) {
		        JBCFPlugin.log("Skipping expression: "+codeRef,JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
		        JBCFPlugin.log(e,JBCFMsgLogger.LOG_WARNING) ;
		        badExprssions.add(codeRef) ;	
		      }
		    }
		}	
		// Clean up
		itr = badExprssions.iterator() ;
		while (itr.hasNext()) {
			CodeExpressionRef codeRef = (CodeExpressionRef) itr.next() ;
			codeRef.getMethod().removeExpressionRef(codeRef) ;
			codeRef.getBean().removeRefExpression(codeRef) ;
			codeRef.getBean().addBadExpresion(codeRef);
		}
	
	
	      // Decoders have analyzed and acted on the Expressions - 
	      // it is time to hook them together withn the Compsition
	      // Model
	      itr = fBeanModel.getBeans().iterator() ;
		  while (itr.hasNext()) {
		  	
		  	if (isReloadPending()) return ;
		  	
	         BeanPart bean = (BeanPart) itr.next() ;
	         // No need to add internal beans to the canvas
	         if (!bean.isInstanceVar()) continue ;
			
		   // if a bean was added to a container, the decoder will reflect this in the BeamModel
		   
		   // Model is build (but annotations).   Turn the model on, as the EditParts may slam dunc
		   // new element (e.g., a content pane).  We need to react and generate the appropriate code.
		   fBeanModel.setState(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL, false) ;
		   fBeanModel.setState(IBeanDeclModel.BDM_STATE_UP_AND_RUNNING,true) ;
		   if (bean.getContainer() == null) {
	       		// We are on the free from, 
	       		BeanSubclassComposition comp = fCompositionModel.getModelRoot() ;
	       		if (bean.getSimpleName().equals(BeanPart.THIS_NAME)) {
	       		    if (comp.isSetThisPart()) throw new CodeGenException ("this Already initialized") ; //$NON-NLS-1$
	           		comp.setThisPart((IJavaObjectInstance)bean.getRefObject()) ;	           	           		
	           		CDEHack.fixMe("Need to deal with FF annotation") ; //$NON-NLS-1$
	       		}
	       		else {	       		
	          		comp.getComponents().add(bean.getRefObject()) ;	          		
	       		}
	       		
	       		if (isReloadPending()) return ;
	       		try {
	       		  fBeanModel.setState(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL, true) ;
	          	  bean.getFFDecoder().decode() ;	       		
	       		}
	       		finally {
	       		  fBeanModel.setState(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL, false) ;
	       		}
	   	   }
		}
	}finally{
		fBeanModel.setState(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL, false) ;
		fBeanModel.setState(IBeanDeclModel.BDM_STATE_UP_AND_RUNNING,true) ;
	}
} 
 
/**
 *  Go for parsing a Java Source
 */ 
public Resource decodeDocument (IDiagramModelInstance cm, String uri, IFile sourceFile,IProgressMonitor pm) throws CodeGenException {
	
	if (sourceFile == null || !sourceFile.exists()) throw new CodeGenException("Invalid Source File") ;	 //$NON-NLS-1$
	fCompositionModel = cm ;
	
    fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_SYNCHING,true) ;
	fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_OUTOFSYNC,true) ;
	fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_UPDATING_JVE_MODEL,true) ;
	
	
    reConnect(sourceFile) ;
    fFile = sourceFile ;

	fdisconnected=false ;
	fRSuri = uri ;
	
	fBeanModel = null ;
	
	
	if (fFirstTimeDecode == true) {
		fFirstTimeDecode = false ;
		return fCompositionModel.getModelResourceSet().getResource(fRSuri)  ;			
	}
	
	com.ibm.etools.cde.CDEHack.fixMe("Need to dispose the working copy at some point") ; //$NON-NLS-1$
	
			
    fJavaModelBldr = new JavaBeanModelBuilder(fWorkingCopy,
                                              fWorkingCopy.getFile().getLocation().toFile().toString(),null) ;              
    
    if (isReloadPending()) return fCompositionModel.getModelResourceSet().getResource(fRSuri)  ;
    
    fJavaModelBldr.setDiagram(fCompositionModel) ;
	fBeanModel = fJavaModelBldr.build() ; 	
	fBeanModel.setSourceSynchronizer(fSrcSync) ;
	fBeanModel.setFStatus(fMsgRrenderer) ;
	fJavaModelBldr = null ;
	
	if (isReloadPending()) return fCompositionModel.getModelResourceSet().getResource(fRSuri)  ;	
		
	
	try {
	  buildCompositionModel() ;
	  fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_OUTOFSYNC,false) ;
	}
	catch (Exception e) {
		JBCFPlugin.log (e, JBCFMsgLogger.LOG_SEVERE) ; //$NON-NLS-1$
	}
	
	fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_SYNCHING,false) ;
	fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_UPDATING_JVE_MODEL,false) ;
		
	return fCompositionModel.getModelResourceSet().getResource(fRSuri) ;
}


/**
 *  Process a member that has no impact on the composition model
 */
protected int processDefaultMemberSave(IMember content, StringBuffer proposedBuff, 
                                       IProgressMonitor pm,int pOffset) 
                 throws JavaModelException {
	pm.subTask(MessageFormat.format(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.MemeberSave"), new Object[]{content.getElementName()})) ; //$NON-NLS-1$
	pm.worked(1) ;  

    // deal with in between gaps	
	int cOffset = content.getSourceRange().getOffset() ;
	if (pOffset>=0 && cOffset>pOffset) {
		proposedBuff.append(content.getCompilationUnit().getBuffer().
		                    getText(pOffset,cOffset-pOffset)) ;		
	}	  			        	
	proposedBuff.append(content.getSource()) ;
	return cOffset+content.getSourceRange().getLength()  ;
}


/**
 *  @deprecated
 *  Reflect Compisition changes effecting a method
 */
protected int processAMethodSave(IMethod content, StringBuffer proposedBuff, 
                                  Vector changedElements,
                                  IProgressMonitor pm,int pOffset) 
                throws JavaModelException, CodeGenException {
	
    pm.subTask(MessageFormat.format(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.MethodSave"), new Object[]{content.getElementName()})) ; //$NON-NLS-1$
    pm.worked(1) ;
	
	// Bring in left overs
    int cOffset = content.getSourceRange().getOffset() ;
    if (pOffset>=0 && cOffset>pOffset) {
	proposedBuff.append(content.getCompilationUnit().getBuffer().
		              getText(pOffset,cOffset-pOffset)) ;		
	}	
    MethodAnalyzer ma = new MethodAnalyzer(content,fBeanModel) ;
    String latestContent = ma.getCompositionReflectedSource() ;
    if (latestContent == null) {
    	JBCFPlugin.log ("processAMethodSave() -- Using current source for: "+content.getElementName()+"()", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$ //$NON-NLS-2$
    	latestContent = content.getSource() ;
    }
    proposedBuff.append(latestContent) ;
    if (!latestContent.equals(content.getSource()))
       changedElements.add(content.getHandleIdentifier()) ;
    return cOffset+content.getSourceRange().getLength() ;
}

/**
 * Process imports, package def, etc.
 */
protected void processPrefix(ICompilationUnit cu,StringBuffer proposedBuff, IProgressMonitor pm) 
                 throws JavaModelException {
	pm.subTask(MessageFormat.format(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.ProcessCUHeaders"), new Object[]{cu.getElementName()})) ; //$NON-NLS-1$
	pm.worked(1) ;   
	int Offset=-1,Index=0 ;
	IJavaElement[] elements = cu.getChildren() ;
	if (elements.length>0) {
		while (Index<elements.length) {
com.ibm.etools.cde.CDEHack.fixMe("need to consider inner classes") ; //$NON-NLS-1$
			if (elements[Index] instanceof IType) {
				IMethod[] methods = ((IType)elements[Index]).getMethods() ;
				if (methods.length>0) {
					Offset = methods[0].getSourceRange().getOffset() ;
					break ;
				}
			} ;
			Index++ ;
		}			
	}
	if (Offset==-1) Offset = cu.getSourceRange().getOffset() ;
	proposedBuff.append(cu.getBuffer().getText(0,Offset)) ;
}


/**
 * Process anything from the end of the last method
 */
protected void processPostfix(ICompilationUnit cu,StringBuffer proposedBuff, IProgressMonitor pm) 
                 throws JavaModelException {
	pm.subTask(MessageFormat.format(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.ProcessCUFooters"), new Object[]{cu.getElementName()})) ; //$NON-NLS-1$
	pm.worked(1) ;   
	int Offset=-1 ;
	IMethod [] methods = CodeGenUtil.getMethods(cu) ;
	if (methods.length>0) {
			Offset = methods[methods.length-1].getSourceRange().getOffset()+
					 methods[methods.length-1].getSourceRange().getLength() ;
	}
	if (Offset>=0 &&cu.getSourceRange().getLength()-Offset>0) 
	   proposedBuff.append(cu.getBuffer().getText(Offset,cu.getSourceRange().getLength()-Offset)) ;
}

/**
 *  Delete the instance variale, and Code associated with bean
 */
protected void deleteBeanPart(ICompilationUnit CU, Vector changedElements, BeanPart bean) throws CodeGenException {

      BeanPartGenerator bgen = new BeanPartGenerator(fBeanModel,fCompositionModel) ;
      bgen.removeBeanPart(bean) ;
}


/**
 *  Remove methods/instance var. for Beans that are not in the composition anymore.
 */
protected boolean removeStaleMethods(ICompilationUnit workingCU, Vector changedElements, IProgressMonitor pm) throws CodeGenException {
	boolean removed=false ;
	
	Iterator itr = fBeanModel.getBeans().iterator() ;
	while (itr.hasNext()) {
		BeanPart bean = (BeanPart)itr.next() ;
		RefObject obj = bean.getRefObject() ;
		if (!CodeGenUtil.isComponentInComposition(obj,fCompositionModel)) {
		   deleteBeanPart(workingCU, changedElements, bean) ;
		   removed=true ;
		}
	}	
	return removed ;
}

/**
 *  @deprecated
 *  Work the current Bean Decleration Model and update the source
 *  @return boolean were any changes made.
 */
protected void walkJavaCode (ICompilationUnit workingCU, Vector changedElements, 
                             IProgressMonitor pm) throws CodeGenException {
	
	boolean needtoCommit ;
	IProgressMonitor monitor=null ;
	
	needtoCommit = removeStaleMethods(workingCU, changedElements, pm) ;
	boolean      didSomeWork = false ;
	
	try {
	workingCU.reconcile() ;
     // Go through the Methods, and figure out if there are expression that need change.
     IJavaElement[] content = CodeGenUtil.getMainType(workingCU).getChildren() ;
     monitor = new SubProgressMonitor(pm, content.length+1) ;     
     // Re-build a proposed content for the compilation unit.
     StringBuffer proposedBuff = new StringBuffer(workingCU.getSourceRange().getLength()*2) ;          
     
     monitor.beginTask(MessageFormat.format(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.SaveStart"), new Object[]{fWorkingCopy.getFile().getName()}),content.length+1) ; //$NON-NLS-1$
     processPrefix(workingCU,proposedBuff,monitor) ;
     // Keep track of the last offset we processed
     int lastOffset = -1 ;
     for (int i=0 ; i<content.length; i++) { 
    	if (content[i] instanceof IMethod) {
    	    lastOffset = processAMethodSave((IMethod)content[i],proposedBuff,
    	                                     changedElements,monitor,lastOffset) ;
    	    didSomeWork = true ;
    	}
    	else if (content[i] instanceof IField) {
    		com.ibm.etools.cde.CDEHack.fixMe() ;    		
    	}
    	else if (content[i] instanceof IMember)
    	    lastOffset = processDefaultMemberSave((IMember)content[i],proposedBuff,monitor,lastOffset) ;
    	else {
    		throw new CodeGenException("Unexpected content: "+content[i]) ; //$NON-NLS-1$
    	}
     } 
     if (!didSomeWork) return ;
     
     processPostfix(workingCU,proposedBuff,monitor) ;     
     String updatedBuf = proposedBuff.toString() ;
     if (updatedBuf.equals(workingCU.getSource())) {
     	
     }
     else { // Save
     	monitor.subTask(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.UpdatingSource")) ;     	 //$NON-NLS-1$
     	monitor.worked (1) ;
     	// This is the local shsdow ..., just replace it
     	workingCU.getBuffer().replace(0,workingCU.getSourceRange().getLength(),updatedBuf)  ;
     	monitor.subTask(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.SaveEnd")) ; //$NON-NLS-1$
        monitor.worked(5) ;             
        needtoCommit = true ;
     }
    } catch (JavaModelException e) {
    	throw new CodeGenException(e) ;
	}	
	finally {
		if (monitor != null) monitor.done() ;
	}	
}

/**
 * If this component is a container, process its children 
 * @return Vector of IMethods denoting the new methods generated
 */
protected Vector processKidsIfNeeded(RefObject component,Vector changedElements, ICompilationUnit wcu, Vector kidsMethods) throws CodeGenException {	
    Vector kids = new Vector() ;
    // Get a decoder that will know how to collect all first level childrens
    IExpressionDecoder decoder = ExpressionDecoderFactory.getDecoder((IJavaObjectInstance)component) ;
    Iterator itr = decoder.getChildren((IJavaObjectInstance)component).iterator() ;
  	// This component has childrents, Process them first, so that we 
    	// can add them as we generate the current method
    while (itr.hasNext()) {
    		RefObject child = (RefObject)itr.next() ;
    		processAComponent(child, changedElements, wcu) ;
    		// No method was generated
    		BeanPart bp = fBeanModel.getABean(child) ;
    		if (bp != null) {
    		   	kids.add(bp) ;
    		   	kids.add(itr.next()) ;
    		}
    		else  // skip SF
    		    itr.next() ;
    		
    }    
    if (kids.size() > 0) return kids ;
    else return null ;	
}

/**
 * If this component is not reflected in code, generate it
 * @return boolean   - were any changes made to the source code.
 */
protected void processAComponent(RefObject component,Vector changedElements, ICompilationUnit wcu) throws CodeGenException {	
	
    Vector kidsMethods = new Vector () ;
    Vector kids = processKidsIfNeeded(component,changedElements, wcu, kidsMethods) ;    
    if (component.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) != null) {
    	return ;
    }
        
    
    IType cuType = CodeGenUtil.getMainType(wcu) ;

    IInstanceVariableCreationRule cRule = (IInstanceVariableCreationRule) JavaPatternRuleFactory.getRule(IInstanceVariableCreationRule.RULE_ID) ;
    String methodName = cRule.getInstanceVariableMethodName(component,cuType,fCompositionModel) ;
    String varName = cRule.getInstanceVariableName(component,cuType,fCompositionModel) ;    
    
    BeanPart bp = fBeanModel.getABean(varName) ;
    if (bp == null) {
       // Set up a new BeanPart in the decleration Model
       String bType = ((IJavaObjectInstance)component).getJavaType().getQualifiedName() ;
       bp = new BeanPart (varName,bType) ;
       bp.setModel(fBeanModel) ;
       bp.setRefObject(component) ;
       fBeanModel.addBean(bp) ;    
    }
    
	MethodTextGenerator mgen = new MethodTextGenerator(component,fBeanModel) ;
	if (((IJavaObjectInstance)component).isImplicit()) {// No need to generate a method/inst.Var    
	    IJavaObjectInstance p = CodeGenUtil.getParent((IJavaObjectInstance)component,fCompositionModel) ;
    	if (p==null) throw new CodeGenException("Could not find parent of implicit: "+varName) ;    	 //$NON-NLS-1$
    	// Parent was not processed yet (most likely)
    	String pName = cRule.getInstanceVariableMethodName(p,cuType,fCompositionModel) ;
    	mgen.setComments(new String[] {"Implicit Defenition from "+pName}) ; //$NON-NLS-1$
	}
	CodeMethodRef   mref = new CodeMethodRef(fBeanModel.getTypeRef(),methodName) ;
	bp.addInitMethod(mref) ;
      bp.addReturnMethod(mref) ;	
	String newMSrc = mgen.generate(mref,methodName,varName,kids) ;
		
    
    IMethod newMethod=null ;
    // Create it as the last method 
    try {
           	                        		
      newMethod = cuType.createMethod(newMSrc,null,false,null) ;      
      if (newMethod != null)
         changedElements.add(newMethod.getHandleIdentifier()) ;
      mref.setOffset(newMethod.getSourceRange().getOffset()) ;
      mref.setMethodHandle(newMethod.getHandleIdentifier()) ;  
      // empty lines may shift to other methods
      mref.setContent(newMethod.getSource()) ;
//      if (newMSrc.length() != newMethod.getSource().length()) {
//          JBCFPlugin.log ("JavaSourceTranslator.processAComponent(): newMethodSource("+newMSrc.length()+") JDOM("+newMethod.getSource().length()+")") ;
//      }
      fBeanModel.addMethodInitializingABean(mref) ;          
    }
    catch (JavaModelException e) {
    	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
    	throw new CodeGenException(e) ;
    }

    
    InstanceVariableTemplate ft = new InstanceVariableTemplate(varName,
                                                              ((IJavaObjectInstance)component).getJavaType().getQualifiedName(),
                                                              new String [] {"VCE Generated instance Variable"}) ; //$NON-NLS-1$
    // Create it as the last field
    ft.setSeperator(fBeanModel.getLineSeperator()) ;
    try {
      IField field = cuType.createField(ft.toString(),null,false,null) ;
      if (field != null) {
      	changedElements.add(field.getHandleIdentifier()) ;
      }
      JBCFPlugin.log("Adding: \n"+ft, JBCFMsgLogger.LOG_FINE) ;		          //$NON-NLS-1$
    }
    catch (JavaModelException e) {
      throw new CodeGenException(e) ;
    }    
    CodeGenUtil.refreshMethodOffsets(cuType,fBeanModel) ;
    JBCFPlugin.log("Adding: \n"+newMSrc, JBCFMsgLogger.LOG_FINE) ;		 //$NON-NLS-1$
	
}
/**
 *  Walk the Composition Model looking for new Components
 *  @return boolean were any changes made to the source code.
 */
protected void walkCompositionModel (ICompilationUnit workingCU,Vector changedElements,IProgressMonitor pm) throws CodeGenException {
	
	boolean needtoCommit = false ;
	IProgressMonitor monitor=null ;
	try {
		
	 Iterator rootComponents = fCompositionModel.getModelRoot().getComponents().iterator() ;
     monitor = new SubProgressMonitor(pm, 10) ;     
     
     while (rootComponents.hasNext()) {
     	RefObject component = (RefObject) rootComponents.next() ;
     	monitor.subTask(MessageFormat.format(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.ProcessRootComponent"), new Object[]{component.refID()})) ; //$NON-NLS-1$
     	monitor.worked(1) ;
     	processAComponent(component,changedElements,workingCU) ;
     	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with Containers"); //$NON-NLS-1$
     }          
    } catch (Exception e) {
    	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
    	throw new CodeGenException(e) ;
	}	
	finally {
		if (monitor != null) monitor.done() ;
	}		    	
}	
/**
 *  At this time the assumption is that the JAVA source code has not changed
 *  since we last parse, and that the MODEL has not updated the source online.
 */ 
public void saveDocument (IProgressMonitor pm) throws CodeGenException {
	if (fBeanModel == null) throw new CodeGenException ("No Bean Model") ; //$NON-NLS-1$
	

    com.ibm.etools.cde.CDEHack.fixMe("CU may have changed by now...") ; //$NON-NLS-1$
pm.beginTask(CodegenMessages.getString("JavaSourceTranslator.ProgressMonitor.SaveDocument"),20) ; //$NON-NLS-1$

    
    
    ICompilationUnit workingCU = null ;
    boolean needToCommit = false ;
    Vector  changedElements = new Vector () ;
    try {     
//    	// workingCU =(ICompilationUnit) fBeanModel.getCompilationUnit().getWorkingCopy() ;
//    	workingCU = fWorkingCopy.getLocalWorkingCopy() ;
////JBCFPlugin.log("Starting with:\n["+workingCU.getSource()+"]\n") ;   	
//    	// Update Code/Model with new components
//    	walkCompositionModel(workingCU,changedElements,pm) ;
//    	// Sync. Code with bean's context 
////JBCFPlugin.log("After walking composition:\n["+workingCU.getSource()+"]\n") ;   	    	
//    	walkJavaCode(workingCU,changedElements,pm) ;
////JBCFPlugin.log("After walking Java:\n["+workingCU.getSource()+"]\n") ;   	    	    	
//    	needToCommit = changedElements.size() > 0 ;
//    	
//    	
//    	if (needToCommit) 
//    		JBCFPlugin.log("Updating Java Working Copy") ;    	
//    	else
//    	   JBCFPlugin.log("No Update is needed to the Java Working Copy") ;
//    	    
//    	//fWorkingCopy.UpdateDeltaToShared(pm,changedElements,true)   ;  		
    	fWorkingCopy.UpdateDeltaToShared(null,null,changedElements,true)   ;  		
    	
    	com.ibm.etools.cde.CDEHack.fixMe("The big red Reload button") ; //$NON-NLS-1$
    	reloadFromScratch(Display.getCurrent(),null) ;
    	      
    } catch (Exception e) {
    	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
    }
    finally {
//    	if (workingCU!=null)  workingCU.destroy() ;    	    	
    }             
}

// Should an empty model before calling the decode() 
public boolean shouldCreateRoot () { 
	return true  ;
} 
	
// What file extention does this decoder works with
public String  getFileExt() {  
	return JAVAExt; 
}

protected void setHoldGUIChanges(boolean flag, String msg) {
	IModelChangeController controller = (IModelChangeController) fEDomain.getData(IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY);
	controller.setHoldChanges(flag, msg) ;
	fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_PARSE_ERRROR,flag) ;
}

public void reloadFromScratch(Display disp, ICancelMonitor monitor) throws CodeGenException {
	
	
	if (fFile==null)
	  fFile = fWorkingCopy.getFile() ; 
	  
	if (fFile == null) throw new CodeGenException ("No Resource") ; //$NON-NLS-1$
	
	try {
		fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS, true);
		JBCFPlugin.log("*** Reloading", JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$
        disconnect(false); // clear the BDM, but do not dispose of local Doc, and Synch.

		// Do not clear the VCE model - may not be on the UI thread.
		final com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition comp = fCompositionModel.getModelRoot();
		if (monitor != null && monitor.isCanceled())
			return;

		disp.syncExec(new Runnable() {
			public void run() {
				try {
					
					fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_SYNCHING, true);
					fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_OUTOFSYNC, true);
					fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_UPDATING_JVE_MODEL, true);
					setHoldGUIChanges(false, null);
					clearModel(true);
					decodeDocument(fCompositionModel, fRSuri, fFile, null);
					if (fMsgRrenderer != null)
						fMsgRrenderer.showMsg(null, fMsgRrenderer.NORMAL_MSG);
					setHoldGUIChanges(false, null);
					fireUpdateNotification() ;
				}
				catch (Throwable t) {
					if (fMsgRrenderer != null)
						fMsgRrenderer.showMsg(t.getMessage(), fMsgRrenderer.ERROR_MSG);
                    
					int severity = JBCFMsgLogger.LOG_WARNING;
					if (t instanceof CodeGenException)
						severity = JBCFMsgLogger.LOG_FINE;
					if (t instanceof CodeGenSyntaxError)
					   JBCFPlugin.log(t.toString(), severity);
					else
					   JBCFPlugin.log(t, severity);
					setHoldGUIChanges(true, t.getMessage());
					fireUpdateNotification() ;
				}
			}
		});
	}
	finally {
		fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS, false);
	}
}

public void pauseRoundTripping(boolean flag) throws CodeGenException {

    // We need to clean up locking and such, but we should not try to lock the model while on the display thread unless nothing
    // is going on
    if (fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_SYNCHING)||
        fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_OUTOFSYNC)||
        fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_UPDATING_SOURCE)||
        fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_UPDATING_JVE_MODEL)||
        fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_PENDING)||
        fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS))
        return ;
        
    if (fpauseRoundTripping && flag) return ;
    IModelChangeController controller = (IModelChangeController) fEDomain.getData(IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY);
    if (flag) {  // Want to pause
    	fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_PAUSE,true) ;
    	if (controller.isHoldChanges()) return ; // Already paused
    	fpauseRoundTripping = true ;
    	Object lock = fBeanModel == null ? new Object() : fBeanModel.getDocumentLock() ;
    	synchronized (lock) {
    	  disconnect(false) ;    	
    	  controller.setHoldChanges(true,fPauseSig) ;    	//$NON-NLS-1$    	
    	}
    } else {
		if (!controller.isHoldChanges())
			return;

		if (fPauseSig.equals(controller.getHoldMsg())) { //$NON-NLS-1$
			try {
				Object lock = fBeanModel == null ? new Object() : fBeanModel.getDocumentLock() ;
				synchronized (lock) {
					controller.setHoldChanges(false, null);
					fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS, true);
					fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_PAUSE, false);
					fpauseRoundTripping = false;
					reconnect(null, null);
				}
			}
			finally {
				fMsgRrenderer.setStatus(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS, false);
			}
		}
	}
}

/**
 * This will clear the BDM, and optionaly the VCE model
 */
private   void clearModel(boolean vceModel) {
	 
	try {
	if (fCompositionModel != null) {
        RefObject root = fCompositionModel.getModelRoot() ;
        if (root != null) {
             ICodeGenAdapter a = (ICodeGenAdapter)root.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
             while  (a != null) {
                root.removeAdapter(a);
                a = (ICodeGenAdapter)root.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
             }
        }    
    }
       
   if (fBeanModel != null) { 
        fBeanModel.dispose() ;
        fBeanModel = null ;        
   }
   
   
   if (vceModel == true && fCompositionModel != null) {
      com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition comp = fCompositionModel.getModelRoot() ;
      CDEHack.fixMe("Deal with multiple diagrams") ; //$NON-NLS-1$
	  Diagram d = (Diagram) comp.getDiagrams().get(0) ;
	  if (comp.getComponents().size()>0)
	     comp.getComponents().clear() ;
	  if (comp.isSetThisPart())
	     comp.unsetThisPart() ;
	  if (d.getVisualInfos().size()>0)                             
	     d.getVisualInfos().clear() ;
	  if (comp.getAnnotations().size()>0)
	     comp.getAnnotations().clear() ;
   }
	}
	catch (Exception e) {
		JBCFPlugin.log(e,JBCFMsgLogger.LOG_FINE) ;
	}
}

public synchronized void reConnect(IFile file) {
	clearModel(true) ;
	if (fWorkingCopy == null) {
	    fWorkingCopy = new WorkingCopyProvider(file) ;		    
    }
    else if (fdisconnected)
       fWorkingCopy.reconnect(file) ;
       
    if (fSrcSync == null) {
	   fSrcSync = new JavaSourceSynchronizer(fWorkingCopy,this) ;
	   fSrcSync.setLocalUpdatingSharedStrategy(new LocalToSharedUpdater());
	   fSrcSync.setSharedUpdatingLocalStrategy(new SharedToLocalUpdater()) ;
	   fSrcSync.setDelay(fSrcSyncDelay) ;
	   fSrcSync.setStatus(fMsgRrenderer) ;
    }
    else if(fdisconnected)
       fSrcSync.connect() ;

	fdisconnected=false ;
}

public synchronized void reconnect(org.eclipse.ui.IFileEditorInput input,IProgressMonitor pm) throws CodeGenException {
	IFile file ;
	if (input != null) 
	  file = input.getFile() ;
    else
      file = fFile ;
	decodeDocument(fCompositionModel, file.getFullPath().toString(), file, pm) ;
}

/**
 * Same as dispose, but do not destroy the WorkingProvider
 */
public synchronized void disconnect(boolean clearVCEModel) {

    if (fSrcSync != null)
       fSrcSync.disconnect() ;


    clearModel(clearVCEModel) ;
        
    if (fSrcSync != null) // fWorkingCopy may not be null yet if called from dispose
       fWorkingCopy.disconnect() ;
    
   
    fTransientErrorManager = new TransientErrorManager();
    fdisconnected=true ;
    

}

/**
 * Clear the VCE Model, disconnect, and get rid of all resources.
 */
public synchronized void dispose() {
	
	if (fSrcSync != null) {
		fSrcSync.setLocalUpdatingSharedStrategy(null) ;
		fSrcSync.setSharedUpdatingLocalStrategy(null) ;
		fSrcSync.uninstall() ;
		fSrcSync = null ;
		// The following is a hack until CodeGen uses the EditDomain.
		com.ibm.etools.jbcf.codegen.java.ExpressionDecoderFactory.clearCache() ;
		CodeGenUtil.clearCache()  ;
		AttributeFeatureMapper.clearCache() ;
		InstanceVariableCreationRule.clearCache() ;
		InstanceVariableRule.clearCache() ;
	}
    
    disconnect(true) ;
		
	if (fWorkingCopy != null) 
	   fWorkingCopy.dispose() ;
	
	fWorkingCopy = null ;
}

public boolean isReloadPending() {
	if (fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_PENDING)) return true ;
	
	Display display = Display.getCurrent();          
    if (display != null) {
    	while (display.readAndDispatch()) ;
    	if (fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_PENDING)) return true ;
    }
    return false ;    	 	
}

/**
 * This one provide a Synchronous call to drive a commit and flush process
 * This call can not be nested.  i.e., one can not call a commitAndFlush indirectly from
 * a strategy (worker) routine.   This will cause the thread to want to wait for itself to finish
 */
public void commitAndFlush(boolean canWait) {
    if (fSrcSync == null) return ;
    if (fBeanModel == null ||
        fBeanModel.isStateSet(fBeanModel.BDM_STATE_DOWN) ||
        fBeanModel.isStateSet(fBeanModel.BDM_STATE_UPDATING_JVE_MODEL)) return ;
    CDEHack.fixMe("This is the really BAD !!! we need to fix the way we synchronize") ; //$NON-NLS-1$
    // The problem is that the Synchronizer will wait untill all workers are done before
    // responding on commit request.  But, in a reload from scratch of an extended
    // JFrame for example, where we have a worker thread calling this in a response
    // of a content pane insertion transaction ... dead lock.
    if (fMsgRrenderer == null ||
        fMsgRrenderer.isStatusSet(fMsgRrenderer.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS))
        return ;
         
    
    final ArrayList waitDone = new ArrayList() ;
    final Display display = Display.getCurrent();
    long  startTime, endTime  ;
    
    // If we are doing reload from scratch, or bring up, no point to wait 
    if (fBeanModel == null || fBeanModel.isStateSet(fBeanModel.BDM_STATE_DOWN) || fBeanModel.isStateSet(fBeanModel.BDM_STATE_UPDATING_JVE_MODEL)) {
       JBCFPlugin.log("JavaSourceTranslator: commitAndFlush(sync, canWait="+canWait+" BringUp - returned")	 ; //$NON-NLS-1$ //$NON-NLS-2$
       return ;
    }
    
    JBCFPlugin.log("JavaSourceTranslator: commitAndFlush(sync, canWait="+canWait+" Display="+display+") - start",JBCFMsgLogger.LOG_FINEST) ;          //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    waitDone.add(Thread.currentThread()) ;
    ISynchronizerListener listener = new ISynchronizerListener() {
                             public void markerProcessed (String marker) {
                                 synchronized (this) {
                                     waitDone.clear() ;
                                     if (display != null)
                                        display.asyncExec(null) ;
                                     else
                                        this.notifyAll() ;
                                        
                                 }
                             }
                          } ;

      fSrcSync.notifyOnMarker(listener,"commitAndFlush",!canWait) ; //$NON-NLS-1$
      startTime = System.currentTimeMillis() ;  
      endTime = startTime+5*Math.max(500, fSrcSync.getDelay()) ;

      int NestGuard ;
      synchronized (this) {
         NestGuard = fCommitAndFlushNestGuard++ ;
      }
      if (display != null) {      	
      	while (!waitDone.isEmpty())  {
         try { 		     
       	// Busy wait, because we do not want to miss a wake up call		  
		 	if (!display.readAndDispatch()) {
		 		if (NestGuard == 0) {
				   display.sleep();
		 		}
				else {
					JBCFPlugin.log("commitAndFlush Nesting = "+NestGuard,JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
					break ;
				}
				   
			}	        
		  }
		  catch (Throwable t) {}				   			  		           
//         if (System.currentTimeMillis()>endTime) {
//        	JBCFPlugin.log("JavaSourceTranslator.commitAndFlush(): Nested call error",JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
//        	waitDone.clear() ;
//         }
	    }
      }
      else { // No Display      	            
        synchronized (listener) {      	 	
      	  while (!waitDone.isEmpty())
      	 	try {
      	 	 listener.wait(endTime-startTime+1) ;
      	 	}
      	 	catch (InterruptedException e) {
      	 		if (System.currentTimeMillis()>endTime) {
        	         JBCFPlugin.log("JavaSourceTranslator.commitAndFlush(): Nested call error",JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
        	         waitDone.clear() ;
      	 		}
            }
        }
      } 
      	 		      	 			                    
    JBCFPlugin.log("JavaSourceTranslator: commitAndFlush - done",JBCFMsgLogger.LOG_FINEST) ;         //$NON-NLS-1$
    synchronized (this) {
      fCommitAndFlushNestGuard-- ;
      if (fCommitAndFlushNestGuard<0) fCommitAndFlushNestGuard=0 ;
    }
}
/**
 * This one provide an Async. registration for a notification on flush process
 */
public void commitAndFlush(ISynchronizerListener listener, String marker) {
    
    // First commit
    
    if (fBeanModel != null)
      fBeanModel.deleteDesignatedBeans() ;
    
    if (listener == null) return ;
    
        // If we are doing reload from scratch, or bring up, no point to wait 
    if (fBeanModel == null || !fBeanModel.isStateSet(fBeanModel.BDM_STATE_UP_AND_RUNNING)) {
       JBCFPlugin.log("JavaSourceTranslator: commitAndFlush(sync,  BringUp - returned")	 ; //$NON-NLS-1$
       listener.markerProcessed(marker) ;
       return ;
    }
    
    JBCFPlugin.log("JavaSourceTranslator: commitAndFlush(async) - start",JBCFMsgLogger.LOG_FINEST) ;          //$NON-NLS-1$
    if (fSrcSync == null) 
      listener.markerProcessed(marker) ;
    else     
      fSrcSync.notifyOnMarker(listener,marker,false) ;    
    JBCFPlugin.log("JavaSourceTranslator: commitAndFlush - done",JBCFMsgLogger.LOG_FINEST) ;         //$NON-NLS-1$
}

public void setSynchronizerSyncDelay(int delay) {
    fSrcSyncDelay = delay ;
    if (fSrcSync != null)
      fSrcSync.setDelay(fSrcSyncDelay) ;
}


protected void fireUpdateNotification() {
    
    JBCFPlugin.log("JavaSourceTranslator.fireUpdateNotification",JBCFMsgLogger.LOG_FINEST) ; //$NON-NLS-1$
    for (Iterator itr = fTranslatorListeners.iterator(); itr.hasNext();) {
        ISourceTranslatorListener element = (ISourceTranslatorListener) itr.next();
        try {
            element.modelUpdated() ;
        }
        catch (Throwable t) {
            JBCFPlugin.log(t,JBCFMsgLogger.LOG_WARNING) ;
        }        
    }
}

public  void  fireSnippetProcessing(boolean start) {
    
    
     for (Iterator itr = fTranslatorListeners.iterator(); itr.hasNext();) {
        ISourceTranslatorListener element = (ISourceTranslatorListener) itr.next();
        try {
            if (start) {
              // Snippet Operations may have started, but cancelled in the middle
              // to make room for the new update that is coming in.
              if (fSnippetProcessingInProgress) return ;
              fSnippetProcessingInProgress = true ;
              element.snippetProcessingStart() ;
            }
            else {  
              if (fSrcSync.isWorkQueued()) return ;  // Wait
              element.snippetProcessingCompleted() ;
              fSnippetProcessingInProgress = false ;
            }
        }
        catch (Throwable t) {
            JBCFPlugin.log(t,JBCFMsgLogger.LOG_WARNING) ;
        }        
     }  
     JBCFPlugin.log("JavaSourceTranslator.fireSnippetProcessin, Start="+start,JBCFMsgLogger.LOG_FINEST) ; //$NON-NLS-1$
}    


public void addTranslatorListener (ISourceTranslatorListener listener) {
    if (listener == null) return ;
    if (fTranslatorListeners.contains(listener)) return ;
    fTranslatorListeners.add(listener) ;
}

public void removeTranslatorListener (ISourceTranslatorListener listener) {
    if (listener == null) return ;
    fTranslatorListeners.remove(listener) ;    
}

public void setMsgRenderer (IJVEStatus mr) {
    fMsgRrenderer = mr ;
    if (fSrcSync != null)
       fSrcSync.setStatus(mr) ;
}
public void setEditDomain(EditDomain d) {
	fEDomain = d ;
}



}