package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
/*
 * SelectionNotifier extends SelectionSynchronizer that is a GEF class that aligns
 * selection between the tree and the GUI edit parts
 * By extending it and adding our own listener interface we can receive notification
 * of selection.  This is required by the JavaVisualEditor that has to drive selection to 
 * the relevent source position irrespective of which viewer(s) are open
 */
import com.ibm.etools.gef.ui.parts.SelectionSynchronizer;
import java.util.*;

import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;

public class SelectionNotifier extends SelectionSynchronizer {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ListenerList fListeners = new ListenerList(1);
	protected boolean isDispatching;

public void selectionChanged(SelectionChangedEvent event ){
	if (isDispatching)
		return;	// Already dispatching.
	isDispatching = true;
	super.selectionChanged(event);
	if (!fListeners.isEmpty()) {
		Object[] listeners = fListeners.getListeners();
		for (int i = 0; i < listeners.length; i++) {
			((ISelectionChangedListener) listeners[i]).selectionChanged(event);
		}
	}
	isDispatching = false;
}
public void addSelectionChangedListener(ISelectionChangedListener aListener){
	fListeners.add(aListener);
}
public void removeListener(ISelectionChangedListener aListener){
	fListeners.remove(aListener);
}
}
