package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.codegen.model.IBeanDeclModel;
import com.ibm.etools.jbcf.codegen.util.ICancelMonitor;

/**
 * @version 	1.0
 * @author
 */
public class TransientErrorManager {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected List fTransientErrorListeners = null;
protected List fTransientErrorHistoryHolder = null;

public TransientErrorManager(){
	fTransientErrorHistoryHolder = new ArrayList();
}

/**
 * Add Transient error listeners
 */
public void addTransientErrorListener(ITransientErrorListener listener){
	if(fTransientErrorListeners==null)
		fTransientErrorListeners = new ArrayList();
	if(listener!=null)
		fTransientErrorListeners.add(listener);
}

/**
 *  Remove Transient error listeners
 */
public void removeTransientErrorListener(ITransientErrorListener listener){
	if(fTransientErrorListeners!=null && listener!=null)
		fTransientErrorListeners.remove(listener);
}

protected void fireTransientError(ITransientErrorEvent event){
    if (fTransientErrorListeners==null) return ;
    
	for(int i=0;i<fTransientErrorListeners.size();i++) {
		  String eType = event.getErrorType() ;
	      if (eType.equals(event.TYPE_CORRECTED_BEAN) || eType.equals(event.TYPE_CORRECTED_FEATURE) || eType.equals(event.TYPE_INFO))
	        ((ITransientErrorListener)fTransientErrorListeners.get(i)).correctionOccured(event);
	      else
		    ((ITransientErrorListener)fTransientErrorListeners.get(i)).errorOccured(event);
	}
}

public void handleLocalToSharedChanged(IBeanDeclModel fBeanModel, ICancelMonitor monitor){
	fireCorrections(fBeanModel, monitor, false);
}

public void handleSharedToLocalChanges(IBeanDeclModel fBeanModel, List errors, ICancelMonitor monitor){
      // Notify of transient errors..
      // TRANSIENT ERRORS
      final List errorsInCodeDelta = errors;
      boolean parseError = false;
      if(errorsInCodeDelta!=null && errorsInCodeDelta.size()>0){
	  	if (monitor != null && monitor.isCanceled()) 
	  		return ;
	  	for(int error=0;error<errorsInCodeDelta.size();error++){
	  		TransientErrorEvent event = ((TransientErrorEvent)errorsInCodeDelta.get(error));
	  		primRegisterError(event);
	  		if(event.getErrorType().equals(event.TYPE_PARSER_ERROR))
	  			parseError=true;
	  	}
      }
	fireCorrections(fBeanModel, monitor, parseError);
}

protected void primRegisterError(TransientErrorEvent event){
	fireTransientError(event);
}

protected void fireCorrections(IBeanDeclModel fBeanModel, ICancelMonitor monitor, boolean isInParseError){
      // TRANSIENT CORRECTIONS
      if(fBeanModel!=null){
      	List inErrorBeans = new ArrayList();
      	Iterator itr = fBeanModel.getBeans().iterator();
      	while(itr.hasNext()){
      		boolean beanPartInError = false;
      		BeanPart bp = (BeanPart) itr.next();
      		if(isInParseError){
      			inErrorBeans.add(bp);
      		}else{
      			Iterator expressions = bp.getRefExpressions().iterator();
	      		while(expressions.hasNext()){
	      			CodeExpressionRef exp = (CodeExpressionRef) expressions.next();
	      			if(exp.isStateSet(exp.STATE_EXP_IN_LIMBO)){
	      				beanPartInError = true;
	      				break;
	      			}
	      		}
      			if(beanPartInError)
      				inErrorBeans.add(bp);
      		}
      	}
      	Iterator prevErrBeans = fTransientErrorHistoryHolder.iterator();
      	while(prevErrBeans.hasNext()){
      		BeanPart bp = (BeanPart) prevErrBeans.next();
      		if(!inErrorBeans.contains(bp)){
      			// Was in error, now is not.. notify
      			fireTransientError(
      				new TransientErrorEvent(
      					ITransientErrorEvent.TYPE_CORRECTED_BEAN,
      					(RefObject)bp.getRefObject(),
      					bp.getUniqueName()));
      		}
      	}
      	fTransientErrorHistoryHolder.clear();
      	fTransientErrorHistoryHolder.addAll(inErrorBeans);
      }
}

public void clear(){
	fTransientErrorHistoryHolder.clear();
}

}
