package com.ibm.etools.jbcf.codegen;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform ;
import org.eclipse.core.runtime.IPluginDescriptor ;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.core.internal.plugins.PluginDescriptor ;
import com.ibm.etools.emf.resource.ResourceSet ;
import com.ibm.etools.jbcf.codegen.java.*;
import com.ibm.etools.jbcf.codegen.util.*;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.editorpart.IJVEStatus;
import com.ibm.etools.jbcf.emf.vce.* ;
import com.ibm.etools.cde.* ;


public class VCEModelBuilder extends AbstractModelBuilder {
	
	
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
static final String JavaDecoderClass = "com.ibm.etools.jbcf.codegen.JavaSourceTranslator" ; //$NON-NLS-1$
static final String PluginID = "com.ibm.etools.jbcf.codegen" ; //$NON-NLS-1$
EditDomain fEDomain = null ;
	

protected void initializeResourceSet() throws Exception {
	
	
}


protected  Object loadTheClass(String clazzName) throws Exception {

   IPluginDescriptor plugin = Platform.getPluginRegistry().getPluginDescriptor(PluginID) ;
   return ((PluginDescriptor) plugin).createExecutableExtension(clazzName,null,null,"class") ;	 //$NON-NLS-1$
	
}

protected void resolveDecoder (String fileExt) throws Exception {
   
   com.ibm.etools.cde.CDEHack.fixMe() ;  
   if (fSourceDecoder != null)
     fSourceDecoder.dispose() ; 
   fSourceDecoder = (IDiagramSourceDecoder) loadTheClass(JavaDecoderClass) ;
   if (fEDomain != null)
      fSourceDecoder.setEditDomain(fEDomain) ;
      	
}


/**
 * Get the composition root
 */
public BeanSubclassComposition getModelRoot(){
	if (fRoot != null) return fRoot ;	

      fRoot = (BeanSubclassComposition) fResource.getExtent().getObjectByType(CDEUtilities.CDM_PACKAGE.getDiagramData()) ;	
	return fRoot ;
}


/**
 *   set the current RS.
 */
public   void  setResourceSet (ResourceSet rs) {
	 fResourceSet = rs ;
}

public void dispose() {
	if (fSourceDecoder != null)
	   fSourceDecoder.dispose() ;
}


/**
 * Denote that a compound operation is completed.  This
 * will actually remove code for deleted components, if needed,
 * and will flush the changes to the Java Editor.
 */
public void commitAndFlush(boolean canWait) {
    if (fSourceDecoder != null)
      fSourceDecoder.commitAndFlush(canWait) ;
}
/**
 * Async. version of commitAndFlush
 * 
 * @param listener implements the call back
 * @param marker is an optional designator of the call back.
 */
public void commitAndFlush(ISynchronizerListener listener, String marker) {
    if (fSourceDecoder != null)
      fSourceDecoder.commitAndFlush(listener,marker) ;
}

/**
 * @param delay will set up the delay time for JavaSourceSynchronizer
 *         value less than JavaSourceSynchronizer.DEFAULT_SYNC_DELAY will be ignored.
 */
public void setSynchronizerSyncDelay(int delay) {
    if (fSourceDecoder != null)
      fSourceDecoder.setSynchronizerSyncDelay(delay) ;
    else
      com.ibm.etools.jbcf.JBCFPlugin.log("setSynchronizerSyncDelay: no Source Decoder",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
}

/**
 * @param listener for Source Deocder events
 */	
public void addTranslatorListener (ISourceTranslatorListener listener) {
   if (fSourceDecoder != null)
      fSourceDecoder.addTranslatorListener(listener) ;
    else
      com.ibm.etools.jbcf.JBCFPlugin.log("setSynchronizerSyncDelay: no Source Decoder",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
}

public void removeTranslatorListener (ISourceTranslatorListener listener) {
        if (fSourceDecoder != null)
      fSourceDecoder.removeTranslatorListener(listener) ;
    else
      com.ibm.etools.jbcf.JBCFPlugin.log("setSynchronizerSyncDelay: no Source Decoder",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
}

public void setMsgRenderer (IJVEStatus mr) {
    super.setMsgRenderer(mr) ;
    if (fSourceDecoder != null)
      fSourceDecoder.setMsgRenderer(mr) ;
}
public void setEditDomain(EditDomain d) {
	fEDomain = d ;
	if (fSourceDecoder != null)
	  fSourceDecoder.setEditDomain(d) ;
}


	/**
	 * @see IDiagramModelBuilder#disconnect(boolean)
	 */
	public void disconnect(boolean clearModel) {
		fSourceDecoder.disconnect(clearModel) ;
	}


	/**
	 * @see IDiagramModelBuilder#reconnect(IFileEditorInput, IProgressMonitor)
	 */
	public void reconnect(IFileEditorInput input, IProgressMonitor pm) throws CodeGenException {
		fSourceDecoder.reconnect(input,pm) ;
	}

	/**
	 * @see com.ibm.etools.jbcf.codegen.IDiagramModelBuilder#pauseRoundTripping(boolean)
	 */
	public void pauseRoundTripping(boolean flag) throws CodeGenException {
		fSourceDecoder.pauseRoundTripping(flag) ;
	}
	public EditDomain getEditDomain(){
		return fEDomain;
	}

}