package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cdm.Annotation;
import com.ibm.etools.cdm.CDMFactory;
import com.ibm.etools.cdm.CDMPackage;
import com.ibm.etools.cdm.KeyedValue;
import com.ibm.etools.cdm.VisualInfo;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.jbcf.codegen.IDiagramModelInstance;
import com.ibm.etools.jbcf.codegen.model.BeanPart;
import com.ibm.etools.jbcf.codegen.model.IBeanDeclModel;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.codegen.util.CodeGenUtil;

/**
 * @version 	1.0
 * @author
 */
public abstract class AbstractAnnotationDecoder implements IAnnotationDecoder {
    
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
    IBeanDeclModel fModel = null ;
    IDiagramModelInstance fCompositionModel = null ;
    BeanPart       fBeanpart = null ;
    
    String         fAnnotationKey = null ;
    String         fContent = null ;

    /**
     * Constructor for AbstractAnnotationDecoder.
     */
    public AbstractAnnotationDecoder(String key,BeanPart bean) {
        super();
        fAnnotationKey = key ;
        fBeanpart = bean ;
        if (bean != null) {
           fModel = bean.getModel() ;          
           fCompositionModel = bean.getModel().getCompositionModel() ;
        }
    }
    
//    public void setAnnotationKey(String key) {
//        fAnnotationKey = key ;
//    }
    
    public String getAnnotationKey() {
        return fAnnotationKey ;
    }
        
    /*
     * @see IAnnotationDecoder#setBeanModel(IBeanDeclModel)
     */
    public void setBeanModel(IBeanDeclModel model) {
        fModel = model ;
    }

    /*
     * @see IAnnotationDecoder#getBeanModel()
     */
    public IBeanDeclModel getBeanModel() {
        return fModel ;
    }

    /*
     * @see IAnnotationDecoder#setCompositionModel(IDiagramModelInstance)
     */
    public void setCompositionModel(IDiagramModelInstance cm) {
        fCompositionModel = cm ;
    }

    /*
     * @see IAnnotationDecoder#getCompositionModel()
     */
    public IDiagramModelInstance getCompositionModel() {
        return fCompositionModel ;
    }

    /*
     * @see IAnnotationDecoder#setBeanPart(BeanPart)
     */
    public void setBeanPart(BeanPart part) {
        fBeanpart = part ;
    }

    /*
     * @see IAnnotationDecoder#getBeanPart()
     */
    public BeanPart getBeanPart() {
        return fBeanpart ;
    }


    protected KeyedValue getAnnotationValue() throws CodeGenException {
        if (fAnnotationKey == null || fBeanpart == null) 
           throw new CodeGenException ("not initialized") ; //$NON-NLS-1$

        Annotation a = CodeGenUtil.getAnnotation(fBeanpart.getRefObject()) ;
        if (a == null) return null ;
        
        VisualInfo vi = a.getVisualInfo(fCompositionModel.getDiagram()) ;
        if (vi == null) return null ;
        return vi.getKeyedValue(fAnnotationKey) ;
    }
    
    protected void setAnnotationValue (KeyedValue val) throws CodeGenException {
        if (fAnnotationKey == null || fBeanpart == null) 
           throw new CodeGenException ("not initialized") ; //$NON-NLS-1$

        Annotation a = CodeGenUtil.getAnnotation(fBeanpart.getRefObject()) ;
        if (a == null) throw new CodeGenException ("not initialized") ; //$NON-NLS-1$
        
        VisualInfo vi = a.getVisualInfo(fCompositionModel.getDiagram()) ;
        if (vi == null)  {
            // Create a new Visual Info
            CDMPackage cPkg = (CDMPackage) RefRegister.getPackage(CDMPackage.packageURI);
	        CDMFactory cFact = cPkg.getCDMFactory();
	        vi = cFact.createVisualInfo() ;	        
            vi.setDiagram(fCompositionModel.getDiagram()) ;
            a.getVisualInfos().add(vi) ;
            
            ICodeGenAdapter adapter = (ICodeGenAdapter)a.getAdapter(ICodeGenAdapter.JVE_CODEGEN_ANNOTATION_ADAPTER) ;
            if (adapter == null) throw new CodeGenException ("No Adapter") ; //$NON-NLS-1$
            vi.addAdapter(adapter) ;
        }
        vi.setKeyedValue(fAnnotationKey,val) ;
    }
    
    /*
     * @see IAnnotationDecoder#isDeleted()
     */
          
    public boolean isDeleted() throws CodeGenException {
        
        if (getAnnotationValue() == null) return true ;
        else return false ;
        
    }

    /*
     * @see IAnnotationDecoder#delete()
     */
    public void delete() {
    }

    /*
     * @see IAnnotationDecoder#dispose()
     */
    public void dispose() {
    }

    /*
     * @see IAnnotationDecoder#deleteFromComposition()
     */
    public void deleteFromComposition() {
    }
    
    public String toString() {
        return super.toString()+":"+fAnnotationKey+"\n\t"+fContent ; //$NON-NLS-1$ //$NON-NLS-2$
    }

}
