package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.jbcf.codegen.*;
import com.ibm.etools.jbcf.codegen.util.*;
import org.eclipse.jdt.internal.compiler.ast.*;
import com.ibm.etools.emf.ref.RefObject ;
import java.util.List;
import com.ibm.etools.java.instantiation.base.* ;
import java.util.* ;
/**
 *  This helper comes to help build a specialize add, which take arguments for other objects.
 *  e.g., JTabbedPanel's addTab() method.
 */
public abstract class AbstractContainerAddDecoderHelper extends ExpressionDecoderHelper {	
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	BeanPart            fAddedPart = null ;
	RefObject           fRootObj = null ;             // The root object that hold all object. 
	String	            fAddedIndex = null;	
	boolean 	        indexValueFound = false;

public AbstractContainerAddDecoderHelper(BeanPart bean, Statement exp,  IJavaFeatureMapper fm, IExpressionDecoder owner){
	super (bean,exp,fm,owner) ;
}

/**
 * Add SourceRange adapters to the arguments of the added component.
 */
protected void addShadowAdapters() {
	// Does not support multi expression adapters per object
	ExpressionDecoderAdapter adapter = (ExpressionDecoderAdapter) getRootObject(false).getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_ADAPTER) ;
	if (adapter != null) {
	   IJavaObjectInstance[] args = getComponentArguments(getRootObject(true)) ;
	   for (int i = 0; i < args.length; i++) 
	   	  if (args[i] != null && 
	   	      args[i].getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_SOURCE_RANGE) == null) {
		   args[i].addAdapter(adapter.getShadowSourceRangeAdapter()) ;		     		  
	      }		
	}	
}

protected void removeShadowAdapters() {
	// Does not support multi expressions adapters per object
	IJavaObjectInstance[] args = getComponentArguments(getRootObject(false)) ;
	for (int i = 0; i < args.length; i++) 
  	  if (args[i] != null) {
  	  	ICodeGenAdapter a =  (ICodeGenAdapter)args[i].getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_SOURCE_RANGE) ;
  	  	// Only remove shadow adapters
  	  	if (a != null && !(a instanceof ExpressionDecoderAdapter))
  	  	  args[i].removeAdapter(a) ;
      }		
			
}

/**
 * Overide the default action, as we are maintained as a list
 */
public void adaptToCompositionModel(IExpressionDecoder decoder) {
	com.ibm.etools.cde.CDEHack.fixMe("Need to clean this up") ; //$NON-NLS-1$

    // Make sure we do not have stale, need to be a bit smarter of how we do this.
    UnadaptToCompositionModel() ;
    BeanDecoderAdapter ba = (BeanDecoderAdapter) fbeanPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;
		
	ExpressionDecoderAdapter adapter = new ExpressionDecoderAdapter(fAddedPart.getRefObject(),decoder) ;		   
	ba.addChildAdapter(fAddedPart,adapter) ;
	RefObject root = getRootObject(false) ;
	if (root != null) {
	   root.addAdapter(adapter) ;
	   // Add Source Range adapters to the arguments, if needed
	   addShadowAdapters() ;
	}
	   	
}

/**
 * Most helpers should consider overiding this one
 */
public void UnadaptToCompositionModel() {
	com.ibm.etools.cde.CDEHack.fixMe("Need to clean this up") ; //$NON-NLS-1$
    
    if (fbeanPart.getRefObject() == null) {
    	com.ibm.etools.jbcf.JBCFPlugin.log("UnadaptToCompositionModel: No RefObject",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ;    	 //$NON-NLS-1$ //$NON-NLS-1$
    	return ;
    }
    
    BeanDecoderAdapter ba = (BeanDecoderAdapter)fbeanPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;		
	ba.removeChildAdapter(fAddedPart) ;
	RefObject root = getRootObject(true) ;
	
	if (root != null) {
	   ExpressionDecoderAdapter adapter = (ExpressionDecoderAdapter) root.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_ADAPTER) ;
	   if (adapter != null) 
	     root.removeAdapter(adapter) ;
	   removeShadowAdapters() ;
	}
}





/*
 * Decoder Specific Add 
 */
protected void add(BeanPart toAdd,BeanPart target) {
	add(toAdd, target, -1);
}

/*
 * Decoder Specific Add 
 * @return RefObject the ConstraintComponent object
 */
protected abstract RefObject add(BeanPart toAdd,BeanPart target, int index)  ;
/**
 * @return The added component instance
 **/
protected abstract IJavaObjectInstance getComponent (RefObject root) ;
/**
 * @return Array of constraint instances associated with getComponent() 
 *          Note: the returned array may contain null entries.
 **/
protected abstract IJavaObjectInstance[] getComponentArguments(RefObject root) ;


/**
 * @return List of all the components aggregated by fbeanPart
 */
protected List getRootComponentList() {
        
    List compList=null ;
    try {
       compList = (List) fbeanPart.getRefObject().refValue(fFmapper.getFeature(null));
    }
    catch (Exception e) { 
    }    
    // Look for out added part
    if (compList != null && compList.size()==0) return null ;
    return compList ;
    
}

/**
 * @return RefObject denoting the root object associated this expression (e.g., CC)
 */
protected RefObject getRootObject(boolean cache) {
	if (fRootObj != null && cache) return fRootObj ;
    List compList=getRootComponentList() ;
    // Look for out added part
    RefObject targetRoot = null ;    
    if (compList != null && compList.size()>0) {
    	// This component has childrents, Process them first, so that we 
    	// can add them as we generate the current method
    	Iterator itr = compList.iterator() ;
    	while (itr.hasNext()) {
    		try {
    		  RefObject root = (RefObject)itr.next() ;
    		  IJavaObjectInstance child = getComponent(root) ;
    		  if (child.equals(fAddedPart.getRefObject())) {
    			targetRoot = root ;
    			break ;
    		  }     			
    		}
    		catch (Exception e) {}
    	}    	
    }
    if (targetRoot != null)
       fRootObj = targetRoot ;
    return targetRoot ;
}

protected void clearPreviousIfNeeded() {

    List compList=getRootComponentList() ;
    // Look for out added part
    RefObject targetRoot = getRootObject(false) ;
    if (targetRoot != null && compList != null && compList.size()>0) {
    	compList.remove(targetRoot) ;
    }
    fAddedPart.setContainer(null) ;
}

protected abstract BeanPart parseAddedPart(MessageSend exp) throws CodeGenException ;


/**
 *   Add new Componet to target Bean,
 */
protected boolean  addComponent () throws CodeGenException {
	
	BeanPart oldAddedPart = fAddedPart ;
	
	fAddedPart = parseAddedPart((MessageSend)fExpr) ;
	if (fAddedPart == null) throw new CodeGenException("No Added Part") ; //$NON-NLS-1$
		    
    if (oldAddedPart != null)
       if (fAddedPart != oldAddedPart) {
       	BeanPart parent = oldAddedPart.getContainer() ;
       	if (parent.equals(fbeanPart))  // May have moved to another container already
       	    oldAddedPart.setContainer(null) ;
       }
          
	
    clearPreviousIfNeeded() ;
      
    Expression[] args = ((MessageSend)fExpr).arguments ;      
    return parseAndAddArguments(args);
}


protected abstract boolean parseAndAddArguments(Expression[] args) throws CodeGenException ;


/**
 *  Look for our added part, 
 */
public boolean primIsDeleted() {
	
    RefObject rootObj = getRootObject(false) ;
    if (rootObj == null) return true ;
    else return false ;	
}



/**
 *   Overidable test: Is this expression fits this decoder
 */
protected abstract boolean isMySigniture() ;

/**
 *   Go for it
 */
public boolean decode() throws CodeGenException {

   if (fFmapper.getFeature(fExpr) == null ||
	   fExpr == null) throw new CodeGenException ("null Feature:"+fExpr) ; //$NON-NLS-1$

   if (isMySigniture())	   
      return addComponent() ;
   else
      return false ;	   
}

public void delete() {
	UnadaptToCompositionModel() ;
	clearPreviousIfNeeded() ;	
}



public Object getPriorityOfExpression(){
	Integer pri = (Integer)super.getPriorityOfExpression();
	int indexOfChildInParent = 0;
	// Always put the expression in the same order as it is in the model
	List rootCompoments = getRootComponentList() ;
	for(int i=0;i<rootCompoments.size();i++){
		RefObject component = getComponent((RefObject)rootCompoments.get(i));
		if(component.equals(fAddedPart.getRefObject())){
			indexOfChildInParent = i;
			break;
		}	
	}
	indexOfChildInParent++;
	return new Integer(IJavaFeatureMapper.PRIORITY_ADD_CHANGE + pri.intValue() + (IJavaFeatureMapper.INTER_LAYOUT_ADD_PRIORITIES_GAP-indexOfChildInParent));}



/**
 *  Overide the abstract method, to deal No Decorations.
 */
protected abstract ExpressionTemplate getExpressionTemplate() throws CodeGenException ;

protected abstract void primRefreshArguments() ;

public String generate(Object[] args) throws CodeGenException {

 	
	if (fFmapper.getFeature(null) == null) 
	    throw new CodeGenException ("null Feature") ; //$NON-NLS-1$
	if (args == null || args[0] == null)
	    throw new CodeGenException ("ContainerAddDecoderHelper.generate() : no Target") ; //$NON-NLS-1$
	    
	RefObject root = (RefObject) args[0] ;
    IJavaObjectInstance comp = getComponent(root) ;      
    fAddedPart = fbeanPart.getModel().getABean(comp) ;
    if (fAddedPart!=null) {        
      fbeanPart.addChild(fAddedPart) ;
      fAddedPart.setContainer(fbeanPart) ;
    }
 //   fAddedConstraintInstance = CodeGenUtil.getCCconstraint(CC) ;
      
    primRefreshArguments() ;
	    		
	ExpressionTemplate exp = getExpressionTemplate() ;
   fExprSig = exp.toString() ;	                                                 
   return fExprSig ;	     	
}

 
public boolean isImplicit(Object args[]) {
    // TBD
    return false ;
}

public Object[] getArgsHandles(Statement expr) {
	Object[] result = null ;    
	try {
      if (fAddedPart == null && expr != null) {
        // Brand new expression
	    BeanPart bp = parseAddedPart((MessageSend)expr) ;
	    if (bp != null) 
	      result = new Object[] { bp.getType()+"["+bp.getSimpleName()+"]" } ; //$NON-NLS-1$ //$NON-NLS-2$
	   }
       else if (fAddedPart != null) {
           return new Object[] { fAddedPart.getType()+"["+fAddedPart.getSimpleName()+"]" } ; //$NON-NLS-1$ //$NON-NLS-2$
       }
    }    
	catch (CodeGenException e) {}
	return result ;
}

/**
 * Prefix/PostFix quoates.
 */
protected void setInitString (IJavaObjectInstance obj, String val) {
    obj.setInitializationString(BeanUtilities.createStringInitString(val)) ;
}

/**
 * Try to figure out where to insert this object, with accordance to its position
 * in the code, in relation to other components.
 */
protected int findIndex (BeanPart toAdd,BeanPart target) {
    
     if (fOwner.getExprRef().getOffset()<0) return -1 ;
    
     java.util.List compList = (java.util.List) target.getRefObject().
                               refValue(fFmapper.getFeature(null));                               
     if (compList == null || compList.size() == 0) return -1 ;
     
     int thisOffset = fOwner.getExprRef().getOffset() ;
     int result = -1 ;
     for (int i = compList.size()-1; i >= 0; i--) {
        try {
          RefObject root = (RefObject) compList.get(i) ;
          ICodeGenAdapter a = (ICodeGenAdapter) root.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_ADAPTER) ;
          if (a!= null && a.getBDMSourceRange().getOffset()>thisOffset)
            result = i ;        
        }
        catch (CodeGenException e) {}
     }
     
     return result ;     
}
 

}


