package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.RefStructuralFeature ;
import com.ibm.etools.java.instantiation.base.* ;
import org.eclipse.jdt.internal.compiler.ast.* ;
import com.ibm.etools.jbcf.codegen.util.* ;
import com.ibm.etools.emf.ecore.EClass ;
import com.ibm.etools.emf.ref.RefObject ;
import java.util.* ;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.codegen.* ;
import com.ibm.etools.jbcf.* ;



public abstract class AbstractExpressionDecoder implements IExpressionDecoder {
	

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
Statement                   fExpr = null ;
CodeExpressionRef           fExprRef = null ;
IJavaFeatureMapper          fFeatureMapper = null ;
IExpressionDecoderHelper    fhelper = null ;

IBeanDeclModel              fBeanModel = null ;
IDiagramModelInstance       fCompositonModel = null ;
BeanPart                    fbeanPart  = null ;
String                      fdebugString = null ;
Object                      fPriority = null;



public AbstractExpressionDecoder(CodeExpressionRef expr, IBeanDeclModel model, IDiagramModelInstance bldr, BeanPart part) {
	fExpr = (Statement) expr.getExpression() ;
	fExprRef = expr ;
	fExprRef.setDecoder(this) ;
	fBeanModel = model ;
	fCompositonModel = bldr ;
	fbeanPart = part ;
	fdebugString = fExpr.toString() ;
}
	
public AbstractExpressionDecoder() {
	super () ;
}

protected abstract void initialFeatureMapper () ;
protected abstract void initialFeatureMapper(RefStructuralFeature sf) ;
protected abstract void initialDecoderHelper() ;

public Object determinePriority(){
    
    if (!Initialize()) return new Integer (fFeatureMapper.PRIORITY_DEFAULT) ;
    
	// If instanceof ContainerAddDecoderHelper, then the expressions
	// priority keeps changing based on the position of the expression
	// inside the containers children.
	if((fPriority==null) || (!isPriorityCacheable())) 
		fPriority = fhelper.getPriorityOfExpression();
	return fPriority;
}

/**
 * The decoder should decide based on which helper 
 * it gives, wether or not the priority should be cached.
 * Since it depends so much on the helper, this method's
 * decision should carefully mirror the decision taken to
 * allocate a helper.
 */
protected abstract boolean isPriorityCacheable();

IExpressionDecoderHelper getHelper() {
    if (fhelper == null) {
	    fPriority = null ; 
	    initialDecoderHelper() ;
	}
	return fhelper ;
}

/**
 * Make sure a IJavaFeatureMapper, and a IExpressionDecoderHelper have been
 * allocated to the specific expression at hand. 
 */
protected synchronized boolean Initialize() {

    if (fFeatureMapper == null) 
      	initialFeatureMapper () ;
    fFeatureMapper.setRefObject((IJavaInstance)fbeanPart.getRefObject()) ;

    // Make sure we could resolve the proper PD/SF           		
	if (fFeatureMapper.getFeature(fExpr) == null && !fExprRef.isStateSet(fExprRef.STATE_NO_OP)) {
	   JBCFPlugin.log ("**** Could not match a Feature on : "+fFeatureMapper.getMethodName()+" - "+this, JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$ //$NON-NLS-2$
	   return false ;
	}
	
	if (getHelper()==null) return false ;
	if (fExpr!=null)
		getHelper().setDecodingContent(fExpr);

   return true ;   	
}

protected boolean Initialize(RefStructuralFeature sf) {
	
	if (fFeatureMapper == null)
	    initialFeatureMapper(sf) ;
	initialDecoderHelper() ;	
	return true ;
}


/**
 *  This is it for now
 */
public boolean decode () throws CodeGenException {

    com.ibm.etools.cde.CDEHack.fixMe("Workaround for constraints") ; //$NON-NLS-1$
    
    if ((getExprRef().getExpression()!=null) && (getExprRef().getExpression() != fExpr))
    	fExpr = getExprRef().getExpression();
    	
    if (fFeatureMapper != null && fFeatureMapper.getMethodName() == null)
       fFeatureMapper = null ;
    
    // If it has a Simple helper, give it a change to get promoted to a (heavy weight) delegate decoder.
    // Failure will re-create a Simple helper   
    if (fhelper!=null && (fhelper instanceof SimpleAttributeDecoderHelper)) 
       fhelper = null ;
       
	if (!Initialize()) return false ;
	
	// Go for it
	boolean result = false ;
	try {
	   result = fhelper.decode() ;
	}
	catch (Exception e) {}

	if (!result && !(fhelper instanceof SimpleAttributeDecoderHelper)){
	    // A specialized decoded had faild.  Try to process with a simple Attribute Decoder
	    fhelper = new SimpleAttributeDecoderHelper(fbeanPart, fExpr,  fFeatureMapper,this) ;	
	    result = fhelper.decode() ;
	    determinePriority() ;
	}
	if (result) {
	   //fExprRef.setState(fExprRef.getState() | CodeExpressionRef.STATE_EXIST | CodeExpressionRef.STATE_IN_SYNC) ;
	   fExprRef.setState(fExprRef.STATE_EXIST, true);
	   fExprRef.setState(fExprRef.STATE_IN_SYNC, true);
	   fhelper.adaptToCompositionModel(this) ;
	}
	return result ;
}


/**
 *  This is it for now
 */
public String generate(RefStructuralFeature feature, Object[] args) throws CodeGenException {
	
	if (!Initialize(feature)) return null ;
	if (isImplicit(args) && !fhelper.isGenerateOnImplicit())
	  return null ;
	
	
	// Go for it

	String result = null ;	
	try {
	    result = fhelper.generate(args) ;
	}
	catch (CodeGenException e) {}
	
	if (result == null && !(fhelper instanceof SimpleAttributeDecoderHelper)) {
	    // Specialized decoder may not be applicable, try a vanilla one
	    fhelper = new SimpleAttributeDecoderHelper(fbeanPart, fExpr,  fFeatureMapper, this) ;
        JBCFPlugin.log("generate():  *Defaulting* to a SimpleAttr. Helper",JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
	    result = fhelper.generate(args) ;
	}

	if (result!=null) {
	   //fExprRef.setState(fExprRef.getState() | CodeExpressionRef.STATE_EXIST | CodeExpressionRef.STATE_IN_SYNC) ;
	   fExprRef.setState(fExprRef.STATE_EXIST, true);
	   fExprRef.setState(fExprRef.STATE_IN_SYNC, true);
	   fhelper.adaptToCompositionModel(this) ;
	}
	else {
	   fExprRef.dispose() ;	   
	}
		
		
	fdebugString=result ;
	return result ;
}


public void setExpression(CodeExpressionRef expr) {
	fExprRef = expr ;
	fExpr =  (Statement) expr.getExpression() ;
	fExprRef.setDecoder(this) ;	
	if (fExpr != null)
	   fdebugString = fExpr.toString() ;
	if (fhelper != null) 
	   fhelper.setDecodingContent(fExpr) ;		
}
public void setBeanModel(IBeanDeclModel model) {
	fBeanModel= model ;
}

public IBeanDeclModel getBeanModel() {
	return fBeanModel ;
}

public void setCompositionModel(IDiagramModelInstance cm) {
	fCompositonModel = cm ;
}

public IDiagramModelInstance getCompositionModel() {
	return fCompositonModel ;
}

public void setBeanPart(BeanPart part) {
	fbeanPart = part ;
}

/**
 *  Mark as delete, And Remove from document
 */
public void delete () {
	fhelper.UnadaptToCompositionModel() ;
	fExprRef.clearAllFlags();
	fExprRef.setState(fExprRef.STATE_NOT_EXISTANT, true); //fExprRef.primSetState(CodeExpressionRef.STATE_NOT_EXISTANT) ;
	fExprRef.updateDocument(true) ;
}


/**
 *  Should this feature be considered as a deleted feature ?
 *  @return boolean  
 */
public boolean isDeleted () {
	boolean result = false ;
	// Some attributes should not be reflected in Java source, and hence
	// be considered as deleted (e.g., constraint of a layout manager).
//	if (MethodTextGenerator.isNotToBeCodedAttribute(fFeatureMapper.getFeature(null),fbeanPart.getRefObject()))
//	   result = true ;
	if (!result)
	   result = getHelper().primIsDeleted() ;
	if (result==true)
	   fhelper.UnadaptToCompositionModel() ;
	return result ;
}

/**
 *  refresh the status of this component 
 *  Although should not expect it to change ...
 */
 
public  boolean isImplicit (Object[] args) {
   com.ibm.etools.cde.CDEHack.fixMe() ;
   if (true) 
      return false ;
   if (getHelper().isImplicit(args) && !fhelper.isGenerateOnImplicit()) {   
	   //fExprRef.setState(CodeExpressionRef.STATE_EXIST | CodeExpressionRef.STATE_IN_SYNC |
	   //                  CodeExpressionRef.STATE_IMPLICIT | fExprRef.getState()) ;
	   fExprRef.setState(fExprRef.STATE_EXIST, true);
	   fExprRef.setState(fExprRef.STATE_IN_SYNC, true);
	   fExprRef.setState(fExprRef.STATE_IMPLICIT, true);
       fExprRef.setContent((ExpressionParser)null) ;
	return true ; 
   }else{   	 
       // fExprRef.setState(fExprRef.getState() & ~CodeExpressionRef.STATE_IMPLICIT) ;	
       fExprRef.setState(fExprRef.STATE_IMPLICIT, false);
       return false ;
   }
}

public String reflectExpression(String expSig) throws CodeGenException {
	if (!isDeleted()) {
	   if (isImplicit(null))
	     return null ;
	     
	   fdebugString =  fhelper.primRefreshFromComposition(expSig);	
	   return fdebugString ;
	}
	else {
	   fdebugString = null ;
	   return null ;
	}
}

public void reflectMOFchange () {
	fExprRef.setState(fExprRef.STATE_IN_SYNC, false); //fExprRef.setState(fExprRef.getState()&~CodeExpressionRef.STATE_IN_SYNC) ;
	if((!fExprRef.isAnyStateSet()) || fExprRef.isStateSet(fExprRef.STATE_NOT_EXISTANT)) // (fExprRef.getState() == CodeExpressionRef.STATE_NOT_EXISTANT) 
	   return ;
	fExprRef.updateDocument(true) ;
}

public String getCurrentExpression()  {
	if (!isDeleted())
	   return fhelper.getCurrentExpression() ;
	else
	   return null ;
}


public RefStructuralFeature getSF() {
    Initialize() ;	
	RefStructuralFeature sf = null ;
	if (fFeatureMapper != null) {
		sf = fFeatureMapper.getFeature(null) ;
	}
	return sf ;
}

public void setSF(RefStructuralFeature sf) {
	Initialize() ;
	if (fFeatureMapper != null) {
	   fFeatureMapper.setFeature(sf) ;
	   // ReDrive
	   Initialize() ;
	}
}
public BeanPart getBeanPart() {
    return fbeanPart ;
}

public synchronized void dispose() {
	if (fhelper != null)
	   fhelper.UnadaptToCompositionModel() ;
	fFeatureMapper = null ;
	fhelper = null ;
	fdebugString = null ;
}

public synchronized void deleteFromComposition () {
  fhelper.delete() ;
}

public Object[] getArgsHandles(Statement expr) throws CodeGenException {
	   if (!Initialize()) 
	       throw new CodeGenException ("Can not Initialize") ; //$NON-NLS-1$
	   
	   return fhelper.getArgsHandles(expr) ;
}

public CodeExpressionRef getExprRef() {
    return fExprRef ;
}


/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	return super.toString() + ":" + fdebugString ; //$NON-NLS-1$
}



	/**
	 * @see com.ibm.etools.jbcf.codegen.java.IExpressionDecoder#isRelevantFeature(RefStructuralFeature)
	 */
	public boolean isRelevantFeature(RefStructuralFeature sf) {
		return fhelper.isRelevantFeature(sf);
	}

}