package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cdm.VisualInfo;
import com.ibm.etools.emf.notify.Notification;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.model.* ;



/**
 * @version 	1.0
 * @author
 */
public class AnnotationDecoderAdapter extends AdapterImpl implements ICodeGenAdapter {

	  
  
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
  protected IAnnotationDecoder fDecoder=null ;


public AnnotationDecoderAdapter(Notifier target,IAnnotationDecoder decoder) {
	super() ;
	this.target = target ;
	fDecoder = decoder ;
}

/**
 * isAdaptorForType.
 */
public boolean isAdapterForType(Object type) {
	return VCE_CODE_GEN_TYPE.equals(type)||
	        JVE_CODEGEN_ANNOTATION_ADAPTER.equals(type);
}

/**
 * applied: A setting has been applied to the mof object,
 * notify the decoder
 */ 
public void notifyChanged(Notification msg){

  try {
      
      if (fDecoder.getBeanModel() == null  || fDecoder.getBeanPart().getModel() == null ||
          fDecoder.getBeanPart().getModel().isStateSet(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL) ||
          fDecoder.getBeanPart().getModel().isStateSet(IBeanDeclModel.BDM_STATE_SNIPPET)) return  ;
      
      if (msg.getStructuralFeature() == null) return ;
      String action=null ;
	switch ( msg.getEventType() ) {
		case Notifier.TOUCH:
		     action = "TOUCH" ; //$NON-NLS-1$
		     break ;		     
		case Notifier.SET : 
		     action = "SET" ;		      //$NON-NLS-1$
		     break ;
		case Notifier.ADD :
		     action = "ADD" ; //$NON-NLS-1$
		     break ;
		case Notifier.UNSET : 
		     action = "UNSET" ; //$NON-NLS-1$
		     break ;
		case Notifier.REMOVE :
		     action = "REMOVE" ; //$NON-NLS-1$
		     break ;
		case Notifier.REMOVING_ADAPTER:
		     action = "REMOVING_ADAPTER" ;		      //$NON-NLS-1$
		     break;
	}
	JBCFPlugin.log(this+" action= "+action, JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
JBCFPlugin.log("SourceRange ="+getJavaSourceRange(), JBCFMsgLogger.LOG_FINE) ;	 //$NON-NLS-1$
	
	if (msg.getNewValue() instanceof Notifier)
	  this.target = (Notifier)msg.getNewValue();
	  
      switch (msg.getEventType()) {
        case Notifier.TOUCH:
        case Notifier.ADD :
              if (msg.getNewValue() instanceof VisualInfo) {
                  ((RefObject)msg.getNewValue()).addAdapter(this) ;
                  return ;
              }
        case Notifier.SET:     
        case Notifier.UNSET:
        case Notifier.REMOVE:
             if (msg.getOldValue() instanceof VisualInfo) {
                  ((RefObject)msg.getOldValue()).removeAdapter(this) ;
             }
        case Notifier.MOVE:        
	    fDecoder.reflectMOFchange() ;		     
	    break ;      	
      }
  }
  catch (Throwable t) {
      JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING) ;
  }
}


public IAnnotationDecoder getDecoder() {
	return fDecoder ;
}

/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	// Insert code to print the receiver here.
	// This implementation forwards the message to super.
	// You may replace or supplement this.
	return "\tAnnotationDecoderAdapter:"+fDecoder; //$NON-NLS-1$
}

public ICodeGenSourceRange getJavaSourceRange() throws CodeGenException {
    //return fDecoder.getExprRef().getTargetSourceRange() ;
    return null ;
}

/**
 * @see ICodeGenAdapter#getBDMSourceRange()
 */
public ICodeGenSourceRange getBDMSourceRange() throws CodeGenException {
    return null;
}

/**
 * @see com.ibm.etools.jbcf.codegen.java.ICodeGenAdapter#getSelectionSourceRange()
 */
public ICodeGenSourceRange getHighlightSourceRange()
	throws CodeGenException {
	return getJavaSourceRange();
}

}
