package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.RefStructuralFeature ;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.beaninfo.IIntrospectionAdapter;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.java.instantiation.base.IJavaInstance ;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;

import org.eclipse.jdt.internal.compiler.ast.* ;
import com.ibm.etools.beaninfo.adapters.Utilities ;
import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.util.* ;
import com.ibm.etools.jbcf.visual.JBCFConstants;
import com.ibm.etools.emf.ecore.EClass ;
import com.ibm.etools.emf.ecore.EModelElement;
import com.ibm.etools.emf.ref.RefObject ;
import java.util.* ;
import com.ibm.etools.beaninfo.PropertyDecorator ;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.Method ;
import com.ibm.etools.emf.ref.RefStructuralFeature ;
import com.ibm.etools.emf.ecore.EStructuralFeature ;
import com.ibm.etools.emf.ref.EList; 

public class AttributeFeatureMapper extends AbstractFeatureMapper {


	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
static HashMap     fdecoratorCache = null ;   
static public ResourceSet fdecoratorRSCache = null ;


private List primGetDecorators(RefObject metaObject) {
    
     JBCFPlugin.log("AttributeFeatureMapper.primGetDecorators: "+metaObject.refID(),com.ibm.etools.jbcf.JBCFMsgLogger.LOG_FINEST) ; //$NON-NLS-1$
     EList properties = ((EClass)metaObject).getEAllAttributes() ;
     Iterator itr = Utilities.getPropertiesIterator(properties);
     ArrayList list = new ArrayList() ;
     while (itr.hasNext()) {
         list.add(itr.next()) ;
     }
     return list ;    
}

/**
 * We are going to catch the PropertyDecorators
 */
protected Iterator getPropertiesIterator(RefObject obj) {
    
    RefObject metaObject = obj.refMetaObject() ;
    if (metaObject==null || metaObject.refResource()==null) return null ;
    ResourceSet rs ;
    if (obj.refResource() != null && obj.refResource().getResourceSet() != null)
       rs = obj.refResource().getResourceSet() ;
    else
       rs = metaObject.refResource().getResourceSet() ;
    if (rs == null) return null ;
    
    if (fdecoratorRSCache==null || rs != fdecoratorRSCache) {
        // Poor Man's cache at this point
        JBCFPlugin.log("AttributeFeatureMapper.getPropertiesIterator():  *ClearingCache*",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
        fdecoratorCache = new HashMap() ;
        fdecoratorRSCache = rs ;
    }
    List decorators = null ;
    String handle = ((JavaClass)(metaObject)).getJavaName() ;
    decorators = (List) fdecoratorCache.get(handle) ;
    // This does not really do much, as it is cleared once BeanInfo recycles
    IIntrospectionAdapter ia = (IIntrospectionAdapter)metaObject.getAdapter(IIntrospectionAdapter.ADAPTER_KEY);    
    if (decorators == null || ia==null || ia.isStale()) {  // Get them again.
       decorators = primGetDecorators(metaObject) ;
       fdecoratorCache.put(handle, decorators) ;
    }
    if (decorators != null)
      return decorators.iterator() ;
    else
      return null ;
}

/**
 * Force a feature mapper to become sf
 */
protected void hardCode(RefStructuralFeature sf) {
	fSF = sf ;
	fPD = Utilities.getPropertyDecorator( (EModelElement) sf);	
}

/**
 * The assumption is that it is a vanilla attribute, and a PropertyDecorator exists
 */
public RefStructuralFeature getFeature (Statement expr) {
	
     if (fSF != null) return fSF ;
     if (fRefObj == null || expr==null) return null ;
     	    
     getMethodName(expr) ;
     Expression args[] = null ;
     if (expr instanceof MessageSend)
        args = ((MessageSend)expr).arguments ;

     
//     com.ibm.etools.cde.CDEHack.fixMe("Need to hash these once per bean part") ; //$NON-NLS-1$
//     EList properties = ((EClass)(((RefObject)fRefObj).refMetaObject())).getEAllAttributes() ;
//     Iterator itr = Utilities.getPropertiesIterator(properties);
     Iterator itr = getPropertiesIterator((RefObject)fRefObj) ;
     // Find a write method that matches the one in the Expression.
     while (itr.hasNext()) {
     	try {
     	   PropertyDecorator pd = (PropertyDecorator) itr.next();
     	
     	   Method m = pd.getWriteMethod() ;
     	   if (m == null) continue ;
     	   com.ibm.etools.cde.CDEHack.fixMe("Need to check for types as well") ; //$NON-NLS-1$
     	   
     	   if (! m.getName().equals(fMethodName)) continue ;
     	   com.ibm.etools.cde.CDEHack.fixMe("Need to have a special Constraint mapper") ; //$NON-NLS-1$
     	   if ((args!=null && (m.listParametersWithoutReturn().length != args.length)) &&
     	       !fMethodName.equals(IJavaFeatureMapper.CONSTRAINT_BOUND) &&
     	       !fMethodName.equals(IJavaFeatureMapper.CONSTRAINT_SIZE) &&
     	       !fMethodName.equals(IJavaFeatureMapper.LOCATION_NAME) )
     	      continue ;
     	   
     	
     	   fSF = (RefStructuralFeature) pd.getEDecorates() ;
     	   fSFname = ((EStructuralFeature)fSF).getName() ;
     	   fPD = pd ;
     	   break ;   	     	
     	}
     	catch (Exception e) {
     	   JBCFPlugin.log ("AttributeFeatureMapper.getFeature() : "+e, JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
     	   continue ;
     	}
     }
     if (fSF == null) {
         // Try to see if our write method is actually a public field access
         CDEHack.fixMe("We really needs to decorate this kind of information") ; //$NON-NLS-1$
         EList properties = ((EClass)(((RefObject)fRefObj).refMetaObject())).getEAllAttributes() ;
         for (itr = properties.iterator(); itr.hasNext();) {
           EStructuralFeature sf = (EStructuralFeature) itr.next();
           if (fMethodName.equals(sf.getName()) && isFieldAccess(sf)) {               
               fSF = (RefStructuralFeature) sf ;
               fSFname = sf.getName();
               fisMethod=false ;
               break ;
           }
            
        }
     }
     // This is a tempory solution for the possibility that size or bound were not exposed
     // by BeanInfo, used the cached SF of component
     if (fSF == null && (fRefObj instanceof IJavaObjectInstance)) {
     	if (fMethodName.equals(IJavaFeatureMapper.CONSTRAINT_SIZE)) 
     		hardCode(JavaInstantiation.getSFeature((IJavaObjectInstance)fRefObj, JBCFConstants.SF_COMPONENT_SIZE));     	
     	else if (fMethodName.equals(IJavaFeatureMapper.CONSTRAINT_BOUND))
     	    hardCode(JavaInstantiation.getSFeature((IJavaObjectInstance)fRefObj, JBCFConstants.SF_COMPONENT_BOUNDS));
     	else if (fMethodName.equals(IJavaFeatureMapper.LOCATION_NAME))
     	    hardCode(JavaInstantiation.getSFeature((IJavaObjectInstance)fRefObj, JBCFConstants.SF_COMPONENT_LOCATION));
     	     
        if (fSF != null) {
     	   fSFname = ((EStructuralFeature)fSF).getName() ;     		
     	}
     }   		
     return fSF ;		
}


/**
 * @see com.ibm.etools.jbcf.codegen.java.IJavaFeatureMapper#getPriorityIncrement(String)
 */
public int getPriorityIncrement(String methodType) {
	if (methodType.equals("setLayout")) //$NON-NLS-1$
		return IJavaFeatureMapper.PRIORITY_LAYOUT_CHANGE;
	if(methodType.equals("setModel")) //$NON-NLS-1$
		return 1;
	if(methodType.equals("setAutoCreateColumnsFromModel")) //$NON-NLS-1$
		return 2;
	return super.getPriorityIncrement(methodType);
}

public static void clearCache() {
	fdecoratorRSCache = null ;
	fdecoratorCache = null ;
}

}