package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.* ;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notification;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.impl.AdapterImpl ;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.* ;
import java.util.* ;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.ISourceRange;
import org.eclipse.ui.part.FileEditorInput;

import com.ibm.etools.java.instantiation.base.IJavaObjectInstance ;
import com.ibm.etools.jbcf.codegen.util.* ;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;

/**
 *  This Adapter represents the VCE model listener for a component.
 *  It will add/remove sub-components, and associated expressions
 */

public class BeanDecoderAdapter extends AdapterImpl implements ICodeGenAdapter , IAdaptable {
	  
  
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
  protected  BeanPart fBean=null ;			   // Component associated with this adapter
  Hashtable  fNullValuedSF = new Hashtable() ;    // Hold Exp. Decoder Adapters for null values
  Hashtable  fChildrens    = new Hashtable() ;    // Hold Exp. Decoder Adapters for components SF
  
  String     traceMsg[] = { "CREATE", //$NON-NLS-1$
	                    	"SET", //$NON-NLS-1$
	                    	"UNSET",    //$NON-NLS-1$
          	            	"ADD", //$NON-NLS-1$
 	                    	"REMOVE", //$NON-NLS-1$
	                    	"ADD_MANY", //$NON-NLS-1$
 	                    	"REMOVE_MANY", //$NON-NLS-1$
	                    	"MOVE", //$NON-NLS-1$
                        	"TOUCH", //$NON-NLS-1$
                        	"REMOVING_ADAPTER" } ; //$NON-NLS-1$
                                  
  


public BeanDecoderAdapter(Notifier target,BeanPart bean) {
	super() ;
	this.target = target ;
	fBean       = bean ;
}

/**
 * isAdaptorForType.
 */
public boolean isAdapterForType(Object type) {
	return VCE_CODE_GEN_TYPE.equals(type) ||	        
	        JVE_CODEGEN_BEAN_PART_ADAPTER.equals(type);
}




/**
 *  Create a new BeanPart instance and it relevant source code
 */
protected void  createBeanInstance(IJavaObjectInstance obj) {

    if (obj == null) return ;
    
    BeanPart bp = fBean.getModel().getDeleteDesignated(obj, true) ;
    if (bp != null) {
        // Bring back to life childrens as well
        Iterator itr = fBean.getModel().getABean(obj).getChildren() ;
        while (itr != null && itr.hasNext()) {
            BeanPart b = (BeanPart) itr.next() ;
            createBeanInstance((IJavaObjectInstance)b.getRefObject()) ;         
        }
        com.ibm.etools.cde.CDEHack.fixMe("We need to check if any attribute was set/removed and update the source") ; //$NON-NLS-1$
        return ;
    }
    
    BeanPartGenerator bgen = new BeanPartGenerator(fBean.getModel(),fBean.getModel().getCompositionModel()) ;
    try {
      bgen.createFromVCEModel(obj,fBean.getModel().getCompilationUnit()) ;
    }
    catch (com.ibm.etools.jbcf.codegen.util.CodeGenException e) {
      JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
      return ;
    }
}

/**
 *  Remove an existing BeanPart instance and its relevant source code
 *  Use the removeChidrenIfNeeded to prune a branch
 */
protected void  removeBeanInstance(IJavaObjectInstance obj) {

         if (obj == null) return ;
         
         fBean.getModel().designateAsDelete(fBean.getModel().getABean(obj)) ;
         
         // Do not remove now, this obj may be added back in later on.         
//         BeanPartGenerator bgen = new BeanPartGenerator(fBean.getModel(),fBean.getModel().getCompositionModel()) ;
//         bgen.removeBeanPart(fBean.getModel().getABean(obj)) ;
}

/**
 *  Determine if the Instance object is a child of the target
 *  @return boolean
 */
protected boolean isChild (IJavaObjectInstance obj) {
    if (obj == null) return false ;
    
    IExpressionDecoder decoder = ExpressionDecoderFactory.getDecoder((IJavaObjectInstance)target) ;
    Iterator itr = decoder.getChildren((IJavaObjectInstance)target).iterator() ;
    while (itr.hasNext()) {
    	RefObject child = (RefObject)itr.next() ;
    	if (obj.equals(child)) return true ;
    	itr.next() ;
    } 
    return false ;
}

/**
 * Get the CODE_GEN adapter for a given object.  For null values, the adapter
 * is stored in fNullValuedSF
 */
protected ICodeGenAdapter getDecoderAdapter(Object obj, RefObject sf) {
	
      if (obj == null)
          return getNullValuedAdapter((RefStructuralFeature)sf) ;		
          
      if (!(obj instanceof Notifier)) 
          return null ;      
          
      ICodeGenAdapter a =  (ICodeGenAdapter) ((Notifier)obj).getAdapter(JVE_CODEGEN_EXPRESSION_ADAPTER) ;
      if (a == null) {
      	 com.ibm.etools.jbcf.JBCFPlugin.log("BeanDecoderAdapter.getDecoderAdapter() : No ExpressionAdapter <-- check me") ; //$NON-NLS-1$
         a = (ICodeGenAdapter) ((Notifier)obj).getAdapter(VCE_CODE_GEN_TYPE) ;
      }
                    
      return a ;
}


protected void removeDecoderAdapter (Object obj, ICodeGenAdapter a, RefObject sf) {
	if (a == null) return ;
      if (obj == null || !(obj instanceof Notifier))
	    removeNullValue((RefStructuralFeature)sf) ;
	else
	   ((Notifier)obj).removeAdapter(a) ;	
}

protected void addDecoderAdapeter  (Object obj, ICodeGenAdapter a, RefObject sf) {
		if (obj != null && obj instanceof Notifier) 
		    ((Notifier)obj).addAdapter(a) ;
		else
		    addNullValue((RefStructuralFeature)sf,a) ;	
}


public void NotifyParentIfNeeded (RefStructuralFeature sf) {
	if (fBean.getContainer() != null) {
		BeanDecoderAdapter a = (BeanDecoderAdapter) fBean.getContainer().getRefObject().getAdapter(JVE_CODEGEN_BEAN_PART_ADAPTER) ;
		a.ChildChanged(fBean,sf) ;
	}
}

public void ChildChanged (BeanPart child, RefStructuralFeature sf) {
	com.ibm.etools.cde.CDEHack.fixMe() ;
	if (sf.refID().equals("Component.constraints")) { //$NON-NLS-1$
	   	ExpressionDecoderAdapter a = (ExpressionDecoderAdapter) getChildAdapter(child) ;
	   	if (a != null)
	   	   a.getDecoder().reflectMOFchange() ;
	}
}

protected boolean sanityCheck(boolean flag, String msg) {
	if (!flag) {
		com.ibm.etools.jbcf.codegen.util.CodeGenException e = new com.ibm.etools.jbcf.codegen.util.CodeGenException(msg) ;
		JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
	}
	return flag ;
}

protected void removeChildrenIfNeeded (BeanPart child) {
	
	ArrayList Grandkids = new ArrayList () ;
	Iterator itr = child.getChildren() ;
	while (itr.hasNext())  Grandkids.add(itr.next()) ;
	
	for (int i=0; i<Grandkids.size(); i++) {
		BeanPart b = (BeanPart) Grandkids.get(i) ;
		removeChildrenIfNeeded (b) ;
		removeBeanInstance((IJavaObjectInstance)b.getRefObject()) ;
	}			
}


/**
 * Determine if the val is a Bean instance value
 */
protected boolean isSpecialInstanceValue (Object val) {
    
    return CodeGenUtil.isSpecialRootComponent(val) ;
}


protected Object[] processSpecialInstanceAdd(Notification msg) {

     Object[] args = null ;
     
     // TabPane or CC
     IJavaObjectInstance comp = getComponentFromSpecialRoot((RefObject)msg.getNewValue()) ;
 	 BeanPart b = fBean.getModel().getABean(comp) ;
 	 if (b == null || fBean.getModel().getDeleteDesignated(comp, false)!=null) {  
       createBeanInstance(comp) ;
       b = fBean.getModel().getABean(comp) ;
 	 }
 	 else { 
 	     JBCFPlugin.log("CheckMe ---> "+this+" AddElement existing Bean", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$ //$NON-NLS-2$
 	 }
 	 
 	 // If CC process the constraint
 	 if (CodeGenUtil.isConstraintComponentValue((RefObject)msg.getNewValue())) { 	 
 	   IJavaObjectInstance constraint = CodeGenUtil.getCCconstraint((RefObject)msg.getNewValue()) ;
 	   BeanPart cb = null ;
 	   if (!isVanillaConstraint(constraint)) {
 	     createBeanInstance(constraint) ;
 	     cb = fBean.getModel().getABean(constraint)    ;
 	     fBean.addChild(cb) ;  // **** Should consider to make the constraint the child of the component instead
 	     cb.setContainer(fBean) ;
 	   }
       args = new Object[] { msg.getNewValue() } ;		      
       fBean.addChild(b) ;
       b.setContainer(fBean) ;			 
 	 }
 	 else if (CodeGenUtil.isTabPaneComponentValue((RefObject)msg.getNewValue())) {
 	     // Process title, tooltip, icon - see visuals.xmi
 	     com.ibm.etools.cde.CDEHack.fixMe("TBD") ; //$NON-NLS-1$
 	     args = new Object[] { msg.getNewValue() } ;
         fBean.addChild(b) ;
         b.setContainer(fBean) ;    
 	 }
 	 return args ;
}


/**
 * Process Notifier.SET message
 */
protected void setElement (Notification msg) {
    
    ICodeGenAdapter oldAdapter =null ;  // Holds the expression associated with this feature, e.g., setLayout()
    Object[] args = null ;
    
         boolean childAdapter = false ;
         if (msg.getOldValue() != null && msg.getOldValue() instanceof RefObject) {
           oldAdapter = getChildAdapter(fBean.getModel().getABean((RefObject)msg.getOldValue())) ;
           childAdapter = true ;
         }
         else { 
           oldAdapter = getNullValuedAdapter((RefStructuralFeature)msg.getStructuralFeature()) ;
           // Need to see if it is a child, or just an attribute
           if (oldAdapter != null && oldAdapter instanceof ExpressionDecoderAdapter) {
	        if (msg.getNewValue() != null && 
	            (isSpecialInstanceValue(msg.getNewValue()) ||
			     isChild((IJavaObjectInstance)msg.getNewValue()))
			    )
			    childAdapter = true ;
           }
         }
           // check to make sure it is 
           
	     // If it is a child, it is already removed - can  not call isChild()
		 if (isSpecialInstanceValue(msg.getNewValue()) ||
	         isSpecialInstanceValue(msg.getOldValue()) ||
	         (msg.getNewValue() instanceof IJavaObjectInstance && isChild((IJavaObjectInstance)msg.getNewValue())) || 
	         (msg.getOldValue() instanceof IJavaObjectInstance && isChild(((IJavaObjectInstance)msg.getOldValue())))||
	         (oldAdapter != null && childAdapter)
	         ) {	// Special Relationship e.g., contentPane
	        
	        IJavaObjectInstance old = null ; 	
	        if (msg.getOldValue() != null) 
	          if (isSpecialInstanceValue(msg.getOldValue()))  
	        	old = getComponentFromSpecialRoot((RefObject)msg.getOldValue()) ;
	          else if (isChild((IJavaObjectInstance)msg.getOldValue()) ||
	                    oldAdapter!= null)
	            old = (IJavaObjectInstance) msg.getOldValue() ;
	         
	        if (old != null) {
	            BeanPart b = fBean.getModel().getABean(old) ;
	            oldAdapter = getChildAdapter(b) ;
	            fBean.removeChild(b) ;	
	            b.setContainer(null) ;
	     	    removeChildAdapter(b) ;
	     	    removeBeanInstance(old) ;
	        }
	        
	        IJavaObjectInstance newv = null ;
	        if (msg.getNewValue() != null) 
	          if (isSpecialInstanceValue(msg.getNewValue()))  
	        	newv = getComponentFromSpecialRoot((RefObject)msg.getNewValue()) ;
	          else if (isChild((IJavaObjectInstance)msg.getNewValue()))
	            newv = (IJavaObjectInstance) msg.getNewValue() ;
        		        		           
	        
	        if (newv != null) { 		          
		      createBeanInstance(newv) ;
		      BeanPart b = fBean.getModel().getABean((RefObject)newv) ;			      
		      fBean.addChild(b) ;
		      b.setContainer(fBean) ;
		      // If the adapter was sitting on the null list, we will re-create the expression/decoder, to make sure
		      // We have a proper decoder (attribute null, vs. Instance null).
		      if (oldAdapter != null && getNullValuedAdapter((RefStructuralFeature)msg.getStructuralFeature())==null)
		        addChildAdapter(b,oldAdapter) ;		        		      
		      else
		         oldAdapter = null ;
	        }
	        else	        	        
	         if (oldAdapter != null)	// Move the adapter to the new/null values list         	           
		      	addDecoderAdapeter(msg.getNewValue(),oldAdapter,msg.getStructuralFeature()) ;		     	   		     	   
		      
		        
	        args = new Object[] { msg.getNewValue() } ;
	     }
	     else {  // An attribute		     	 
	     	 oldAdapter = getDecoderAdapter(msg.getOldValue(),msg.getStructuralFeature()) ; 
	     	 if (oldAdapter != null) {
	     	   // Move the decoder to the new valued instance
	     	   removeDecoderAdapter(msg.getOldValue(),oldAdapter,msg.getStructuralFeature()) ;
	     	   addDecoderAdapeter(msg.getNewValue(),oldAdapter,msg.getStructuralFeature()) ;		     	   		     	   
	     	 }
	     	 NotifyParentIfNeeded((RefStructuralFeature) msg.getStructuralFeature()) ;	     	 
	     }
	     
		if (oldAdapter == null) { // Brand new expression
			// If we have created a new child, we may have a lingering (demoted attribute left over)
			if (msg.getOldValue() != null)
				oldAdapter = getDecoderAdapter(msg.getOldValue(), msg.getStructuralFeature());
			else {
				oldAdapter = getNullValuedAdapter((RefStructuralFeature) msg.getStructuralFeature());
				removeNullValue((RefStructuralFeature) msg.getStructuralFeature());
			}

			if (oldAdapter != null && oldAdapter instanceof ExpressionDecoderAdapter) {
				((ExpressionDecoderAdapter) oldAdapter).getDecoder().delete();
			}
			ExpressionRefGenerator eGen = new ExpressionRefGenerator(fBean, (RefStructuralFeature) msg.getStructuralFeature());
			try {
				CodeExpressionRef exp = eGen.createFromVCEModel(args);
				if (exp != null)
					exp.insertContentToDocument(true);
			}
			catch (com.ibm.etools.jbcf.codegen.util.CodeGenException e) {
				JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
				return;
			}
		}
	     else {
	     	  // Let he decoder update the document.		     		     	  
	     	  oldAdapter.notifyChanged(msg) ;
	     }		
}

protected IJavaObjectInstance getComponentFromSpecialRoot(RefObject root) {
    if (CodeGenUtil.isConstraintComponentValue(root)) 
      return CodeGenUtil.getCCcomponent(root) ;
    else if (CodeGenUtil.isTabPaneComponentValue(root))
      return (IJavaObjectInstance) root.refValue(
                   (RefStructuralFeature) root.refMetaObject().metaObject(JTabbedPaneAddDecoderHelper.COMPONENT_ATTR_NAME)
                   ) ; 
    return null ;      
}

protected IJavaObjectInstance getConstraintFromSpecialRoot(RefObject root) {
    if (CodeGenUtil.isConstraintComponentValue(root)) 
       return CodeGenUtil.getCCconstraint(root) ;
    return null ;
}

/**
 * Process Notifier.REMOVE message
 */
protected void removeElement (Notification msg) {
                        
            ICodeGenAdapter oldAdapter =null ;
            Object[] args = null ;
            
            if (msg.getOldValue() != null)
              oldAdapter = getChildAdapter(fBean.getModel().getABean((RefObject)msg.getOldValue())) ;
	         // If it is a child, it is already removed - can  not call isChild()
             if (isSpecialInstanceValue(msg.getOldValue()) || oldAdapter != null) {
		          
	           IJavaObjectInstance obj = null ;
	           IJavaObjectInstance constraint = null ;
	           if (isSpecialInstanceValue(msg.getOldValue())) {
	               obj = getComponentFromSpecialRoot((RefObject)msg.getOldValue()) ;
	               constraint = getConstraintFromSpecialRoot((RefObject)msg.getOldValue()) ;
	               if (isVanillaConstraint(constraint))
	                  constraint=null ;
	           }
	           else
	               obj = (IJavaObjectInstance) msg.getOldValue() ;
	               	           
	           if (constraint != null) {
	               BeanPart cb = fBean.getModel().getABean(constraint) ;
	               
	               fBean.removeChild(cb) ;
	               removeBeanInstance(constraint) ;
	           }
	               
	     
	     	   BeanPart b = fBean.getModel().getABean(obj) ;
	     	   oldAdapter = getChildAdapter(b) ;
	     	     
	     	   removeChildrenIfNeeded(b) ;
	     	   fBean.removeChild(b) ;
	     	   b.setContainer(null) ;
	     	   removeChildAdapter(b) ;
	     	   removeBeanInstance(obj) ;
		     }
		     else {  // An attribute		     	 		     	 
		     	 oldAdapter = getDecoderAdapter(msg.getOldValue(),msg.getStructuralFeature()) ;
		     	 if (oldAdapter != null) {
		     	   // Move the decoder to the new valued instance
		     	   removeDecoderAdapter(msg.getOldValue(),oldAdapter,msg.getStructuralFeature()) ;
		 //    	   addDecoderAdapeter(msg.getNewValue(),oldAdapter,msg.getStructuralFeature()) ;		     	   
		     	 }
		     }		     
		     if (oldAdapter != null) {
		     	  // Let he decoder update the document.		     		     	  
		     	  oldAdapter.notifyChanged(msg) ;
		     }	
}

protected boolean isVanillaConstraint(IJavaObjectInstance c) {
     return (!(c != null && !c.getJavaType().getJavaName().equals("java.lang.String"))) ;  //$NON-NLS-1$
}


/**
 * Process Notifier.ADD message
 */
protected void addElement (Notification msg) {
	 if (!sanityCheck((msg.getOldValue()==null),"Unexpected adapter on ADD")) return ;		      //$NON-NLS-1$
	 
	 ICodeGenAdapter oldAdapter =null ;
     Object[] args = null ;
		     
     if (isSpecialInstanceValue(msg.getNewValue()))  {	// Special Relationship e.g., contentPane		     	 
 		 // Process the Component        	
     	 args = processSpecialInstanceAdd(msg)  ;
      }else{
		if(msg.getNewValue() instanceof IJavaObjectInstance){
			IJavaObjectInstance comp = (IJavaObjectInstance) msg.getNewValue();
			BeanPart b = fBean.getModel().getABean(comp) ;
			if (b == null || fBean.getModel().getDeleteDesignated(comp, false)!=null) {  
				createBeanInstance(comp) ;
				b = fBean.getModel().getABean(comp) ;
			}else{ 
				JBCFPlugin.log("CheckMe ---> "+this+" AddElement existing Bean", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$ //$NON-NLS-2$
			}
			args = new Object[] {comp} ;
			fBean.addChild(b) ;
			b.setContainer(fBean) ;			 
      	}
     }
      // Brand new expression
      ExpressionRefGenerator eGen = new ExpressionRefGenerator(fBean,(RefStructuralFeature)msg.getStructuralFeature()) ;
      try {
	     CodeExpressionRef exp = eGen.createFromVCEModel(args) ;
	     exp.insertContentToDocument(true) ;			   
      }
      catch (com.ibm.etools.jbcf.codegen.util.CodeGenException e) {
         JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
         return ;
      }	
}


protected boolean ignoreMsg(Notification msg) {
    // Should not happen
    if (msg.getStructuralFeature() == null) return true ;
    
      // Construction time -- do not update source or generate anything
      if (fBean.getModel() == null || 
          fBean.getModel().isStateSet(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL)  ||
          fBean.getModel().isStateSet(IBeanDeclModel.BDM_STATE_SNIPPET) ||
          !fBean.getModel().isStateSet(IBeanDeclModel.BDM_STATE_UP_AND_RUNNING) ||
          ((msg.getStructuralFeature() instanceof EStructuralFeature) && ((EStructuralFeature)msg.getStructuralFeature()).isTransient()) ||
          fBean.getModel().getFStatus().isStatusSet(ICodeGenStatus.JVE_CODEGEN_STATUS_RELOAD_PENDING) 
//        ||  fBean.getModel().getFStatus().isStatusSet(ICodeGenStatus.JVE_CODEGEN_STATUS_RELOAD_IN_PROGRESS) 
          ) 
          return true  ;
          
      // Annotations has a special adapter
      if (msg.getNewValue() != null && (msg.getNewValue() instanceof com.ibm.etools.cdm.AnnotationEMF)) 
        return true ;
        
      return false ;
}

/**
 * applied: A setting has been applied to the mof object,
 * notify the decoder
 */ 
public void notifyChanged(Notification msg){

 try {
      // In process of building the composition ??
      if (ignoreMsg(msg)) return ;
      
      switch ( msg.getEventType()) {
      	case Notifier.TOUCH:
		case Notifier.NO_INDEX:		     
		case Notifier.REMOVING_ADAPTER:
			com.ibm.etools.cde.CDEHack.fixMe(); // Should really do something else.
	        JBCFPlugin.log(this+" No action on "+traceMsg[msg.getEventType()], JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
		    return ;
	}



      ICodeGenAdapter oldAdapter =null ;
      Object[] args = null ;
      
      
      
    String action=null ;
	switch ( msg.getEventType()) {	
		case Notifier.SET : // New, Different Value
		     action = "SET" ;	 //$NON-NLS-1$
		     setElement(msg) ;	     		     
		     break ;		     
		case Notifier.ADD_MANY:
                 action = "ADD_MANY" ; //$NON-NLS-1$
                 List al = (List) msg.getNewValue() ;
                 int i = 0 ;
                 Iterator aitr = al.iterator() ;
                 com.ibm.etools.cde.CDEHack.fixMe() ; // This should be changed to have the add function in a method and call it explicitly instead of resending a notifyChanged.
                 int pos = msg.getPosition();
                 while (aitr.hasNext()) {
                 	int mpos = pos<0 ? pos : pos + i++ ;
                 	Object o = aitr.next() ;
                 	addElement(new NotificationImpl(msg.getNotifier(), Notifier.ADD, msg.getStructuralFeature(), msg.getOldValue(), o, mpos)) ;
                 }             
             break ;
        case Notifier.REMOVE_MANY:
                 action = "REMOVE_MANY" ; //$NON-NLS-1$
                 List rl = (List) msg.getOldValue() ;
                 Iterator ritr = rl.iterator() ;
                 pos = msg.getPosition();
                 while (ritr.hasNext()) {
                 	Object o = ritr.next() ;
	                 com.ibm.etools.cde.CDEHack.fixMe() ; // This should be changed to have the remove function in a method and call it explicitly instead of resending a notifyChanged.
                 	removeElement(new NotificationImpl(msg.getNotifier(), Notifier.REMOVE, msg.getStructuralFeature(), o, msg.getNewValue(), pos)) ;
                 }
             break ;
		case Notifier.ADD :
		     action = "ADD" ; //$NON-NLS-1$
		     addElement(msg) ;
		     break ;
		     
		case Notifier.UNSET : 
		     action = "UNSET" ; //$NON-NLS-1$
		     if (msg.getOldValue() instanceof RefObject)
                   oldAdapter = getChildAdapter(fBean.getModel().getABean((RefObject)msg.getOldValue())) ;
                     
		     if (isSpecialInstanceValue(msg.getOldValue()) | oldAdapter != null)  {
		     	
		     	 IJavaObjectInstance obj = null ;
		         if (isSpecialInstanceValue(msg.getOldValue())) 
		               obj = getComponentFromSpecialRoot((RefObject)msg.getOldValue()) ;
		         else
		               obj = (IJavaObjectInstance) msg.getOldValue() ;
		     	 
		             
		     	 BeanPart b = fBean.getModel().getABean(obj) ;
		     	 if (oldAdapter == null)
		     	   oldAdapter = getChildAdapter(b) ;
		     	     
		     	 removeChildrenIfNeeded(b) ;
		     	 b.setContainer(null) ;
		     	 fBean.removeChild(b) ;
		     	 removeChildAdapter(fBean.getModel().getABean((RefObject)msg.getOldValue())) ;
		     	 removeBeanInstance((IJavaObjectInstance)obj) ;
		     }
		     else {  // An attribute		     	 
		     	 oldAdapter = getDecoderAdapter(msg.getOldValue(),msg.getStructuralFeature()) ;
		     	 if (oldAdapter != null) {
		     	   // Move the decoder to the new valued instance
		     	   removeDecoderAdapter(msg.getOldValue(),oldAdapter,msg.getStructuralFeature()) ;
//		     	   addDecoderAdapeter(msg.getNewValue(),oldAdapter,msg.getStructuralFeature()) ;			     	   		     	   	     	   
		     	 }	
		     	 NotifyParentIfNeeded((RefStructuralFeature) msg.getStructuralFeature()) ;	     	 		     	 	     	 
		     }
		     if (oldAdapter != null) {
		     	  // Let he decoder update the document.		     		     	  
		     	  oldAdapter.notifyChanged(msg) ;
		     }		       
		     break ;
		   
		case Notifier.REMOVE :
		     action = "REMOVE" ; //$NON-NLS-1$
		     removeElement(msg) ;
		     break ;		     		    
	      default:
	           JBCFPlugin.log(this+" No action= ????? ("+msg.getEventType()+")", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$ //$NON-NLS-2$
	           return ;
	           
	}	
	JBCFPlugin.log(this+" action= "+action+" completed.", JBCFMsgLogger.LOG_FINE) ;		 //$NON-NLS-1$ //$NON-NLS-2$

JBCFPlugin.log("SourceRange: "+getJavaSourceRange(), JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
 }
 catch (Throwable t) {
     JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING) ;
 }
}


public BeanPart getBeanPart() {
	return fBean ;
}

/* Null attributes support */
public void  addNullValue(RefStructuralFeature sf,ICodeGenAdapter adapter) {
	fNullValuedSF.put(sf,adapter) ;
}

public void  removeNullValue(RefStructuralFeature sf) {
	fNullValuedSF.remove(sf) ;
}
public ICodeGenAdapter getNullValuedAdapter(RefStructuralFeature sf) {
	return (ICodeGenAdapter) fNullValuedSF.get(sf) ;
}


/*  Container support */
public void  addChildAdapter(BeanPart child, ICodeGenAdapter adapter) {
	if (child != null)
	   fChildrens.put(child,adapter) ;
}

public void  removeChildAdapter(BeanPart child) {
	if (child != null)
	   fChildrens.remove(child) ;
}
public ICodeGenAdapter getChildAdapter(BeanPart child) {
	if (child != null)
	   return (ICodeGenAdapter) fChildrens.get(child) ;
	else
	   return null ;
}

/**
 * @return the Java Source Range of the method that initializes the bean.
 */
public ICodeGenSourceRange getJavaSourceRange() throws CodeGenException {
   
    CodeMethodRef mr = fBean.getInitMethod() ;
    if (mr != null)
      return  mr.getTargetSourceRange() ;
    else
      return null ;
     
}

/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	// Insert code to print the receiver here.
	// This implementation forwards the message to super.
	// You may replace or supplement this.
	return "\t"+"BeanDecoderAdapter:"+fBean.getUniqueName(); //$NON-NLS-1$ //$NON-NLS-2$
}

/**
 * @see ICodeGenAdapter#getBDMSourceRange()
 */
public ICodeGenSourceRange getBDMSourceRange() throws CodeGenException {
   return new CodeGenSourceRange(fBean.getInitMethod().getOffset(),
                                  fBean.getInitMethod().getLen()) ;
}


/**
 * @see com.ibm.etools.jbcf.codegen.java.ICodeGenAdapter#getSelectionSourceRange()
 */
public ICodeGenSourceRange getHighlightSourceRange()
	throws CodeGenException {
	CodeMethodRef mr = fBean.getInitMethod() ;
	if (mr != null)
	  return  mr.getHighlightSourceRange() ;
	else
	  return null ;
}
public Object getAdapter(Class aClass){
	if ( aClass == IJavaElement.class ) { 
		// Return the IField for this bean.
		IJavaElement e = (IField) JavaCore.create(fBean.getFieldDeclHandle());
		ICompilationUnit cu = fBean.getModel().getWorkingCopyProvider().getSharedWorkingCopy() ;
		IJavaElement[] es = cu.findElements(e);
		if (es != null && es.length == 1)
			return es[0];
	}
	return null;
}
}