package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jdt.core.*;

import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.codegen.util.*;

import java.util.*;

import org.eclipse.jface.text.BadLocationException;

public class CodeSnippetMergelet {

	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	ICodeDelta     fBDMDelta ;
	String         fElementContent=null ;
	String         fElementHandle=null ;
	boolean        fisMethod ;
	IBeanDeclModel fBeanModel = null ;
	

 


public CodeSnippetMergelet (ICodeDelta delta, String elementContent, String elementHandle, boolean isMethodElement) {
	fBDMDelta = delta ;
	fElementContent = elementContent ;
	fElementHandle = elementHandle ;
	fisMethod = isMethodElement ;
}


private CodeMethodRef getOriginalMethodElement() throws CodeGenException {
	if (!fisMethod) throw new CodeGenException ("Invalid Context") ;	 //$NON-NLS-1$
	return fBeanModel.getMethodInitializingABean(fElementHandle) ;	
}
/**
 * Get the corresponding expression
 */
private CodeExpressionRef getOriginal(CodeExpressionRef exp) throws CodeGenException {
  if (!fisMethod) throw new CodeGenException("Invalid Context") ; //$NON-NLS-1$
  
  CodeMethodRef m = getOriginalMethodElement() ;
  if (m == null) return null ;
  Iterator itr = m.getExpressions() ;
  while (itr.hasNext()) {
  	CodeExpressionRef e = (CodeExpressionRef) itr.next() ;
  	if (e==exp || e.isEquivalent(exp)>=0) return e ;
  }
  return null ;
}


/**
 * Get the coresponding BeanPart
 */
private BeanPart getOriginal (BeanPart b) {
	Iterator itr = fBeanModel.getBeans().iterator() ;
	while (itr.hasNext()) {
		BeanPart bean = (BeanPart) itr.next() ;
		if (bean.isEquivalent(b)) 
			if(bean.getUniqueName().equals(b.getUniqueName()))
				return bean ;
	}
	return null ;
}

/** 
 *  Dump the text into the local document
 */
private void replaceLocalContent(int left,int len,String Content) throws CodeGenException {
	try {
	  fBeanModel.getDocument().replace(left,len,Content) ;	  	
	}
	catch (BadLocationException e) {
        	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
        	throw new CodeGenException(e) ;		
	}		
}

/**
 * @return int new method offset
 */
private int insertNewMethodContent() throws CodeGenException {
	
	if (!fisMethod) throw new CodeGenException ("Invalid Context") ; //$NON-NLS-1$
	try {
	  IType t = CodeGenUtil.getMainType(fBeanModel.getCompilationUnit()) ;
  	  IMethod newMethod = t.createMethod(fElementContent,null,(CodeGenUtil.getMethod(t, fElementHandle)!=null),null) ;
	  return newMethod.getSourceRange().getOffset() ;
	}
	catch (JavaModelException e) {
		JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
		throw new CodeGenException(e) ;
	}
}

/**
 *  Generate an expression that is hooked into the main BDM
 */
private CodeExpressionRef createNewExpression(CodeExpressionRef e, CodeMethodRef m, boolean decode) throws CodeGenException{
    if (getOriginal(e) != null) throw new CodeGenException("duplicate Expression") ; //$NON-NLS-1$
    BeanPart b = getOriginal(e.getBean()) ;
    if (b == null) throw new CodeGenException("No Bean Part") ; //$NON-NLS-1$
    ExpressionRefGenerator gen = new ExpressionRefGenerator(b,null) ;
    CodeExpressionRef newe = gen.createFromSource(e,m) ;
    newe.setState(newe.STATE_NO_OP, e.isStateSet(e.STATE_NO_OP));
    newe.setState(newe.STATE_INIT_EXPR, e.isStateSet(e.STATE_INIT_EXPR));
    if (decode) {
       if (!newe.decodeExpression()) {
         newe.dispose() ;
         newe = null ;
       }
    }
    return newe ;
}

private boolean processExpressionDelta (CodeExpressionRef dExp, CodeExpressionRef oExp, CodeMethodRef oMethod, int status) throws CodeGenException {
	
	    boolean updated = false ;
		if (oExp != null && dExp != null && !dExp.isStateSet(dExp.STATE_EXP_IN_LIMBO)) {
			if (oExp.isEquivalent(dExp) < 0) throw new CodeGenException("No the same Expressions"); //$NON-NLS-1$
			oExp.setExpression(dExp.getExpression());
		}
        if (oExp != null && dExp != null)
           oExp.updateLimboState(dExp) ;
        if (oExp == null && status != ICodeDelta.ELEMENT_ADDED && status != ICodeDelta.ELEMENT_UNDETERMINED) {
            com.ibm.etools.jbcf.JBCFPlugin.log("CodeSnippetMergelent.processExpressionDelta(): could not find"+dExp, //$NON-NLS-1$
                                                JBCFMsgLogger.LOG_FINE) ;
        }
        switch (status) { 
        	case ICodeDelta.ELEMENT_UNDETERMINED:                 
                 com.ibm.etools.jbcf.JBCFPlugin.log("CodeSnippetMergelent.processExpressionDelta() In Limbo: "+oExp, //$NON-NLS-1$
                                                     JBCFMsgLogger.LOG_FINE) ;
                 if (oExp != null)  
                    oExp.setState(oExp.STATE_EXP_IN_LIMBO, true) ;
                    
                 // VCE model is not updated
        		 break;
        	case ICodeDelta.ELEMENT_UPDATED_OFFSETS:
        		 oExp.setOffset(dExp.getOffset());
        		 // No need to refresh when a shadow expression 
        		 if(!dExp.isStateSet(dExp.STATE_SHADOW))
        		 	// Will take care of reordering of expressions
        		 	oExp.refreshFromJOM(dExp); 
        	     break;
        	case ICodeDelta.ELEMENT_NO_CHANGE:   
                 if (oExp != null)
                    oExp.setContent(dExp.getContentParser())  ;
        	     break ;        	   
        	case ICodeDelta.ELEMENT_CHANGED:
                 if (oExp != null) {
                 	if (oExp.isStateSet(oExp.STATE_INIT_EXPR)) {
                 		// Change in a "new" statement we can not refresh at this point
                 		fBeanModel.setState(fBeanModel.BDM_STATE_DOWN,true) ;
                 		return false  ;
                 	}
                 	if(oExp.getOffset()!=dExp.getOffset())
                 		oExp.setOffset(dExp.getOffset());
        	        oExp.refreshFromJOM(dExp);                 
        	        updated = true ;
                 }
        	     break ;
        	case ICodeDelta.ELEMENT_DELETED:
                 if (oExp != null) {
                    if (oExp.isStateSet(oExp.STATE_INIT_EXPR)) {
                 		// Change in a "new" statement we can not refresh at this point
                 		fBeanModel.setState(fBeanModel.BDM_STATE_DOWN,true) ;
                 		return false ;
                 	}                   
        	       oExp.deleteFromComposition() ;
        	       oExp.dispose() ;
        	       updated = true ;
                 }
        	     break ;
        	case ICodeDelta.ELEMENT_ADDED:
                 try {
                  if (dExp.isStateSet(oExp.STATE_INIT_EXPR)) {
                 		// Change in a "new" statement we can not refresh at this point
                 		fBeanModel.setState(fBeanModel.BDM_STATE_DOWN,true) ;
                 		return false ;
                   }                 	
        	       createNewExpression(dExp,oMethod,!dExp.isStateSet(dExp.STATE_NO_OP));//((dExp.getState() & dExp.STATE_NO_OP) != dExp.STATE_NO_OP)) ;
        	       updated = true ;
                 }
                 catch (CodeGenException e) {} ; // Do not create dup expressions
        	     break ;
        	default:
        	     throw new CodeGenException("Invalid Status") ; //$NON-NLS-1$
        }	
        return updated ;
}


/**
 *  Method is already in the BDM, update changes to its content
 */
private boolean updateMethodDelta (CodeMethodRef m, List deleteList) throws CodeGenException {

	CodeMethodRef     oMethod = getOriginalMethodElement() ;   // Original method
	
	boolean  updated = false ;
      
    // Update:
    // If the methodRef is in state ELEMENT_UNDETERMINED, then it has a parse
    // error, and some of the expressions dont know where they are. Since
    // some of them DO know where they are, we have to update those
    // expression offsets. (also when the method is in ELEMENT_UNDETERMINED state
    // we are guaranteed of having a one-to-one relationship between the old
    // and new expressions).
	if((fBDMDelta.getElementStatus(m)&ICodeDelta.ELEMENT_UNDETERMINED)==ICodeDelta.ELEMENT_UNDETERMINED){  // Plan B
		List deltaExpressions = new ArrayList();
		List oldExpressions = new ArrayList();
		Iterator newItr = m.getExpressions() ;
		Iterator oldItr = oMethod.getExpressions();
		while(newItr.hasNext())	
			deltaExpressions.add(newItr.next());
		while(oldItr.hasNext())
			oldExpressions.add(oldItr.next());
		// Need to move expressions to new Lists , or else
		// there will be java.util.ConcurrentModificationExceptions
		// thrown when expressions are removed from methods.
		newItr = deltaExpressions.iterator();
		oldItr = oldExpressions.iterator();
		while (newItr.hasNext() && oldItr.hasNext()) {
			CodeExpressionRef dExp = (CodeExpressionRef) newItr.next() ; // Delta exp
			CodeExpressionRef oExp = (CodeExpressionRef) oldItr.next();  // Original corresponding exp
			updated |= processExpressionDelta(dExp,oExp,oMethod,fBDMDelta.getElementStatus(dExp)) ;
		}
	}else{
		// Delete 
		Iterator itr ;
		if (deleteList != null) {   
			itr = deleteList.iterator() ;
			while (itr.hasNext()) {
				CodeExpressionRef dExp = (CodeExpressionRef) itr.next() ; // Delta exp	 
				CodeExpressionRef oExp = dExp;           // Original exp is given by the delta
				updated |= processExpressionDelta(dExp,oExp,oMethod,fBDMDelta.getElementStatus(dExp)) ;
				if (fBeanModel.isStateSet(fBeanModel.BDM_STATE_DOWN)) return updated ;
			}
		}
          
		itr = m.getExpressions() ;
		while (itr.hasNext()) {		
			CodeExpressionRef dExp = (CodeExpressionRef) itr.next() ; // Delta exp	 
			CodeExpressionRef oExp = getOriginal(dExp) ;              // Original exp
			updated |= processExpressionDelta(dExp,oExp,oMethod,fBDMDelta.getElementStatus(dExp)) ;
		}
	}
	return updated ;
	
}

/**
 *  New Method, create the meta data for it, and its related expressions
 */
private void createNewMethod(CodeMethodRef m, int Offset) throws CodeGenException {
	
   if (getOriginalMethodElement() != null) throw new CodeGenException ("Duplicated Method") ; //$NON-NLS-1$
   
   CodeMethodRef   mref = new CodeMethodRef(fBeanModel.getTypeRef(),m.getMethodName()) ;
   mref.setMethodHandle(fElementHandle) ;
   mref.setOffset(Offset) ;
   mref.setContent(fElementContent) ;
   
   Iterator itr = m.getExpressions() ;
   while (itr.hasNext()) {
   	CodeExpressionRef exp = (CodeExpressionRef) itr.next() ;
   	BeanPart oBean = getOriginal (exp.getBean()) ;
   	oBean.addInitMethod(mref) ;
   	//if (exp.getBean().getReturnedMethod().equals(m)) -- consider already declared fields, with no init/return methods.
   	oBean.addReturnMethod(mref) ;
   	createNewExpression(exp,mref,true) ;
   }  		
}

/**
 *  Remove the original method from the VCE model, and local document
 */
private void deleteExistingMethod(CodeMethodRef m) throws CodeGenException {
	CodeMethodRef oMethod = getOriginalMethodElement() ;
	IMethod CUmethod = CodeGenUtil.getMethod(CodeGenUtil.getMainType(fBeanModel.getCompilationUnit()),
   	                                         oMethod.getMethodHandle()) ;
   	try {
	  CUmethod.delete(true,null) ;
   	}
   	catch (JavaModelException e) {
   	  JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
      }
	oMethod.deleteFromComposition() ;
	oMethod.dispose() ;
}

private void updateExpressionOffset (CodeMethodRef m) throws CodeGenException {
	if((fBDMDelta.getElementStatus(m) & ICodeDelta.ELEMENT_UNDETERMINED)==ICodeDelta.ELEMENT_UNDETERMINED){
		CodeMethodRef oldMethod = getOriginalMethodElement();
		Iterator oldItr = oldMethod.getExpressions();
		Iterator newItr = m.getExpressions();
		while(oldItr.hasNext() && newItr.hasNext()){
			CodeExpressionRef oExp = (CodeExpressionRef) oldItr.next();
			CodeExpressionRef dExp = (CodeExpressionRef) newItr.next();
			if(dExp.isStateSet(dExp.STATE_EXP_IN_LIMBO)){ //(dExp.getState() & dExp.STATE_EXP_IN_LIMBO)==dExp.STATE_EXP_IN_LIMBO){
				// Parse error for expression, hence we dont know where it is.
			}else{
				if((fBDMDelta.getElementStatus(dExp) & ICodeDelta.ELEMENT_UPDATED_OFFSETS) == ICodeDelta.ELEMENT_UPDATED_OFFSETS){
					oExp.setContent(dExp.getParser());
					oExp.setOffset(dExp.getOffset());
				}
			}
		}
	}else{
		Iterator itr = m.getExpressions() ;
		while (itr.hasNext()) {		
		  CodeExpressionRef dExp = (CodeExpressionRef) itr.next() ; // Delta Element	 
		  CodeExpressionRef oExp = getOriginal(dExp) ;
		  if (oExp != null) { // Expression was created successfully
	          oExp.setContent(dExp.getParser()) ;
		    oExp.setOffset(dExp.getOffset()) ;
		  }
		}
	}
}

protected void replaceMethodContent() throws CodeGenException {
	
	try {
	 IMethod m = CodeGenUtil.getMethod(CodeGenUtil.getMainType(fBeanModel.getCompilationUnit()),fElementHandle) ;	
	 replaceLocalContent(m.getSourceRange().getOffset(),m.getSourceRange().getLength(),fElementContent) ;
	}
	catch (JavaModelException e) {
		throw new CodeGenException(e) ;
	}
}

/**
 *  Assume a single element is to be merged (Method/Instance Var.)
 * @return boolean if any update was made to the model
 */
public boolean updateBDM(IBeanDeclModel model) throws CodeGenException {
	
	if (model == null || fElementHandle == null) throw new CodeGenException("Invalid Arg") ; //$NON-NLS-1$
		
	fBeanModel = model ;
	
	boolean   modelUpdated = false ;
	
	CodeMethodRef mref = fBDMDelta.getDeltaMethod() ;
	if (mref != null) { // Method Update
	  switch (fBDMDelta.getElementStatus(mref)) {
	 	case ICodeDelta.ELEMENT_NO_CHANGE:
	 	     break ;
		case ICodeDelta.ELEMENT_ADDED:
		     int Offset = insertNewMethodContent() ;
		     // createNewMethod(mref,Offset) ;
		     // At this time induce the model to be reloaded.
		     fBeanModel.setState(fBeanModel.BDM_STATE_DOWN,true) ;
		     modelUpdated = true ;
		     break ;
		case ICodeDelta.ELEMENT_DELETED:
		     // deleteExistingMethod(mref) ;
		     // At this time induce the model to be reloaded.
		     fBeanModel.setState(fBeanModel.BDM_STATE_DOWN,true) ;
		     modelUpdated = true ;
		     break ;
		case ICodeDelta.ELEMENT_UNDETERMINED:
		case ICodeDelta.ELEMENT_UPDATED_OFFSETS:
		case ICodeDelta.ELEMENT_CHANGED:
		     modelUpdated |= updateMethodDelta(mref,fBDMDelta.getDeletedElements(mref)) ;
		     // Check if we need to reLoad 
		     if (fBeanModel.isStateSet(fBeanModel.BDM_STATE_DOWN)) return modelUpdated ;
             CodeMethodRef oriMref = getOriginalMethodElement() ;
             try {
                oriMref.disconnectFromDoc() ;
                // We may entered non relevant (or non parsable) content
		        replaceMethodContent() ;
		        BeanPartGenerator.fixOffsetIfNeeded(fElementContent,oriMref) ;
             }
             finally {
               oriMref.connectToDoc() ;
             }
		     break ;
		default:
		     throw new CodeGenException("Invalid State") ; //$NON-NLS-1$
	  }
	}
	else { // Instance Variable update
	 	// TBD
	 	com.ibm.etools.jbcf.JBCFPlugin.log("CodeSnippetMerglent.updateBDM() : no CodeMethodRef",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
	}
	return modelUpdated ;
}

public void updateBDMwithLock(IBeanDeclModel model) throws CodeGenException {
    synchronized (fBeanModel.getDocumentLock()) {
    	updateBDM(model) ;
    }
}

}