package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.codegen.util.*;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.codegen.util.IWorkingCopyProvider;

import org.eclipse.jdt.internal.core.*;
import org.eclipse.jdt.internal.core.BasicCompilationUnit;
import org.eclipse.jface.text.*;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.jdt.internal.compiler.ast.*;

import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.codegen.model.IBeanDeclModel;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import java.util.*;

public class CodeSnippetModelBuilder 
	extends JavaBeanShadowModelBuilder {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$

protected String contents;
protected int[] importStarts;
protected int[] importEnds;
protected int[] fieldStarts;
protected int[] fieldEnds;
protected int[] methodStarts;
protected int[] methodEnds;
protected String[] methodHandles;


public CodeSnippetModelBuilder(String contents, String[] methodHandles, int[] importStarts, int[] importEnds, int[] fieldStarts, int[] fieldEnds, int[] methodStarts, int[] methodEnds, ICompilationUnit referenceCU){
	super("CodeSnippetClass_2",null); //$NON-NLS-1$
	this.referenceCU = referenceCU;
	this.contents = contents;
	this.importStarts=importStarts;
	this.importEnds=importEnds;
	this.fieldStarts=fieldStarts;
	this.fieldEnds=fieldEnds;
	this.methodStarts=methodStarts;
	this.methodEnds=methodEnds;
	this.methodHandles=new String[methodHandles.length+1];
	this.methodHandles = methodHandles;
	this.fWCP = createPseudoWorkingCopyProvider(referenceCU) ;
}



protected char[] getFileContents() throws CodeGenException {
	return contents.toCharArray();
}

public CompilationUnitDeclaration getModelFromParser(
	ProblemReporter reporter,
	CompilationResult result,
	BasicCompilationUnit cu){
	CompilationUnitDeclaration decl = super.getModelFromParser(reporter, result, cu);
	updateModelPositions(decl);
	return decl;
}

/** 
 * Now the JDOM has different offsets and lengths, so 
 * set them to how we like.
 * Ex: When you have multiple comment blocks prefixing 
 * the method declaration, then the JDOM takes only the 
 * last comment block, whereas JDT takes in all the 
 * multiple comment blocks. Since the BDM is based on
 * the JDT, we adjust the JDOM's offsets.
 */
protected void updateModelPositions(CompilationUnitDeclaration decl){
	TypeDeclaration type = decl.types[0];
	FieldDeclaration[] decls = type.fields;
	AbstractMethodDeclaration[] methods = type.methods;
	if(decl!=null && fieldStarts!=null && decls != null && decls.length==fieldStarts.length){ 
		// liner mapping.. no problems.
		for(int dc=0;dc<decls.length;dc++){
			decls[dc].sourceStart = fieldStarts[dc];
			decls[dc].sourceEnd = fieldEnds[dc];
		}
	}
	if(methods!=null && methodStarts!=null){
		int count = 0;
		for(int mc=1;mc<methods.length;mc++){
			if(methods[mc].isConstructor())
				continue;
			if(methods[mc].isClinit())
				continue;
			if(methods[mc]!=null){
				methods[mc].declarationSourceStart=methodStarts[count];
				methods[mc].declarationSourceEnd=methodEnds[count];
				count++;
			}
		}
	}
}

protected void visitType(TypeDeclaration type, IBeanDeclModel model, char[] content, List tryAgain){
	new TypeVisitor(type,model,content,methodHandles,tryAgain,true).visit()  ;
}


}