package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import java.util.List;

import com.ibm.etools.emf.notify.* ;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.emf.ref.* ;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition;
import com.ibm.etools.java.instantiation.base.* ;


public class CompositionDecoderAdapter extends BeanDecoderAdapter {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public CompositionDecoderAdapter (Notifier target, BeanPart b) {
		super(target,b) ;
	}

protected boolean isInBeanSubClassComposition(IJavaObjectInstance obj) {
	
	// If the bean is dedignated as deleted, it is "not" there
	if (fBean.getModel().getDeleteDesignated(obj,false)!=null) return false ;
	
	Iterator itr = fBean.getModel().getRootBeans().iterator() ;
    while (itr.hasNext()) {
		BeanPart b = (BeanPart) itr.next();
		if (b.getRefObject()==obj) return true ;
	}
	return false ;
}	
	
public void notifyChanged(Notification msg){

 try {
      // In process of building the composition ??
      if (ignoreMsg(msg)) return  ;
      
      
      if (msg.getStructuralFeature()==null || 
          msg.getStructuralFeature().refID() == null || 
          !msg.getStructuralFeature().refID().equals("BeanSubclassComposition.components")) //$NON-NLS-1$
         return ;
         
      String action=null ; 
	switch ( msg.getEventType()) {
		case Notifier.TOUCH:
		     action = "TOUCH" ; //$NON-NLS-1$
		     break ;	
		case Notifier.SET : 
		     action = "SET" ; //$NON-NLS-1$
		     break ;
		case Notifier.ADD :
		     action = "ADD" ; //$NON-NLS-1$
		     if (!isInBeanSubClassComposition((IJavaObjectInstance)msg.getNewValue())) 
		         createBeanInstance((IJavaObjectInstance)msg.getNewValue()) ;
		     break ;
		case Notifier.UNSET : 
		     action = "UNSET" ; //$NON-NLS-1$
		     break ;
		case Notifier.REMOVE :
		     action = "REMOVE" ; //$NON-NLS-1$
		     BeanPart b = fBean.getModel().getABean((RefObject)msg.getOldValue()) ;
		     removeChildrenIfNeeded(b) ;
		     removeBeanInstance((IJavaObjectInstance)msg.getOldValue()) ;		     		     		     
		     break ;
		case Notifier.REMOVING_ADAPTER:
		     action = "REMOVING_ADAPTER" ;		      //$NON-NLS-1$
		     break;
		case Notifier.ADD_MANY:
             action = "ADD_MANY" ; //$NON-NLS-1$
             List al = (List) msg.getNewValue() ;
             int i = 0 ;
             Iterator aitr = al.iterator() ;
             com.ibm.etools.cde.CDEHack.fixMe() ; // This should be changed to have the add function in a method and call it explicitly instead of resending a notifyChanged.
             int pos = msg.getPosition();
             while (aitr.hasNext()) {
                 	int mpos = pos<0 ? pos : pos + i++ ;
                 	Object o = aitr.next() ;
                 	notifyChanged(new NotificationImpl(msg.getNotifier(), Notifier.ADD, msg.getStructuralFeature(), msg.getOldValue(), o, mpos)) ;
             }             
             break ;
       case Notifier.REMOVE_MANY:
             action = "REMOVE_MANY" ; //$NON-NLS-1$
             List rl = (List) msg.getOldValue() ;
             Iterator ritr = rl.iterator() ;
             pos = msg.getPosition();
             while (ritr.hasNext()) {
                 	Object o = ritr.next() ;
	                 com.ibm.etools.cde.CDEHack.fixMe() ; // This should be changed to have the remove function in a method and call it explicitly instead of resending a notifyChanged.
                 	notifyChanged(new NotificationImpl(msg.getNotifier(), Notifier.REMOVE, msg.getStructuralFeature(), o, msg.getNewValue(), pos)) ;
             }
             break ;		     
	}
	JBCFPlugin.log(this+" action= "+action, JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
 }
 catch (Throwable t) {
     JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING) ;
 }
}

protected void createBeanInstance(IJavaObjectInstance obj){
	BeanPart bp = fBean.getModel().getDeleteDesignated(obj, false) ;
	super.createBeanInstance(obj);
    if (bp != null) {
        try{
        	bp.getFFDecoder().decode();
        }catch(CodeGenException e){
        	JBCFPlugin.log("Couldnt generate FF Annotation for component",JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
        	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
        }
    }
}

/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	// Insert code to print the receiver here.
	// This implementation forwards the message to super.
	// You may replace or supplement this.
		 return "\t"+"CompositionDecoderAdapter"; //$NON-NLS-1$ //$NON-NLS-2$
}
	

}