package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.*;
import com.ibm.etools.jbcf.codegen.util.*;
import org.eclipse.jdt.internal.compiler.ast.*;
import java.util.List;
import com.ibm.etools.jbcf.codegen.* ;
import org.eclipse.jdt.core.Signature ;
import java.util.* ;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance ;
import com.ibm.etools.emf.ref.RefObject ;


public class ConstraintDecoderHelper extends ExpressionDecoderHelper {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	int  fConstraints[]=null ;		

public ConstraintDecoderHelper(BeanPart bean, Statement exp,  IJavaFeatureMapper fm, IExpressionDecoder owner){
	super (bean,exp,fm,owner) ;
}


/**
 *  Add a new constraint
 */
protected boolean	addConstraintFeature() throws CodeGenException {
	if (fbeanPart.getRefObject() == null || fFmapper.getMethodName() == null) throw new CodeGenException ("null RefObject:"+fExpr) ;       //$NON-NLS-1$
		
	try {
        if (fFmapper.getMethodName().equals(IJavaFeatureMapper.CONSTRAINT_BOUND)) {
     	    com.ibm.etools.cde.CDEHack.fixMe("Need to deal with Rectangle arg") ; //$NON-NLS-1$
    	    if (((MessageSend)fExpr).arguments.length == 4)   {
   		   // Create a dimension from the width and height of the rectangle
   		   fConstraints = new int[4] ;
   		   for (int i=0; i<4; i++) 
   		     fConstraints[i] = Integer.parseInt(((MessageSend)fExpr).arguments[i].toString()) ;
               CodeGenUtil.addRactangleFeatureValue(
   		                       fbeanPart.getRefObject(),
   		                       fConstraints,
   		                       fFmapper.getFeature(null),   		                       
   		                       fOwner.getCompositionModel()) ;
    	    }
          else {
           	  JBCFPlugin.log("ConstraintDecoder: bound != 4 arguments", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
           	  return false ;
          }   		                 
        }
        else
        	if (fFmapper.getMethodName().equals(IJavaFeatureMapper.CONSTRAINT_SIZE)) {
		       	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with Dimension arg") ;       	 //$NON-NLS-1$
		       	if (((MessageSend)fExpr).arguments.length == 2)  {
		       	   fConstraints = new int[2] ;
		   		   for (int i=0; i<2; i++) 
		   		     fConstraints[i] = Integer.parseInt(((MessageSend)fExpr).arguments[i].toString()) ;
		       	   
		       	   // Create a dimension from the width and height of the rectangle    
		       	   CodeGenUtil.addDimensionFeatureValue(   	
		       	                       fbeanPart.getRefObject(),
		       	                       fConstraints,
                                               fFmapper.getFeature(null),
		   		                       fOwner.getCompositionModel()) ;
		       	}
		   		else {
		   			  JBCFPlugin.log("ConstraintDecoder: Size != 2 arguments", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
		           	  return false ;
		   		}
          }
          else 
          		// Check if only string is inside. We assume that this decoder helper 
          		// _really_ trusts the expression it has been given to handle.
			if((((MessageSend)fExpr).arguments.length == 1) && 
			   (((MessageSend)fExpr).arguments[1] instanceof StringLiteral)){
				// String is the constraint parameter 
		            String constraint = new String (((MessageSend)fExpr).arguments[1].toString()) ;
		            // Add the constraint to the added part
		           	CodeGenUtil.addConstraintString(fbeanPart.getRefObject(),
		           	                                constraint,
                                                        fFmapper.getFeature(null),                                                        
		           	                                fOwner.getCompositionModel()) ;
          		}
          		else return false ;
	}
	catch (Exception e) {
		throw new CodeGenException(e) ;
	}
	return true ;   
}


/**
 *  Get the args part of the expression
 */
protected String getArgs(String exp) {
  int[] indexes = getArgsIndexes(exp) ;
  if (indexes == null) return null ;
  return exp.substring(indexes[0],indexes[1]+1) ;
}
/**
 *  Get the args part of the expression
 */
protected int[] getArgsIndexes(String exp) {
	
  int left  = exp.indexOf(Signature.C_PARAM_START) ;
  int right = exp.indexOf(Signature.C_PARAM_END) ;
  
  if (left < 0 || right < 0) return null ;  	
  left ++ ; right -- ;
  if (left>right) return null ;
  int[] indexes = new int[2] ;
  indexes[0] = left ;
  indexes[1] = right ;
  return indexes ;
}

/**
 *  Get the dimension of the constraint from the arguments
 */
protected int[] parseArgs(String exp) {
	
	if (exp == null || getArgs(exp)==null) return null ;		
	StringTokenizer st = new StringTokenizer(getArgs(exp),",") ; //$NON-NLS-1$
	int args[] = new int[st.countTokens()] ;
	for (int i=0; i<args.length; i++) {
		args[i] = Integer.parseInt(st.nextToken().trim()) ;
	}
	return args ;
}

/**
 *  Compare the arrays
 */
protected boolean sameConstraints(int[] A, int[] B) {
	if (A.length != B.length) return false ;
	for (int i=0; i<A.length; i++)
	   if (A[i] != B[i]) return false ;
	return true ;
}


/**
 *  Get the current constraints in the Composition Model
 */
protected int[]   getCompositionConstraints() throws CodeGenException {

  IJavaObjectInstance curValue = (IJavaObjectInstance)fbeanPart.
                                 getRefObject().refValue(fFmapper.getFeature(fExpr)) ;  
  return parseArgs(curValue.getInitializationString()) ;	
} 
/**
 *  Create initialization arguments
 */
protected String constraintsToString(int[] constraints) {
	StringBuffer sb = new StringBuffer() ;
      for (int i=0; i<constraints.length; i++) {
     	  if (i>0) sb.append(", ") ; //$NON-NLS-1$
     	  sb.append(Integer.toString(constraints[i])) ;
      }
      return sb.toString() ; 
}

public String primRefreshFromComposition(String expSig) throws CodeGenException {
	  
  int [] curConstraints = getCompositionConstraints() ;
  if (fConstraints != null)
    if (sameConstraints(curConstraints,fConstraints)) {
  	fExprSig = expSig ; // spacing and such may have changed.
    }
    else {
      if (curConstraints.length != fConstraints.length) {
      	fExprSig = generate(null) ;
      	// Remove the generated comments etc.
      	fExprSig = fExprSig.substring(0,fExprSig.indexOf(")")+1) ; //$NON-NLS-1$
      }      
      else {  	     
        int[] curArgsIndexes = getArgsIndexes(expSig) ;
        String newArgs = constraintsToString(curConstraints) ;     
        fConstraints = curConstraints ;
        // Replace the arguments part with the current constraints
        fExprSig = expSig.substring(0,curArgsIndexes[0]) +
                newArgs +
                expSig.substring(curArgsIndexes[1]+1,expSig.length()) ;
      }
    }
  
     	
   return fExprSig ;	
}

public boolean primIsDeleted() {
  // If a constraint was changed to non-free form, remove the code
  return (fbeanPart.getRefObject().refValue(fFmapper.getFeature(fExpr)) == null);
}

/**
 *   Go for it
 */
public boolean decode() throws CodeGenException {

	com.ibm.etools.cde.CDEHack.fixMe("Need to validate that the constraint is valid for a given layout") ; //$NON-NLS-1$
      return addConstraintFeature() ;
}

public void delete() {
	UnadaptToCompositionModel() ;
	
	com.ibm.etools.emf.ref.RefStructuralFeature sf = fFmapper.getFeature(fExpr) ;
	IJavaObjectInstance parent = (IJavaObjectInstance)fbeanPart.getRefObject() ;
	parent.refUnsetValue(sf) ; 
}


/**
 *  Overide the default to deal with the lack of Decorator
 */
protected ExpressionTemplate getExpressionTemplate(String[] args) throws CodeGenException {
	
	if (fFmapper.getDecorator() != null) 
	   return super.getExpressionTemplate(args) ;
	   
      String mtd ;
      if (fConstraints.length == 2)
        mtd = IJavaFeatureMapper.CONSTRAINT_SIZE ;
      else if(fConstraints.length == 4)
        mtd = IJavaFeatureMapper.CONSTRAINT_BOUND ;
      else
        throw new CodeGenException("Can not resolve write method") ;                 //$NON-NLS-1$
	String sel = fbeanPart.getSimpleName() ;
	ExpressionTemplate exp = new ExpressionTemplate (sel,mtd, 
	                                                 args,
	                                                 null,
	                                                 0 ) ;
	exp.setLineSeperator(fbeanPart.getModel().getLineSeperator()) ;
	return exp ;   	
}

public Object getPriorityOfExpression(){
	Integer pri = (Integer)super.getPriorityOfExpression();
	return new Integer(IJavaFeatureMapper.PRIORITY_CONSTRAINT_CHANGE + pri.intValue());
}

/**
 *  Generate from scratch
 */
public String generate(Object[] noArgs) throws CodeGenException {
	
	if (fFmapper.getFeature(null) == null) 
	    throw new CodeGenException ("null Feature") ; //$NON-NLS-1$
	try {
	   fConstraints = getCompositionConstraints() ;
	}
	catch (CodeGenException e) {
		return null ; 
	}
		
	ExpressionTemplate exp = getExpressionTemplate(new String[] {constraintsToString(fConstraints)}) ;
      fExprSig = exp.toString() ;	                                                 
      return fExprSig ;	     	
}

/**
 * Determine if the current OCM model holds a bound/size attribute constraint
 */
//public static boolean  primIsFreeFormConstraint(RefObject obj) {
//
//  IJavaObjectInstance curValue = (IJavaObjectInstance) obj.refValue(CodeGenUtil.getConstraintFeature(obj)) ;  
//  if (curValue == null) return false ;
//  
//  String typeName = curValue.getJavaType().getQualifiedName() ;
//  
//  if (typeName.equals(IJavaFeatureMapper.CONSTRAINT_BOUND_CLASS) ||
//      typeName.equals(IJavaFeatureMapper.CONSTRAINT_SIZE_CLASS))
//      return true ;
//  
//  com.ibm.etools.cde.CDEHack.fixMe("Need to address containers");
//  return false ;
//  
//}

public boolean isImplicit(Object args[]) {
   return false ;		
}

public Object[] getArgsHandles(Statement expr) {
   // No unique arguments ;
   return null ;
}


}


