package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.RefStructuralFeature ;
import com.ibm.etools.java.instantiation.base.IJavaInstance ;
import org.eclipse.jdt.internal.compiler.ast.* ;
import com.ibm.etools.jbcf.codegen.util.* ;
import com.ibm.etools.emf.ecore.EClass ;
import com.ibm.etools.emf.ref.RefObject ;
import java.util.* ;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.codegen.* ;
import com.ibm.etools.jbcf.* ;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance ;


public class ContainerDecoder extends ComponentDecoder {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected final static  String ADD_METHOD = "add" ;	 //$NON-NLS-1$
protected final static  String DEFAULT_CONSTRAINT = ".getName()" ; //$NON-NLS-1$
protected final static  String LAYOUT_METHOD = "setLayout" ; //$NON-NLS-1$
protected final static  String LAYOUT_FEATURE = "layout" ; //$NON-NLS-1$

/**
 *  A JPanel (maybe changing to Container ) Decoder will also add  ...
 *  May need to augment it with other features in the future
 */

public ContainerDecoder (CodeExpressionRef expr, IBeanDeclModel model, IDiagramModelInstance cm, BeanPart part) {
	super (expr,model,cm,part) ;
}

public ContainerDecoder() {	
	super () ;
}


/**
 *  Is this a component add expression
 */
protected boolean isMethod(String mSig) {	

	  return isMethod(mSig,CodeGenUtil.getComponentFeature(fbeanPart.getRefObject())) ;
} 
/**
 *  Is this a component add expression
 */
protected boolean isMethod(String mSig, RefStructuralFeature sf) {	

	  String method=null ;
	  
	  if (fFeatureMapper!= null)
	    if(fFeatureMapper.getDecorator()!= null &&
	       fFeatureMapper.getDecorator().getWriteMethod()!=null) 
	       method = fFeatureMapper.getDecorator().getWriteMethod().getName() ;
	    else 
	    	 if (fFeatureMapper.getFeature(null).equals(sf))
	    	    return true ;
	    	    
	  if (method == null)
	    method = CodeGenUtil.getWriteMethod(fExpr) ;	  
	  return method != null && method.equals(mSig) ;	
} 

/**
 *  Which FeatureMapper to use
 */
protected void initialFeatureMapper(){
	
	  if (isMethod(ADD_METHOD)) 
	     fFeatureMapper = new ContainerFeatureMapper() ;
	  else
	     super.initialFeatureMapper() ;
               
        
}

/**
 *
 */
protected void initialFeatureMapper(RefStructuralFeature sf){
   if (sf.equals(CodeGenUtil.getComponentFeature(fbeanPart.getRefObject()))) {
       fFeatureMapper = new ContainerFeatureMapper() ;
       fFeatureMapper.setFeature(sf) ;
   }
   else
       super.initialFeatureMapper(sf) ;
}

/**
 * @see com.ibm.etools.jbcf.codegen.java.AbstractExpressionDecoder#isPriorityCacheable()
 */
protected boolean isPriorityCacheable(){
	if(isMethod(ADD_METHOD))
		return false;
	return super.isPriorityCacheable();
}

protected void initialDecoderHelper() {
	if (isMethod(ADD_METHOD)) 
	    fhelper = new ContainerAddDecoderHelper(fbeanPart, fExpr,  fFeatureMapper,this) ;
	else if (isMethod(LAYOUT_METHOD))
	    fhelper = new DelegateRelationshipDecoderHelper(fbeanPart, fExpr,  fFeatureMapper, this);
	else 	      	
	    super.initialDecoderHelper() ;
}

/**
 *  Get the first level descendents
 */
public Vector getChildren(IJavaObjectInstance component) {
  Vector kids = super.getChildren(component) ; 
  java.util.List compList = CodeGenUtil.getChildrenComponents(component) ;
  if (compList != null && compList.size()>0) {    	
	  Iterator itr = compList.iterator() ;
	   while (itr.hasNext()) {    	   
		RefObject child = CodeGenUtil.getCCcomponent((RefObject)itr.next()) ;
		kids.add(child) ;
		kids.add(CodeGenUtil.getComponentFeature(component)) ;
	   }
  }

  // Consider the Layout Manager as a child as it could have attributes with it.  
  RefStructuralFeature sf = (RefStructuralFeature) component.refMetaObject().metaObject(LAYOUT_FEATURE) ;
  if (sf != null) {
    IJavaObjectInstance child = (IJavaObjectInstance)component.refValue(sf) ;
    if (child != null) {
	  kids.add(child) ;
	  kids.add(sf) ;
    }
  }
	
 return kids ; 
}



}