package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import java.util.List;

import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.codegen.util.* ;
import org.eclipse.jdt.internal.compiler.ast.* ;

import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefObject ;
import com.ibm.etools.java.instantiation.base.* ;
import com.ibm.etools.emf.ref.RefStructuralFeature ;


public class DelegateRelationshipDecoderHelper extends ExpressionDecoderHelper {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
static final String                   NULL_STRING ="null" ;	 //$NON-NLS-1$
	
	DelegatedComponentRelationship  fRelashanship = null ;
	BeanPart                        fAddedPart = null ;

public DelegateRelationshipDecoderHelper(BeanPart bean, Statement exp,  IJavaFeatureMapper fm, IExpressionDecoder owner){
	super (bean,exp,fm,owner) ;
}


/**
 * Overide the default action, as we are maintained as a list
 */
public void adaptToCompositionModel(IExpressionDecoder decoder) {
	com.ibm.etools.cde.CDEHack.fixMe("Need to clean this up") ; //$NON-NLS-1$

    // Make sure we do not have stale, need to be a bit smarter of how we do this.
    UnadaptToCompositionModel() ;
    
    BeanDecoderAdapter ba = (BeanDecoderAdapter) fbeanPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;
		
	ExpressionDecoderAdapter adapter = new ExpressionDecoderAdapter(fAddedPart.getRefObject(),decoder) ;		   
	ba.addChildAdapter(fAddedPart,adapter) ;
	if (fAddedPart != null && fAddedPart.getRefObject() != null) {		
	    RefObject ap = fAddedPart.getRefObject() ;
	   // Add a SourceRange adapter for the AddedPart
	   if (ap.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_SOURCE_RANGE) == null) 
	       	  ap.addAdapter(adapter.getShadowSourceRangeAdapter()) ;
	}
}

/**
 * Most helpers should consider overiding this one
 */
public void UnadaptToCompositionModel() {
	com.ibm.etools.cde.CDEHack.fixMe("Need to clean this up") ; //$NON-NLS-1$
	// In the case that this helper is used as a simple attribute.
	try {
      super.UnadaptToCompositionModel() ;
	}
	catch (Exception e) {}
    if (fbeanPart.getRefObject() != null) {
      BeanDecoderAdapter ba = (BeanDecoderAdapter)fbeanPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;		
      if (ba != null)
	     ba.removeChildAdapter(fAddedPart) ;
	  if (fAddedPart!=null && fAddedPart.getRefObject() != null) {
	  	ICodeGenAdapter a = (ICodeGenAdapter) fAddedPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_SOURCE_RANGE) ;
	  	// Only remove shadow adapters
	  	if (a != null && !(a instanceof ExpressionDecoderAdapter))
	  	   fAddedPart.getRefObject().removeAdapter(a) ;
	  }
    }
    else {
    	com.ibm.etools.jbcf.JBCFPlugin.log("UnadaptToCompositionModel: No RefObject",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ;    	 //$NON-NLS-1$
    	return ;
    }
}


/**
 *  We use implicit methods for childrens
 */
public boolean isGenerateOnImplicit() {
	return true ;
}



/*
 * Decoder The Delegate Attribute
 */
protected void add(BeanPart toAdd,BeanPart target) throws CodeGenException {	
      //  Make this relationship in the Composition Model	
      Object previousValue = target.getRefObject().refValue(fFmapper.getFeature(null));
      int index = findIndex(toAdd,target);
      if(previousValue instanceof EList){
      	EList list = (EList) previousValue;
      	if(index<0)
      		list.add(toAdd.getRefObject());
      	else{
      		// The list contains the expression being moved. hence the if().
      		if(list.contains(toAdd.getRefObject()))
      			list.remove(toAdd.getRefObject());
      		else
      			index++;
      		list.add(index, toAdd.getRefObject());
      	}
      }else{
//      	target.getRefObject().refSetValue(fFmapper.getFeature(null),toAdd.getRefObject()) ;
      	CodeGenUtil.refSetValue(target.getRefObject(), fFmapper.getFeature(null), toAdd.getRefObject(), index) ;
      }
      toAdd.setContainer(target) ;
      target.addChild(toAdd) ;      
      JBCFPlugin.log("DelegateRelationShipDecoderHelper.add("+toAdd+","+target+")", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
}

/**
 * Try to figure out where ti insert this object, with accordance to its position
 * in the code, in relation to other components.
 */
protected int findIndex(BeanPart toAdd, BeanPart target) {

	if (fOwner.getExprRef().getOffset() < 0)
		return -1;

	Object val = target.getRefObject().refValue(fFmapper.getFeature(null));
	if (val instanceof List) {
		List compList = (List) val;
		if (compList == null || compList.size() == 0)
			return -1;
	
		int thisOffset = fOwner.getExprRef().getOffset();
		int result = -1;
		for (int i = 0; i < compList.size(); i++) {
			try {
				RefObject cc = (RefObject) compList.get(i);
				if(cc.equals(target.getRefObject()))	
					continue; // account for presence of refobject in the list.
				ICodeGenAdapter a = (ICodeGenAdapter) cc.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_SOURCE_RANGE);
				int tmpOffset = a.getBDMSourceRange().getOffset();
				if (a != null && thisOffset >= tmpOffset)
					result = i;
			} catch (CodeGenException e) {
			}
		}
		return result;
	}
	return -1;
}

protected BeanPart parseAddedPart(MessageSend expr)  throws CodeGenException {
    
     BeanPart added=null ;
     
    
     Expression[] args = (expr).arguments ;
     if (args==null || args.length != 1) throw new CodeGenException("No Arguments !!! " + expr) ; //$NON-NLS-1$
      
      // Parse the arguments to figure out which bean to add to this container
     if (args[0] instanceof MessageSend)  {
          // Look to see of if this method returns a Bean
          String selector = new String (((MessageSend)args[0]).selector) ;  
          added = fOwner.getBeanModel().getBeanReturned(selector) ;           
     }
     else if (args[0] instanceof SingleNameReference){
            // Simple reference to a bean
            String selector = new String (((SingleNameReference)args[0]).token);
            added = fOwner.getBeanModel().getABean(selector) ;
            if(added == null)
            	added = fOwner.getBeanModel().getABean(BeanDeclModel.constructUniqueName(fOwner.getExprRef().getMethod(),selector));
            	//added = fOwner.getBeanModel().getABean(fOwner.getExprRef().getMethod().getMethodHandle()+"^"+selector);
     }
          
     return added ;    
}

/**
 *   Add new Componet to target Bean,
 */
protected boolean  addComponent () throws CodeGenException {
	
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with multiple arguments, and nesting") ; //$NON-NLS-1$
	
	 BeanPart oldAddedPart = fAddedPart ;
    if (!(fExpr instanceof MessageSend)) return false ;
    
    fAddedPart = parseAddedPart((MessageSend)fExpr)  ;
    
    if (oldAddedPart != null)
       if (fAddedPart != oldAddedPart) {
       	BeanPart parent = oldAddedPart.getContainer() ;
       	if (parent.equals(fbeanPart))  // May have moved to another container already
       	    oldAddedPart.setContainer(null) ;
       }
          
    	          
    if (fAddedPart!= null) {	
           add(fAddedPart,fbeanPart) ;
           if (fRelashanship == null)
             fRelashanship = new DelegatedComponentRelationship(fFmapper.getFeatureName(),
                                                           fbeanPart,fAddedPart,
                                                           fOwner.getBeanModel()) ;           
           return true ;
    }		            		
	return fAddedPart != null ;
}

/**
 * Returns the beanpart in the BDM which reflects to
 * the respective StructuralFeature(SF). Sometimes the value
 * of the SF is not a single object, in those instances
 * the help of the passed in argument is taken to determine
 * the required value.
 */
protected BeanPart getCurrentAddedPart(Object arg) {
      Object currentVal = fbeanPart.getRefObject().refValue(fFmapper.getFeature(fExpr)) ;	
      BeanPart  currentBean=null ;
	if (currentVal != null && currentVal instanceof RefObject)  {
		currentBean = fOwner.getBeanModel().getABean((RefObject)currentVal) ;		
	}else 
		if(currentVal != null && currentVal instanceof EList) {
			EList addedList = (EList) currentVal;
			Iterator itr = addedList.iterator();
			RefObject added = null;
			while(itr.hasNext()){
				RefObject ro = (RefObject) itr.next();
				if(ro.equals(arg)){
					added = ro;
					break;
				}
			}
			if(added!=null)
				currentBean = fOwner.getBeanModel().getABean((RefObject)added);
		}
	
	return currentBean ;  
}

public String primRefreshFromComposition(String expSig) throws CodeGenException {
      
      BeanPart  currentBean=getCurrentAddedPart(null) ;
	if (currentBean ==null || !currentBean.equals(fAddedPart)) {
        // Isolate the constraint argument using the last used constraint
            
          
	  int start = expSig.indexOf(fFmapper.getMethodName())+fFmapper.getMethodName().length()+1 ;	
	  int end = expSig.lastIndexOf(')') ;
	  
	  fAddedPart = currentBean ;
	  if (fRelashanship == null) {
	      fRelashanship = new DelegatedComponentRelationship(fFmapper.getFeatureName(),
                                                           fbeanPart,fAddedPart,
                                                           fOwner.getBeanModel()) ;
	  }
	  fRelashanship.setDelegateComponent(fAddedPart) ;
	  
	  String arg = getSourceCodeArg() ;
	  
	  StringBuffer newExp = new StringBuffer(expSig) ;
	  newExp.replace(start,end,arg) ;
 	  fExprSig = newExp.toString() ;
	  return fExprSig ;
	}
	else 
	  return (expSig) ;
}

/**
 *  Deleted from a decoder's point of view is that there is no source code associated
 *  with it. 
 */
public boolean primIsDeleted() {
   
   IJavaObjectInstance obj = (IJavaObjectInstance) fbeanPart.getRefObject() ;
   com.ibm.etools.emf.ref.RefStructuralFeature sf = fFmapper.getFeature(fExpr) ;
   if (sf.refIsMany()) {
       RefObject added = fAddedPart.getRefObject() ;
       return !((List) obj.refValue(sf)).contains(added) ;
   }
   else
      return !obj.refIsSet(fFmapper.getFeature(fExpr)) ;
}

/**
 *   Go for it
 */
public boolean decode() throws CodeGenException {

   if (fFmapper.getFeature(fExpr) == null ||
	   fExpr == null) throw new CodeGenException ("null Feature:"+fExpr) ; //$NON-NLS-1$

      return addComponent() ;
}

public void delete() {
	UnadaptToCompositionModel() ;
	
	com.ibm.etools.emf.ref.RefStructuralFeature sf = fFmapper.getFeature(fExpr) ;
	IJavaObjectInstance parent = (IJavaObjectInstance)fbeanPart.getRefObject() ;	
    if (sf.refIsMany()) {
        RefObject added = fAddedPart.getRefObject() ;
       ((List) parent.refValue(sf)).remove(added) ;
    }
    else	
	    parent.refUnsetValue(sf) ; 
    fAddedPart.setContainer(null) ;
}


public Object getPriorityOfExpression(){
	Integer pri = (Integer)super.getPriorityOfExpression();
	if(fFmapper!=null && fFmapper.getMethodName()!=null && fFmapper.getMethodName().equals(JTableDecoder.JTABLE_ADDCOLUMN_METHOD)){
		int indexOfChildInParent = 0;
		if(fAddedPart!=null){
			if(fFmapper.getFeature(fExpr).refIsMany()){
				List ccs = (List)fAddedPart.getContainer().getRefObject().refValue(fFmapper.getFeature(fExpr));
				for(int i=0;i<ccs.size();i++){
					RefObject cc = (RefObject)ccs.get(i);
					if("ConstraintComponent".equals(cc.refMetaObject().refName())) //$NON-NLS-1$
						cc = CodeGenUtil.getCCcomponent(cc);
					if(cc.equals(fAddedPart.getRefObject())){
						indexOfChildInParent = i;
						break;
					}
				}
			}
		}
		indexOfChildInParent++;
		return new Integer(IJavaFeatureMapper.PRIORITY_ADD_CHANGE + pri.intValue() + (IJavaFeatureMapper.INTER_LAYOUT_ADD_PRIORITIES_GAP-indexOfChildInParent));
	}
	return new Integer(IJavaFeatureMapper.PRIORITY_IMPLICIT + pri.intValue());
}

/**
 *  Detemine the source code argument for the setXXX() method
 */
private String getSourceCodeArg()  {
   
   if (fAddedPart == null) {
   	  Object currentVal = fbeanPart.getRefObject().refValue(fFmapper.getFeature(fExpr)) ;
      if (currentVal == null && !(currentVal instanceof IJavaInstance))
             return NULL_STRING ;
      else         
             return ((IJavaInstance)currentVal).getInitializationString() ;
   }
      
   StringBuffer st = new StringBuffer() ;
   String ref ;
   if (fAddedPart.getReturnedMethod() != null) 
     ref = fAddedPart.getReturnedMethod().getMethodName() ;
   else
     ref = fAddedPart.getSimpleName() ;
   st.append(ref) ;  
   // Send the implicit argument     
   com.ibm.etools.cde.CDEHack.fixMe("Support for implicit") ; //$NON-NLS-1$
   if (false && isImplicit(new Object[] {fAddedPart})) {
   	String rMethod = fFmapper.getReadMethodName() ;
   	st.append (ExpressionTemplate.LPAREN+fbeanPart.getSimpleName()+ExpressionTemplate.DOT+
   	rMethod+ExpressionTemplate.LPAREN+ExpressionTemplate.RPAREN+
   	ExpressionTemplate.RPAREN) ;
   }
   else {
     if (fAddedPart.isInstanceVar())
       st.append (ExpressionTemplate.LPAREN+ExpressionTemplate.RPAREN) ;   
   }
   return st.toString() ;
}
/**
 *  Overide the abstract method, to deal No Decorations.
 */
protected ExpressionTemplate getExpressionTemplate() throws CodeGenException {
		
	
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with layout constraints") ; //$NON-NLS-1$
	
	String arg = getSourceCodeArg() ; // Arguments for the add() method
	
	String sel = fbeanPart.getSimpleName() ;
	String mtd = fFmapper.getMethodName() ;
	ExpressionTemplate exp = new ExpressionTemplate (sel,mtd, new String[] { arg },	                                                
	                                                 null,
	                                                 0 ) ;
	exp.setLineSeperator(fbeanPart.getModel().getLineSeperator()) ;	
	return exp ;   	
}

public String generate(Object[] args) throws CodeGenException {

 	
	if (fFmapper.getFeature(null) == null) 
	    throw new CodeGenException ("null Feature") ; //$NON-NLS-1$
      fAddedPart = getCurrentAddedPart(args==null? null:args[0]) ;
      if (fAddedPart == null) throw new CodeGenException ("No Added Part") ; //$NON-NLS-1$      
      fbeanPart.addChild(fAddedPart) ;
      fAddedPart.setContainer(fbeanPart) ;
      
      if (fRelashanship == null)
        fRelashanship = new DelegatedComponentRelationship(fFmapper.getFeatureName(),
                                                           fbeanPart,fAddedPart,
                                                           fOwner.getBeanModel()) ;
	ExpressionTemplate exp = getExpressionTemplate() ;
      fExprSig = exp.toString() ;	                                                 
      return fExprSig ;	     	
}

 
public boolean isImplicit(Object args[]) {
	BeanPart obj = (BeanPart) getCurrentAddedPart(args==null?null:args[0]) ;
	
	if (obj != null)	
	   return ((IJavaObjectInstance)obj.getRefObject()).isImplicit() ;
	else
	   return false ;
}

public Object[] getArgsHandles(Statement expr) {
	if (!fFmapper.getFeature(fExpr).refIsMany())
	    return null ;  // Argument is an attribute if it is not a many feature.
    Object[] result = null ;    
    try {
      if (fAddedPart == null && expr != null) {
        // Brand new expression
        BeanPart bp = parseAddedPart((MessageSend)expr) ;
        if (bp != null) 
          result = new Object[] { bp.getType()+"["+bp.getSimpleName()+"]" } ; //$NON-NLS-1$ //$NON-NLS-2$
       }
       else if (fAddedPart != null) {
           return new Object[] { fAddedPart.getType()+"["+fAddedPart.getSimpleName()+"]" } ; //$NON-NLS-1$ //$NON-NLS-2$
       }
    }    
    catch (CodeGenException e) {}
    return result ;
}


}