package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import org.eclipse.jdt.core.ISourceRange;

import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;


public class ExpressionDecoderAdapter extends AdapterImpl implements ICodeGenAdapter {
	  
  
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
  protected IExpressionDecoder fDecoder=null ;


public ExpressionDecoderAdapter(Notifier target,IExpressionDecoder decoder) {
	super() ;
	this.target = target ;
	fDecoder = decoder ;
}

/**
 * isAdaptorForType.
 */
public boolean isAdapterForType(Object type) {
	return VCE_CODE_GEN_TYPE.equals(type) ||
	        JVE_CODEGEN_EXPRESSION_SOURCE_RANGE.equals(type)||
	        JVE_CODEGEN_EXPRESSION_ADAPTER.equals(type);
}


protected void primPromoteAttribute(Notification msg, RefObject bInstance, RefStructuralFeature sf) {
    
    BeanDecoderAdapter bAdapter = (BeanDecoderAdapter) bInstance.getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;
    Notification nMsg = new NotificationImpl(
                                 msg.getNotifier(), 
                                 Notifier.SET, 
                                 sf,
                                 null, msg.getNotifier(), 
                                 -1)   ;
    bAdapter.notifyChanged(nMsg)  ;
}

/**
 * If an event occur which adds attributes to the instance
 * of "this" attribute, we need to promote the attribute to
 * a full plagged BeanPart instance.
 */
protected boolean processPromotionIfNeeded(Notification msg) {
    if (msg.getStructuralFeature() == null) return false ;
    if (msg.getStructuralFeature().refID() == null) 
       if (fDecoder.getSF().refID() != null) return false ;
    else
       if (fDecoder.getSF().refID() == null) return false ;
       
    if (msg.getStructuralFeature().refID().equals(fDecoder.getSF().refID()))
      return false ;
    // Is the current value of the SF (arg) now notifies us on its own new attributes ?
    Object val = fDecoder.getBeanPart().getRefObject().refValue(fDecoder.getSF()) ;
    if (!(val instanceof RefObject))return false;
    RefObject curVal = (RefObject) val;
    if (curVal == null) return  false ;
    if (!curVal.equals(msg.getNotifier())) return false ;
    
    RefObject bInstance = fDecoder.getBeanPart().getRefObject() ;
    RefStructuralFeature sf = fDecoder.getSF() ;
    fDecoder.delete() ;
    // A new Expression will be generated by default
    primPromoteAttribute(msg,bInstance,sf) ;
    return true ;
}

/**
 * applied: A setting has been applied to the mof object,
 * notify the decoder
 */ 
public void notifyChanged(Notification msg){

  try {
      
      if (fDecoder.getBeanModel() == null  || 
          fDecoder.getBeanPart().getModel().isStateSet(IBeanDeclModel.BDM_STATE_UPDATING_JVE_MODEL) ||
          fDecoder.getBeanPart().getModel().isStateSet(IBeanDeclModel.BDM_STATE_SNIPPET)) return  ;
      
      
      String action=null ;
	switch ( msg.getEventType() ) {
		case Notifier.TOUCH:
		     action = "TOUCH" ; //$NON-NLS-1$
		     break ;		     
		case Notifier.SET : 
		     action = "SET" ;		      //$NON-NLS-1$
		     break ;
		case Notifier.ADD :
		     action = "ADD" ; //$NON-NLS-1$
		     break ;
		case Notifier.UNSET : 
		     action = "UNSET" ; //$NON-NLS-1$
		     break ;
		case Notifier.REMOVE :
		     action = "REMOVE" ; //$NON-NLS-1$
		     break ;
		case Notifier.REMOVING_ADAPTER:
		     action = "REMOVING_ADAPTER" ;		      //$NON-NLS-1$
		     break;
	}
	JBCFPlugin.log(this+" action= "+action, JBCFMsgLogger.LOG_FINEST) ; //$NON-NLS-1$
JBCFPlugin.log("SourceRange ="+getJavaSourceRange(), JBCFMsgLogger.LOG_FINEST) ;	 //$NON-NLS-1$
	
	if (msg.getNewValue() instanceof Notifier)
	  this.target = (Notifier)msg.getNewValue();
	  
      switch (msg.getEventType()) {
        case Notifier.SET:
              if (processPromotionIfNeeded(msg)) 
                 return ;
        case Notifier.UNSET:
        case Notifier.REMOVE:
        case Notifier.MOVE:          
        if (fDecoder.isRelevantFeature((RefStructuralFeature)msg.getStructuralFeature()))
	       fDecoder.reflectMOFchange() ;      		     
	    break ;      	
      }
  }
  catch (Throwable t) {
      JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING) ;
  }
}


public IExpressionDecoder getDecoder() {
	return fDecoder ;
}

/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	// Insert code to print the receiver here.
	// This implementation forwards the message to super.
	// You may replace or supplement this.
	return "\tExpressionDecoderAdapter:"+fDecoder; //$NON-NLS-1$
}

public ICodeGenSourceRange getJavaSourceRange() throws CodeGenException {
    return fDecoder.getExprRef().getTargetSourceRange() ;
}

/**
 * @see ICodeGenAdapter#getBDMSourceRange()
 */
public ICodeGenSourceRange getBDMSourceRange() throws CodeGenException {
    return new CodeGenSourceRange(fDecoder.getExprRef().getOffset(),fDecoder.getExprRef().getLen()) ;
}

/**
 * @see com.ibm.etools.jbcf.codegen.java.ICodeGenAdapter#getSelectionSourceRange()
 */
public ICodeGenSourceRange getHighlightSourceRange()
	throws CodeGenException {
	return getJavaSourceRange();
}

public ICodeGenAdapter getShadowSourceRangeAdapter() {
	
	return new ICodeGenAdapter() {

		public ICodeGenSourceRange getHighlightSourceRange() throws com.ibm.etools.jbcf.codegen.util.CodeGenException {
			return ExpressionDecoderAdapter.this.getHighlightSourceRange() ;
		}

		public ICodeGenSourceRange getJavaSourceRange() throws com.ibm.etools.jbcf.codegen.util.CodeGenException {
			return ExpressionDecoderAdapter.this.getJavaSourceRange() ;
		}

		public ICodeGenSourceRange getBDMSourceRange() throws com.ibm.etools.jbcf.codegen.util.CodeGenException {
			return ExpressionDecoderAdapter.this.getBDMSourceRange() ;
		}
		public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {}

		public void notifyChanged(Notification msg) {
			com.ibm.etools.jbcf.JBCFPlugin.log("SourceRangeExpressionDecoderAdapter Shadow: msgType=: "+msg.getEventType(), JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
		}

		public com.ibm.etools.emf.notify.Notifier getTarget() {
			return  ExpressionDecoderAdapter.this.getTarget();
		}

		public boolean isAdapterForType(java.lang.Object type) {
			return JVE_CODEGEN_EXPRESSION_SOURCE_RANGE.equals(type);
		}
		public void setTarget(com.ibm.etools.emf.notify.Notifier newTarget) {}
	} ;
	
}

}


