package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.codegen.* ;
import com.ibm.etools.jbcf.codegen.util.* ;
import org.eclipse.jdt.internal.compiler.ast.* ;
import java.util.List ;
import com.ibm.etools.emf.notify.* ;
import com.ibm.etools.emf.ref.NotImplementedException;
import com.ibm.etools.emf.ref.RefStructuralFeature;



public abstract class ExpressionDecoderHelper implements IExpressionDecoderHelper {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
    IExpressionDecoder          fOwner = null ;
    IJavaFeatureMapper          fFmapper = null ;
    Statement                   fExpr ;            // This hold the parsed Source
    String                      fExprSig ;         // This holds the actual source
    BeanPart                    fbeanPart  = null ;
    String                      fdebugString = null ;
	
	
	
public ExpressionDecoderHelper (BeanPart bean, Statement exp, IJavaFeatureMapper fm,IExpressionDecoder owner) {
    fOwner = owner ;
    fbeanPart = bean ;
    fFmapper = fm ;
    fExpr = exp ;
    if (fExpr != null)
       fdebugString = fExpr.toString() ;    
}

public abstract boolean decode () throws CodeGenException ;
public abstract String generate(Object[] args) throws CodeGenException ;
public abstract void delete() ;


/**
 *  If the current value in mof is not the same as we last decoded,
 *  return a string reflecting the new expression.  Rturn null if no
 *  refresh is needed.
 */
public abstract String primRefreshFromComposition(String expSig) throws CodeGenException ;

/**
 *  Is this feature still part the composition
 */
public abstract boolean primIsDeleted() ;

/**
 *  Get the current expression the decoder has
 */
public String getCurrentExpression() {
	return fExprSig ;
}

/**
 *  If this expression is implicit, should the generate() be called ?
 *  Default is no, helpers should overide this otherwise.
 */
public boolean isGenerateOnImplicit() {
	return false ;
}

/**
 * Most helpers should consider overiding this one
 */
public void adaptToCompositionModel(IExpressionDecoder decoder) {
	com.ibm.etools.cde.CDEHack.fixMe("Need to clean this up") ; //$NON-NLS-1$
	// Make sure we do not have stale, need to be a bit smarter of how we do this.
    UnadaptToCompositionModel() ;
	
	Object o = fbeanPart.getRefObject().refValue(fFmapper.getFeature(fExpr)) ;	
	if (o != null && o instanceof Notifier) {
	  Notifier currentVal = (Notifier) o ;
	  if (currentVal.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_ADAPTER) == null) {
	    ExpressionDecoderAdapter adapter = new ExpressionDecoderAdapter(currentVal,decoder) ;		   
	    currentVal.addAdapter(adapter) ;	
	  }
	}
	else if (o == null) {  // null valued e.g., setLayout(null) 
	  BeanDecoderAdapter a = (BeanDecoderAdapter) fbeanPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;
	  ExpressionDecoderAdapter adapter = new ExpressionDecoderAdapter(null,decoder) ;		   	  
	  a.addNullValue(fFmapper.getFeature(null),adapter) ;
	}
}

/**
 * Most helpers should consider overiding this one
 */
public void UnadaptToCompositionModel() {
	com.ibm.etools.cde.CDEHack.fixMe("Need to clean this up") ; //$NON-NLS-1$
	
	if (fbeanPart.getRefObject() == null) {
    	com.ibm.etools.jbcf.JBCFPlugin.log("UnadaptToCompositionModel: No RefObject",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ;    	 //$NON-NLS-1$
    	return ;
    }
	
	Object o = null;
	try {
		o = fbeanPart.getRefObject().refValue(fFmapper.getFeature(fExpr));
	} catch (NotImplementedException e) {}
	if (o instanceof Notifier) {
	  Notifier currentVal = (Notifier) o ;
	  ICodeGenAdapter a = (ICodeGenAdapter)currentVal.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_ADAPTER) ;
	  while (a != null) {
	    currentVal.removeAdapter(a);
	    a = (ICodeGenAdapter)currentVal.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_ADAPTER) ;
	  }
	}
	else if (o == null) {  // null valued e.g., setLayout(null) 
	  BeanDecoderAdapter ba = (BeanDecoderAdapter) fbeanPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;
	  ICodeGenAdapter a = ba.getNullValuedAdapter(fFmapper.getFeature(null)) ;
	  if (a != null)
		try {
			ba.removeNullValue(fFmapper.getFeature(null));
		} catch (NotImplementedException e) {};
	}
}


/**
 *  Create a default ExpressionTemplate with the existing arguments
 */
protected ExpressionTemplate getExpressionTemplate(String[] args) throws CodeGenException {
      String mtd = fFmapper.getDecorator().getWriteMethod().getName() ;
	String sel = fbeanPart.getSimpleName() ;
	ExpressionTemplate exp = new ExpressionTemplate (sel,mtd, 
	                                                 args,
	                                                 null,
	                                                 0 ) ;
	exp.setLineSeperator(fbeanPart.getModel().getLineSeperator()) ;	
	return exp ;   	
}

public Object getPriorityOfExpression(){
	return new Integer(IJavaFeatureMapper.PRIORITY_DEFAULT);
}

public void setDecodingContent (Statement exp) {
	fExpr = exp ;
}

/**
 *  Return a list of unique strings for arguments that make this expression
 *         unique.  e.g.  P1.add(B1,"North") ;   B1 is unique.  "North" is not
 */
public abstract Object[] getArgsHandles(Statement expr) ;


/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	// Insert code to print the receiver here.
	// This implementation forwards the message to super.
	// You may replace or supplement this.
	return (super.toString() + ": " + fdebugString) ; //$NON-NLS-1$
}


/**
 * @see com.ibm.etools.jbcf.codegen.java.IExpressionDecoderHelper#isRelevantFeature(RefStructuralFeature)
 */
public boolean isRelevantFeature(RefStructuralFeature sf) {
	   if (sf == null || fFmapper == null) return false ;
	   RefStructuralFeature hSF = fFmapper.getFeature(null) ;
	   if (hSF == null) return false ;
	   
	   if (hSF.equals(sf) || 
	       (hSF.refID() != null && hSF.refID().equals(sf.refID()))) return true ;
	       
	   return false;
}

}