package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jdt.core.IField;
import org.eclipse.jface.text.BadLocationException;

import com.ibm.etools.cdm.CDMFactory;
import com.ibm.etools.cdm.CDMPackage;
import com.ibm.etools.cdm.KeyedConstraint;
import com.ibm.etools.cdm.model.CDMModelConstants;
import com.ibm.etools.cdm.model.Rectangle;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.model.BeanPart;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.codegen.util.CodeGenUtil;
import com.ibm.etools.jbcf.codegen.util.FreeFormAnnotationTemplate;

/**
 * @version 	1.0
 * @author
 */
public class FreeFormAnnoationDecoder extends AbstractAnnotationDecoder {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
     FreeFormAnnotationTemplate fFFtemplate = null ;        

    /**
     * Constructor for FreeFormAnnoationDecoder.
     */
    public FreeFormAnnoationDecoder(BeanPart bean) {
        super(CDMModelConstants.VISUAL_CONSTRAINT_KEY,bean);
    }
    
    FreeFormAnnotationTemplate getFFtemplate() {
        if (fFFtemplate!=null) return fFFtemplate ;
        
        fFFtemplate = new FreeFormAnnotationTemplate() ;          
        fFFtemplate.setSeperator(fBeanpart.getModel().getLineSeperator()) ;    
        
        return fFFtemplate ;
        
    }
    
    public String generate(RefStructuralFeature sf, Object[] args) throws CodeGenException {
        KeyedConstraint constraint = (KeyedConstraint) getAnnotationValue() ;
        if (constraint == null) {
            fContent = "" ;             //$NON-NLS-1$
        }
        else {
          FreeFormAnnotationTemplate fft = getFFtemplate() ;
          fft.setPosition(constraint.getValue().x,constraint.getValue().y) ;
          fContent = fft.toString() ;
        }
        return fContent ;
    }
    
    protected boolean decode(String src) {
    	  String curAnnotation = FreeFormAnnotationTemplate.getCurrentAnnotation(src) ;
          if (curAnnotation == null)  {
             // No Free Form Information
             return true ;
          }
                    
          JBCFPlugin.log(fBeanpart.getUniqueName()+" Decoding FF annotation", JBCFMsgLogger.LOG_FINE) ;    //$NON-NLS-1$
          
          int[] args = FreeFormAnnotationTemplate.getAnnotationArgs(src,0) ;
          if (args == null) return false ;
          
          CDMPackage cPkg = (CDMPackage) RefRegister.getPackage(CDMPackage.packageURI);
	      CDMFactory cFact = cPkg.getCDMFactory();
	      KeyedConstraint c = cFact.createKeyedConstraint();
	      c.setValue(new Rectangle(args[0],args[1],-1,-1)) ;
	      try {
	      	setAnnotationValue (c) ;
	      }
	      catch (Exception e) {
			return false ;
		  }
	      return true ;    	    	
    }
    
    /*
     * @see IAnnotationDecoder#decode()
     */
    public boolean decode() throws CodeGenException {

      synchronized (fBeanpart.getModel().getDocumentLock()) {
        IField f = CodeGenUtil.getFieldByName(fBeanpart.getSimpleName(),
                                              fBeanpart.getModel().getCompilationUnit()) ;
        
        if (f == null) return false ;
        
        fBeanpart.setFieldDeclHandle(f.getHandleIdentifier()) ;
        try {
         
          int    srcStart = f.getSourceRange().getOffset() ;
          int    srcLen   = f.getSourceRange().getLength() ;
          String src = fBeanpart.getModel().getDocument().get().substring(srcStart+srcLen) ;        
          return decode(src) ;                    
        }
        catch (Exception e) {
            JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
            return false ;
        }
     }     
    }
    
    
    public void reflectMOFchange() {
               
     synchronized (fBeanpart.getModel().getDocumentLock()) {
        IField f = CodeGenUtil.getField(fBeanpart.getFieldDeclHandle(),
                                        fBeanpart.getModel().getCompilationUnit()) ;
        
        try {
         
          int    srcStart = f.getSourceRange().getOffset() ;
          int    srcLen   = f.getSourceRange().getLength() ;
          String src = fBeanpart.getModel().getDocument().get().substring(srcStart+srcLen) ;
          String newSrc = null ;
          int start, len ;
          
          // We want to keep start withing the range of the def., so that the JModel will pick up the comment
          start = srcStart+srcLen ; 
          String curAnnotation = FreeFormAnnotationTemplate.getCurrentAnnotation(src) ;
          if (curAnnotation == null) {
            // Brand New Anotation 
              newSrc = FreeFormAnnotationTemplate.getAnnotationPrefix() + generate(null,null) ;
              if (newSrc == null || newSrc.length() == 0) {
                JBCFPlugin.log(fBeanpart.getUniqueName()+" No FF annotation.", JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
                return ;
              }
              
              
              int commentStart = FreeFormAnnotationTemplate.getAnnotationStart(src) ;
              if (commentStart <0)
                 len = 0 ;                    
              else
                 len = commentStart + FreeFormAnnotationTemplate.ANNOTATION_START.length() ;                 
              
              JBCFPlugin.log(fBeanpart.getUniqueName()+" Creating FF annotation", JBCFMsgLogger.LOG_FINE) ;                          //$NON-NLS-1$
          }
          else {
              JBCFPlugin.log(fBeanpart.getUniqueName()+" Updating FF annotation", JBCFMsgLogger.LOG_FINE) ;    //$NON-NLS-1$
              newSrc = FreeFormAnnotationTemplate.getAnnotationPrefix() + generate(null,null) ;
              int s = FreeFormAnnotationTemplate.getAnnotationStart(src) ;
              int end = FreeFormAnnotationTemplate.getAnnotationEnd(src,s) ;
              len = end+1 ;                
          }
              

          try {
	              fBeanpart.getModel().getDocument().replace(start,len,newSrc) ;
	              JBCFPlugin.log(newSrc, JBCFMsgLogger.LOG_FINE) ;
          }
          catch (BadLocationException e) {
        	    JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;   
          }
          fBeanpart.getModel().updateJavaSource(fBeanpart.getFieldDeclHandle()) ;
        }
        catch (Exception e) {
            JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
        }
     }
        
    }

}
