package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.ISourceRange;
import org.eclipse.jface.text.BadLocationException;

import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.model.BeanPart;
import com.ibm.etools.jbcf.codegen.util.*;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.codegen.util.CodeGenUtil;

/**
 * @author gmendel
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class FreeFormThisAnnotationDecoder extends FreeFormAnnoationDecoder {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * Constructor for FreeFormThisAnnotationDecoder.
	 * @param bean
	 */
	public FreeFormThisAnnotationDecoder(BeanPart bean) {
		super(bean);
	}

	/**
	 * Ties to get the area between the main class' decleration and the end of line.
	 * e.g.,  public myClass extends Foo {   // lskjfdlsjfd 
	 *                                    ^                  ^
	 */	
	public static ICodeGenSourceRange getDesignatedAnnotationRange (ICompilationUnit cu) {
		
/******  The following code used to put the annotation just after the extend,
 *       the problem is that the Java Model think it is part of the next method if
 *       there are not fields in it
 * 		
		    String st ;		    
            int start, len ;
			try {
				ISourceRange nameSR = CodeGenUtil.getMainType(cu).getNameRange() ;
				start = nameSR.getOffset() + nameSR.getLength() ;
				st = cu.getSource().substring(start);				
				
			} catch (JavaModelException e) {
				return null ;
			}
		    com.ibm.etools.cde.CDEHack.fixMe("Should consider syntax error") ;
		    // May not be on the same line
			int braceIndex = st.indexOf('{') ;
			if (braceIndex<0) return null ;
			
			len = 0 ;			
			st = st.substring(braceIndex+1) ;
			
			
			// Terminate it at the EOL
			int endIndex = st.indexOf('\r') ;
			if (endIndex<0)
			   endIndex = st.indexOf('\n') ;
		    if (endIndex<0) return null ;
		    len+=endIndex ;
		    
		    CodeGenSourceRange SR = new CodeGenSourceRange(start+braceIndex+1,len) ;
		    return SR ;
 ******
 *         For now, put it at the end of the class
 */	

		    String st ;		    
            int start, len ;
			try {
				IType t = CodeGenUtil.getMainType(cu) ;
				if(t==null)
					return null;
				ISourceRange range = t.getSourceRange() ;
				start = t.getSource().lastIndexOf('}') ;  	// Method offset
				if (start<0) return null ;
				
				start += range.getOffset() ;  				// CU offset
				st = cu.getSource().substring(start);  
				
			} catch (JavaModelException e) {
				return null ;
			}
		    com.ibm.etools.cde.CDEHack.fixMe("Should consider syntax error") ; //$NON-NLS-1$
		    // May not be on the same line
			
			len = 0 ;			
			
			// Terminate it at the EOL
			int endIndex = st.indexOf('\r') ;
			if (endIndex<0)
			   endIndex = st.indexOf('\n') ;
			if (endIndex<0)
                endIndex=st.length() ;
            
		    len+=endIndex-1 ;
		    
		    CodeGenSourceRange SR = new CodeGenSourceRange(start+1,len) ;
		    return SR ;	    
		    	
	}
	public static String getDesignatedAnnotationString (ICompilationUnit cu) {
		ICodeGenSourceRange sr = getDesignatedAnnotationRange(cu) ;
		if (sr == null) return null ;
		try {			
			return cu.getSource().substring(sr.getOffset(),sr.getOffset()+sr.getLength()) ;
		} catch (JavaModelException e) {
			return null ;
		}
	}
	
	public boolean decode() throws CodeGenException {
		synchronized (fBeanpart.getModel().getDocumentLock()) {
			ICompilationUnit cu = fBeanpart.getModel().getCompilationUnit() ;
			// Main Type decleration name offset
			String src = getDesignatedAnnotationString(cu);
			if (src==null) return false ;
			fBeanpart.setFieldDeclHandle(fBeanpart.THIS_HANDLE) ;
		    return decode(src) ;		
		}		
	}


	/**
	 * @see com.ibm.etools.jbcf.codegen.java.IAnnotationDecoder#reflectMOFchange()
	 */
	public void reflectMOFchange() {
		
	 synchronized (fBeanpart.getModel().getDocumentLock()) {
        
        try {
         
          String src = getDesignatedAnnotationString(fBeanpart.getModel().getCompilationUnit()) ;
          ICodeGenSourceRange curSR = getDesignatedAnnotationRange(fBeanpart.getModel().getCompilationUnit()) ;
          if (src == null || curSR == null) {
          	JBCFPlugin.log("FFThisAnnotationDecoder.reflectMOFchange(): Could not insert THIS FF annotation",JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
          	return ;          	
          }
          String newSrc = null ;
          int start, len ;
                     
          String curAnnotation = FreeFormAnnotationTemplate.getCurrentAnnotation(src) ;
          
          if (curAnnotation == null) {
            // Brand New Anotation 
              newSrc = FreeFormAnnotationTemplate.getAnnotationPrefix() + generate(null,null) ;
              if (newSrc == null || newSrc.length() == 0) {
                JBCFPlugin.log(fBeanpart.getUniqueName()+" No FF annotation.", JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
                return ;
              }
              
              
              int commentStart = FreeFormAnnotationTemplate.getAnnotationStart(src) ;
              if (commentStart <0)
                 len = 0 ;                    
              else
                 len = commentStart + FreeFormAnnotationTemplate.ANNOTATION_START.length() ;                 
              
              JBCFPlugin.log(fBeanpart.getUniqueName()+" Creating FF annotation", JBCFMsgLogger.LOG_FINE) ;                          //$NON-NLS-1$
          }
          else {
              JBCFPlugin.log(fBeanpart.getUniqueName()+" Updating FF annotation", JBCFMsgLogger.LOG_FINE) ;    //$NON-NLS-1$
              newSrc = FreeFormAnnotationTemplate.getAnnotationPrefix() + generate(null,null) ;
              int s = FreeFormAnnotationTemplate.getAnnotationStart(src) ;
              int end = FreeFormAnnotationTemplate.getAnnotationEnd(src,s) ;
              len = end+1 ;                
          }
              

          try {
	              fBeanpart.getModel().getDocument().replace(curSR.getOffset(),len,newSrc) ;
	              JBCFPlugin.log(newSrc, JBCFMsgLogger.LOG_FINE) ;
          }
          catch (BadLocationException e) {
        	    JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;   
          }
          fBeanpart.getModel().updateJavaSource(fBeanpart.getFieldDeclHandle()) ;
        }
        catch (Exception e) {
            JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
        }
     }
        
    }
}


