package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.jbcf.codegen.util.CodeGenException ;
import org.eclipse.jdt.internal.compiler.ast.* ;


/**
 *  A Decoder Helper is designed to deal a with specific type of expressions.
 *  The IExpressionDecoder will provide the logic to determine which helper 
 *  to instanciate.
 */
public interface IExpressionDecoderHelper {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	// Returns the priority of the expression this 
	// decoder helper is the helper of.
	public Object getPriorityOfExpression();
	
   // A decode method will decode specific expressions.
   // If the expression was decoded, a true will be returned.
   // if a false is returned, the expression does not belong to
   // this decoder.
   public boolean decode () throws CodeGenException ;
   // Generate expression source.  Args may or may not be null,
   // depending on the expression type
   public String generate(Object[] args) throws CodeGenException ;
   
   // Delete the existing VCE model element associated with this decoder.
   public void delete() ;
   
   
/**
 *  Reflect the current value in the Composition Model 
 *  @param exprSig represent the current expression signiture in the source code
 *  @return a string representing the expression as reflected from the Compotision
 */   
   public String primRefreshFromComposition(String exprSig) throws CodeGenException ;
   
/**
 *  Is this feature still part the composition
 *  @return a boolean denoting the existance of this feature in the composition
 */
   public boolean primIsDeleted() ;
   
   public boolean isImplicit(Object args[]) ;
   
   public String getCurrentExpression() ;
   
   public void adaptToCompositionModel(IExpressionDecoder decoder) ;
   public void UnadaptToCompositionModel() ;
   /**
    *  If this expression is implicit, should the generate() be called ?
    *  Default is no, helpers should overide this otherwise.
    */
   public boolean isGenerateOnImplicit() ;
   public void setDecodingContent (Statement exp) ;
   public Object[] getArgsHandles(Statement exp) ;
   // is a change in this sf impact this decoder
   public boolean isRelevantFeature (RefStructuralFeature sf) ;
   
   
   
   
}