package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.internal.compiler.ast.*;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.MessageSend;

import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefPackage;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.jbcf.codegen.IDiagramModelInstance;
import com.ibm.etools.jbcf.codegen.model.BeanDeclModel;
import com.ibm.etools.jbcf.codegen.model.BeanPart;
import com.ibm.etools.jbcf.codegen.util.*;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.codegen.util.CodeGenUtil;

/**
 * @version 	1.0
 * @author
 */
public class JTabbedPaneAddDecoderHelper extends AbstractContainerAddDecoderHelper {
    
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
    public static final String ROOT_NAME 				= "plugin.com.ibm.etools.jbcf.visual/overrides/javax/swing/jtabbedPaneVisuals.xmi#JTabComponent" ; //$NON-NLS-1$
    public static final String COMPONENT_ATTR_NAME 		= "component" ; //$NON-NLS-1$
    public static final String TAB_TITLE_ATTR_NAME 		= "tabTitle" ; //$NON-NLS-1$
    public static final String TAB_TOOLTIP_ATTR_NAME 	= "tabTooltipText" ; //$NON-NLS-1$
    public static final String TAB_ICON_ATTR_NAME 		= "tabIcon" ; //$NON-NLS-1$
    
    public static final String TAB_EMPTY_STRING			= "" ; //$NON-NLS-1$
    
    IJavaObjectInstance	fTitleInstance = null,
    						fToolTipInstance = null,
    						fIconInstance = null ;

    /**
     * Constructor for JTabbedPaneAddDecoderHelper.
     * @param bean
     * @param exp
     * @param fm
     * @param owner
     */
    public JTabbedPaneAddDecoderHelper(
        BeanPart bean,
        Statement exp,
        IJavaFeatureMapper fm,
        IExpressionDecoder owner) {
        super(bean, exp, fm, owner);
    }
    
   
 protected  IJavaObjectInstance getRootElement(RefObject root, String sfName) {
     if (root == null) return null ;
     RefStructuralFeature sf = (RefStructuralFeature) root.refMetaObject().metaObject(sfName) ;
     if (sf != null)     
        return (IJavaObjectInstance) root.refValue(sf) ;
     else
        return null ;
 }
 
 protected IJavaObjectInstance getComponent(RefObject root) {
     return getRootElement(root,COMPONENT_ATTR_NAME) ;
 }
 
 protected IJavaObjectInstance getTabTitle(RefObject root) {
     return getRootElement(root,TAB_TITLE_ATTR_NAME) ;
 }
 protected IJavaObjectInstance getToolTip(RefObject root) {
     return getRootElement(root,TAB_TOOLTIP_ATTR_NAME) ;
 }
 protected IJavaObjectInstance getIcon(RefObject root) {
     return getRootElement(root,TAB_ICON_ATTR_NAME) ;
 }
 
/**
 * Decoder Specific Add 
 * @return RefObject the ConstraintComponent object
 */
  protected RefObject add(BeanPart toAdd,BeanPart target, int index) {
    int i = index ;
    if (i<0) {
         i = findIndex(toAdd,target) ;
         // fAddedIndex may inforce an insert mathod, vs. and add method
         //  fAddedIndex = Integer.toString(i) ;
    }
 	toAdd.setContainer(target) ;
	target.addChild(toAdd) ;
      //  Make this relationship in the Composition Model
 	IDiagramModelInstance cm = fbeanPart.getModel().getCompositionModel() ;
      
   EClass rootClass = (EClass) cm.getModelResourceSet().getObjectAndLoad(ROOT_NAME) ;
   RefObject root = (RefObject) ((RefPackage) rootClass.getEObjectContainer()).getFactory().create(rootClass) ;
   
//   root.refSetValue((RefStructuralFeature) root.refMetaObject().metaObject(COMPONENT_ATTR_NAME),
//                    toAdd.getRefObject()) ;
   CodeGenUtil.refSetValue(root, 
                           (RefStructuralFeature) root.refMetaObject().metaObject(COMPONENT_ATTR_NAME), 
                           toAdd.getRefObject(), -1) ;
                    
   if (fTitleInstance != null)
      root.refSetValue((RefStructuralFeature) root.refMetaObject().metaObject(TAB_TITLE_ATTR_NAME),
                        fTitleInstance) ;
                        
   if (fToolTipInstance != null)
      root.refSetValue((RefStructuralFeature) root.refMetaObject().metaObject(TAB_TOOLTIP_ATTR_NAME),
                        fToolTipInstance) ;
                        
   if (fIconInstance != null)
      root.refSetValue((RefStructuralFeature) root.refMetaObject().metaObject(TAB_ICON_ATTR_NAME),
                        fIconInstance) ;
                        
   

   // target's add feature
   RefStructuralFeature cf = fFmapper.getFeature(null) ;
   java.util.List compList = (java.util.List) target.getRefObject().refValue(cf);
   JBCFPlugin.log("JTabbedPaneAddDecoderHelper.add("+toAdd+","+target+"@"+i+")", JBCFMsgLogger.LOG_FINE) ;       //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
   if(i<0)
   	compList.add(root) ;
   else
   	compList.add(Math.min(compList.size(),i), root) ;
//   cf = CodeGenUtil.getParentContainerFeature(root) ;
//   if (cf != null)
//      root.refSetValue(cf,target.getRefObject()) ;
   return root ;

}

protected BeanPart parseAddedPart(MessageSend exp) throws CodeGenException {
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with multiple arguments, and nesting") ; //$NON-NLS-1$
	
	if (exp == null) return null ;
	
	BeanPart bp = null ;
	
	Expression[] args = exp.arguments ;
	if (args.length < 4) throw new CodeGenException("Bad Arguments !!! " + exp) ; //$NON-NLS-1$
      
      // Parse the arguments to figure out which bean to add to this container
      if (args[2] instanceof MessageSend)  {
      	// Look to see of if this method returns a Bean
      	String selector = new String (((MessageSend)args[2]).selector) ;  
      	bp = fOwner.getBeanModel().getBeanReturned(selector) ;       	
      }
      else if (args[2] instanceof SingleNameReference){
            // Simple reference to a bean
            String selector = new String (((SingleNameReference)args[2]).token);
            bp = fOwner.getBeanModel().getABean(selector) ;
            if(bp==null){
            	bp = fOwner.getBeanModel().getABean(BeanDeclModel.constructUniqueName(fOwner.getExprRef().getMethod(),selector));
            	//bp = fOwner.getBeanModel().getABean(fOwner.getExprRef().getMethod().getMethodHandle()+"^"+selector);
            }
      }
            
	return bp ;
}



protected boolean parseAndAddArguments(Expression[] args) throws CodeGenException{
    //  fAddedConstraintInstance = null ;
      if (fAddedPart!= null) {
          // process the Title
          if (args[0] instanceof NullLiteral)
             fTitleInstance = null ;
          else if (args[0] instanceof StringLiteral) {
              fTitleInstance = (IJavaObjectInstance) CodeGenUtil.createInstance("java.lang.String",fbeanPart.getModel().getCompositionModel()) ; //$NON-NLS-1$
              setInitString(fTitleInstance,new String(((StringLiteral)args[0]).source())) ;
          }
          
          com.ibm.etools.cde.CDEHack.fixMe("Need to support local declarations") ; //$NON-NLS-1$
          if (args[1] instanceof NullLiteral)
             fIconInstance = null ;
          else if (args[1] instanceof AllocationExpression) {
              fIconInstance = (IJavaObjectInstance) CodeGenUtil.createInstance("javax.swing.ImageIcon",fbeanPart.getModel().getCompositionModel()) ; //$NON-NLS-1$
              fIconInstance.setInitializationString(((AllocationExpression)args[1]).toString());
          }
          
          if (args[3] instanceof NullLiteral)
             fToolTipInstance = null ;
          else if (args[3] instanceof StringLiteral) {
              fToolTipInstance = (IJavaObjectInstance) CodeGenUtil.createInstance("java.lang.String",fbeanPart.getModel().getCompositionModel()) ; //$NON-NLS-1$
              setInitString(fToolTipInstance,new String(((StringLiteral)args[3]).source())) ;
          }
          
          int index = -1 ;
          if (args.length == 5 && args[4] instanceof IntLiteral) {
                fAddedIndex = ((IntLiteral)args[4]).toStringExpression() ;
                index = Integer.parseInt(fAddedIndex) ;
          }          
          
          RefObject root = add(fAddedPart, fbeanPart, index) ;
          return true ;
          
      }		
	return false ;
}

public String primRefreshFromComposition(String expSig) throws CodeGenException {
        		
      // Isolate the constraint argument using the last used constraint
    String[] args = getSourceCodeArgs() ;
          
	int start = expSig.indexOf(args[0]) ;	
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with identical ") ; //$NON-NLS-1$
	int end = start<0 ? -1 : 
	          expSig.lastIndexOf(args[args.length-1])+ args[args.length-1].length();
	if (start<0 || end<0) {
		JBCFPlugin.log ("SimpleAttr.DecoderHelper.primRefreshFromComposition(): Error", JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
		return expSig ;
	}
	
	removeShadowAdapters() ;
	// Get the latest constraint		
	primRefreshArguments();
	addShadowAdapters() ;
	
	args = getSourceCodeArgs() ;
	// Regenerate the arguments, only
	StringBuffer sb = new StringBuffer() ;
	for (int i=0; i<args.length; i++) {
	   if (i>0) sb.append(", ") ; //$NON-NLS-1$
	   sb.append(args[i]) ;
	}
	// Replace the arguments part	
	StringBuffer newExp = new StringBuffer(expSig) ;
	newExp.replace(start,end,sb.toString()) ;
	fExprSig = newExp.toString() ;
	return fExprSig ;
}

/**
 *   Overidable test: Is this expression fits this decoder
 */
protected boolean isMySigniture()
{
   if (fbeanPart.getRefObject() == null) return false ;
   
   RefStructuralFeature sf = (RefStructuralFeature) fbeanPart.getRefObject().refMetaObject().metaObject(JTabbedPaneDecoder.JTABBED_PANE_FEATURE_NAME) ; 
   if (sf != null)
      return  fFmapper.getFeature(fExpr).equals(sf);	
   else
      return false ;
}

/**
 *  Detemine the source code argument for the add() method
 */
private String[] getSourceCodeArgs()  {
   
    String AddedArg ;
	if (fAddedPart.getInitMethod().equals(fbeanPart.getInitMethod()))   // Added part is defined in the same method as the container
		 AddedArg = fAddedPart.getSimpleName() ;
    else	
	     AddedArg= fAddedPart.getReturnedMethod().getMethodName()+
	                  ExpressionTemplate.LPAREN+ExpressionTemplate.RPAREN ;
	 
	List finalArgs = new ArrayList();
	
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with non String instances") ; //$NON-NLS-1$
	if (fTitleInstance != null)
	   finalArgs.add(fTitleInstance.getInitializationString()) ;
	else
	   finalArgs.add(SimpleAttributeDecoderHelper.NULL_STRING) ;
	   
	if (fIconInstance != null)
	   finalArgs.add(fIconInstance.getInitializationString()) ;
	else
	   finalArgs.add(SimpleAttributeDecoderHelper.NULL_STRING) ;
	   
	finalArgs.add(AddedArg);
	
	if (fToolTipInstance != null)
	   finalArgs.add(fToolTipInstance.getInitializationString()) ;
	else
	   finalArgs.add(SimpleAttributeDecoderHelper.NULL_STRING) ;
	   
	
	if(fAddedIndex!=null){
		finalArgs.add(fAddedIndex);
	}
	String[] args = new String[finalArgs.size()];
	for(int i=0;i<finalArgs.size();i++)
		args[i] = (String)finalArgs.get(i);
	return args;
}

/**
 *  Overide the abstract method, to deal No Decorations.
 */
protected ExpressionTemplate getExpressionTemplate() throws CodeGenException {
		
	
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with layout constraints") ; //$NON-NLS-1$
	
	String[] args = getSourceCodeArgs() ; // Arguments for the add() method
	
	String sel = fbeanPart.getSimpleName() ;
	String mtd ;
	if (fAddedIndex != null)
	  mtd = fFmapper.getIndexMethodName() ;
	else
	  mtd = fFmapper.getMethodName() ;
	ExpressionTemplate exp = new ExpressionTemplate (sel,mtd, args,	                                                
	                                                 null,
	                                                 0 ) ;
	exp.setLineSeperator(fbeanPart.getModel().getLineSeperator()) ;	
	return exp ;   	
}

protected void primRefreshArguments() {
	
	
	// indexValueFound will designate if to use and index APi or not
    if(indexValueFound || fAddedIndex!=null){ 	
    	// Add an index position, as it was found.
    	List parentsChildren = getRootComponentList() ; 
    	int indexOfAddedPart = parentsChildren.indexOf(getRootObject(false));
    	fAddedIndex = Integer.toString(indexOfAddedPart);	
    }
    else 
        fAddedIndex = null;
	
	RefObject root = getRootObject(false) ;	
	fIconInstance = getIcon(root) ;
	fTitleInstance = getTabTitle(root) ;
	fToolTipInstance = getToolTip(root) ;
	
}

/**
 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#getComponentArguments(RefObject)
 */
protected IJavaObjectInstance[] getComponentArguments(RefObject root) {
	IJavaObjectInstance args[] = new IJavaObjectInstance[3] ;
	args[0] = getIcon(root) ;
	args[1] = getTabTitle(root) ;
	args[2] = getToolTip(root) ;
	
	return args ;
}

/**
 * Overide to include dependant SFs
 */
public boolean isRelevantFeature(RefStructuralFeature sf) {
	if (super.isRelevantFeature(sf)) return true ;	
	RefObject root = getRootObject(true) ;
	if (root == null) return false ;
	
	RefStructuralFeature hSF[] = new RefStructuralFeature[3] ;
	hSF[0] = (RefStructuralFeature) root.refMetaObject().metaObject(TAB_ICON_ATTR_NAME) ;
	hSF[1] = (RefStructuralFeature) root.refMetaObject().metaObject(TAB_TOOLTIP_ATTR_NAME) ;
	hSF[2] = (RefStructuralFeature) root.refMetaObject().metaObject(TAB_TITLE_ATTR_NAME) ;
	
	if (sf != null) {
		for (int i=0; i<hSF.length; i++) {
		  if (sf.equals(hSF[i]) ||
		     (hSF[i].refID() != null && hSF[i].refID().equals(sf.refID()))) return true ;
		}
	}
	return false ;
}

}
