package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.Vector;

import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.EList;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.codegen.IDiagramModelInstance;
import com.ibm.etools.jbcf.codegen.model.*;

/**
 * @version 	1.0
 * @author
 */
public class JTabbedPaneDecoder extends AbstractCompositionalDecoder {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String JTABBED_PANE_METHOD 		= "addTab"; //$NON-NLS-1$
	public static final String JTABBED_PANE_INDEX_METHOD	= "insertTab";	 //$NON-NLS-1$
	public static final String JTABBED_PANE_FEATURE_NAME	= "tabs"; //$NON-NLS-1$
	protected static String[] structualFeatures = new String[] {JTABBED_PANE_FEATURE_NAME, JTABBED_PANE_FEATURE_NAME};
	protected static String[] writeMethodNames = new String[] {JTABBED_PANE_METHOD, JTABBED_PANE_INDEX_METHOD};
	
	/*
	 * @see AbstractCompositionalDecoder#getAppropriateDecoderHelper(String)
	 */
	/**
	 * Constructor for JTabbedPaneDecoder.
	 * @param expr
	 * @param model
	 * @param cm
	 * @param part
	 * @param structualFeatures
	 * @param writeMethodNames
	 */
	public JTabbedPaneDecoder(
		CodeExpressionRef expr,
		IBeanDeclModel model,
		IDiagramModelInstance cm,
		BeanPart part) {
		super(expr, model, cm, part, structualFeatures, writeMethodNames);
	}

	/**
	 * Constructor for JTabbedPaneDecoder.
	 * @param structualFeatures
	 * @param writeMethodNames
	 */
	public JTabbedPaneDecoder() {
		super(structualFeatures, writeMethodNames);
	}

	protected IExpressionDecoderHelper getAppropriateDecoderHelper(String structuralFeature) {
		return new JTabbedPaneAddDecoderHelper (fbeanPart, fExpr,  fFeatureMapper, this);
	}

	public Vector getChildren(IJavaObjectInstance component) {
		Vector kids = super.getChildren(component);
		for(int sfc=0;sfc<structuralFeatures.length;sfc++){ // Is SF containing children present?
			if(structuralFeatures[sfc].length()<1)
				continue;
			RefStructuralFeature sf = (RefStructuralFeature) component.refMetaObject().metaObject(structuralFeatures[sfc]) ;
			if(!(component.refValue(sf) instanceof EList))
				continue;
			EList children = (EList)component.refValue(sf) ;
			Iterator itr = children.iterator();
			while(itr.hasNext()){
				RefObject child = (RefObject) itr.next();
				if (child != null) {
					kids.add(child) ;
					kids.add(sf) ;
				}
			}
		}
		return kids ;
	}
	/*
	 * @see AbstractCompositionalDecoder#getAppropriateFeatureMapper(String)
	 */
	protected IJavaFeatureMapper getAppropriateFeatureMapper(String structuralFeature) {
		return new JTabbedPaneFeatureMapper() ;
	}
	
	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractCompositionalDecoder#isInternalPriorityCacheable()
	 */
	protected boolean isInternalPriorityCacheable() {
		return false;
	}

}
