package com.ibm.etools.jbcf.codegen.java; 
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import  com.ibm.etools.jbcf.codegen.util.* ;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.IDiagramModelInstance;
import  com.ibm.etools.jbcf.codegen.model.* ;
import  java.io.* ;
import  java.util.* ;

import  org.eclipse.jdt.internal.compiler.parser.* ;
import  org.eclipse.jdt.internal.core.* ;
import  org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration ;
import  org.eclipse.jdt.internal.compiler.problem.ProblemReporter ;
import  org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory ;
import org.eclipse.jdt.internal.compiler.*;
import  org.eclipse.jdt.internal.compiler.CompilationResult ;
import  org.eclipse.jdt.internal.compiler.DefaultErrorHandlingPolicies ;
import  org.eclipse.jdt.internal.compiler.impl.CompilerOptions ;
import  org.eclipse.jdt.internal.compiler.ast.TypeDeclaration ;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.compiler.IProblem;




/**
 * This class is used to build a main BDM. Will creat use the Local compilation
 * unit, and will set up the BDM appropriatly.   
 * 
 * Do not use this builder to build a BDM if a main BDM already exist.  Use the JavaBeanShadowModelBuilder
 * instead.
 */

public class JavaBeanModelBuilder {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
  String     fFileName = null ;     //  Java Source 
  char[]    fFileContent = null ;
  char[][]  fPackageName = null ;
  
  CompilationUnitDeclaration 	fJDOM  = null ;  // JDOM
  IBeanDeclModel 				fModel = null ; 
  ICompilationUnit				fCU = null ;
  IWorkingCopyProvider          fWCP = null ;
  IDiagramModelInstance         fDiagram  = null ;
	
public JavaBeanModelBuilder(String fileName, char[][] packageName) {
  	this.fFileName = fileName ;
  	this.fPackageName = packageName ;

}

public void setDiagram(IDiagramModelInstance diag) {
    fDiagram = diag ;
}


/**
 *  
 */
public JavaBeanModelBuilder(IWorkingCopyProvider wcp, String filePath, char[][] packageName) {
	this (filePath,packageName) ;
  	fCU = wcp.getLocalWorkingCopy() ;
  	fWCP = wcp ;
}


/**
 *  Read the source file
 */
protected char[] getFileContents() throws CodeGenException {

  char[] result = null ; 
  
  try {  
    if (fCU != null) { // even if the CU is not parsable, get it.
  	    result = fCU.getSource().toCharArray() ;  	  	
    }
    else 
      throw new CodeGenException ("Could not get a valid source text") ; //$NON-NLS-1$
  }
  catch (JavaModelException e) {} 
  
  return result ;
}

/**
 *  Get the JDOM for the input source file
 */
protected void ParseJavaCode() throws CodeGenException
{
	// To get the JDOM of the file we need a compilation unit
	String encoding = (String) JavaCore.getOptions().get(CompilerOptions.OPTION_Encoding);
	if ("".equals(encoding)) encoding = null;  //$NON-NLS-1$
	BasicCompilationUnit sourceUnit = new BasicCompilationUnit( fFileContent , fPackageName, fFileName,  encoding) ;
	// Create a default reporter to handle any parser errors.
	ProblemReporter reporter =new ProblemReporter(
		DefaultErrorHandlingPolicies.exitAfterAllProblems(),
		new CompilerOptions(),
		new DefaultProblemFactory(Locale.getDefault())
	);
	CompilationResult result = new CompilationResult(sourceUnit ,1,1,20);
	fJDOM = getModelFromParser(reporter, result, sourceUnit);	


    // Need to refine here, right now pick up the first problem
    // Note that we do not pick up compile problems, like extending an object 
    // without the proper constructors
	if (result.hasErrors())  {
		IProblem[] problems = result.getProblems() ;
		String forDebugging = new String (fFileContent)  ;
		for (int i=0; i< problems.length; i++) {
			if (problems[i].isError()) {
			  
			  throw new CodeGenSyntaxError ("JVE Parsing Error: "+problems[i].getMessage()+ //$NON-NLS-1$
			                              " - Line: "+Integer.toString(problems[i].getSourceLineNumber())) ;			   //$NON-NLS-1$
			}
		}
		
	}
} 
 
  
public CompilationUnitDeclaration getModelFromParser(
	ProblemReporter reporter,
	CompilationResult result,
	BasicCompilationUnit cu){
	Parser aParser = new Parser(reporter,true,true);
	return aParser.parse(cu,result);	
}
	
  
/**
 *  Initialize a new Bean Decleration Model
 */  
protected void CreateBeanDeclModel() throws CodeGenException {
	fModel = createDefaultModel();
    fModel.setJDOM(fJDOM) ;
    fModel.setWorkingCopyProvider(fWCP) ;
    if (fJDOM.types == null) throw new CodeGenException ("No Type to work on") ; //$NON-NLS-1$
    fModel.setTypeDecleration((TypeDeclaration)fJDOM.types[0]) ;    
    fModel.setCompositionModel(fDiagram) ;
}  

protected IBeanDeclModel createDefaultModel(){
	return new BeanDeclModel() ;
}

/**
 *
 */
void  setLineSeperator() {
	fModel.setLineSeperator(System.getProperty("line.separator")) ; //$NON-NLS-1$
	for (int i=0; i<fFileContent.length; i++) {
		if (fFileContent[i] == '\r' || fFileContent[i] == '\n' ) {
			if (fFileContent[i] == '\r')
			   if (i+1<fFileContent.length && fFileContent[i+1] == '\n')
			     fModel.setLineSeperator("\r\n") ; //$NON-NLS-1$
			   else
			     fModel.setLineSeperator("\r") ; //$NON-NLS-1$
			else
			     fModel.setLineSeperator("\n") ; //$NON-NLS-1$
			break ;
		}
	}
}


/**
 * The BDM reflecting the parsed code may contain BeanParts that have no 
 * associated expression/s, or no method with an initialization expression.
 * 
 * This method will remove these instances from the model
 */
protected void cleanModel () {

	Iterator itr = fModel.getBeans().iterator() ;
	ArrayList err = new ArrayList() ;
	
	while (itr.hasNext()) {
		BeanPart bean = (BeanPart) itr.next();
		boolean removeFlag = false;

		if (!bean.getSimpleName().equals(bean.THIS_NAME)) {
			if (BeanPartGenerator.getInstanceInitializationExpr(bean) == null){
//			    &&  (bean.getReturnedMethod() == null || bean.getRefExpressions().isEmpty())) {
				JBCFPlugin.log("*Discarting a beanPart " + bean, JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$
				removeFlag = true;
			}
		}

		if (removeFlag == true) {
			err.add(bean);
			// Children will not be connected to the VCE model
			Iterator bItr = bean.getChildren();
			if (bItr != null)
				while (bItr.hasNext())
					err.add(bItr.next());
		}
	}
	for (int i = 0; i < err.size(); i++) {
		((BeanPart) err.get(i)).dispose();
	}
}

/**
 *  Go for it
 */  
public IBeanDeclModel build () throws CodeGenException {

JBCFPlugin.log ("JavaBeanModelBuilder.build() starting .... ", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$


    if (fFileName == null || fFileName.length() == 0) throw new CodeGenException("null Input Source") ;     //$NON-NLS-1$
    fFileContent = getFileContents() ;
    // Build a JDOM
    ParseJavaCode () ;    
	CreateBeanDeclModel() ;
	setLineSeperator() ;
	  
    // Some of the Visitors may not be able to resolve some construct on the first path,
    // Run them again, if they put themself on the re-try list.
    try {
		List  tryAgain = new Vector () ;
	    
	    if (fJDOM.types[0] == null) throw new CodeGenException(".. no type was parsed ...") ; //$NON-NLS-1$
	    visitType((TypeDeclaration)fJDOM.types[0],fModel,fFileContent,tryAgain) ;
	
	    // Let the non resolved visitor a chance to run again.    
	    for (int i=0; i<tryAgain.size(); i++) {
	    	ISourceVisitor visitor = (ISourceVisitor) tryAgain.get(i) ;
	    	visitor.setNoRetry() ;
	    	visitor.visit() ;
	    }
	    
	    // Update the parent
	    Iterator itr = fModel.getBeans().iterator() ;
		while (itr.hasNext()) {
	       BeanPart bean = (BeanPart) itr.next() ;
	       bean.setModel(fModel) ;
		}
		
		cleanModel() ;
		
	} catch(Exception e) {
	    com.ibm.etools.jbcf.JBCFPlugin.log(e) ;
	}
	finally {        
       JBCFPlugin.log ("JavaBeanModelBuilder.build(), Done.", JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
	}
   return fModel ;
   
}
    
protected void visitType(TypeDeclaration type, IBeanDeclModel model, char[] content, List tryAgain){
	new TypeVisitor(type,model,tryAgain,false).visit()  ;
}
	
}

