/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.etools.jbcf.codegen.java;

import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.ui.IFileEditorInput;

import com.ibm.etools.jbcf.codegen.model.BeanDeclModel;
import com.ibm.etools.jbcf.codegen.model.IBeanDeclModel;
import com.ibm.etools.jbcf.codegen.util.*;

/**
 * @author gmendel
 * 
 * This class is intendent to build a logical BDM that will not harm, or be connected
 * to existing JVE model or source code. 
 * The compilation unit that is used, is the shared compilation unit.
 * 
 * It is used mainly to build a temporary BDM for comparisons
 * 
 */
public class JavaBeanShadowModelBuilder extends JavaBeanModelBuilder { 
 
		
	protected ICompilationUnit referenceCU =null;
	protected CodegenTypeResolver resolver = null ;
	protected String			   fwcContents = null ;
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	
	/**
	 * Constructor for JavaBeanShadowModelBuilder.
	 * @param wcp
	 * @param filePath
	 * @param packageName
	 */
	public JavaBeanShadowModelBuilder(IWorkingCopyProvider wcpArg, String filePath, char[][] packageName) {
		this(filePath, packageName);

		fWCP = createPseudoWorkingCopyProvider(wcpArg.getSharedWorkingCopy());
		try {
			fwcContents = wcpArg.getSharedWorkingCopy().getSource() ;
		}
		catch (JavaModelException e) {
			com.ibm.etools.jbcf.JBCFPlugin.log("JavaBeanShadowModelBuilder - *error* getting source code") ; //$NON-NLS-1$
		}
		// Make sure fCU use is limited to fWCP
		fCU = null ;

	}
	
	protected IWorkingCopyProvider createPseudoWorkingCopyProvider(ICompilationUnit cu) {
		referenceCU = cu ;
		resolver = new CodegenTypeResolver(CodeGenUtil.getMainType(referenceCU));

		//	Create a pseudo working provider that is limited to resolve, and provide a CU
		IWorkingCopyProvider wcp = new IWorkingCopyProvider() {

			public ICompilationUnit getSharedWorkingCopy() {
				return null;
			}
			public ICompilationUnit getSharedWorkingCopy(boolean force) {
				return null;
			}
			/**
			 * Let the visitors the ability to get element handels etc.
			 */
			public ICompilationUnit getLocalWorkingCopy() {
				return  referenceCU;
			}
			public IFileEditorInput getEditor() {
				return null;
			}
			public IFile getFile() {
				return null;
			}
			public IDocument getSharedDocument() {
				return null;
			}
			public IDocument getLocalDocument() {
				return null;
			}
			public Object getLocalDocLock() {
				return null;
			}
			public void ReplaceWithLocalContent(IProgressMonitor pm, boolean commit) throws CodeGenException {
				;
			}
			public void UpdateDeltaToShared(ICancelMonitor pm, IDocumentListener docListener, List handles, boolean commit) throws CodeGenException {
				;
			}
			public void aboutToChangeShared() {
				;
			}
			public void changeCompleteShared() {
				;
			}
			public ISourceRange getSharedSourceRange(String handle) {
				return null;
			}
			public int getSharedLineNo(int Offset) {
				return -1;
			}
			public void dispose() {
				;
			}
			public String resolve(String unresolved) {
				try {
					if (resolver == null)
						return unresolved;
					else
						return resolver.resolveTypeComplex(unresolved);
				}
				catch (Exception e) {
					return unresolved;
				}
			}
			public String resolveThis() {
				return CodeGenUtil.getMainType(referenceCU).getTypeQualifiedName();
			}
			public ITypeHierarchy getHierarchy() {
				try {
					IType t = CodeGenUtil.getMainType(referenceCU);
					return t.newSupertypeHierarchy(null);
				}
				catch (org.eclipse.jdt.core.JavaModelException e) {}
				return null;
			}
			public void disconnect() {}
			public void reconnect(IFile file) {}
			public String toString() {
				return "Shadow - "+super.toString() ;
			}
				
		};
		
		return wcp ;
	}

	public JavaBeanShadowModelBuilder(String fileName, char[][] packageName) {
		super(fileName, packageName);
	}

	protected IBeanDeclModel createDefaultModel() {
		BeanDeclModel model = new BeanDeclModel();
		try {
			model.setState(IBeanDeclModel.BDM_STATE_SNIPPET, true);
		}
		catch (CodeGenException e) {}
		return model;
	}
	
	/**
	*  Read the source file
	*/
	protected char[] getFileContents() throws CodeGenException {
		if (fwcContents != null)
			return fwcContents.toCharArray();
		else
			return new char[0];
	}


	
}


