package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import org.eclipse.jdt.core.*;
import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.codegen.util.*;
import java.util.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.java.instantiation.base.* ;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;

import org.eclipse.jdt.core.jdom.* ;

import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.java.rules.* ;
import com.ibm.etools.java.impl.JavaClassImpl ;
import com.ibm.etools.java.JavaHelpers ;
import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.emf.ecore.EClassifier ;
import com.ibm.etools.emf.ecore.EStructuralFeature;



public class MethodTextGenerator {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected   IBeanDeclModel    fModel ;
protected   IDOMMethod        fJdomMethod ;
protected   CodeMethodRef     fMethodRef ;
protected   RefObject         fComponent ;
protected   String            freturnType ;
protected   String            fName ;
protected   String            fMethodName ;
protected   String[]          fComments = null ;

	
public MethodTextGenerator (RefObject component, IBeanDeclModel model) {
	fModel = model ;
	fComponent = component ;
}



/**
 *  Create a new Model element and a corresponding Java Source
 */
protected CodeExpressionRef GenerateAttribute(RefStructuralFeature sf,BeanPart bean,Object [] args, boolean updateDoc) throws CodeGenException {
	
    ExpressionRefGenerator egen = new ExpressionRefGenerator(bean,sf) ;      	
	CodeExpressionRef exp = egen.createFromVCEModel(args) ;	
	if (updateDoc)
     if (exp.getMethod().getMethodHandle() != null)
	     exp.insertContentToDocument(true) ;
     else
         exp.setState(exp.STATE_EXP_NOT_PERSISTED, true) ;
	return exp ;
}

public void setComments (String[] comments) {
	fComments = comments ;
}

/**
 *  This is a generic utility to note attributes that should not be reflected in the Java
 *  source code as a generic attribute.
 *  @return boolean, true to omit, false attribute shuld be reflected.
 */
public static boolean isNotToBeCodedAttribute(RefStructuralFeature sf, RefObject component) {		
	// Container attributes are addressed seperatly
	if (sf==null ||
       ((sf instanceof EStructuralFeature) && ((EStructuralFeature)sf).isTransient()) ||
//	    sf.equals(CodeGenUtil.getParentContainerFeature(component)) ||
	    sf.equals(CodeGenUtil.getComponentFeature(component)) ||
	    sf.equals(CodeGenUtil.getImplicitFeature(component)) ||
	    sf.equals(CodeGenUtil.getInitStringFeature(component))
	    )
	    return true ;	

//	// If it is a layout manager constraint of my container, let container's add() method take care of it
//	if (sf.equals(CodeGenUtil.getConstraintFeature(component)) &&
//	    !ConstraintDecoderHelper.primIsFreeFormConstraint(component) &&
//	    !CodeGenUtil.getCompositionInstance(component).isAssignableFrom((EClassifier) ((IJavaObjectInstance)component).getJavaType()))
//	    return true ;	
	    
	return false ;
}

/**
 *  Determine if a SF impact java source
 */
protected boolean ignoreSF(RefStructuralFeature sf) {
	com.ibm.etools.cde.CDEHack.fixMe("Need to set up a hash somewere") ; //$NON-NLS-1$
	return isNotToBeCodedAttribute(sf,fComponent) ;	
}

protected void appendNewSource(StringBuffer buf,BeanPart bean, List kids) throws CodeGenException {
    appendNewSource(buf,bean,kids,false) ;
}
/**
 *  Look for added attributes
 *  
 */
protected void appendNewSource(StringBuffer buf,BeanPart bean, List kids, boolean updateDoc) throws CodeGenException {
	
	RefObject obj = bean.getRefObject() ;
	Iterator itr = obj.refCurrentValues(true).iterator() ;
	while (itr.hasNext()) {
		Value val = (Value) itr.next() ;
		if (ignoreSF(val.getSF()) || kids.contains(val.getSF())) continue ;
		if (val.getValue() != null && val.getValue() instanceof Notifier)  {
		   Notifier elm = (Notifier) val.getValue() ;
		   // Source already exists
		   if (elm.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) != null) 
		     continue ;
		}
		else if (val.getValue() == null) {
		  // Null valued SF (e.g., setLayout(null)) ;
		  ICodeGenAdapter a = (ICodeGenAdapter) obj.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
		  if (a != null && a instanceof BeanDecoderAdapter &&
		      ((BeanDecoderAdapter)a).getNullValuedAdapter(val.getSF()) != null)
		      continue ;
		}
        CodeExpressionRef newExpr = GenerateAttribute(val.getSF(),bean,null,updateDoc) ;
		String src = newExpr.getContent() ;
		if (src == null) throw new CodeGenException ("Could not Generate Source") ;	 //$NON-NLS-1$
		JBCFPlugin.log ("\tAdding: "+src, JBCFMsgLogger.LOG_FINE) ;	       //$NON-NLS-1$
        if (!updateDoc)
          newExpr.setOffset(buf.length()) ;
		buf.append(src) ;		      	
	}
}

/**
 *  Create a NOop expression reference for the initializationn expression..
 */
CodeExpressionRef createInitExpression(BeanMethodTemplate mt,BeanPart bean) {
	ExpressionRefGenerator eg = new ExpressionRefGenerator(bean,null) ;
	return eg.createInitExpression(mt) ;
}

/**
 * Parent Child relationship may be aggregated by a middle object (e.g., Container Constraint)
 */
protected Object getChildRoot(BeanPart bean, BeanPart child,RefStructuralFeature sf) {
    if (bean == null || child == null) return null ;
    if (child.getRefObject().refContainer().equals(bean.getRefObject()))
       return child.getRefObject() ;
    else
       return child.getRefObject().refContainer() ;
}

/**
 *  Look for added attributes
 *  
 */
protected void GenerateAChild(StringBuffer buf,BeanPart bean, BeanPart child,RefStructuralFeature sf) throws CodeGenException {
	
	Object actualChild = getChildRoot(bean,child,sf) ;
    CodeExpressionRef newExpr = GenerateAttribute(sf,bean,new Object[] { actualChild },false) ; ;
	String src =  newExpr.getContent() ;
	if (src == null) {
		JBCFPlugin.log ("No Source Generated for "+bean.getUniqueName()+"("+sf+")", JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		return ;
	}	
    
    // The generator assume that offsets will be update once we insert the exp. to the doc.
    // We are going to set the offset explicitly
    CodeMethodRef mref = newExpr.getMethod() ;
    mref.removeExpressionRef(newExpr) ;
    newExpr.setOffset(buf.length()) ;
    mref.addExpressionRef(newExpr) ;
	buf.append(src) ;		      	
	JBCFPlugin.log ("\tAttribute source added: "+src, JBCFMsgLogger.LOG_FINE) ;	       //$NON-NLS-1$
	
}

protected void setConstractorString (BeanMethodTemplate tp, Object component) {
    if (component instanceof IJavaObjectInstance) {
     // use the init string rather than default template string
        IJavaObjectInstance obj = (IJavaObjectInstance) component ;
        String st = obj.getInitializationString() ;  
        CDEHack.fixMe("Temp. until we have a constructor based init support") ; //$NON-NLS-1$
        int index =  st.indexOf("(,") ; //$NON-NLS-1$
        if (index>=0) {            
            RefObject parent = (RefObject) obj.refContainer() ;
            BeanPart pBP = fModel.getABean(parent) ;
            String toAdd = pBP.getSimpleName();
            StringBuffer s = new StringBuffer (st) ;            
            s.replace(index, index+2, "("+toAdd+", ") ; //$NON-NLS-1$ //$NON-NLS-2$
            st = s.toString() ;
        }
        tp.setBeanConstructorString(st) ;
    }
}

public String generateInLine(CodeMethodRef method,String beanName, List kids) throws CodeGenException {
    
    fMethodRef = method ;    
    freturnType = ((IJavaObjectInstance)fComponent).getJavaType().getQualifiedName() ;
    fName=beanName ;
    
                   
    // Set up a new BeanPart in the decleration Model
    BeanPart bp = fModel.getABean(fName) ;
    if(bp==null)
    	bp = fModel.getABean(BeanDeclModel.constructUniqueName(method,fName));//method.getMethodHandle()+"^"+fName);
 	BeanMethodTemplate template = new BeanMethodTemplate(freturnType,fName,"",fComments) ; //$NON-NLS-1$
    setConstractorString (template, fComponent) ;
 	template.setSeperator(fModel.getLineSeperator());
 	template.setInLineMethod(true) ;
    
   StringBuffer sb = new StringBuffer () ;
    
   CodeExpressionRef initExp = createInitExpression(template,bp) ;
   // Allow the expression sorted to find a nice spot for this one
   initExp.setState(initExp.STATE_SRC_LOC_FIXED, false); // initExp.setState(initExp.getState()&~initExp.STATE_SRC_LOC_FIXED) ;
   initExp.setOffset(-1) ;
   try {   
       method.updateExpressionOrder() ;
   }
   catch (Throwable e) {
        JBCFPlugin.log(e, JBCFMsgLogger.LOG_SEVERE) ;
        return null ;
   }
   // We may be processing a nested child, 
   // and the method is not in the source yet
   if (method.getMethodHandle() != null)
      initExp.insertContentToDocument(true) ;
   else
      initExp.setState(initExp.STATE_EXP_NOT_PERSISTED, true) ;
            
    appendNewSource(sb,bp,kids,true) ;
    if (kids != null) {
    	Iterator itr = kids.iterator() ;
    	while (itr.hasNext()) {
    	  BeanPart child = (BeanPart)itr.next() ;
    	  child.setContainer(bp) ;    	      	      	  
    	  GenerateAChild(sb,bp,child,(RefStructuralFeature)itr.next()) ;    	  
      }
    }    
    sb.append(template.getPostfix()) ;
    
    return (sb.toString()) ;
}


public String generate(CodeMethodRef method,String methodName,String beanName, List kids) throws CodeGenException {
	  
    fMethodRef = method ;    
    freturnType = ((IJavaObjectInstance)fComponent).getJavaType().getQualifiedName() ;
    fName=beanName ;
    fMethodName = methodName ;
                   
    // Set up a new BeanPart in the decleration Model
    BeanPart bp = fModel.getABean(fName) ;
 	
    BeanMethodTemplate template = new BeanMethodTemplate(freturnType,fName,fMethodName,fComments) ;
    setConstractorString (template, fComponent) ;
    template.setSeperator(fModel.getLineSeperator()) ;
//    if (((IJavaObjectInstance)fComponent).isImplicit()) {
//    	template.setImplicit(true) ;
//    }
    StringBuffer sb = new StringBuffer () ;
    sb.append (template.getPrefix()) ;
    
    // Some Expressions may have generated by nested, internal children of this bean.
    ArrayList shadowExprList = new ArrayList() ;
    Iterator itr = fMethodRef.getExpressions() ;
    while (itr.hasNext()) 
        shadowExprList.add(itr.next()) ;
    for (int i=0; i<shadowExprList.size(); i++) 
        fMethodRef.removeExpressionRef((CodeExpressionRef)shadowExprList.get(i)) ;
        
    createInitExpression(template,bp) ;
    
    // Add back the internal expressions, if any 
    for (int i=0; i<shadowExprList.size(); i++) {
        CodeExpressionRef exp = (CodeExpressionRef)shadowExprList.get(i) ;
        exp.setOffset(sb.length()) ;
        sb.append(exp.getContent()) ;
        // How else will we have a shadow expression ?
        if (!exp.isStateSet(exp.STATE_EXP_NOT_PERSISTED)) throw new CodeGenException ("STATE_EXP_NOT_PERSISTED is not set for existing expression") ; //$NON-NLS-1$
        exp.setState(exp.STATE_EXP_NOT_PERSISTED, false) ;
        fMethodRef.addExpressionRef(exp) ;        
    }
            
    appendNewSource(sb,bp,kids) ;
    if (kids != null) {
    	itr = kids.iterator() ;
    	while (itr.hasNext()) {
    	  BeanPart child = (BeanPart)itr.next() ;
    	  child.setContainer(bp) ;    	      	      	  
    	  GenerateAChild(sb,bp,child,(RefStructuralFeature)itr.next()) ;    	  
      }
    }    
    sb.append(template.getPostfix()) ;
    
    return (sb.toString()) ;
        
}

public String toString() {
	return super.toString() + fMethodName;
}


}


