package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.codegen.util.* ;
import org.eclipse.jdt.internal.compiler.ast.* ;
import org.eclipse.jdt.internal.compiler.ast.* ;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.java.rules.* ;
import java.util.List ;
import org.eclipse.jdt.core.IMethod ;
import org.eclipse.jdt.core.ISourceRange ;

/**
 *  This Visitor will analyze a method or call and overrding Method Visitor
 */

public class MethodVisitor extends SourceVisitor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	CodeMethodRef fMethod = null ;
	

/**
 *  A bean is declared locally in this method.
 *  At this time, no deferentiation to the variable's scope.
 */	
protected void	processLocalDecleration (LocalDeclaration stmt) {
	IMethodVariableRule methodVarRule = (IMethodVariableRule) JavaPatternRuleFactory.getRule(IMethodVariableRule.RULE_ID) ;
	if (methodVarRule!=null && methodVarRule.ignoreVariable(stmt,fModel,fModel.getCompositionModel())) return ;
	
	BeanPart bp = new BeanPart(stmt) ;
	bp.setModel(fModel) ;
	bp.setInstanceVar(false) ;
	bp.addInitMethod(fMethod) ;
	fModel.addBean (bp) ;
	if (stmt.initialization instanceof AllocationExpression || stmt.initialization instanceof CastExpression || stmt.initialization instanceof MessageSend) {
	    // Decleration and initialization
	    new ExpressionVisitor(fMethod,(Statement)stmt,fModel,fReTryLater).visit();
	}		
}	
		
/**
 *   Drive processAStatement() for each element.
 */	
protected  void  processStatementArray (Statement[] statements) throws CodeGenException {
	if (statements == null) {
		// Typically a "new" allocation statement will initialize a method, so would a this.setFoo().  But
		// and empty Initialize() method need some help.
		IThisReferenceRule thisRule = (IThisReferenceRule) JavaPatternRuleFactory.getRule(IThisReferenceRule.RULE_ID) ;
        String [] mNameModifier = thisRule.getThisInitMethodName(fModel.getClassHierarchy()) ;
		if (fMethod.getMethodName().equals(mNameModifier[0])) {
			BeanPart bean = fModel.getABean(BeanPart.THIS_NAME) ;
			if (bean != null)
			   bean.addInitMethod(fMethod) ;
			
		}
		return ;
	}
	
      for (int i=0; i<statements.length; i++)
	   processAStatement(statements[i]) ;	
}		
		
/**
 *   Re-drive a statement block
 */	
protected  void  processBlockStatement(Block stmt) throws CodeGenException {	
	processStatementArray(stmt.statements) ;
} 

/**
 *   Re-drive a Sync block
 */	
protected  void  processSynchStatement(SynchronizedStatement stmt)  throws CodeGenException  {	
	processBlockStatement(stmt.block) ;
} 
/**
 *   Re-drive a Sync block
 */	
protected  void  processTryStatement(TryStatement stmt)  throws CodeGenException  {	
	if(stmt.tryBlock!=null)
		processBlockStatement(stmt.tryBlock) ;
	if(stmt.finallyBlock!=null)
		processBlockStatement(stmt.finallyBlock);
} 

 


		
/**
 *  Use the rule base engine to parse an IF statement
 */	
protected void	processIFStatement(IfStatement stmt) throws CodeGenException{
	IIfStatementRule ifRule = (IIfStatementRule) JavaPatternRuleFactory.getRule(IIfStatementRule.RULE_ID) ;
	if (ifRule == null) throw new CodeGenException("No IIfStatementRule") ; //$NON-NLS-1$
	
	ISourceVisitor overideIf = ifRule.overideIFVisit(fMethod.getDeclMethod(),stmt,fModel) ;
	if (overideIf != null) {
		overideIf.visit() ;
		return ;
	}
	
	
	int processPattern = ifRule.whichPartToProcess(fMethod.getDeclMethod(),stmt) ;
	if ((processPattern & IIfStatementRule.PROCESS_IF) > 0) 
         processAStatement(stmt.thenStatement) ;	
	
	if ((processPattern & IIfStatementRule.PROCESS_ELSE) > 0) 
         processAStatement(stmt.elseStatement) ;
		
}


/**
 *   Start pilling off the statements
 */	
protected void	processAStatement(Statement stmt) throws CodeGenException {
		
	// Local Variable Decleration
      if (stmt instanceof LocalDeclaration) 
          processLocalDecleration((LocalDeclaration)stmt) ;
      // Block Statement
      else if (stmt instanceof Block)
          processBlockStatement((Block)stmt) ;
      // Try Block
      else if (stmt instanceof TryStatement)
          processTryStatement ((TryStatement)stmt) ;
      // If Statement
      else if (stmt instanceof IfStatement) 
          processIFStatement((IfStatement)stmt) ;
      // Synchronized
      else if (stmt instanceof SynchronizedStatement) 
          processSynchStatement ((SynchronizedStatement)stmt) ;
      else if (stmt instanceof ReturnStatement)
          new ReturnStmtVisitor(fMethod,(ReturnStatement)stmt,fModel,fReTryLater).visit();

      // Handle an Expression          
      else if (stmt instanceof Expression) 
          new ExpressionVisitor(fMethod,(Expression)stmt,fModel,fReTryLater).visit();
      else
         JBCFPlugin.log ("Did not MethodVisitor.processAStatement : "+stmt, JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
}	
	
/**
 *
 */		
public MethodVisitor (AbstractMethodDeclaration node, IBeanDeclModel model,List reTryList,CodeTypeRef typeRef, String methodHandle, ISourceRange range, String content) {
	super(node,model,reTryList) ;	
	fMethod = new CodeMethodRef (node,typeRef,methodHandle,range,content) ;
}

/**
 * Go for it
 */
 
public void visit() {
	
	
	// Check to see if we have a special Method Visitor to use instead this one.
	IOverideMethodVisitRule overideRule = (IOverideMethodVisitRule) JavaPatternRuleFactory.getRule(IOverideMethodVisitRule.RULE_ID) ;
	if (overideRule != null) {
		ISourceVisitor overideVisitor = overideRule.overideMethodVisit(fMethod.getDeclMethod(),fModel) ;
		if (overideVisitor != null) {
		   overideVisitor.visit() ;
		   return ;
		}
	}	
	
	// A temporary limitation so that if one add local JFrame for example,
	// and set its content pane with an instance JPanel, we will loose it as
	// the JFrame is local and would not put in the FF
	if (fMethod.getDeclMethod() instanceof ConstructorDeclaration) {
		JBCFPlugin.log("Skiping Custructor parsing: "+fMethod.getMethodName()) ;
		return ;
	}	
	
	try {
	  processStatementArray(fMethod.getDeclMethod().statements) ;		
	}
	catch (CodeGenException  e) {
		// Will have to pass it on later on
		JBCFPlugin.log (e, JBCFMsgLogger.LOG_WARNING) ;
	}
}

}