package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.internal.compiler.ast.*;

import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.model.BeanDeclModel;
import com.ibm.etools.jbcf.codegen.model.BeanPart;
import com.ibm.etools.jbcf.codegen.util.*;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.codegen.util.ExpressionTemplate;

/**
 * @author gmendel
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class NoConstraintAddDecoderHelper extends AbstractContainerAddDecoderHelper {

	/**
	 * Constructor for NoConstraintAddDecoderHelper.
	 * @param bean
	 * @param exp
	 * @param fm
	 * @param owner
	 */
	
	
	
	public NoConstraintAddDecoderHelper(
		BeanPart bean,
		Statement exp,
		IJavaFeatureMapper fm,
		IExpressionDecoder owner) {
		super(bean, exp, fm, owner);
	}

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	String [][]  NoConstraintSF = {
		JMenuBarDecoder.structuralFeatures,             // JMenu Bar
		JMenuDecoder.structuralFeatures                 // JMenu
	} ;

	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#add(BeanPart, BeanPart, int)
	 */
protected RefObject add(BeanPart toAdd, BeanPart target, int index) {
	int i = index ;
    if (i<0) {
         i = findIndex(toAdd,target) ;
         // fAddedIndex may inforce an insert mathod, vs. and add method
         //  fAddedIndex = Integer.toString(i) ;
    }
 	toAdd.setContainer(target) ;
	target.addChild(toAdd) ;
      //  Make this relationship in the Composition Model
   
//   root.refSetValue((RefStructuralFeature) root.refMetaObject().metaObject(COMPONENT_ATTR_NAME),
//                    toAdd.getRefObject()) ;
   CodeGenUtil.refSetValue(target.getRefObject(), 
                           fFmapper.getFeature(null), 
                           toAdd.getRefObject(), i) ;
                    
   return toAdd.getRefObject() ;
}

	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#getComponent(RefObject)
	 */
	protected IJavaObjectInstance getComponent(RefObject root) {
		return (IJavaObjectInstance) root ;
	}

	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#parseAddedPart(MessageSend)
	 */
protected BeanPart parseAddedPart(MessageSend exp) throws CodeGenException {
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with multiple arguments, and nesting") ; //$NON-NLS-1$
	
	if (exp == null) return null ;
	
	BeanPart bp = null ;
	
	Expression[] args = exp.arguments ;
	if (args.length < 1) throw new CodeGenException("No Arguments !!! " + exp) ; //$NON-NLS-1$
      
      // Parse the arguments to figure out which bean to add to this container
      if (args[0] instanceof MessageSend)  {
      	// Look to see of if this method returns a Bean
      	String selector = new String (((MessageSend)args[0]).selector) ;  
      	bp = fOwner.getBeanModel().getBeanReturned(selector) ;       	
      }
      else if (args[0] instanceof SingleNameReference){
            // Simple reference to a bean
            String beanName = new String (((SingleNameReference)args[0]).token);
            bp = fOwner.getBeanModel().getABean(beanName) ;
            if(bp==null)
            	bp = fOwner.getBeanModel().getABean(BeanDeclModel.constructUniqueName(fOwner.getExprRef().getMethod(),beanName));
            	//bp = fOwner.getBeanModel().getABean(fOwner.getExprRef().getMethod().getMethodHandle()+"^"+beanName);
      }
	return bp ;
}

	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#parseAndAddArguments(Expression[])
	 */
protected boolean parseAndAddArguments(Expression[] args) throws CodeGenException {
	if (fAddedPart!= null) {
      
          int index = -1 ;
          if (args.length == 2 && args[1] instanceof IntLiteral) {
                fAddedIndex = ((IntLiteral)args[1]).toStringExpression() ;
                index = Integer.parseInt(fAddedIndex) ;
          }          
          
          RefObject root = add(fAddedPart, fbeanPart, index) ;
          return true ;
      }		
	return false ;
}

	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#isMySigniture()
	 */
protected boolean isMySigniture() {
   if (fbeanPart.getRefObject() == null) return false ;
   com.ibm.etools.cde.CDEHack.fixMe() ; // Need to make a generic no conttraint component feature
   for (int i = 0; i < NoConstraintSF.length; i++) {
	String[] elements = NoConstraintSF[i];
	for (int j = 0; j < elements.length; j++) {		
		RefStructuralFeature sf = (RefStructuralFeature) fbeanPart.getRefObject().refMetaObject().metaObject(elements[j]) ; 
        if (sf != null && fFmapper.getFeature(fExpr).equals(sf)) return true ;	
	}      
   }   
   return false ;
}
	
	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#getExpressionTemplate()
	 */
protected ExpressionTemplate getExpressionTemplate() throws CodeGenException {
			
    String[] args = getSourceCodeArgs() ;
		
	String sel = fbeanPart.getSimpleName() ;
	String mtd ;
	if (fAddedIndex != null)
	  mtd = fFmapper.getIndexMethodName() ;
	else
	  mtd = fFmapper.getMethodName() ;
	ExpressionTemplate exp = new ExpressionTemplate (sel,mtd, args,	                                                
	                                                 null,
	                                                 0 ) ;
	exp.setLineSeperator(fbeanPart.getModel().getLineSeperator()) ;	
	return exp ;   	
}

	/**
	 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#primRefreshArguments()
	 */
	protected void primRefreshArguments() {
			
	// indexValueFound will designate if to use and index APi or not
    if(indexValueFound || fAddedIndex!=null){ 	
    	// Add an index position, as it was found.
    	List parentsChildren = getRootComponentList() ; 
    	int indexOfAddedPart = parentsChildren.indexOf(getRootObject(false));
    	fAddedIndex = Integer.toString(indexOfAddedPart);	
    }
    else 
        fAddedIndex = null;
	
	RefObject root = getRootObject(false) ;	

	}


private String[] getSourceCodeArgs() {
	String AddedArg ;
	if (fAddedPart.getInitMethod().equals(fbeanPart.getInitMethod()))   // Added part is defined in the same method as the container
		 AddedArg = fAddedPart.getSimpleName() ;
    else	
	     AddedArg= fAddedPart.getReturnedMethod().getMethodName()+
	                  ExpressionTemplate.LPAREN+ExpressionTemplate.RPAREN ;   
	List finalArgs = new ArrayList();
		   
	finalArgs.add(AddedArg);
		   	
	if(fAddedIndex!=null){
		finalArgs.add(fAddedIndex);
	}
	String[] args = new String[finalArgs.size()];
	for(int i=0;i<finalArgs.size();i++)
		args[i] = (String)finalArgs.get(i);
    return args ;
}


	/**
	 * @see com.ibm.etools.jbcf.codegen.java.IExpressionDecoderHelper#primRefreshFromComposition(String)
	 */
public String primRefreshFromComposition(String expSig) throws CodeGenException {


      // Isolate the constraint argument using the last used constraint
    String[] args = getSourceCodeArgs() ;
          
	int start = expSig.indexOf(args[0]) ;	
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with identical ") ; //$NON-NLS-1$
	int end = start<0 ? -1 : 	          
	          expSig.lastIndexOf(")"); //$NON-NLS-1$
	if (start<0 || end<0) {
		JBCFPlugin.log ("NoConstraintAdDecoderHelper.primRefreshFromComposition(): Error", JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$
		return expSig ;
	}
	
	// Get the latest constraint		
	primRefreshArguments();
	
	args = getSourceCodeArgs() ;
	// Regenerate the arguments, only
	StringBuffer sb = new StringBuffer() ;
	for (int i=0; i<args.length; i++) {
	   if (i>0) sb.append(", ") ; //$NON-NLS-1$
	   sb.append(args[i]) ;
	}
	// Replace the arguments part	
	StringBuffer newExp = new StringBuffer(expSig) ;
	newExp.replace(start,end,sb.toString()) ;
	fExprSig = newExp.toString() ;
	return fExprSig ;

}


/**
 * Typically the expression adapter is stored on the root component,
 * given that we do not have a root component, we may have multiple adapters.
 */
public void UnadaptToCompositionModel() {
	com.ibm.etools.cde.CDEHack.fixMe("Need to clean this up") ; //$NON-NLS-1$


    if (fbeanPart.getRefObject() == null) {
    	com.ibm.etools.jbcf.JBCFPlugin.log("UnadaptToCompositionModel: No RefObject",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_WARNING) ;    	 //$NON-NLS-1$
    	return ;
    } 
    
    BeanDecoderAdapter ba = (BeanDecoderAdapter)fbeanPart.getRefObject().getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;		
	ba.removeChildAdapter(fAddedPart) ;
	RefObject root = getRootObject(true) ;
	
	if (root != null) {
	   Iterator itr = root.getAdapters().iterator() ;
	   while (itr.hasNext()) {
		Adapter a = (Adapter) itr.next();
		if (a instanceof ExpressionDecoderAdapter)
		   root.removeAdapter(a) ;
	   }
	}
}

		

/**
 * @see com.ibm.etools.jbcf.codegen.java.AbstractContainerAddDecoderHelper#getComponentArguments(RefObject)
 */
protected IJavaObjectInstance[] getComponentArguments(RefObject root) {
	return new IJavaObjectInstance[0];
}


}
