package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import org.eclipse.jdt.internal.compiler.ast.*;
import com.ibm.etools.jbcf.codegen.util.*;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ref.RefObject;


import java.util.*;
import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.codegen.*;
import com.ibm.etools.jbcf.* ;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance ;
import com.ibm.etools.emf.ref.Value; 



/**
 *  An Object Decoder will only work on Simple Attributes
 */
public class ObjectDecoder extends AbstractExpressionDecoder {
    
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	// SIMPLE_INSTANCES MUST be qualified with a period for inner classes. Do not use the "$" format.
    public static final String[]  SIMPLE_INSTANCES = new String[]  { "java.lang.String",  //$NON-NLS-1$
                                                                       "java.awt.Dimension",  //$NON-NLS-1$
                                                                       "java.awt.Rectangle" } ; //$NON-NLS-1$
    

public ObjectDecoder (CodeExpressionRef expr, IBeanDeclModel model, IDiagramModelInstance cm, BeanPart part) {
	super (expr,model,cm,part) ;
}

public ObjectDecoder() {
	super () ;
}


protected void initialFeatureMapper(){	
	   // Use a Simple Attribute Mapper
         fFeatureMapper = new AttributeFeatureMapper()  ;                      
}

protected void initialFeatureMapper(RefStructuralFeature sf) {
         fFeatureMapper = new AttributeFeatureMapper() ;
         fFeatureMapper.setFeature(sf) ;
}


protected void initialDecoderHelper() {
	// Bind an Attribute Mapper
    if (isChildValue(fFeatureMapper.getFeature(null), (IJavaObjectInstance)fbeanPart.getRefObject(), false))  {
        JBCFPlugin.log("ObjectDecoder using *Delegate Helper* for "+fFeatureMapper.getFeature(null), //$NON-NLS-1$
        JBCFMsgLogger.LOG_FINE) ;
        fhelper = new DelegateRelationshipDecoderHelper(fbeanPart, fExpr,  fFeatureMapper, this);
    }
    else
	   fhelper = new SimpleAttributeDecoderHelper(fbeanPart, fExpr,  fFeatureMapper,this) ;	
}


protected boolean isAnyAttributeSet(IJavaObjectInstance obj) {
    
    if (obj == null) return false ;    
	Iterator itr = obj.refCurrentValues(true).iterator() ;
	while (itr.hasNext()) {
		Value val = (Value) itr.next() ;
		if (MethodTextGenerator.isNotToBeCodedAttribute(val.getSF(),obj)) continue ;
		return true ;
	}
	return false ;    
}

protected boolean isSimpleObject(IJavaObjectInstance obj) {
    if (obj == null) return true ;
    String type = obj.getJavaType().getQualifiedName() ;
    for (int i = 0; i < SIMPLE_INSTANCES.length; i++) {
        if (type.equals(SIMPLE_INSTANCES[i])) return true ;
    }
    return false ;
}


/**
 * Determine if the the SF is a heavy weight or not
 * @param check if any attributes are set.  During decode time, attributes may not be set yet.
 */
protected boolean isChildValue(RefStructuralFeature sf, IJavaInstance val, boolean checkAttributes) {
    if (sf == null || val == null || !(val instanceof IJavaObjectInstance)) return false ;
    // Known SF to be ignored
    if (MethodTextGenerator.isNotToBeCodedAttribute(sf,(RefObject)val)) return false ;
    // Known Instances to be ignored
    if (isSimpleObject((IJavaObjectInstance)val)) return false ;
    if (checkAttributes)
      if (! isAnyAttributeSet((IJavaObjectInstance)val)) return false ;
    
    return true ;
}

/**
 * A simple attribute is a component that has no special settings on it.
 * If it does has settings on it (and not a child), we need to create a BeanPart and 
 * Treat it as a regular bean instance (e.g., setLayout(xxx) ; where xxx was initialized with
 * some attributes.
 */
protected void addNonSimpleAttribute (IJavaObjectInstance obj, Vector list) {
    
    if (obj == null) return  ;
    
	Iterator itr = obj.refCurrentValues(true).iterator() ;
	while (itr.hasNext()) {
		Value val = (Value) itr.next() ;
		if ( !(val.getValue() instanceof IJavaInstance) ||
             !isChildValue(val.getSF(),(IJavaInstance)val.getValue(),true))
		   continue ;
		   
		list.add(val.getValue()) ;
		list.add(val.getSF()) ;
	}
}


/**
 *  Get the first level descendents
 */
public Vector getChildren(IJavaObjectInstance component) {
	// Simple Object has not childrens
	Vector v = new Vector() ;
//  In the future we should support adding non-simple attribute (an instance of an attribute)
	addNonSimpleAttribute(component,v) ;
	return v ;
}



/**
 * @see com.ibm.etools.jbcf.codegen.java.AbstractExpressionDecoder#isPriorityCacheable()
 */
protected boolean isPriorityCacheable() {
	return true;
}

}


