package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.*;
import com.ibm.etools.jbcf.codegen.util.*;
import org.eclipse.jdt.internal.compiler.ast.*;
import java.util.List;
import com.ibm.etools.jbcf.codegen.* ;
import org.eclipse.jdt.core.Signature ;
import java.util.* ;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance ;
import com.ibm.etools.emf.ref.RefObject ;


public class PointDecoderHelper extends ExpressionDecoderHelper {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	int  fpointArgs[]=null ;		

public PointDecoderHelper(BeanPart bean, Statement exp,  IJavaFeatureMapper fm, IExpressionDecoder owner){
	super (bean,exp,fm,owner) ;
}


/**
 *  Add a new constraint
 */
protected boolean	addPointArg() throws CodeGenException {
	if (fbeanPart.getRefObject() == null || fFmapper.getMethodName() == null) throw new CodeGenException ("null RefObject:"+fExpr) ;       //$NON-NLS-1$
		
	try {
		if (((MessageSend) fExpr).arguments.length == 2) {
			fpointArgs = new int[2];
			for (int i = 0; i < 2; i++)
				fpointArgs[i] = Integer.parseInt(((MessageSend) fExpr).arguments[i].toString());

            IJavaObjectInstance value = (IJavaObjectInstance) CodeGenUtil.createInstance("java.awt.Point",fOwner.getCompositionModel()) ;	 //$NON-NLS-1$
            value.setInitializationString("new java.awt.Point("+Integer.toString(fpointArgs[0])+    //$NON-NLS-1$
                                                            ","+Integer.toString(fpointArgs[1])+")") ;                                 //$NON-NLS-1$ //$NON-NLS-2$
            fbeanPart.getRefObject().refSetValue(fFmapper.getFeature(null),value)   ;  
		}
		else {
			JBCFPlugin.log("PointDecoderHelper: Size != 2 arguments", JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$
			return false;
		}

	}
	catch (Exception e) {
		throw new CodeGenException(e);
	}
	return true;
}


/**
 *  Get the args part of the expression
 */
protected String getArgs(String exp) {
  int[] indexes = getArgsIndexes(exp) ;
  if (indexes == null) return null ;
  return exp.substring(indexes[0],indexes[1]+1) ;
}
/**
 *  Get the args part of the expression
 */
protected int[] getArgsIndexes(String exp) {
	
  int left  = exp.indexOf(Signature.C_PARAM_START) ;
  int right = exp.indexOf(Signature.C_PARAM_END) ;
  
  if (left < 0 || right < 0) return null ;  	
  left ++ ; right -- ;
  if (left>right) return null ;
  int[] indexes = new int[2] ;
  indexes[0] = left ;
  indexes[1] = right ;
  return indexes ;
}

/**
 *  Get the dimension of the constraint from the arguments
 */
protected int[] parseArgs(String exp) {
	
	if (exp == null || getArgs(exp)==null) return null ;		
	StringTokenizer st = new StringTokenizer(getArgs(exp),",") ; //$NON-NLS-1$
	int args[] = new int[st.countTokens()] ;
	for (int i=0; i<args.length; i++) {
		args[i] = Integer.parseInt(st.nextToken().trim()) ;
	}
	return args ;
}

/**
 *  Compare the arrays
 */
protected boolean sameArgs(int[] A, int[] B) {
	if (A.length != B.length) return false ;
	for (int i=0; i<A.length; i++)
	   if (A[i] != B[i]) return false ;
	return true ;
}


/**
 *  Get the current constraints in the Composition Model
 */
protected int[]   getCompositionArgs() throws CodeGenException {

  IJavaObjectInstance curValue = (IJavaObjectInstance)fbeanPart.
                                 getRefObject().refValue(fFmapper.getFeature(fExpr)) ;  
  return parseArgs(curValue.getInitializationString()) ;	
} 
/**
 *  Create initialization arguments
 */
protected String argsToString(int[] constraints) {
	StringBuffer sb = new StringBuffer() ;
      for (int i=0; i<constraints.length; i++) {
     	  if (i>0) sb.append(", ") ; //$NON-NLS-1$
     	  sb.append(Integer.toString(constraints[i])) ;
      }
      return sb.toString() ; 
}

public String primRefreshFromComposition(String expSig) throws CodeGenException {
	  
  int [] curConstraints = getCompositionArgs() ;
  if (fpointArgs != null)
    if (sameArgs(curConstraints,fpointArgs)) {
  	fExprSig = expSig ; // spacing and such may have changed.
    }
    else {
      if (curConstraints.length != fpointArgs.length) {
      	fExprSig = generate(null) ;
      	// Remove the generated comments etc.
      	fExprSig = fExprSig.substring(0,fExprSig.indexOf(")")+1) ; //$NON-NLS-1$
      }      
      else {  	     
        int[] curArgsIndexes = getArgsIndexes(expSig) ;
        String newArgs = argsToString(curConstraints) ;     
        fpointArgs = curConstraints ;
        // Replace the arguments part with the current constraints
        fExprSig = expSig.substring(0,curArgsIndexes[0]) +
                newArgs +
                expSig.substring(curArgsIndexes[1]+1,expSig.length()) ;
      }
    }
  
     	
   return fExprSig ;	
}

public boolean primIsDeleted() {
  // If a constraint was changed to non-free form, remove the code
  return (fbeanPart.getRefObject().refValue(fFmapper.getFeature(fExpr)) == null);
}

/**
 *   Go for it
 */
public boolean decode() throws CodeGenException {

      return addPointArg() ;
}

public void delete() {
	UnadaptToCompositionModel() ;
	
	com.ibm.etools.emf.ref.RefStructuralFeature sf = fFmapper.getFeature(fExpr) ;
	IJavaObjectInstance parent = (IJavaObjectInstance)fbeanPart.getRefObject() ;
	parent.refUnsetValue(sf) ; 
}


/**
 *  Overide the default to deal with the lack of Decorator
 */
protected ExpressionTemplate getExpressionTemplate(String[] args) throws CodeGenException {
	
	if (fFmapper.getDecorator() != null) 
	   return super.getExpressionTemplate(args) ;
	   
      String mtd  ;
      if (fpointArgs.length == 2)
        mtd = IJavaFeatureMapper.LOCATION_NAME ;
      else
        throw new CodeGenException("Can not resolve write method") ;                 //$NON-NLS-1$
	String sel = fbeanPart.getSimpleName() ;
	ExpressionTemplate exp = new ExpressionTemplate (sel,mtd, 
	                                                 args,
	                                                 null,
	                                                 0 ) ;
	exp.setLineSeperator(fbeanPart.getModel().getLineSeperator()) ;
	return exp ;   	
}

public Object getPriorityOfExpression(){
	Integer pri = (Integer)super.getPriorityOfExpression();
	return new Integer(IJavaFeatureMapper.PRIORITY_DEFAULT + pri.intValue());
}

/**
 *  Generate from scratch
 */
public String generate(Object[] noArgs) throws CodeGenException {
	
	if (fFmapper.getFeature(null) == null) 
	    throw new CodeGenException ("null Feature") ; //$NON-NLS-1$
	try {
	   fpointArgs = getCompositionArgs() ;
	}
	catch (CodeGenException e) {
		return null ; 
	}
		
	ExpressionTemplate exp = getExpressionTemplate(new String[] {argsToString(fpointArgs)}) ;
      fExprSig = exp.toString() ;	                                                 
      return fExprSig ;	     	
}



public boolean isImplicit(Object args[]) {
   return false ;		
}

public Object[] getArgsHandles(Statement expr) {
   // No unique arguments ;
   return null ;
}


}


