package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;

import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.compiler.ast.*;

import com.ibm.etools.beaninfo.PropertyDecorator;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.Value;
import com.ibm.etools.java.JavaDataType;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.jbcf.codegen.model.BeanPart;
import com.ibm.etools.jbcf.codegen.util.*;

public class SimpleAttributeDecoderHelper extends ExpressionDecoderHelper {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String NULL_STRING = "null"; //$NON-NLS-1$

	String fInitString = null; // last value known in MOF
    String fUnresolveInitString = null ;
	IJavaInstance fPropInstance = null;

	public SimpleAttributeDecoderHelper(
		BeanPart bean,
		Statement exp,
		IJavaFeatureMapper fm,
		IExpressionDecoder owner) {
		super(bean, exp, fm, owner);
	}

	/**
	 *  Figure out what is the initialization string for an attribute
	 */
	protected String getInitString(Literal arg, RefStructuralFeature sf) {
		if (arg instanceof NullLiteral)
			return null;
		else if (arg instanceof StringLiteral) {
			String val = new String(arg.source());
			return (BeanUtilities.createStringInitString(val));
		} else {
			com.ibm.etools.cde.CDEHack.fixMe("need more thinking"); //$NON-NLS-1$
			return new String(arg.source());
		}
	}

	public Object getPriorityOfExpression() {
		Integer pri = (Integer) super.getPriorityOfExpression();
		if(fFmapper!=null){
			String methodName = null;
			if (fFmapper.getDecorator() != null)
				methodName = fFmapper.getDecorator().getWriteMethod().getName();
			if (methodName == null)
				methodName = CodeGenUtil.getWriteMethod(fExpr);
			if(methodName!=null)
				pri = new Integer(pri.intValue()+fFmapper.getPriorityIncrement(methodName));
		}
		return pri;
	}

	protected boolean isMethodType(String writeMethodName) {
		String method = null;
		if (fFmapper != null)
			if (fFmapper.getDecorator() != null)
				method = fFmapper.getDecorator().getWriteMethod().getName();
		if (method == null)
			method = CodeGenUtil.getWriteMethod(fExpr);
		return method != null && method.equals(writeMethodName);
	}

	/**
	 *  Figure out what is the initialization string for an attribute
	 */
	protected String getInitString(QualifiedNameReference arg) {
		String initString = CodeGenUtil.tokensToString(arg.tokens);
		try{
			if(fbeanPart!=null && fbeanPart.getModel()!=null && 
			   fbeanPart.getModel().getCompilationUnit()!=null && 
			   fbeanPart.getModel().getCompilationUnit().getTypes().length>0){
				//String resType = CodeGenUtil.resolveTypeComplex(fbeanPart.getModel().getCompilationUnit().getTypes()[0], initString);
				String resType = fbeanPart.getModel().resolve(initString);
				if(resType!=null)	{
                    fUnresolveInitString = initString ;
					initString = resType;
                }                    
			}
		}catch(JavaModelException e){
			JBCFPlugin.log(e);
		}
		return initString;
	}
	
	protected String getInitString(MessageSend msg){
		String initstring = msg.toString();
		if (msg.receiver instanceof QualifiedNameReference ||
			msg.receiver instanceof SingleNameReference){
			String unResolved = msg.receiver.toString();
			if(fbeanPart!=null && fbeanPart.getModel()!=null){
				String resolved = fbeanPart.getModel().resolve(unResolved);
				if(!resolved.equals(unResolved)){
					int from = initstring.indexOf(unResolved);
					int to = from+unResolved.length();
					if(from>-1 && from<initstring.length() && 
					   to>-1 && to<initstring.length() && from<=to){
					   	String newInitstring = initstring.substring(0,from)+resolved+initstring.substring(to,initstring.length());
						initstring = newInitstring;
					}
				}
			}
		}
		return initstring;
	}
	
	/**
	 *  Figure out what is the initialization string for an attribute
	 */
	protected String getInitString(AllocationExpression arg) {
		String initString = arg.toString();
		fUnresolveInitString = arg.toString();
		try{
			if(fbeanPart!=null && fbeanPart.getModel()!=null && 
			   fbeanPart.getModel().getCompilationUnit()!=null && 
			   fbeanPart.getModel().getCompilationUnit().getTypes().length>0){
			   	String type = CodeGenUtil.tokensToString(arg.type.getTypeName());
			   	String resolvedType = null;
		   		//String rt = CodeGenUtil.resolveTypeComplex(fbeanPart.getModel().getCompilationUnit().getTypes()[0], type);
		   		String rt = fbeanPart.getModel().resolve(type);
		   		if(rt != null)
		   			resolvedType = rt;
		   		else
		   			resolvedType = new String(type);
				StringBuffer initConstruction = new StringBuffer("new "); //$NON-NLS-1$
				initConstruction.append(resolvedType);
				initConstruction.append("("); //$NON-NLS-1$
				for(int i=0;arg.arguments!=null && i<arg.arguments.length;i++){
					initConstruction.append(arg.arguments[i].toStringExpression());
					if(i!=arg.arguments.length-1)
						initConstruction.append(", "); //$NON-NLS-1$
				}
				initConstruction.append(")"); //$NON-NLS-1$
				initString = initConstruction.toString();
			}
		}catch(JavaModelException e){
			JBCFPlugin.log(e);
		}
		return initString;
	}
	
	protected String getInitString(SingleNameReference snr){
		int loc = snr.sourceStart;
		if(fOwner.getExprRef()!=null && fOwner.getExprRef().getMethod()!=null)
			loc = fOwner.getExprRef().getOffset()+fOwner.getExprRef().getMethod().getOffset();
		return fOwner.getBeanModel().resolveSingleNameReference(snr.toString(), loc);
	}
	/**
	 * 
	 */
	protected String parseInitString(Expression exp) {
		String initString = null;
		try{
		    if (exp instanceof Literal)
    			initString = getInitString((Literal)exp, fFmapper.getFeature(fExpr));
    		else if (exp instanceof QualifiedNameReference)
    			initString = getInitString((QualifiedNameReference)exp);
    		else if(exp instanceof AllocationExpression)
    			initString = getInitString((AllocationExpression)exp);
    		else if(exp instanceof MessageSend)
    			initString = getInitString((MessageSend)exp);
    		else if (exp instanceof SingleNameReference)
    			initString = getInitString((SingleNameReference)exp);
    		else{
    			com.ibm.etools.cde.CDEHack.fixMe("Will do for now?????"); //$NON-NLS-1$
    			initString = exp.toString();
    		}
		}catch(Exception e){
			JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
			initString = exp.toString();
		}
		return initString;
    }
    
    protected boolean dealwithInternalBean(AllocationExpression exp) {
        
        try {
          String unresolved = CodeGenUtil.tokensToString(exp.type.getTypeName());
          String resolved = fbeanPart.getModel().resolve(unresolved) ;
          
          String origInitString = exp.toString() ;
          int start = origInitString.indexOf(unresolved);
          int end = start + unresolved.length();
          String newInitString = origInitString.substring(0, start)+resolved+origInitString.substring(end, origInitString.length());
          RefStructuralFeature sf = (RefStructuralFeature) fFmapper.getFeature(fExpr) ;
        
          IJavaObjectInstance attr = (IJavaObjectInstance)CodeGenUtil.createInstance(resolved, fbeanPart.getModel().getCompositionModel()) ;
          attr.setInitializationString(newInitString) ;
          fbeanPart.getRefObject().refSetValue(sf,attr) ; 
          fInitString = parseInitString(exp) ;       
        }
        catch (CodeGenException e) {
            return false ;
        }
        
        return true ;
    }

	/**
	 *  Add a new Simple Structured Feature to a target object instance
	 *  e.g., setFoo("boo") ;
	 */
	protected boolean addFeature() throws CodeGenException {

		if (fbeanPart.getRefObject() == null)
			throw new CodeGenException("null RefObject:" + fExpr); //$NON-NLS-1$

		

        EClassifier argType = null ;
                
        if (fFmapper.isFieldFeature()) {
            RefStructuralFeature sf = fFmapper.getFeature(fExpr) ;
            if (sf == null) throw new CodeGenException("Invalid SF"); //$NON-NLS-1$
            argType = (EClassifier) sf.refType() ;   
            fInitString = parseInitString(((Assignment)fExpr).expression) ;
        }
        else {
            // Regular setter Method
            PropertyDecorator pd = fFmapper.getDecorator();
		    if (pd == null) 
		       throw new CodeGenException("Invalid PropertyDecorator"); //$NON-NLS-1$
			argType =  pd.getPropertyType();            
			com.ibm.etools.cde.CDEHack.fixMe("We really need to parse the argument for the true value"); //$NON-NLS-1$
		    //As the PD type may be an abstract or interface !!!!!!!" 
		    Expression[] argExpr = ((MessageSend)fExpr).arguments;
		    if (argExpr.length != 1) throw new CodeGenException("Expression has more than one argument"); //$NON-NLS-1$
            if (argExpr[0] instanceof AllocationExpression) 
               return dealwithInternalBean((AllocationExpression)argExpr[0]) ;
    		// Determine the value of the attribute
    		fInitString = parseInitString(argExpr[0]) ;		    
        }
        		

		// Create an attribute instance
		if (argType instanceof EClass) {
			RefFactory fact =
				((EPackage) ((EClass) argType).getEObjectContainer()).getFactory();
			fPropInstance = (IJavaInstance) fact.create(argType);
		} else if (
			argType instanceof JavaDataType && ((JavaDataType) argType).isPrimitive()) {
			fPropInstance =
				(IJavaInstance) (((JavaDataType) argType).createFromString(fInitString));
		} else {
			JBCFPlugin.log(
				"SimpleAttributeDecoderHelper.addFeature() : Not a Class or a DataType: " //$NON-NLS-1$
					+ argType, JBCFMsgLogger.LOG_FINE);
			return false;
		}

		// Add the attribute
		CodeGenUtil.addAttribute(
			fbeanPart.getRefObject(),
			(RefStructuralFeature) fFmapper.getFeature(fExpr),
			fPropInstance,
			fInitString);

		if (fInitString == null)
			fInitString = NULL_STRING;
		return true;
	}

	/**
	 *  Get the initialization string currently held in the composition model
	 */
	protected String primGetInitString() {

		Object currentVal =
			fbeanPart.getRefObject().refValue(fFmapper.getFeature(fExpr));
		fPropInstance = (IJavaInstance) currentVal;
		if (currentVal != null) {
			if (currentVal instanceof IJavaObjectInstance)
				return ((IJavaObjectInstance) currentVal).getInitializationString();
			else if (currentVal instanceof JavaDataTypeInstance)
				return ((JavaDataTypeInstance) currentVal).getInitializationString();

		} else { // Is it a null value ??
			Iterator itr = fbeanPart.getRefObject().refCurrentValues(true).iterator();
			com.ibm.etools.cde.CDEHack.fixMe();
			while (itr.hasNext()) {
				Value val = (Value) itr.next();
				if (val.getSF().equals(fFmapper.getFeature(null)))
					return NULL_STRING;
			}
		}
		return null;
	}

	/**
	 *   Check to see if this attribute is still in the composition model
	 */
	public boolean primIsDeleted() {
		return primGetInitString() == null;
	}

	/**
	 *  Reflect the current initialization string from the composition model
	 */
	public String primRefreshFromComposition(String expSig)
		throws CodeGenException {
		if (fExprSig == null)
			fExprSig = expSig;
		com.ibm.etools.cde.CDEHack.fixMe(
			"need to deal with un initialized init string"); //$NON-NLS-1$
		// Isolate the initialization string int he current expression
		int[] positions =
			CodeSnippetTranslator.indexOfIgnoringSpace(fExprSig, "("+fInitString+")");         //$NON-NLS-1$ //$NON-NLS-2$
		if (fUnresolveInitString!=null && (positions[0] < 0 || positions[1] < 0)) {
            positions = CodeSnippetTranslator.indexOfIgnoringSpace(fExprSig, "("+fUnresolveInitString+")"); //$NON-NLS-1$ //$NON-NLS-2$
        
            if (positions[0] < 0 || positions[1] < 0)
            {
            	// Could not match delta for the existing expression
            	// At this time will not preserve existing
    			JBCFPlugin.log(
    				"SimpleAttr.DecoderHelper.primRefreshFromComposition(): Error", JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$
    			return generate(null);
    		}
    		positions[0]++ ;
    		positions[1]-- ;
        }
        else {
        	if (positions[0] < 0 || positions[1] < 0)  {        		
    			JBCFPlugin.log(
    				"SimpleAttr.DecoderHelper.primRefreshFromComposition(): Error", JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$
    			return generate(null);
        		
        	}
            positions[0]++ ;
            positions[1]-- ;
        }
		fInitString = primGetInitString();
		StringBuffer sb = new StringBuffer(expSig);
		sb.replace(positions[0], positions[1], fInitString);
		fExprSig = sb.toString();
		return fExprSig;
	}

	public boolean decode() throws CodeGenException {

		if (fFmapper.getFeature(fExpr) == null || fExpr == null)
			throw new CodeGenException("null Feature:" + fExpr); //$NON-NLS-1$

		if ((fExpr instanceof Assignment) ||
		    ((fExpr instanceof MessageSend) && ((MessageSend)fExpr).arguments.length == 1))
			return (addFeature());
		else
			return (false);

	}

	public void delete() {
		UnadaptToCompositionModel();

		com.ibm.etools.emf.ref.RefStructuralFeature sf = fFmapper.getFeature(fExpr);
		IJavaObjectInstance parent = (IJavaObjectInstance) fbeanPart.getRefObject();
		parent.refUnsetValue(sf);
	}

	public String generate(Object[] noArgs) throws CodeGenException {
		if (fFmapper.getFeature(null) == null)
			throw new CodeGenException("null Feature"); //$NON-NLS-1$
		fInitString = primGetInitString();
		if (fInitString == null)
			fInitString = NULL_STRING; // null value
		com.ibm.etools.cde.CDEHack.fixMe("should cache this in the Feature Mapper"); //$NON-NLS-1$
		String mtd = fFmapper.getMethodName();
        if (mtd == null) {
            JBCFPlugin.log("No Write Method found for "+fFmapper.getFeature(null)+" on "+fbeanPart.getUniqueName(),JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$ //$NON-NLS-2$
            return null ;
        }
		String sel = fbeanPart.getSimpleName();
		ExpressionTemplate exp =
			new ExpressionTemplate(
				sel,
				mtd,
				new String[] { fInitString },
				null,
				0);
		exp.setLineSeperator(fbeanPart.getModel().getLineSeperator());
		if (fFmapper.isFieldFeature())
		  exp.setFieldAccess(true) ;
		fExprSig = exp.toString();
		return fExprSig;

	}

	public boolean isImplicit(Object args[]) {
		// refresh fPropInstanc
		primGetInitString();
		if (fPropInstance instanceof IJavaObjectInstance)
			return ((IJavaObjectInstance) fPropInstance).isImplicit();
		else
			return false;
	}
	
	public Object[] getArgsHandles(Statement expr) {
      // No unique arguments ;
      return null ;
    }
	

}