package com.ibm.etools.jbcf.codegen.java;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jdt.internal.compiler.ast.* ;
import org.eclipse.jdt.internal.compiler.ast.AstNode ;
import com.ibm.etools.jbcf.codegen.model.* ;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.java.rules.* ;
import java.util.* ;
import org.eclipse.jdt.core.* ;
import com.ibm.etools.jbcf.codegen.util.* ;

public class TypeVisitor extends SourceVisitor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	CodeTypeRef fType = null ;
	
	char[] content = null;
	String[] methodHandles = null;
	
	boolean forceJDOMUsage = false;
	
public TypeVisitor (TypeDeclaration node, IBeanDeclModel model, char[] content, String[] methodHandles, List reTryList, boolean forceJDOMUsage) {
	this(node,model,reTryList,forceJDOMUsage) ;	
	this.content = content;
	this.methodHandles = methodHandles;
}

public TypeVisitor (TypeDeclaration node, IBeanDeclModel model,List reTryList, boolean forceJDOMUsage) {
	super(node,model,reTryList) ;	
	this.forceJDOMUsage = forceJDOMUsage;
	fType = new CodeTypeRef (node,model) ;
	com.ibm.etools.cde.CDEHack.fixMe("Consider Multiple Types in a CU") ; //$NON-NLS-1$
	model.setTypeRef(fType) ;
}

/**
 *  Build a Method array which corresponds to the parsed methods array
 *  @return  Array of IMethods
 */
protected  Object[]  getCUMethods(AbstractMethodDeclaration aMethods[]) {

	com.ibm.etools.cde.CDEHack.fixMe("Assumes order is the same") ; //$NON-NLS-1$
	Vector   methods = new Vector() ;
	IMethod[] elements = CodeGenUtil.getMethods(fModel.getCompilationUnit()) ;
	if (elements == null || elements.length == 0) return null ;
	// Pad the array with these 
	for (int i=0; i<aMethods.length; i++) {
		String Name = new String (aMethods[i].selector) ;
		int j,Prev = 0 ;
		// Deal with duplicate names
		int dupCount = 0 ;
		for (i=0; i<methods.size(); i++) {
		    if (methods.get(i)!= null && ((IMethod)methods.get(i)).getElementName().equals(Name))
		      dupCount++ ;
		}
		for (j=Prev; j<elements.length; j++) {
			if (Name.equals(elements[j].getElementName())) {
			    dupCount-- ;
			    if (dupCount<0)
			      break ;
			}
		}
		if (j>=elements.length) {
			// Method declared implicitly (e.g., empty constructor) ;
			methods.add(null) ;
		}
		else {
		    methods.add(elements[j]) ;				
		    Prev=j+1 ;
		}
	}
	
	
	return methods.toArray() ;	
}


protected void createThisIfNecessary() {
    
        
    IThisReferenceRule thisRule = (IThisReferenceRule) JavaPatternRuleFactory.getRule(IThisReferenceRule.RULE_ID) ;
    String typeName = fModel.resolveThis() ;
    String superName = null ;
    if (fType.getTypeDecl().superclass != null)
      superName = fModel.resolve(CodeGenUtil.tokensToString(fType.getTypeDecl().superclass.getTypeName())) ;
    ResourceSet rs = fModel.getCompositionModel().getModelResourceSet() ;
    // The rule uses MOF reflection to introspect attributes : this works when the file is saved at this point.
    // So, try the super first    
    if ((superName!= null && (thisRule.useInheritance(superName,rs)) || thisRule.useInheritance(typeName,rs))) {
        BeanPartGenerator bpg = new BeanPartGenerator(fType.getBeanModel(),null) ;
        // No Init method yet.
	   	bpg.createThisBeanPartIfNeeded(null) ;	 
    }
}

public void visit()  {
	// First look a instance variables 
	FieldDeclaration[] fields = fType.getTypeDecl().fields;
	IInstanceVariableRule instVarRule = (IInstanceVariableRule) JavaPatternRuleFactory.getRule(IInstanceVariableRule.RULE_ID) ;
	if (fields != null) 
		for (int i=0 ; i<fields.length; i++) {
			// Should we skip this field ??
			if (instVarRule!=null && instVarRule.ignoreVariable(fields[i],fModel,fModel.getCompositionModel())) continue ;
				fModel.addBean(new BeanPart(fields[i])) ;
		}
  
     
	createThisIfNecessary() ;

	com.ibm.etools.cde.CDEHack.fixMe("Need to validate the method's order is the same"); //$NON-NLS-1$
	AbstractMethodDeclaration[] methods = fType.getTypeDecl().methods;
	if (forceJDOMUsage) {
		// No compilation unit... depend on jdom solely
		if(methods==null || methods.length==0)
			return;
		try{
			int methodHandleUseCount = 0;
			for(int i=0; i<methods.length; i++){
				if(	(methods[i]!=null) &&
					(methods[i] instanceof MethodDeclaration || 
					 methods[i] instanceof ConstructorDeclaration)){
					String thisMethodHandle = ""; //$NON-NLS-1$
					if (methods[i] instanceof MethodDeclaration) {
						thisMethodHandle = methodHandles[methodHandleUseCount];
						methodHandleUseCount++;						
					}
com.ibm.etools.cde.CDEHack.fixMe("Need to move the updateMethodOffset to this visitor, so that the logic is encapsulated") ;//$NON-NLS-1$
   					
					// Please see CodeSnippetModelBuilder.updateMethodOffsets() to see how
					// the declaration Source starts, and the declaration source ends are 
					// being modified. This is due to inconsistency between JDOM and JDT.
					new MethodVisitor( methods[i], fModel, fReTryLater, fType, thisMethodHandle, // null,
								 getSourceRange(methods[i].declarationSourceStart,methods[i].declarationSourceEnd), 
								 new String(content).substring(methods[i].declarationSourceStart, methods[i].declarationSourceEnd)).
								 visit();
				}
			}
		}catch(Exception e){
			JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
		}
	}else{
		Object cuMethods[] = getCUMethods(methods);
		if(cuMethods==null || cuMethods.length<1)
			return;
		// Compilation unit methods and jdom methods should match.
		if (cuMethods.length != methods.length) 
			throw new RuntimeException("methods length error") ; //$NON-NLS-1$
		int i=0 ;
		try {
			for (; i < methods.length ; i++){
				// Visit each method with the correct visitor
				if ( cuMethods[i] != null && 
					(methods[i] instanceof MethodDeclaration ||
					 methods[i] instanceof ConstructorDeclaration)) {
					new MethodVisitor(methods[i],fModel,fReTryLater,fType,((IMethod)cuMethods[i]).getHandleIdentifier(),
								((IMethod)cuMethods[i]).getSourceRange(),
								((IMethod)cuMethods[i]).getSource()).visit();
				}
			}
		}catch (JavaModelException e) {
			JBCFPlugin.log ("TypeVisitor.visit() could not visit"+String.valueOf(methods[i].selector)+" : "+e.getMessage(), JBCFMsgLogger.LOG_WARNING) ; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
}

protected ISourceRange getSourceRange(int start, int end){
	final int offset = start;
	final int length = end-start+1;
	return new ISourceRange(){
		public int getOffset(){return offset;}
		public int getLength(){return length;}
	};
}

/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	
	return super.toString();
}

}