package com.ibm.etools.jbcf.codegen.java.rules;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefObject ;
import com.ibm.etools.emf.resource.ResourceSet;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.jdt.core.* ;

import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance ;
import com.ibm.etools.emf.ref.RefStructuralFeature ;
import com.ibm.etools.jbcf.codegen.* ;
import com.ibm.etools.jbcf.codegen.util.CodeGenUtil;
import com.ibm.etools.jbcf.vce.VCEPlugin;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.emf.ecore.EClassifier ;

import java.util.* ;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.properties.*;
import com.ibm.etools.cde.properties.NameInCompositionPropertyDescriptor;
import com.ibm.etools.cdm.* ;
import com.ibm.etools.cdm.Annotation;
import com.ibm.etools.cdm.KeyedValue;

public class InstanceVariableCreationRule implements IInstanceVariableCreationRule {
    
    
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
    // The following types will not use the ivj Prefix, and will not use a declaration Method.
    public static final String[]  internalTypes = { "java.awt.GridBagConstraints", //$NON-NLS-1$
                                                       "java.awt.LayoutManager", //$NON-NLS-1$
                                                       "javax.swing.text.Caret" } ; //$NON-NLS-1$
    public static  ArrayList		internalHelpers = null ;      
    private static ResourceSet    fRS = null ;                                                 
    public static final String[]  internalPrefix = { "cons", //$NON-NLS-1$
          												"lay", //$NON-NLS-1$
          												"caret" } ; //$NON-NLS-1$
    public static int             internalIndex=1 ;
    public static Preferences 	fPrefStore = null ;
    public static JavaHelpers      fComponentMeta = null ;

 

private  static List  getMetaTypes(ResourceSet rs) {    
        
    if (fRS != null && fRS.equals(rs)) return internalHelpers ;
        
    ArrayList a = new ArrayList() ;
    com.ibm.etools.jbcf.JBCFPlugin.log("InstanceVariableCreationRule: loading cache",com.ibm.etools.jbcf.JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
    for (int i = 0; i < internalTypes.length; i++) {
        JavaHelpers sType = JavaClassImpl.reflect (internalTypes[i],rs) ;
        if (sType == null) throw new IllegalArgumentException("Invalid Type") ; //$NON-NLS-1$
        a.add(sType) ; 
    }          
    internalHelpers = a ;
    
    fComponentMeta = JavaClassImpl.reflect("java.awt.Component",rs) ; //$NON-NLS-1$
    
    fRS = rs ;
    return internalHelpers ;    
}
	
/**
 * It the type is/extends any element in internalTypes, than it should be an internally
 * defined class. If the parent of the input is the BeanSubclassComposition, then it 
 * should not be internal. 
 */
private boolean isInternalType(RefObject obj) {
    
	String clazzName = null ;
	EClassifier  meta = (EClassifier) ((IJavaInstance)obj).getJavaType() ;
	ResourceSet rs = obj.getResourceSet() ;
	boolean result = false;
	for (int i=0; i<internalTypes.length; i++) 
	    	if (((JavaHelpers)(getMetaTypes(rs).get(i))).isAssignableFrom(meta)) {	    		
	           result = true ;
	    	}   
	if(result){
		RefBaseObject parent = obj.refContainer();
		String containerName = parent.refMetaObject().refID();
		if("BeanSubclassComposition".equals(containerName)) //$NON-NLS-1$
			result = false;
	}      
    return result ;
    
}


public static boolean usePrefix(EClassifier meta, ResourceSet rs) {
   if (rs == null || meta == null) return true ;	
   return (!getComponentMeta(rs).isAssignableFrom((EClassifier)meta)) ;
}


/**
 * Get the default prefix name for a given object type.
 */
public static String getPrefix(EClassifier meta, ResourceSet rs) {
    
    String clazzName = null ;

	for (int i=0; i<internalTypes.length; i++) 
	    	if (((JavaHelpers)(getMetaTypes(rs).get(i))).isAssignableFrom(meta)) {	    		
	           return internalPrefix[i]  ;
	    	}
	                     
	if (usePrefix(meta,rs))        
      return DEFAULT_VAR_PREFIX ;
    else
      return "" ; //$NON-NLS-1$
}

public static String addPrefix (String pre, String name) {
	if (pre == null || pre.length() == 0) return name ;
	StringBuffer sb = new StringBuffer(pre) ;
	sb.append(Character.toUpperCase(name.charAt(0))) ;
	sb.append (name.substring(1)) ;
	return sb.toString() ;
}

public static String lowCaseFirstCharacter(String name) {
	if (name == null || name.length() == 0) return null ;
	if (Character.isUpperCase(name.charAt(0))) {
	    char[] a = name.toCharArray() ;
	    a[0] = Character.toLowerCase(a[0]) ;
	    return new String(a) ;
	}
	else return name ;
}

/**
 *  
 *  
 */
public String getInstanceVariableName (RefObject obj, IType currentType, IDiagramModelInstance cm){
	
	String name = null ;
	KeyedValue kv = null  ;
	boolean   updateNameInComposition = false ;

    // Try to get its annotated value 
    com.ibm.etools.cde.CDEHack.fixMe() ;	
    Annotation a = CodeGenUtil.getAnnotation(obj) ;
    if (a != null) {
          kv = a.getKeyedValue(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY);
          if (kv != null)
            name = ((KeyedString) kv).getValue();
    }
            

    if (name == null) {
     try {	
  	  // Try to get the name attribute, first
  	  // Do not lowCase first letter !!!
	  RefStructuralFeature sf = (RefStructuralFeature) obj.refMetaObject().metaObject("name") ; //$NON-NLS-1$
	  if (sf != null) {
	    Object o = obj.refValue(sf) ;
	    if (o instanceof String)
	        name = (String) o ;
	    else if (o instanceof com.ibm.etools.java.instantiation.base.JavaObjectInstance)	       	    
	        name =  ((com.ibm.etools.java.instantiation.base.JavaObjectInstance)o).getInitializationString() ;
	  }
     }
     catch (Throwable e) {
       //  JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
     }
    }
    
	
	if (name == null) {
		name = ((IJavaObjectInstance) obj).getJavaType().getJavaName();
		if (name.indexOf('.') > 0)
			name = name.substring(name.lastIndexOf('.') + 1);

		name = lowCaseFirstCharacter(name);
	}
	
	ResourceSet rs = obj.getResourceSet() ;
	EClassifier meta = (EClassifier) ((IJavaInstance)obj).getJavaType() ;
	if (name.charAt(0) == '"') name = name.substring(1) ;
	if (name.charAt(name.length()-1) == '"') name = name.substring(0,name.length()-2) ;
			
	String prefix = getPrefix(meta,rs) ;
	if (prefix.length() == 0) {
		// The following is a temporary overide for the palette.
		if (name.startsWith(DEFAULT_VAR_PREFIX)) {
			name = lowCaseFirstCharacter(name.substring(DEFAULT_VAR_PREFIX.length())) ;
			if (kv != null) updateNameInComposition = true ;
		}
		else { 
			if (a != null && kv == null) updateNameInComposition = true ;
		}
	}
	else {
		if (!name.startsWith(prefix)) 
		  name = addPrefix(prefix,name) ;
	    if (isInternalType(obj))
	       name += Integer.toString(internalIndex++) ;
		
	}
		 	
	String ori = name ;
	
	int Index=2 ;
	try {
 	 IField[] fields = currentType.getFields() ;
	 for (int i=0; i<fields.length; i++) {
		if (fields[i].getElementName().equals(name)) {  // name conflict
			name = ori + Integer.toString(Index++) ;
			if (a != null) updateNameInComposition = true ;
			// Try again
			i=-1 ; 
		}
	 }
	 
 	  if (updateNameInComposition) {
			KeyedString ks = ((CDMPackage) RefRegister.getPackage(CDMPackage.packageURI)).getCDMFactory().createKeyedString();
			ks.setKey(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY);				
			ks.setValue(name);
			a.setKeyedValue(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY, ks);
	  }
	}
	

	
	catch (JavaModelException e) {} 
	return name ;	   
}

/**
 *  
 *  
 */
public String getInstanceVariableMethodName (RefObject obj, IType currentType, IDiagramModelInstance cm){
	
	if (isInternalType(obj)) return null ; // No method for a utility object
	
	String name = getInstanceVariableName(obj,currentType,cm) ;
//	int index = name.indexOf(DEFAULT_VAR_PREFIX)+DEFAULT_VAR_PREFIX.length() ;
	
	String methodName = addPrefix(DEFAULT_METHOD_PREFIX,name) ;
    String ori = methodName ;
		
	int Index=2 ;
	try {
 	 IMethod[] methods = currentType.getMethods() ;
	 for (int i=0; i<methods.length; i++) {
		if (methods[i].getElementName().equals(methodName)) {  // name conflict
			methodName = ori + Integer.toString(Index++) ;
			// Try again
			i=-1 ; 
		}
	 }
	}
	catch (JavaModelException e) {} 
	return methodName ;	   
}

/**
 * Should an instance Variable be declared, or a local one ?
 * @return true if local declaration, false if instance variable
 */
public boolean isLocalDecleration (RefObject obj, IType currentType, IDiagramModelInstance cm) {
    if (isInternalType(obj)) 
       return true ;  
    else
       return false ;  
}

/**
 * Is a uniqe method required to initialize this bean
 */
public boolean isGenerateAMethod(RefObject obj, IType currentType, IDiagramModelInstance cm) {
    if (isLocalDecleration(obj,currentType,cm)) return false ;
    // At this time default is always to generate a method
    return true ;
    
}



/**
 * Returns the fPrefStore.
 * @return Preferences
 */
public static Preferences getPrefStore() {
	    if (fPrefStore==null)
	       fPrefStore = VCEPlugin.getPlugin().getPluginPreferences();
		return fPrefStore;
}


/**
 * Returns the fComponentMeta.
 * @return JavaHelpers
 */
public static JavaHelpers getComponentMeta(ResourceSet rs) {
	
    if (fRS != null && fRS.equals(rs)) return fComponentMeta ;
    getMetaTypes(rs) ;	
	return fComponentMeta;
}

public static void clearCache() {
	fRS = null ;
	internalHelpers = null ;
	fComponentMeta = null;
}

}


