package com.ibm.etools.jbcf.codegen.java.rules;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.*;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaURL;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.IDiagramModelInstance;
import com.ibm.etools.jbcf.codegen.java.ITypeResolver;
import com.ibm.etools.jbcf.codegen.model.IBeanDeclModel;
import com.ibm.etools.jbcf.codegen.util.CodeGenUtil;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaModelException;
import  org.eclipse.jdt.internal.compiler.ast.AbstractVariableDeclaration ;
import org.eclipse.jdt.internal.compiler.ast.QualifiedTypeReference;
import  org.eclipse.jdt.internal.compiler.ast.SingleTypeReference ;

public class InstanceVariableRule implements IInstanceVariableRule, IMethodVariableRule{
    
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
    static List utilityClasses = null ;
    static ResourceSet  utilityRS = null ;
    static HashMap		 internalsCache = null ;

protected List getUtilityClasses (IDiagramModelInstance di) {
    if (utilityRS != null && utilityRS.equals(di.getModelResourceSet())) 
       return utilityClasses ;

    utilityClasses = new ArrayList() ;
    utilityRS = di.getModelResourceSet() ;
    internalsCache = null ;
    for (int i = 0; i < InstanceVariableCreationRule.internalTypes.length; i++) {          
     EClassifier iClass =  (EClassifier) di.getModelResourceSet().getObjectAndLoad((
                                 new JavaURL(InstanceVariableCreationRule.internalTypes[i])).getFullString());         
     utilityClasses.add(iClass) ;              
    }
    return utilityClasses ;
}

public boolean ignoreVariable(AbstractVariableDeclaration field, ITypeResolver resolver, IDiagramModelInstance di){
	
   //  Need to filter arrays, 
   //
   if (isUtilityVariable(field,resolver,di)) return false ;
   
   try {
   	// VAJava legacy
    if (field.name().startsWith(IInstanceVariableCreationRule.DEFAULT_VAR_PREFIX)) { //$NON-NLS-1$
       if (field.name().startsWith("ivjConn")) //$NON-NLS-1$
          return true ;
       else
          return false ;

    }
    else {    
    	String type = resolveType(field,resolver) ;
    	if (type == null) return true ;
    	
    	ResourceSet rs = di.getModelResourceSet() ;
    	EClassifier meta = (EClassifier) com.ibm.etools.java.impl.JavaClassImpl.reflect(type,rs) ;
    	    	
    	String pre = InstanceVariableCreationRule.getPrefix(meta,rs) ;
    	if (pre == null || pre.length() == 0) return false ;
    	return true  ;          
    }
   }
   catch (Throwable t) {
   	return true ;
   }   	
}

protected String getType(AbstractVariableDeclaration field) {
	String t=null ;    
    if (field.type instanceof SingleTypeReference) 
       t = new String(((SingleTypeReference)field.type).token) ;
    else if (field.type instanceof QualifiedTypeReference) {
        t = CodeGenUtil.tokensToString(((QualifiedTypeReference)field.type).tokens) ;        
    }
    else 
      return null ;
    return t ;	
}
protected String resolveType (AbstractVariableDeclaration field, ITypeResolver resolver) {
	String theType = getType(field) ;
	if (theType==null) return null ;
    return resolver.resolve(getType(field));
}

/**
 * e.g., GridBagConstraint.  The InstanceVariableCreationRule maintains the list
 *       of utility objects.
 */
protected boolean isUtilityVariable(AbstractVariableDeclaration field, ITypeResolver resolver, IDiagramModelInstance di) {
    CDEHack.fixMe("Need to support arrays etc.") ; //$NON-NLS-1$
    
    // This will also clear the internalsCache
    List utilClasses = getUtilityClasses(di) ;
    
    // Try to bypass resolving, and isAssignableFrom
    if (internalsCache==null) internalsCache = new HashMap(200) ;
    Boolean internal = (Boolean) internalsCache.get(getType(field)) ;
    if (internal != null) return internal.booleanValue() ;
    
    
    String t=resolveType(field,resolver) ;    ;
    if (t==null) return false ;
    try {
      EClassifier iClass =  (EClassifier) di.getModelResourceSet().getObjectAndLoad((new JavaURL(t)).getFullString());
            
      for (Iterator iterator = utilClasses.iterator(); iterator.hasNext();) {
        JavaClass uc = (JavaClass) iterator.next();
        if (uc.isAssignableFrom(iClass)) {
        	internalsCache.put(getType(field),Boolean.TRUE) ;
        	return true ;
        }
      }
    }
    catch (Exception e) {
         com.ibm.etools.jbcf.JBCFPlugin.log("InstanceVariableRule.isUtility(): Could not resolve - "+t,com.ibm.etools.jbcf.JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
    }    

    internalsCache.put(getType(field),Boolean.FALSE) ;
    return false ;
}
public static void clearCache() {
	utilityRS = null ;
	utilityClasses = null ;
}

}