package com.ibm.etools.jbcf.codegen.java.rules;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;

import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.internal.compiler.ast.*;

import com.ibm.etools.beaninfo.PropertyDecorator;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ecore.EAttribute;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ref.EList;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.codegen.java.ISourceVisitor;
import com.ibm.etools.jbcf.codegen.model.*;



public class ThisReferenceRule implements IThisReferenceRule {
    
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
    public final static String COMPONENT_CLASS = "java.awt.Component" ; //$NON-NLS-1$
    public final static String APPLET_CLASS =    "java.applet.Applet" ; //$NON-NLS-1$
    
    public final static int COMPONENT_CLASS_INDEX     = 0 ;
    public final static int APPLET_CLASS_INDEX        = 1 ;
    public final static int DEFAULT_CLASS_INDEX       = COMPONENT_CLASS_INDEX ;
    public final static String [] INIT_METHOD_NAME =      { "initialize", "init" } ; //$NON-NLS-1$ //$NON-NLS-2$
    public final static String [] INIT_METHOD_MODIFIERS = { "private",    "public" } ; //$NON-NLS-1$ //$NON-NLS-2$
    
    // The following are list of sf that should not determine if to have a "this" part
    public final static String[] ignoredAttributes = new String[] { "initializationString", "implicit","instantiateUsing", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                                                                       "serializeData","class" } ; //$NON-NLS-1$ //$NON-NLS-2$

public ISourceVisitor overideThisReferenceVisit(AbstractMethodDeclaration mDecl, MessageSend exp, IBeanDeclModel model){
	return null;
}

protected boolean ignoreAttribute(String att) {
    if (att==null) return true ;
    for (int i = 0; i < ignoredAttributes.length; i++) {
        if (att.equals(ignoredAttributes[i])) return true  ;     
    }
    return false ;
}

public boolean shouldProcess(AbstractMethodDeclaration method, MessageSend stmt){
	
	// VAJ's initialization method
    String sel = new String (method.selector) ;
    for (int i=0; i<INIT_METHOD_NAME.length; i++)
	  if (sel.equals(INIT_METHOD_NAME[i])) return true ;
      
	return false ;
}

public String[] getThisInitMethodName(ITypeHierarchy h) {
    IType[] classes = h.getAllClasses() ;
    for (int i = 0; i < classes.length; i++) {
      if (classes[i].getFullyQualifiedName().equals(APPLET_CLASS)) 
         return new String[] {INIT_METHOD_NAME[APPLET_CLASS_INDEX],INIT_METHOD_MODIFIERS[APPLET_CLASS_INDEX]} ;         
    }
    return new String[]  {INIT_METHOD_NAME[DEFAULT_CLASS_INDEX],INIT_METHOD_MODIFIERS[DEFAULT_CLASS_INDEX]} ;        
}

    /*
     * @see IThisReferenceRule#useInheritance(ITypeHierarchy)
     */
    public boolean useInheritance(String superClass, ResourceSet rs) {
        
        JavaHelpers clazz = com.ibm.etools.java.impl.JavaClassImpl.reflect (superClass,rs) ;
        if (clazz instanceof EClass) {           
           EList attr = ((EClass)clazz).getEAllAttributes() ;
           for (Iterator iterator = attr.iterator(); iterator.hasNext();) {
            EAttribute a = (EAttribute) iterator.next();
            if (a.isTransient()) continue ;
            if (ignoreAttribute(a.refName())) continue ;
            // Check to see if the attribute is a design time attribute
            if (a instanceof EModelElement) {
            	PropertyDecorator pd = Utilities.getPropertyDecorator((EModelElement)a) ;
            	if (pd != null) {
            	  if (pd.isSetIsDesignTime()) {
            	     if (!pd.isDesignTime()) continue ;
            	  }           	  
            	  if (pd.getWriteMethod()==null) continue ;
            	}
            }
            return true ;  
           }          
        }           
        return false ;            
    }
        
        
        

}