package com.ibm.etools.jbcf.codegen.model;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.util.* ;
import com.ibm.etools.jbcf.codegen.util.* ;
import org.eclipse.jdt.internal.compiler.ast.* ;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.ICompilationUnit ;
import org.eclipse.jdt.ui.IWorkingCopyManager ;
import org.eclipse.jdt.internal.ui.JavaPlugin ;

import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.codegen.* ; 
import com.ibm.etools.emf.ref.RefObject ;
import org.eclipse.jface.text.IDocument ;
import com.ibm.etools.jbcf.codegen.java.* ;



/*
 *  This class is the root for the Bean Decleration Model
 */
public class BeanDeclModel implements IBeanDeclModel {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	Vector      fBeans=null ;                           // Root Beans
	Vector      fTypes ;
	Vector      fRelationships = null ;                 
	Hashtable   fBeansKey = new Hashtable () ;   	    // Keep a hash key for the fBeans vector
	Hashtable   fBeanReturns = new Hashtable() ;        // Keep a hash of Methods with return Beans ... e.g., getJButton1() ;
	Hashtable	fBeanInitMethod = new Hashtable() ;     // Keep a hash to all Methods  that update beans 
	Hashtable   fRefObjKey = new Hashtable () ;         // Keep a hash from a RefObject to Bean Part
    ArrayList   fBeanDeleteCandidates = new ArrayList() ; // Hold before an Editor command is completed
	
	TypeDeclaration				fTypeDeclaration = null ;           // The class element of the JDOM model
	CodeTypeRef                 fTypeRef = null ;
	CompilationUnitDeclaration	fJDOM = null  ;                     // Root of the JDOM
	IWorkingCopyProvider		fWorkCopyP = null ;
	JavaSourceSynchronizer      fSrcSync = null ;
	IDiagramModelInstance		fCompositionModel = null ;
	String                      fLineSeperator = null ;
	int                         fState = BDM_STATE_DOWN ;
	ICodeGenStatus				fStatus = null ;
 
	
	

/**
 * Get all Beans
 */	
public Vector getBeans() {
	Vector v = new Vector() ;
	Enumeration e = fBeansKey.elements() ;
	while (e.hasMoreElements()) 
	  v.add(e.nextElement()) ;
	return v ;
}	


public synchronized void dispose() {
	fState = BDM_STATE_DOWN ;
//	if (fSrcSync != null) 
//	   fSrcSync.uninstall() ;
	Vector beans = getBeans() ;
	for (int i=beans.size()-1; i>=0; i--) {
		BeanPart b = (BeanPart) beans.remove(i) ;
		// clean up all expression adapters
		b.dispose () ;
	}
	fTypeRef.dispose() ;
	fCompositionModel = null ;
	fSrcSync = null ;
	fWorkCopyP = null ;
}




/**
 * Get beans directly sitting on the Compositions Beans
 */	
public Vector getRootBeans() {
	if (fBeans == null) fBeans = new Vector() ;
	return fBeans ;
}	

public void setLineSeperator(String sep) {
	fLineSeperator = sep ;
}
public String getLineSeperator() {
	return fLineSeperator ;
}

/**
 * When the VCE model removes a BeanPart from the model, it may be a temporary
 * stage were it is about to be added onto a different container/aggregator.
 */
public void designateAsDelete(BeanPart bean) {
    if (bean == null || fBeanDeleteCandidates.contains(bean)) return ;
    synchronized(fBeanDeleteCandidates) {
      fBeanDeleteCandidates.add(bean) ;
    }
}

/**
 * When the VCE model is added with an object, a client should check to see
 * if the tobe added is a delete candidate.
 * 
 * @param obj instance in question
 * @param remove determine if to remove the object fromt he delete candidate list
 * @return the BeanPart associated with the object, if it is in the delete candidate
 */
public BeanPart getDeleteDesignated(IJavaObjectInstance obj, boolean remove) {
    if (obj == null) return null ;
    BeanPart bean = getABean(obj) ;
    if (bean != null && fBeanDeleteCandidates.contains(bean)) {
       if (remove) 
         synchronized(fBeanDeleteCandidates) {
          fBeanDeleteCandidates.remove(bean) ;
         }
       return bean ;
    } else
        return null ;
}

/**
 * An editor compound command has completed.  Those elements that have not
 * bean added back to the JVE model will be removed from the BDM and source code.
 */
public void deleteDesignatedBeans() {
    
    synchronized (fBeanDeleteCandidates) {
        for(int i=0; i<fBeanDeleteCandidates.size(); i++) {
          try {
            BeanPart b = (BeanPart) fBeanDeleteCandidates.get(i) ;        
            BeanPartGenerator bgen = new BeanPartGenerator(this,getCompositionModel()) ;
            bgen.removeBeanPart(b) ;
          }
          catch (Throwable e) {
              com.ibm.etools.jbcf.JBCFPlugin.log(e) ;
          }
        }
        fBeanDeleteCandidates.clear() ;
    }
    
}


/**
 * Return all beans initialized by this method
 */	
public BeanPart[] getBeansInializedByMethod(String methodHandle) {
	Vector beans = new Vector () ;
	Iterator itr = getBeans().iterator() ;
	while (itr.hasNext()) {
		BeanPart b = (BeanPart)itr.next() ;
		if(b.getInitMethod()!=null && // consider beanparts with fields but no methods
		   b.getInitMethod().getMethodHandle().equals(methodHandle))
		   beans.add(b) ;
	}
	BeanPart [] result ;
	if (beans.size() > 0) {
		result = new BeanPart[beans.size()] ;
		for (int i=0; i<beans.size(); i++) {
			result[i] = (BeanPart)beans.elementAt(i) ;
		}
	}
	else
	    result = null ;
	    
	return result ;	
}


/**
 *  Add a BeanPart
 */
public void addBean (BeanPart bean) {

   com.ibm.etools.cde.CDEHack.fixMe("Need to deal with duplicates") ; //$NON-NLS-1$
   bean.setModel(this) ;
   if (bean.getContainer() == null) {
      if (!getRootBeans().contains(bean))
         getRootBeans().add(bean) ;
   }
   else
      getRootBeans().remove(bean) ;
   fBeansKey.put(bean.getUniqueName(),bean) ;
   if (bean.getRefObject() != null)
     fRefObjKey.put(bean.getRefObject(),bean) ;
}

/**
 *
 */
public void addRelationship(IBeanRelationship rel) {
	if (fRelationships == null) fRelationships = new Vector () ;
	if (!fRelationships.contains(rel))
	   fRelationships.add(rel) ;
}
/**
 *
 */
public void removeRelationship(IBeanRelationship rel) {
	if (fRelationships == null) fRelationships = new Vector () ;
	fRelationships.remove(rel) ;
}

/**
 * BeanPart has a Container, remove from root elements
 */
public void removeBeanFromRoot (BeanPart bean) {

   getRootBeans().remove(bean) ;
}
/**
 * BeanPart has a Container, remove from root elements
 */
public void removeBean (BeanPart bean) {

   removeBeanFromRoot(bean) ;
   fBeansKey.remove(bean.getUniqueName()) ; 
   if (bean.getRefObject()!= null)  
      fRefObjKey.remove(bean.getRefObject()) ;
   fBeanReturns.values().remove(bean) ;
   bean.setModel(null) ;
}


/**
 *
 */
public BeanPart getABean(String name) {
   return (BeanPart) fBeansKey.get(name)	 ;
}

/**
 *
 */
public BeanPart getABean(RefObject obj) {
   return (BeanPart) fRefObjKey.get(obj)	 ;
}

public void UpdateRefObjKey(BeanPart bean,RefObject prev) {
	if (prev != null) fRefObjKey.remove(prev) ;
	if (bean.getRefObject() != null)
	   fRefObjKey.put(bean.getRefObject(),bean) ;
}

/**
 *
 */
public TypeDeclaration getTypeDecleration() {
   return fTypeDeclaration ;
}

/**
 *  Set Type Decleration
 */
public void setTypeDecleration(TypeDeclaration decl) {
    fTypeDeclaration=decl ;
}
/**
 *  
 */
public CompilationUnitDeclaration getJDOM() {
   return fJDOM ;
}

public CodeTypeRef getTypeRef() {
	return fTypeRef ;
}
public void setTypeRef (CodeTypeRef tr) {
	fTypeRef = tr ;
}

/**
 *  Set Type Compilation Unit Decleration
 */
public void setJDOM(CompilationUnitDeclaration dom) {
    fJDOM=dom ;
}

public IDiagramModelInstance getCompositionModel() {
	return fCompositionModel ;
}
public void setCompositionModel(IDiagramModelInstance cm) {
    
    if (cm == null)
      if (fCompositionModel == null) return ;
   
	fCompositionModel = cm ;
	
	com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition c = cm.getModelRoot() ;
	
	   
	BeanPart b = new BeanPart("BeanSubclassComposition","") ; //$NON-NLS-1$ //$NON-NLS-2$
	if (!isStateSet(BDM_STATE_SNIPPET)) {
	   // Main BDM, adaptet to the composition
	   b.setRefObject(c) ;
	   b.setModel(this) ;
	   	// Remove the vanilla BeanPart adapter put by the setRefObject()
		ICodeGenAdapter a = (ICodeGenAdapter) c.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
		while (a != null){
		   c.removeAdapter(a) ;
		   a = (ICodeGenAdapter) c.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
		}

		// Set up a Root CodeGen adapter		
		CompositionDecoderAdapter adapter = new CompositionDecoderAdapter(c,b) ;
	    c.addAdapter(adapter) ;
	}
	else  // do not adapt Snippet BDMs
	   b.setModel(this) ;
}

/**
 *  Setter/Getter for the Compilation Unit Associated with this Model
 */
public void setWorkingCopyProvider(IWorkingCopyProvider wcp) {
	fWorkCopyP = wcp ;
	if (fSrcSync != null) 
	   fSrcSync.uninstall() ;
	fSrcSync = null ;	   
}

public void setSourceSynchronizer(JavaSourceSynchronizer sync) {
	if (fSrcSync != null) 
	   fSrcSync.uninstall() ;
	fSrcSync = sync ;	
}

public ICompilationUnit getCompilationUnit() {
	if (fWorkCopyP != null)
	    return fWorkCopyP.getLocalWorkingCopy() ;
	else
	    return null ;
}

public IDocument getDocument() {
	if (fWorkCopyP != null)
	    return fWorkCopyP.getLocalDocument() ;
	else
	    return null ;	
}

public Object getDocumentLock() {
	return fWorkCopyP.getLocalDocLock() ;
}

public IWorkingCopyProvider getWorkingCopyProvider() {
	return fWorkCopyP ;
}

public void updateJavaSource (String elementHandle) {
	fSrcSync.updateSharedFromLocal(elementHandle) ;	
}



/**
 *  If a method returns a Bean instance, this function will 
 *  return the corresponding bean part.
 */
public BeanPart getBeanReturned(String methodName) {
   return (BeanPart) fBeanReturns.get(methodName)	 ;
}
/**
 *  The method returns an instance of beanName
 */
public void addMethodReturningABean(String method,String beanName) throws CodeGenException {

   BeanPart bean = getABean(beanName) ;
   if (bean == null || method == null) throw new CodeGenException("Invalid Bean Name "+beanName) ; //$NON-NLS-1$
   
   fBeanReturns.put(method,bean);
}

/**
 *  Add a key to the methodRef, using the IMethod handle string
 */
public void addMethodInitializingABean (CodeMethodRef methodRef) {
	if (methodRef.getMethodHandle() != null)
	   fBeanInitMethod.put(methodRef.getMethodHandle(),methodRef) ;
}
/**
 *  methodRef, using the IMethod handle string 
 *  @return 
 */
public CodeMethodRef getMethodInitializingABean (String methodHandle) {
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with more than one bean") ; //$NON-NLS-1$
	return (CodeMethodRef) fBeanInitMethod.get(methodHandle) ;
}

public void removeMethodRef(CodeMethodRef mr) {
	com.ibm.etools.cde.CDEHack.fixMe("Need to deal with multi beans per method"); //$NON-NLS-1$
	fBeanInitMethod.remove(mr.getMethodHandle()) ;
	fBeanReturns.remove(mr.getMethodName()) ;
	Iterator allBeans = getBeans().iterator();
	while(allBeans.hasNext()){
		BeanPart bean = (BeanPart)allBeans.next();
		if(bean.getInitMethod()!=null && bean.getInitMethod().equals(mr))
			bean.removeInitMethod(mr);
		if(bean.getReturnedMethod()!=null && bean.getReturnedMethod().equals(mr))
			bean.removeReturnMethod(mr);
	}
	allBeans = getBeans().iterator();
	while(allBeans.hasNext()){
		BeanPart bean = (BeanPart)allBeans.next();
		CodeExpressionRef exps[] = new CodeExpressionRef[bean.getRefExpressions().size()];
		Iterator beansExps = bean.getRefExpressions().iterator();
		int count =0;
		while(beansExps.hasNext()){
			exps[count] = (CodeExpressionRef)beansExps.next();
			count++;
		}
		if (!isStateSet(BDM_STATE_DOWN)) {
			for (count = 0; count < exps.length; count++) {
				CodeExpressionRef exp = exps[count];
				if (exp != null && exp.getCodeContent() != null && CodeGenUtil.isExactlyPresent(exp.getCodeContent(),mr.getMethodName())){
					exp.deleteFromComposition();
					exp.dispose();
				}
			}
		}
	}
}

/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {	
	
	StringBuffer sb = new StringBuffer () ;
	sb.append("BDM : [") ; //$NON-NLS-1$
	if (isStateSet(BDM_STATE_SNIPPET))
	   sb.append(" DELTA") ; //$NON-NLS-1$
	if (isStateSet(BDM_STATE_UP_AND_RUNNING))
	   sb.append (" UP") ; //$NON-NLS-1$
	if (isStateSet(BDM_STATE_UPDATING_JVE_MODEL))
	   sb.append (" UPDATING") ; //$NON-NLS-1$
	sb.append ("]") ; //$NON-NLS-1$
	
	return sb.toString();
}

public String resolve(String unresolved){
	return getWorkingCopyProvider().resolve(unresolved);
}

/*
 * @see IBeanDeclModel#resolveThis()
 */
public String resolveThis() {
	return getWorkingCopyProvider().resolveThis();
}

public ITypeHierarchy getClassHierarchy() {
    return getWorkingCopyProvider().getHierarchy() ;
}

protected void setState(int flag) {
	fState = flag ;
}

protected int getState() {
	return fState ;
}

public void setState(int flag, boolean state) throws CodeGenException {
	
	if (((flag & BDM_STATE_UP_AND_RUNNING)>0 && (flag & BDM_STATE_DOWN)>0) || // Up and Down
	    ((flag & BDM_STATE_UP_AND_RUNNING)>0 && (getState() & BDM_STATE_SNIPPET)>0) // Up and Snippet
	   )
	   throw new CodeGenException ("Invalid BDM state: "+flag) ; //$NON-NLS-1$
	
	if(state) {
		setState(getState() | flag);
		if ((flag & BDM_STATE_UP_AND_RUNNING)>0) // If Up, set the Down Flag.
		       setState(getState()&~BDM_STATE_DOWN) ;
		else if ((flag & BDM_STATE_DOWN)>0)
		       setState(getState()&~BDM_STATE_UP_AND_RUNNING) ;		       
	}
	else {
		setState(getState() & (~flag));
    }
}

public boolean isStateSet(int state){
	return ((getState() & state) == state);
}

private static String constructUniqueName(String methodHandle, String simpleName){
	return methodHandle+"^"+simpleName; //$NON-NLS-1$
}

public static String constructUniqueName(IMethod method, String simpleName){
	return constructUniqueName(method.getHandleIdentifier(), simpleName);
}

public static String constructUniqueName(CodeMethodRef method, String simpleName){
	return constructUniqueName(method.getMethodHandle(),simpleName);
}

public Collection getBeansInitilizedByMethod(CodeMethodRef mref) {
   com.ibm.etools.cde.CDEHack.fixMe("We should have a hash table for these") ; //$NON-NLS-1$
   if (mref == null) return null ;
   ArrayList beans = new ArrayList() ;
   Iterator itr = getBeans().iterator() ;
   while (itr.hasNext()) {
	BeanPart b = (BeanPart) itr.next();
	if (b.getInitMethod()!=null && b.getInitMethod().equals(mref)) beans.add(b) ;
   }	
   return beans ;
}	

/**
 * If a method's name has changed, so was its handle, and so would a bean's unique name
 */
public void updateBeanNameChange(BeanPart bp) {
	
	fBeansKey.values().remove(bp) ;
	fBeansKey.put(bp.getUniqueName(),bp) ;
}


	/**
	 * Returns the fStatus.
	 * @return ICodeGenStatus
	 */
	public ICodeGenStatus getFStatus() {
		return fStatus;
	}

	/**
	 * Sets the fStatus.
	 * @param fStatus The fStatus to set
	 */
	public void setFStatus(ICodeGenStatus fStatus) {
		this.fStatus = fStatus;
	}

	/**
	 * @see com.ibm.etools.jbcf.codegen.model.IBeanDeclModel#resolveSingleNameReference(String)
	 */
	public String resolveSingleNameReference(String selector, int location) {
		return ASTHelper.resolveSingleNameReference(
			selector, location,
			getWorkingCopyProvider().getSharedDocument().get());
	}

}