package com.ibm.etools.jbcf.codegen.model;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.internal.compiler.ast.* ;


import java.util.* ;

import com.ibm.etools.cdm.Annotation;
import com.ibm.etools.cdm.VisualInfo;
import com.ibm.etools.emf.ref.RefObject ;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.java.* ;
import com.ibm.etools.jbcf.codegen.util.CodeGenUtil;

public class BeanPart {
    
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
    public final static String         THIS_NAME = "this" ; //$NON-NLS-1$
    public final static String		 THIS_HANDLE = SynchronizerWorkItem.THIS_ANNOTATION_HANDLE;
	
	String 	fName ;
	String 	fType ;
	AbstractVariableDeclaration fFieldDecl = null ;
	Vector 		fBeanInitMethods = new Vector () ;                   // Method/s where the Bean is created
	Vector      fBeanRefExpressions =  new Vector () ;               // Method/s which update bean attributes
	Vector		fBeanReturnMethods = new Vector () ;                 // Metods/s which return the value of this bean
	Vector      fRelationships = new Vector () ;                     // Relationships associated with this bean
	RefObject   fRefObject = null ;                                  // Mof Instance of this Bean
	BeanPart    fContainer = null ;								 	 // Mof Object that contains this object - null ? part of Composition
	Vector      fChildren = new Vector () ;                          // Beans this part may contain components	
	IBeanDeclModel fModel = null ;    
    BeanPart    fProxyBeanPart = null ;							 // This bean part is not in the BeanDecModel
    boolean    fisInstanceVar = true ;                             // Is the bean an instance variable ?
    FreeFormAnnoationDecoder fFFDecoder = null ;                    // Responsible parse/generate FF tags
    String      fFieldDeclHandle = null ;
    List        fBadExpressions = null;
    

/**
 *  
 */
public  BeanPart (AbstractVariableDeclaration decl) {
	
	fFieldDecl = decl ;
	fName = decl.name() ;		
	setType(com.ibm.etools.jbcf.codegen.util.CodeGenUtil.tokensToString(decl.type.getTypeName()));
}

/**
 * @return the FreeForm Decoder associated with this bean
 */
public FreeFormAnnoationDecoder getFFDecoder() {
    boolean newDecoder = false ;
    if (fFFDecoder==null) {
       if (getSimpleName().equals(THIS_NAME)) 
          fFFDecoder = new FreeFormThisAnnotationDecoder(this) ;
       else
          fFFDecoder = new FreeFormAnnoationDecoder(this) ;
       newDecoder = true ;
    }
    
    Annotation ant = CodeGenUtil.getAnnotation(getRefObject()) ;    
    
    if (ant != null) {
        ICodeGenAdapter a = (ICodeGenAdapter)ant.getAdapter(ICodeGenAdapter.JVE_CODEGEN_ANNOTATION_ADAPTER) ;
        if (a != null && newDecoder) {
            ant.removeAdapter(a) ;
            a = null ;
        }
        
        // Need an adapter on the annotation in case the Visual Info is added/changed/removed
        if (a==null) {
          a = new AnnotationDecoderAdapter(getRefObject(),fFFDecoder) ;
          ant.addAdapter(a) ;
        }
        // This is where the FF constraint is on
        VisualInfo vi = ant.getVisualInfo(getModel().getCompositionModel().getDiagram()) ;
        if (vi != null) {
            ICodeGenAdapter va = (ICodeGenAdapter)vi.getAdapter(ICodeGenAdapter.JVE_CODEGEN_ANNOTATION_ADAPTER) ;
            if (va != null && newDecoder) {
              vi.removeAdapter(va) ;
              vi.addAdapter(a) ;
            }
        }           
    }

    return fFFDecoder ;    
}

public void setFieldDeclHandle(String dec) {
    fFieldDeclHandle = dec ;
}

public String getFieldDeclHandle() {
    return fFieldDeclHandle ;
}

/**
 *  
 */
public  BeanPart (AbstractVariableDeclaration decl,CodeMethodRef method,boolean initMethod)  {
	
	this(decl) ;
	
    if (method != null) {	
	  if (initMethod) fBeanInitMethods.add(method) ;
	  else fBeanReturnMethods.add(method) ;
    }		
}


/**
 *  
 */
public  BeanPart (AbstractVariableDeclaration decl,CodeExpressionRef exp)  {
	
	this(decl) ;

    if (exp != null)
       fBeanRefExpressions.add(exp) ;	
       
}


/**
 *  Construct a BeanPart for the class (this) we are parsing
 */
public  BeanPart (String thisType) 	 {
     this(THIS_NAME, thisType);
}

/**
 *  Construct a BeanPart for a bean discovered from C. Model
 */
public  BeanPart (String name,String theType) 	 {
     fName = name ;
     setType(theType);
}

public boolean isInstanceVar() {
    return fisInstanceVar ;
}

public void setInstanceVar(boolean flag) {
    fisInstanceVar = flag ;
}


/**
 *   Add a method where the bean is created
 */
public synchronized void addInitMethod(CodeMethodRef methodRef) {
	com.ibm.etools.cde.CDEHack.fixMe("need to put the one with the new() first") ; //$NON-NLS-1$
	if (!fBeanInitMethods.contains(methodRef))
	   fBeanInitMethods.add(methodRef) ;
	if (fModel != null) {
	   fModel.addMethodInitializingABean(methodRef);
	   methodRef.setModel(fModel) ;
	   fModel.updateBeanNameChange(this);
	}
}

public synchronized void removeInitMethod (CodeMethodRef m) {
	fBeanInitMethods.remove(m) ;
}

public synchronized void removeReturnMethod (CodeMethodRef m) {
	fBeanReturnMethods.remove(m) ;
}
/**
 *   Get the one and only one we will use
 */
public CodeMethodRef getInitMethod() {
	if (fBeanInitMethods.size() == 0) return null ;
	return (CodeMethodRef) fBeanInitMethods.get(0) ;
}

/**
 *   Get the one and only one we will use
 */
public CodeMethodRef getReturnedMethod() {
	if (fBeanReturnMethods.size() == 0) return null ;
	return (CodeMethodRef) fBeanReturnMethods.get(0) ;
}

/**
 *   Add a method where the bean instance is returned
 */
public synchronized void addReturnMethod(CodeMethodRef methodRef) {
	if (!fBeanReturnMethods.contains(methodRef))
	   fBeanReturnMethods.add(methodRef) ;
	if (fModel != null)
	  try {
	   fModel.addMethodReturningABean(methodRef.getMethodName(),getUniqueName()) ;
	   methodRef.setModel(fModel) ;
	  }
	  catch (com.ibm.etools.jbcf.codegen.util.CodeGenException e) {
	  	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
	  }
}


/**
 * Add a method where a bean's setXX() method is called
 */
public synchronized void addRefExpression(CodeExpressionRef exp) {
	if (!fBeanRefExpressions.contains(exp))
	   fBeanRefExpressions.add(exp) ;
}

/**
 * Remove a method where a bean's setXX() method is called
 */
public synchronized void removeRefExpression(CodeExpressionRef exp) {
	fBeanRefExpressions.remove(exp) ;
}



/**
 *
 */
public String getSimpleName () {
	return fName ;
}

public String getUniqueName() {
	if(!isInstanceVar() && getInitMethod()!=null){
		return BeanDeclModel.constructUniqueName(getInitMethod(),getSimpleName()); //getInitMethod().getMethodHandle()+"^"+getSimpleName();
	}else{
		return getSimpleName();
	}
}

/**
 *
 */
public Collection getRefExpressions () {
	if (getInitMethod()==null) {
		for (Iterator itr = fBeanRefExpressions.iterator(); itr.hasNext();) {
			CodeExpressionRef exp = (CodeExpressionRef) itr.next();
			if (exp.getMethod() != null) {
				exp.getMethod().removeExpressionRef(exp) ;
			}
		}
		fBeanRefExpressions.clear() ;
	}
	else {
		CodeMethodRef mr = getInitMethod() ;
		for (int I=fBeanRefExpressions.size()-1; I>=0; I--) {
			CodeExpressionRef exp = (CodeExpressionRef) fBeanRefExpressions.get(I) ;
			if (exp.getMethod() != mr) {
				exp.getMethod().removeExpressionRef(exp) ;
				fBeanRefExpressions.removeElementAt(I) ;
			}
		}				
	}	   
	return fBeanRefExpressions ;
}


/**
 *
 */
public String getType () {
	return fType ;
}

/**
 *
 */
public AbstractVariableDeclaration getFieldDecl() {
      return fFieldDecl ;	
}
///**
// *
// */
//public boolean isInstanceVariable() {
//	return fFieldDecl instanceof FieldDeclaration ;
//}
///**
// *
// */
//public boolean isLocalVariable() {
//	return fFieldDecl instanceof LocalDeclaration ;
//}
//	
/**
 *
 */
public RefObject getRefObject() {
	if (!isProxy())
	   return fRefObject ;
	else
	   return fProxyBeanPart.getRefObject() ;
}


/**
 * null, will clear the current RefObject ;
 */
public void setRefObject (RefObject obj) {
	
	RefObject old = fRefObject ;
	if (fRefObject != null) {
       ICodeGenAdapter a = (ICodeGenAdapter)fRefObject.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
       while (a != null) {
          fRefObject.removeAdapter(a) ;
          a = (ICodeGenAdapter)fRefObject.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
       }
	}
	if (obj != null) {
	  ICodeGenAdapter a = (ICodeGenAdapter)obj.getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER) ;
	  if (a == null) {
		BeanDecoderAdapter ba ;
		if (getSimpleName().equals(THIS_NAME))
		  ba = new ThisBeanDecoderAdapter(obj,this) ;
		else
		  ba = new BeanDecoderAdapter(obj,this) ;
		obj.addAdapter(ba) ;
	  }
	}
	fRefObject=obj ;
	if (fModel != null)	
	   fModel.UpdateRefObjKey(this,old) ;
}

protected void setType(String fType){	
	// The BeanSubClassComposition pseodo bean part as an empty type
    if (getModel() != null && fType !=null && !fType.equals("")) { //$NON-NLS-1$
    	if(getModel().getCompilationUnit()!=null){
		  //IType t = CodeGenUtil.getMainType(getModel().getCompilationUnit()) ;
		  //String rt = CodeGenUtil.resolveTypeComplex(t,fType) ;
		  String rt = getModel().resolve(fType);
		  if(rt!=null)
		  	this.fType = rt;
		  else
		  	this.fType = fType;
    	}
    }
    else
      this.fType = fType ;
}

/**
 *
 */
public BeanPart getContainer() {
	return fContainer ;
}

/**
 *
 */
public void setContainer (BeanPart bean) {
	fContainer=bean ;
    // Refresh the bean's status
    fModel.addBean(this) ;
}

/**
 *
 */
public void addChild (BeanPart bean) {
    if (!fChildren.contains(bean))
	   fChildren.add(bean) ;
}
/**
 *
 */
public void removeChild (BeanPart bean) {
	fChildren.remove(bean) ;
}

public Iterator getChildren() {
	return fChildren.iterator() ;
}

public IBeanDeclModel getModel () { 
	if (isProxy())
	  return fProxyBeanPart.getModel() ;
	else
	  return fModel ;
}

public void setModel(IBeanDeclModel model) {
    if (fModel != null && model != null && fModel.equals(model)) 
       return ;  // No need to set the type and resolve
	fModel = model ;
	if(fType!=null)
		setType(getType()) ; // Refresh Type
    if (getFieldDeclHandle() == null && model!=null) {
    	IField f = CodeGenUtil.getFieldByName(getSimpleName(),model.getCompilationUnit()) ;
    	if (f !=null)
    	   setFieldDeclHandle(f.getHandleIdentifier()) ;
    }
    else if (getSimpleName().equals(THIS_NAME)) 
        setFieldDeclHandle(THIS_HANDLE) ;
}

public String toString () {
   return super.toString() + "  " + fName + "(" + fType + ")";	 //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
}

public synchronized void addRelationship(IBeanRelationship rel) {	
	if (fRelationships == null) fRelationships = new Vector () ;
	if (!fRelationships.contains(rel))
	  fRelationships.add(rel) ;
}
/**
 *
 */
public synchronized void removeRelationship(IBeanRelationship rel) {
	if (fRelationships == null) fRelationships = new Vector () ;
	fRelationships.remove(rel) ;
}

public Object[] getRelationships() {
	return fRelationships.toArray() ;
}



public boolean isProxy () {
	return fProxyBeanPart!=null ;
}

public void setProxy (BeanPart proxy) {
    
	if(proxy==this) throw new IllegalArgumentException() ;	
	fProxyBeanPart = proxy ;	
	if(fType!=null)
		setType(getType()) ; // Refresh Type
}

public BeanPart getProxy() {
	return fProxyBeanPart ;
}



public synchronized void dispose() {
	
	CodeMethodRef m = getInitMethod() ;
	if (m != null) {
		com.ibm.etools.cde.CDEHack.fixMe("What about multi beans in a method") ; //$NON-NLS-1$
		if(fModel!=null){
			Collection parts = fModel.getBeansInitilizedByMethod(m);
			if(parts!=null){
				if(parts.size()<1 || (parts.size()<=1 && parts.contains(this))){
					// no more beans in that method OR I am the last bean
					m.dispose(); 
				}else{
					// has more beans - dont dispose the method.
					if (fBeanRefExpressions!=null)
					 for (int i=fBeanRefExpressions.size()-1; i>=0; i--) {
					 	((CodeExpressionRef)fBeanRefExpressions.get(i)).dispose() ;
					 }
				}
			}else{
				m.dispose();
			}
		}else{
			m.dispose() ;
		}
	}
	   
	if (fModel != null)
	  fModel.removeBean(this) ;
	fModel = null ;
	fBeanInitMethods.clear() ;
	fBeanRefExpressions.clear() ;
	fBeanReturnMethods.clear() ;
	fContainer = null ;
	fChildren.clear() ;
	if (fRefObject != null) {
	  ICodeGenAdapter a = (ICodeGenAdapter)fRefObject.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
	  while (a != null) {
	    fRefObject.removeAdapter(a);
	    a = (ICodeGenAdapter)fRefObject.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE) ;
	  }
	}	
	fRefObject = null ;	
			
}

public boolean isEquivalent(BeanPart b) {
   if (b==null) return false ;
   
   if (getSimpleName().equals(b.getSimpleName()) &&
       getType().equals(b.getType()))
       return true ;
   return false ;
}

	/**
	 * TEMPORARY: The BDM's should contain bad expressions in some 
	 *            IGNORE state, instead of just removing them.
	 * Bad expressions are stored for situations which need their
	 * presence - like when import statements are added - When the
	 * Delta bdm and the main bdm are compared we dont know if an
	 * expression is not in the main bdm because it was not added
	 * or because it was undecodeable.
	 * @return List
	 */
	public List getBadExpressions() {
		if(fBadExpressions==null)
			fBadExpressions = new ArrayList();
		return fBadExpressions;
	}

	public void addBadExpresion(CodeExpressionRef fBadExpression) {
		if(fBadExpressions==null)
			fBadExpressions = new ArrayList();
		fBadExpressions.add(fBadExpression);
	}
	
	public void removeAllBadExpressions(){
		if(fBadExpressions!=null)	
			fBadExpressions.clear();
	}

}