package com.ibm.etools.jbcf.codegen.model;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jdt.internal.compiler.ast.* ;
import com.ibm.etools.emf.ref.RefStructuralFeature ;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.java.*;
import com.ibm.etools.jbcf.codegen.java.CodeGenSourceRange;
import com.ibm.etools.jbcf.codegen.java.ICodeGenSourceRange;
import com.ibm.etools.jbcf.codegen.util.*;
import org.eclipse.jdt.core.* ;
import java.util.* ;
import org.eclipse.jface.text.* ;


public class CodeMethodRef extends AbstractCodeRef {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected   AbstractMethodDeclaration   fdeclMethod = null ;
//protected   IMethod                     fMethod = null ;
protected   CodeTypeRef	             fTypeRef = null ;
protected   Vector                      fExpressions = new Vector () ;
protected	 String                      fMethodHandle = null ;  // Unique handle in a class
protected   String                      fMethodName ;
protected   DocListener                 fdocListener=null ;
protected   Object                      fSync ;
protected   IBeanDeclModel              fModel = null ;
	


    /* This class will keep the expression's offset in sync with the doc. */
    class DocListener implements IDocumentListener {
        
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
        IDocument       fDoc = null ;
        boolean        fActive = false ;
  	
  	public void documentAboutToBeChanged(DocumentEvent event) {  
  				
  	}
  	
  	private void refresh() {
		ICompilationUnit cu = fModel.getCompilationUnit() ;		
		try {
			if (!cu.isConsistent())		     
				cu.reconcile() ; 
			if (fMethodHandle == null) 
				throw new RuntimeException ("No Method Handle") ; //$NON-NLS-1$
			/* Make sure we reflect the latest context of the method */
			IType mainType = CodeGenUtil.getMainType(fModel.getCompilationUnit());
			IMethod fMethod = CodeGenUtil.getMethod(mainType, getMethodHandle());
			if(fMethod==null)
				return;
			// No need to reconcile, we just did.
			IMethod mArray[] = mainType.findMethods(fMethod) ;
			if (mArray == null || mArray.length == 0) {
				com.ibm.etools.jbcf.JBCFPlugin.log("CodeMethodRef$DocListener.refresh(), No method found; "+fMethod,JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
				return ;
			}
			fMethod = mArray[0] ;
			if(fMethod!=null)
				setOffset(fMethod.getSourceRange().getOffset()) ;
		}catch (Exception e) {}
	}
	
	public void documentChanged(DocumentEvent event) {
        if (!fActive) return ;
        
		com.ibm.etools.cde.CDEHack.fixMe("Need to deal with removal of elements") ; //$NON-NLS-1$
		if(fModel==null)
			return;
		IType mainType = CodeGenUtil.getMainType(fModel.getCompilationUnit());
		IMethod fMethod = CodeGenUtil.getMethod(mainType, getMethodHandle());
		if(fMethod==null)
			return;
		synchronized (fSync) {
			if (fExpressions == null || event.getOffset() > getOffset()+getLen()) 
				return ;
			// No need to synchronize here, as the instigator of this change had to sync.
			// System.out.println ("CodeMethodRef-documentChanged("+fMethodName+"): Off("+event.getOffset()+"), Len("+event.getLength()+")") ;		
			refresh() ;		   		 
			if (event.getOffset() <= getOffset()) {
				if (event.getOffset()+event.getLength()>getOffset()) {
					com.ibm.etools.cde.CDEHack.fixMe() ;
					// Delta spans this method.
					/************************************************************** fix */
					return ;
				}else{
					// Delta a prefix, bu totally outside this method		 	
				}
			}else{
				//  Delta is within this method, need to update expression's offset.		 		 		 	      		 	
				int delta = event.getText().length()-event.getLength() ;
				Iterator itr = getExpressions() ;			  
				while (itr.hasNext()) {
					CodeExpressionRef exp = (CodeExpressionRef) itr.next() ;
					// If this expression has updated the document, skip it
					if (exp.isStateSet(exp.STATE_UPDATING_SOURCE) || exp.isStateSet(exp.STATE_NOT_EXISTANT)) 
						continue ;
					if (exp.getOffset()+getOffset() >= event.getOffset())
						exp.setOffset(exp.getOffset()+delta) ;				
		  		}
			}
			try {
				setOffset(fMethod.getSourceRange().getOffset()) ;
				setContent(fMethod.getSource()) ;
			}catch(JavaModelException e) {
				JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
			}			
		}
	}
	
	
	public void  connectToDocument() {
		if(fDoc == null && fTypeRef.getBeanModel().getDocument()!=null) {
            fDoc = fTypeRef.getBeanModel().getDocument() ;
			fDoc.addDocumentListener(this) ;
            fActive = true ;
        }
	}
	
	public void  disconnectFromDocument() {
        if (fDoc != null) {
			fDoc.removeDocumentListener(this) ;
            fDoc = null ;
            fActive = false ;
        }
	}  	  	
  }

public CodeMethodRef (AbstractMethodDeclaration method,CodeTypeRef tRef, String methodHandle,ISourceRange range, String content) {
	super(range.getOffset(),range.getLength(),content) ;
	fSync = this ;
	fTypeRef = tRef ;	      
	connectToDoc() ;
	setMethodHandle(methodHandle) ;
	setDeclMethod(method) ;
	tRef.addRefMethod(this)	 ;
}

public void connectToDoc() {
    if (fdocListener == null) {
        fdocListener = new DocListener() ;
        fdocListener.connectToDocument() ;
    }
}

public CodeMethodRef (CodeTypeRef tr,String mName) {
	super() ;	
	fSync = this ;
	fTypeRef = tr ;
	tr.addRefMethod(this) ;
	fMethodName=mName ;
	connectToDoc() ;
}

//public void setMethod (IMethod method) {
//	fMethod = method ;
//	if(fMethod!=null){
//		fMethodName = fMethod.getElementName() ;
//		fMethodHandle = fMethod.getHandleIdentifier() ;	
////		try {
////		  setOffset(method.getSourceRange().getOffset()) ;
////		  setContent(method.getSource()) ;
////		}
////		catch (JavaModelException e) {
////		    JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
////		}
//	}
//	if (fdocListener == null) {
//		fdocListener = new DocListener() ;
//		fdocListener.connectToDocument() ;
//	}
//}

protected void setDeclMethod(AbstractMethodDeclaration method){
	fdeclMethod = method ;
	if(fdeclMethod!=null){// && fMethod==null){
		fMethodName = new String(fdeclMethod.selector);
	}
}

public String getMethodName() {
	return fMethodName ;
}

public AbstractMethodDeclaration getDeclMethod() {
	return fdeclMethod ;
}

public void setMethodHandle(String handle) {
	fMethodHandle = handle ;
	// The unique name of beans may have changed. 
	if (fModel!=null) { 
		Collection beans = fModel.getBeansInitilizedByMethod(this) ;
		for (Iterator iter = beans.iterator(); iter.hasNext();) {
			BeanPart bean = (BeanPart) iter.next();
			fModel.updateBeanNameChange(bean) ;
		}
	}
}
public String getMethodHandle() {
	return fMethodHandle ;
}

public int isEquivalent(AbstractCodeRef code){
	return -1;
}

/**
 *  Adds an expression to this method. Initially this process was 
 *  totally based off offsets of the expressions. Now, we are
 *  concerned about offsets *only* if the expression originated
 *  from the *original* source. Else they will be appended in the 
 *  end, and then the method will be 'refreshed' by the expression.
 */
public synchronized void addExpressionRef(CodeExpressionRef ref) {
	int offset = ref.getOffset() ;
	if (fExpressions == null) 
		fExpressions = new Vector() ;
	if((ref.isStateSet(ref.STATE_SRC_LOC_FIXED)) && //((ref.getState()&CodeExpressionRef.STATE_SRC_LOC_FIXED)==CodeExpressionRef.STATE_SRC_LOC_FIXED) && 
	   (offset>-1)){
		int index;
		for (index=0; index<fExpressions.size(); index++) 
			if (((CodeExpressionRef)fExpressions.elementAt(index)).getOffset()>offset) break ;
	      if (index == fExpressions.size())
		   fExpressions.add(ref) ;
		else
		   fExpressions.add(index,ref) ;
	}else{
		fExpressions.add(ref);
	}
}

public synchronized void removeExpressionRef(CodeExpressionRef ref) {
	fExpressions.remove(ref) ;
}

/**
 * Get all expressions in this method that impact bean/s in the model
 */
public synchronized Iterator getExpressions() {
	if (fExpressions == null) fExpressions = new Vector () ;
	return fExpressions.iterator() ;
}

/**
 * Updates the expression order so that:
 * 
 * (*) Expressions modifying a particular bean always stick
 *     togeether. Except the declaration statements, because
 *     they might be used by other beans
 * 
 * The way it is done is:
 * (*) Determine the priority of all the expressions. Priority
 *     of expressions is determined by the expression's decoders.
 *     If the expression contains a 'new', it is assigned 
 *     Integer.MAX_VALUE priority. If any problem occurs, the
 *     expression is set a priority of -1.
 * (*) 
 */
public synchronized void updateExpressionOrder() throws CodeGenException{
	for(int count=0;count<fExpressions.size();count++){
		CodeExpressionRef expression = (CodeExpressionRef)fExpressions.get(count);
		try{			
			if ((!expression.isStateSet(expression.STATE_NO_OP)) &&  //((expression.getState()& expression.STATE_NO_OP) == 0) && 
			    (expression.isAnyStateSet()) &&
			    (!expression.isStateSet(expression.STATE_NOT_EXISTANT))) { //expression.getState() != expression.STATE_NOT_EXISTANT) {
				 expression.setProprity(expression.getDecoder().determinePriority());
			}else{
				com.ibm.etools.cde.CDEHack.fixMe("hard coding here is not scalable - we may have ivjFoo = getFrame().getContentPane() ; as init exp.") ; //$NON-NLS-1$
				if(expression.isStateSet(expression.STATE_INIT_EXPR)){ //$NON-NLS-1$
					expression.setProprity(new Integer(Integer.MAX_VALUE));
				}else{
					expression.setProprity(new Integer(-1));
				}
			}
		}catch(Exception e ){JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;}
	}
	List newfExpressions = new ArrayList(fExpressions);

	Hashtable beanToExpHash = new Hashtable();
	for(int count=0;count<newfExpressions.size();count++){
		CodeExpressionRef exp = (CodeExpressionRef)newfExpressions.get(count);
		if(!beanToExpHash.containsKey(exp.getBean())){
			beanToExpHash.put(exp.getBean(), new ArrayList());;
		}
		List expHolder = (List) beanToExpHash.get(exp.getBean());
		expHolder.add(exp);
	}
	
	Vector tmpFinalOrder = new Vector();
	int firstReferenceIndex = -1;
	int firstReferenceOffset = Integer.MAX_VALUE;
	String firstReferenceFiller = null;
	// Now there are unique beans (as keys) in the hashtable.
	Enumeration beansToBeVisited = determineBeanOrder(beanToExpHash.keys());
	while(beansToBeVisited.hasMoreElements()){
		Vector sortedExpressions = new Vector();  // Will hold the expressions sorted purely by priority.
		List expressions = (List) beanToExpHash.get(beansToBeVisited.nextElement());
		SortedSet methodSorterSet = new TreeSet(getDefaultMethodComparator());
		for(int ec=0;ec<expressions.size();ec++){
			methodSorterSet.add(expressions.get(ec));
		}
		Iterator sorted = methodSorterSet.iterator();
		while(sorted.hasNext())
			sortedExpressions.add(sorted.next());

		Vector beanFinalOrder = new Vector(sortedExpressions.size());
		for(int expc=0;expc<sortedExpressions.size();expc++){
			CodeExpressionRef exp = (CodeExpressionRef)sortedExpressions.get(expc);
			if(exp.isStateSet(exp.STATE_SRC_LOC_FIXED)){
				// Record the top-most available STATE_SRC_LOC_FIXED,
				// as this is our reference for newly generated expressions.
				if(exp.getOffset()<firstReferenceOffset){
					firstReferenceOffset=exp.getOffset();
					firstReferenceIndex=expc;
					firstReferenceFiller=exp.getFillerContent();
				}
				// Location fixed.. find the right place 
				// in the final order, irrespective of 
				// priority
				int bestPosition = expc;
				for(int i=0;i<beanFinalOrder.size();i++){
					CodeExpressionRef texp = (CodeExpressionRef)beanFinalOrder.get(i);
					if(texp.isStateSet(texp.STATE_SRC_LOC_FIXED)){
						if(texp.getOffset()>exp.getOffset()){
							bestPosition=i;	// insert at this place
						}else{
							// Do nothing as the expression is in the right place.
						}
					}
				}
				beanFinalOrder.insertElementAt(exp, bestPosition);
			}else{
				beanFinalOrder.add(expc, exp);
			}
		}
		tmpFinalOrder.addAll(beanFinalOrder);
	};
	
	//Now, push all the 'new's to the top, with the returned
	//'new' expression being at the top.
	int maxPriorityCount = 0;
	Iterator allExpressions = tmpFinalOrder.iterator();
	Vector finalOrder = new Vector(tmpFinalOrder.size());
	while(allExpressions.hasNext()){
		CodeExpressionRef exp = (CodeExpressionRef) allExpressions.next();
		if(((Integer)exp.getPriority()).intValue()==Integer.MAX_VALUE){
			if(exp.getBean()!=null &&
			   exp.getBean().getReturnedMethod()!=null &&
			   exp.getBean().getReturnedMethod().equals(this))
				finalOrder.insertElementAt(exp,0);
			else
				finalOrder.insertElementAt(exp,maxPriorityCount);
			maxPriorityCount++;
		}else{
			finalOrder.add(exp);
		}
	}

	String bestUsableFiller = firstReferenceFiller==null?null:new String(firstReferenceFiller);
	if(firstReferenceOffset==Integer.MAX_VALUE) {
		IType mainType = CodeGenUtil.getMainType(fModel.getCompilationUnit());
		IMethod fMethod = CodeGenUtil.getMethod(mainType, getMethodHandle());
		if(fMethod!=null){
		    ICodeGenSourceRange sr = ExpressionRefGenerator.getOffsetForFirstExpression(fMethod);
		    if (sr == null) throw new CodeGenException ("Could not find Offset") ;     //$NON-NLS-1$
			firstReferenceOffset=sr.getOffset() ;
			StringBuffer sb = new StringBuffer() ;
			for (int i = 0; i < sr.getLength(); i++) {
	            sb.append(' ') ;
	        }
    	    bestUsableFiller = sb.toString() ;   
		}
	}
	
	
	int bestUsablePosition = firstReferenceOffset;
	for(int c=0;c<finalOrder.size();c++){
		CodeExpressionRef exp = (CodeExpressionRef)finalOrder.get(c);
		if(exp.isStateSet(exp.STATE_SRC_LOC_FIXED)){
			if(bestUsablePosition<(exp.getOffset()+exp.getLen())){
				bestUsableFiller = new String(exp.getFillerContent());
				bestUsablePosition = exp.getOffset() + exp.getLen();
			}
		}else{
			if(bestUsableFiller!=null)
				exp.setFillerContent(bestUsableFiller);
			{// ALWAYS keep an expression which is floating after ALL
			  // the 'new' expressions in the method ! 
				Iterator itr = finalOrder.iterator();
				while(itr.hasNext()){
					CodeExpressionRef tmpexp = (CodeExpressionRef) itr.next();
					if(tmpexp.isStateSet(tmpexp.STATE_SRC_LOC_FIXED) && 
					   ((Integer)tmpexp.getPriority()).intValue()==Integer.MAX_VALUE &&
					   tmpexp.getOffset()>bestUsablePosition)
					   	bestUsablePosition = tmpexp.getOffset()+tmpexp.getLen();
				}
			}
			exp.setOffset(bestUsablePosition);
			bestUsablePosition += exp.getLen();
			exp.setState(exp.STATE_SRC_LOC_FIXED, true); //exp.setState(exp.getState() | exp.STATE_SRC_LOC_FIXED); // line is not readjusted again
		}
	}
	// Now you have all the expressions in the finalOrder list.
	fExpressions = finalOrder;
}


protected Enumeration determineBeanOrder(Enumeration beans){
	Vector parentCount = new Vector();
	SortedSet beanSorter = new TreeSet(getDefaultBeanOrderComparator());
	while(beans.hasMoreElements()){
		boolean added = beanSorter.add((BeanPart) beans.nextElement());
		if(!added)
			JBCFPlugin.log("No addition to the treeset", JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$
	};
	Vector order = new Vector();
	Iterator sorted = beanSorter.iterator();
	while(sorted.hasNext())
		order.add(sorted.next());
	return order.elements();
}

/**
 *  @return a negative integer, zero, or a positive integer as the
 * 	       first argument is less than, equal to, or greater than the
 *	       second.
 *  warning: Never return zero, as values are overwritten if equal.
 */
protected Comparator getDefaultBeanOrderComparator(){
	return new Comparator(){
		public int compare(Object o1, Object o2){
			BeanPart bp1 = (BeanPart) o1;
			BeanPart bp2 = (BeanPart) o2;
			int np1 = getImportanceCount(bp1, bp2);
			int np2 = getImportanceCount(bp2, bp1);
			if(np1 < np2){
				return -1;
			}else{
				if(np1 == np2){
					return 1;
				}else{
					return 1;
				}
			}
		}
		
		protected int getImportanceCount(BeanPart main, BeanPart subMain){
			if(isAChildOf(main, subMain))
				return Integer.MAX_VALUE;
			int parentCount = getParentCount(main);
			int constraintCount = getConstraintCount(main);
			return parentCount + constraintCount;
		}
		
		protected boolean isAChildOf(BeanPart parent, BeanPart reference){
			Iterator children = parent.getChildren();
			while(children.hasNext()){
				BeanPart child = (BeanPart) children.next();
				if(child.getSimpleName().equals(reference.getSimpleName()))
					return true;
				if(isAChildOf(child, reference))
					return true;
			}
			return false;
		}
		
		protected int getConstraintCount(BeanPart b){
			RefStructuralFeature sf = (RefStructuralFeature)b.getRefObject().refMetaObject().metaObject("components"); //$NON-NLS-1$
			if(sf!=null)
				return 1;
			if(CodeGenUtil.getConstraintFeature(b.getRefObject())!=null)
				return 1;
			return 0;
		}
		
		protected int getParentCount(BeanPart b){
			int pc = 0;
			int safetyCount = 1000;
			while(b.getContainer()!=null && safetyCount>0){
				pc++;
				safetyCount--;
				b = b.getContainer();
			}
			return pc;
		}
	};
}

/**
 *  @return a negative integer, zero, or a positive integer as the
 * 	       first argument is less than, equal to, or greater than the
 *	       second.
 *  warning: Never return zero, as values are overwritten if equal.
 */
protected Comparator getDefaultMethodComparator(){
	com.ibm.etools.cde.CDEHack.fixMe("SortedMap doesnt store equal objects - please investigate"); //$NON-NLS-1$
	return 
	new Comparator(){
		public int compare(Object e1, Object e2){
			CodeExpressionRef r1 = (CodeExpressionRef)e1;
			CodeExpressionRef r2 = (CodeExpressionRef)e2;
			int v1 = ((Integer)r1.getPriority()).intValue();
			int v2 = ((Integer)r2.getPriority()).intValue();
			int o1 = r1.getOffset();
			int o2 = r2.getOffset();
			if(o1<0)					// If the offsets are less than 0
				o1 = Integer.MAX_VALUE;		// then the expression is a new one.
			if(o2<0)					// Setting their offsets to MAX will
				o2 = Integer.MAX_VALUE;		// make the expression add itself at the end of the 
			if(v1<v2){					// list of similar priority expressions.
				return 1;  // Descending
			}else{
				if(v1==v2){ // Same Priority, look at offsets
					if(o1<o2){
						return -1; // Less Than
					}else{
						return 1; // Greater than
					}
				}else{
					return -1;  //Descending
				}
			}
		}
		public boolean equals(Object o){
			return super.equals(o);
		}
	};
}

public synchronized void aboutToDispose() {
	if (fdocListener != null)
	   fdocListener.disconnectFromDocument() ;	
}


public void disconnectFromDoc() {
    if (fdocListener != null)
       fdocListener.disconnectFromDocument() ;
    fdocListener = null ;
}

public synchronized void dispose() {
	disconnectFromDoc() ;
	
	if (fModel != null) {
		Iterator itr = fModel.getBeansInitilizedByMethod(this).iterator();
		while (itr.hasNext()) {
			BeanPart b = (BeanPart) itr.next();
			b.removeInitMethod(this);
			b.removeReturnMethod(this);
		}
	}
	
	if (fExpressions != null) {
	 // fExpression will be updated by the call to Exp.dispose() ;
	 Object[] tmp = fExpressions.toArray() ;
	 for (int i=0; i<tmp.length; i++) {
		((CodeExpressionRef)tmp[i]).dispose() ;
	 }
	}
	fExpressions = null ;
	if (fTypeRef != null)
	  fTypeRef.removeRefMethod(this) ;
	  	
	if (fModel != null)  
	   fModel.removeMethodRef(this) ;
	fTypeRef = null ;
	fdeclMethod = null ;
}

public synchronized void deleteFromComposition () {
	if (fExpressions != null) {
		Iterator itr = fExpressions.iterator() ;
		while (itr.hasNext()) {
			((CodeExpressionRef)itr.next()).deleteFromComposition() ;
		}
	}
}

public ICodeGenSourceRange getTargetSourceRange() {
    if (fModel == null) return null ;
    ISourceRange sr = fModel.getWorkingCopyProvider().getSharedSourceRange(getMethodHandle()) ;
    CodeGenSourceRange result = new CodeGenSourceRange(sr) ;
    if (sr != null)
        result.setLineOffset(fModel.getWorkingCopyProvider().getSharedLineNo(result.getOffset())) ;
    return result ;
}

public ICodeGenSourceRange getHighlightSourceRange() {
    if (fModel == null || fModel.getWorkingCopyProvider()==null) 
    	return null ;
    try{
	    ICompilationUnit cu = fModel.getWorkingCopyProvider().getSharedWorkingCopy();
	    IType[] types = cu.getTypes();
	    IMethod mtd = null;
	    for(int i=0;types!=null&&i<types.length;i++)
	    	if(CodeGenUtil.getMethod(types[i],getMethodHandle())!=null)
	    		mtd = CodeGenUtil.getMethod(types[i],getMethodHandle());
	    if(mtd==null)
	    	return null;
	    int start = mtd.getNameRange().getOffset();
	    int end = mtd.getSourceRange().getOffset()+mtd.getSourceRange().getLength();
	    start = cu.getSource().lastIndexOf('\n', start)+1;
	    CodeGenSourceRange result = new CodeGenSourceRange(start, end-start) ;
	    result.setLineOffset(fModel.getWorkingCopyProvider().getSharedLineNo(result.getOffset())) ;
	    return result ;
    }catch(JavaModelException e){
    	return null;
    }
}

public void setModel(IBeanDeclModel model) {
    fModel = model ;
}

public void refreshIMethod(){
	try{
	    try {
	      if (!fModel.getCompilationUnit().isConsistent()) fModel.getCompilationUnit().reconcile() ;
	    }
	    catch (JavaModelException e) {}
		IType mainType = CodeGenUtil.getMainType(fModel.getCompilationUnit());
		IMethod fMethod = CodeGenUtil.getMethod(mainType, getMethodHandle());
		if(fMethod==null)
			return;		
		if(fMethod!=null){
			setOffset(fMethod.getSourceRange().getOffset()) ;
			setContent(fMethod.getSource());
		}
		connectToDoc() ;
	}catch(Exception e){
		JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
	}
}
public String _debugExpressions() {
    
    StringBuffer sb = new StringBuffer() ;
    int mOffset = getOffset() ;
    String   doc = fModel.getDocument().get() ;
    Iterator itr = getExpressions() ;
    while (itr.hasNext()) {
        CodeExpressionRef exp = (CodeExpressionRef)itr.next() ;
        sb.append(exp.toString()+"\n\"") ; //$NON-NLS-1$
        if (!exp.isStateSet(exp.STATE_NOT_EXISTANT))
            sb.append(doc.substring(mOffset+exp.getOffset(),mOffset+exp.getOffset()+exp.getLen())) ;
        else
            sb.append("STATE_NOT_EXISTANT") ;
        sb.append("\"\n") ; //$NON-NLS-1$
    }
    return sb.toString() ;    
}

}