package com.ibm.etools.jbcf.codegen.util;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import java.util.*;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.compiler.ast.*;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.ui.IFileEditorInput;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.properties.*;
import com.ibm.etools.cde.properties.NameInCompositionPropertyDescriptor;
import com.ibm.etools.cdm.*;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EPackage;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.JavaURL;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.codegen.IDiagramModelInstance;
import com.ibm.etools.jbcf.codegen.java.ExpressionDecoderFactory;
import com.ibm.etools.jbcf.codegen.java.IExpressionDecoder;
import com.ibm.etools.jbcf.codegen.java.ITypeResolver;
import com.ibm.etools.jbcf.codegen.model.*;
import com.ibm.etools.jbcf.codegen.model.CodeMethodRef;
import com.ibm.etools.jbcf.codegen.model.IBeanDeclModel;


/**
 *  Assortments of utilities for Code Generation
 */

public class CodeGenUtil {
		

public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
private static com.ibm.etools.java.JavaHelpers _CompositionInstance=null ;



/**
 *  Extract the end resulted write method or public field access.
 */	
static public String getWriteMethod(Statement expr)	{
   if (expr == null) return null ;
   com.ibm.etools.cde.CDEHack.fixMe("must deal with non Literal also")	 ; //$NON-NLS-1$
   if (expr instanceof MessageSend)
       return new String (((MessageSend)expr).selector) ;
   else if (expr instanceof Assignment) {
       // public field access, e.g., gridBagConstraints.gridx = 0 ;
       if (((Assignment)expr).lhs instanceof QualifiedNameReference) {
           char[][] tokens = ((QualifiedNameReference)((Assignment)expr).lhs).tokens ;
          return new String (tokens[tokens.length-1]) ;
       }
   }
   else if (expr instanceof LocalDeclaration) {
       TypeReference tr = ((LocalDeclaration)expr).type ;
       if (tr instanceof QualifiedTypeReference)
           return "new " + tokensToString(((QualifiedTypeReference)tr).tokens) ; //$NON-NLS-1$
       else {
           com.ibm.etools.cde.CDEHack.fixMe("need to resolve") ; //$NON-NLS-1$
           return "new" ; //$NON-NLS-1$
       }
   }
   return null ;
}

public static boolean isExactlyPresent(String main, String target){
	if(main==null || target==null)
		return false;
	int index = main.indexOf(target);
	if(index<0)
		return false;
	index += target.length();
	if(index>=main.length())
		return true;
	// Any more characters present?
	if(Character.isJavaIdentifierPart(main.charAt(index)))
		return false;
	else
		return true;
}

/**
 *  Convert an array of token elements, to a '.' seperated String
 */
static public String tokensToString(char tokens[][]) {

    StringBuffer buf = new StringBuffer() ;	
	for (int i=0; i<tokens.length; i++) {
		if (i>0 && buf.length()>0) buf.append(".") ; //$NON-NLS-1$
		buf.append(tokens[i]) ;
	}
	return buf.toString() ;	
}


/**
 * Create and Java Object Instance for a given type
 */
public  static RefObject  createInstance (String instanceType,IDiagramModelInstance cm) throws CodeGenException {
  if (cm.getModelResourceSet() == null) throw new CodeGenException ("MOF is not set up") ; //$NON-NLS-1$
JBCFPlugin.log("CodeGenUtil.createInstance("+instanceType+")", JBCFMsgLogger.LOG_FINE) ;    //$NON-NLS-1$ //$NON-NLS-2$

  EClassifier iClass =  (EClassifier) cm.getModelResourceSet().getObjectAndLoad((new JavaURL(instanceType)).getFullString());
  RefObject inst = (RefObject) ((RefPackage) iClass.getEObjectContainer()).getFactory().create(iClass) ;
     
//  com.ibm.etools.cde.CDEHack.fixMe("need to use package/factory") ;
//  RefObject inst = ((JavaClassImpl)JavaClassImpl.reflect(instanceType,cm.getModelResourceSet())).eCreateInstance() ;		
  return inst ;
  
}

public static EClassifier getMetaClass (String qualifiedName, IDiagramModelInstance cm) {
     if (cm.getModelResourceSet() == null || qualifiedName == null) return null ;
     EClassifier iClass =  (EClassifier) cm.getModelResourceSet().getObjectAndLoad((new JavaURL(qualifiedName)).getFullString());	
     return iClass ;
}

/**
 * Create a "name in composition" annotation for the given obj, if a VisualInfo exists
 */
public static void addAnnotatedName(IJavaObjectInstance obj, String name, IDiagramModelInstance cm)
	throws CodeGenException {
	if (cm.getModelResourceSet() == null)
		throw new CodeGenException("MOF is not set up"); //$NON-NLS-1$

	Annotation a = CodeGenUtil.getAnnotation(obj);
	if (a != null) {
		KeyedString ks =
			((CDMPackage) RefRegister.getPackage(CDMPackage.packageURI)).getCDMFactory().createKeyedString();
		ks.setKey(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY);
		if (name != null && name.length() > 0)
			ks.setValue(name);
		a.setKeyedValue(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY, ks);
	}

}


/**
 * Get the one and only constraint feature for this project.
 * This is a one to one feature
 */
public static RefStructuralFeature getConstraintFeature(RefObject componentConstraint) {
   return  (RefStructuralFeature) componentConstraint.refMetaObject().metaObject("constraint") ;      // NO global constant at this time    //$NON-NLS-1$
}

/**
 * Add a Rectangle constraint on a target object - for none free formed objects
 * Constraints should hold x, y, width, and height, in that order.
 */
public static void addRactangleFeatureValue (RefObject target,int args[], RefStructuralFeature sf, IDiagramModelInstance cm) throws Exception {
  if (args.length != 4) throw new RuntimeException ("Invalid Args") ;  //$NON-NLS-1$
  
  IJavaObjectInstance value = (IJavaObjectInstance) createInstance("java.awt.Rectangle",cm) ;	 //$NON-NLS-1$
  value.setInitializationString("new java.awt.Rectangle("+ //$NON-NLS-1$
                                        Integer.toString(args[0]) +   "," + Integer.toString(args[1]) + "," + //$NON-NLS-1$ //$NON-NLS-2$
                                        Integer.toString(args[2])+"," + Integer.toString(args[3])+")") ;                                 //$NON-NLS-1$ //$NON-NLS-2$
  target.refSetValue(sf,value);    
}

/**
 * Add a generic constraint to a target object 
 */
public static void addConstraintString(RefObject target,String initVal, RefStructuralFeature sf, IDiagramModelInstance cm) throws Exception {
  
  IJavaObjectInstance value = null ;
  if (initVal != null) {
     value = (IJavaObjectInstance) createInstance("java.lang.String",cm) ;	 //$NON-NLS-1$
     value.setInitializationString(initVal) ;                                
  }
  target.refSetValue(sf,value);  	
}
/**
 * Add a generic constraint to a target object 
 */
public static void addConstraintInstance(RefObject target,IJavaObjectInstance val, RefStructuralFeature sf) throws Exception {
  
  target.refSetValue(sf,val);  	
}
/**
 * Add a Dimension constraint on a target object - for free form objects
 * Constraints should hold with and height, in that order.
 */
public static void addDimensionFeatureValue (RefObject target,int args[], RefStructuralFeature sf, IDiagramModelInstance cm) throws Exception {
  if (args.length != 2) throw new RuntimeException ("Invalid args") ; //$NON-NLS-1$
  
  IJavaObjectInstance value = (IJavaObjectInstance) createInstance("java.awt.Dimension",cm) ;	 //$NON-NLS-1$
  value.setInitializationString("new java.awt.Dimension("+Integer.toString(args[0])+ //$NON-NLS-1$
                                ","+Integer.toString(args[1])+")") ;                                 //$NON-NLS-1$ //$NON-NLS-2$
  target.refSetValue(sf,value);    
}

private static Hashtable   fComponentCache 	= null ;
private static ResourceSet fCacheRS  			= null ; // do not use directly as it can be reSetted mid stream

/**
 * Get the one and only component feature for this project.
 * This is a one to many feature
 */
public static RefStructuralFeature getComponentFeature(RefObject target) { 

   RefObject meta = target.refMetaObject() ;
   if (fCacheRS == null || fCacheRS != meta.getResourceSet()) {
   	fCacheRS = meta.getResourceSet() ;
   	fComponentCache = new Hashtable (20) ;
   }
   RefStructuralFeature sf = (RefStructuralFeature) fComponentCache.get(meta) ;
   if (sf == null) {
   	   sf = (RefStructuralFeature) target.refMetaObject().metaObject("components") ;      // NO global constant at this time       //$NON-NLS-1$       
   	   if (meta != null && sf != null)
   	      fComponentCache.put(meta,sf) ;
   }
   return sf ;
}

/**
 * Get the one and only Parent component feature for this project.
 * This is a one to many feature
 * @deprecated - parent componentContainer is not used anymove
 */
public static RefStructuralFeature getParentContainerFeature(RefObject target) {  
   return  (RefStructuralFeature) target.refMetaObject().metaObject("parentContainer") ;      // NO global constant at this time       //$NON-NLS-1$
}


public static RefObject  getNewCC(IDiagramModelInstance cm) {
   EClass ccClass = (EClass) cm.getModelResourceSet().getObjectAndLoad("plugin.com.ibm.etools.jbcf.visual/overrides/java/awt/containerVisuals.xmi#ConstraintComponent") ; //$NON-NLS-1$
   RefObject CC = (RefObject) ((RefPackage) ccClass.getEObjectContainer()).getFactory().create(ccClass) ;
   return CC ;
}

/**
 * Add a component into a container object at a specified index
 */
public static RefObject addComponentToContainer (RefObject cc, RefObject componentToAdd, RefObject targetContainer, int index, IDiagramModelInstance cm)
{ 
   // Create a ConstraintComponet object   
   RefObject CC = (cc == null) ? getNewCC(cm)  : cc ;
   
   CodeGenUtil.refSetValue(CC, 
                           (RefStructuralFeature) CC.refMetaObject().metaObject("component"), //$NON-NLS-1$
                           componentToAdd, -1) ;
			
   RefStructuralFeature cf = getComponentFeature(targetContainer) ;
   java.util.List compList = (java.util.List) targetContainer.refValue(cf);
   JBCFPlugin.log("CodeGenUtil.addComponentToContainer("+componentToAdd.refMetaObject()+","+targetContainer.refMetaObject()+"@"+index+")", JBCFMsgLogger.LOG_FINE) ;       //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
   if(index<0)
   	compList.add(CC) ;
   else
   	compList.add(Math.min(compList.size(),index), CC) ;
   	// Not used anymore
//   cf = getParentContainerFeature(CC) ;
//   if (cf != null)
//      CC.refSetValue(cf,targetContainer) ;

   return CC ;
}

/**
 *  Add an attribute,
 */
public static void addAttribute (RefObject target,RefStructuralFeature attrFeature,IJavaInstance sValue , String initString) throws CodeGenException {
	CDEHack.fixMe("FactorOut to exception method") ; //$NON-NLS-1$
	// Can potentially break the VM
	if (attrFeature.refID().equals("JFrame/contentPane")) //$NON-NLS-1$
	   initString = null ;
   if (sValue instanceof IJavaObjectInstance)
     ((IJavaObjectInstance)sValue).setInitializationString(initString) ;
JBCFPlugin.log("CodeGenUtil.addAttribute("+target.refMetaObject()+","+attrFeature.refID()+","+initString+")", JBCFMsgLogger.LOG_FINE) ;    //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
   com.ibm.etools.cde.CDEHack.fixMe("Temp fix to deal with setLayout(null)") ;   //$NON-NLS-1$
   if (initString == null)
       sValue = null ;
   target.refSetValue(attrFeature,sValue) ;	
}


/**
 * Get the main Type of a compilation unit
 */
public static IType getMainType(ICompilationUnit cu) {
	if(cu == null)
		return null;
	IJavaElement[] elements = null ;
	try {
	  elements = cu.getChildren() ;
	  if (elements != null) 
		for (int i=0; i<elements.length; i++) {
            com.ibm.etools.cde.CDEHack.fixMe("need to consider inner classes") ; //$NON-NLS-1$
			if (elements[i] instanceof IType) 
				return  (IType)elements[i] ;
		}	  
	}
	catch (JavaModelException e) {}
    return null ;		
}
/**
 *  Get the methods associated with the main type of a compilation unit
 */ 
public static IMethod[] getMethods(ICompilationUnit cu) {
	if(cu == null)
		return null;
	IType mt = getMainType(cu) ;
	try {
	 if (mt != null)	
	    return mt.getMethods() ;
	}
	catch (JavaModelException e) {}
    return null ;
}

/**
 *   CU's content might have changed, and an old method is stale.
 */
public static IMethod refreshMethod(IMethod method) {
	if (method == null) return method ;
	
	ICompilationUnit cu = method.getCompilationUnit() ;
	try {
	  if (!cu.isConsistent()) cu.reconcile() ;
	}
	catch (JavaModelException e) {}
	
	IType t = CodeGenUtil.getMainType(cu) ;
	return getMethod(t,method) ;
}

public static IMethod refreshMethod(String handle, ICompilationUnit cu) {
	if (handle == null) return null ;
	try {
	  if (!cu.isConsistent()) cu.reconcile() ;
	}
	catch (JavaModelException e) {}
	return getMethod(getMainType(cu),handle) ;
}

public static IField getField (String handle, ICompilationUnit cu) {
	if (handle == null) return null ;
	try {
	    if (!cu.isConsistent()) cu.reconcile() ;
		
    	IType t = getMainType(cu) ;
    	IField[] flds = t.getFields() ;
    	for (int i = 0; i < flds.length; i++) {
            IField iField = flds[i];
            if (iField.getHandleIdentifier().equals(handle)) 
               return iField ;
        }	
    }
	catch (JavaModelException e) {}
	return null ;
}

public static IField getFieldByName (String name, ICompilationUnit cu) {
	if (name == null || cu==null) return null ;
	try {
	    if (!cu.isConsistent()) cu.reconcile() ;
		
    	IType t = getMainType(cu) ;
    	IField[] flds = t.getFields() ;
    	for (int i = 0; i < flds.length; i++) {
            IField iField = flds[i];
            if (iField.getElementName().equals(name)) 
               return iField ;
        }	
    }
	catch (JavaModelException e) {}
	return null ;
}

public static IMethod getMethod (IType t, String handle) {
	
	try {
	 IMethod[] mt = t.getMethods() ;
	 for (int index=0;index<mt.length;index++) 
	  if (mt[index].getHandleIdentifier().equals(handle)) // getHandleIdentifier() is slow, consider a cache
	      return (mt[index]) ;
	}
	catch (JavaModelException e) {}
    return null ;	
}

public static IMethod getMethod (IType t, IMethod old) {
	
	try {
	 IMethod[] mt = t.getMethods() ;
	 for (int index=0;index<mt.length;index++) 
	  if (mt[index].isSimilar(old)) 
	      return (mt[index]) ;
	}
	catch (JavaModelException e) {}
    return null ;	
}

/**
 *  Update methods source offset
 */ 
public static void refreshMethodOffsets (IType t, IBeanDeclModel model) throws CodeGenException {
	try {
	 IMethod[] methods = t.getMethods() ;
	 for (int i=0; i<methods.length; i++) {
	   CodeMethodRef mr = model.getMethodInitializingABean(methods[i].getHandleIdentifier()) ;	  
	    if (mr != null)
	       mr.setOffset(methods[i].getSourceRange().getOffset()) ;
	 }
	}
	catch (JavaModelException e) {
	  	throw new CodeGenException (e) ;
	}
}


/**
 * 
 */ 
private static boolean isComponentContained (RefObject component, RefObject parent) throws CodeGenException {
   if (parent==null || component==null) return false ;
   if (component.equals(parent)) return true ;
   
   IExpressionDecoder decoder = ExpressionDecoderFactory.getDecoder((IJavaObjectInstance)parent) ;
   Iterator itr = decoder.getChildren((IJavaObjectInstance)parent).iterator() ;
   while (itr.hasNext()) {
   	RefObject child = (RefObject) itr.next() ;
   	itr.next() ; // Skip the SF
   	if (isComponentContained(component,child)) 
   	   return true ;
   }   
   return false ;   	
}
/**
 *  Update methods source offset
 */ 
public static boolean isComponentInComposition (RefObject component, IDiagramModelInstance model) throws CodeGenException {
	Iterator itr = model.getModelRoot().getComponents().iterator() ;
	while (itr.hasNext()) {
		if (isComponentContained(component,(RefObject)itr.next())) 
		   return true ;
	}
	return false ;	
}

/**
 *
 */

private static IJavaObjectInstance getParent (IJavaObjectInstance child, IJavaObjectInstance parent) throws CodeGenException {
   if (parent==null || child==null) return null ;
   if (parent.equals(child)) return null ;
   
   IExpressionDecoder decoder = ExpressionDecoderFactory.getDecoder((IJavaObjectInstance)parent) ;
   Iterator itr = decoder.getChildren((IJavaObjectInstance)parent).iterator() ;
   while (itr.hasNext()) {
   	IJavaObjectInstance c = (IJavaObjectInstance) itr.next() ;
   	itr.next() ; // Skip the SF
   	if (c.equals(child)) return parent ;
   	IJavaObjectInstance p = (getParent(child,c))  ;
   	if (p != null) return p ;
   }   
   return null ;   	
}

/**
 *  Get parent (implicit or not implicit) of this component
 */
public static IJavaObjectInstance getParent (IJavaObjectInstance child, IDiagramModelInstance model) throws CodeGenException {
	Iterator itr = model.getModelRoot().getComponents().iterator() ;
	while (itr.hasNext()) {
		IJavaObjectInstance p = getParent (child,(IJavaObjectInstance)itr.next()) ;
		if (p != null) return p ;
	}
	return null ;	
}




/**
 *  Check if the layout property is set to NULL
 */ 
public static boolean isContainerHasLayoutManager(RefObject container) {
	if (container == null) return false ;
	RefStructuralFeature sf = (RefStructuralFeature) container.refMetaObject().metaObject("layout") ; //$NON-NLS-1$
	Iterator itr = container.refCurrentValues(true).iterator() ;
	while (itr.hasNext()) {
	   Value val = (Value) itr.next() ;
	   if (val.getSF().equals(sf) && val.getValue() == null) return false ;	
	}
	return (true) ;
	
	
}
/**
 *  Get a containers top childrens
 *  @return List of children instances
 */
public static List getChildrenComponents (RefObject parent) {
    if (parent == null) return null ;
    List compList=null ;
    RefStructuralFeature sf = getComponentFeature(parent) ;
    if (sf != null) {
     try {
       compList = (List) parent.refValue(getComponentFeature(parent));
     }
     catch (Exception e) { 
     }
    }    
    // Look for out added part
    if (compList != null && compList.size()==0) return null ;
    return compList ;	
}
/**
 *  Get a component's container
 *  @return the container's instances
 */
public static RefObject getParentComponent (RefObject child) {
	
    if (child == null) return null ;
    // Not there anymore because a parent may use different features CC, Tabbed, NoConstraint etc.
    return null ;
//    RefObject obj = null ;
//    try {
//       obj  = (RefObject) child.refValue(getParentContainerFeature(child));
//    }
//    catch (Exception e) { 
//    }    
//    return obj ;
}

public static IJavaObjectInstance getCCcomponent(RefObject constraintComponent) {
	IJavaObjectInstance comp = (IJavaObjectInstance)
	                      constraintComponent.refValue((RefStructuralFeature) constraintComponent.refMetaObject().metaObject("component")) ; //$NON-NLS-1$
	return comp ;
}

public static IJavaObjectInstance getCCconstraint(RefObject constraintComponent) {
	try {
		IJavaObjectInstance cons = (IJavaObjectInstance) constraintComponent.refValue((RefStructuralFeature) constraintComponent.refMetaObject().metaObject("constraint")); //$NON-NLS-1$
		return cons;
	} catch (NotImplementedException e) {
	};
	return null;
}

/**
 *  Get an instance of a Composition class
 */
public static com.ibm.etools.java.JavaHelpers getCompositionInstance(RefObject component) {
	com.ibm.etools.cde.CDEHack.fixMe("multiple projects") ; //$NON-NLS-1$
	if (_CompositionInstance != null)
	   return _CompositionInstance ;
	_CompositionInstance = (com.ibm.etools.java.JavaHelpers)JavaClassImpl.reflect ("com.ibm.etools.ocm.Composition",component) ;	 //$NON-NLS-1$
	return _CompositionInstance ;
}

/**
 * Get the implicit feature
 */
public static RefStructuralFeature getImplicitFeature(RefObject target) {
   return  (RefStructuralFeature) target.refMetaObject().metaObject("implicit") ;       //$NON-NLS-1$
}

/**
 * Get the initialization String Feature
 */
public static RefStructuralFeature getInitStringFeature(RefObject target) {
   return  (RefStructuralFeature) target.refMetaObject().metaObject("initializationString") ;       //$NON-NLS-1$
}

/**
 * Returns the index of searchFor in searchIn, taking into 
 * account the presence of 
 * 
 * @param searchIn
 * @param seachFor
 * @return int
 */
public static int getExactJavaIndex(String searchIn, String seachFor){
	int index = -1;
	int indexEnd = -1;
	// prepare for [<spc>|\t|\r|\n]new[<spc>|\t|\r|\n] etc.
	while(searchIn.indexOf(seachFor, indexEnd)>-1){
		index = searchIn.indexOf(seachFor, indexEnd);
		indexEnd = index+seachFor.length();
		boolean startAllright = index==0 || (index>0 && !Character.isJavaIdentifierPart(searchIn.charAt(index-1)));
		boolean endAllright = indexEnd==searchIn.length() || (indexEnd<searchIn.length()&&!Character.isJavaIdentifierPart(searchIn.charAt(indexEnd+1)));
		if(startAllright && endAllright)
			return index;
	}
	return -1;
}

public static String getResolvedInitString(Statement stmt, ITypeResolver resolver){
	String resolved = null;
	if (stmt instanceof TypeReference && resolved==null) {
		TypeReference tr = (TypeReference) stmt;
		return resolver.resolve(CodeGenUtil.tokensToString(tr.getTypeName()));
	}
	if (stmt instanceof SingleNameReference) {
		SingleNameReference snr = (SingleNameReference) stmt;
		return resolver.resolve(new String(snr.token));
	}
	if(stmt instanceof AllocationExpression && resolved==null){
		AllocationExpression ae = (AllocationExpression) stmt;
		resolved = ae.toString();
		TypeReference str = ae.type;
		String type = CodeGenUtil.tokensToString(str.getTypeName());
		String resolvedType = getResolvedInitString(str,resolver);
		int newStart = getExactJavaIndex(resolved,"new"); //$NON-NLS-1$
		// prepare for new<spc>\t\r\n etc.
		int start = resolved.indexOf(type, newStart);
		int end = start+type.length();
		if(start>0 && end>0 && start<=resolved.length() && end<=resolved.length())
			resolved = resolved.substring(0,start) + resolvedType + resolved.substring(end, resolved.length());
	}
	if (stmt instanceof CastExpression && resolved==null) {
		CastExpression castExpr = (CastExpression) stmt;
		resolved = castExpr.toString();
		String castUnResolved = castExpr.type.toString();
		String castResolved = getResolvedInitString(castExpr.type, resolver);
		String expUnResolved = castExpr.expression.toString();
		String expResolved = getResolvedInitString(castExpr.expression, resolver);
		int castFrom = getExactJavaIndex(resolved,castUnResolved);
		int castTo = castFrom<0?-1:castFrom+castUnResolved.length();
		if(castFrom>-1 && castTo>-1 && castFrom<=resolved.length() && castTo<=resolved.length() && 
		   (!castUnResolved.equals(castResolved)))
			resolved = resolved.substring(0,castFrom)+castResolved+resolved.substring(castTo,resolved.length());
		int expFrom = getExactJavaIndex(resolved,expUnResolved);
		int expTo = expFrom<0?-1:expFrom+expUnResolved.length();
		if(expFrom>-1 && expTo>-1 && expFrom<=resolved.length() && expTo<=resolved.length() && 
		   (!expUnResolved. equals(expResolved)))
			resolved = resolved.substring(0,expFrom)+expResolved+resolved.substring(expTo,resolved.length());
	}
	if (stmt instanceof MessageSend) {
		MessageSend m = (MessageSend) stmt;
		StringBuffer ms = new StringBuffer (m.toString()) ;
		String r = m.receiver.toString() ;
		ms.replace(0,r.length(),resolver.resolve(r)) ;
		resolved = ms.toString() ;			
	}
	if (stmt instanceof Assignment && resolved==null) {
		Assignment assgn = (Assignment) stmt;
		resolved = assgn.toString();
		if(assgn.expression instanceof AllocationExpression)
			resolved = getResolvedInitString(assgn.expression, resolver);
		else if(assgn.expression instanceof MessageSend) 
			resolved = getResolvedInitString(assgn.expression, resolver);
		else if(assgn.expression instanceof CastExpression)
			resolved = getResolvedInitString(assgn.expression, resolver);
	}
	if (stmt instanceof LocalDeclaration && resolved==null) {
		LocalDeclaration ld = (LocalDeclaration) stmt;
		resolved = ld.toString();
		if(ld.initialization instanceof AllocationExpression ||
		   ld.initialization instanceof CastExpression ||
		   ld.initialization instanceof MessageSend){
			resolved = getResolvedInitString(ld.initialization, resolver);
		}
	}
	if(resolved==null)
		resolved = stmt.toString();
	return resolved;
}

public static boolean isConstraintComponentValue (Object val) {
	return (val != null && val instanceof RefObject && 
		  ((RefObject)val).refMetaObject().refName() != null && 
		  ((RefObject)val).refMetaObject().refName().equals("ConstraintComponent")) ; //$NON-NLS-1$
	
}

public static boolean isTabPaneComponentValue (Object val) {
	return (val != null && val instanceof RefObject && 
		  ((RefObject)val).refMetaObject().refName() != null && 
		  ((RefObject)val).refMetaObject().refName().equals("JTabComponent")) ; //$NON-NLS-1$
	
}

/**
 * Is this object a meta root object that agregate a JavaObjectInstance (e.g., CC)
 */
public static boolean isSpecialRootComponent(Object val) {
    
    boolean result = false ;
    if (val instanceof RefObject) {
	   result = isConstraintComponentValue(val) ||
	            isTabPaneComponentValue(val);
    }
    return result ;	    
}

/**
 * @return the annotation held by obj
 */
public static Annotation getAnnotation (RefObject obj) {

   // The opposite end is the feature for modelobject to annotation
   return (Annotation) obj.refValue(CDEUtilities.CDM_PACKAGE.getAnnotationEMF_Annotates().refOppositeEnd());	
}

/**
 * Create a new annotation structure for obj
 */
public static Annotation addAnnotation(RefObject obj) {
        CDMFactory fact = CDEUtilities.CDM_PACKAGE.getCDMFactory();
		AnnotationEMF annotation = fact.createAnnotationEMF();
		annotation.setAnnotates(obj);
		return annotation ;
}

/**
 *   This is a workaround to touch obj, up the container stack, so that the proxy adapter
 *   forces an invalidate on the VM.
 */
public static void snoozeAlarm(RefObject obj, ResourceSet rs) {
	CDEHack.fixMe() ;

    
	if (obj!=null && obj.refContainer() != null && (obj.refContainer() instanceof RefObject)) {
		RefObject container = (RefObject) obj.refContainer() ;
		RefStructuralFeature sf = obj.refContainerSF() ;
		// Do not go up if it is a Visual Edit Part (i.e, a component on the canvas)
		if (sf instanceof RefReference) return ;		
		JavaClass componentClass =  (JavaClass) rs.getObjectAndLoad((new JavaURL("java.awt.Component")).getFullString());          //$NON-NLS-1$
		if (componentClass.isInstance(obj)) return ;
		JBCFPlugin.log("snoozing "+obj,com.ibm.etools.jbcf.JBCFMsgLogger.LOG_FINEST) ; //$NON-NLS-1$
		if (sf.refIsMany()) {
		   List elements = (List) container.refValue(sf) ;
		   int index = elements.indexOf(obj) ;
		   elements.set (index, obj) ;
		}
		else {
			container.refSetValue(sf, obj) ;
		}
		snoozeAlarm(container,rs) ;		
	}
	
}

/**
 * Currently MOF move's of element from one container to another will not guarentee
 * an unset before the set notifications
 */
public static void refSetValue(RefObject obj, RefStructuralFeature sf, RefObject val, int index) {
	// First, unset a previous container, if needed.
	if (val !=null && val.refContainer()!= null && (val.refContainer() instanceof RefObject)) {
		RefStructuralFeature psf = val.refContainerSF() ;
		if (psf.refIsMany())
		   ((List)((RefObject)val.refContainer()).refValue(psf)).remove(val) ;
		else
		   ((RefObject)val.refContainer()).refUnsetValue(psf) ;		   		   
	}
	
	if (sf.refIsMany()) {
		List elements = (List) obj.refValue(sf) ;
		if (index>=0)
		   if(elements.contains(val))
		      elements.set(index,val) ;
	       else
	          elements.add(index,val) ;
		else
		   elements.add(val) ;
	}
	else
	   obj.refSetValue(sf, val) ;
}
/**
 * Generate a simple working copy provider, using the Compilation Unit as a reference.
 * This provider will only a small set of APIs.
 */
public static IWorkingCopyProvider getRefWorkingCopyProvider(ICompilationUnit refCU) {
	        final ICompilationUnit cu = refCU ;
	  	    IWorkingCopyProvider wcp = new IWorkingCopyProvider(){			
	  	    	CodegenTypeResolver resolver = new CodegenTypeResolver(getMainType(cu));
	  	    		  	    		  	    	
				public ICompilationUnit getSharedWorkingCopy(){return cu;}
				public ICompilationUnit getSharedWorkingCopy(boolean force){return cu;}
				public ICompilationUnit getLocalWorkingCopy(){return cu;}
				public IFileEditorInput getEditor(){return null;}
				public IFile getFile(){return null;}
				public IDocument getSharedDocument(){return null;}
				public IDocument getLocalDocument(){return null;}
				public Object    getLocalDocLock(){return null;}
				public void ReplaceWithLocalContent(IProgressMonitor pm, boolean commit) throws CodeGenException{;}
				public void UpdateDeltaToShared(ICancelMonitor pm, IDocumentListener docListener, List handles, boolean commit) throws CodeGenException{;}
				public void aboutToChangeShared(){;}
				public void changeCompleteShared(){;}
				public ISourceRange getSharedSourceRange(String handle){return null;}
				public int getSharedLineNo(int Offset){return -1;}
				public void dispose(){;}
				public String resolve(String unresolved){
					try{
						if(resolver==null)
							return unresolved;
						else
							return resolver.resolveTypeComplex(unresolved);
					}catch(Exception e){
						return unresolved;
					}
				}
				public String resolveThis(){
					return getMainType(cu).getTypeQualifiedName();
				}
		        public ITypeHierarchy getHierarchy() {
		          try {
		            IType t = getMainType(cu) ;
		            return t.newSupertypeHierarchy(null) ;
		          }
		          catch (org.eclipse.jdt.core.JavaModelException e) {}
		          return null ;
		        }
		        public void disconnect() {}
		        public void reconnect(IFile file) {}
	 	};
	 	return wcp ;
}
public static void clearCache () {
	fCacheRS = null ;
	fComponentCache = null ;
}
}


