package com.ibm.etools.jbcf.codegen.util;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.StringTokenizer;

import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.ElementChangedEvent;
import org.eclipse.jdt.internal.core.*;
import org.eclipse.jdt.internal.core.JarPackageFragmentRoot;

import com.ibm.etools.jbcf.*;

/**
 * @version 	1.0
 * @author
 */
public class CodegenTypeResolver implements IElementChangedListener{
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private IType fReferenceIType = null;
	private boolean isConnected = false;
	private Hashtable tempHash = null;
	
	public CodegenTypeResolver(IType referenceIType){
		fReferenceIType = referenceIType;
	}
	
	public void connect(){
		if(!isConnected){
			JavaCore.addElementChangedListener(this);
			isConnected = true;
		}
	}	/**
	 * Returns a null if not able to resolve, or
	 * returns a {"package","class"} parts if resolved.
	 * It DOES NOT resolve things like "Class.InnerClass".
	 * Resolves ONLY classes inside packages. It resolves
	 * "pkg.pkg1.Class.InnerClass".
	 */
	private String[] resolveSimpleType(String toResolve){
		try{
			String[][] types = fReferenceIType.resolveType(toResolve);
			if(types!=null && types.length>0)
				return types[0];
		}catch(JavaModelException e){
			return null;
		}
		return null;
	}
	
	/*
	 * P = package
	 * C = class
	 * V = variable
	 * Handles:
	 * (*) C1             >> P1.P2.C1        
	 * (*) P1.P2.C1       >> P1.P2.C1        
	 * (*) C1.C2          >> P1.P2.C1$C2     
	 * (*) P1.P2.C1.C2    >> P1.P2.C1$C2     
	 * (*) P1.P2.C1.C2.V1 >> P1.P2.C1$C2.V1  
	 * (*) C1.C2.V1       >> P1.P2.C1$C2.V1  
	 * 
	 */
	public String resolveTypeComplex(String toResolve){
		if(tempHash==null)
			tempHash = new Hashtable();
		if(tempHash.containsKey(toResolve))
			return (String) tempHash.get(toResolve);
		StringTokenizer dotTokenizer = new StringTokenizer(toResolve,"$.",false); //$NON-NLS-1$
		String finalResolved = new String(toResolve);
		String[] tokens = new String[dotTokenizer.countTokens()];
		int count = 0;
		while(dotTokenizer.hasMoreTokens()){
			tokens[count] = dotTokenizer.nextToken();
			count++;
		}
		
		// tokens first go into seensofar, and when decided
		// on the seperator go into correctedfinalresolved.
		String correctedFinalResolved = new String();
		String seenSoFar = new String();
		boolean lastTokenWasAClass = false;
		for(int i=0;i<tokens.length;i++){
			String token = tokens[i];
			seenSoFar = correctedFinalResolved.replace('$','.');
			if(i>0)
				seenSoFar = seenSoFar.concat("."); //$NON-NLS-1$
			seenSoFar = seenSoFar.concat(token);
			String[] ret = resolveSimpleType(seenSoFar);
			boolean tokenPresentInHierarchy = false;
			if(ret==null && lastTokenWasAClass)
				ret = checkInHierarchy(seenSoFar);
			if((ret==null || ret.length<1) && !tokenPresentInHierarchy){
				if(i>0)
					correctedFinalResolved = correctedFinalResolved.concat("."); //$NON-NLS-1$
				correctedFinalResolved = correctedFinalResolved.concat(token);
				lastTokenWasAClass = false;
			}else{
				String pkg = ret[0];
				String cls = ret[1];
				correctedFinalResolved = pkg+"."+cls.replace('.','$'); //$NON-NLS-1$
				lastTokenWasAClass = true;
			}
		}
		if(correctedFinalResolved!=null)
			finalResolved = correctedFinalResolved;
		if(finalResolved.startsWith(".")){ //$NON-NLS-1$
			finalResolved = finalResolved.substring(1,finalResolved.length());
		}
		tempHash.put(toResolve, finalResolved);
		return finalResolved;
	}
	
	/**
	 * Method checkInHierarchy.
	 * Calls to this are made ONLY when the previous class
	 * has been confirmed to be a class, and an inner class 
	 * checking is required. 
	 * 
	 * @param inner  fully resolved inner class name.
	 * @return String[]  null if not an inner class, else
	 * returns array of strings: {{package},{class}}. where
	 * class is either <class> or <class>.<class>
	 * 
	 */
	private String[] checkInHierarchy(String inner){
		try {
			String outerClassName = inner.substring(0,inner.lastIndexOf('.'));
			String innerClassName = inner.substring(inner.lastIndexOf('.')+1,inner.length());
			IType outerClassType = fReferenceIType.getJavaProject().findType(outerClassName);
			if(outerClassType==null)
				return null;
			return traverseHierarchy(outerClassType, innerClassName);
		} catch (JavaModelException e) {
		}
		return null;
	}
	
	private String[] traverseHierarchy(IType outerType, String innerClassName){
		try {
			
			String[] ret = resolveSimpleType(outerType.getFullyQualifiedName()+"."+innerClassName); //$NON-NLS-1$
			if(ret!=null)
				return ret;	

			// Go through the interfaces first
			String[] implementsNames = outerType.getSuperInterfaceNames();
			if(implementsNames!=null && implementsNames.length>0){
				for(int i=0;i<implementsNames.length;i++){
					IType implementType = null;
					if(outerType instanceof SourceType){
						String[][] tmp = outerType.resolveType(implementsNames[i]);
						if(tmp==null || tmp.length<1)
							continue;
						String implement[] = tmp[0];
						implementType = fReferenceIType.getJavaProject().findType(implement[0],implement[1]);
					}else{ // Binary type
						implementType = fReferenceIType.getJavaProject().findType(implementsNames[i]);
					}
					if(implementType!=null){
						ret = traverseHierarchy(implementType,innerClassName);
						if(ret!=null)
							return ret;
					}
				}
			}

			// Go through the super hierarchy next
			String superName = outerType.getSuperclassName();
			if(superName!=null){
				IType superType = null;
				if(outerType instanceof SourceType){
					String[][] tmp = outerType.resolveType(superName);
					if(tmp!=null || tmp.length>0){
						String supers[] = tmp[0];
						superType = fReferenceIType.getJavaProject().findType(supers[0],supers[1]);
					}
				}else{ // Binary type
					superType = fReferenceIType.getJavaProject().findType(superName);
				}
				if(superType!=null){
					ret = traverseHierarchy(superType,innerClassName);
					if(ret!=null)
						return ret;
				}
			}
		} catch (JavaModelException e) {
		}
		return null;
	}
	
	/*
	 * @see IElementChangedListener#elementChanged(ElementChangedEvent)
	 */
	public void elementChanged(ElementChangedEvent event) {
		if(tempHash!=null)
			tempHash.clear();
	    com.ibm.etools.cde.CDEHack.fixMe("Temporary") ; //$NON-NLS-1$
	    com.ibm.etools.jbcf.codegen.java.AttributeFeatureMapper.clearCache() ;
	    com.ibm.etools.jbcf.codegen.java.ExpressionDecoderFactory.clearCache() ;
		CodeGenUtil.clearCache()  ;
		com.ibm.etools.jbcf.codegen.java.AttributeFeatureMapper.clearCache() ;
		com.ibm.etools.jbcf.codegen.java.rules.InstanceVariableCreationRule.clearCache() ;
		com.ibm.etools.jbcf.codegen.java.rules.InstanceVariableRule.clearCache() ;
	}	
	
	public void disconnect() {
		if(isConnected){
			JavaCore.removeElementChangedListener(this);
			isConnected = false;
		}
	}
}
