package com.ibm.etools.jbcf.codegen.util;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;

import org.eclipse.jdt.core.compiler.*;
import org.eclipse.jdt.core.compiler.InvalidInputException;
import org.eclipse.jdt.internal.compiler.parser.Scanner ;

public class ExpressionParser {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	String fSource ;
	
	int    fSourceOff ;   // not including a filler, or comments
	int    fSourceLen ;
	
	int    fExpOff = -1 ; // The works
	int    fExpLen = -1 ;
	
	int    fCommentsOff = -1 ; 
	int    fCommentsLen = -1 ;
	
	int    fFillerOff = -1 ; // Prefixed  white space
	int    fFillerLen = -1 ;
	
	
	
/**
 *  @param sourceSnippet the source code body where the expression is part off
 *  @param expOffset  where the expression java source starts (    setFoo() ;)
 *                                                                 ^      ^
 *  @param expLen     the length of the source .
 */	
public ExpressionParser(String sourceSnippet, int expOffset, int expLen) {
		fSource = sourceSnippet ;
		fSourceOff = expOffset ;
		fSourceLen = expLen ;
}


protected void clear() {
	fFillerOff = fFillerLen = fCommentsOff = fCommentsLen = fExpOff = fExpLen = -1 ;
}


public void replaceCode(String code) {
	StringBuffer st = new StringBuffer (fSource) ;
	st.replace(getCodeOff(),getCodeOff()+getCodeLen(),code) ;
	fSource=st.toString() ;
	fSourceLen = code.length() ;
	clear() ;
}

public void replaceComments(String comment) {
	StringBuffer st = new StringBuffer (fSource) ;
	st.replace(getCommentOff(),getCommentOff()+getCommentLen(),comment) ;
	fSource=st.toString() ;
	clear() ;
}

public void replaceFiller (String filler) {
	int delta = filler.length() - getFillerLen() ;
	StringBuffer st = new StringBuffer (fSource) ;
	st.replace(getFillerOff(),getFillerOff()+getFillerLen(),filler) ;	
	fSource = st.toString() ;
	fSourceOff+= delta ;
    clear() ;
}

public String getCode() {
	return fSource.substring(fSourceOff,fSourceOff+fSourceLen) ;
}

public int getCodeOff() {
	return fSourceOff ;
}

public int getCodeLen() {
	return fSourceLen ;
}

protected int getLineStartOff() {
	int left = fSourceOff ;
	// Include the filler on the left (spaces, tabs
	while ((left-1)>=0) {
		char ch = fSource.charAt(left-1) ;
		if (Character.isWhitespace(ch) && ch != '\r' && ch != '\n') {
			left -- ;
			continue ;
		} 
		break ;
	}
	if (left == 0 || fSource.charAt(left-1) == '\r' || fSource.charAt(left-1) == '\n')
	  // First exp. on the line
 	  return left ;	
    else
      // The filler belongs to the previous expression
      return fSourceOff ;
}	


protected int advanceLineSeperator(int position) {
	int index = position ;
	while (index<fSource.length() && 
	   	   (fSource.charAt(index) == '\r' || fSource.charAt(index) == '\n'))
	   	    index ++ ;	
	return index ;
}

protected int reverseLineSeperator(int position) {
	int index = position-1 ;
	while (index>0&& 
	   	   (fSource.charAt(index) == '\r' || fSource.charAt(index) == '\n'))
	   	    index -- ;	
	return index+1 ;
}

/**
 *  @return String represending the second Identifier e.g., ivjFoo.setBar()
 *                                                                 ^    ^
 */
public String getSelectorContent() {
	String code = getCode() ;
	Scanner scanner = new Scanner() ;
	scanner.setSource(code.toCharArray()) ;
	scanner.recordLineSeparator = true ;
	scanner.tokenizeWhiteSpace = true ;
	scanner.tokenizeComments = true ;
	
	int token ;
	String prevIdentifier=null ;
	try {
	 token = scanner.getNextToken() ;
	 while (token != scanner.TokenNameEOF &&
	        token != scanner.TokenNameLPAREN &&
	        token != scanner.TokenNamenew) {
	       
	       if (token == scanner.TokenNameIdentifier)
	         if (prevIdentifier == null)
	            prevIdentifier = new String(scanner.getCurrentTokenSource()) ;
	         else
	            break ;
	       
	       token = scanner.getNextToken() ;
	 }
	}
	catch (InvalidInputException e) {
		return null;
	}
	
	if (token == scanner.TokenNameIdentifier || token == scanner.TokenNamenew)
	   return new String (scanner.getCurrentTokenSource()) ;
	else
	   if (prevIdentifier != null)
	      return prevIdentifier ;
	   else
	      return null ;
}

protected int skipSemiColonifNeeded(int right) {
    {
    	// The range from fSourceOff to fSourceOff+fSourceLength could
    	// hold Comments in them - so strip the comments off.
    	
    	String targetSrc = fSource.substring(fSourceOff,right);
		Scanner scanner = new Scanner() ;
		scanner.setSource(targetSrc.toCharArray()) ;
		scanner.recordLineSeparator = true ;
		scanner.tokenizeWhiteSpace = true ;
		scanner.tokenizeComments = true ;
		
		int semicolonFound = -1;
		try{
			int token = scanner.getNextToken();
			for(int i=0;i<targetSrc.length();i++){
				if(token==Scanner.TokenNameEOF)
					break;
				if(token==Scanner.TokenNameSEMICOLON)
					return right ;
				token = scanner.getNextToken();
			}
		}
		catch(InvalidInputException e){
			com.ibm.etools.jbcf.JBCFPlugin.log(e,JBCFMsgLogger.LOG_FINE) ;
		}		
    }
    int index = fSource.substring(right).indexOf(';') ;
    return index<0 ? right : right+index+1 ;
    
}
	
protected void primParseExpression() {
		
	int left = getLineStartOff() ;
	
	primParseComment() ;
	
	int right = fCommentsOff+fCommentsLen ;
	if (right < 0)	// No Comments
	  right = fSourceOff+fSourceLen ;
	  
	right = skipSemiColonifNeeded(right) ;
	
	// Skip white space until EOL
	  
	Scanner scanner = new Scanner() ;
	scanner.setSource(fSource.substring(right).toCharArray()) ;
	scanner.recordLineSeparator = true ;
	scanner.tokenizeWhiteSpace = true ;
	scanner.tokenizeComments = true ;
	
	try {
	 int token = scanner.getNextToken() ;	 
	 int prevToken = -1, prevStart = -1 ;
	 while (token == scanner.TokenNameWHITESPACE) {
	     prevToken = token ;
	     prevStart = scanner.startPosition ;
	     token = scanner.getNextToken() ;
	 }
	     		   
	 com.ibm.etools.cde.CDEHack.fixMe("workaround: the Scanner will not record Line Seperators if it tokenizes white space")  ; //$NON-NLS-1$
     int overide = -1 ;
	 if (prevToken == scanner.TokenNameWHITESPACE) {
	 	int index ;
	 	for (index=prevStart; index<scanner.startPosition; index++) 
	 	   if (fSource.charAt(right+index) == '\r' || fSource.charAt(right+index) == '\n') {
	 	   	overide = right+index ;
	 	   	break ;
	 	   }
	 	if (overide>=0) 
	 	   overide = advanceLineSeperator(overide) ;
	 }	 
	 
	 if (scanner.lineEnds.length > 0 && scanner.getLineEnd(1)>0) { 
	 	// We advanced to the next line/s
	   right += scanner.getLineEnd(1) ;
	   right = advanceLineSeperator(right) ;
	 }
	 else if (token == scanner.TokenNameEOF) {
	 	right += scanner.currentPosition-1 ;
	 }
	 else {
	 	// We have more than one expression on a given line, keep the white space
	 	if (overide > 0)
	 	   right = overide ;
	 	else
	 	   right += scanner.startPosition-1 ;
	 }
	 
	 
	 
	 fFillerOff=fExpOff=left ;
	 fFillerLen=fSourceOff-fFillerOff ;
	 fExpLen=right-left ;

    } 
    catch (InvalidInputException e) {
    	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
    }
}

/**
 * @return String denoting the expression including the filler and comments
 * @param boolean includeEOL noting if to include the line seperator at the end of
 *                           the expression, if one exists.
 */

public String getExpression(boolean includeEOL) {
	if (fExpLen < 0 || fExpOff < 0)
	   primParseExpression() ;
	if (fExpLen < 0 || fExpOff < 0)
	   return null ;
	else {
	   int right = fExpOff+fExpLen ;
	   if (!includeEOL) 
	      right = reverseLineSeperator(right) ;
	   return fSource.substring(fExpOff,right) ;
	}
}

public int getExpressionOff() {
	if (fExpLen < 0 || fExpOff < 0)
	   primParseExpression() ;
	if (fExpLen < 0 || fExpOff < 0)
	   return -1 ;
	else 
	   return fExpOff ;
}


public int getExpressionLen(boolean includeEOL) {
	if (fExpLen < 0 || fExpOff < 0)
	   primParseExpression() ;
	if (fExpLen < 0 || fExpOff < 0)
	   return -1 ;
	else {
	   int right = fExpOff+fExpLen ;
	   if (!includeEOL) 
	      right = reverseLineSeperator(right) ;
	   return right-fExpOff ;
	}
}

public int getExpressionLen() {
	return getExpressionLen(true) ;
}


/**
 * @return String denoting the expression inculing the filler comments and EOL
 */
public String getExpression() {
	 return getExpression(true) ;
}	


protected void primParseComment() {
 
	Scanner scanner = new Scanner() ;
	int scanOff = fSourceOff+fSourceLen ;
	scanner.setSource(fSource.substring(scanOff).toCharArray()) ;
	scanner.recordLineSeparator = true ;
	scanner.tokenizeComments = true ;
	
	try {
	 int token = scanner.getNextToken() ;	 
	 while (token == scanner.TokenNameSEMICOLON ||	
	        token == scanner.TokenNameWHITESPACE) 
	     token = scanner.getNextToken() ;
	 
     if (
         ((token == scanner.TokenNameCOMMENT_LINE) &&
          (scanner.getLineEnd(1)==0 || (scanner.getLineEnd(1)>0 && scanner.currentPosition == scanner.getLineEnd(1)-1))          
         )  ||
         ((token == scanner.TokenNameCOMMENT_BLOCK) &&
     	  (scanner.lineEnds.length == 0 || scanner.getLineEnd(1)<=0 ||     	
     	     scanner.startPosition < scanner.getLineEnd(1)) 
     	 )) {
     	     // Comment starts on our line
	 	     fCommentsOff = scanOff+scanner.commentStarts[0] ;
       	     fCommentsLen = scanOff+scanner.currentPosition-fCommentsOff ;
	 }	      
    } 
    catch (InvalidInputException e) {
    	JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING) ;
    }	 
}
	
/**
 * @return String denoting the comment portion at the end of the expression
 */	
public String getComment() {

  if (fCommentsOff<0) 
     primParseComment() ;
  
  if (fCommentsOff>0) {
  	return fSource.substring(fCommentsOff,fCommentsOff+fCommentsLen) ;
  }
  else return null ;
}	

public int getCommentOff() {
	if (fCommentsOff<0) 
        primParseComment() ;
  
    if (fCommentsOff>0) 
       return fCommentsOff ;
    else
       return -1 ;
}

public int getCommentLen() {
	if (fCommentsLen<0) 
        primParseComment() ;
  
    if (fCommentsLen>0) 
       return fCommentsLen ;
    else
       return -1 ;
}


/**
 * @return String denoting the white space preceeding the expression
 */	
public String getFiller() {
   if (fFillerOff < 0)  {
      fFillerOff = getLineStartOff() ;
      fFillerLen=fSourceOff-fFillerOff ;
   }
   if (fFillerOff < 0 || fFillerLen < 0)
	   return null ;
	else if (fFillerLen == 0) return "" ; //$NON-NLS-1$
	else
	   return fSource.substring(fFillerOff,fFillerOff+fFillerLen) ;         
}

public int getFillerOff() {
   if (fFillerOff < 0)  {
      fFillerOff = getLineStartOff() ;
      fFillerLen=fSourceOff-fFillerOff ;
   }
   if (fFillerOff < 0 || fFillerLen < 0)
	   return -1 ;
	else
	   return fFillerOff ;
}
public int getFillerLen() {
   if (fFillerLen < 0)  {
      fFillerOff = getLineStartOff() ;
      fFillerLen=fSourceOff-fFillerOff ;
   }
   if (fFillerOff < 0 || fFillerLen < 0)
	   return -1 ;
	else
	   return fFillerLen ;
}

public String toString() {
	// Debugger may get nested here,
	if (fExpOff>0 && fExpLen>0)
	  return getExpression() ;
	else
	  return super.toString() ;
}


	
}


	

