package com.ibm.etools.jbcf.editorpart;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.text.MessageFormat;
import java.util.*;

import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditorActionContributor;
import org.eclipse.jface.action.*;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.*;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.actions.*;
import org.eclipse.ui.internal.*;
import org.eclipse.ui.texteditor.*;
import org.eclipse.ui.texteditor.ITextEditorExtension;
import org.eclipse.ui.texteditor.StatusLineContributionItem;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.palette.*;
import com.ibm.etools.gef.palette.*;
import com.ibm.etools.gef.tools.CreationTool;
import com.ibm.etools.gef.ui.actions.ActionRegistry;
import com.ibm.etools.gef.ui.palette.PaletteViewer;
import com.ibm.etools.gef.ui.palette.PaletteViewerImpl;
import com.ibm.etools.jbcf.CustomizeJavaBeanAction;
import com.ibm.etools.jbcf.visual.AlignmentWindowAction;

public class JavaVisualEditorActionContributor extends CompilationUnitEditorActionContributor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	
	protected IEditorPart fLastEditorPart;
	protected PaletteViewerImpl fLastPaletteViewer;
	protected List fItemsCreated = new ArrayList(0);
	protected PaletteListener fPaletteListener;
	protected RetargetAction fDeleteAction;
	protected Action fSelectedToolEntryAction;
	protected RetargetAction fUndoAction;
	protected RetargetAction fRedoAction;
	protected AlignmentWindowAction fAlignmentWindowAction;
	protected IPartListener fPartListener;
	
	// The following will add the JVE status fields in the action bar
	public final static String STATUS_CATEGORY_SYNC_STATUS = "JVE_SyncStatus" ;   //$NON-NLS-1$
	public final static String STATUS_MSG_SYNC_STATUS_INSYNC = CodegenEditorPartMessages.getString("JVE_STATUS_MSG_INSYNC") ;  //$NON-NLS-1$
	public final static String STATUS_MSG_SYNC_STATUS_OUTOFSYNC = CodegenEditorPartMessages.getString("JVE_STATUS_MSG_NOT_IN_SYNC") ;  //$NON-NLS-1$
	public final static String STATUS_MSG_SYNC_STATUS_SYNCHING = CodegenEditorPartMessages.getString("JVE_STATUS_MSG_SYNCHRONIZING") ;  //$NON-NLS-1$
	public final static String STATUS_MSG_SYNC_STATUS_REALOAD = CodegenEditorPartMessages.getString("JVE_STATUS_MSG_RELOAD") ;  //$NON-NLS-1$
	public final static String STATUS_MSG_SYNC_STATUS_PAUSE = CodegenEditorPartMessages.getString("JVE_STATUS_MSG_PAUSE") ;  //$NON-NLS-1$
	public final static String STATUS_MSG_SYNC_STATUS_PARSE_ERROR = CodegenEditorPartMessages.getString("JVE_STATUS_MSG_ERROR") ; //$NON-NLS-1$
	
	public final static String STATUS_CATEGORY_SYNC_ACTION = "JVE_SyncAction" ;   //$NON-NLS-1$
	private Map fJVEStatusFields;
	
		
public JavaVisualEditorActionContributor(){
	fPaletteListener = new PaletteListener(){
		public void entrySelected(PaletteEvent event){
			if (event.getEntry() instanceof PaletteEntry){
				paletteEntrySelected((PaletteEntry)event.getEntry());
			} else {
				paletteEntrySelected(null);
			}
		}
	};
	// The delete action is retargeted so that it works with the JavaBeans viewer and the edit part
	// and also it actually works with the source viewer and the outline list of methods and fields
	fDeleteAction = new LabelRetargetAction(org.eclipse.ui.IWorkbenchActionConstants.DELETE,CodegenEditorPartMessages.getString("Action.Delete")); //$NON-NLS-1$
	fDeleteAction.setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_DELETE_EDIT_HOVER));
	fDeleteAction.setImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_DELETE_EDIT));
	fDeleteAction.setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_DELETE_EDIT_DISABLED));
	fDeleteAction.setEnabled(false);
	markAsPartListener(fDeleteAction);
	
	fUndoAction = new LabelRetargetAction(IWorkbenchActionConstants.UNDO,CodegenEditorPartMessages.getString("Action.Undo")); //$NON-NLS-1$
	fUndoAction.setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_UNDO_EDIT_HOVER));
	fUndoAction.setImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_UNDO_EDIT));
	fUndoAction.setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_UNDO_EDIT_DISABLED));
	fUndoAction.setEnabled(false);
	markAsPartListener(fUndoAction);
	
	fRedoAction = new LabelRetargetAction(IWorkbenchActionConstants.REDO,CodegenEditorPartMessages.getString("Action.Redo")); //$NON-NLS-1$
	fRedoAction.setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_REDO_EDIT_HOVER));
	fRedoAction.setImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_REDO_EDIT));
	fRedoAction.setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_REDO_EDIT_DISABLED));
	fRedoAction.setEnabled(false);
	markAsPartListener(fRedoAction);
	
	fJVEStatusFields= new HashMap(2);
	fJVEStatusFields.put(STATUS_CATEGORY_SYNC_STATUS, new StatusLineContributionItem(STATUS_CATEGORY_SYNC_STATUS));
	fJVEStatusFields.put(STATUS_CATEGORY_SYNC_ACTION, new JVEActionStatusLineContributionItem(STATUS_CATEGORY_SYNC_ACTION));
		
}
public void contributeToToolBar(IToolBarManager tbm) {
	super.contributeToToolBar(tbm);	
	tbm.add(fDeleteAction);
	tbm.add(fUndoAction);
	tbm.add(fRedoAction);
}

private void addStatusFields(IEditorPart part) {
	if (part instanceof JavaVisualEditorPart) {
		JavaVisualEditorPart extension = (JavaVisualEditorPart) part;
		extension.setStatusField(
			(IStatusField) fJVEStatusFields.get(STATUS_CATEGORY_SYNC_STATUS),
			STATUS_CATEGORY_SYNC_STATUS);
		extension.setStatusField(
			(IJVEActionField) fJVEStatusFields.get(STATUS_CATEGORY_SYNC_ACTION),
			STATUS_CATEGORY_SYNC_ACTION);
	}
}

/**
 * Make sure the JavaEditor and TextEditor actions think that the JavaEditor is
 * the active part so they can work with it
 * We also get actions from the JVE editor such as Delete, Undo, Redo
 * as well as the palette
 */
public void setActiveEditor(IEditorPart part) {

	// Either it is JavaVisualEditorPart or it is null, in which case we are deactivating.
	super.setActiveEditor(part != null ? ((JavaVisualEditorPart)part).getJavaEditor() : null);
	
	// If the part is the same one we are already working with then just exit - this saves on having to remove
	// and re-create all the same toolbar actions we already have.
	// Or we are going away (which is what null means). We should clean up in dispose.
	if ( part == null ) return;

	if ( part == fLastEditorPart ) {
		IStructuredSelection selection = (IStructuredSelection) ((JavaVisualEditorPart)part).getVCEEditor().getPrimaryViewer().getSelection();	
		fAlignmentWindowAction.setEditorPart(((JavaVisualEditorPart)part).getVCEEditor(),selection);
		return;
	}
		
	IToolBarManager tbm = getActionBars().getToolBarManager();
	// We have a new editor or we are going away.  Remove all of the items created before
	Iterator iter = fItemsCreated.iterator();
	while(iter.hasNext()){
		Object action = iter.next();
		tbm.remove(((IAction)action).getId());
		// If the action is a retarget action then it will be a page listener and needs removing
		if ( action instanceof RetargetAction ) {
			getPage().removePartListener((RetargetAction)action);
		}
	}
			
	
	// Get the edit domain from the active part
	// If this is the edit part it knows how to return this, and the BeansList
	// defers over to the edit part that created it
	EditDomain editDomain = (EditDomain)part.getAdapter(EditDomain.class);			
	fItemsCreated = new ArrayList();		
	if (fLastEditorPart != null ) {
		fLastPaletteViewer.removePaletteListener(fPaletteListener);
	}	
	// We have a JavaVisualEditorPart - create the toolbar actions for it
	if ( part instanceof JavaVisualEditorPart ) {

		// Listen to the palette on the new viewer
		PaletteViewerImpl newPaletteViewer = (PaletteViewerImpl)editDomain.getData(PaletteViewer.class);
		fLastPaletteViewer = newPaletteViewer;
		newPaletteViewer.addPaletteListener(fPaletteListener);
		// Put the delete action on the toolbar
		IEditorPart oldEditorPart = fLastEditorPart;
		fLastEditorPart = part;
		// DeleteAction is done with a RetargetAction that is redirected towards the active part
		// This allows it to work with the Java Beans viewer as well as the GUI edit parts
		ActionRegistry actionRegistry = (ActionRegistry)part.getAdapter(ActionRegistry.class);
		IAction deleteAction = actionRegistry.getAction(org.eclipse.ui.IWorkbenchActionConstants.DELETE);
		getActionBars().setGlobalActionHandler(org.eclipse.ui.IWorkbenchActionConstants.DELETE, deleteAction);
		tbm.add(new Separator());
		JavaVisualEditorPart jvePart = (JavaVisualEditorPart)part;
		// Do the palette entries.  These are got by asking the JavaVisualEditor part for its toolbar entries
		// and adding them, but also recording them in the fItemsCreated so they can be removed
		Iterator paletteToolActions = jvePart.getPaletteToolbarActions().iterator();
		while ( paletteToolActions.hasNext() ) {
			Action paletteToolAction = (Action)paletteToolActions.next();
			// So that the palette actions work with the JavaVisualEditor AND the BeansList turn them into
			// RetargetActions and add a global action handler
			RetargetAction retargetAction = getRetargetAction(paletteToolAction);
			// The action is stored in the list of remembered actions so it can be removed later
			fItemsCreated.add(retargetAction);
			// It is added to the toolbar
			tbm.add(retargetAction);
			// The action is made a listener to the workbench page so it knows when parts are being activated and 
			// de-activated and fetch actions accordingly from the JavaVisualEditor and BeansList
			// To do this global action handlers must be created.  The JVE one is done here and the BeansList is done
			// when it creates itself
			getActionBars().setGlobalActionHandler(retargetAction.getId(),paletteToolAction);
			getPage().addPartListener(retargetAction);
		}
		// Having added the palette items show the currently active entry on the status bar and in a checked state
		paletteEntrySelected(newPaletteViewer.getSelectedEntry());		
		// Don't show the grid stuff for now because the toolbar gets too wide and also it only works for free
		// form alignment - it needs to be coded for components inside a container that is null layout
		// before it becomes really useful for the JVE
//		tbm.add(new Separator());
//		processVCEAction(ShowGridAction.ACTION_ID,oldEditorPart);		
//		processVCEAction(GridPropertiesAction.ACTION_ID,oldEditorPart);
//		processVCEAction(SnapToGridAction.ACTION_ID,oldEditorPart);					
		// After putting the palette items put the alignment actions
		// For now just get the VCE actions and slam them onto the toolbar
		// although this needs more design thought so we cope with different
		// layout managers, etc... probably with a button that shows a window of actions
		// that are relevent for the selection component's parent layout managers
		tbm.add(new Separator());		
		processVCEAction(CustomizeJavaBeanAction.ACTION_ID,oldEditorPart);

		// The action to open the window with alignment controls is retrieved one and cache'd and then\
		// given the new editor part.  This is so that it can remain open and just unhook and re-hook itself to
		// the editor part
		if(fAlignmentWindowAction == null){
			fAlignmentWindowAction = (AlignmentWindowAction)actionRegistry.getAction(AlignmentWindowAction.ACTION_ID);
			// Set the selection to be the selection from the current part
			fAlignmentWindowAction.setSelection((IStructuredSelection) jvePart.getVCEEditor().getPrimaryViewer().getSelection());
			fItemsCreated.add(fAlignmentWindowAction);
			getActionBars().getToolBarManager().add(fAlignmentWindowAction);
			
			// Having created the alignment window action we must listen to part activation and deactivation
			fPartListener = new IPartListener(){
				public void partActivated(IWorkbenchPart part){
				}
				public void partDeactivated(IWorkbenchPart part){
					// If the part being deactivated is the last part then disable the alignment window's actions
					if( part == fLastEditorPart ) {
						fAlignmentWindowAction.editorPartDeactivated();
					}
				}
				public void partActiavted(IWorkbenchPart part){
				}
				public void partOpened(IWorkbenchPart part){
				}
				public void partClosed(IWorkbenchPart part){
					// If the part being deactivated is the last part then disable the alignment window's actions
					if( part == fLastEditorPart ) {					
						fAlignmentWindowAction.editorPartClosed();
					}
				}
				public void partBroughtToTop(IWorkbenchPart part){
				}
			};
			getPage().addPartListener(fPartListener);	
		} else {
			// Tell the alignment window action about the new editor part so it can hook to it
			// Because the JVE is a combined Java and JBCF editor we must give the alignment actions the actual
			// selection because if they query it from the page then they will find it is a TextSelection from the
			// text editor - the JBCF only fires on change of selection
			IStructuredSelection selection = (IStructuredSelection) jvePart.getVCEEditor().getPrimaryViewer().getSelection();
			fItemsCreated.add(fAlignmentWindowAction);
			tbm.add(fAlignmentWindowAction);
			fAlignmentWindowAction.setEditorPart(jvePart.getVCEEditor(),selection);
		}

	}
	// If we don't do this then there is an Eclipse bug where the old toolbar icons
	// remain showing unless you click off onto a new viewer and back again
	tbm.update(true);
	addStatusFields(part) ;
}
public void paletteEntrySelected(PaletteEntry entry){
	// If we have selected a tool entry based on a palette entry being selected then disable it
	if ( fSelectedToolEntryAction != null ) {
		fSelectedToolEntryAction.setChecked(false);
		getActionBars().getToolBarManager().update(false);
	}
	
	if ( entry == null){
		getActionBars().getStatusLineManager().setMessage(null);
	} else {
		// A palette entry has been selected.  Show this in the status bars
		if ( entry instanceof AbstractToolEntry && ((AbstractToolEntry)entry).getTool() instanceof CreationTool ){
			getActionBars().getStatusLineManager().setMessage(
							entry.getSmallIcon() , 
							MessageFormat.format(CodegenEditorPartMessages.getString("JVEActionContributor.Status.Creating(label)"), new Object[]{entry.getLabel()})); //$NON-NLS-1$
		} 
		// If the entry can give us a tool then we must select the action that has that tool
		// Fortunately the ID of the entry has been used as the actionID so we can just crank over the 
		// paletteToolEntry actions and find the one that matches the ID
		String id = ((AbstractToolEntry)entry).refID();
		// Once an entry is selected we should find the Action that is associated with that entry
		// and select it, as well as de-select the one that was previously selected
		// PaletteToolEntryAction contains a method selectIfContainsId that checks the id of the action
		// and also recurses through for the case of the PaletteCategories that is the parent of entry actions
		Iterator toolbarItems = fItemsCreated.iterator();
		while( toolbarItems.hasNext() ) {
			Action toolbarItem = (Action)toolbarItems.next();
			if(toolbarItem.getId().equals(id)){
				toolbarItem.setChecked(true);
				getActionBars().getToolBarManager().update(true);
				fSelectedToolEntryAction = toolbarItem;
				break;					
			}
		}	
	}	
}
/**
 * Return a RetargetAction that has the same ID and icon and text, etc... as the argument
 * This allows the action to be targeted towards the JavaBeans viewer as well as the JavaVisualEditor part
 */
protected RetargetAction getRetargetAction(Action anAction){
	
	RetargetAction result = null;
	if ( anAction instanceof PaletteCategoriesToolEntryAction ) {
		result = new MenuCreatorRetargetAction(anAction.getId(),anAction.getText());
	} else {
		result = new RetargetAction(anAction.getId(),anAction.getText());
	}
	result.setImageDescriptor(anAction.getImageDescriptor());
	result.setHoverImageDescriptor(anAction.getHoverImageDescriptor());
	result.setDisabledImageDescriptor(anAction.getDisabledImageDescriptor());
	result.setToolTipText(anAction.getToolTipText());
	result.setChecked(anAction.isChecked());
	return result;	
	
}
/**
 * Remove the old action and get the action for the ID from the current editor part
 * and add it to the toolbar manager
 */
protected void processVCEAction(Object actionKey,IEditorPart oldEditorPart){
	ActionRegistry actionRegistry = (ActionRegistry)fLastEditorPart.getAdapter(ActionRegistry.class);
	IAction vceAction = actionRegistry.getAction(actionKey);
	// If the lastEditorPart is not null then we had a previous editor part
	// this would have added a delete action which we must now remove
	IToolBarManager tbm = getActionBars().getToolBarManager();	
	// The previous item should have been added to the fItems created which are all removed so we 
	// no longer have to remove it here
	tbm.add(vceAction);
	fItemsCreated.add(vceAction);
}
	/**
	 * @see IEditorActionBarContributor#dispose()
	 */
	public void dispose() {
		super.dispose();
		
		if (fLastEditorPart != null ) {
			fLastPaletteViewer.removePaletteListener(fPaletteListener);
		}		
		
		if (fPartListener != null){
			getPage().removePartListener(fPartListener);
		}
		
		JVEActionStatusLineContributionItem statusItem = (JVEActionStatusLineContributionItem) fJVEStatusFields.get(STATUS_CATEGORY_SYNC_ACTION);	
		if (statusItem != null)
			statusItem.dispose();
			
		// But need to remove part listeners
		Iterator iter = fItemsCreated.iterator();
		while(iter.hasNext()){
			Object action = iter.next();
			// If the action is a retarget action then it will be a page listener and needs removing
			if ( action instanceof RetargetAction ) {
				getPage().removePartListener((RetargetAction)action);
			}
		}
		
		if (fAlignmentWindowAction != null)
			fAlignmentWindowAction.dispose();			
	}
	/**
	 * @see EditorActionBarContributor#contributeToStatusLine(IStatusLineManager)
	 */
	public void contributeToStatusLine(IStatusLineManager statusLineManager) {
		super.contributeToStatusLine(statusLineManager);		

		statusLineManager.add((IContributionItem) fJVEStatusFields.get(STATUS_CATEGORY_SYNC_STATUS));
		statusLineManager.add((IContributionItem) fJVEStatusFields.get(STATUS_CATEGORY_SYNC_ACTION));
	}
	
}