package com.ibm.etools.jbcf.editorpart;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.io.*;
import java.io.IOException;
import java.net.*;
import java.net.URL;
import java.util.*;
import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.FocusEvent;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jdt.internal.corext.template.Templates;
import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.*;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.IRewriteTarget;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;
import org.eclipse.ui.internal.WWinPartService;
import org.eclipse.ui.part.EditorActionBarContributor;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.texteditor.*;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.*;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheet;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.IModelChangeController;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.draw2d.geometry.Rectangle;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.GraphicalEditPart;
import com.ibm.etools.gef.palette.PaletteRoot;
import com.ibm.etools.gef.ui.actions.*;
import com.ibm.etools.gef.ui.actions.ActionRegistry;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.codegen.*;
import com.ibm.etools.jbcf.codegen.CodeGenVisualGraphicalEditorPart;
import com.ibm.etools.jbcf.codegen.ICodeGenFlushController;
import com.ibm.etools.jbcf.codegen.java.*;
import com.ibm.etools.jbcf.codegen.util.CodeGenException;
import com.ibm.etools.jbcf.vce.VCEPlugin;
import com.ibm.etools.logging.util.Level;
import com.ibm.etools.propertysheet.AbstractPropertySheetEntry;

public class JavaVisualEditorPart extends EditorPart implements ITextEditor {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected VCECompilationUnitEditor fJavaEditor;
	protected CodeGenVisualGraphicalEditorPart fVCEEditor;
	protected JavaVisualEditorSelectionProvider fSelectionProvider;
	protected EditorPart fActiveEditor;
	protected JavaVisualEditorCommandStack fCommandStack;
	protected Preferences fStore;
	protected ActionRegistry fActionRegistry;
	public final static String MARKER_PropertySelection	= "propertySheetSelection" ; //$NON-NLS-1$
	public final static String MARKER_PropertyChange 	= "propertySheetValueChange" ; //$NON-NLS-1$
	public final static String MARKER_JVESelection	  	= "VCE Selection" ; //$NON-NLS-1$
	
	private final static String CURRENT_DEFAULT_TEMPLATE_VERSION  = "10012002_GA"; //$NON-NLS-1$
	

	// class that is used between the palette and the rest of the JVE editor
	// that has an API to allow the palette to be hidden and shown

	protected CustomSashForm fPaletteSashForm; // The sash form separating the palette from the editor
	protected CustomSashForm fSourceSash;	// The sash form for the source/design view.

	protected JavaModelChangeController fModelChangeController = new JavaModelChangeController();

	private class JavaModelChangeController implements IModelChangeController, ICodeGenFlushController {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		// The undomanager doesn't handle nesting of compound changes, so we need to do it here.
		private int compoundChangeCount = 0;
		private boolean inTransaction = false;
		private int     isRunning = 0 ;   // Note that a Runable is outstanding
		private boolean allowingChanges = false;
		private boolean fHoldChanges = false;
		private String fHoldMsg = null;
		private int redrawCntr = 0;
		private ArrayList fCodeGenFlushReq = new ArrayList() ;

		public synchronized boolean inTransaction() {
			return inTransaction;
		}
		public synchronized void setHoldChanges(boolean flag, String msg) {			
			fHoldChanges = flag;
			fHoldMsg = msg;
			while (flag && inTransaction && isRunning>0)  {
				try {
					this.wait() ;
				} catch (InterruptedException e) {}
			}						
		}

		public synchronized boolean isHoldChanges() {
			return fHoldChanges;
		}

		public void waitForCompleteTransaction() {
			fVCEEditor.commitAndFlush(false);
			// Make sure there are no changes outstanding. Will wait until they are committed. Will immediately wake up committer.
		}
		// This is a hack, we should have an Editor Error manager in the future to handle
		// errors, their context, and the way we denote them to the user.
		private void showMsg(String msg) {
			if (msg != null)
				showMessage(IErrorHolder.ErrorType.getSevereErrorImage(), msg);
			else
				showMessage(null, null);
		}

		public boolean run(Runnable runnable) {
			boolean localAllowChanges = false;
			boolean localHoldChanges; // Hold instance may be changing in the middle
			boolean localRunit;
			String localHoldMsg = null;
			synchronized (this) {
				// No point to query the JavaEditor if we are on hold
				isRunning ++ ;
				localHoldChanges = fHoldChanges;
				localHoldMsg = fHoldMsg;
			}

			try {
				localAllowChanges = startChange();
				localRunit = !localHoldChanges && localAllowChanges;

				if (localRunit)
					runnable.run();
				else if (localHoldChanges)
					showMsg(localHoldMsg);
				else
					showMsg(CodegenEditorPartMessages.getString("EditorPart.Msg.FileReadOnly_ERROR_")); //$NON-NLS-1$
			} finally {
				if (localAllowChanges)
					stopChange();
			    synchronized (this) {
			    	isRunning-- ;
			    	if (isRunning<=0) {
			    	   this.notifyAll() ;
			    	   isRunning=0 ;
			    	}
			    }
			}

			return localRunit;
		}

		// There is still a small window for a race condition, but it can't be addressed for beta. Needs better
		// support within codegen to transactialize it.
		//
		// If startChange returns true, then stopChange MUST be called, otherwise counters will get messed up.
		//
		// NOTE: This thread must NOT already be sync on "this" or else it won't work.
		private boolean startChange() {
			// starts changes and returns whether changes allowed.
			boolean doit = false;
			final IRewriteTarget rewriteTarget = (IRewriteTarget) fJavaEditor.getAdapter(IRewriteTarget.class);
			JBCFPlugin.log("(+1) Starting change", Level.FINEST); //$NON-NLS-1$
			synchronized (this) {
				if (compoundChangeCount++ == 0) {
					if (rewriteTarget != null) {
						// We only check for allow changes once, when we start a nested change group.
						// While nested we don't allow changes depending upon the initial test.
						try {  // allowEditing() Must be called on the Display thread 
							   // It is important that we clear the compoundChangCount if there are any fooFoo.
						   allowingChanges = fJavaEditor.allowEditing();
						}
						catch (Throwable t) {
							JBCFPlugin.log(t,JBCFMsgLogger.LOG_WARNING) ;
						}
						if (allowingChanges)
							doit = true;
						else
							compoundChangeCount = 0; // Go back to no transaction.
					} else
						allowingChanges = true; // Editor not fully up yet, so not a transaction that can be undone. Just go do it.		
				} else
					JBCFPlugin.log("(+2) Starting nested transaction", Level.FINEST); //$NON-NLS-1$
			}

			if (doit) {
				JBCFPlugin.log("(+1a) Starting wait for completion", Level.FINEST); //$NON-NLS-1$
				// Needed to have commitAndFlush outside of sync block because the commitAndFlushAsync below may also try to access "this", but from a different thread.
				// Can't have any pending changes before beginning next group of changes.				
				waitForCompleteTransaction();
				JBCFPlugin.log("(-1a) Ending wait for completion", Level.FINEST); //$NON-NLS-1$
				JBCFPlugin.log("(+1b) Starting begin compound change", Level.FINEST); //$NON-NLS-1$
				Display.getDefault().syncExec(new Runnable() {
					public void run() {
						if (redrawCntr++ <= 0) {
							rewriteTarget.setRedraw(false); // So we don't get so much flicker.							
							JBCFPlugin.getPlugin().getMsgLogger().log("Redraw off", Level.FINER); //$NON-NLS-1$
						}
						rewriteTarget.beginCompoundChange();
					}
				}); // setRedraw needs to be in a UI thread.
				JBCFPlugin.log("(-1b) Ending begin compound change", Level.FINEST); //$NON-NLS-1$
				synchronized (this) {
					inTransaction = true;
				}
				JBCFPlugin.getPlugin().getMsgLogger().log("(+3) Transaction started", Level.FINEST); //$NON-NLS-1$
			}
			JBCFPlugin.log("(-1) Ending starting change", Level.FINEST); //$NON-NLS-1$
			return allowingChanges;
		}

		private synchronized void stopChange() {
			// stops changes
			if (--compoundChangeCount <= 0) {
				JBCFPlugin.log("(3a) Starting end transaction", Level.FINEST); //$NON-NLS-1$
				compoundChangeCount = 0; // In case we get out of sync.
				if (inTransaction) {
					// We have done a beginCompoundChange, must now do endCompoundChange
					boolean localAllow = allowingChanges;
					allowingChanges = false;
					if (localAllow) {
						// When the CDE has finished executing the command we need to flush and commit
						// This tells the code generation to complete updating the source.					
						fVCEEditor.commitAndFlush(new ISynchronizerListener() {
							public void markerProcessed(String marker) {
								final IRewriteTarget rewriteTarget = (IRewriteTarget) fJavaEditor.getAdapter(IRewriteTarget.class);
								boolean doEnd = true;
								JBCFPlugin.log("(+3b) Starting async commit processing", Level.FINEST); //$NON-NLS-1$
								synchronized (JavaModelChangeController.this) {
									if (compoundChangeCount <= 0) { 										
										inTransaction = false; // We haven't started a new one while processing the termination.
										JavaModelChangeController.this.notifyAll() ;
									}
									else {
										doEnd = false;
										// We've started a new transaction since this end request went through, so treat it as nested.
										JBCFPlugin.log("(3b) Another transaction has started, treat it as nested for now.", Level.FINEST); //$NON-NLS-1$
									}
								}
								processCodeGenFlushIfNeeded() ;
								final boolean finalDoEnd = doEnd;
								Display.getDefault().syncExec(new Runnable() {
									public void run() {
										if (finalDoEnd) {
											rewriteTarget.endCompoundChange();
											JBCFPlugin.log("(-3) End transaction", Level.FINEST); //$NON-NLS-1$
										}
										if (--redrawCntr <= 0) {
											redrawCntr = 0;
											rewriteTarget.setRedraw(true);
											JBCFPlugin.getPlugin().getMsgLogger().log("Redraw on", Level.FINER); //$NON-NLS-1$
										}
									}
								}); // setRedraw needs to be in a UI thread.
								JBCFPlugin.log("(-3b) End async commit processing", Level.FINEST); //$NON-NLS-1$
							}
						}, "modelChangeMarker"); //$NON-NLS-1$
					}
				}
			} else
				JBCFPlugin.log("(-2) Ending nested transaction", Level.FINEST); //$NON-NLS-1$
		}

		/**
		 * @see com.ibm.etools.cde.IModelChangeController#getHoldMsg()
		 */
		public String getHoldMsg() {
			return fHoldMsg;
		}
		
		/**
		 * Called from stopChage();
		 * 
		 * Assuming that this is called after we finished a transaction
		 * (where a commit/Flush was already called)
		 */
		private void processCodeGenFlushIfNeeded() {
			
			ArrayList localList ;
			synchronized (this) {
				localList = fCodeGenFlushReq ;
				fCodeGenFlushReq = new ArrayList() ;
			}
			
			Iterator itr = localList.iterator() ;
			while (itr.hasNext()) {
				ISynchronizerListener l = (ISynchronizerListener) itr.next();
				String  m = (String) itr.next() ;
				// No need to call commitAndFlush - already did, and is too expensive
				// at this point.
				try {
				  l.markerProcessed(m) ;
				}
				catch (Throwable t) {
			    	JBCFPlugin.log(t) ;
			    }
				
			}			
		}
		/**
		 * This method is to be used to ask for CodeGen to flush generated code to the JavaEditor.
		 *  
		 * If we are in the middle of a transaction, queue the flush request to CodeGen.
		 */
		public synchronized void flushCodeGen(ISynchronizerListener listener, String marker) {
			if (inTransaction) {
				fCodeGenFlushReq.add(listener) ;
				fCodeGenFlushReq.add(marker) ;
			}
			else if (!isHoldChanges()) 
			    fVCEEditor.commitAndFlush(listener,marker) ;
			else
			    try {
			    	listener.markerProcessed(marker) ;
			    }			
			    catch (Throwable t) {
			    	JBCFPlugin.log(t) ;
			    }
		}

	};

	/**
	 * Method installTopTemplates installs the top 15 templates of
	 * JVE. The installation takes place when the value of the
	 * INSTALLED_TEMPLATES_VERSION of the preference store is
	 * different than the CURRENT_DEFAULT_TEMPLATE_VERSION in the
	 * code - When this happens the Templates are blasted in.
	 */
	private static void installTopTemplates(){
		if(!CodeGenPlugin.PLUGIN.getPluginPreferences().getString(CodeGenPlugin.INSTALLED_TEMPLATES_VERSION ).equals(CURRENT_DEFAULT_TEMPLATE_VERSION)){
			try {
				InputStream is = CodeGenPlugin.PLUGIN.openStream(new Path("resources/TopFifteen.xml")); //$NON-NLS-1$
				Templates.getInstance().addFromStream(is);
				Templates.getInstance().save();
				CodeGenPlugin.PLUGIN.getPluginPreferences().setValue(CodeGenPlugin.INSTALLED_TEMPLATES_VERSION , CURRENT_DEFAULT_TEMPLATE_VERSION);
			} catch (IOException e){
			} catch (CoreException e){
			}
		}
	}
	
	public JavaVisualEditorPart() {
		// Before doing anything, install the top templates if they havent been.
		installTopTemplates();
		
		fJavaEditor = new VCECompilationUnitEditor(this);
		// The CodeGenVisualGraphicalEditorPart is given a special command stack
		// that signals a begin and an end and executes command but never keeps them on a stack
		// This is because all undo and redo are directed to the JavaEditor and not the JVE JBCF editor part

		fCommandStack = new JavaVisualEditorCommandStack(fModelChangeController);
		fVCEEditor = new CodeGenVisualGraphicalEditorPart(fCommandStack,this);
		fVCEEditor.setRealEditorPart(this);					
		// Listen to changes in the java editor and forward them to listeners on us
		// This way things like the dirty flag, the save action, etc... are kept refreshed	
		fJavaEditor.addPropertyListener(new IPropertyListener() {
			public void propertyChanged(Object source, int propertyId) {
				if (propertyId != IWorkbenchPart.PROP_TITLE)
					JavaVisualEditorPart.this.firePropertyChange(propertyId);
				else
					setTitle(fJavaEditor.getTitle()); // Copy their title over, let them handle controlling title.
			}
		});
	}

	public void doSave(IProgressMonitor aMonitor) {
		fModelChangeController.waitForCompleteTransaction();
		fJavaEditor.doSave(aMonitor);
	}
	public void gotoMarker(IMarker aMarker) {
		fJavaEditor.gotoMarker(aMarker);
	}
	public void doSaveAs() {
		fModelChangeController.waitForCompleteTransaction();
		fJavaEditor.doSaveAs();
	}
	public boolean isSaveAsAllowed() {
		return fJavaEditor.isSaveAsAllowed();
	}
	public boolean isDirty() {
		if (fJavaEditor.isDirty())
			return true;

		return fModelChangeController.inTransaction(); // Else are we in a transaction, if we are, then we are potentially dirty.
	}

	public void dispose() {
		try {
			fVCEEditor.dispose();
		} catch (Throwable t) {
			com.ibm.etools.jbcf.JBCFPlugin.log(t);
		}
		try {
			fJavaEditor.dispose();
		} catch (Throwable t) {
			com.ibm.etools.jbcf.JBCFPlugin.log(t);
		}
		super.dispose();
	}

	/*
	 * This is called from VCECompilationUnitEditor for changing the input.
	 */
	public void resetInput(IEditorInput input) {
		fModelChangeController.waitForCompleteTransaction(); // Don't want any transactions pending when we change input.
		setInput(input);
		getVCEEditor().resetInput(input);
	}

	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		setSite(site);
		setInput(input);

		openVCEViewersIfRequired(site);

		fJavaEditor.init(site, input);

		fVCEEditor.setInitializationData(getConfigurationElement(), null, null);
		fVCEEditor.init(site, input);

		// Setup the model change controller into the editdomain.
		EditDomain domain = (EditDomain) getAdapter(EditDomain.class);
		domain.setData(IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY, fModelChangeController);
		
		// The following is a hack to induce a Reload from scratch to occur after we
		// have the editor comes up.
        Display.getCurrent().asyncExec(new Runnable() {
			/**
			 * @see java.lang.Runnable#run()
			 */
			public void run() {
				IJVEStatus status = fVCEEditor.getStatus() ;
				status.setStatus(status.JVE_CODEGEN_STATUS_SYNCHING,false) ;
				status.setStatus(status.JVE_CODEGEN_STATUS_OUTOFSYNC,false) ;
				status.setStatus(status.JVE_CODEGEN_STATUS_UPDATING_JVE_MODEL,false) ;
				fVCEEditor.pauseRoundTripping(true) ;
				fVCEEditor.pauseRoundTripping(false) ;
			}
		});
		
		// Kludge needed because the JavaEditor adds a part listener that only removes itself when partClosed for the Java Text Editor
		// is received. But since the Java Text Editor is not the main editor, it never receives this notification. To force it to
		// receive this, we will listen and any for our editor will be resent as if for the Java Text Editor.
		getSite().getWorkbenchWindow().getPartService().addPartListener(new IPartListener() {
			/**
			 * @see org.eclipse.ui.IPartListener#partActivated(IWorkbenchPart)
			 */
			public void partActivated(IWorkbenchPart part) {
			}

			/**
			 * @see org.eclipse.ui.IPartListener#partBroughtToTop(IWorkbenchPart)
			 */
			public void partBroughtToTop(IWorkbenchPart part) {
			}

			/**
			 * @see org.eclipse.ui.IPartListener#partClosed(IWorkbenchPart)
			 */
			public void partClosed(IWorkbenchPart part) {
				if (part == JavaVisualEditorPart.this) {
					((WWinPartService) getSite().getWorkbenchWindow().getPartService()).partClosed(fJavaEditor);
					getSite().getWorkbenchWindow().getPartService().removePartListener(this);
				}
			}

			/**
			 * @see org.eclipse.ui.IPartListener#partDeactivated(IWorkbenchPart)
			 */
			public void partDeactivated(IWorkbenchPart part) {
			}

			/**
			 * @see org.eclipse.ui.IPartListener#partOpened(IWorkbenchPart)
			 */
			public void partOpened(IWorkbenchPart part) {
			}
		});
	} 

	protected void openVCEViewersIfRequired(IEditorSite site) throws PartInitException {

		// Open the properties and Java beans viewer if we are in the Java or Java Browsing perspective
		IWorkbenchPage page = site.getWorkbenchWindow().getActivePage();
		if (page == null)
			return;

		// Calling showView will open the editor if it isn't already opened, and bring it to the front
		// if it already is ( for example in the J2EE perspective the Properties viewer may already be open
		// so showView brings it to the front )
		site.getPage().showView("org.eclipse.ui.views.PropertySheet"); //$NON-NLS-1$
		site.getPage().showView("com.ibm.etools.jbcf.editorpart.BeansList"); //$NON-NLS-1$

	}

	protected void firePropertyChange(int id) {
		super.firePropertyChange(id);
	}

	public void createPartControl(Composite parent) {

		// We are either a split pane or a notebook depending on the user preference
		fStore = VCEPlugin.getPlugin().getPluginPreferences();
		if (fStore.getString(VCEPlugin.SOURCE_SYNC_DELAY).length() > 0)
			fVCEEditor.setSynchronizerSyncDelay(Integer.parseInt(fStore.getString(VCEPlugin.SOURCE_SYNC_DELAY)));
			
		boolean isNotebook = fStore.getBoolean(VCEPlugin.NOTEBOOK_PAGE);
		if (isNotebook) {
			final CTabFolder folder = new CTabFolder(parent, SWT.BOTTOM);
			Composite fVCEComposite = new Composite(folder, SWT.NONE);
			fVCEComposite.setLayout(new FillLayout());

			// Create a 1:4 sash with the palette on the left and the VCE on the right
			fPaletteSashForm = new CustomSashForm(fVCEComposite, SWT.HORIZONTAL, CustomSashForm.NO_MAX_RIGHT);
			fVCEEditor.createPaletteControl(fPaletteSashForm);
			fVCEEditor.createPartControl(fPaletteSashForm);
			fPaletteSashForm.setWeights(getPaletteSashWeights());
			// Displayu the palette if the preferences state it should be initially shown
			boolean createPalette = fStore.getBoolean(VCEPlugin.NOTEBOOK_SHOW_GEF_PALETTE);
			if (!createPalette)
				fPaletteSashForm.maxLeft();			
			// Create a notebook folder
			CTabItem vceItem = new CTabItem(folder, SWT.NONE);
			vceItem.setControl(fVCEComposite);
			vceItem.setText(CodegenEditorPartMessages.getString("JavaVisualEditorPart.DesignPart")); //$NON-NLS-1$

			Composite fJavaComposite = new Composite(folder, SWT.NONE);
			fJavaComposite.setLayout(new FillLayout());
			fJavaEditor.createPartControl(fJavaComposite);
			CTabItem javaItem = new CTabItem(folder, SWT.NONE);
			javaItem.setControl(fJavaComposite);
			javaItem.setText(CodegenEditorPartMessages.getString("JavaVisualEditorPart.SourcePart")); //$NON-NLS-1$
			// Selection must be on the first page otherwise it does not activate correctly
			folder.setSelection(vceItem);
		} else {
			fPaletteSashForm = new CustomSashForm(parent, SWT.HORIZONTAL, CustomSashForm.NO_MAX_RIGHT);
			fVCEEditor.createPaletteControl(fPaletteSashForm);
			Composite editorComposite = new Composite(fPaletteSashForm, SWT.NONE);
			editorComposite.setLayout(new FillLayout());
			fSourceSash = new CustomSashForm(editorComposite, SWT.VERTICAL | SWT.BORDER);
			fVCEEditor.createPartControl(fSourceSash);
			Composite fJavaComp = new Composite(fSourceSash, SWT.BORDER);
			fJavaComp.setLayout(new FillLayout());
			fJavaEditor.createPartControl(fJavaComp);
			fPaletteSashForm.setWeights(getPaletteSashWeights());
			// Display the palette if the preferences state it should be initially shown
			boolean createPalette = fStore.getBoolean(VCEPlugin.SPLITPANE_SHOW_GEF_PALETTE);
			if (!createPalette)
				fPaletteSashForm.maxLeft();
		}

		// Add listeners so we know when the user has clicked between the source or the VCE canvas
		fVCEEditor.getPrimaryViewer().getControl().addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent fe) {
				setActiveEditor(fVCEEditor);
			}
			public void focusLost(FocusEvent fe) {
				setActiveEditor(null);
			}
		});
		// Add a listener to the text widget of the java editor so we know when it receives focus
		fJavaEditor.getTextWidget().addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				setActiveEditor(fJavaEditor);
			}
			public void focusLost(FocusEvent e) {
				setActiveEditor(null);
			}
		});

		createActions();

		attachListeners();

	}
	/** The domain holds the action registry.  Get this and replace its delete action
	 * with one that is aware of both the JBCF and the source so it can re-direct its delete
	 * to whoever last signalled an event		
	 */
	protected void createActions(){
		
		fActionRegistry = (ActionRegistry)fVCEEditor.getAdapter(ActionRegistry.class);
		
		// The undo and redo actions currently registered in the ActionRegistry of the VCE editor are those
		// for the GEF undo and redo.  However these are not used by the JVE, which instead uses the JavaEditor's
		// undo and redo actions as everything goes through the source.  We must replace the actionRegistry ones
		// with these, because the EMFContentOutlinePage uses the ActionRegistry actions as part of LabelRetargetAction
		// so that undo and redo are available with the Java Beans editor
		IAction undoAction = fJavaEditor.getAction(IWorkbenchActionConstants.UNDO);
		// The OTI actions don't have an ID, however it is required by the GEF registry so we must set it
		undoAction.setId(IWorkbenchActionConstants.UNDO);
		fActionRegistry.registerAction(undoAction);

		IAction redoAction = fJavaEditor.getAction(IWorkbenchActionConstants.REDO);
		redoAction.setId(IWorkbenchActionConstants.REDO);
		fActionRegistry.registerAction(redoAction);			
			
		IAction jveDeleteAction = fActionRegistry.getAction(org.eclipse.ui.IWorkbenchActionConstants.DELETE);
		IAction javaDeleteAction = fJavaEditor.getAction(org.eclipse.ui.IWorkbenchActionConstants.DELETE);
		class HybridDeleteAction extends SelectionAction{
			protected IAction fJveDeleteAction;
			protected IAction fJavaDeleteAction;
			HybridDeleteAction(IAction jveDeleteAction, IAction javaDeleteAction, IEditorPart anEditorPart){
				super(anEditorPart);
				fJveDeleteAction = jveDeleteAction;
				fJavaDeleteAction = javaDeleteAction;
				setText(fJveDeleteAction.getText());
				setToolTipText(fJveDeleteAction.getToolTipText());
				setId(fJveDeleteAction.getId());
				setHoverImageDescriptor(fJveDeleteAction.getHoverImageDescriptor());
				setImageDescriptor(fJveDeleteAction.getImageDescriptor());
				setDisabledImageDescriptor(fJveDeleteAction.getDisabledImageDescriptor());
				setEnabled(false);
				
			}
			protected boolean calculateEnabled(){
				if ( getSelection() instanceof TextSelection ) {
					return fJavaDeleteAction.isEnabled();
				} else {
					return fJveDeleteAction.isEnabled();
				}
			}
			public void run() {
				if ( getSelection() instanceof TextSelection ) {
					fJavaDeleteAction.run();
				} else {
					fJveDeleteAction.run();
				}
			}	
		}
		IAction hybridDeleteAction = new HybridDeleteAction(jveDeleteAction,javaDeleteAction,this){
			
		};
		fActionRegistry.registerAction(hybridDeleteAction);
		
	}
		
	protected int[] getPaletteSashWeights() {
		// The weights are stored as preferences so as the user changes it new editors open with the 
		// customized size
		int leftRatio = VCEPlugin.getPlugin().getPluginPreferences().getInt(VCEPlugin.PALETTE_SASH_LEFT_RATIO);
		if (leftRatio == 0) {
			leftRatio = VCEPlugin.DEFAULT_PALETTE_SASH_LEFT_RATIO;
		}
		int rightRatio = VCEPlugin.getPlugin().getPluginPreferences().getInt(VCEPlugin.PALETTE_SASH_RIGHT_RATIO);
		if (rightRatio == 0) {
			rightRatio = VCEPlugin.DEFAULT_PALETTE_SASH_RIGHT_RATIO;
		}
		return new int[] { leftRatio, rightRatio };
	}

	protected void savePaletteSashPosition() {
		// Save the positions of the palette sash
		int[] paletteWeights = fPaletteSashForm.getWeights();
		VCEPlugin.getPlugin().getPluginPreferences().setValue(VCEPlugin.PALETTE_SASH_LEFT_RATIO, paletteWeights[0]);
		VCEPlugin.getPlugin().getPluginPreferences().setValue(VCEPlugin.PALETTE_SASH_RIGHT_RATIO, paletteWeights[1]);
	}


	protected JBCFCommandStackPropertySheetEntry fCurrentPropertySheetEntry;
	
	protected void attachListeners() {

		// We must listen to this here, rather than in init(site,editor) because of the order in which
		// the compilation unit editor creates the control that it actually uses as the selection provider itself
		fJavaEditor.getSelectionProvider().addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				// Only fire an event if the javaEditor is active.  This is because clicking the outline
				// or the visual editor can drive the source cursor which changes selection and fires events
				if (fActiveEditor == fJavaEditor) {
					fSelectionProvider.fireSelectionChanged(event);
				}
			}
		});
		fVCEEditor.getSelectionNotifier().addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				fSelectionProvider.fireSelectionChanged(event);
				JavaVisualEditorPart.this.vceSelectionChanged(event);
			}
		});
		// Create our own selection provider that copes with us having two editor parts
		// - the Java source page as well as the JavaVisualEditor
		fSelectionProvider = new JavaVisualEditorSelectionProvider(this);
		// The Java Editor inside createPartControl overwrites the selection provider
		// so we must put our own one in here after calling its createPartControl...
		getSite().setSelectionProvider(fSelectionProvider);

		// When the user types into a java editor the model is changed incrementally
		// We need to know when this happens so we can drive a refresh on the property sheet
		fVCEEditor.addTranslatorListener(new ISourceTranslatorListener() {
			public void modelUpdated() {
				if (fVCEEditor.isPropertySheetCreated()) {
					// Horrid kludge: The model could be reloaded from scratch at this point,
					// which means any editpart has been deselected. However, if the PropertySheet
					// was the active viewer, it wouldn't know this because for him he listens to
					// Workbench selection provider, and when he is the active viewer, he IS the
					// selection provider. In that case the deselect in the graph and tree viewer
					// would go unnoticed. Also, the editparts would no longer exist at this point,
					// so the PS is using old deactivated editparts. So to get around this, we will get
					// the root entry and see if it is an editpart and that editpart is not selected.
					// If that is so, we will set no selection into the PS so that it won't have 
					// an invalid value.
					//
					// The reason this happens is because if you do a ctrl-z (undo) from the PS,
					// this could cause a reload from scratch to occur if complicated enough. Now the
					// PS doesn't expect the model that it is currently handling to be pulled out
					// from under it while it has focus. It uses the Workbench Selection provider to
					// handle changes.
					
					AbstractPropertySheetEntry pse = (AbstractPropertySheetEntry) fVCEEditor.getPropertySheetPage().getRootEntry();
					Object[] value = pse.getValues();
					if (value.length == 1 && value[0] instanceof EditPart) {
						if (((EditPart) value[0]).getSelected() == EditPart.SELECTED_NONE)
							pse.setValues(new Object[0]);
						else
							pse.refreshFromRoot();
					} else
						pse.refreshFromRoot();
				}
				fVCEEditor.refresh() ;
			}
			public void snippetProcessingStart() {
			}
			public void snippetProcessingCompleted() {
				Display.getDefault().syncExec(new Runnable() {
					public void run() {
						// Force the editor part to receive focus if our workbench window is active
						if (getSite().getWorkbenchWindow() == getSite().getWorkbenchWindow().getWorkbench().getActiveWorkbenchWindow()) {
							setFocus();
						}
					}
				});
			}
		});

		// When the user selects an entry in the property sheet we want to drive
		// the source towards that line if it has one
		fVCEEditor.addPropertySheetSelectionChangeListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				ISelection selection = event.getSelection();
				if (!selection.isEmpty()) {
					// We know the property sheet is a single selection viewer, so we don't need to worry about multiple selections
					Object oentry = ((IStructuredSelection) selection).getFirstElement();
					if (oentry instanceof JBCFCommandStackPropertySheetEntry) {
						// An entry in the property sheet has been selected
						// We want to see whether this has a line of source we can drive selection to
						JBCFCommandStackPropertySheetEntry entry = (JBCFCommandStackPropertySheetEntry) oentry;
						// We must listen to each entry as it is selected ( and stop listener to it as well )
						// so that we can listen to when the value is changed so we can drive selection
						// We might have problems if the property sheet is closed and re-opened as we need to test for this
						if(fCurrentPropertySheetEntry != null){
							fCurrentPropertySheetEntry.removePropertySheetEntryListener(getPropertySheetEntryListener());
						}
						entry.addPropertySheetEntryListener(getPropertySheetEntryListener());
						fCurrentPropertySheetEntry = entry;
						if (!entry.isStale()) {
							// If stale, then don't go there.
							Object source = entry.getEditValue();
							if ((source instanceof IJavaInstance) && 
				 			    (getSite().getPage().getActivePart() instanceof PropertySheet)) {
				 			    	// Also make sure that the PS is in focus for it to drive to source
				 			    PropertySheet ps = (PropertySheet) getSite().getPage().getActivePart() ;
				 			    // Are we the active page ??
				 			    if (ps.getCurrentPage().equals(fVCEEditor.getPropertySheetPage()))
								   // It is possible that there are pending changes that need to be generated to code
								   // before we can select the source range
								   fModelChangeController.flushCodeGen(new SelectionSynchronizer((IJavaInstance) source), MARKER_PropertySelection); //$NON-NLS-1$
							}
						}
					}
				} else {
					// need to remove if one is listening
					if(fCurrentPropertySheetEntry != null){
						fCurrentPropertySheetEntry.removePropertySheetEntryListener(getPropertySheetEntryListener());
					}
				}
			}

		});

	}
	
	class SelectionSynchronizer implements ISynchronizerListener {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		Notifier fJavaBean;
		SelectionSynchronizer(IJavaInstance aJavaBean) {
			fJavaBean = aJavaBean;
		}
		public void markerProcessed(String marker) {
			ICodeGenAdapter codeGenAdapter ;
			if (MARKER_JVESelection.equals(marker))  // Select the method
			   codeGenAdapter = (ICodeGenAdapter) fJavaBean.getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER);
			else     // Select the Expression
			   codeGenAdapter = (ICodeGenAdapter) fJavaBean.getAdapter(ICodeGenAdapter.JVE_CODEGEN_EXPRESSION_SOURCE_RANGE);
			if (codeGenAdapter == null)
			   codeGenAdapter = (ICodeGenAdapter) fJavaBean.getAdapter(ICodeGenAdapter.VCE_CODE_GEN_TYPE);
			if (codeGenAdapter == null) {
				com.ibm.etools.jbcf.JBCFPlugin.log("JavaVisualEditorPart.markerProcessed(): No CodeGen Adapter on: " + fJavaBean, //$NON-NLS-1$
				com.ibm.etools.jbcf.JBCFMsgLogger.LOG_FINE);
			} else {
				try {
					final ICodeGenSourceRange sourceRange = codeGenAdapter.getHighlightSourceRange();
					// Drive the selection in the java editor
					// This must be done on the Display thread so that SWT works correctly
					// Only do this if the source editor is not active, because if so then the user
					// may be typing in it and if we drive the cursor away from them they won't like it
					if (fActiveEditor != fJavaEditor && sourceRange != null) {
						Display.getDefault().syncExec(new Runnable() {
							public void run() {
								try {
									fJavaEditor.setHighlightRange(sourceRange.getOffset(), sourceRange.getLength(), true);
								} catch (Exception exc) {
									exc.printStackTrace();
									// Do nothing - We get assertion failures that I don't fully understand, especially when
									// dropping onto the JavaBeans viewer
								}
							}
						});
					}
				} catch (CodeGenException exc) {
					exc.printStackTrace();
				}
			}
		}
	}

	protected IPropertySheetEntryListener fPropertySheetEntryListener;
	protected IPropertySheetEntryListener getPropertySheetEntryListener(){
		if ( fPropertySheetEntryListener == null){
			fPropertySheetEntryListener = new IPropertySheetEntryListener(){
				public void childEntriesChanged(IPropertySheetEntry entry){
				}
				public void errorMessageChanged(IPropertySheetEntry entry){
				}
				public void valueChanged(IPropertySheetEntry entry){
					Object source = ((JBCFCommandStackPropertySheetEntry)entry).getEditValue();
					if ((source instanceof IJavaInstance) && 					
				 		 (getSite().getPage().getActivePart() instanceof PropertySheet)) {
				 			// Also make sure that the PS is in focus for it to drive to source
				 			PropertySheet ps = (PropertySheet) getSite().getPage().getActivePart() ;
				 			// Are we the active page ??
				 			if (ps.getCurrentPage().equals(fVCEEditor.getPropertySheetPage()))							
								fModelChangeController.flushCodeGen(new SelectionSynchronizer((IJavaInstance)source), MARKER_PropertyChange); //$NON-NLS-1$
					}
				}		
			};
		}
		return fPropertySheetEntryListener;
	}
	

	protected void vceSelectionChanged(SelectionChangedEvent event) {
		// Get the JavaBeans that are selected
		StructuredSelection selection = (StructuredSelection) event.getSelection();
		List selectedEditParts = selection.toList();
		showMessage(null, null); // Clear the status bar as we try and show any errors for the selected JavaBean
		// Only try and drive the source editor selection if one edit part is selected
		// If more than one is selected then don't bother 
		if (selectedEditParts.size() == 1) {
			EditPart editPart = (EditPart) selectedEditParts.iterator().next();
			if (editPart.getModel() instanceof Notifier) {
				IErrorNotifier errNotifier =
					(IErrorNotifier) ((Notifier) editPart.getModel()).getAdapter(IErrorNotifier.ERROR_NOTIFIER_TYPE);
				if (errNotifier != null) {
					// See whether or not there are any errors associated with the part - if so show them on the taskBar
					if (errNotifier.getErrorStatus() != IErrorNotifier.ERROR_NONE) {
						Iterator errors = errNotifier.getErrors().iterator();
						IErrorHolder.ErrorType error = (IErrorHolder.ErrorType) errors.next();
						showMessage(error.getImage(), error.getMessage());
					}
				}
			}
			if (editPart.getModel() instanceof IJavaInstance) {
				// It is possible that the javaBean has just beeen added or changes made to it
				// that have not made it to the source yet.
				// We therefore need to flush any pending changes through before we query the source range
				// to drive selection otherwise we are going to drive source changes ahead of the source being actually changed
				fModelChangeController.flushCodeGen(new SelectionSynchronizer((IJavaInstance) editPart.getModel()), MARKER_JVESelection); //$NON-NLS-1$
			} else { 
				fJavaEditor.resetHighlightRange();
			}
		}
	}
	protected void showMessage(Image anImage, String aMessage) {
		org.eclipse.jface.action.IStatusLineManager lm =
			((EditorActionBarContributor) getEditorSite().getActionBarContributor()).getActionBars().getStatusLineManager();
		lm.setMessage(anImage, aMessage);
		if (lm instanceof org.eclipse.jface.action.SubContributionManager
			&& !((org.eclipse.jface.action.SubContributionManager) lm).isVisible())
			 ((org.eclipse.jface.action.SubContributionManager) lm).setVisible(true);
	}

	public void setActiveEditor(EditorPart anEditorPart) {
		if (anEditorPart == fActiveEditor)
			return;
		fActiveEditor = anEditorPart;
		if (fActiveEditor != null
			&& (fActiveEditor instanceof VCECompilationUnitEditor || fActiveEditor instanceof CodeGenVisualGraphicalEditorPart)
			&& fVCEEditor != null)
			fVCEEditor.refreshStatus();
	}
	public EditorPart getActiveEditor() {
		return fActiveEditor;
	}
	public boolean isVCEEditorActive() {
		return fActiveEditor == fVCEEditor;
	}
	public void setFocus() {
		if (fJavaEditor != null) {
			fJavaEditor.setFocus();
		}
	}
	public CompilationUnitEditor getJavaEditor() {
		return fJavaEditor;
	}
	public CodeGenVisualGraphicalEditorPart getVCEEditor() {
		return fVCEEditor;
	}
	public Object getAdapter(Class type) {
		if (type == IPropertySheetPage.class) {
			return fVCEEditor.getAdapter(type);
		} else if (type == IContentOutlinePage.class) {
			return fJavaEditor.getAdapter(type);
		} else if (type == BeansList.class) {
			// The BeansList is asked for on us and we fill it with the
			// outline view of the JBCF part
			return fVCEEditor.getAdapter(IContentOutlinePage.class);
		} else if (type == EditDomain.class) {
			return fVCEEditor.getDomain();
		} else if (type == PaletteRoot.class) {
			return fVCEEditor.getAdapter(PaletteRoot.class);
		} else if (type == ActionRegistry.class) {
			return fVCEEditor.getAdapter(ActionRegistry.class);
		}
		return fJavaEditor.getAdapter(type);
	}
	/* Return the palette toolbar actions form the VCE Editor
	 */
	public List getPaletteToolbarActions() {

		return fVCEEditor.getPaletteToolbarActions();

	}

	/**
	 * To satisfy a lot of menu options ( Add Task, ones for breakpoints etc... ) we must implement ITextEditor
	 */
	public void close(boolean save) {
		fJavaEditor.close(save);
	}
	public void doRevertToSaved() {
		fJavaEditor.doRevertToSaved();
	}
	public IAction getAction(String actionID) {
		return fJavaEditor.getAction(actionID);
	}
	public IDocumentProvider getDocumentProvider() {
		return fJavaEditor.getDocumentProvider();
	}
	public IRegion getHighlightRange() {
		return fJavaEditor.getHighlightRange();
	}
	public ISelectionProvider getSelectionProvider() {
		return fJavaEditor.getSelectionProvider();
	}
	public boolean isEditable() {
		return fJavaEditor.isEditable();
	}
	public void removeActionActivationCode(String actionID) {
		fJavaEditor.removeActionActivationCode(actionID);
	}
	public void resetHighlightRange() {
		fJavaEditor.resetHighlightRange();
	}
	public void selectAndReveal(int start, int length) {
		fJavaEditor.selectAndReveal(start, length);
	}
	public void setAction(String actionID, IAction action) {
		fJavaEditor.setAction(actionID, action);
	}
	public void setActionActivationCode(String actionID, char activationCharacter, int activationKeyCode, int activationStateMask) {
		fJavaEditor.setActionActivationCode(actionID, activationCharacter, activationKeyCode, activationStateMask);
	}
	public void setHighlightRange(int start, int length, boolean moveCursor) {
		fJavaEditor.setHighlightRange(start, length, moveCursor);
	}
	public void showHighlightRangeOnly(boolean showHighlightRangeOnly) {
		fJavaEditor.showHighlightRangeOnly(showHighlightRangeOnly);
	}
	public boolean showsHighlightRangeOnly() {
		return fJavaEditor.showsHighlightRangeOnly();
	}

	/**
	 * 
	 * Status Fields are maintained in the VCE Editor,
	 * ... Passing through
	 */
	public void setStatusField(IStatusField field, String category) {
		fVCEEditor.setStatusField(field, category);
	}
	public void setStatusField(IJVEActionField field, String category) {
		fVCEEditor.setStatusField(field, category);
	}
	// Static helper method to allow wizards and other classes to open a resource using us
	public static void openResource(final IResource resource,final Display display){
		if (resource.getType() == IResource.FILE) {
			IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
			if (window != null) {
				final IWorkbenchPage activePage= window.getActivePage();
				if (activePage != null) {
					if (display != null) {
						display.asyncExec(new Runnable() {
							public void run() {
								try {
									activePage.openEditor((IFile)resource,"com.ibm.etools.jbcf.editorpart.JavaVisualEditor"); //$NON-NLS-1$
								} catch (PartInitException e) {
									JBCFPlugin.log(e);
								}
							}
						});
					}
				}
			}
		}
	}	
}