package com.ibm.etools.jbcf.editorpart;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.*;
import org.eclipse.jface.dialogs.*;
import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.jbcf.JBCFPlugin;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.*;

public class OpenVCEViewersDialog extends TitleAreaDialog {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected boolean fIsPropertySheetOpen;
	protected Button fOpenPropertySheet;
	protected boolean fShouldOpenPropertySheet;
	
	protected Button fDoNotShowOpenViewersDialogButton;
	protected boolean fShowOpenViewersDialog;

	protected Image fVCEImage;
	protected Image fPropSheetImage;
	
	protected boolean fIsBeansListOpen;
	protected Button fOpenBeansList;
	protected boolean fShouldOpenBeansList;

protected OpenVCEViewersDialog(Shell parentShell) {
	super(parentShell);
}
public boolean showOpenViewersDialog(){
	return fShowOpenViewersDialog;
}
public void showOpenViewersDialog(boolean aBool){
	fShowOpenViewersDialog = aBool;
}
protected void createButtonsForButtonBar(Composite parent) {
	// create just an OK button
	createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
}
protected void configureShell(Shell shell) {
	super.configureShell(shell);
	shell.setText(CodegenEditorPartMessages.getString("VCEViewersDialog.Title")); //$NON-NLS-1$
}
protected Control createContents(Composite parent) {
	
	Control contents = super.createContents(parent);
	setTitle(CodegenEditorPartMessages.getString("VCEViewersDialog.Title")); //$NON-NLS-1$
	setMessage(CodegenEditorPartMessages.getString("VCEViewersDialog.Message")); //$NON-NLS-1$
	Image image = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf.codegen/icons/full/wizban/javavisualeditor_wiz.gif"); //$NON-NLS-1$
	setTitleImage(image);
	return contents;
}
public void setIsPropertySheetOpen(boolean isOpen){
	fIsPropertySheetOpen = isOpen;
}
public boolean shouldOpenPropertySheet(){
	return fShouldOpenPropertySheet;
}
public void setIsBeansListOpen(boolean isOpen){
	fIsBeansListOpen = isOpen;
}
public boolean shouldOpenBeansList(){
	return fShouldOpenBeansList;
}
protected Control createDialogArea(Composite topParent) {
	
	Composite parent = (Composite)super.createDialogArea(topParent);
	
	Composite composite = new Composite(parent, SWT.NULL);
	GridLayout gridLayout = new GridLayout();
	gridLayout.numColumns = 2;
	composite.setLayout(gridLayout);
	composite.setLayoutData(
		new GridData(GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL));	
		
	Label label3 = new Label(composite,SWT.LEFT);
	GridData gridData = new GridData();
	gridData.horizontalSpan = 2;
	label3.setLayoutData(gridData);
	label3.setText(CodegenEditorPartMessages.getString("VCEViewersDialog.Label.AdditionalViewersMsg1")); //$NON-NLS-1$

	Label label4 = new Label(composite,SWT.LEFT);
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	label4.setLayoutData(gridData);
	label4.setText(CodegenEditorPartMessages.getString("VCEViewersDialog.Label.AdditionalViewersMsg2")); //$NON-NLS-1$
	
	createSpacer(composite,2);

	if (!fIsBeansListOpen){
		
		Label openBLLabel1 = new Label(composite, SWT.LEFT);
		openBLLabel1.setText(CodegenEditorPartMessages.getString("VCEViewersDialog.Label.JavaBeansViewIntro1")); //$NON-NLS-1$
		GridData data = new GridData();
		data.horizontalSpan = 2;
		openBLLabel1.setLayoutData(data);	
		
		Label openBLLabel2 = new Label(composite, SWT.LEFT);
		openBLLabel2.setText(CodegenEditorPartMessages.getString("VCEViewersDialog.Label.JavaBeansViewIntro2")); //$NON-NLS-1$
		data = new GridData();
		data.horizontalSpan = 2;
		openBLLabel2.setLayoutData(data);	
				
		fOpenBeansList = createCheckbox(composite,CodegenEditorPartMessages.getString("VCEViewersDialog.Checkbox.JavaBeansView"),1); //$NON-NLS-1$
		fOpenBeansList.setSelection(true);
		
		Label fBeansListImageLabel = new Label(composite,SWT.LEFT);
		fBeansListImageLabel.setImage(CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf.codegen/icons/full/ctool16/javavisualeditor_co.gif")); //$NON-NLS-1$
		data = new GridData();
		fBeansListImageLabel.setData(data);		
				
	}
	
	createSpacer(composite,2);	

	if (!fIsPropertySheetOpen){
		
		Label openPSLabel1 = new Label(composite, SWT.LEFT);
		openPSLabel1.setText(CodegenEditorPartMessages.getString("VCEViewersDialog.Label.PropertySheetIntro1")); //$NON-NLS-1$
		GridData data = new GridData();
		data.horizontalSpan = 2;
		openPSLabel1.setLayoutData(data);	
		
		Label openPSLabel2 = new Label(composite, SWT.LEFT);
		openPSLabel2.setText(CodegenEditorPartMessages.getString("VCEViewersDialog.Label.PropertySheetIntro2")); //$NON-NLS-1$
		data = new GridData();
		data.horizontalSpan = 2;
		openPSLabel2.setLayoutData(data);	
		
		fOpenPropertySheet = createCheckbox(composite,CodegenEditorPartMessages.getString("VCEViewersDialog.Checkbox.PropertiesView"),1); //$NON-NLS-1$
		fOpenPropertySheet.setSelection(true);
		
		Label fOpenPSImageLabel = new Label(composite,SWT.LEFT);
		fPropSheetImage = CDEPlugin.getImageFromURLString("platform:/plugin/org.eclipse.ui/icons/full/cview16/prop_ps.gif"); //$NON-NLS-1$
		fOpenPSImageLabel.setImage(fPropSheetImage);		

	}

	createSpacer(composite,2);
	
	fDoNotShowOpenViewersDialogButton = createCheckbox(composite,CodegenEditorPartMessages.getString("VCEViewersDialog.Checkbox.ShowAgainInFuture"),2); //$NON-NLS-1$
	// Always by default use false so the user does not see the dialog again
	// unless they ask to do so
	fDoNotShowOpenViewersDialogButton.setSelection(true);
		
	return composite;
	
}
public void dispose(){
	if ( fVCEImage != null ) {
		fVCEImage.dispose();
	}
	if ( fPropSheetImage != null ) {
		fPropSheetImage.dispose();
	}
}
protected void okPressed() {
	if ( fOpenPropertySheet != null){
		fShouldOpenPropertySheet = fOpenPropertySheet.getSelection();
	}
	if ( fOpenBeansList != null){
		fShouldOpenBeansList = fOpenBeansList.getSelection();
	}
	fShowOpenViewersDialog = !fDoNotShowOpenViewersDialogButton.getSelection();
	
	super.okPressed();
}
protected Button createCheckbox(Composite parent, String label, int columns){
	Button button = new Button(parent, SWT.CHECK | SWT.LEFT);
	button.setText(label);
	GridData data = new GridData();
	data.horizontalSpan = columns;
	button.setData(data);
	return button;
}
protected Group createGroup(Composite aParent, String title){
	Group group = new Group(aParent,SWT.NONE);
	if(title!=null){
		group.setText(title);
	}
	GridLayout gridLayout = new GridLayout();
	gridLayout.numColumns = 2;
	group.setLayout(gridLayout);
	GridData data = new GridData();
	data.verticalAlignment = GridData.FILL;
	data.horizontalAlignment = GridData.FILL;
	data.grabExcessHorizontalSpace = true;
	group.setLayoutData(data);
	return group;
}	
private void createSpacer(Composite parent, int columns) {
	Label spacer = new Label(parent, SWT.NONE);
	GridData data = new GridData();
	data.horizontalAlignment = GridData.FILL;
	data.verticalAlignment = GridData.BEGINNING;
	data.horizontalSpan = columns;
	spacer.setLayoutData(data);
}
}
