package com.ibm.etools.jbcf.editorpart;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.lang.reflect.*;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Collections;
import java.util.List;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.SourceType;
import org.eclipse.jdt.internal.corext.refactoring.rename.RenameFieldRefactoring;
import org.eclipse.jdt.internal.debug.ui.ExceptionHandler;
import org.eclipse.jdt.internal.ui.actions.ActionUtil;
import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditor;
import org.eclipse.jdt.internal.ui.refactoring.RefactoringMessages;
import org.eclipse.jdt.internal.ui.refactoring.RefactoringPreferences;
import org.eclipse.jdt.internal.ui.refactoring.RefactoringSupportFactory;
import org.eclipse.jdt.internal.ui.refactoring.actions.RefactoringStarter;
import org.eclipse.jdt.internal.ui.refactoring.actions.RenameJavaElementAction;
import org.eclipse.jdt.internal.ui.reorg.IRefactoringRenameSupport;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchSite;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.gef.editparts.AbstractEditPart;
import com.ibm.etools.gef.ui.actions.SelectionAction;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.CodeGenVisualGraphicalEditorPart;
import com.ibm.etools.jbcf.codegen.java.BeanDecoderAdapter;
import com.ibm.etools.jbcf.codegen.java.ICodeGenAdapter;
import com.ibm.etools.jbcf.codegen.java.ThisBeanDecoderAdapter;
import com.ibm.etools.logging.util.Level;

/**
 */
public class RenameJavaBeanAction extends Action {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ISelectionProvider fSelectionProvider;
	protected JavaVisualEditorPart fVisualEditorPart;	
	
public RenameJavaBeanAction(JavaVisualEditorPart editorPart, ISelectionProvider selectionProvider){
	setId("com.ibm.etools.jbcf.codegen.renamejavabean"); //$NON-NLS-1$
	setText(RefactoringMessages.getString("RefactoringGroup.rename_field_title")); //$NON-NLS-1$
	fSelectionProvider = selectionProvider;
	fVisualEditorPart = editorPart;
}
public boolean isEnabled(){
	// We are enabled if one and only one edit part for a JavaBean is selected
	List editParts = getSelectedEditParts();
	if ( editParts.size() == 1 ) {
		Object model = ((AbstractEditPart)editParts.get(0)).getModel();
		if ( model instanceof IJavaObjectInstance ) {
			IJavaObjectInstance javaModel = (IJavaObjectInstance)model;
			// Make sure that the field inherits from java.awt.Component.  We don't want to encourage renaming
			// of non visuals because for parsing we insist on the ivj prefix, so to prompt the user to rename a non-visual might be misleading
			JavaHelpers componentClass = (JavaHelpers)Utilities.getJavaClass("java.awt.Component",((IJavaObjectInstance)model).getResourceSet()); //$NON-NLS-1$
			if ( componentClass.isAssignableFrom((EClassifier)javaModel.getJavaType())){
				// The this part cannot be renamed, but BeanDecoderAdapter ( that represent fields can )
				Object beanDecoderAdapter = javaModel.getAdapter(ICodeGenAdapter.JVE_CODEGEN_BEAN_PART_ADAPTER);
				// ThisBeanDecoderAdapter subclasses BeanDecoderAdapter, hence the wierd logic			
				return ( beanDecoderAdapter instanceof BeanDecoderAdapter && !(beanDecoderAdapter instanceof ThisBeanDecoderAdapter));
			}
		}
	}
	return false;
}
protected List getSelectedEditParts(){
	ISelection selection = fSelectionProvider.getSelection();
	if ( selection != null ) {
		return ((IStructuredSelection)selection).toList();
	}
	return Collections.EMPTY_LIST;
}
/*
 * A private class that allows us to rename the java field and automatically
 * turn on the rename getter and setter flags too. Big kludge. Will need
 * to access private methods and fields through reflection.
 */
private static class RenameJBCFJavaFieldAction extends RenameJavaElementAction {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public RenameJBCFJavaFieldAction(CompilationUnitEditor editor) {
		super(editor);
	}
	
	private void run(Object element) {
		// Work around for http://dev.eclipse.org/bugs/show_bug.cgi?id=19104		
		if (!ActionUtil.isProcessable(getShell(), element))
			return;
		IRefactoringRenameSupport refactoringSupport= RefactoringSupportFactory.createRenameSupport(element);
		try {
			Method canRename = RenameJavaElementAction.class.getDeclaredMethod("canRename", new Class[] {IRefactoringRenameSupport.class, Object.class}); //$NON-NLS-1$
			canRename.setAccessible(true);
			if (! ((Boolean) canRename.invoke(this, new Object[] {refactoringSupport, element})).booleanValue())
				return;
			try{
				Field refactoringField = refactoringSupport.getClass().getSuperclass().getDeclaredField("fRefactoring"); //$NON-NLS-1$
				refactoringField.setAccessible(true);
				RenameFieldRefactoring renamerf = (RenameFieldRefactoring) refactoringField.get(refactoringSupport);
				if (renamerf.canEnableGetterRenaming())
					renamerf.setRenameGetter(true);
				if (renamerf.canEnableSetterRenaming())
					renamerf.setRenameSetter(true);
				refactoringSupport.rename(element);
			} catch (JavaModelException e){
				ExceptionHandler.handle(e, RefactoringMessages.getString("RenameJavaElementAction.name"), RefactoringMessages.getString("RenameJavaElementAction.exception"));  //$NON-NLS-1$ //$NON-NLS-2$
			}	
		} catch (NoSuchMethodException e) {
			runDefault(e, element);
		} catch (IllegalAccessException e) {
			runDefault(e, element);			
		} catch (InvocationTargetException e) {
			runDefault(e, element);			
		} catch (NoSuchFieldException e) {
			runDefault(e, element);			
		}
	}
	
	private void runDefault(Exception e, Object element) {
		JBCFPlugin.getPlugin().getMsgLogger().log(e, Level.WARNING);
		// If anything fails, just run the standard action instead.
		super.run(new StructuredSelection(element));
	}
}
public void run(){
	RenameJBCFJavaFieldAction renameAction = new RenameJBCFJavaFieldAction(fVisualEditorPart.getJavaEditor());
	// Get the first selected fields which will be an edit part
	AbstractEditPart editPart = (AbstractEditPart)getSelectedEditParts().get(0);
	IJavaElement field= (IJavaElement)editPart.getAdapter(IJavaElement.class);
	renameAction.run(field);

}
}
