package com.ibm.etools.jbcf.editorpart;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.lang.reflect.Field;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditor;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.*;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.*;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPartService;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.DefaultUndoManager;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.text.IUndoManager;

/**
 * This subclass is required because we need control over the IUndoManager on 
 * the TextViewer.  This is not given up by the superclass so right now we must
 * put our own one in place on the TextViewer that allows us to
 * batch up changes.  
 */
public class VCECompilationUnitEditor extends CompilationUnitEditor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ActivationListener fActivationListener = new ActivationListener();	// Our version of the listener.
	protected JavaVisualEditorPart javaVisualEditorPart;

	public VCECompilationUnitEditor(JavaVisualEditorPart javaVisualEditorPart) {
		this.javaVisualEditorPart = javaVisualEditorPart;
	}

	protected StyledText getTextWidget(){
		return getSourceViewer().getTextWidget();
	}
	
	/*
	 * Test if editing is allowed.
	 */
	public boolean allowEditing() {
		boolean allowEditing = false;
		if (isEditable()) {
			validateState(getEditorInput());
			sanityCheckState(getEditorInput());
			allowEditing = isEditable();
		}
		
		return allowEditing;
	}
	
	public void createPartControl(Composite parent) {	
		super.createPartControl(parent);
				
		// Remove the activation listeners for AbstractTextEditor because we are going to put our own in
		// Ours knows that the JavaVisualEditorPart is our active editor part, not "this".
		// We are removing them so as not to waste resources having them listen.
		try {
			Class abstractTextEditorClass = Class.forName("org.eclipse.ui.texteditor.AbstractTextEditor"); //$NON-NLS-1$
			Field field = abstractTextEditorClass.getDeclaredField("fActivationListener"); //$NON-NLS-1$
			field.setAccessible(true);
			IPartListener activationListener = (IPartListener) field.get(this);
			IWorkbenchWindow window= getSite().getWorkbenchWindow();
			window.getPartService().removePartListener(activationListener);
			Shell shell= window.getShell();
			if (shell != null && !shell.isDisposed())
				shell.removeShellListener((ShellListener) activationListener);
			field.set(this, null);
			
			// Now add in our activation listener.
			window.getPartService().addPartListener(fActivationListener);
			shell.addShellListener(fActivationListener);
		} catch (Exception exc) {
		}			
			
	}

	
	protected boolean isActivePart() {
		boolean isActivePart = super.isActivePart();
		if (isActivePart) return true;
		// See if the active page is a JavaVisualEditor that contains us
		IWorkbenchWindow window= getSite().getWorkbenchWindow();
		IPartService service= window.getPartService();
		return service.getActivePart() ==  javaVisualEditorPart;
	}
	
	/**
	 * Internal part and shell activation listener.
	 * We need our own version because the one added by AbstractTextEditor doesn't
	 * handle our case where the activeEditPart is our main part, not itself.
	 */
	class ActivationListener extends ShellAdapter implements IPartListener {
		
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		private IWorkbenchPart fActivePart;
		private boolean fIsHandlingActivation= false;
		
		/*
		 * @see IPartListener#partActivated(IWorkbenchPart)
		 */
		public void partActivated(IWorkbenchPart part) {
			fActivePart= part;
			handleActivation();
		}
	
		/*
		 * @see IPartListener#partBroughtToTop(IWorkbenchPart)
		 */
		public void partBroughtToTop(IWorkbenchPart part) {
		}
	
		/*
		 * @see IPartListener#partClosed(IWorkbenchPart)
		 */
		public void partClosed(IWorkbenchPart part) {
		}
	
		/*
		 * @see IPartListener#partDeactivated(IWorkbenchPart)
		 */
		public void partDeactivated(IWorkbenchPart part) {
			fActivePart= null;
		}
	
		/*
		 * @see IPartListener#partOpened(IWorkbenchPart)
		 */
		public void partOpened(IWorkbenchPart part) {
		}
	
		/*
		 * @see ShellListener#shellActivated(ShellEvent)
		 */
		public void shellActivated(ShellEvent e) {
			handleActivation();
		}
		
		private void handleActivation() {
			if (fIsHandlingActivation)
				return;
				
			if (fActivePart == javaVisualEditorPart) {
				fIsHandlingActivation= true;
				try {
					safelySanityCheckState(getEditorInput());
				} finally {
					fIsHandlingActivation= false;
				}
			}
		}
	};	

	/**
	 * @see IWorkbenchPart#dispose()
	 */
	public void dispose() {
		super.dispose();
		
		if (fActivationListener != null) {
			IWorkbenchWindow window= getSite().getWorkbenchWindow();
			window.getPartService().removePartListener(fActivationListener);
			Shell shell= window.getShell();
			if (shell != null && !shell.isDisposed())
				shell.removeShellListener(fActivationListener);
			fActivationListener = null;			
		}
	}

	/**
	 * @see AbstractTextEditor#doSetInput(IEditorInput)
	 */
	protected void doSetInput(IEditorInput input) throws CoreException {
		javaVisualEditorPart.getVCEEditor().resetInput(null);	// So that it disconnects
		
		super.doSetInput(input);
		
		javaVisualEditorPart.resetInput(input);
	}

	/*
	 * @see ITextEditor#close
	 */
	public void close(final boolean save) {
		// We have to override because we want to close the main editor instead.
		      			
		Display display= getSite().getShell().getDisplay();
		
		display.asyncExec(new Runnable() {
			public void run() {
				if (getSourceViewer() != null) {
					// check whether editor has not been disposed yet
					getSite().getPage().closeEditor(javaVisualEditorPart, save);
				}
			}
		});
	}

}
