package com.ibm.etools.jbcf.internal;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.lang.reflect.InvocationTargetException;
import java.util.*;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.JavaPluginImages;
import org.eclipse.jdt.ui.wizards.IClasspathContainerPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.jdt.internal.ui.JavaPluginImages;

import com.ibm.etools.cde.palette.Category;
import com.ibm.etools.cde.palette.Entry;
import com.ibm.etools.cde.palette.PalettePackage;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.resource.Context;
import com.ibm.etools.emf.resource.Resource;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.emf.resource.impl.ContextImpl;
import com.ibm.etools.emf.resource.impl.ResourceSetImpl;
import com.ibm.etools.emf.workbench.WorkbenchURIConverterImpl;
import com.ibm.etools.java.JavaVisibilityKind;
import com.ibm.etools.jbcf.JBCFPlugin;

/**
 *  This is part of an experimental API for palette extensions and is not guarantted to be supported in future release
 */
public class ClasspathWizardPage extends WizardPage implements IClasspathContainerPage , IExecutableExtension {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IClasspathEntry fContainerEntry;
	protected String variableName;
	protected IConfigurationElement configData;
	protected static Image fVariableImage;	
	protected static Image fExternalJarImage;
	protected static Image fExternalJarWithSourceImage;	
	
	public ClasspathWizardPage(){
		super("Configure project");
		setImageDescriptor(JBCFPlugin.getWizardTitleImageDescriptor());
	}
	public void createControl(Composite parent) {

		Composite c = new Composite(parent,SWT.NONE);
		setControl(c);
		GridLayout layout = new GridLayout(2,false);
		c.setLayout(layout);
		// Create a double width spacer
//		createSpacer(c, 2);

		// See if we can find configuration data for the extension
		if ( configData == null ) {
			setErrorMessage("The " + variableName + " has not been defined correctly");
			new Label(c,SWT.NONE).setText("There is no com.ibm.etools.jbcf.registrations extension point for " + variableName);
			setControl(c);
			return;
		}

		// Show an image of a variable and let them user know whether the variable will be created or already exists
		// and what the variable points to
		if ( fVariableImage == null ) {
			fVariableImage = org.eclipse.jdt.internal.ui.JavaPlugin.getDefault().getImageRegistry().get(JavaPluginImages.IMG_OBJS_ENV_VAR);
		}		
		new Label(c,SWT.NONE).setImage(fVariableImage);
		// From here we get the variable and also the details to expand the classpath to
		// and show the user these in nice labels and list
		// we also show them the extension to the palette
		Label label = new Label(c,SWT.NONE);
		// See whether or not the variable exists so we can tell the user whether it exists or will be created
		String[] existingVariableNames = JavaCore.getClasspathVariableNames();
		boolean variableExists = false;
		for (int i = 0; i < existingVariableNames.length; i++) {
			if ( existingVariableNames[i].equals(variableName) ) {
				variableExists = true;
				break;
			}
		}
		if ( !variableExists ) {
			label.setText("The Java variable " + variableName + " will be created");
		} else { 
			label.setText("The existing Java variable " + variableName + " will be used");
			// Beneath the variable create a spacer and tell the user where the variable is located
			createSpacer(c,1);
			Label l2 = createIndentedLabel(c);
			l2.setText(JavaCore.getClasspathVariable(variableName).toString());
		}
		// Another spacer before we tell the user about the classpath entries that will be added together with the palette details
//		createSpacer(c,2);

		// Create a group that spans two columns - the width of the composite
		Group g = createGroup(c);
		g.setText("The build path will be updated to include");
		if ( fExternalJarImage == null ) {
			fExternalJarImage = org.eclipse.jdt.internal.ui.JavaPlugin.getDefault().getImageRegistry().get(JavaPluginImages.IMG_OBJS_EXTJAR);
		}
		if ( fExternalJarWithSourceImage == null ) {
			fExternalJarWithSourceImage = org.eclipse.jdt.internal.ui.JavaPlugin.getDefault().getImageRegistry().get(JavaPluginImages.IMG_OBJS_EXTJAR_WSRC);			
		}
		// Get the entries
		IConfigurationElement[] entries = configData.getChildren(JBCFPlugin.PI_EXTEND);
		for (int i = 0; i < entries.length; i++) {
			IConfigurationElement iConfigurationElement = entries[i];
			String runtime = iConfigurationElement.getAttributeAsIs(JBCFPlugin.PI_RUNTIME);
			String source = iConfigurationElement.getAttributeAsIs(JBCFPlugin.PI_SOURCE);			
			String prefix = iConfigurationElement.getAttributeAsIs(JBCFPlugin.PI_SOURCE);					
			// Put a graphic in front of the variable to show whether it is a JAR with or without source
			if ( source == null ) {
				new Label(g,SWT.NONE).setImage(fExternalJarImage);			
			} else { 
				new Label(g,SWT.NONE).setImage(fExternalJarWithSourceImage);				
			}
			// Followed by the variable name
			new Label(g,SWT.NONE).setText(variableName + "/" + runtime);			
			// If we have source put it on the next line and indent it
			if ( source != null ) {
				createSpacer(g,1);
				Label l = createIndentedLabel(g);
				l.setText("with source attachment " + variableName + "/" + source);
			} 
		}

		// Show the user any palette extensions that occur
		String paletteExtension = configData.getAttributeAsIs(JBCFPlugin.PI_PALETTECATS);
		if ( paletteExtension!= null){
			// Read the palette file and crawl it to see what categories and JavaBeans it includes
			ResourceSet rs = new ResourceSetImpl();
			Context ctx = new ContextImpl();
			rs.setContext(ctx);
			ctx.setURIConverter(new WorkbenchURIConverterImpl());
			try { 
				Resource r = rs.getResourceAndLoad(paletteExtension);
				// Get the categories from the palette extension
				PalettePackage palettePackage = (PalettePackage) RefRegister.getPackage(PalettePackage.packageURI);			
				Collection categories = r.getExtent().getObjectsByType(palettePackage.getCategory());
				// Deal with one category right now
				Iterator iter = categories.iterator();
				while(iter.hasNext()){
					Category category = (Category) iter.next();
					Group g2 = createGroup(c);
					// Show the user the palette extensions that exist
					g2.setText("The category \"" + category.getLabel() + "\" will be added to the Visual Editor for Java palette");
					// Now show the individual category entries - as labels for now,  maybe in a list later - JRW
					Iterator groups = category.getGroups().iterator();
					while( groups.hasNext() ) {
						com.ibm.etools.cde.palette.Group group = (com.ibm.etools.cde.palette.Group) groups.next();
						Iterator items = group.getEntries().iterator();
						while (items.hasNext()) {
							Entry element = (Entry) items.next();
							// Create a label that shows the user the palette icon and name
							new Label(g2,SWT.NONE).setImage(element.getSmallIcon());
							new Label(g2,SWT.NONE).setText(element.getLabel());
							// Put some code in here so that if the graphic is an undefined one we tell the user why we think this is
						}
					}
				} 
			} catch ( Exception exc ) {
				// Some kind of error occured.  Tell the user about this
				new Label(c,SWT.NONE).setText("Unable to load the palette extension document " + paletteExtension);
				new Label(c,SWT.NONE).setText("The error is " + exc.getMessage());
				new Label(c,SWT.NONE).setText("For more information look at the .log file in the .metadata directory");
			}
		}
	}

	public boolean finish() {
		
		if ( getErrorMessage() != null ) {
			return false;
		}
		// We need to know the runtime and the source location.  
		if (configData != null) {
			// Just by asking for the classpath variable will create it
			JavaCore.getClasspathVariable(variableName);
			// We are going to create a single container called JAVA_BEANS that has the name of the variable appended to it, e.g. JAVA_BEANS/WIZZO_WIDGETS
			// The ContainerInitializer registered for JAVA_BEANS will then pick this up, read the variable name and then do the appropiate manipulation of the
			// project's build path
			String pathAttribute = configData.getAttributeAsIs(JBCFPlugin.PI_PATH);	
			final IPath containerPath = new Path(ContainerInitializer.JAVA_BEANS).append(pathAttribute);
			// Now append the name of the variable the user has supplied in the initialization data to the container
			WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
				protected void execute(IProgressMonitor monitor) throws CoreException, InterruptedException {
					try {
						monitor.beginTask("Setting up project", 2000);
						setSelection(JavaCore.newContainerEntry(containerPath));
					} finally {
						monitor.done();
					}
				}
			};			
			
			try {
				getContainer().run(true, true, op);
			} catch (InterruptedException e) {
				return false;
			} catch (InvocationTargetException e) {
				if (e.getTargetException() instanceof CoreException) {
					setErrorMessage(((CoreException) e.getTargetException()).getStatus().getMessage());
				}
				else {
					// CoreExceptions are handled above, but unexpected runtime exceptions and errors may still occur.
					setErrorMessage(e.getTargetException().getLocalizedMessage());
				}
				return false;
			}			
		}
		return true;			
	}
	protected IClasspathEntry[] addVariableEntry(IClasspathEntry[] raw, IClasspathEntry newEntry) {
		for (int i = 0; i < raw.length; i++) {
			if (newEntry.equals(raw[i])) {
				return raw;
			}
		}
		
		// Add it at the end.
		IClasspathEntry[] newRaw = new IClasspathEntry[raw.length + 1];
		System.arraycopy(raw, 0, newRaw, 0, raw.length);
		newRaw[raw.length] = newEntry;
		return newRaw;
	}	

	public IClasspathEntry getSelection() {
		return fContainerEntry;
	}
	
	protected Label createIndentedLabel(Composite c){
		Label result = new Label(c,SWT.NONE);
		GridData indentData = new GridData();
		indentData.horizontalIndent = 30;
		result.setLayoutData(indentData);			
		return result;
	}
	
	protected Group createGroup(Composite c){
		Group g = new Group(c,SWT.NONE);
		g.setLayout(new GridLayout(2,false));
		GridData gData = new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL);
		gData.horizontalSpan = 2;
		g.setLayoutData(gData);
		return g;
	}
	
	protected void createSpacer(Composite c, int span){
		Label label = new Label(c,SWT.NONE);
		if ( span > 1 ) {
			GridData data = new GridData();
			data.horizontalSpan = 2;
			label.setLayoutData(data);
		}
	}

	public void setSelection(IClasspathEntry containerEntry) {
		fContainerEntry = containerEntry;
	}
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
		// Parse the initialization data to find all variable/jar combinations.  These are , delimited
		if ( data instanceof String ) {
			variableName = (String) data;
			// Need to collect extension points details.  The variableName will be the variable name that we must ensure is initialize and also correctly 
			// added to the project
			// These variable names are defined by the extension point <extension point="com.ibm.etools.jbcf.registrations"> and collected by JBCFPlugin
			IConfigurationElement[] configDataArray = JBCFPlugin.getPlugin().getRegistrations(new Path(variableName));		
			if ( configDataArray != null && configDataArray.length == 1 ){
				configData = configDataArray[0];
			}
			// See if the config Data has a title 
			setTitle(variableName);
		}
	}
}
