package com.ibm.etools.jbcf.remotevm;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

/**
 * This is a listener for either a property editor dialog
 * or a customizer dialog.
 * 
 * The property editor dialog,
 * which is an internal VCE class, uses this to know
 * which button has been hit and to do the right thing.
 *
 * If a user customizer/Property dialog implements this
 * interface, then it will be added as a listener to the
 * dialog so that it can know if cancel or save has been
 * hit. In that case:
 *   1) On Save it must make all of its changes, if it has
 *      not yet made them.
 *   2) On Revert it must undo all of its changes, if it has
 *      made any yet.
 *
 * Creation date: (11/24/99 3:55:27 PM)
 * @author: Joe Winchester
 */
public interface IPropertyEditorDialogListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
/**
 * If a user customizer or Propery Dialog implements this
 * interface then on revert it must undo all of its changes
 * back to the last save, if it has not yet made them.
 * Creation date: (11/24/99 3:55:40 PM)
 */
void revertPropertyValue();
/**
 * If a user customizer or Propery Dialog implements this
 * interface then on Save it must make all of its changes, 
 * if it has not yet made them.
 * Creation date: (11/24/99 3:55:40 PM)
 */
void savePropertyValue();
}
