package com.ibm.etools.jbcf.vce;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.ui.views.properties.PropertyDescriptor;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.emf.vce.Method;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.widgets.Composite;

import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.common.command.*;
import com.ibm.etools.propertysheet.*;
import org.eclipse.ui.views.properties.*;
import java.util.*;

public class MethodPropertyDescriptor extends PropertyDescriptor implements ICommandPropertyDescriptor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static String METHOD = "Method"; //$NON-NLS-1$
	protected Method vceMethod;	
	protected MethodLabelProvider methodLabelProvider;
		
public MethodPropertyDescriptor(Method aMethod){
	super(METHOD,"Method"); //$NON-NLS-1$
	vceMethod = aMethod;
}
/**
 * Show the user a drop down list of methods
 */	
public CellEditor createPropertyEditor(Composite parent) {
	return new MethodsComboBoxCellEditor(parent);
}
public ILabelProvider getLabelProvider(){
	if ( methodLabelProvider == null ) {
		methodLabelProvider = new MethodLabelProvider();
	}
	return methodLabelProvider;
}
/**
 * The source is the EventToMethodPropertySourceAdapter, the value is the MethodProxy
 * The structure is the vce document is
 * <nodes vce:Event xmi:id="e1">
 * <nodes vce:Method xmi:id="m1" method="Dialog.setTitle"/>
 * <connections vce:EventToMethodConnection sourceNode="e1" targetNode="m1"/>
 * Parameters are done through connections, e.g. for constants
 * <nodes vce:BeanConstant xmi:id="c1"/>
 * <connections vce:ParameterConnection sourceNode="c1" targetNode="m1"/>
 * or for parameters from connections
 * <nodes vce:Method xmi:id="m2" method="Button.getText"/>
 * <connections vce:ParameterConnection sourceNode="m2" targetNode="m1"/>
 */
public Command setValue(IPropertySource source, Object value){
	// Because of the odd way we use ICommandPropertyDescriptors for this feature
	// and for its parms we want to reject a change where just the parm is being 
	// changed
	// We cannot return null because the compound command is unwrapped and returns
	// a non executable command which stops the parm change from being applied
	// so just return a do nothing command instead
	if ( !(value instanceof com.ibm.etools.emf.ecore.EBehavioralFeature) ) {
		return new AbstractCommand(){
			public boolean canExecute(){
				return true;
			}
			public void redo(){
				// Do nothing
			}
			public void execute(){
				// Do nothing
			}
		};
	}
	JBCFPlugin.log("MethodCommand PS source=" + source + " value=" + value, JBCFMsgLogger.LOG_FINE); //$NON-NLS-1$ //$NON-NLS-2$
	// First - look at the existing connection.  If it has a method then
	// generate a command to change the method of this method
	// Also, if there are any other connections whose target is the existing method
	// then remove these and remove the source nodes of these connections as well	
	// This is because they may no longer be valid
	com.ibm.etools.cde.CDEHack.fixMe("Needs more thought - maybe morph connections - VAJ keeps connection parms"); //$NON-NLS-1$
	// Find the existing method and change the method it points to
	CommandBuilder builder = new CommandBuilder("Change method of connection"); //$NON-NLS-1$
	builder.applyAttributeSetting(vceMethod,JBCFVCEContext.SF_METHODNODE_METHOD,value);
	// Do something with parms now - check the existing ones and their types
	return builder.getCommand();
}
public Command resetValue(IPropertySource source){
	return null;
}
}