package com.ibm.etools.jbcf.vce;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.ContentsGraphicalEditPart;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.gef.*;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.proxy.ProxyFactoryRegistry;
import com.ibm.etools.cde.VisualInfoXYLayoutEditPolicy;
import java.util.*;

import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyHost;
import com.ibm.etools.jbcf.JavaBeanGraphicalEditPart;
import com.ibm.etools.jbcf.emf.vce.*;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.emf.ref.RefObject;
/**
 * Subclass Composition Graphical Edit Part for a bean subclass.
 */
public class SubclassCompositionComponentsGraphicalEditPart extends ContentsGraphicalEditPart implements IEMFListener {


	public SubclassCompositionComponentsGraphicalEditPart(Object model) {
		setModel(model);
	}

	protected void createEditPolicies() {
		VisualInfoXYLayoutEditPolicy ep = new VisualInfoXYLayoutEditPolicy(new SubclassCompositionContainerPolicy(EditDomain.getEditDomain(this)));
		ep.setZoomable(true);
		installEditPolicy(EditPolicy.LAYOUT_ROLE, ep);
	}
	
	protected List getModelChildren() {
		BeanSubclassComposition comp = (BeanSubclassComposition) getModel();
		if (comp != null) {
			List components = comp.getComponents();
			if (comp.isSetThisPart()) {
				ArrayList children = new ArrayList(components.size()+1);
				children.add(comp.getThisPart());
				children.addAll(components);
				return children;
			} else
				return components;
		} else
			return Collections.EMPTY_LIST;
	}
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter compositionAdapter;
	
	public void activate() {
		super.activate();
		if (getModel() != null) 
			((BeanSubclassComposition) getModel()).addAdapter(getModelAdapter());
	}
	
	public void deactivate() {
		super.deactivate();
		if (getModel() != null)
			((BeanSubclassComposition) getModel()).removeAdapter(getModelAdapter());
	}
	
	protected Adapter getModelAdapter() {
		if (compositionAdapter == null) {
			// Create an adaptor that is used to listen to changes from the
			// MOF model object.  The MOF notifications are used by this edit part to stay updated.
			sfComponents = ((BeanSubclassComposition) getModel()).ePackageVce().getBeanSubclassComposition_Components();
			sfThisPart = ((BeanSubclassComposition) getModel()).ePackageVce().getBeanSubclassComposition_ThisPart();			
			compositionAdapter = new EMFListenerAdapter(this, true);
		}
		return compositionAdapter;
	}
	
	private RefStructuralFeature sfComponents, sfThisPart;
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sfComponents || sf == sfThisPart)
			refreshChildren();
	}
	
	protected EditPart createChild(Object model) {
		// If the model object is in error then we create a special placeholder
		if ( model instanceof IJavaInstance){
			IBeanProxyHost modelBeanProxy = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)model);
			// If we have a fatal error then we use a special graphical edit part
			// We must NOT use the one defined on the class as for some classes, e.g. Component it
			// has a lot of behavior that relies on the live JavaBean being present
			if(modelBeanProxy.getErrorStatus() == IBeanProxyHost.ERROR_SEVERE){
				// The DefaultGraphicalEditPart will show the icon and its label provider will indicate to the
				// user that the JavaBean failed to be created
				JavaBeanGraphicalEditPart result = new JavaBeanGraphicalEditPart();
				result.setModel(model);
				return result;
			}
		}
		EditDomain dom = EditDomain.getEditDomain(this);
		return CDEUtilities.getEditPartFactory(this).createEditPart(model);
	}
}
