package com.ibm.etools.jbcf.vce;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.editparts.AbstractTreeEditPart;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.notify.*;
import java.util.*;

import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.cde.TreeContainerEditPolicy;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
/**
 * Composition Graphical Edit Part that instantiates and disposes bean proxies
 */
public class SubclassCompositionComponentsTreeEditPart extends AbstractTreeEditPart implements IEMFListener {


	public SubclassCompositionComponentsTreeEditPart(Object model) {
		setModel(model);
	}
	

	protected List getModelChildren() {
		BeanSubclassComposition comp = (BeanSubclassComposition) getModel();
		if (comp != null) {
			List components = comp.getComponents();
			if (comp.isSetThisPart()) {
				ArrayList children = new ArrayList(components.size()+1);
				children.add(comp.getThisPart());
				children.addAll(components);
				return children;
			} else
				return components;
		} else
			return Collections.EMPTY_LIST;
	}
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter compositionAdapter;
	
	public void activate() {
		super.activate();
		if (getModel() != null)
			((BeanSubclassComposition) getModel()).addAdapter(getModelAdapter());
	}
	
	public void deactivate() {
		super.deactivate();
		if (getModel() != null)
			((BeanSubclassComposition) getModel()).removeAdapter(getModelAdapter());
	}
	
	protected Adapter getModelAdapter() {
		if (compositionAdapter == null) {
			// Create an adaptor that is used to listen to changes from the
			// MOF model object.  The MOF notifications are used by this edit part to stay updated.
			sfComponents = ((BeanSubclassComposition) getModel()).ePackageVce().getBeanSubclassComposition_Components();
            sfThisPart = ((BeanSubclassComposition) getModel()).ePackageVce().getBeanSubclassComposition_ThisPart();
			compositionAdapter = new EMFListenerAdapter(this, true);
		}
		return compositionAdapter;
	}
	
	private RefStructuralFeature sfComponents, sfThisPart;
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sfComponents || sf==sfThisPart)
			refreshChildren();
	}
	
	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		return CDEUtilities.getEditPartFactory(this).createEditPart(model);
	}
	
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.TREE_CONTAINER_ROLE, new TreeContainerEditPolicy(new SubclassCompositionContainerPolicy(EditDomain.getEditDomain(this))));
	}
	/**
	 * Initializes the entire state of the EditPart.
	 */
	protected void doInitialize(){	
		super.doInitialize();
		TreeItem[] items = ((Tree)getWidget()).getItems();
		for(int i=0;i<items.length;i++){
			items[i].setExpanded(true);
		}
	}
}
