package com.ibm.etools.jbcf.vce;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFContainerPolicy;
import com.ibm.etools.jbcf.emf.vce.VcePackage;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.gef.Request;
import com.ibm.etools.gef.requests.ForwardedRequest;
import com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.gef.requests.ChangeBoundsRequest;
import java.util.*;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.requests.GroupRequest;
/**
 * Container Policy for Bean Subclass Compositions.
 */
public class SubclassCompositionContainerPolicy extends EMFContainerPolicy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public SubclassCompositionContainerPolicy(EditDomain domain) {
		super(((VcePackage) RefRegister.getPackage(VcePackage.packageURI)).getBeanSubclassComposition_Components(), domain);
	}
	
	protected BeanSubclassComposition getComposition() {
		return (BeanSubclassComposition) container;
	}
	
	protected Command getDeleteDependentCommand(Object child, RefStructuralFeature containmentSF) {
		if (child == getComposition().getThisPart())
			return UnexecutableCommand.INSTANCE;	// Can't delete the this part.
		return super.getDeleteDependentCommand(child, containmentSF);
	}
	
	public Command getMoveChildrenCommand(List children, Object position) {
		Object thisPart = getComposition().getThisPart();
		if (thisPart != null) {
			Iterator itr = children.iterator();
			while (itr.hasNext()) {
				if (itr.next() == thisPart)
					return UnexecutableCommand.INSTANCE;	// Can't move the this part.
			}
		}
		return super.getMoveChildrenCommand(children, position);
	}
	
	public Command getOrphanChildrenCommand(List children) {	
		Object thisPart = getComposition().getThisPart();
		if (thisPart != null) {
			Iterator itr = children.iterator();
			while (itr.hasNext()) {
				if (itr.next() == thisPart)
					return UnexecutableCommand.INSTANCE;	// Can't orphan the this part.
			}
		}
		return super.getOrphanChildrenCommand(children);
	}


}