package com.ibm.etools.jbcf.vce;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import com.ibm.etools.jbcf.CompositionProxyAdapter;
import com.ibm.etools.jbcf.JBCFGraphicalEditorPart;
import com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPartViewer;
import com.ibm.etools.gef.palette.*;
import com.ibm.etools.gef.ui.actions.ActionRegistry;
import com.ibm.etools.gef.ui.parts.TreeViewer;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.Resource;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.decorators.ClassDescriptorDecorator;
import com.ibm.etools.cde.emf.ClassDescriptorDecoratorPolicy;
import com.ibm.etools.cde.palette.*;
import com.ibm.etools.cde.properties.*;
import com.ibm.etools.gef.DefaultEditDomain;
/**
 * Graphical editor part for the VCE.
 */
public class VCEGraphicalEditorPart extends JBCFGraphicalEditorPart {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected List fPaletteToolbarActions;

	protected EditPart getEditPart(RefObject rootModel) {
		return new SubclassCompositionComponentsGraphicalEditPart(rootModel);
	}

	protected EditPart getTreeEditPart(RefObject rootModel) {
		return new SubclassCompositionComponentsTreeEditPart(rootModel);
	}

	protected DefaultEditDomain createEditDomain() {
		EditDomain domain = (EditDomain) super.createEditDomain();
		// Register the descriptor for handling name in composition.
		NameInCompositionPropertyDescriptor desc = new NameInCompositionPropertyDescriptor(VCEMessages.getString("nameInComposition.displayName"), new IVJNameValidator()); //$NON-NLS-1$
		desc.setReadOnly(true);	// For now these are read-only because code-gen can't handle rename (requires changing field name, method name, all references to these names,...)
		domain.registerKeyedPropertyDescriptor(
			NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY,
			desc);
		// Make the default add annotations command be the one to make names unique.
		try {
			domain.setDefaultAddAnnotationsCommandClass(AddAnnotationsWithName.class);
		} catch(ClassCastException e) {
			// Can't happen, but it throws it
		} catch(NoSuchMethodException e) {
			// Can't happen, but it throws it
		}
		
		ClassDescriptorDecoratorPolicy policy = ClassDescriptorDecoratorPolicy.getPolicy(domain);
		ClassDescriptorDecorator cd = (ClassDescriptorDecorator) policy.getDefaultDecorator(ClassDescriptorDecorator.class);
		cd.setLabelProviderClassname("com.ibm.etools.cde/com.ibm.etools.cde.properties.DefaultLabelProviderWithName"); //$NON-NLS-1$		
		return domain;
	}
	
	/*
	 * Used to validate that name starts with "ivj". Currently can only handle that.
	 */
	protected static class IVJNameValidator implements ICellEditorValidator {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		public String isValid(Object value) {
			if (value instanceof String && ((String) value).startsWith("ivj")) //$NON-NLS-1$
				return null;	// It's valid
			return VCEMessages.getString("ivjValidator.notstartIVJ"); //$NON-NLS-1$
		}
	}
	
	public List getPaletteToolbarActions(){
	
		if ( fPaletteToolbarActions == null ) {
			fPaletteToolbarActions = new ArrayList(3);
			// Get the palette root
			EditDomain editDomain = (EditDomain)getDomain();
			PaletteRoot paletteRoot = (PaletteRoot)editDomain.getData(PaletteRoot.class);
			// Get the control group.  
			com.ibm.etools.cde.palette.Group controlGroup = null;
			if (paletteRoot instanceof Palette) {
				controlGroup = ((Palette) paletteRoot).getControlGroup();
				if ( controlGroup != null) {
					// Iterate over the control group
					Iterator iter = controlGroup.getChildren().iterator();
					while (iter.hasNext()) {
						PaletteEntry entry = (PaletteEntry) iter.next();
						// If the entry is a tool one then create an item for the toolbar
							if (entry instanceof AbstractToolEntry) {
							AbstractToolEntry toolEntry = (AbstractToolEntry) entry;
							PaletteToolEntryAction action = null;
							// One of the entries is the choose bean that has a drop down list
							// with the list of JavaBeans to add
							// Hard code this to the id of choosebean right now - This needs a better way in future
								if ("choosebean".equals(toolEntry.refID())) { //$NON-NLS-1$
								action = new PaletteCategoriesToolEntryAction(toolEntry, paletteRoot);
							} else {
								action = new PaletteToolEntryAction(toolEntry);
							}
							action.setEditDomain(editDomain);
							fPaletteToolbarActions.add(action);
						}
					}
				}
			}
		}
		return fPaletteToolbarActions;	
	}
	
	/**
	 * So that the toolbar actions can work with the JavaBeans outline viewer
	 * as well as the graphical edit parts we must contribute global actions handlers
	 * for them so the RetargetActions that are placed on the tool bar manager can operate
	 */
	class VCEContentOutlinePage extends JBCFOutlinePage {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		VCEContentOutlinePage(EditPartViewer aViewer){
			super(aViewer);
		}
		public void setActionBars(IActionBars actionBars) {
			super.setActionBars(actionBars);
					
			// Get the toolbar actions
			List toolbarActions = VCEGraphicalEditorPart.this.getPaletteToolbarActions();
			Iterator iter = toolbarActions.iterator();
			while ( iter.hasNext() ) {
				IAction action = (IAction)iter.next();
				actionBars.setGlobalActionHandler(action.getId(),action);
			}
		}
	}
	
	protected EMFContentOutlinePage createOutlinePage(){
		return new VCEContentOutlinePage(new TreeViewer());
	}	
	/**
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		if (fPaletteToolbarActions != null) {
			for (int i=0; i<fPaletteToolbarActions.size(); i++) {
				if (fPaletteToolbarActions.get(i) instanceof PaletteCategoriesToolEntryAction)
					((PaletteCategoriesToolEntryAction) fPaletteToolbarActions.get(i)).dispose();
			}
		}
		super.dispose();
	}

}