package com.ibm.etools.jbcf.vce;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;

import java.io.IOException;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.preference.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.propertysheet.PropertysheetMessages;

public class VCEPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static String[] LABELS = new String[] {VCEMessages.getString("PreferencePage.LookAndFeel.Default"),VCEMessages.getString("PreferencePage.LookAndFeel.Metal"),VCEMessages.getString("PreferencePage.LookAndFeel.Windows"),VCEMessages.getString("PreferencePage.LookAndFeel.Motif")}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	public static String[] CLASSES = new String[] {
		"javax.swing.plaf.metal.MetalLookAndFeel", //$NON-NLS-1$
		"com.sun.java.swing.plaf.windows.WindowsLookAndFeel", //$NON-NLS-1$
		"com.sun.java.swing.plaf.motif.MotifLookAndFeel" //$NON-NLS-1$
	};
	protected Preferences fStore;
	protected Label lookAndFeelLabel;
	protected List lookAndFeelList;
	protected Button showWindowCheckBox;
	protected Button showXMLTextCheckBox;
	protected Button splitRadioButton;
	protected Button notebookRadioButton;
	protected Button showNotebookGEFPaletteButton;
	protected Button showSplitPaneGEFPaletteButton;	
    protected Button generateExpressionComment;
//    protected Button requireIVJforComponents ;
    protected Button requireIVJforNonComponents ;
    protected Button generateTryCatchBlock;
	protected Text   sourceSyncDelayText ;
	protected Text   sourceToVisual ;
	protected Text  fSourceToJavaBeansTotalTimeLabel;
	protected Text  fSourceToJavaBeansTimeLabel;
	protected Group  sourceSyncGrp;	
	
	protected static Image fJavaBeansToSourceImage;
	protected static Image fSourceToJavaBeansImage;
		
protected Label createLabel(Composite group, String aLabelString, Image aLabelImage) {
	Label label = new Label(group, SWT.LEFT);
	if ( aLabelImage != null ) {
		label.setImage(aLabelImage);
	} else {
		label.setText(aLabelString);
	}
	GridData data = new GridData();
	label.setLayoutData(data);
	return label;
} 

private Button createCheckBox(Composite group, String label, int indent) {
	Button button = new Button(group, SWT.CHECK | SWT.LEFT);
	button.setText(label);
	GridData data = new GridData();
	data.horizontalIndent = indent;
	button.setLayoutData(data);
	return button;
} 

protected Group createGroup(Composite aParent, String title, int numColumns){
	Group group = new Group(aParent,SWT.NONE);
	group.setText(title);
	GridLayout gridLayout = new GridLayout();
	gridLayout.numColumns = numColumns;
	group.setLayout(gridLayout);
	GridData data = new GridData();
	data.verticalAlignment = GridData.FILL;
	data.horizontalAlignment = GridData.FILL;
	data.grabExcessHorizontalSpace = true;
	group.setLayoutData(data);
	return group;
}

private void createSpacer(Composite parent) {
	Label spacer = new Label(parent, SWT.NONE);
	GridData data = new GridData();
	data.horizontalAlignment = GridData.FILL;
	data.verticalAlignment = GridData.BEGINNING;
	spacer.setLayoutData(data);
}	

protected List createList(Composite group){
	List list = new List(group, SWT.BORDER);
	GridData data = new GridData();
	data.horizontalAlignment = GridData.FILL;
	data.verticalAlignment = GridData.BEGINNING;
	list.setLayoutData(data);
	return list;
}
protected Button createRadioButton(Composite parent, String label){
	Button button = new Button(parent, SWT.RADIO | SWT.LEFT);
	button.setText(label);
	GridData data = new GridData();
	button.setData(data);
	return button;
}
protected Text createText(Composite parent,int noCharacters, int span) {
    
    Text  text = new Text(parent, SWT.SINGLE | SWT.BORDER) ;    
    GridData data = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);   
    data.widthHint = convertWidthInCharsToPixels(noCharacters) ;
    data.horizontalSpan = span;
    text.setLayoutData(data) ;
	return text ;
	
}

protected Text createLabelText(Composite parent, String label,int noCharacters) {
    
	Composite c = new Composite(parent,SWT.NONE);
	GridLayout gridLayout = new GridLayout() ;
    gridLayout.marginHeight = 0 ;    
	gridLayout.numColumns = 2 ;
//	c.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
	c.setLayoutData(new GridData());	
	c.setLayout(gridLayout);
    
    Label l = createLabel(c,label,null) ;
    l.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_CENTER)) ;
    Text  text = new Text(c, SWT.SINGLE | SWT.BORDER) ;    
    GridData data = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);   
    data.widthHint = convertWidthInCharsToPixels(noCharacters) ;
    text.setLayoutData(data) ;
	return text ;
}
public static Image getJavaBeansToSourceImage(){
	
	if ( fJavaBeansToSourceImage == null ) {
		fJavaBeansToSourceImage = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf/icons/full/cview16/gui2src.gif");//$NON-NLS-1$
	}
	return fJavaBeansToSourceImage;
}
public static Image getSourceToJavaBeansImage(){
	if ( fSourceToJavaBeansImage == null ) {
		fSourceToJavaBeansImage = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf/icons/full/cview16/src2gui.gif");//$NON-NLS-1$		
	}
	return fSourceToJavaBeansImage;
}
protected Control createContents(Composite parent) {
	
	Composite composite = new Composite(parent, SWT.NULL);
	composite.setLayout(new GridLayout());
	composite.setLayoutData(
		new GridData(GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL));
	
	Group group = createGroup(composite,VCEMessages.getString("PreferencePage.EditorGroup.Title"),2); //$NON-NLS-1$
	splitRadioButton = createRadioButton(group,VCEMessages.getString("PreferencePage.EditorGroup.SplitPaneOrientation")); //$NON-NLS-1$
	showSplitPaneGEFPaletteButton = createCheckBox(group,VCEMessages.getString("PreferencePage.EditorGroup.PaletteInEditor.InSplitPane"),0); //$NON-NLS-1$
	notebookRadioButton = createRadioButton(group,VCEMessages.getString("PreferencePage.EditorGroup.UseNoteBook")); //$NON-NLS-1$
	showNotebookGEFPaletteButton = createCheckBox(group,VCEMessages.getString("PreferencePage.EditorGroup.PaletteInEditor.InNotebook"),0); //$NON-NLS-1$
	
	// Only enable showing the GEF palette in notebook mode
	SelectionListener listener = new SelectionListener(){
		public void widgetDefaultSelected(SelectionEvent event){
			showNotebookGEFPaletteButton.setEnabled(notebookRadioButton.getSelection());
			showSplitPaneGEFPaletteButton.setEnabled(splitRadioButton.getSelection());
		}
		public void widgetSelected(SelectionEvent event){
			showNotebookGEFPaletteButton.setEnabled(notebookRadioButton.getSelection());
			showSplitPaneGEFPaletteButton.setEnabled(splitRadioButton.getSelection());
		}
	};
	
	notebookRadioButton.addSelectionListener(listener);
	splitRadioButton.addSelectionListener(listener);

	createSpacer(composite);

	lookAndFeelLabel = createLabel(composite, VCEMessages.getString("PreferencePage.LookAndFeel.Title"),null); //$NON-NLS-1$
	lookAndFeelList = createList(composite);
	
//	Group codeGenGroup = createGroup(composite,"",1); //$NON-NLS-1$	
	
	Group codeGenGroup_Gen = createGroup(composite,VCEMessages.getString("PreferencePage.CodeGen.ParsingGeneration_Style_1"),1); 	 //$NON-NLS-1$

    generateExpressionComment = createCheckBox(codeGenGroup_Gen,VCEMessages.getString("PreferencePage.NewExpressionCommentPrompt"),0);  //$NON-NLS-1$
    generateTryCatchBlock = createCheckBox(codeGenGroup_Gen, VCEMessages.getString("PreferencePage.GenerateTryCatchBlock"),0); //$NON-NLS-1$
    
    Group codeGenGroup_Parsing = createGroup(composite,VCEMessages.getString("PreferencePage.CodeGen.ParsingParsing_Filters_2"),1);  //$NON-NLS-1$
    
//    requireIVJforComponents = createCheckBox(codeGenGroup_Parsing,VCEMessages.getString("PreferencePage.CodeGen.ParsingRequire_prefix_of___ivj___for_AWT/Swing_component_field_names_3"),0);   //$NON-NLS-1$
    requireIVJforNonComponents = createCheckBox(codeGenGroup_Parsing,VCEMessages.getString("PreferencePage.CodeGen.ParsingRequire_prefix_of___ivj___for_objects_other_than_AWT/Swing_component_field_names_4"),0);   //$NON-NLS-1$
    
    
	createSpacer(parent);
	
	// Create a listener to see when the values in text boxes changes so we can trigger revalidation
	ModifyListener modifyListener = new ModifyListener() {
		public void modifyText(ModifyEvent e) {
			// Validate that the value in the sourceToVisual is an integer between 1 and 9;
			validateFields();
		}
	};
	 
	sourceSyncGrp = createGroup(composite,VCEMessages.getString("PreferencePage.CodeGen.Source_Synchronization_Delay"),6) ;	 //$NON-NLS-1$
	// The group is 5 wide.
	
	// For JavaBeansToSource show an image of the up arrow, then the label, text and spacers
	createLabel(sourceSyncGrp,null,getJavaBeansToSourceImage());	
	createLabel(sourceSyncGrp,VCEMessages.getString("PreferencePage.CodeGen.JavaBeansToSource"),null); //$NON-NLS-1$
	sourceSyncDelayText = createText(sourceSyncGrp,11,1) ; //$NON-NLS-1$
	sourceSyncDelayText.setTextLimit(9);
	createLabel(sourceSyncGrp,"",null); //$NON-NLS-1$
	createLabel(sourceSyncGrp,"",null); //$NON-NLS-1$
	createLabel(sourceSyncGrp,"",null); //$NON-NLS-1$	
		
	// The JavaBeansToSource are 6 wide in the group
	// First the down arrow, then the label, then a * and the text box
	createLabel(sourceSyncGrp,null,getSourceToJavaBeansImage());		
	createLabel(sourceSyncGrp,VCEMessages.getString("PreferencePage.CodeGen.SourceToJavaBeans"),null); //$NON-NLS-1$
	fSourceToJavaBeansTimeLabel = createText(sourceSyncGrp,11,1);	 //$NON-NLS-1$
	fSourceToJavaBeansTimeLabel.setEditable(false);
	createLabel(sourceSyncGrp,"*",null);	 //$NON-NLS-1$
	sourceToVisual = createText(sourceSyncGrp,1,1) ; //$NON-NLS-1$
	sourceToVisual.addModifyListener(modifyListener);
	sourceToVisual.setTextLimit(1);
	fSourceToJavaBeansTotalTimeLabel = createText(sourceSyncGrp,14,1); //$NON-NLS-1$
	fSourceToJavaBeansTotalTimeLabel.setEditable(false);

	sourceSyncDelayText.addModifyListener(modifyListener);

	// These are only shown if a special debug mode is set up
	// with a .options whereby the live AWT/Swing windows can be seen and also
	// the XML text for the JVE object model can be seen, both of which help debugging
		
	showWindowCheckBox = createCheckBox(parent,VCEMessages.getString("PreferencePage.ShowLiveWindow"),15); //$NON-NLS-1$
	showXMLTextCheckBox = createCheckBox(parent,VCEMessages.getString("PreferencePage.ShowXMLText"),15); //$NON-NLS-1$
    if (!isLiveWindow())
	  showWindowCheckBox.setVisible(false);
    if (!isXMLText())
  	  showXMLTextCheckBox.setVisible(false);

	for(int i=0;i<LABELS.length;i++){
		lookAndFeelList.add(LABELS[i]);
	}	
	
	// set initial values
	fStore= VCEPlugin.getPlugin().getPluginPreferences();
	String lookAndFeelClass = fStore.getString(VCEPlugin.SWING_LOOKANDFEEL);
	if ( lookAndFeelClass == null | lookAndFeelClass.length() == 0 ) {
		lookAndFeelList.setSelection(0);
	} else {
		for(int i=0;i<CLASSES.length;i++){
			if (CLASSES[i].equals(lookAndFeelClass)){
				lookAndFeelList.setSelection(i+1);
				break;
			}
		}
	}
	showWindowCheckBox.setSelection(fStore.getBoolean(VCEPlugin.SHOW_LIVE_WINDOW));
	showXMLTextCheckBox.setSelection(CDEPlugin.getPlugin().getPluginPreferences().getBoolean(CDEPlugin.SHOW_XML));
    generateExpressionComment.setSelection(fStore.getBoolean(VCEPlugin.GENERATE_COMMENT));
    generateTryCatchBlock.setSelection(fStore.getBoolean(VCEPlugin.GENERATE_TRY_CATCH_BLOCK));
	showNotebookGEFPaletteButton.setSelection(fStore.getBoolean(VCEPlugin.NOTEBOOK_SHOW_GEF_PALETTE));
	showSplitPaneGEFPaletteButton.setSelection(fStore.getBoolean(VCEPlugin.SPLITPANE_SHOW_GEF_PALETTE));
	sourceSyncDelayText.setText(fStore.getString(VCEPlugin.SOURCE_SYNC_DELAY).length()==0 ?
	                            Integer.toString(VCEPlugin.DEFAULT_SYNC_DELAY) : fStore.getString(VCEPlugin.SOURCE_SYNC_DELAY)) ;	
	sourceToVisual.setText(fStore.getString(VCEPlugin.SOURCE_DELAY_FACTOR).length()==0 ?
	                            Integer.toString(VCEPlugin.DEFAULT_L2R_FACTOR) : fStore.getString(VCEPlugin.SOURCE_DELAY_FACTOR)) ;	
	calculateTotalSourceToVisualTime();	                            
	boolean showAsNotebook = fStore.getBoolean(VCEPlugin.NOTEBOOK_PAGE);
	splitRadioButton.setSelection(!showAsNotebook);
	notebookRadioButton.setSelection(showAsNotebook);
	showNotebookGEFPaletteButton.setEnabled(notebookRadioButton.getSelection());
	showSplitPaneGEFPaletteButton.setEnabled(splitRadioButton.getSelection());
	
	
	
//	requireIVJforComponents.setSelection(fStore.getString(VCEPlugin.REQUIRE_IVJ_COMPONENTS).length()==0 ?
//	                                     false : fStore.getBoolean(VCEPlugin.REQUIRE_IVJ_COMPONENTS)) ;		
	requireIVJforNonComponents.setSelection(true) ;
	requireIVJforNonComponents.setEnabled(false) ;
	
	return composite;
}
protected void validateFields(){
	// Validate that the sourceSyncDelayText is an integer greater than the default of 500ms
	boolean isValid = true ;
    try {
		int val = Integer.parseInt(sourceSyncDelayText.getText()) ;
		if ( val < VCEPlugin.DEFAULT_SYNC_DELAY ) {
			isValid = false;
			setErrorMessage(VCEMessages.getString("PreferencePage.CodeGen.Error.DelayTimeMinimum")); //$NON-NLS-1$
			fSourceToJavaBeansTotalTimeLabel.setText("="); //$NON-NLS-1$
			setValid(false);
		}
	} catch (NumberFormatException ex) {
		isValid = false;
		setErrorMessage(VCEMessages.getString("PreferencePage.CodeGen.Error.DelayTimeMustBeInteger")); //$NON-NLS-1$
		setValid(false);		
		fSourceToJavaBeansTotalTimeLabel.setText("=");		 //$NON-NLS-1$
	}
	// If we are valid check that the sourceToVisual delay factor is an integer between 1 and 9
	if ( isValid ) {
		try {
			int val = Integer.parseInt(sourceToVisual.getText());
			if ( val < 1 || val > 9 ) {
				isValid = false;
				setErrorMessage(VCEMessages.getString("PreferencePage.CodeGen.Error.DelayTimeValidRange")); //$NON-NLS-1$
				fSourceToJavaBeansTotalTimeLabel.setText("="); //$NON-NLS-1$
				setValid(false);				
			}
		} catch ( NumberFormatException ex){
			isValid = false;
			setErrorMessage(VCEMessages.getString("PreferencePage.CodeGen.Error.DelayTimeValidRange")); //$NON-NLS-1$
			fSourceToJavaBeansTotalTimeLabel.setText("=");			 //$NON-NLS-1$
			setValid(false);
		}
	}
	// If we got no errors then clear the error message and refresh the total source to visual time
	if ( isValid ) {
		// The next step is to calculate the total time which is the delay time * the factor
		// it is possible that is an integer that is too large.
		boolean isValidTotalTime = calculateTotalSourceToVisualTime();
		if ( isValidTotalTime ) {
			setValid(true);
			setErrorMessage(null);			
		} else {
			setValid(false);
		}
	}
}
protected boolean calculateTotalSourceToVisualTime(){
	try {
		int javaBeansToSourceDelayTime = Integer.parseInt(sourceSyncDelayText.getText());
		int increaseFactorForSourceToJavaBeans = Integer.parseInt(sourceToVisual.getText());
		Integer total = new Integer(javaBeansToSourceDelayTime * increaseFactorForSourceToJavaBeans); //$NON-NLS-1$
		// I can find no way of seeing whether the result has overflowed or not.  Therefore do a division to see whether the
		// answer is correct
		if ( total.intValue() / increaseFactorForSourceToJavaBeans == javaBeansToSourceDelayTime ) {
			fSourceToJavaBeansTotalTimeLabel.setText("= " + total); //$NON-NLS-1$
			fSourceToJavaBeansTimeLabel.setText(Integer.toString(javaBeansToSourceDelayTime));
			// Because we have change the contents of the label its size needs updating as its preferred size has changed
			// A layout on its container will force the layout to occur
			sourceSyncGrp.layout();
			return true;			
		} else { 
			// For the error to show grab the one used by the property sheet to show an invalid integer
			setErrorMessage(PropertysheetMessages.getString(PropertysheetMessages.NOT_INTEGER));			
			fSourceToJavaBeansTotalTimeLabel.setText("= " +  total); //$NON-NLS-1$
			sourceSyncGrp.layout();
			return false;
		}
	} catch ( NumberFormatException exc ) {
		// This should not occur because the number have already been validated
		return false;
	}
}
public boolean performOk(){
	
	// Save the Swing look and feel
	String lookAndFeelClass = ""; //$NON-NLS-1$
	if(lookAndFeelList.getSelectionIndex() != 0){
		lookAndFeelClass = CLASSES[lookAndFeelList.getSelectionIndex()-1];
	}
	String existingLookAndFeelClass = fStore.getString(VCEPlugin.SWING_LOOKANDFEEL);
	if ( !lookAndFeelClass.equals(existingLookAndFeelClass)) {
		fStore.setValue(VCEPlugin.SWING_LOOKANDFEEL,lookAndFeelClass);
	}
	// Save whether or not to show the live window
	fStore.setValue(VCEPlugin.SHOW_LIVE_WINDOW,showWindowCheckBox.getSelection());
	// Save whether to show the VCE and Source as a split pane or notebook
	fStore.setValue(VCEPlugin.NOTEBOOK_PAGE,notebookRadioButton.getSelection());
	fStore.setValue(VCEPlugin.NOTEBOOK_SHOW_GEF_PALETTE,showNotebookGEFPaletteButton.getSelection());
	fStore.setValue(VCEPlugin.SPLITPANE_SHOW_GEF_PALETTE,showSplitPaneGEFPaletteButton.getSelection());
	
	// Save whether to show the dialog that prompts the user to open the BeansList and property sheet
	// if they are not already opened
    fStore.setValue(VCEPlugin.GENERATE_COMMENT,generateExpressionComment.getSelection());
    fStore.setValue(VCEPlugin.GENERATE_TRY_CATCH_BLOCK, generateTryCatchBlock.getSelection());
	fStore.setValue(VCEPlugin.SOURCE_SYNC_DELAY,sourceSyncDelayText.getText()) ;
	fStore.setValue(VCEPlugin.SOURCE_DELAY_FACTOR,sourceToVisual.getText()) ;
//	fStore.setValue(VCEPlugin.REQUIRE_IVJ_COMPONENTS,requireIVJforComponents.getSelection()) ;

	VCEPlugin.getPlugin().savePluginPreferences();
	
	// Do the CDE one now
	Preferences cdePreferenceStore = CDEPlugin.getPlugin().getPluginPreferences();
	boolean existingShowXMLText = cdePreferenceStore.getBoolean(CDEPlugin.SHOW_XML);
	if ( existingShowXMLText != showXMLTextCheckBox.getSelection() ) {
		cdePreferenceStore.setValue(CDEPlugin.SHOW_XML,showXMLTextCheckBox.getSelection());
	}
	
	CDEPlugin.getPlugin().savePluginPreferences();
		
	return true;
}
protected void performDefaults() {
	Preferences store = VCEPlugin.getPlugin().getPluginPreferences();
	showXMLTextCheckBox.setSelection(store.getDefaultBoolean(VCEPlugin.SELECT_SOURCE));
	showWindowCheckBox.setSelection(store.getDefaultBoolean(VCEPlugin.SHOW_LIVE_WINDOW));
	lookAndFeelList.setSelection(0);
	notebookRadioButton.setSelection(store.getDefaultBoolean(VCEPlugin.NOTEBOOK_PAGE));		
	showNotebookGEFPaletteButton.setSelection(store.getDefaultBoolean(VCEPlugin.NOTEBOOK_SHOW_GEF_PALETTE));
    generateExpressionComment.setSelection(store.getDefaultBoolean(VCEPlugin.GENERATE_COMMENT));    
    generateTryCatchBlock.setSelection(store.getDefaultBoolean(VCEPlugin.GENERATE_TRY_CATCH_BLOCK));
	splitRadioButton.setSelection(!store.getDefaultBoolean(VCEPlugin.NOTEBOOK_PAGE));
	showSplitPaneGEFPaletteButton.setSelection(store.getDefaultBoolean(VCEPlugin.SPLITPANE_SHOW_GEF_PALETTE));	
	sourceSyncDelayText.setText(Integer.toString(VCEPlugin.DEFAULT_SYNC_DELAY))  ;
	sourceToVisual.setText(Integer.toString(VCEPlugin.DEFAULT_L2R_FACTOR))  ;
	fStore.setValue(VCEPlugin.ALIGNMENTWINDOW_X,200);
	fStore.setValue(VCEPlugin.ALIGNMENTWINDOW_Y,25);
	
	// Having set the radio button for the split pane versus notebook to the default
	// We need to make sure that the check boxes for whether the palette should be shown or not
	// is enabled accordingly
	showSplitPaneGEFPaletteButton.setEnabled(splitRadioButton.getSelection());
	showNotebookGEFPaletteButton.setEnabled(notebookRadioButton.getSelection());
//	requireIVJforComponents.setSelection(false) ;
	
}

/**
 * @return true/false if a given option is given to the VM
 */
protected static boolean isOptionSet(String option) {
    String  fullOption = VCEPlugin.getPlugin().getDescriptor().getUniqueIdentifier()+option ;
    return   "true".equalsIgnoreCase(Platform.getDebugOption(fullOption)) ; //$NON-NLS-1$
}
/**
 * Is a VM option given to display the XML Text option
 */
protected static boolean isXMLText ()
{
   return (isOptionSet(VCEPlugin.DEBUG_XMITEXT_OPTION)) ;
}
/**
 * Is a VM option given to display the XML Text option
 */
protected static boolean isLiveWindow ()
{
   return (isOptionSet(VCEPlugin.DEBUG_LIVEWINDOW_OPTION)) ;
}

/**
 * Is XML Text setting turned on.
 */
public static boolean isXMLTextOn() {
	// Is the debug option on AND is the setting actually set.
	return isXMLText() && CDEPlugin.getPlugin().getPluginPreferences().getBoolean(CDEPlugin.SHOW_XML);
}

/**
 * Is Live Window setting turned on.
 */
public static boolean isLiveWindowOn() {
	// Is the debug option on AND is the setting actually set.
	return isLiveWindow() && VCEPlugin.getPlugin().getPluginPreferences().getBoolean(VCEPlugin.SHOW_LIVE_WINDOW);
}

public void init(IWorkbench aWorkbench){
}

public static void initDefaults(Preferences aStore){
	aStore.setDefault(VCEPlugin.NOTEBOOK_SHOW_GEF_PALETTE,true);
	aStore.setDefault(VCEPlugin.SPLITPANE_SHOW_GEF_PALETTE, true) ;
	aStore.setDefault(VCEPlugin.SOURCE_SYNC_DELAY,Integer.toString(VCEPlugin.DEFAULT_SYNC_DELAY)) ;	
	aStore.setDefault(VCEPlugin.PALETTE_SASH_LEFT_RATIO,Integer.toString(VCEPlugin.DEFAULT_PALETTE_SASH_LEFT_RATIO));
	aStore.setDefault(VCEPlugin.PALETTE_SASH_RIGHT_RATIO,Integer.toString(VCEPlugin.DEFAULT_PALETTE_SASH_RIGHT_RATIO));	
	aStore.setDefault(VCEPlugin.ALIGNMENTWINDOW_X,Integer.toString(200));
	aStore.setDefault(VCEPlugin.ALIGNMENTWINDOW_Y,Integer.toString(25));	
}
}
