package com.ibm.etools.jbcf.vce.javabeanviewer;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.*;

public class BorderLayoutManager extends AbstractLayout {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	// This holds a map of rectangles of the sizes of our figures
	protected BorderFigure northFigure;
	protected BorderFigure centerFigure;
	protected BorderFigure eastFigure;
	protected BorderFigure westFigure;
	protected BorderFigure southFigure;
		
public void layout(IFigure aFigure){
	
	// If we are incomplete we cannot layout any figures, wait until we get them all
	if(northFigure == null || centerFigure == null || eastFigure == null || westFigure == null || southFigure == null){
		return;
	}
	
	Iterator children = aFigure.getChildren().iterator();
	Point offset = aFigure.getClientArea().getLocation();
	IFigure f;
	int gap = 5;
	int x=2;
	int y=2;
	// Find the total width.  This is either the north width
	// or else east + center + west
	// or else the south width
	int width = northFigure.getPreferredSize().width;
	width = Math.max(width,
		eastFigure.getPreferredSize().width + centerFigure.getPreferredSize().width + westFigure.getPreferredSize().width + gap + gap
	);
	width = Math.max(width,southFigure.getPreferredSize().width);
	// Now we can lay out north at this width
	northFigure.setBounds(new Rectangle(x,y,width,northFigure.getPreferredSize().height));
	// After placing north put west, center and east
	// The height of these is the biggest height
	int height = westFigure.getPreferredSize().height;
	height = Math.max(height,centerFigure.getPreferredSize().height);
	height = Math.max(height,eastFigure.getPreferredSize().height);
	y = y + northFigure.getPreferredSize().height + gap;
	westFigure.setBounds(new Rectangle(new Rectangle(x,y,westFigure.getPreferredSize().width,height)));
	x = x + westFigure.getPreferredSize().width + gap;
	centerFigure.setBounds(new Rectangle(x,y,centerFigure.getPreferredSize().width,height));
	x = x + centerFigure.getPreferredSize().width + gap;
	eastFigure.setBounds(new Rectangle(x,y,eastFigure.getPreferredSize().width,height));
	// Now layout the south figure
	y = y + height + gap;
	southFigure.setBounds(new Rectangle(2,y,width,southFigure.getPreferredSize().height));
		
}	
protected Dimension calculatePreferredSize(IFigure f){
	return new Dimension(200,200);
}
public void setConstraint(IFigure aFigure, Object arg) {
	BorderFigure figure = (BorderFigure)aFigure;
	String constraint = figure.fConstraint;
	if(constraint.equals("North")){ //$NON-NLS-1$
		northFigure = figure;
	} else if(constraint.equals("East")){ //$NON-NLS-1$
		eastFigure = figure;
	} else if(constraint.equals("South")){ //$NON-NLS-1$
		southFigure = figure;
	} else if(constraint.equals("West")){ //$NON-NLS-1$
		westFigure = figure;
	} else if (constraint.equals("Center")){ //$NON-NLS-1$
		centerFigure = figure;
	} else {
		throw new IllegalArgumentException("Bad constraint value"); //$NON-NLS-1$
	}
}
}