package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.propertysheet.EToolsPropertyDescriptor;
import com.ibm.etools.cde.properties.AbstractPropertyDescriptorAdapter;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.decorators.*;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.cde.emf.ClassDecoratorFeatureAccess;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.JBCFMsgLogger;

import org.eclipse.core.runtime.*;
/**
 * This is a default base for the Constraint Property Descriptor.
 * LayoutPolicyFactories can choose to use a subclass of this,
 * or create their own if they wish.
 *
 * Subclasses will need to:
 *   1) Set nulls invalid to true if nulls are not valid as a constraint.
 */
public abstract class AbstractConstraintPropertyDescriptor extends EToolsPropertyDescriptor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * Construct with the Constraint Structural Feature from the Constraint class.
	 */
	public AbstractConstraintPropertyDescriptor(EStructuralFeature sfConstraintConstraint) {
		super(sfConstraintConstraint, VisualMessages.getString("ConstraintComponent.constraint"));  //$NON-NLS-1$
		setNullInvalid(false);	// By default, nulls are not invalid.
		setAlwaysIncompatible(true);	//By default, constraints are incompatible.
	}
	
	
	/**
	 * Default label provider that looks at the type of the setting and sees if there is a label
	 * provider for that type, and if there is, it will use that type. Otherwise, 
	 */
	public static class DefaultLabelProvider extends LabelProvider implements INeedData {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		protected EditDomain editDomain;
		
		public void setData(Object data) {
			this.editDomain = (EditDomain) data;
		}
		
		public String getText(Object element) {
			if (!(element instanceof RefObject))
				return super.getText(element);
				
			// The default is to see if label provider has been provided for the class, and if so,
			// use it. Else just have one that does a toBeanString().
			ILabelProvider provider = null;
		
			DecoratorsPackage dp = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
			BasePropertyDecorator bdec = (BasePropertyDecorator) ClassDecoratorFeatureAccess.getDecoratorWithFeature((EClassifier) ((RefObject) element).refMetaObject(), BasePropertyDecorator.class, dp.getBasePropertyDecorator_LabelProviderClassname());
			if (bdec != null) {
				try {
					String classNameAndData = bdec.getLabelProviderClassname();
					Class labelProviderClass = CDEPlugin.getPlugin().getClassFromString(classNameAndData);
					provider = AbstractPropertyDescriptorAdapter.createLabelProviderInstance(labelProviderClass, classNameAndData, null, null);
				} catch (ClassNotFoundException e) {
					// One specified, but incorrect, log it, but continue and see if we can get another way.
					JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, JBCFVisualPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, "", e), JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
				}
			}
			
			if (provider != null) {
				if (provider instanceof INeedData)
					((INeedData) provider).setData(editDomain);
					
				return provider.getText(element);
			}
			
			if (element instanceof IJavaInstance) {
				// Do a toBeanString
				IBeanProxy elementProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance)element, JBCFEditDomainHelper.getResourceSet(editDomain));
				return elementProxy.toBeanString();
			}
			
			return super.getText(element);
		}
	}
	
	/**
	 * Return the default label provider. If the
	 * default is not sufficient, overrides should provide the appropriate one.
	 */
	public ILabelProvider getLabelProvider() {
		return new DefaultLabelProvider();
	}
}