package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.*;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.internal.CoolBarLayout;
import org.eclipse.ui.internal.CoolBarManager;
import org.eclipse.swt.SWT;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.AlignmentAction;
import com.ibm.etools.cde.AlignmentCommandRequest;
import com.ibm.etools.gef.ui.actions.ActionRegistry;

/**
 * This is the dialog that appears when the user selects Alignment
 */
public class AlignmentDialog extends Window {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IEditorPart fEditorPart;

	private AlignmentAction fAlignLeftAction;
	private AlignmentAction fAlignRightAction;
	private AlignmentAction fAlignCenterAction;	
	private AlignmentAction fAlignTopAction;
	private AlignmentAction fAlignMiddleAction;
	private AlignmentAction fAlignBottomAction;
	private AlignmentAction fMatchWidthAction;
	private AlignmentAction fMatchHeightAction;					
	private ShowDistributeBoxAction fShowDistributeBoxAction;
	private DistributeAction fDistributeHorizontalAction;
	private DistributeAction fDistributeVerticalAction;
	private ToolBarManager mgr1,mgr2,mgr3;
	private Shell fShell;
	private String fTitle;
	
public AlignmentDialog(IEditorPart anEditorPart,String aTitle){
	// Create us with a new shell
	super(anEditorPart.getSite().getShell());
	fEditorPart = anEditorPart;
	getAlignmentActions();
	setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
	setBlockOnOpen(false);
	fTitle = aTitle;
}
protected Control createContents(Composite parent) {
	
	Composite group = new Composite(parent,SWT.NONE);
	group.setLayout(new GridLayout(1,true));
	
	ToolBar toolBar1 = new ToolBar(group,SWT.FLAT);
	
	mgr1 = new ToolBarManager(toolBar1);
	mgr1.add(fAlignLeftAction);
	mgr1.add(fAlignTopAction);
	mgr1.add(fMatchWidthAction);
	mgr1.add(fShowDistributeBoxAction);
	mgr1.update(true);
	
	ToolBar toolBar2 = new ToolBar(group,SWT.FLAT);
	mgr2 = new ToolBarManager(toolBar2);
	mgr2.add(fAlignCenterAction);
	mgr2.add(fAlignMiddleAction);
	mgr2.add(fMatchHeightAction);
	mgr2.add(fDistributeHorizontalAction);
	mgr2.update(true);
		
	ToolBar toolBar3 = new ToolBar(group,SWT.FLAT);
	mgr3 = new ToolBarManager(toolBar3);
	mgr3.add(fAlignRightAction);
	mgr3.add(fAlignBottomAction);
	mgr3.add(new NullAction());
	mgr3.add(fDistributeVerticalAction);
	mgr3.update(true);
	
	Point shellSize = getShell().computeSize(100,90);
	getShell().setSize(shellSize);
	
	getShell().setText(fTitle);
	
	return group;
	
}
class NullAction extends Action{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	NullAction(){
		try {
			URL nullURL = new URL("platform:/plugin/com.ibm.etools.jbcf.visual/icons/transp16.gif"); //$NON-NLS-1$
			ImageDescriptor nullDescriptor = ImageDescriptor.createFromURL(nullURL);
			setImageDescriptor(nullDescriptor);
			setHoverImageDescriptor(nullDescriptor);
			setDisabledImageDescriptor(nullDescriptor);
		} catch ( MalformedURLException exc ) {
		}
		NullAction.this.setEnabled(false);
	}
}
protected void addGridButton(Composite group, final IAction action){
	Button button = new Button(group,SWT.NONE);
	button.setImage(new Image(null,action.getImageDescriptor().getImageData()));
	button.addSelectionListener(new SelectionAdapter(){
		public void widgetSelected(SelectionEvent event){
			action.run();
		}
	});
	GridData data = new GridData();
	button.setLayoutData(data);
}
protected void addSpacer(Composite group){
	Label label = new Label(group,SWT.NONE);
	label.setLayoutData(new GridData());
	
}
public void setSelection(IStructuredSelection aSelection){
	fAlignLeftAction.setSelection(aSelection);
	fAlignCenterAction.setSelection(aSelection);
	fAlignRightAction.setSelection(aSelection);
	fAlignTopAction.setSelection(aSelection);
	fAlignMiddleAction.setSelection(aSelection);
	fAlignBottomAction.setSelection(aSelection);
	fMatchWidthAction.setSelection(aSelection);
	fMatchHeightAction.setSelection(aSelection);
	fDistributeHorizontalAction.setSelection(aSelection);
	fDistributeVerticalAction.setSelection(aSelection);	
}
/**
 * Create no buttons for the dialog
 */
protected void createButtonsForButtonBar(Composite parent) {
//	createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CLOSE_LABEL, false);
}
public void setEnabled(boolean aBool){
	fAlignLeftAction.setEnabled(aBool);
	fAlignCenterAction.setEnabled(aBool);
	fAlignRightAction.setEnabled(aBool);
	fAlignTopAction.setEnabled(aBool);
	fAlignMiddleAction.setEnabled(aBool);
	fAlignBottomAction.setEnabled(aBool);
	fMatchWidthAction.setEnabled(aBool);
	fMatchHeightAction.setEnabled(aBool);
	fDistributeHorizontalAction.setEnabled(aBool);
	fDistributeVerticalAction.setEnabled(aBool);
	fShowDistributeBoxAction.setEnabled(aBool);
}
// Forward the editor part and the current selection to each of the actions
public void setEditorPart(IEditorPart anEditorPart,IStructuredSelection aSelection){
		
	fAlignLeftAction.setEditorPart(anEditorPart,aSelection);
	fAlignCenterAction.setEditorPart(anEditorPart,aSelection);	
	fAlignRightAction.setEditorPart(anEditorPart,aSelection);
	fAlignTopAction.setEditorPart(anEditorPart,aSelection);
	fAlignMiddleAction.setEditorPart(anEditorPart,aSelection);
	fAlignBottomAction.setEditorPart(anEditorPart,aSelection);
	fMatchWidthAction.setEditorPart(anEditorPart,aSelection);
	fMatchHeightAction.setEditorPart(anEditorPart,aSelection);
	fDistributeHorizontalAction.setEditorPart(anEditorPart,aSelection);
	fDistributeVerticalAction.setEditorPart(anEditorPart,aSelection);
	fShowDistributeBoxAction.setEditorPart(anEditorPart);

}
protected void getAlignmentActions(){
	ActionRegistry actionRegistry = (ActionRegistry)fEditorPart.getAdapter(ActionRegistry.class);	
	fAlignLeftAction = (AlignmentAction)actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.LEFT_ALIGN));
	fAlignCenterAction = (AlignmentAction)actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.CENTER_ALIGN));
	fAlignRightAction = (AlignmentAction)actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.RIGHT_ALIGN));
	fAlignTopAction = (AlignmentAction) actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.TOP_ALIGN));
	fAlignMiddleAction = (AlignmentAction) actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.MIDDLE_ALIGN));
	fAlignBottomAction = (AlignmentAction) actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.BOTTOM_ALIGN));
	fMatchWidthAction = (AlignmentAction)actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.MATCH_WIDTH));
	fMatchHeightAction = (AlignmentAction)actionRegistry.getAction(AlignmentAction.getActionId(AlignmentCommandRequest.MATCH_HEIGHT));
	fShowDistributeBoxAction = (ShowDistributeBoxAction) actionRegistry.getAction(ShowDistributeBoxAction.ACTION_ID);
	fDistributeHorizontalAction = (DistributeAction) actionRegistry.getAction(DistributeAction.getActionId(DistributeCommandRequest.HORIZONTAL));
	fDistributeVerticalAction = (DistributeAction) actionRegistry.getAction(DistributeAction.getActionId(DistributeCommandRequest.VERTICAL));
}
}
