package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;

import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.cde.CDEMessages;
import com.ibm.etools.cde.emf.*;
import com.ibm.etools.jbcf.vce.VCEPlugin;

/**
 * @author JoeWin
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AlignmentWindowAction extends Action {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static String ACTION_ID = "AlignmentWindowAction"; //$NON-NLS-1$
	protected static Rectangle fDialogBounds;
	protected static AlignmentDialog fDialog;	// Hold a singleton instance of the dialog
	protected IEditorPart fEditorPart;
	protected static ResourceBundle bundle;
	protected IStructuredSelection fSelection;
	protected String fTitle;
	
public AlignmentWindowAction(IEditorPart anEditorPart){
	setId(ACTION_ID);
	fTitle = CDEMessages.getPluginString("AlignmentAction.submenu.label"); //$NON-NLS-1$
	setToolTipText(fTitle);	// Use for hover help in the toolbar
	setText(fTitle);		// Used for the menu when the toolbar is off the RHS
	fEditorPart = anEditorPart;
	try {
		URL enableImageURL = new URL("platform:/plugin/com.ibm.etools.cde/icons/full/elcl16/aligndialog_obj.gif"); //$NON-NLS-1$
		setImageDescriptor(ImageDescriptor.createFromURL(enableImageURL));

		URL disableImageURL = new URL("platform:/plugin/com.ibm.etools.cde/icons/full/dlcl16/aligndialog_obj.gif"); //$NON-NLS-1$
		setDisabledImageDescriptor(ImageDescriptor.createFromURL(disableImageURL));		
		
		URL colorImageURL = new URL("platform:/plugin/com.ibm.etools.cde/icons/full/clcl16/aligndialog_obj.gif"); //$NON-NLS-1$
		setHoverImageDescriptor(ImageDescriptor.createFromURL(colorImageURL));
		
	} catch ( MalformedURLException exc ){
	}		
	// We are checked or unchecked based on whether there is a dialog showing
	setChecked(fDialog != null);
	
}
public void setEditorPart(IEditorPart anEditorPart,IStructuredSelection aSelection){
	
	fEditorPart = anEditorPart;
	fSelection = aSelection;
	if( fDialog != null){
		fDialog.setEditorPart(anEditorPart,aSelection);
	}
}
public void setSelection(IStructuredSelection aSelection){
	fSelection = aSelection;
}

public void dispose() {
	if (fDialog != null) {
		fDialog.close();
		fDialog = null;
	}
}
public void run(){
	// If we are checked then close the dialog
	if ( fDialog != null ) {
		fDialog.close();
		fDialog = null;		
		setChecked(false);
	} else {
	// If we are not checked then the dialog is closed so we should open it
		fDialog = new AlignmentDialog(fEditorPart,fTitle);
		// Refresh the selection from the editor part
		setSelection((IStructuredSelection) ((EMFGraphicalEditorPart)fEditorPart).getPrimaryViewer().getSelection());
		fDialog.setSelection(fSelection);
		setChecked(true);
		fDialog.open();
		if(fDialogBounds == null){
			// Get the persisted position of the dialog
			Preferences preferences = VCEPlugin.getPlugin().getPluginPreferences();
			int x = preferences.getInt(VCEPlugin.ALIGNMENTWINDOW_X);
			int y = preferences.getInt(VCEPlugin.ALIGNMENTWINDOW_Y);
			fDialog.getShell().setLocation(x,y);
			fDialogBounds = fDialog.getShell().getBounds();
		} else {
			fDialog.getShell().setBounds(fDialogBounds);
		}
		fDialog.getShell().addControlListener(new ControlListener(){
			public void controlMoved(ControlEvent event){
				fDialogBounds = fDialog.getShell().getBounds();			
			}
			public void controlResized(ControlEvent event){
				fDialogBounds = fDialog.getShell().getBounds();			
			}
		});
		// When the dialog is closed null the pointer to it
		fDialog.getShell().addShellListener(new ShellAdapter(){
			public void shellClosed(ShellEvent event){
				persistPreferences();
				fDialog = null;
				setChecked(false);				
			}
		});
	}
}
	public void editorPartDeactivated() {
		if (fDialog != null)
		   fDialog.setEnabled(false);
	}
	public void editorPartClosed() {
		if (fDialog != null)
		   fDialog.setEnabled(false);
	}
	protected void persistPreferences(){
		if ( fDialogBounds != null){
			Preferences preferences = VCEPlugin.getPlugin().getPluginPreferences();
			preferences.setValue(VCEPlugin.ALIGNMENTWINDOW_X,fDialogBounds.x);
			preferences.setValue(VCEPlugin.ALIGNMENTWINDOW_Y,fDialogBounds.y);
			VCEPlugin.getPlugin().savePluginPreferences();
		}
	}
}
