package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.AnnotationLinkagePolicy;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cde.properties.NameInCompositionPropertyDescriptor;
import com.ibm.etools.cdm.Annotation;
import com.ibm.etools.cdm.KeyedString;
import com.ibm.etools.common.command.CommandWrapper;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;

/**
 * @author pwalker
 *
 * Special command for applying a constraint to a component dropped onto
 * a container with a java.awt.CardLayout.
 * 
 * Since a String constraint is required for a CardLayout, we use 
 * the 'name' property, if it is set, in the proxy adapter. If not, we create
 * this command so we can set the name with a unique name which is not available
 * when we first construct it... only at execution time.
 */
public class ApplyCardLayoutConstraintCommand extends CommandWrapper {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected RefObject fConstraintComponent, fComponent;
	protected ResourceSet fResourceSet;
	protected EditDomain fDomain;

	/**
	 * Constructor for ApplyCardLayoutConstraintCommand.
	 */
	public ApplyCardLayoutConstraintCommand(RefObject constraintComponent, RefObject component, EditDomain domain) {
		super();
		fConstraintComponent = constraintComponent;
		fComponent = component;
		fDomain = domain;
		fResourceSet = JBCFEditDomainHelper.getResourceSet(domain);
	}

	protected boolean prepare() {
		// Need to override prepare because prepare expects to have a command
		// create, and at the time of prepare being called, we don't have a command yet.
		return fConstraintComponent != null && fComponent != null && fDomain != null && fResourceSet != null;
	}
	public void execute() {
		CommandBuilder cb = new CommandBuilder(null);
		EStructuralFeature sfName =
			JavaInstantiation.getSFeature((IJavaObjectInstance) fComponent, JBCFConstants.SF_COMPONENT_NAME),
			sfConstraintConstraint =
				JavaInstantiation.getSFeature(fResourceSet, JBCFConstants.SF_CONSTRAINT_CONSTRAINT);
		String cardname = null;
		if (!fComponent.refIsSet(sfName)) {
			// Since 'name' is not set, we need to set it so the cardlayout will use it
			// by default for it's constraint. We'll get this name from the annotation.
			IJavaObjectInstance stringObject = null;
			AnnotationLinkagePolicy policy = fDomain.getAnnotationLinkagePolicy();
			Annotation ann = policy.getAnnotation(fComponent);
			if (ann != null) {
				KeyedString ks =
					(KeyedString) ann.getKeyedValue(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY);
				if (ks != null) {
					cardname = ks.getValue();
				}
			}
			// Found a name
			if (cardname != null)
				stringObject = BeanUtilities.createString(fResourceSet, cardname);
			else {
				// Either didn't find the annotation or didn't find annotation name... so create one
				String classname = ((IJavaObjectInstance) fComponent).getJavaType().getJavaName();
				int index = classname.lastIndexOf("."); //$NON-NLS-1$
				if (index != -1)
					classname = classname.substring(index + 1);
				cardname = NameInCompositionPropertyDescriptor.getUniqueNameInComposition(fDomain, "ivj" + classname); //$NON-NLS-1$
			}
			// Need to apply the name property and than unset the dummy constraint that was set earlier.
			cb.applyAttributeSetting(fComponent, sfName, stringObject);
		}
		cb.cancelAttributeSetting(fConstraintComponent, sfConstraintConstraint);
		command = cb.getCommand();
		command.execute();
	}
}
