package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import com.ibm.etools.common.command.Command;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.gef.commands.CompoundCommand;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.awt.IPointBeanProxy;
/**
 * This helper class is used when a container's layout is changed
 * to BorderLayout. Each of the constraints for the container's children 
 * needs to be set based on each child's order and position.
 */
public class BorderLayoutSwitcher extends LayoutSwitcher {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	BorderLayoutPolicyHelper helper;

public BorderLayoutSwitcher(ContainerPolicy aPolicy) {
	super(aPolicy);
	helper = new BorderLayoutPolicyHelper(aPolicy);
}
/**
 * Create the change constraint commands for the bean's children
 * We need to create BorderComponent objects for the new constraints
 */
protected Command getChangeConstraintsCommand(List children) {
	// use the helper to help with constraint conversion
	List regions = new ArrayList(5);
	List internalTags = helper.REAL_INTERNAL_TAGS;
	for (int i=0; i<5; i++) {
		regions.add(internalTags.get(i));
	}
	ArrayList remComps = new ArrayList();
	CompoundCommand cc = new CompoundCommand("Create constraints"); //$NON-NLS-1$
	// get the components
	Iterator comps = children.iterator();
	// In the first pass, loop thru the components and assign a region to the location the object is in
	List constraints = new ArrayList(5);
	List compsWithConstraints = new ArrayList(5);
	while (comps.hasNext()) {
		IJavaObjectInstance comp = (IJavaObjectInstance)comps.next();
		// get the location from the bean since flow does not have constraints
		IBeanProxy proxy = BeanProxyUtilities.getBeanProxy(comp);
		IPointBeanProxy pointProxy = BeanAwtUtilities.invoke_getLocation(proxy);
		Point loc = new Point(pointProxy.getX(),pointProxy.getY());
		// get the location and decide which region it should be placed in
		if (loc != null) {
			String str = helper.getCurrentConstraint(helper.convertLocation(loc));
			if (str != null && regions.contains(str)) {
				constraints.add(str);
				compsWithConstraints.add(comp);
				// Remote the region just used up
				regions.remove(str);
			} else
				// remComps is the people we can't allocate a constraint to yet
				remComps.add(comp);
		}
	}
	// If there are any regions still unoccupied and components targeted for removal,
	// assign the remaining regions until all regions are occupied or no more components
	// are left to be assigned.
	while (!remComps.isEmpty() && !regions.isEmpty()) {
		constraints.add(regions.get(0));
		compsWithConstraints.add((IJavaObjectInstance)remComps.get(0));
		regions.remove(regions.get(0));
		remComps.remove(remComps.get(0));
	}
	// For all the components that have regions let the helper add them
	cc.add(helper.getAddChildrenCommand(compsWithConstraints, constraints, null));
	
	// If there are components still remaining, remove them from the container - Border only
	// has five regions	
	if (!remComps.isEmpty()) {
		cc.add(removeComponents(remComps));
	}
	if (cc.size() > 0)
		return cc.unwrap();
	else
		return null;
}
/**
 * Create the cancel command to remove the remaining components from the container.
 */
public Command removeComponents(List comps) {
	Iterator iterComps = comps.iterator();
	CompoundCommand cc = new CompoundCommand("Remove components"); //$NON-NLS-1$
	while (iterComps.hasNext()) {
		cc.add(policy.getDeleteDependentCommand(iterComps.next()));
	}
	return cc;
}

}
