package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.BeanPropertyDescriptorAdapter;
import com.ibm.etools.propertysheet.ICommandPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.propertysheet.RestoreDefaultPropertyValueCommand;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefStructuralFeature;
/**
 * Provide some specific overrides for Bounds property.
 */
public class BoundsPropertyDescriptor extends BeanPropertyDescriptorAdapter implements ICommandPropertyDescriptor {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public boolean areNullsInvalid() {
		return true;
	}
	
	public Command setValue(IPropertySource source, Object setValue) {
		// Set the bounds and unset the location and size, if set.
		// Unset the location and size first before applying bounds because
		// they interfere with each other.
		CommandBuilder cb = new CommandBuilder(null);
		IJavaObjectInstance comp = (IJavaObjectInstance) source.getEditableValue();
		EStructuralFeature 
			sfComponentSize = JavaInstantiation.getSFeature(comp, JBCFConstants.SF_COMPONENT_SIZE),
			sfComponentLocation = JavaInstantiation.getSFeature(comp, JBCFConstants.SF_COMPONENT_LOCATION);			
		if (comp.refIsSet(sfComponentSize))
			cb.cancelAttributeSetting(comp, sfComponentSize);
		if (comp.refIsSet(sfComponentLocation))
			cb.cancelAttributeSetting(comp, sfComponentLocation);
			
		cb.applyAttributeSetting(comp, (RefStructuralFeature) getTarget(), setValue);
		return cb.getCommand();
	}
	
	public Command resetValue(IPropertySource source) {
		RestoreDefaultPropertyValueCommand cmd  = new RestoreDefaultPropertyValueCommand();
		cmd.setTarget(source);
		cmd.setPropertyId(getTarget());
		return cmd;
	}

}