package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.ui.views.properties.IPropertyDescriptor;

import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.BeanUtilities;

public class BoxLayoutPolicyFactory implements ILayoutPolicyFactory {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public BoxLayoutPolicyFactory() {
		super();
	}
	public Class getLayoutInputPolicyClass() {
		return FlowLayoutEditPolicy.class;
	}
	public ILayoutPolicyHelper getLayoutPolicyHelper(ContainerPolicy ep) {
		return new FlowLayoutPolicyHelper(ep);
	}
	public ILayoutSwitcher getLayoutSwitcher(ContainerPolicy cp) {
		return new FlowLayoutSwitcher(cp);
	}

	public IPropertyDescriptor getConstraintPropertyDescriptor(EStructuralFeature sfConstraint) {
		return null; // No constraint, so no property descriptor.
	}

	/**
	 * Hack for now since we can't set a property for Axis on a boxlayout. 
	 * There are two possible BoxLayout's in the layout dropdown list. One for
	 * each possible axis orientation. A special object is used for each. 
	 * Use the type of object to determine the initialization string for the
	 * ctor of the BoxLayout.
	 */
	public IJavaInstance getLayoutManagerInstance(JavaHelpers javaClass, ResourceSet rset) {
		JavaHelpers boxLayoutJavaClass = JavaClassImpl.reflect("javax.swing.BoxLayout", rset); //$NON-NLS-1$
		String initString = "new javax.swing.BoxLayout(, javax.swing.BoxLayout."; //$NON-NLS-1$
		String javaClassName = ((JavaClass) javaClass).getName();
		if (javaClassName.equals("BoxLayoutX_Axis")) //$NON-NLS-1$
			initString += "X_AXIS)"; //$NON-NLS-1$
		else if (javaClassName.equals("BoxLayoutY_Axis")) //$NON-NLS-1$
			initString += "Y_AXIS)"; //$NON-NLS-1$
		else
			initString += "X_AXIS)";	// default to X_AXIS //$NON-NLS-1$
		return BeanUtilities.createJavaObject(boxLayoutJavaClass, rset, initString);
	}
}