package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefBaseObject;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;

/**
 * @version 	1.0
 * @author
 */
public class CardLayoutPolicyHelper extends LayoutPolicyHelper {
	protected EStructuralFeature sfName;	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$

	public CardLayoutPolicyHelper(ContainerPolicy ep) {
		super(ep);
	}
	
	public CardLayoutPolicyHelper() {
	}

	/*
	 * @see LayoutPolicyHelper#convertConstraint(Object)
	 */
	protected IJavaObjectInstance convertConstraint(Object constraint) {
		if (constraint instanceof String)
			return BeanUtilities.createString(getContainer().getResourceSet(), (String) constraint);
		else
			return null;
	}
	/*
	 * @see ILayoutPolicyHelper#getDefaultConstraint(List)
	 */
	public List getDefaultConstraint(List children) {
		return Collections.nCopies(children.size(), NO_CONSTRAINT_VALUE);
	}

	/**
	 * @see com.ibm.etools.jbcf.visual.ILayoutPolicyHelper#getCreateChildCommand(Object, Object, Object)
	 */
	public Command getCreateChildCommand(Object childComponent, Object constraint, Object position) {		
		CommandBuilder cb = new CommandBuilder(""); //$NON-NLS-1$
		ApplyCardLayoutConstraintCommand applyCmd = null;
		RefObject constraintComponent = (RefObject) visualFact.create(classConstraintComponent);
		if (constraint != NO_CONSTRAINT_VALUE)
			constraintComponent.refSetValue(JavaInstantiation.getSFeature(getContainer(), JBCFConstants.SF_CONSTRAINT_CONSTRAINT), constraint != null ? convertConstraint(constraint) : null);	// Put the constraint into the constraint component.
		else 
			applyCmd = getConstraintCommand(childComponent, constraintComponent);
		cb.append(policy.getCreateCommand(constraintComponent, childComponent, position));
		if (applyCmd != null)
			cb.append(applyCmd);
		return cb.getCommand();
	}

	public Command getAddChildrenCommand(List childrenComponents, List constraints, Object position) {
		CompoundCommand constraintCommands = new CompoundCommand();
		ArrayList componentConstraints = new ArrayList(childrenComponents.size());
		ApplyCardLayoutConstraintCommand applyCmd = null;
		int i = 0;
		Iterator itr = constraints.iterator();
		while (itr.hasNext()) {
			RefObject constraintComponent = (RefObject) visualFact.create(classConstraintComponent);
			componentConstraints.add(constraintComponent);
			Object constraint = itr.next();
			if (constraint != NO_CONSTRAINT_VALUE)
				constraintComponent.refSetValue(sfConstraintConstraint, constraint != null ? convertConstraint(constraint) : null);	// Put the constraint into the constraint component.
			else {
				applyCmd = getConstraintCommand(childrenComponents.get(i), constraintComponent);
				if (applyCmd != null)
					constraintCommands.append(applyCmd);
			}
			i++;
		}
		CommandBuilder cb = new CommandBuilder(""); //$NON-NLS-1$
		cb.append(policy.getAddCommand(componentConstraints, childrenComponents, position));
		if (!constraintCommands.isEmpty())
			cb.append(constraintCommands);
		return cb.getCommand();
	}
	/**
	 * Check to see if the 'name' attribute is part-of, and set for a given RefObject
	 */
	protected boolean isNameSet(RefObject obj) {
		// First check to see that 'name' is a valid attribute for this object		
		RefMetaObject container = (RefMetaObject)sfName.refContainer();
		if (!container.isInstance(obj)) return false ;
		
		return obj.refIsSet(sfName) ;
	}

	protected ApplyCardLayoutConstraintCommand getConstraintCommand(Object childComponent, RefObject constraintComponent) {
		// Check if 'name' is set... which is used by default as the constraint.
		// If not,we need to set a place holder constraint and a special command
		// so that later on when we execute the command, we can derive the constraint
		// name from the annotation composition name and set the name property.

		if (!isNameSet((RefObject)childComponent)) {
			constraintComponent.refSetValue(JavaInstantiation.getSFeature(getContainer(), JBCFConstants.SF_CONSTRAINT_CONSTRAINT), getDefaultStringConstraintObject());
			return new ApplyCardLayoutConstraintCommand(constraintComponent, (RefObject)childComponent, policy.getEditDomain());
		}
		return null;
	}

	/*
	 * Provide a dummy string value for now so the card layout won't throw an exception
	 */	
	protected IJavaObjectInstance getDefaultStringConstraintObject() {
		return BeanUtilities.createString(getContainer().getResourceSet(), "NO_CONSTRAINT_VALUE"); //$NON-NLS-1$
	}

	/**
	 * @see com.ibm.etools.jbcf.visual.ILayoutPolicyHelper#setContainerPolicy(ContainerPolicy)
	 */
	public void setContainerPolicy(ContainerPolicy policy) {
		super.setContainerPolicy(policy);
		if (policy != null) {
			// Eventually we will be set with a policy. At that time we can compute these.
			ResourceSet rset = JBCFEditDomainHelper.getResourceSet(policy.getEditDomain());
			sfName = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_COMPONENT_NAME);
		}
	}

}
