package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.lang.reflect.*;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.visual.tab.IOrderManager;
import com.ibm.etools.jbcf.visual.tab.JTabOrderManager;
import com.ibm.etools.jbcf.visual.tab.ShowTabOrderAction;

import java.util.*;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.DefaultContainerEditPolicy;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.cde.VisualComponentsLayoutPolicy;
import com.ibm.etools.draw2d.IFigure;
import com.ibm.etools.draw2d.XYLayout;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.proxy.IBeanProxy;

/**
 * ViewObject for the awt Container.
 * Creation date: (2/16/00 3:45:46 PM)
 * @author: Joe Winchester
 */
public class ContainerGraphicalEditPart extends ComponentGraphicalEditPart implements IEMFListener {

public ContainerGraphicalEditPart(Object model) {
	super(model);
}

protected ContainerPolicy getContainerPolicy() {
	return new ContainerPolicy(EditDomain.getEditDomain(this));	// AWT standard Contained Edit Policy
}

protected IFigure createFigure() {
	IFigure fig = super.createFigure();
	fig.setLayoutManager(new XYLayout());
	return fig;
}

protected void createEditPolicies() {
	super.createEditPolicies();
	installEditPolicy(EditPolicy.CONTAINER_ROLE, new DefaultContainerEditPolicy());
	installEditPolicy(VisualComponentsLayoutPolicy.LAYOUT_POLICY, new VisualComponentsLayoutPolicy());	// This is a special policy that just handles the size/position of visual components wrt/the figures. It does not handle changing size/position.
	createLayoutEditPolicy();
}

/**
 * Because java.awt.Container can vary its layout manager we need to use 
 * the correct layout input policy for the layout manager that is calculated by
 * a factory
 */
protected void createLayoutEditPolicy() {
	// Need to create the correct layout input policy depending upon the layout manager setting.
	EditPolicy layoutPolicy = null;
	// Get the layout input policy class from the layout policy factory 
	IBeanProxy containerProxy = getComponentProxy().getBeanProxy();
	if (containerProxy != null) {
		// a container was created. 
		ILayoutPolicyFactory lpFactory = BeanAwtUtilities.getLayoutPolicyFactory(containerProxy, EditDomain.getEditDomain(this));
		if (lpFactory.getLayoutInputPolicyClass() != null) {
			try {
				Class policyClass = lpFactory.getLayoutInputPolicyClass();
				// See if there is a ctor that takes an IContainerInputPolicyHelper, and if there is,
				// use that ctor with the helper assigned to this view object.
				Constructor ctor = null;
				try {
					ctor = policyClass.getConstructor(new Class[] {ContainerPolicy.class});
					layoutPolicy = (EditPolicy) ctor.newInstance(new Object[] {getContainerPolicy()});
				} catch (NoSuchMethodException e) {
					layoutPolicy = (EditPolicy) policyClass.newInstance();
				}
			} catch (Throwable e) {
				JBCFPlugin.log("Unable to create the layout policy", JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
				JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
			}
		};
	}
	// If the LayoutPolicyFactory didn't specifiy a LayoutInputPolicy, use UnknownLayoutInputPolicy
	if (layoutPolicy == null) {
		layoutPolicy = new UnknownLayoutInputPolicy(getContainerPolicy());
	}
	
	removeEditPolicy(EditPolicy.LAYOUT_ROLE); // Get rid of old one, if any
	// Layout policies put figure decorations for things like grids so we should remove this
	installEditPolicy(EditPolicy.LAYOUT_ROLE, layoutPolicy);
}

protected List getModelChildren() {
	// Model children is the components feature.
	// However, this returns the constraint components, but we want to return instead
	// the components themselves. They are the "model" that gets sent to the createChild and
	// component edit part.
	List constraintChildren = (List) ((RefObject) getModel()).refValue(sf_containerComponents);
	ArrayList children = new ArrayList(constraintChildren.size());
	Iterator itr = constraintChildren.iterator();
	while (itr.hasNext()) {
		RefObject con = (RefObject) itr.next();		
		IJavaInstance component = (IJavaInstance)con.refValue(sf_constraintComponent);	// Get the component out of the constraint
		if (component != null) {
			// It was added. It would be null if it couldn't be added for some reason
			children.add(component);
		}
	}
	return children;
}


	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected EMFListenerAdapter containerAdapter;

public void activate() {
	super.activate();	
	((RefObject) getModel()).addAdapter(getModelAdapter());
}

public void deactivate() {
	super.deactivate();
	((RefObject) getModel()).removeAdapter(getModelAdapter());
}

protected Adapter getModelAdapter() {
	if (containerAdapter == null) {
		containerAdapter = new EMFListenerAdapter(this, false);
	}
	return containerAdapter;
}

private EStructuralFeature 
	sf_containerLayout,
	sf_constraintComponent,
	sf_containerComponents;
	
public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
	if (sf == sf_containerComponents)
		refreshChildren();
	else if (sf == sf_containerLayout)
		createLayoutEditPolicy();
}

protected EditPart createChild(Object model) {
	EditDomain dom = EditDomain.getEditDomain(this);
	EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
	((ComponentGraphicalEditPart) ep).setPropertySource((IPropertySource) ((RefObject) model).refContainer().getAdapter(IPropertySource.class));	// This is the property source of the actual model which is part of the constraintComponent.
	((ComponentGraphicalEditPart) ep).setTransparent(true);	// So that it doesn't create an image, we subsume it here.
	return ep;
}

	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		
		ResourceSet rset = ((IJavaObjectInstance) model).getResourceSet();
		sf_containerLayout = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_LAYOUT);
		sf_constraintComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_COMPONENT);	
		sf_containerComponents = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_COMPONENTS);		
	}

}
