package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import java.util.List;

import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.ThrowableProxy;

public class ContainerProxyAdapter extends ComponentProxyAdapter {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfConstraintConstraint, sfContainerComponents, sfConstraintComponent, sfName;

	public ContainerProxyAdapter(IBeanProxyDomain domain) {
		super(domain);

		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain.getEditDomain());
		sfConstraintConstraint = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_CONSTRAINT);
		sfContainerComponents = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_COMPONENTS);
		sfConstraintComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_COMPONENT);
		sfName = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_COMPONENT_NAME);
	}

	protected void applied(RefStructuralFeature as, Object newValue, int position) {

		if (as == sfContainerComponents) {
			addComponent((RefObject) newValue, position);
		} else {
			super.applied(as, newValue, position);
		}

	}
	protected void canceled(RefStructuralFeature sf, Object oldValue, int position) {

		if (sf == sfContainerComponents) {
			removeComponent((RefObject) oldValue);
		} else {
			super.canceled(sf, oldValue, position);
		}

	}

	public void releaseBeanProxy() {
		// Need to release all of the components. This isn't being done in BeanProxyAdapter because
		// the constraint components aren't bean proxies directly.

		// NOTE: Using refBasicValue so that we don't go creating a list and a settings adapter
		// if none were set.
		List containerComponents = (List) ((Internals) target).refBasicValue(sfContainerComponents);
		if (containerComponents != null) {
			Iterator itr = containerComponents.iterator();
			while (itr.hasNext()) {
				InternalProxy ip = (InternalProxy) itr.next();
				// Get all of the basic contains and release those that are beans. 
				// For constraintComponents that would normally be the constraint and the component.
				Iterator settings = ip.refBasicContains().iterator(); // Get only the attrs and composite refs.
				while (settings.hasNext()) {
					Object v = settings.next();
					// Other settings like initializationString exist, only interested in java instances.
					if (v instanceof IJavaInstance) {
						// Also, if it is a proxy (i.e. never resolved) don't release it since we never did anything with it.
						if (!(v instanceof InternalProxy) || !((InternalProxy) v).refIsProxy()) {
							IBeanProxyHost value = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) v);
							if (value != null)
								value.releaseBeanProxy();
						}
					}
				}
			}
		}

		super.releaseBeanProxy();
	}
	public void reinstantiateChild(IBeanProxyHost aChildBeanProxy) {

		// The child component is asking to be created.  We must create him at the correct index on us
		// To do this we must first get the target object of BeanProxy - the IJavaObjectInstance
		IJavaObjectInstance component = (IJavaObjectInstance) aChildBeanProxy.getTarget();
		// Now get the constraintComponent object
		RefObject constraintComponent = (RefObject) component.refContainer();
		if (((EClass) sfContainerComponents.refType()).isInstance(constraintComponent)) {		
			// See what index this constraint component is present in our "components" relationship
			List components = ((List) ((RefObject) getTarget()).refValue(sfContainerComponents));
			for (int i = 0; i < components.size(); i++) {
				if (components.get(i) == constraintComponent) {
					removeComponent(constraintComponent);
					addComponent(constraintComponent, i);
					break;
				}
			}
		} else
			super.reinstantiateChild(aChildBeanProxy);

	}
	protected void removingAdapter() {
		// The adapter is being removed, so remove any Constraint Adapters that are pointing to us.
		// NOTE: Using refBasicValue so that we don't go creating a list and a settings adapter
		// if none were set. We are going away, we don't want to add a new adapter.
		List containerComponents = (List) ((Internals) target).refBasicValue(sfContainerComponents);
		if (containerComponents != null) {
			Iterator itr = containerComponents.iterator();
			while (itr.hasNext()) {
				Notifier n = (Notifier) itr.next();
				removeAdapters(n);
			}
		}
	}

	protected void removeAdapters(Notifier n) {
		Adapter a = n.getExistingAdapter(ComponentConstraintAdapter.class);
		if (a != null)
			n.removeAdapter(a);
		n = (Notifier)((RefObject)n).refValue(sfConstraintComponent);
		if (n != null) {
			a = n.getExistingAdapter(ComponentNameAdapter.class);
			if (a != null)
				n.removeAdapter(a);
		}
	}

	protected void removeComponent(RefObject aConstraintComponent) {
		// Remove the old constraint adapter, if one.
		removeAdapters(aConstraintComponent);

		// The component to actually remove is within the ConstraintComponent too.
		clearError(sfContainerComponents, aConstraintComponent);		
		IJavaInstance component = (IJavaInstance) aConstraintComponent.refValue(sfConstraintComponent);
		IBeanProxyHost aComponentBeanProxyHost = BeanProxyUtilities.getBeanProxyHost(component);
		((ComponentProxyAdapter) aComponentBeanProxyHost).clearError(sfContainerComponents, aConstraintComponent);

		// It is possible the component or ourselves didn't instantiate.  We then can't cancel it.
		if (aComponentBeanProxyHost != null && aComponentBeanProxyHost.getErrorStatus() != IBeanProxyHost.ERROR_SEVERE && getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
			BeanAwtUtilities.invoke_remove_Component(getBeanProxy(), aComponentBeanProxyHost.getBeanProxy());
			aComponentBeanProxyHost.releaseBeanProxy();
			// This is required because AWT will not invalidate and relayout the container
			revalidateBeanProxy();
		}
	}
	
	protected void addComponent(RefObject aConstraintComponent, int position) throws ReinstantiationNeeded {
		IJavaInstance constraintAttributeValue = null;
		boolean constraintIsSet = false;
		// Get the value of the constraint attribute of the component, the component in this case is a ConstraintsComponent.
		if (aConstraintComponent.refIsSet(sfConstraintConstraint)) {
			constraintAttributeValue = (IJavaInstance) aConstraintComponent.refValue(sfConstraintConstraint);
			constraintIsSet = true;
		}
		
		// Add a listener to the constraint component for when the constraint changes.
		// To be on the safe side, remove any old ones, they shouldn't be there, but...
		Adapter a = aConstraintComponent.getExistingAdapter(ComponentConstraintAdapter.class);
		if (a != null)
			aConstraintComponent.removeAdapter(a);
		a = new ComponentConstraintAdapter();
		aConstraintComponent.addAdapter(a);
		a.setTarget(aConstraintComponent);
				
		addComponentWithConstraint(aConstraintComponent, constraintAttributeValue, position, constraintIsSet);
	}

	/*
	 * Return the first instantiated bean proxy at or after the given index.
	 * It is assumed that tests for the container being instantiated has already been done.
	 * Return null if not found.
	 */
	protected IBeanProxy getBeanProxyAt(int position) {
		List components = (List) ((RefObject) getTarget()).refValue(sfContainerComponents);
		for (int i=position; i<components.size(); i++) {
			RefObject componentConstraint = (RefObject) components.get(i);
			IBeanProxyHost componentProxyHost =
				BeanProxyUtilities.getBeanProxyHost((IJavaInstance) componentConstraint.refValue(sfConstraintComponent));
			if (componentProxyHost.isBeanProxyInstantiated())
				return componentProxyHost.getBeanProxy();
		}
		
		return null;
	}
	
	protected void addComponentWithConstraint(RefObject aConstraintComponent, IJavaInstance constraintAttributeValue, int position, boolean constraintIsSet) throws ReinstantiationNeeded {		
		// The component to actually add is within the ConstraintComponent too.
		IJavaInstance component = (IJavaInstance) aConstraintComponent.refValue(sfConstraintComponent);
		if (component == null) {
			// We have an error, the component couldn't be found. This usually means it was set somewhere else.
			processError(sfContainerComponents, new IllegalAccessException(VisualMessages.getString("Container.moved._WARN_")), aConstraintComponent); //$NON-NLS-1$
		}
		IBeanProxyHost componentProxyHost =	BeanProxyUtilities.getBeanProxyHost(component);
		// Ensure the bean is instantiated
		IComponentProxyHost componentAdapter = (IComponentProxyHost) componentProxyHost;
		componentProxyHost.instantiateBeanProxy();
		// It is possible the component didn't instantiate.  We then can't apply it
		// and we should flag it as having a warning error
		if (componentProxyHost.getErrorStatus() == IBeanProxyHost.ERROR_SEVERE || getErrorStatus() == IBeanProxyHost.ERROR_SEVERE) {
			// Either we or the setting could not be instantiated, so don't add. This guy already has his error so we don't need to
			// duplicate it.
			return;
		}
		if (!constraintIsSet) {
			Adapter a = component.getExistingAdapter(ComponentNameAdapter.class);
			if (a != null) 
				component.removeAdapter(a);
			a = new ComponentNameAdapter();
			component.addAdapter(a);
			a.setTarget(component);
			constraintAttributeValue = componentProxyHost.getBeanPropertyValue(sfName);
		} else {
			Adapter a = component.getExistingAdapter(ComponentNameAdapter.class);
			if (a != null) 
				component.removeAdapter(a);
		}

		IBeanProxy componentBeanProxy = componentProxyHost.getBeanProxy();
		IBeanProxy beforeBeanProxy = null;	// The beanproxy to go before.

		if (position != -1)
			beforeBeanProxy = getBeanProxyAt(position+1);	// Need to do +1 because we (componentBeanProxy) are already at that position in the EMF list. So we want to go before next guy.

		IBeanProxy constraintBeanProxy = null;
		if (constraintAttributeValue != null) {
			IBeanProxyHost constraintHost = BeanProxyUtilities.getBeanProxyHost(constraintAttributeValue);
			constraintBeanProxy = constraintHost.instantiateBeanProxy();
			if (constraintHost.getErrorStatus() == ERROR_SEVERE)
				((ComponentProxyAdapter) componentProxyHost).processError(sfContainerComponents, ((IErrorHolder.ExceptionError) constraintHost.getErrors().get(0)).error, aConstraintComponent);
		}

		try {
			// Invoke a method to add the component to the container.  Use add(Component) if we don't have a constraint,
			if (constraintBeanProxy == null) {
				if (beforeBeanProxy != null) {
					BeanAwtUtilities.invoke_add_Component_before(getBeanProxy(), componentBeanProxy, beforeBeanProxy);
				} else {
					BeanAwtUtilities.invoke_add_Component(getBeanProxy(), componentBeanProxy);
				}
			} else if (beforeBeanProxy != null) {
				BeanAwtUtilities.invoke_add_Component_Object_before(getBeanProxy(), componentBeanProxy, constraintBeanProxy, beforeBeanProxy);
			} else {
				BeanAwtUtilities.invoke_add_Component_Object(getBeanProxy(), componentBeanProxy, constraintBeanProxy);
			}
	
			if (BeanAwtUtilities.invoke_getLayout(getBeanProxy()) == null)
				componentAdapter.applyNullLayoutConstraints(); // Make sure the bounds are applied.
		} catch (ThrowableProxy e) {
			((ComponentProxyAdapter) componentProxyHost).processError(sfContainerComponents, e, aConstraintComponent);
		}
			

		// Now that we've added it, set the parent component.
		componentAdapter.setParentComponentProxyHost((IComponentProxyHost) this);
		// This is required because AWT will not invalidate and relayout the container	
		revalidateBeanProxy();
		
		clearError(sfContainerComponents, aConstraintComponent);
		((ComponentProxyAdapter) componentProxyHost).clearError(sfContainerComponents, aConstraintComponent);
				

	}

	/*
	 * An internal adapter added to the ConstraintComponent to listen for changes to the 
	 * constraints so that we can reapply if necessary.
	 */
	private class ComponentConstraintAdapter extends AdapterImpl {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		public ComponentConstraintAdapter() {
		}

		public boolean isAdapterForType(Object type) {
			return type == ComponentConstraintAdapter.class;
		}

		public void notifyChanged(Notification msg) {
			if (msg.getStructuralFeature() == sfConstraintConstraint) {
				switch (msg.getEventType()) {
					// ADD,REMOVE don't apply because this is a single valued feature
					case Notifier.SET :
					case Notifier.TOUCH :
						int componentPos = ((List) ((RefObject) ContainerProxyAdapter.this.getTarget()).refValue(sfContainerComponents)).indexOf(getTarget());
						addComponentWithConstraint((RefObject) getTarget(), (IJavaInstance) msg.getNewValue(), componentPos, true);
						break;
					case Notifier.UNSET :
						// This is still an add but with no constraint this time.
						componentPos = ((List) ((RefObject) ContainerProxyAdapter.this.getTarget()).refValue(sfContainerComponents)).indexOf(getTarget());
						addComponentWithConstraint((RefObject) getTarget(), null, componentPos, false);
						break;
				}
			}
		}

	}
	/*
	 * An internal adapter added to the ConstraintComponent to listen for changes to the 
	 * constraints so that we can reapply if necessary.
	 */
	private class ComponentNameAdapter extends AdapterImpl {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		public ComponentNameAdapter() {
		}

		public boolean isAdapterForType(Object type) {
			return type == ComponentNameAdapter.class;
		}

		public void notifyChanged(Notification msg) {
			if (msg.getStructuralFeature() == sfName) {
				switch (msg.getEventType()) {
					// ADD,REMOVE don't apply because this is a single valued feature
					case Notifier.SET :
					case Notifier.TOUCH :
						RefObject constraintComponent = (RefObject)((RefObject)getTarget()).refContainer();
						int componentPos = ((List) ((RefObject) ContainerProxyAdapter.this.getTarget()).refValue(sfContainerComponents)).indexOf(constraintComponent);
						addComponentWithConstraint(constraintComponent, null, componentPos, false);
						break;
					case Notifier.UNSET :
						// This is still an add but with no constraint this time.
						constraintComponent = (RefObject)((RefObject)getTarget()).refContainer();
						componentPos = ((List) ((RefObject) ContainerProxyAdapter.this.getTarget()).refValue(sfContainerComponents)).indexOf(constraintComponent);
						addComponentWithConstraint(constraintComponent, null, componentPos, false);
						break;
				}
			}
		}

	}	
	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#isValidFeature(RefStructuralFeature, Object)
	 */
	protected boolean isValidFeature(RefStructuralFeature sf, Object object) {
		// Kludge to test component too
		if (sf == sfContainerComponents) {
			IJavaInstance component = (IJavaInstance) ((RefObject) object).refValue(sfConstraintComponent);
			if (component != null) {
				ComponentProxyAdapter componentProxyHost =	(ComponentProxyAdapter) BeanProxyUtilities.getBeanProxyHost(component);
				return componentProxyHost.isTrulyValidFeature(sf, object);
			}
		}
		return super.isValidFeature(sf, object);
	}
	
}