package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyHost;
import com.ibm.etools.proxy.IBeanProxy;
/**
 * TreeEditPart for an awt Container.
 */
public class ContainerTreeEditPart extends ComponentTreeEditPart implements IEMFListener {

	public ContainerTreeEditPart(Object model) {
		super(model);
	}

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected TreeContainerEditPolicy treeContainerPolicy;
		
	protected ContainerPolicy getContainerPolicy() {
		return new ContainerPolicy(EditDomain.getEditDomain(this));	// AWT standard Contained Edit Policy
	}
	
	protected List getModelChildren() {
		// Model children is the components feature.
		// However, this returns the constraint components, but we want to return instead
		// the components themselves. They are the "model" that gets sent to the createChild and
		// component edit part.
		List constraintChildren = (List) ((RefObject) getModel()).refValue(sf_containerComponents);
		ArrayList children = new ArrayList(constraintChildren.size());
		Iterator itr = constraintChildren.iterator();
		while (itr.hasNext()) {
			RefObject con = (RefObject) itr.next();
			Object child = con.refValue(sf_constraintComponent);
			if (child != null)
				children.add(child);	// Get the component out of the constraint
				
		}
		return children;
	}


	protected EMFListenerAdapter containerAdapter;
	
	public void activate() {
		super.activate();	
		((RefObject) getModel()).addAdapter(getModelAdapter());
	}
	
	public void deactivate() {
		super.deactivate();
		((RefObject) getModel()).removeAdapter(getModelAdapter());
	}
	
	protected Adapter getModelAdapter() {
		if (containerAdapter == null) {
			containerAdapter = new EMFListenerAdapter(this, false);
		}
		return containerAdapter;
	}
	
	private EStructuralFeature 
		sf_containerLayout,
		sf_constraintComponent,
		sf_containerComponents;
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sf_containerComponents)
			refreshChildren();
		else if (sf == sf_containerLayout)
			createLayoutPolicyHelper();
	}
	
	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		((ComponentTreeEditPart) ep).setPropertySource((IPropertySource) ((RefObject) model).refContainer().getAdapter(IPropertySource.class));	// This is the property source of the actual model which is part of the constraintComponent.
		return ep;
	}
	
	protected void createEditPolicies() {
		super.createEditPolicies();
		treeContainerPolicy = new TreeContainerEditPolicy(getContainerPolicy());
		installEditPolicy(EditPolicy.TREE_CONTAINER_ROLE, treeContainerPolicy);
		createLayoutPolicyHelper();
	}
	
	protected void createLayoutPolicyHelper() {
		if (treeContainerPolicy != null) {
			// Get the layout policy helper class from the layout policy factory and
			// set it in the container helper policy.
			IJavaInstance container = (IJavaInstance)getModel();
			// It is possible the live JavaBean failed to create
			ILayoutPolicyHelper lpHelper = null;
			if (BeanProxyUtilities.getBeanProxyHost(container).getErrorStatus() != IBeanProxyHost.ERROR_SEVERE){
				IBeanProxy containerProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance)getModel());
				if (containerProxy != null) {
					ILayoutPolicyFactory lpFactory = BeanAwtUtilities.getLayoutPolicyFactory(containerProxy, EditDomain.getEditDomain(this));
					lpHelper = lpFactory.getLayoutPolicyHelper(null);
				}
			}
			
			if (lpHelper == null)
				lpHelper = new UnknownLayoutPolicyHelper();
				
			treeContainerPolicy.setPolicyHelper(lpHelper);
		}
	}
	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		
		ResourceSet rset = ((IJavaObjectInstance) model).getResourceSet();
		sf_containerLayout = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_LAYOUT);
		sf_constraintComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_COMPONENT);		
		sf_containerComponents = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_COMPONENTS);			
	}

}
