package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyDomain;
import com.ibm.etools.proxy.IBeanProxy;

/**
 * Proxy adapter for FileDialog.
 * This is needed to prevent the actual instanciation of the FileDialog in the remote VM.
 * Otherwise a modal file dialog starts up prompting us for a file.
 * For now it is just a big kludge.
 */
public class FileDialogProxyAdapter extends FrameConstructorProxyAdapter {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	// Need these features often, but they depend upon the class we are in,
	// can't get them as statics because they would be different for each Eclipse project.
	protected EStructuralFeature sfDialogModal;
	
	/**
	 * Constructor for FileDialogProxyAdapter.
	 * @param domain
	 */
	public FileDialogProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
	}

	/*
	 * @see Adapter#setTarget(Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		super.setTarget(newTarget);
		if (newTarget != null)
			sfDialogModal = JavaInstantiation.getSFeature((IJavaObjectInstance) newTarget, JBCFConstants.SF_DIALOG_MODAL);
	}

	/**
	 * @see BeanProxyAdapter#primInstantiateBeanProxy()
	 */
	protected void primInstantiateBeanProxy() {
		// This is a super duper kludge. Need a better way to do this. We can't let the FileDialog default
		// to modal because when setVisible is done, this will lock up until the dialog is dismissed, which
		// we can't do because it is not within view of the user. So here we will make sure that the default
		// setting for modal is false.
		//

		super.primInstantiateBeanProxy();
		
		if (getBeanProxy() != null) {
			IJavaInstance modal = BeanProxyUtilities.wrapperBeanProxy(
				domain.getProxyFactoryRegistry().getBeanProxyFactory().createBeanProxyWith(false),
				((RefObject)target).getResourceSet(),
				true);
			super.applied(sfDialogModal, modal, 0);	// Force it not modal.
			super.applyVisibility(false, Boolean.FALSE);	// Must never become visible.
		}
	}

	/**
	 * @see BeanProxyAdapter#applied(RefStructuralFeature, Object, int)
	 */
	protected void applied(RefStructuralFeature sf, Object newValue, int position) {
		if (sf != sfDialogModal)
			super.applied(sf, newValue, position);
	}

	/**
	 * @see ComponentProxyAdapter#applyVisibility(boolean, Boolean)
	 */
	public void applyVisibility(boolean apply, Boolean setToVisibility) {
		// There is also another kludge in that we can't let it be visible either. FileDialogs will always show up
		// in the right corner on Windows(TM) no matter what it was set to if it becomes visible.		
		super.applyVisibility(false, Boolean.FALSE);
	}

}