package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;

import com.ibm.etools.beaninfo.MethodProxy;
import com.ibm.etools.cde.emf.CDECreationTool;
import com.ibm.etools.common.command.*;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.requests.CreateRequest;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.Method;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.*;

public class FrameConstructorCreationPolicy implements CDECreationTool.CreationPolicy {
		
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
public Command getCommand(Command aCommand, final CreateRequest aCreateRequest){
	
	Command setInitStringCommand = new AbstractCommand(){
		public void execute(){
			JavaClass javaClass = (JavaClass)aCreateRequest.getNewObjectType();
			// Look at the constructors to see whether there is a null constructor or not
			// If so then just leasve the initString blank
			// Remember that the absence of any contructors means that there is an implicit null constructor
			Iterator behaviors = javaClass.getEBehaviors().iterator();
			boolean hasNullConstructor = false;
			boolean hasAnyConstructors = false;
			boolean hasFrameConstructor = false;
			while ( behaviors.hasNext() ) {
				Method javaMethod = ((MethodProxy)behaviors.next()).getMethod();
				// If we have a constructor then flag this
				if (javaMethod.isConstructor()){
					hasAnyConstructors = true;
					List methodInputParms = javaMethod.getParametersWithoutReturn();
					// See whether we have a null constructor
					if (methodInputParms.isEmpty()){
						hasNullConstructor = true;
						break;
					} else if ( methodInputParms.size() == 1 ) {
						// See whether or not there is a constructor that takes a frame argument						
						Object inputParm = methodInputParms.iterator().next();
						inputParm.toString();
					}
				}
			}
			// If we have a null constructor then leave the initializationString alone and do nothing
			if (hasNullConstructor) return;
			// Create the frame argument constructor
			// For some reason the behaviors does not return anything other than set methods - it has no constructors
			// so right now always do this
			IJavaObjectInstance newJavaObject = (IJavaObjectInstance)aCreateRequest.getNewObject();
			// For a dialog or window set the initializationString to be new java.awt.Dialog(new java.awt.Frame());
			String qualifiedClassName = ((JavaClass)newJavaObject.getJavaType()).getQualifiedNameForReflection();			
			String initString = "new " + qualifiedClassName + "(new java.awt.Frame())"; //$NON-NLS-1$ //$NON-NLS-2$
			newJavaObject.setInitializationString(initString);
		}
		// We don't really care about redo and undo for now
		public void redo(){
		}
		public boolean canExecute(){
		    return true;
		}
	};
	
	// Put the command to set the initString BEFORE the other commands, so that by the time the BeanProxyAdapter
	// is called the correct string is in place
	CompoundCommand result = new CompoundCommand(aCommand.getLabel());
	result.append(setInitStringCommand);
	result.append(aCommand);
	return result;
}
    /**
     * @see CreationPolicy#getDefaultSuperString(EClass)
     */
    public String getDefaultSuperString(EClass superClass) {    
        // Look for a null constructor up the stack, if there is one,
        // return super(), else return Frame() constructor
        Iterator behaviors = superClass.getEBehaviors().iterator();
        boolean hasNullConstructor = false;
        boolean hasAnyConstructors = false ;
        while ( behaviors.hasNext() ) {
            Method javaMethod = ((MethodProxy)behaviors.next()).getMethod();
            // If we have a constructor then flag this
            if (javaMethod.isConstructor()){  
                hasAnyConstructors = true ;              
                List methodInputParms = javaMethod.getParametersWithoutReturn();
                // See whether we have a null constructor
                if (methodInputParms==null || methodInputParms.isEmpty()){
                    hasNullConstructor = true;
                    break;
                }
            }
        }
        if (hasNullConstructor)
           return ("super()") ; //$NON-NLS-1$
        else { // No null constructor
            if (!hasAnyConstructors  &&
                superClass.getESuper() != null && 
                !superClass.getESuper().isEmpty()) 
               return getDefaultSuperString((EClass)superClass.getESuper().get(0)) ;
            else
               return ("super(new java.awt.Frame())") ;  //$NON-NLS-1$
        }
                      
    }

}
