package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.notify.Notification;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.emf.vce.BeanSubclassComposition;
import com.ibm.etools.jbcf.emf.vce.VcePackage;
import com.ibm.etools.proxy.*;
/**
 * This class is holds a proxy to the frame that is used to host bean proxies for free form components that are beans
 * so that they are parented
 * It adapts the a BeanSubclassComposition and listens to changes to the freeFormComponents relationship
 */
public class FreeFormComponentsHostAdapter extends AdapterImpl {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		
	/**
	 * This is a IComponentProxyHost wrappering a freeform dialog.
	 * There is one for each type, i.e swing and awt.
	 * @author richkulp
	 */
	private class FreeFormComponentProxyHost implements IComponentProxyHost {

		IBeanProxy dialogProxy;

		public FreeFormComponentProxyHost(String dialogClass) {
			IBeanTypeProxy dialogTypeProxy = beanProxyDomain.getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy(dialogClass);
			try {
				Point loc = BeanAwtUtilities.getOffScreenLocation();				
				dialogProxy = dialogTypeProxy.newInstance("new "+dialogClass+"("+String.valueOf(loc.x)+","+String.valueOf(loc.y)+")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				// For debug purposes it is possible to position this on screen through a preference
				final Shell shell = workbenchPage.getWorkbenchWindow().getShell();
				shell.getDisplay().asyncExec(new Runnable(){
					public void run(){
						shell.forceActive();
//						// There may be some child shells up, so bring them forward too, else they stay behind, which is annoying. 
//						Shell[] shells = shell.getShells();
//						for (int i = 0; i < shells.length; i++) {
//							shells[i].forceActive();
//						}						
					}
				});				
			} catch (ThrowableProxy e) {
				JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
			} catch (InstantiationException e) {
				JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);
			}				
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#disposeOnFreeForm(IBeanProxy)
		 */
		public void disposeOnFreeForm(IBeanProxy freeFormDialog) {
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#getParentComponentProxyHost()
		 */
		public IComponentProxyHost getParentComponentProxyHost() {
			return null;
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#getVisualComponentBeanProxy()
		 */
		public IBeanProxy getVisualComponentBeanProxy() {
			return dialogProxy;
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#hasImageListeners()
		 */
		public boolean hasImageListeners() {
			return false;
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#instantiateOnFreeForm(IBeanProxy)
		 */
		public IBeanProxy instantiateOnFreeForm(IBeanProxy freeFormDialog) {
			return null;
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#reinstantiateChild(IBeanProxyHost)
		 */
		public void reinstantiateChild(IBeanProxyHost aChildProxyHost) {
			RefObject aChildComponent = (RefObject) aChildProxyHost.getTarget();
			canceled(aChildComponent);
			// Normally we don't release because the BeanSubclassComposition adapter handles this, but we know we need to do it here.				
			aChildProxyHost.releaseBeanProxy();
			applied(aChildComponent);
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#invalidateImage()
		 */
		public void invalidateImage() {
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#refreshImage()
		 */
		public void refreshImage() {
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#setParentComponentProxyHost(IComponentProxyHost)
		 */
		public void setParentComponentProxyHost(IComponentProxyHost aParentComponent) {
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#applyNullLayoutConstraints()
		 */
		public void applyNullLayoutConstraints() {
		}

		IMethodProxy useSizeMethod;
		/*
		 * useComponentSize: Method called by main adapter when needed.
		 */
		protected void useComponentSize(IComponentProxyHost aProxy, boolean useComponentSize) {
			if (aProxy.getVisualComponentBeanProxy() != null && aProxy.getVisualComponentBeanProxy().isValid()) {
				if (useSizeMethod == null) {
					useSizeMethod = dialogProxy.getTypeProxy().getMethodProxy("setUseComponentSize", //$NON-NLS-1$
						new String[] { "java.awt.Component", "boolean" } //$NON-NLS-1$ //$NON-NLS-2$
						);
				}
	
				useSizeMethod.invokeCatchThrowableExceptions(
					dialogProxy,
					new IBeanProxy[] {
						aProxy.getVisualComponentBeanProxy(),
						beanProxyDomain.getProxyFactoryRegistry().getBeanProxyFactory().createBeanProxyWith(useComponentSize)});
	
				// Now make sure size set correctly because size would of already been reset by the dialog's pack with preferred size.
				if (useComponentSize)
					aProxy.applyNullLayoutConstraints();		
			}
		}

		/*
		 * Add a component to this dialog.
		 */
		protected void add(IComponentProxyHost aComponentProxyHost) {
			// Host the bean inside the dialog that is used so that free components are instantiated correctly
			aComponentProxyHost.instantiateOnFreeForm(dialogProxy);
			// Make our dialog bean proxy the parent of the component.  That way it will invalidate us correctly
			aComponentProxyHost.setParentComponentProxyHost(this);
			((IBeanProxyHost) aComponentProxyHost).revalidateBeanProxy();
		}
		
		/*
		 * dispose of the proxy.
		 */
		protected void dispose() {
			if (dialogProxy != null && dialogProxy.isValid())
				BeanAwtUtilities.invoke_dispose(dialogProxy);
				useSizeMethod = null;
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#childInvalidated(IComponentProxyHost)
		 */
		public void childInvalidated(IComponentProxyHost childProxy) {
		}

		/**
		 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#childValidated(IComponentProxyHost)
		 */
		public void childValidated(IComponentProxyHost childProxy) {
		}

	}
	
	public static final String FREE_FORM_COMPONENTS_HOST = "FREE_FORM_COMPONENTS_HOST_ADAPTER"; //$NON-NLS-1$
	protected FreeFormComponentProxyHost awtDialogHost, swingDialogHost;

	protected IBeanProxyDomain beanProxyDomain;
	protected BeanSubclassComposition composition;
	protected IWorkbenchPage workbenchPage;

	protected EStructuralFeature sfComponentSize, sfComponentBounds;

	protected ResourceSet rset;

	protected EClass classComponent, classJComponent;
	/**
	 * When we are created add ourself as an adaptor to the argument and make it our target
	 */
	public FreeFormComponentsHostAdapter(
		IBeanProxyDomain beanProxyDomain,
		BeanSubclassComposition composition,
		IWorkbenchPage workbenchPage) {
		super();
		this.beanProxyDomain = beanProxyDomain;
		this.composition = composition;
		this.workbenchPage = workbenchPage;

		rset = composition.getResourceSet();
		sfComponentSize = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_COMPONENT_SIZE);
		sfComponentBounds = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_COMPONENT_BOUNDS);

		classComponent = (EClass) Utilities.getJavaClass("java.awt.Component", rset); //$NON-NLS-1$
		classJComponent = (EClass) Utilities.getJavaClass("javax.swing.JComponent", rset); //$NON-NLS-1$	
	}

	protected boolean isSwingType(IBeanProxyHost aProxy) {
		return classJComponent.isSupertypeOf((EGeneralizableElement) ((RefObject) aProxy.getTarget()).refMetaObject());
	}

	protected FreeFormComponentProxyHost getDialogProxy(boolean swingType) {
		if (swingType) {
			if (swingDialogHost == null) {
				swingDialogHost = new FreeFormComponentProxyHost("com.ibm.etools.jbcf.visual.vm.FreeFormSwingDialog"); //$NON-NLS-1$
			}
			return swingDialogHost;
		} else {
			if (awtDialogHost == null) {
				awtDialogHost = new FreeFormComponentProxyHost("com.ibm.etools.jbcf.visual.vm.FreeFormAWTDialog");	//$NON-NLS-1$
			}
			return awtDialogHost;
		}
	}

	/*
	 * This is used to tell the freeform dialog that for this component it should use
	 * the size of the component and not its preferred size. This is for when a size has
	 * been explicitly set onto the component. In those cases the set size is the size to use.
	 */
	protected void useComponentSize(IComponentProxyHost aProxy, boolean useComponentSize) {
		IComponentProxyHost parent = aProxy.getParentComponentProxyHost();
		if (parent instanceof FreeFormComponentProxyHost)
			 ((FreeFormComponentProxyHost) parent).useComponentSize(aProxy, useComponentSize);
	}

	/*
	 * Add this component to the property dialog.
	 * Note: The component proxy host in this case, must also be an IBeanProxyHost.
	 */
	protected void add(IComponentProxyHost aComponentProxyHost) {
		FreeFormComponentProxyHost dialogHost = getDialogProxy(isSwingType((IBeanProxyHost) aComponentProxyHost));
		dialogHost.add(aComponentProxyHost);
	}

	private static final RefStructuralFeature SF_COMPONENTS, SF_THIS;
	static {
		VcePackage vcePack = (VcePackage) RefRegister.getPackage(VcePackage.packageURI);
		SF_COMPONENTS = vcePack.getBeanSubclassComposition_Components();
		SF_THIS = vcePack.getBeanSubclassComposition_ThisPart();
	}
	public void notifyChanged(Notification msg) {

		if (msg.getEventType() == Notifier.REMOVING_ADAPTER) {
			disposeDialogs();
		} else if (msg.getStructuralFeature() == SF_COMPONENTS || msg.getStructuralFeature() == SF_THIS) {
			// Add/Remove are used because the feature root components is multi-valued.
			switch (msg.getEventType()) {
				case Notifier.ADD :
				case Notifier.SET :
					applied(msg.getNewValue());
					break;
				case Notifier.ADD_MANY :
					Iterator itr = ((List) msg.getNewValue()).iterator();
					while (itr.hasNext())
						applied(itr.next());
					break;
				case Notifier.UNSET :
				case Notifier.REMOVE :
					canceled(msg.getOldValue());
					break;
				case Notifier.REMOVE_MANY :
					itr = ((List) msg.getOldValue()).iterator();
					while (itr.hasNext())
						canceled(itr.next());
					break;
			}
		} else if (msg.getStructuralFeature() == JBCFGraphicalEditorPart.SF_RELEASE_PROXIES) {
			disposeDialogs();
		} else if (msg.getStructuralFeature() == JBCFGraphicalEditorPart.SF_INSTANTIATE_PROXIES) {
			instantiateDialogs();
		};
	}

	private class ComponentAdapter extends AdapterImpl {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		public void notifyChanged(Notification msg) {
			RefObject sf = msg.getStructuralFeature();
			if (sf == sfComponentBounds || sf == sfComponentSize) {
				switch (msg.getEventType()) {
					case Notifier.SET :
						useComponentSize(
							(IComponentProxyHost) BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance) getTarget()),
							true);
						break;
					case Notifier.UNSET :
						// Little tricker, need to see if the other setting is still set.
						if (sf == sfComponentBounds)
							if (((RefObject) getTarget()).refIsSet(sfComponentSize))
								break; // The other is still set, so leave alone.
						if (sf == sfComponentSize)
							if (((RefObject) getTarget()).refIsSet(sfComponentBounds))
								break; // The other is still set, so leave alone.
						useComponentSize(
							(IComponentProxyHost) BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance) getTarget()),
							false);
						break;
				}
			}
		}

		public boolean isAdapterForType(Object type) {
			return ComponentAdapter.class == type;
		}
	}

	/**
	 * List for the freeFormParts relationship and see whether or not this is a component bean
	 * that requires hosting
	 */
	protected void applied(Object newValue) {
		if (!beanProxyDomain.getProxyFactoryRegistry().isValid() || !classComponent.isInstance(newValue))
			return; // Proxy VM not yet up or it isn't even an awt component.

		IJavaObjectInstance comp = (IJavaObjectInstance) newValue;
		IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost(comp);
		if (beanProxyHost instanceof IComponentProxyHost) {
			IComponentProxyHost compProxyHost = (IComponentProxyHost) beanProxyHost;

			ComponentAdapter a = (ComponentAdapter) comp.getExistingAdapter(ComponentAdapter.class);
			if (a == null) {
				// Need to listen for size/bounds change so that we know whether to be preferred size or actual size.
				a = new ComponentAdapter();
				a.setTarget(comp);
				comp.addAdapter(a);
			}

			add(compProxyHost);

			// Need to see if we should initially turn on useComponentSize.
			if (comp.refIsSet(sfComponentBounds) || comp.refIsSet(sfComponentSize))
				useComponentSize(compProxyHost, true);
		}
	}
	/**
	 * List for the freeFormParts relationship and see whether or not this is a component bean
	 * that requires hosting
	 */
	protected void canceled(Object oldValue) {
		if (!classComponent.isInstance(oldValue))
			return; // It isn't even an awt component.

		IJavaObjectInstance comp = (IJavaObjectInstance) oldValue;
		ComponentAdapter a = (ComponentAdapter) comp.getExistingAdapter(ComponentAdapter.class);
		if (a != null)
			comp.removeAdapter(a);

		IBeanProxyHost componentProxyHost = BeanProxyUtilities.getBeanProxyHost(comp);
		if (componentProxyHost instanceof IComponentProxyHost) {
			IComponentProxyHost ph = (IComponentProxyHost) componentProxyHost;
			if (ph.getParentComponentProxyHost() != null)
				ph.disposeOnFreeForm(ph.getParentComponentProxyHost().getVisualComponentBeanProxy());
		}
	}

	/** 
	 * Throwaway the dialogs being used to host free form components
	 */
	public void disposeDialogs() {
		Iterator components = composition.getComponents().iterator();
		while (components.hasNext()) {
			canceled(components.next());
		}
		if (composition.isSetThisPart()) {
			canceled(composition.getThisPart());
		}

		if (awtDialogHost != null)
			awtDialogHost.dispose();
		if (swingDialogHost != null)
			swingDialogHost.dispose();

		awtDialogHost = swingDialogHost = null;

	}

	public synchronized void instantiateDialogs() {
		// Having instantiated the bean proxy we need to apply all attribute settings that are free form parts
		Iterator components = composition.getComponents().iterator();
		while (components.hasNext()) {
			applied(components.next());
		}
		if (composition.isSetThisPart()) {
			applied(composition.getThisPart());
		}
	}

	/**
	 * isAdaptorForType method comment.
	 */
	public boolean isAdapterForType(Object type) {
		return FREE_FORM_COMPONENTS_HOST.equals(type);
	}
}