package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.draw2d.geometry.Insets;
import com.ibm.etools.draw2d.geometry.Rectangle;
import com.ibm.etools.draw2d.geometry.Dimension;
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.awt.*;
import com.ibm.etools.jbcf.InsetsJavaClassCellEditor;
import com.ibm.etools.jbcf.GridBagConstraint;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.emf.ClassDecoratorFeatureAccess;
import com.ibm.etools.emf.jbcf.BeanDecorator;
import com.ibm.etools.cdm.KeyedInteger;
import java.util.*;
/**
 * GridBag layout policy helper.
 *
 * Note: the constraint is a GridBagConstraint. It will be converted
 * to the appropriate java object instance when necessary.
 */
public class GridBagLayoutPolicyHelper extends LayoutPolicyHelper {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	static Integer ZERO_INTEGER = new Integer(0);

	private IJavaObjectInstance fContainerBean;
	List fComponents = null;
	ArrayList 
		childrenXYBounds, 
		columnDividerPositions, rowDividerPositions, 
		sortedLeftEdges, sortedRightEdges, sortedTopEdges, sortedBottomEdges = null;
		
	private EStructuralFeature
		sfGridX,
		sfGridY,
		sfGridWidth,
		sfGridHeight,
		sfWeightX,
		sfWeightY,
		sfFill,
		sfInsets,
		sfIPadX,
		sfIPadY;
		
	private JavaHelpers primInt;
		
public GridBagLayoutPolicyHelper(ContainerPolicy ep) {
	super(ep);
}

public GridBagLayoutPolicyHelper() {
}

/**
 * Assign the fill value from a fill preference based on the component type.
 * Assign the weightx and weighty based on the fill preference.
 */
protected void adjustForFillPreferences(IJavaObjectInstance component, GridBagConstraint constraint) {
	
	int fillPref = getGridBagConstraintsFillPreference(component);
	
	if (fillPref == GridBagConstraint.NONE) return;	// no need to adjust if the fill preference is NONE
	
	constraint.fill = fillPref;
	if (fillPref == GridBagConstraint.HORIZONTAL) {
			constraint.weightx = 1.0d;
	} else if (fillPref == GridBagConstraint.VERTICAL) {
			constraint.weighty = 1.0d;
	} else if (fillPref == GridBagConstraint.BOTH) {
			constraint.weightx = 1.0d;
			constraint.weighty = 1.0d;
	}
}
	
/**
 * Convert the IDE GridBagConstraint to the java object instance constraint
 */
protected IJavaObjectInstance convertConstraint(Object constraint) {
	GridBagConstraint gridBagConstraint = (GridBagConstraint) constraint;
	GridBagConstraint defaultConstraint = new GridBagConstraint();	//  used to compare default values
	ResourceSet rset = getContainer().getResourceSet();
	IJavaObjectInstance javaGridBagConstraint = (IJavaObjectInstance) BeanUtilities.createJavaObject(
		"java.awt.GridBagConstraints", //$NON-NLS-1$
		rset,
		null
	);
	if (gridBagConstraint.gridx != defaultConstraint.gridx) {
		Object gridx = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.gridx));
		javaGridBagConstraint.refSetValue(sfGridX, gridx);
	}
	if (gridBagConstraint.gridy != defaultConstraint.gridy) {
		Object gridy = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.gridy));
		javaGridBagConstraint.refSetValue(sfGridY, gridy);
	}
	if (gridBagConstraint.gridwidth != defaultConstraint.gridwidth) {
		Object gridwidth = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.gridwidth));
		javaGridBagConstraint.refSetValue(sfGridWidth, gridwidth);
	}
	if (gridBagConstraint.gridheight != defaultConstraint.gridheight) {
		Object gridheight = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.gridheight));
		javaGridBagConstraint.refSetValue(sfGridHeight, gridheight);
	}
	if (gridBagConstraint.weightx != defaultConstraint.weightx) {
		Object weightx = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.weightx));
		javaGridBagConstraint.refSetValue(sfWeightX, weightx);
	}
	if (gridBagConstraint.weighty != defaultConstraint.weighty) {
		Object weighty = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.weighty));
		javaGridBagConstraint.refSetValue(sfWeightY, weighty);
	}
	if (gridBagConstraint.fill != defaultConstraint.fill) {
		Object fill = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.fill));
		javaGridBagConstraint.refSetValue(sfFill, fill);
	}
	// The insets value is a class, not a primitive.
	if (!gridBagConstraint.insets.equals(defaultConstraint.insets)) {
		Insets insets = gridBagConstraint.insets;
		IJavaObjectInstance insetsBean = (IJavaObjectInstance) BeanUtilities.createJavaObject(
			"java.awt.Insets", //$NON-NLS-1$
			rset, 
			InsetsJavaClassCellEditor.getJavaInitializationString(insets)
		);
		javaGridBagConstraint.refSetValue(sfInsets, insetsBean);
	}
	if (gridBagConstraint.ipadx != defaultConstraint.ipadx) {
		Object ipadx = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.ipadx));
		javaGridBagConstraint.refSetValue(sfIPadX, ipadx);
	}
	if (gridBagConstraint.ipadx != defaultConstraint.ipadx) {
		Object ipady = BeanUtilities.createJavaObject(primInt, rset, String.valueOf(gridBagConstraint.ipady));
		javaGridBagConstraint.refSetValue(sfIPadY, ipady);
	}
	
	return javaGridBagConstraint;
}

/**
 * Determine what the default constraint(s) are for this layout manager
 * and assign a constraint to each child.
 * Return a List with a constraint for each child.
 */
public List getDefaultConstraint(List children) {
	ArrayList constraints = new ArrayList(children.size());

	// We are going to iterate through the children and generate a new GridBagConstraints bean
	for (int i = 0; i < children.size(); i++) {
		constraints.add(getDefaultConstraint((IJavaObjectInstance)children.get(i)));
	}
	return constraints;
}
/**
 * Return a default GridBagConstaints constructed with a null ctor
 */
public GridBagConstraint getDefaultConstraint(IJavaObjectInstance component) {
	GridBagConstraint constraint = new GridBagConstraint();
	adjustForFillPreferences(component, constraint);
	return constraint;
}
/**
 * Return a list of gridBag constraints to be assigned to each of the components.
 */
protected List getGridBagConstraints (List components) {
	resetLists();
	fComponents = components;
	ArrayList constraints = new ArrayList(components.size());
	for (int i = 0; i < components.size(); i++) {
		// Find the nearby Column and Row dividers
		int nearestLeftColumnDivider = 0,
			nearestRightColumnDivider = 0,
			nearestAboveRowDivider = 0,
			nearestBelowRowDivider = 0,
			position = 0;
		Rectangle childBounds = getBounds((IJavaObjectInstance) components.get(i));
		// find the nearest left column divider
		for (int j = 0; j < getColumnDividerPositions().size(); j++) {
			position = ((Integer)getColumnDividerPositions().get(j)).intValue();
			if (position <= childBounds.x) {
				nearestLeftColumnDivider = j;
			}
		}
		// find the nearest right column divider
		for (int j = 0; j < getColumnDividerPositions().size(); j++) {
			position = ((Integer)getColumnDividerPositions().get(j)).intValue();
			if (  position >= (childBounds.x + childBounds.width) ) {
				nearestRightColumnDivider = j;
				break;
			}
		}
		// find the nearest top row divider
		for (int j = 0; j < getRowDividerPositions().size(); j++) {
			position = ((Integer)getRowDividerPositions().get(j)).intValue();
			if (position <= childBounds.y) {
				nearestAboveRowDivider = j;
			}
		}
		// find the nearest bottom row divider
		for (int j = 0; j < getRowDividerPositions().size(); j++) {
			position = ((Integer)getRowDividerPositions().get(j)).intValue();
			if ( position >= (childBounds.y + childBounds.height) ) {
				nearestBelowRowDivider = j;
				break;
			}
		}
		// Calulate the Insets
		Insets insets = new Insets (
			childBounds.y - ((Integer)getRowDividerPositions().get(nearestAboveRowDivider)).intValue(),
			childBounds.x - ((Integer)getColumnDividerPositions().get(nearestLeftColumnDivider)).intValue(),
			((Integer)getRowDividerPositions().get(nearestBelowRowDivider)).intValue() - (childBounds.y + childBounds.height),
			((Integer)getColumnDividerPositions().get(nearestRightColumnDivider)).intValue() - (childBounds.x + childBounds.width)
			);
		// Calculate the padding.
		Dimension dim = getPreferredSize((IJavaObjectInstance)components.get(i));
		int ipadx = childBounds.width - dim.width;
		int ipady = childBounds.height - dim.height;
		
		// Create a new GridBagConstaints and assign the values that we calculated previously.
		GridBagConstraint constraint = new GridBagConstraint();
		constraint.gridx = nearestLeftColumnDivider;
		constraint.gridy = nearestAboveRowDivider;
		constraint.gridwidth = nearestRightColumnDivider - nearestLeftColumnDivider;
		constraint.gridheight = nearestBelowRowDivider - nearestAboveRowDivider;
		constraint.insets = insets;
		constraint.ipadx = ipadx;
		constraint.ipady = ipady;
		// Adjust for fill preference and set the weightx and weighty
		adjustForFillPreferences((IJavaObjectInstance) components.get(i), constraint);
		// Add this constraint to the list of constraints
		constraints.add(constraint);
	}
	return constraints;
}
/**
 * Adds additional column or row dividers based on the position of each child in the container
 */
protected void addInternalDividerPositionsTo(List dividerPositions, List near, List far) {
	int farIndex = 0, nearIndex = 0, thisEdge = 0;
	int prevEdge = ((Integer)near.get(0)).intValue();
	List prevEdgeSource = near;
	List thisEdgeSource = null;
	while (nearIndex < near.size() && farIndex < far.size()) {
		if ( ((Integer)near.get(nearIndex)).intValue() < ((Integer)far.get(farIndex)).intValue() ) {
			thisEdge = ((Integer)near.get(nearIndex)).intValue();
			nearIndex++;
			thisEdgeSource = near;
		} else {
			thisEdge = ((Integer)far.get(farIndex)).intValue();
			farIndex++;
			thisEdgeSource = far;
		}
		if ( prevEdgeSource.equals(far) && thisEdgeSource.equals(near) ) {
			// Creates divider using middle of near and far
			dividerPositions.add(new Integer((thisEdge + prevEdge) / 2));
		}
		prevEdge = thisEdge;
		prevEdgeSource = thisEdgeSource;
	}
}
protected List getChildrenXYBounds() {
	if (childrenXYBounds == null) {
		childrenXYBounds = new ArrayList(fComponents.size());
		// create a list of the xy constraints for each child component
		for (int i = 0; i < fComponents.size(); i++) {
			IJavaObjectInstance comp = (IJavaObjectInstance)fComponents.get(i);
			IBeanProxy compProxy = BeanProxyUtilities.getBeanProxy(comp);
			IRectangleBeanProxy rectangleProxy = (IRectangleBeanProxy) BeanAwtUtilities.invoke_getBounds(compProxy);
			Rectangle r = new Rectangle(rectangleProxy.getX(),rectangleProxy.getY(),rectangleProxy.getWidth(),rectangleProxy.getHeight());
			childrenXYBounds.add(i, r);
		}
	}
	return childrenXYBounds;
}
/**
 * Answer a collection of start x positions for each column in the gridbag layout
 */
 protected List getColumnDividerPositions () {
 	if (columnDividerPositions == null) {
 		columnDividerPositions = new ArrayList();
 		// Now add the left-most edge, either the container's left edge (0) or the leftest left edge of a component.
 		Integer leftMostEdge = (Integer)getSortedLeftEdges().get(0);
		if (ZERO_INTEGER.compareTo( leftMostEdge ) <= 0) 
	 		columnDividerPositions.add(ZERO_INTEGER);
		else
	 		columnDividerPositions.add( leftMostEdge );
	 	// Add the internal boundaries based on the right and left edges
	 	addInternalDividerPositionsTo(columnDividerPositions, getSortedLeftEdges(), getSortedRightEdges());
 		// Add the right-most boundary, either the containers right edge or the rightest-right of a component.
 		Integer rightMostEdge = (Integer)getSortedRightEdges().get(getSortedRightEdges().size() - 1);
 		Rectangle containerBounds = getBounds(getContainer());
 		Integer containerWidth = new Integer(containerBounds.width);
		if (containerWidth.compareTo( rightMostEdge ) >= 0) 
	 		columnDividerPositions.add(containerWidth);
		else
	 		columnDividerPositions.add(rightMostEdge);
	 	Collections.sort(columnDividerPositions);
 	}
 	return columnDividerPositions;
 }
/**
 * Return a Rectangle containing the bounds of a bean.
 */
protected Rectangle getBounds(IJavaObjectInstance aBean) {
	IBeanProxy beanProxy = BeanProxyUtilities.getBeanProxy(aBean);
	IRectangleBeanProxy rectangleProxy = (IRectangleBeanProxy) BeanAwtUtilities.invoke_getBounds(beanProxy);
	Rectangle r = new Rectangle(rectangleProxy.getX(),rectangleProxy.getY(),rectangleProxy.getWidth(),rectangleProxy.getHeight());
	return r;
}

/**
 * Return the GridBagLayout origin
 */
public com.ibm.etools.draw2d.geometry.Point getContainerLayoutOrigin() {
	IBeanProxy aContainerBeanProxy = BeanProxyUtilities.getBeanProxy(getContainer());
	IBeanProxy aLayoutManagerBeanProxy = BeanAwtUtilities.invoke_getLayout(aContainerBeanProxy);
	IMethodProxy getLayoutOrigin =
		aLayoutManagerBeanProxy.getProxyFactoryRegistry().getMethodProxyFactory().getMethodProxy(
			aLayoutManagerBeanProxy.getTypeProxy().getTypeName(),
			"getLayoutOrigin", //$NON-NLS-1$
			null
		);
	IPointBeanProxy pointProxy = (IPointBeanProxy)getLayoutOrigin.invokeCatchThrowableExceptions(aLayoutManagerBeanProxy);
	return new com.ibm.etools.draw2d.geometry.Point(pointProxy.getX(), pointProxy.getY());
}
/**
 * Return the GridBagLayout dimensions which is 2 dimensional array that contains 2 arrays:
 *  1. an int array of all the column widths
 *  2. an int array of all the row heights
 */
public int[ ][ ] getContainerLayoutDimensions() {
	int [ ] 	columnWidths = null, 
		rowHeights = null;
	int [ ][ ] result = new int[2][];
	IBeanProxy aContainerBeanProxy = BeanProxyUtilities.getBeanProxy(getContainer());
	IBeanProxy aLayoutManagerBeanProxy = BeanAwtUtilities.invoke_getLayout(aContainerBeanProxy);
	IMethodProxy getLayoutDimensions =
		aLayoutManagerBeanProxy.getProxyFactoryRegistry().getMethodProxyFactory().getMethodProxy(
			aLayoutManagerBeanProxy.getTypeProxy().getTypeName(),
			"getLayoutDimensions", //$NON-NLS-1$
			null
		);
	IArrayBeanProxy arrayProxy = (IArrayBeanProxy)getLayoutDimensions.invokeCatchThrowableExceptions(aLayoutManagerBeanProxy);
	try {
		IArrayBeanProxy arrayProxyColumnWidths = (IArrayBeanProxy)arrayProxy.get(0);
		if (arrayProxyColumnWidths != null) {
			columnWidths = new int[arrayProxyColumnWidths.getLength()];
			for (int i = 0; i < arrayProxyColumnWidths.getLength(); i++) {
				columnWidths[i] = ((IIntegerBeanProxy)arrayProxyColumnWidths.get(i)).intValue();
			}
			result[0] = columnWidths;
		}
		IArrayBeanProxy arrayProxyRowHeights = (IArrayBeanProxy)arrayProxy.get(1);
		if (arrayProxyRowHeights != null) {
			rowHeights = new int[arrayProxyRowHeights.getLength()];
			for (int i = 0; i < arrayProxyRowHeights.getLength(); i++) {
				rowHeights[i] = ((IIntegerBeanProxy)arrayProxyRowHeights.get(i)).intValue();
			}
			result[1] = rowHeights;
		}
	} catch (ThrowableProxy exc) {
		return null;
	}
	return result;
}

/**
 * Key for accessing the gridbar fill preference on a per class basis. Stored on BeanDecorator.
 */
public static final String GRIDBAG_FILL_PREFERENCE_KEY = "com.ibm.etools.jbcf.visual.gridbagfilepreferencekey"; //$NON-NLS-1$

/**
 * Get the fill preference for a specific component type... i.e. for JTextField, GridBagConstraints.HORIZONTAL
 */
protected int getGridBagConstraintsFillPreference(IJavaObjectInstance component) {
	
	BeanDecorator bd = (BeanDecorator) ClassDecoratorFeatureAccess.getDecoratorWithKeyedFeature((EClassifier) component.getJavaType(), BeanDecorator.class, GRIDBAG_FILL_PREFERENCE_KEY);
	if (bd != null) {
		KeyedInteger ki = (KeyedInteger) bd.getKeyedValue(GRIDBAG_FILL_PREFERENCE_KEY);
		return ki.getValueValue();
	}

	return GridBagConstraint.NONE;	// default for Component and all others
}
/**
 * Return a Dimension containing the preferred size of a bean.
 */
protected Dimension getPreferredSize(IJavaObjectInstance aBean) {
	IBeanProxy aBeanProxy = BeanProxyUtilities.getBeanProxy(aBean);
	IMethodProxy getPreferredSizeMethodProxy =
		aBeanProxy.getProxyFactoryRegistry().getMethodProxyFactory().getMethodProxy(
			aBeanProxy.getTypeProxy().getTypeName(),
			"getPreferredSize", //$NON-NLS-1$
			null
		);
	IDimensionBeanProxy dimensionProxy = (IDimensionBeanProxy)getPreferredSizeMethodProxy.invokeCatchThrowableExceptions(aBeanProxy);
	Dimension dim = new Dimension(dimensionProxy.getWidth(),dimensionProxy.getHeight());
	return dim;
}

/**
 * Answer a collection of start y positions for each row in the gridbag layout
 */
protected List getRowDividerPositions () {
 	if (rowDividerPositions == null) {
 		rowDividerPositions = new ArrayList();
 		// Now add the left-most edge, either the container's left edge (0) or the leftest left edge of a component.
 		Integer topMostEdge = (Integer)getSortedTopEdges().get(0);
		if (ZERO_INTEGER.compareTo( topMostEdge ) <= 0) 
	 		rowDividerPositions.add(ZERO_INTEGER);
		else
	 		rowDividerPositions.add( topMostEdge );
	 	// Add the internal boundaries based on the top and bottom edges
	 	addInternalDividerPositionsTo(rowDividerPositions, getSortedTopEdges(), getSortedBottomEdges());
 		// Add the right-most boundary, either the containers right edge or the rightest-right
 		Integer bottomMostEdge = (Integer)getSortedBottomEdges().get(getSortedBottomEdges().size() - 1);
  		Rectangle containerBounds = getBounds(getContainer());
		Integer containerHeight = new Integer(containerBounds.height);
		if (containerHeight.compareTo( bottomMostEdge ) >= 0) 
	 		rowDividerPositions.add(containerHeight);
		else
	 		rowDividerPositions.add(bottomMostEdge);
	 	Collections.sort(rowDividerPositions);
 	}
 	return rowDividerPositions;
 }
/**
 *Return a sorted collection of ints representing the bottom edges of all components.
 */
protected List getSortedBottomEdges() {
	if (sortedBottomEdges == null) {
		sortedBottomEdges = new ArrayList(getChildrenXYBounds().size());
		for (int i = 0; i < getChildrenXYBounds().size(); i++) {
			Rectangle rect = (Rectangle)getChildrenXYBounds().get(i);
			sortedBottomEdges.add(new Integer(rect.y + rect.height));
		}
		Collections.sort(sortedBottomEdges);
	}
	return sortedBottomEdges;
}
/**
 *Return a sorted collection of ints representing the left edges of all components.
 */
protected List getSortedLeftEdges() {
	if (sortedLeftEdges == null) {
		sortedLeftEdges = new ArrayList(getChildrenXYBounds().size());
		for (int i = 0; i < getChildrenXYBounds().size(); i++) {
			Rectangle rect = (Rectangle)getChildrenXYBounds().get(i);
			sortedLeftEdges.add(new Integer(rect.x));
		}
		Collections.sort(sortedLeftEdges);
	}
	return sortedLeftEdges;
}
/**
 *Return a sorted collection of ints representing the right edges of all components.
 */
protected List getSortedRightEdges() {
	if (sortedRightEdges == null) {
		sortedRightEdges = new ArrayList(getChildrenXYBounds().size());
		for (int i = 0; i < getChildrenXYBounds().size(); i++) {
			Rectangle rect = (Rectangle)getChildrenXYBounds().get(i);
			sortedRightEdges.add(new Integer(rect.x + rect.width));
		}
		Collections.sort(sortedRightEdges);
	}
	return sortedRightEdges;
}
/**
 *Return a sorted collection of ints representing the top edges of all components.
 */
protected List getSortedTopEdges() {
	if (sortedTopEdges == null) {
		sortedTopEdges = new ArrayList(getChildrenXYBounds().size());
		for (int i = 0; i < getChildrenXYBounds().size(); i++) {
			Rectangle rect = (Rectangle)getChildrenXYBounds().get(i);
			sortedTopEdges.add(new Integer(rect.y));
		}
		Collections.sort(sortedTopEdges);
	}
	return sortedTopEdges;
}
private void resetLists() {
	childrenXYBounds = null;
	columnDividerPositions = null;
	rowDividerPositions = null;
	sortedLeftEdges = null;
	sortedRightEdges = null;
	sortedTopEdges = null;
	sortedBottomEdges = null;
}
public void setContainerPolicy(ContainerPolicy policy) {
	super.setContainerPolicy(policy);
	
	if (policy != null) {
		// Eventually we will be set with a policy. At that time we can compute these.
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(policy.getEditDomain());
		sfGridX = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_GRIDX);
		sfGridY = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_GRIDY);
		sfGridWidth = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_GRIDWIDTH);
		sfGridHeight = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_GRIDHEIGHT);	
		sfWeightX = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_WEIGHTX);
		sfWeightY = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_WEIGHTY);
		sfFill = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_FILL);	
		sfInsets = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_INSETS);
		sfIPadX = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_IPADX);	
		sfIPadY = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_GRIDBAGCONSTRAINTS_IPADY);		
		
		primInt = (JavaHelpers) Utilities.getJavaClass("int", rset);	 //$NON-NLS-1$
	}
	resetLists();
}
}