package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.common.command.Command;
import com.ibm.etools.gef.commands.CompoundCommand;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.cde.commands.ApplyAttributeSettingCommand;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.emf.resource.ResourceSet;
import java.util.*;

/**
 * This helper class is used when a container's layout is changed
 * to GridLayout. Each of the constraints for the container's children 
 * need to removed.
 */
public class GridLayoutSwitcher extends LayoutSwitcher {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected FlowLayoutPolicyHelper helper;

public GridLayoutSwitcher(ContainerPolicy cp) {
	super(cp);
	helper = new FlowLayoutPolicyHelper(cp);
}
/**
 * Return a command to apply the correct number to the property "rows" of a GridLayout. 
 * By explicitly setting the number of rows in a GridLayout, the columns created will default 
 * based on the number of components. For example, if you set the rows=3, and there are 
 * 10 components, the grid will be 3rows X 4columns.
 */
public Command getCommand(RefStructuralFeature sf, IJavaObjectInstance newManager) {
	// Calculate the number of rows based on the number of children and create the command
	// to apply it to the layout manager.
	IJavaObjectInstance containerBean = getContainerBean();
	ResourceSet rset = containerBean.getResourceSet();
	List components = (List) containerBean.refValue(JavaInstantiation.getSFeature(containerBean, JBCFConstants.SF_CONTAINER_COMPONENTS));
	int rows = 1;
	// The calculation is such that we want a grid that is somewhat larger horizontally (#columns)
	// than vertically (#rows). Here's an example table that represents the algorithm:
	
	// #components	          #rows    #columns
	//	1		1	1
	//	2		1	2
	//	3		2	2	<-- row increments here
	//	6		2	3
	//	7		3	3	<-- row increments here
	//	12		3	4
	//	13		4	4	<-- row increments here
	//	...		   ...
	for (rows = 1; rows*(rows+1) < components.size(); rows++){};
	
	ApplyAttributeSettingCommand setRowsCmd = new ApplyAttributeSettingCommand();
	setRowsCmd.setTarget(newManager);
	JavaClass GRID_LAYOUT = (JavaClass)Utilities.getJavaClass("java.awt.GridLayout",rset); //$NON-NLS-1$
	RefStructuralFeature SF_ROWS = (RefStructuralFeature)GRID_LAYOUT.metaObject("rows"); //$NON-NLS-1$
	setRowsCmd.setAttribute(SF_ROWS);
	Object rowsSetting = BeanUtilities.createJavaObject((JavaHelpers) Utilities.getJavaClass("int", rset), rset, String.valueOf(rows)); //$NON-NLS-1$
	setRowsCmd.setAttributeSettingValue(rowsSetting);
	// Now create the commands to remove the constraints from the children and apply 
	// the layout manager to the container.
	return setRowsCmd.chain(super.getCommand(sf, newManager));
}
/**
 * Returns a compound command containing the child constraint commands to position the
 * children within a flow Layout container.
 */
protected Command getChangeConstraintsCommand(List children) {		
	return helper.getAddChildrenCommand(children, Collections.nCopies(children.size(), null), null);	// Now let the helper add them back correctly.
}
}
