package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.IBeanProxyHost;
import com.ibm.etools.proxy.IBeanProxy;
/**
 * Interface for a proxy host that is a component
 */
public interface IComponentProxyHost  {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
/**
 * Dispost the bean proxy giving it the dialog that is used to host
 * components for the free form
 */
void disposeOnFreeForm(IBeanProxy freeFormDialog);
/**
 * Return the parent component proxy adaptor
 */
IComponentProxyHost getParentComponentProxyHost();
/**
 * Return a bean proxy to the component
 */
IBeanProxy getVisualComponentBeanProxy();
/**
 * Answer whether or not we have image listeners attached to the component image
 */
boolean hasImageListeners();
/**
 * Instantiate the bean proxy giving it the dialog that is used to host
 * components for the free form
 */
IBeanProxy instantiateOnFreeForm(IBeanProxy freeFormDialog);
/**
 * Reinstantiate the child proxy.
 */
public void reinstantiateChild(IBeanProxyHost aChildProxy);
/**
 * Invalidate any image we may have on behalf of the component
 */
void invalidateImage();
/**
 * Refresh the image
 */
void refreshImage();

/**
 * child has been invalidated. Called when a direct has been invalidated.
 * Implementor may decide whether to pass on up to parent or not.
 */
void childInvalidated(IComponentProxyHost childProxy);

/**
 * child has been validated. Called when a direct has been validated.
 * Implementor may decide whether to pass on up to parent or not.
 */
void childValidated(IComponentProxyHost childProxy);

/**
 * Set the parent component proxy adaptor
 */
void setParentComponentProxyHost(IComponentProxyHost aParentComponent);

/**
 * Apply null layout constraints. This is called when added to a parent
 * so it should now apply the null layout constraints, i.e. bounds/size/location.
 */
void applyNullLayoutConstraints();
}