package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.IFileEditorInput;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.propertysheet.INeedData;

/**
 * Cell editor for javax.swing.ImageIcon.
 * @version 	1.0
 * @author
 */
public class ImageCellEditor extends DialogCellEditor implements IJBCFCellEditor, INeedData {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain fEditDomain;
	private IconDialog iconDialog;
	private String path = ""; //$NON-NLS-1$
	private Object fRealValue;

	public ImageCellEditor(Composite parent) {
		super(parent);
	}
	/* 
	 * Create an instance of the MOF BeanType with string specified
	 */
	private Object createValue(String path) {
		if (path == null || path.equals("")) { //$NON-NLS-1$
			return null;
		}

		this.path = path;
		return BeanUtilities.createJavaObject("java.awt.Image", //$NON-NLS-1$
		JBCFEditDomainHelper.getResourceSet(fEditDomain), getJavaInitializationString());
	}

	private static final String IMAGE_INITSTRING_START = "java.awt.Toolkit.getDefaultToolkit().getImage("; //$NON-NLS-1$
	/**
	 * Parse through the initialization string and strip out the path information.
	 */
	protected String getPathFromInitializationString(String initStr) {
		if (initStr == null || !initStr.startsWith(IMAGE_INITSTRING_START))
			return ""; // Not valid format //$NON-NLS-1$

		int lastParen = initStr.lastIndexOf(')');
		if (lastParen == -1)
			return ""; // Not valid format //$NON-NLS-1$

		return initStr.substring(IMAGE_INITSTRING_START.length(), lastParen).trim(); // Get the arg
	}
	/**
	 * getJavaInitializationString method comment.
	 */
	public String getJavaInitializationString() {
		return IMAGE_INITSTRING_START + path + ")"; //$NON-NLS-1$
	}

	protected void updateContents(Object aValue) {
		Label lbl = getDefaultLabel();
		if (lbl == null)
			return;

		String text = ""; //$NON-NLS-1$
		if (aValue != null) {
			String initString = getPathFromInitializationString(((IJavaObjectInstance) aValue).getInitializationString());
			if (initString != "") //$NON-NLS-1$
				text = initString;
		}
		lbl.setText(text);
	}

	public Object openDialogBox(Control cellEditorWindow) {
		IconDialog iconDialog =
			new IconDialog(
				cellEditorWindow.getShell(),
				((IFileEditorInput) fEditDomain.getEditorPart().getEditorInput()).getFile().getProject());

		IJavaObjectInstance aValue = (IJavaObjectInstance) getValue();
		if (aValue != null) {
			path = getPathFromInitializationString(aValue.getInitializationString());
			if (path != "") { //$NON-NLS-1$
				iconDialog.setValue(path);
			}
		}
		int returnCode = iconDialog.open();
		// The return code says whether or not OK was pressed on the property editor
		if (returnCode == Window.OK) {
			return createValue(iconDialog.getValue());
		} else
			return null;
	}
	public void setData(Object data) {
		fEditDomain = (EditDomain) data;
	}
}