package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.awt.*;
import com.ibm.etools.proxy.*;

/**
 * Standard AWT Constants for the JCF.
 *
 * Use the static accessor method (getConstants()) to get the appropriate 
 * constants instance for the registry. This is done so that until the
 * the constants are needed, they aren't registered.
 *
 * NOTE: Since everything in here just proxies, there is
 *       no need to have them separated by VM. 
 *
 * This is final because this can't be extended. This specific one
 * will be registered with the factory for this key. Any extensions
 * must be done in their own constants and registry key.
 *
 * Creation date: (4/7/00 4:47:48 PM)
 * @author: Administrator
 */
public final class JCFAwtConstants {
		
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String REGISTRY_KEY = "JCF.PROXY.AWT.CONSTANTS:"; //$NON-NLS-1$
	
		
	private ProxyFactoryRegistry fRegistry;
				
	private IMethodProxy 
		fDataCollectorAbort,
		fDataCollectorBusy,
		fDataCollectorWait,
		fDataCollectorStartImage,
		fDataCollectorStartComponent,
		fDataCollectorStartProducer;
	
	/**
	 * Get the constants instance for the specified registry.
	 */
	public static JCFAwtConstants getConstants(ProxyFactoryRegistry registry) {
		JCFAwtConstants constants = (JCFAwtConstants) registry.getConstants(REGISTRY_KEY);
		if (constants == null)
			registry.registerConstants(REGISTRY_KEY, constants = new JCFAwtConstants(registry));
		return constants;
	}
			
	
	protected JCFAwtConstants(ProxyFactoryRegistry registry) {
		fRegistry = registry;
	}
	
	public IMethodProxy getDataCollectorAbort() {
		if (fDataCollectorAbort == null) {
			fDataCollectorAbort = fRegistry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.ImageDataCollector").getMethodProxy("abort"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return fDataCollectorAbort;
	}
	
	public IMethodProxy getDataCollectorStartImage() {
		if (fDataCollectorStartImage == null) {
			fDataCollectorStartImage = fRegistry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.ImageDataCollector").getMethodProxy("start", new String[] {"java.awt.Image"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		return fDataCollectorStartImage;
	}

	public IMethodProxy getDataCollectorStartComponent() {
		if (fDataCollectorStartComponent == null) {
			fDataCollectorStartComponent = fRegistry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.ImageDataCollector").getMethodProxy("start", new String[] {"java.awt.Component"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		return fDataCollectorStartComponent;
	}
	
	public IMethodProxy getDataCollectorStartProducer() {
		if (fDataCollectorStartProducer == null) {
			fDataCollectorStartProducer = fRegistry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.ImageDataCollector").getMethodProxy("start", new String[] {"java.awt.image.ImageProducer"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		return fDataCollectorStartProducer;
	}
	
	public IMethodProxy getDataCollectorWait() {
		if (fDataCollectorWait == null) {
			fDataCollectorWait = fRegistry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.ImageDataCollector").getMethodProxy("waitForCompletion"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return fDataCollectorWait;
	}
	
	public IMethodProxy getDataCollectorBusy() {
		if (fDataCollectorBusy == null) {
			fDataCollectorBusy = fRegistry.getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.ImageDataCollector").getMethodProxy("isCollecting"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return fDataCollectorBusy;
	}		
	
}
