package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaURL;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyDomain;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.IMethodProxy;

public class JDesktopPaneProxyAdapter extends JLayeredPaneProxyAdapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IMethodProxy fGetDesktopManagerMethodProxy;
	protected IMethodProxy fActivateFrameMethodProxy;
	protected JavaClass fJInternalFrameClass;

	public JDesktopPaneProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
		
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain.getEditDomain());		
		fJInternalFrameClass = (JavaClass) rset.getObjectAndLoad(new JavaURL("javax.swing.JInternalFrame").getFullString()); //$NON-NLS-1$
	}
	/**
     * Activtates the selected JInternalFrame for this desktop pane. 
     * This is public and is used by the edit parts to select a frame 
     * so it can be brought to the front in the desktop pane.
	 */
	public void activateFrame(IJavaObjectInstance internalFrameModel) {
		if (!fJInternalFrameClass.isInstance(internalFrameModel)) return;
		
		// To activate the frame we need to first get the DesktopManager
		// from the DesktopPane, then have the DTM activate the frame.
		if (fGetDesktopManagerMethodProxy == null) {
			fGetDesktopManagerMethodProxy =
				getBeanProxy().getTypeProxy().getMethodProxy("getDesktopManager"); //$NON-NLS-1$
		}
		IBeanProxy dtMgrBeanProxy = fGetDesktopManagerMethodProxy.invokeCatchThrowableExceptions(getBeanProxy());
		if (dtMgrBeanProxy != null) {
			if (fActivateFrameMethodProxy == null) {
				fActivateFrameMethodProxy =
					dtMgrBeanProxy.getTypeProxy().getMethodProxy("activateFrame", "javax.swing.JInternalFrame"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			IBeanProxy frameBeanProxy = BeanProxyUtilities.getBeanProxy(internalFrameModel);
			fActivateFrameMethodProxy.invokeCatchThrowableExceptions(dtMgrBeanProxy, frameBeanProxy);
		}
		revalidateBeanProxy();
	}
	/**
	 * @see BeanProxyAdapter#applied(RefStructuralFeature, Object, int)
	 */
	protected void applied(RefStructuralFeature sf, Object newValue, int position) {
		if (sf == sfContainerComponents) {
			// We are adding a new guy. Need to make it always visible.
			// Get the value of the constraint attribute of the component, the component in this case is a ConstraintsComponent.
			if (((RefObject) newValue).refIsSet(sfConstraintComponent)) {
				IJavaInstance constraintAttributeValue = (IJavaInstance) ((RefObject) newValue).refValue(sfConstraintComponent);
				ComponentProxyAdapter componentProxyHost = (ComponentProxyAdapter) BeanProxyUtilities.getBeanProxyHost(constraintAttributeValue);
				componentProxyHost.applyVisibility(false, Boolean.TRUE);
			}
		}		
		super.applied(sf, newValue, position);				
	}

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#releaseBeanProxy()
	 */
	public void releaseBeanProxy() {
		super.releaseBeanProxy();
		
		if (fActivateFrameMethodProxy != null) {
			fActivateFrameMethodProxy.getProxyFactoryRegistry().releaseProxy(fActivateFrameMethodProxy);
			fActivateFrameMethodProxy = null;
		}
		
		if (fGetDesktopManagerMethodProxy != null) {
			fGetDesktopManagerMethodProxy.getProxyFactoryRegistry().releaseProxy(fGetDesktopManagerMethodProxy);
			fGetDesktopManagerMethodProxy = null;
		}		
	}

}
