package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.widgets.Display;

import com.ibm.etools.cde.IModelChangeController;
import com.ibm.etools.draw2d.geometry.Dimension;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.awt.IDimensionBeanProxy;

/**
 * Proxy adapter for javax.swing.JFrame.
 * Need to setDefaultCloseOperation to WindowConstants.DO_NOTHING_ON_CLOSE 
 * so that the live window can't be closed if the user tries to close it from
 * the task bar or from the window itself.
 */
public class JFrameProxyAdapter extends WindowProxyAdapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$

	/**
	 * Constructor for JFrameProxyAdapter.
	 * @param domain
	 */
	public JFrameProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
	}

	/**
	 * Create the bean proxy. We override because we want to set the default close operation 
	 * such that it can't be closed from the task bar.
	 */
	protected void primInstantiateBeanProxy() {
		super.primInstantiateBeanProxy();
		if (getBeanProxy() != null) {
			IBeanTypeProxy intType = getBeanProxy().getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy("int"); //$NON-NLS-1$
			try {
				IIntegerBeanProxy intBeanProxy = (IIntegerBeanProxy) intType.newInstance("javax.swing.JFrame.DO_NOTHING_ON_CLOSE"); //$NON-NLS-1$
				IMethodProxy setDefaultCloseOperationMethodProxy = getBeanProxy().getTypeProxy().getMethodProxy("setDefaultCloseOperation", "int"); //$NON-NLS-1$ //$NON-NLS-2$

				if (intBeanProxy != null && setDefaultCloseOperationMethodProxy != null)
					setDefaultCloseOperationMethodProxy.invokeCatchThrowableExceptions(getBeanProxy(), intBeanProxy);
			} catch (ThrowableProxy e) {
			} catch (InstantiationException e) {
			}
		}
	}
	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#applied(RefStructuralFeature, Object, int)
	 */
	protected void applied(RefStructuralFeature sf, Object newValue, int position) {
		// We never apply defaultCloseOperation, we override to always be do nothing on close.
		if (!("defaultCloseOperation".equals(sf.refName()))) //$NON-NLS-1$
			super.applied(sf, newValue, position);
	}

	/**
	 * @see com.ibm.etools.jbcf.visual.ComponentProxyAdapter#appliedSize(RefStructuralFeature, Object, int)
	 */
	protected void appliedSize(final RefStructuralFeature as, Object newValue, int position) {
		IDimensionBeanProxy dim = (IDimensionBeanProxy) BeanProxyUtilities.getBeanProxy((IJavaInstance) newValue);
		if (dim != null && (dim.getWidth() == -1 || dim.getHeight() == -1)) {
			// If the either width or height is -1, then we need to adjust to preferred size.
			// This will do an actual set of the setting in the EMF object. This is OK because
			// -1 is not a valid setting anyway, and doing an undo followed by redo will come back
			// with -1, and then at this point in time it would reconstruct the setting.
			Dimension size = new Dimension(dim.getWidth(), dim.getHeight());
			if (NullLayoutEditPolicy.adjustForPreferredSize(getBeanProxy(), size)) {
				String initString = DimensionJavaClassCellEditor.getJavaInitializationString(size.width, size.height);
				final IJavaInstance dimensionBean = BeanUtilities.createJavaObject("java.awt.Dimension", ((RefObject) target).getResourceSet(), initString); //$NON-NLS-1$
				Display.getDefault().asyncExec(new Runnable() {
					/**
					 * @see java.lang.Runnable#run()
					 */
					public void run() {
						// We may not be within the context of a change control, so we need to get a controller to handle the change.
						IModelChangeController controller =
							(IModelChangeController) getBeanProxyDomain().getEditDomain().getData(
								IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY);
						controller.run(new Runnable() {
							public void run() {
								// Set the constraints on the component bean.  This will change the size of the component
								// Because we will be called back with notify and apply the constraints rectangle to the live bean
								 ((RefObject) target).refSetValue(as, dimensionBean);
							}
						});
					}
				});
				return; // Let the notify back from the set here do the actual apply.
			}
		}

		super.appliedSize(as, newValue, position);
	}

	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#canceled(RefStructuralFeature, Object, int)
	 */
	protected void canceled(RefStructuralFeature sf, Object oldValue, int position) {
		// We never apply defaultCloseOperation, we override to always be do nothing on close.
		if (!("defaultCloseOperation".equals(sf.refName()))) //$NON-NLS-1$
			super.canceled(sf, oldValue, position);
	}

}