package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
 
import java.util.*;
import java.util.Iterator;
import java.util.List;

import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.IBeanProxyDomain;
import com.ibm.etools.jbcf.IBeanProxyHost;
import com.ibm.etools.proxy.IArrayBeanProxy;
import com.ibm.etools.proxy.ThrowableProxy;

/**
 * PRoxy adapter for JLayeredPane
 * @author richkulp
 */
public class JLayeredPaneProxyAdapter extends ContainerProxyAdapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$	

	protected boolean addingAll = false;
	/**
	 * Constructor for JLayeredPaneProxyAdapter.
	 * @param domain
	 */
	public JLayeredPaneProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
	}
	
	
	
	/**
	 * Return the current order of the children, as the mof objects.
	 */
	public List getCurrentOrder() {
		List children = Collections.EMPTY_LIST;
		if (getErrorStatus() != ERROR_SEVERE) {
			instantiateBeanProxy();	// Make sure instantiated, else this will fail.
			IArrayBeanProxy childrenArray = BeanAwtUtilities.invoke_getComponents(getBeanProxy());
			int clen = childrenArray.getLength();
			// The order of paint is reverse of GEF, awt paints from end to first, while gef paints
			// first to end. So we will reverse it here.
			children = new ArrayList(clen);
			for (int i=clen-1; 0<=i; i--) {
				try {
					children.add(childrenArray.get(i));
				} catch (ThrowableProxy e) {
				}
			}
			childrenArray.getProxyFactoryRegistry().getBeanProxyFactory().releaseProxy(childrenArray);
		}
		
		List constraintComponentsEMF = (List) getRefTarget().refValue(sfContainerComponents);
		RefObject[] childrenEMF = new RefObject[constraintComponentsEMF.size()];
		int notfound = childrenEMF.length;
		for (Iterator iter = constraintComponentsEMF.iterator(); iter.hasNext();) {
			RefObject element = (RefObject) iter.next();
			RefObject child = (RefObject) element.refValue(sfConstraintComponent);
			if (child != null) {
				IBeanProxyHost host = (IBeanProxyHost) child.getExistingAdapter(IBeanProxyHost.BEAN_PROXY_TYPE);
				if (host != null) {
					int childIndex = children.indexOf(host.getBeanProxy());
					if (childIndex != -1) {
						childrenEMF[childIndex] = child;
						continue;
					}
				}
				childrenEMF[--notfound] = child;	// Not found, so add at the end, in reverse order, just so we don't lose it.
			}
		}
		
		return Arrays.asList(childrenEMF);
	}

	/**
	 * @see com.ibm.etools.jbcf.visual.ContainerProxyAdapter#addComponentWithConstraint(RefObject, IJavaInstance, int)
	 */
	protected void addComponentWithConstraint(RefObject aConstraintComponent, IJavaInstance constraintAttributeValue, int position, boolean constraintSet)
		throws ReinstantiationNeeded {
		if (addingAll) 
			super.addComponentWithConstraint(aConstraintComponent, constraintAttributeValue, position, constraintSet);	// Go ahead do it, we are in rebuild.		
		else
			readdAll(false);
	}
	
	/*
	 * Need to readd all of them. Any add requires do from top.
	 */
	protected void readdAll(boolean testValidity) {
		addingAll = true;
		try {
			BeanAwtUtilities.invoke_removeAll_Components(getBeanProxy());
			super.appliedList(sfContainerComponents, (List) getRefTarget().refValue(sfContainerComponents), Notifier.NO_INDEX, testValidity);
		} finally {
			addingAll = false;
		}
	}

	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#applyAllSettings()
	 */
	protected void applyAllSettings() {
		// If we are applying all settings, then we are doing an add all.
		addingAll = true;
		try {
			super.applyAllSettings();
		} finally {
			addingAll = false;
		}
	}

	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#appliedList(RefStructuralFeature, List, int, boolean)
	 */
	protected void appliedList(RefStructuralFeature sf, List newValues, int position, boolean testValidity) {
		if (sf != sfContainerComponents || addingAll)
			super.appliedList(sf, newValues, position, testValidity);
		else if (!addingAll) {
			readdAll(testValidity);	// Not adding all, so we need to re add all, but pass in the validity test.
		}
	}

}
