package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;

/**
 * @author pwalker
 *
 * Graphical editpart for JMenuBar
 */
public class JMenuBarGraphicalEditPart extends ContainerGraphicalEditPart {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfMenus;

	/**
	 * Constructor for JMenuBarGraphicalEditPart.
	 * @param model
	 */
	public JMenuBarGraphicalEditPart(Object model) {
		super(model);
	}

	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		((ComponentGraphicalEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
		((ComponentGraphicalEditPart) ep).setTransparent(true); // So that it doesn't create an image, we subsume it here.
		return ep;
	}

	/**
	 * Use a JMenuBarLayoutPolicy which is a FlowLayout and only allows JMenus to be added
	 */
	protected void createLayoutEditPolicy() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new JMenuBarLayoutEditPolicy(this));
	}

	public List getModelChildren() {
		return (List) ((RefObject) getModel()).refValue(sfMenus);
	}

	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sfMenus)
			refreshChildren();
	}

	public void setModel(Object model) {
		super.setModel(model);
		ResourceSet rset = ((RefObject) model).getResourceSet();
		sfMenus = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JMENUBAR_MENUS);
	}

}
