package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.ThrowableProxy;

/**
 * @author pwalker
 *
 * Proxy adapter for JMenuBar. Adds and removes JMenus.
 */
public class JMenuBarProxyAdapter extends ComponentProxyAdapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfMenus;

	/**
	 * Constructor for JMenuBarProxyAdapter.
	 * @param domain
	 */
	public JMenuBarProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain.getEditDomain());
		sfMenus = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JMENUBAR_MENUS);
	}
	
	protected void applied(RefStructuralFeature as, Object newValue, int position) {

		if (as == sfMenus) {
			addComponent((RefObject) newValue, position);
		} else {
			super.applied(as, newValue, position);
		}
	}
	protected void canceled(RefStructuralFeature sf, Object oldValue, int position) {

		if (sf == sfMenus) {
			removeComponent((RefObject) oldValue);
		} else {
			super.canceled(sf, oldValue, position);
		}

	}
	private void addComponent(RefObject aComponent,int position) {
		IBeanProxyHost componentProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) aComponent);
		// Ensure the bean is instantiated
		IComponentProxyHost componentAdapter = (IComponentProxyHost) componentProxyHost;
		componentProxyHost.instantiateBeanProxy();
		// It is possible the component didn't instantiate.  We then can't apply it
		// and we should flag it as having a warning error
		if (componentProxyHost.getErrorStatus() == IBeanProxyHost.ERROR_SEVERE || getErrorStatus() == IBeanProxyHost.ERROR_SEVERE) {
			// Either we or the setting could not be instantiated, so don't add.
			// Need to get the ability to add an error for a structural feature against a single attribute setting
			return;
		}
		IBeanProxy componentBeanProxy = componentProxyHost.getBeanProxy();
		IBeanProxy beforeBeanProxy = null;	// The beanproxy to go before.

		if (position != -1)
			beforeBeanProxy = getBeanProxyAt(position+1);	// Need to do +1 because we (componentBeanProxy) are already at that position in the EMF list. So we want to go before next guy.

		try {
			// Invoke a method to add the menu to the menubar.
			if (beforeBeanProxy != null) {
				BeanAwtUtilities.invoke_add_Component_before(getBeanProxy(), componentBeanProxy, beforeBeanProxy);
			} else {
				BeanAwtUtilities.invoke_add_Component(getBeanProxy(), componentBeanProxy);
			}
		} catch (ThrowableProxy e) {
			processError(sfMenus, e, aComponent);
		}
		
		// Now that we've added it, set the parent component.
		componentAdapter.setParentComponentProxyHost((IComponentProxyHost) this);
		
		revalidateBeanProxy();
		
		clearError(sfMenus, aComponent);		

	}
	/*
	 * Return the first instantiated menu bean proxy at or after the given index.
	 * It is assumed that tests for the menubar being instantiated has already been done.
	 * Return null if not found.
	 */
	protected IBeanProxy getBeanProxyAt(int position) {
		List menus = (List) ((RefObject) getTarget()).refValue(sfMenus);
		for (int i=position; i<menus.size(); i++) {
			RefObject menu = (RefObject) menus.get(i);
			IBeanProxyHost componentProxyHost =	BeanProxyUtilities.getBeanProxyHost((IJavaInstance) menu);
			if (componentProxyHost.isBeanProxyInstantiated())
				return componentProxyHost.getBeanProxy();
		}
		
		return null;
	}
	protected void removeComponent(RefObject aComponent) {
		clearError(sfMenus, aComponent);		
		// The component to actually remove is within the ConstraintComponent too.
		IBeanProxyHost aComponentBeanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) aComponent);

		// It is possible the component or ourselves didn't instantiate.  We then can't cancel it.
		if (aComponentBeanProxyHost.getErrorStatus() != IBeanProxyHost.ERROR_SEVERE && getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
			BeanAwtUtilities.invoke_remove_Component(getBeanProxy(), aComponentBeanProxyHost.getBeanProxy());
			aComponentBeanProxyHost.releaseBeanProxy();
			// This is required because AWT will not invalidate and relayout the container
			revalidateBeanProxy();
		}
	}

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#reinstantiateChild(IBeanProxyHost)
	 */
	public void reinstantiateChild(IBeanProxyHost aChildProxyHost) {
		IJavaObjectInstance component = (IJavaObjectInstance) aChildProxyHost.getTarget();
		removeComponent(component);
		super.reinstantiateChild(aChildProxyHost);
	}

}
