package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;

/**
 * @author pwalker
 *
 * Tree editpart for JMenu
 */
public class JMenuTreeEditPart extends ComponentTreeEditPart implements IEMFListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfItems;
	protected EMFListenerAdapter containerAdapter;

	/**
	 * Constructor for JMenuTreeEditPart.
	 * @param model
	 */
	public JMenuTreeEditPart(Object model) {
		super(model);
	}

	protected void createEditPolicies() {
		// The TreeContainerEditPolicy is the CDE one
		super.createEditPolicies();
		installEditPolicy(
			EditPolicy.TREE_CONTAINER_ROLE,
			new com.ibm.etools.cde.TreeContainerEditPolicy(
				new JMenuContainerPolicy(EditDomain.getEditDomain(this))));
	}

	public void activate() {
		super.activate();
		((RefObject) getModel()).addAdapter(getModelAdapter());
	}

	public void deactivate() {
		super.deactivate();
		((RefObject) getModel()).removeAdapter(getModelAdapter());
	}
	protected Adapter getModelAdapter() {
		if (containerAdapter == null) {
			containerAdapter = new EMFListenerAdapter(this, false);
		}
		return containerAdapter;
	}
	
	public List getModelChildren() {
		return (List) ((RefObject) getModel()).refValue(sfItems);
	}
	public void notifyChanged(
		Notifier notifier,
		int eventType,
		RefObject sf,
		Object oldValue,
		Object newValue,
		int pos) {
		if (sf == sfItems)
			refreshChildren();
	}
	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		if (ep instanceof ComponentTreeEditPart)
			((ComponentTreeEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
		return ep;
	}
	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		ResourceSet rset = ((RefObject) model).getResourceSet();
		sfItems = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JMENU_ITEMS);
	}
}
