package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.BeanProxyUtilities;

public class JScrollPaneGraphicalEditPart extends ContainerGraphicalEditPart {

public JScrollPaneGraphicalEditPart (Object model) {
	super(model);
}

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected EStructuralFeature sf_scrollpaneViewportView;

public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
	if (sf == sf_scrollpaneViewportView)
		refreshChildren();
}
/**
 * The layoutEditPolicy deals with the fact that the child is a viewportView
 */
protected void createLayoutEditPolicy() {
	installEditPolicy(EditPolicy.LAYOUT_ROLE,new JScrollPaneLayoutEditPolicy(EditDomain.getEditDomain(this)));
}
public List getModelChildren(){
	ArrayList result = new ArrayList(1);
	Object viewportView = ((RefObject)getModel()).refValue(sf_scrollpaneViewportView);
	if ( viewportView != null ) {
		result.add(viewportView);
		return result;
	} else {
		return Collections.EMPTY_LIST;
	}
}
protected EditPart createChild(Object model) {
	// We need to set the relative parent of the viewportView so it creates its location for the
	// the graph view editPart relative to us as its parent instead of some internal parent container
	IComponentProxyHost jScrollPaneProxyAdapter = (IComponentProxyHost)BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());
	IComponentProxyHost viewportViewProxyAdapter = (IComponentProxyHost)BeanProxyUtilities.getBeanProxyHost((IJavaInstance)model);
	if (viewportViewProxyAdapter != null){
		viewportViewProxyAdapter.setParentComponentProxyHost(jScrollPaneProxyAdapter);
	}	
	// Return the edit part
	EditDomain dom = EditDomain.getEditDomain(this);
	EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
	((ComponentGraphicalEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
	((ComponentGraphicalEditPart) ep).setTransparent(true);	// So that it doesn't create an image, we subsume it here.
	return ep;
}

	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		
		sf_scrollpaneViewportView = JavaInstantiation.getSFeature((IJavaObjectInstance) model, JBCFConstants.SF_JSCROLLPANE_VIEWPORTVIEW);
	}

}