package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.gef.EditPart;
import java.util.*;
/**
 * This inherits from Container because Container knows how to create the correct
 * editPart for the child, although unlike Container which gives us components as children
 * we give up the viewportView
 * In future we may want to give up things like the topRight, etc... for all 9 possible children
 */
public class JScrollPaneTreeEditPart extends ContainerTreeEditPart {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected EStructuralFeature sf_scrollpaneViewportView;

public JScrollPaneTreeEditPart (Object model) {
	super(model);
}
protected void createEditPolicies() {
	// The TreeContainerEditPolicy is the CDE one
	// We don't care about being a Container with components
	// We are just interested in showing the viewPortView as a child
	super.createEditPolicies();
	installEditPolicy(EditPolicy.TREE_CONTAINER_ROLE, new com.ibm.etools.cde.TreeContainerEditPolicy(new JScrollPaneContainerPolicy(EditDomain.getEditDomain(this))));
}
public List getModelChildren(){
	ArrayList result = new ArrayList(1);
	Object viewportView = ((RefObject)getModel()).refValue(sf_scrollpaneViewportView);
	if ( viewportView != null ) {
		result.add(viewportView);
		return result;
	} else {
		return Collections.EMPTY_LIST;
	}
}
public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
	if (sf == sf_scrollpaneViewportView)
		refreshChildren();
}
protected EditPart createChild(Object model) {
	EditDomain dom = EditDomain.getEditDomain(this);
	EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
	((ComponentTreeEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
	return ep;
}

	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		
		sf_scrollpaneViewportView = JavaInstantiation.getSFeature((IJavaObjectInstance) model, JBCFConstants.SF_JSCROLLPANE_VIEWPORTVIEW);
	}
}