package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.IBeanProxyDomain;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;

/**
 * @author richkulp
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class JSliderProxyAdapter extends ComponentProxyAdapter {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected RefStructuralFeature sfMajorTicks, sfLabelTable;
	/**
	 * Constructor for JSliderProxyAdapter.
	 * @param domain
	 */
	public JSliderProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
		
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain.getEditDomain());
		sfMajorTicks = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSLIDER_MAJORTICKS);		
		sfLabelTable = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSLIDER_LABELTABLE);		
	}
	

	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#applied(RefStructuralFeature, Object, int)
	 */
	protected void applied(RefStructuralFeature sf, Object newValue, int position) {
		// Because of the way the labels work on JSlider, once you set the majorTicks it creates
		// the labels, but if you then change the majorTicks again, the labels are not changed again.
		// So we need to test to see if it is major ticks, and if it is, then request reinstantiation
		// if not in instantiation.
		if (sf == sfMajorTicks && !inInstantiation() && !getRefTarget().refIsSet(sfLabelTable))
			throw new ReinstantiationNeeded();
		super.applied(sf, newValue, position);
	}

	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#canceled(RefStructuralFeature, Object, int)
	 */
	protected void canceled(RefStructuralFeature sf, Object oldValue, int position) {
		// Because of the way the labels work on JSlider, once you set the majorTicks it creates
		// the labels, but if you then change the majorTicks again, the labels are not changed again.
		// So we need to test to see if it is major ticks, and if it is, then request reinstantiation
		// if not in instantiation.
		if (sf == sfMajorTicks && !inInstantiation() && !getRefTarget().refIsSet(sfLabelTable))
			throw new ReinstantiationNeeded();		
		super.canceled(sf, oldValue, position);
	}

}
