package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;

import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyHost;

/**
 * Label decorator for JSplitPane children in the Outline Viewer. 
 * Will add text to the component text based on the split pane's orientation.
 */
public class JSplitPaneChildTreeLabelDecorator implements ILabelDecorator {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfLeftComponent,
		sfRightComponent,
		sfTopComponent,
		sfBottomComponent,
		sf_constraintComponent,
		sf_constraintConstraint;

	public void initializeSFs(RefObject component) {
		ResourceSet rset = component.getResourceSet();
		if (sfLeftComponent == null)
			sfLeftComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_LEFTCOMPONENT);
		if (sfRightComponent == null)
			sfRightComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_RIGHTCOMPONENT);
		if (sfBottomComponent == null)
			sfBottomComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_BOTTOMCOMPONENT);
		if (sfTopComponent == null)
			sfTopComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_TOPCOMPONENT);
		if (sf_constraintComponent == null)
			sf_constraintComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_COMPONENT);
		if (sf_constraintConstraint == null)
			sf_constraintConstraint = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_CONSTRAINT);
	}

	public Image decorateImage(Image image, Object anObject) {
		return image;
	}
	public String decorateText(String text, Object anObject) {
		if (anObject == null)
			return ""; //$NON-NLS-1$
		if (!(anObject instanceof IJavaObjectInstance))
			return anObject.toString();
		initializeSFs((RefObject) anObject);
		IJavaObjectInstance component = (IJavaObjectInstance) anObject;
		RefStructuralFeature sf = component.refContainerSF();
		if (sf == sfLeftComponent) {
			text = VisualMessages.getString("Decorator.JSplitPane.Left") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
		} else if (sf == sfRightComponent) {
			text = VisualMessages.getString("Decorator.JSplitPane.Right") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
		} else if (sf == sfTopComponent) {
			text = VisualMessages.getString("Decorator.JSplitPane.Top") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
		} else if (sf == sfBottomComponent) {
			text = VisualMessages.getString("Decorator.JSplitPane.Bottom") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
		} else if (sf == sf_constraintComponent) {
			RefObject constraintComponent = (RefObject) component.refContainer();
			// See whether the component is in severe error.  If so then don't include it here
			if (BeanProxyUtilities.getBeanProxyHost((IJavaInstance) component).getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
				IJavaObjectInstance constraintString = (IJavaObjectInstance) constraintComponent.refValue(sf_constraintConstraint);
				if (constraintString != null) {
					// We know the constraints value should be a bean so we can use its toString to get the string value
					String constraint = BeanProxyUtilities.getBeanProxy(constraintString).toBeanString();
					if (constraint != null) {
						if (constraint.equals("left")) //$NON-NLS-1$
							text = VisualMessages.getString("Decorator.JSplitPane.Left") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
						else if (constraint.equals("right")) //$NON-NLS-1$
							text = VisualMessages.getString("Decorator.JSplitPane.Right") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
						else if (constraint.equals("top")) //$NON-NLS-1$
							text = VisualMessages.getString("Decorator.JSplitPane.Top") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
						else if (constraint.equals("bottom")) //$NON-NLS-1$
							text = VisualMessages.getString("Decorator.JSplitPane.Bottom") + " - " + text; //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			}
		}
		return text;
	}
	public void addListener(ILabelProviderListener p1) {
	}
	public void dispose() {
	}
	public boolean isLabelProperty(Object p1, String p2) {
		return false;
	}
	public void removeListener(ILabelProviderListener p1) {
	}
}