package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.cde.properties.PropertySourceAdapter;
import com.ibm.etools.draw2d.IFigure;
import com.ibm.etools.draw2d.XYLayout;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyHost;

/**
 * Graphical edit part for handling JSplitPane in the Graph viewer
 */
public class JSplitPaneGraphicalEditPart extends ComponentGraphicalEditPart implements IEMFListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter containerAdapter;

	protected EStructuralFeature sfLeftComponent,
		sfRightComponent,
		sfTopComponent,
		sfBottomComponent,
		sf_constraintComponent,
		sf_containerComponents;

	public JSplitPaneGraphicalEditPart(Object model) {
		super(model);
	}
	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		((ComponentGraphicalEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
		((ComponentGraphicalEditPart) ep).setTransparent(true); // So that it doesn't create an image, we subsume it here.
		return ep;
	}
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.CONTAINER_ROLE, new DefaultContainerEditPolicy());
		installEditPolicy(VisualComponentsLayoutPolicy.LAYOUT_POLICY, new VisualComponentsLayoutPolicy());
		// This is a special policy that just handles the size/position of visual components wrt/the figures. It does not handle changing size/position.
		createLayoutEditPolicy();
	}
	protected void createLayoutEditPolicy() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new JSplitPaneLayoutEditPolicy(EditDomain.getEditDomain(this)));
	}
	protected IFigure createFigure() {
		IFigure fig = super.createFigure();
		fig.setLayoutManager(new XYLayout());
		return fig;
	}
	public void activate() {
		super.activate();
		((RefObject) getModel()).addAdapter(getModelAdapter());
	}

	public void deactivate() {
		super.deactivate();
		((RefObject) getModel()).removeAdapter(getModelAdapter());
	}

	protected Adapter getModelAdapter() {
		if (containerAdapter == null) {
			containerAdapter = new EMFListenerAdapter(this, false);
		}
		return containerAdapter;
	}
	/**
	 * JSplitPane's children are really set as the left or top component and the right or bottom component. 
	 * If the JSplitPane's orientation is HORIZONTAL_SPLIT, the components will be set left and right.
	 * If the JSplitPane's orientation is VERTICAL_SPLIT, the components will be set top and bottom.
	 * We'll check for both since we don't know the orientation... and even if we did, Sun lets you set
	 * the components regardless of what the orientation is set to. In other words, you can set the left
	 * and right components even though the orientation is vertical... and the opposite is true as well. 
	 */
	public List getModelChildren() {
		ArrayList result = new ArrayList();
		Object leftComponent = ((RefObject) getModel()).refValue(sfLeftComponent);
		Object topComponent = ((RefObject) getModel()).refValue(sfTopComponent);
		Object rightComponent = ((RefObject) getModel()).refValue(sfRightComponent);
		Object bottomComponent = ((RefObject) getModel()).refValue(sfBottomComponent);
		if (leftComponent != null) {
			result.add(leftComponent);
		} else if (topComponent != null) {
			result.add(topComponent);
		}
		if (rightComponent != null) {
			result.add(rightComponent);
		} else if (bottomComponent != null) {
			result.add(bottomComponent);
		}
		List componentList = getConstraintComponentsModelChildren();
		if (!componentList.isEmpty()) {
			for (int i = 0; i < componentList.size(); i++) {
				result.add(componentList.get(i));
			}
		}
		if (result.isEmpty())
			return Collections.EMPTY_LIST;

		return result;
	}
	/**
	 * Sun let's you set the children in JSplitPane with just an add(aComponent, "left"), etc.,
	 * so we have to handle the cases whereas the components of the JSplitPane are constraint components.
	 */
	protected List getConstraintComponentsModelChildren() {
		// Model children is the components feature.
		// However, this returns the constraint components, but we want to return instead
		// the components themselves. They are the "model" that gets sent to the createChild and
		// component edit part.
		List constraintChildren = (List) ((RefObject) getModel()).refValue(sf_containerComponents);
		ArrayList children = new ArrayList(constraintChildren.size());
		Iterator itr = constraintChildren.iterator();
		while (itr.hasNext()) {
			RefObject con = (RefObject) itr.next();
			IJavaInstance component = (IJavaInstance) con.refValue(sf_constraintComponent);
			// See whether the component is in severe error.  If so then exlude if from the list of children
			if (BeanProxyUtilities.getBeanProxyHost(component).getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
				children.add(con.refValue(sf_constraintComponent)); // Get the component out of the constrain
			}
		}
		return children;
	}
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		refreshChildren();
	}

	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		ResourceSet rset = ((RefObject) model).getResourceSet();
		sfLeftComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_LEFTCOMPONENT);
		sfRightComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_RIGHTCOMPONENT);
		sfBottomComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_BOTTOMCOMPONENT);
		sfTopComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_TOPCOMPONENT);
		sf_constraintComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_COMPONENT);
		sf_containerComponents = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_COMPONENTS);
	}
}