package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.gef.editpolicies.LayoutEditPolicy;
import com.ibm.etools.gef.editpolicies.*;
import com.ibm.etools.gef.*;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.gef.requests.*;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import java.util.*;


public class JSplitPaneLayoutEditPolicy extends LayoutEditPolicy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected JSplitPaneContainerPolicy containerPolicy;

public JSplitPaneLayoutEditPolicy(EditDomain anEditDomain) {
	containerPolicy = new JSplitPaneContainerPolicy(anEditDomain);
}

public void activate() {
	super.activate();
	containerPolicy.setContainer(getHost().getModel());
}
public void deactivate() {
	super.deactivate();
	containerPolicy.setContainer(null);
}
public EditPolicy createChildEditPolicy(EditPart aChild){
	return new NonResizableEditPolicy();
}
/**
 * Add of children requested.
 * 
 * See the comments in the getCreateCommand(Request request) method.
 */
protected Command getAddCommand(Request request) {
	Command addContributionCmd = null;
	if (RequestConstants.REQ_ADD.equals(request.getType())) {
		if (containerPolicy.isRightComponentOccupiedAndOnlyComponent())
			addContributionCmd = containerPolicy.getAddCommand(containerPolicy.getChildren((GroupRequest) request), getHost().getChildren().get(0));
		else
			addContributionCmd = containerPolicy.getAddCommand(containerPolicy.getChildren((GroupRequest) request), null);
	}
	if (addContributionCmd == null)
		return UnexecutableCommand.INSTANCE;	// It can't be added.
	return addContributionCmd;
}

/**
 * A new child is being created in this container.
 * 
 * We need to handle the situation in which the JSplitPane has a right/bottom component
 * occupied and the left/top component is unoccupied.
 * Since the SplitPaneContainerPolicy thinks that a null for the positionBeforeChild object
 * means to put it after or at the end, in the graph viewer it will put the new component in the
 * right/bottom and move the original right/bottom to the left/top position. This is rather
 * annoying and since we know we are in the graph viewer when we get the request, we need
 * to check if this is the case and if so, send the request to the container policy with
 * the right/bottom component as the positionBeforeChild parameter to force it to leave the
 * right/bottom component as is and put the new component in the left/top position.
 */
protected Command getCreateCommand(CreateRequest request) {
	Command createContributionCmd = null;
	if (RequestConstants.REQ_CREATE.equals(request.getType())) {
		if (containerPolicy.isRightComponentOccupiedAndOnlyComponent())
			createContributionCmd = containerPolicy.getCreateCommand(((CreateRequest) request).getNewObject(), getHost().getChildren().get(0));
		else 
			createContributionCmd = containerPolicy.getCreateCommand(((CreateRequest) request).getNewObject(), null);
	}
	if (createContributionCmd == null)
		return UnexecutableCommand.INSTANCE;	// It can't be created.
	return createContributionCmd;
}

/**
 * getDeleteDependantCommand method comment.
 */
protected Command getDeleteDependantCommand(Request request) {
	Command deleteContributionCmd = containerPolicy.getCommand(request);
	if (deleteContributionCmd == null)
		return UnexecutableCommand.INSTANCE;	// It can't be created.
	return deleteContributionCmd;
}
/**
 * getMoveChildCommand method comment.
 * We don't perform move/resize since we don't know how.
 */
protected Command getMoveChildrenCommand(Request request) {
	return null;
}
/**
 * getOrphanChildCommand. To orphan, we delete only the child. We don't
 * delete the subpart since the part is not going away, just somewhere's
 * else. So the subpart stays.
 */
protected Command getOrphanChildrenCommand(Request request) {
	Command orphanContributionCmd = containerPolicy.getCommand(request);
	if (orphanContributionCmd == null)
		return UnexecutableCommand.INSTANCE;	// It can't be created.
	return orphanContributionCmd;
}
}