package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.io.InputStream;

import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.visual.common.Common;
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.ICallback;

/**
 * This is the IDE class that interfaces to the JSplitPaneManager on the remote vm
 * @author richkulp
 */
public class JSplitPaneManager implements ICallback {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IBeanProxy splitpaneManager;
	protected JSplitPaneProxyAdapter proxyAdapter;

	/**
	 * Set the bean proxy of the splitpane that we are manageing 
	 */
	public void setJSplitPaneBeanProxy(JSplitPaneProxyAdapter proxyAdapter, IBeanProxy splitpaneProxy) {
		this.proxyAdapter = proxyAdapter;
		try {
			if (splitpaneProxy != null && splitpaneManager == null) {
				IBeanTypeProxy splitpaneManagerType = splitpaneProxy.getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.JSplitPaneManager"); //$NON-NLS-1$
				splitpaneManager = splitpaneManagerType.newInstance();
				splitpaneManager.getProxyFactoryRegistry().getCallbackRegistry().registerCallback(splitpaneManager, this);
			}

			if (splitpaneManager != null)
				BeanAwtUtilities.invoke_set_JSplitPaneBean_Manager(splitpaneManager, splitpaneProxy);
		} catch (ThrowableProxy e) {
			JBCFPlugin.log(e);
		}
	}

	public void dispose() {
		if (splitpaneManager != null && splitpaneManager.isValid()) {
			BeanAwtUtilities.invoke_set_JSplitPaneBean_Manager(splitpaneManager, null);
			splitpaneManager.getProxyFactoryRegistry().getCallbackRegistry().deregisterCallback(splitpaneManager);
			splitpaneManager.getProxyFactoryRegistry().releaseProxy(splitpaneManager);
		}
		splitpaneManager = null;
		proxyAdapter = null;		
	}
	
	/**
	 * Set the divider location.
	 */
	public void setDividerLocation(IBeanProxy dividerLocation) {
		BeanAwtUtilities.invoke_set_JSplitPane_DividerLocation_Manager(splitpaneManager, dividerLocation);		
	}
	
	/**
	 * Reset to preferred sizes
	 */
	public void resetToPreferredSizes() {
		BeanAwtUtilities.invoke_reset_JSplitPane_PreferredSizes_Manager(splitpaneManager);			
	}
	
	/**
	 * @see com.ibm.etools.proxy.ICallback#calledBack(int, IBeanProxy)
	 */
	public Object calledBack(int msgID, IBeanProxy parm) {
		if (msgID == Common.JSP_INVALIDATE) {
			proxyAdapter.revalidateBeanProxy();	// Cause it to get a new image.
		}
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.ICallback#calledBack(int, Object[])
	 */
	public Object calledBack(int msgID, Object[] parms) {
		throw new RuntimeException("A jsplitpane manager has been called back incorrectly"); //$NON-NLS-1$	
	}

	/**
	 * @see com.ibm.etools.proxy.ICallback#calledBackStream(int, InputStream)
	 */
	public void calledBackStream(int msgID, InputStream is) {
		throw new RuntimeException("A jsplitpane manager has been called back incorrectly"); //$NON-NLS-1$		
	}

}
