package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.BeanPropertyDescriptorAdapter;
import com.ibm.etools.propertysheet.ICommandPropertyDescriptor;
import com.ibm.etools.propertysheet.RestoreDefaultPropertyValueCommand;

/**
 * Property descriptor for JSplitPane.orientation.
 * 
 * This is needed for reseting the components in the split pane to the correct positions. 
 * For example, if the orientation is horizontal, the positions are left and right. If the
 * orientation is changed to vertical, the components need to be reset to top and bottom.
 * This is so the code is gen'd correctly (even though Swing allows for this discrepancy)
 * and so the label provider will reflect the positions correctly.
 * 
 */
public class JSplitPaneOrientationPropertyDescriptor
	extends BeanPropertyDescriptorAdapter
	implements ICommandPropertyDescriptor {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	int VERTICAL_SPLIT = 0, HORIZONTAL_SPLIT = 1;

	public Command setValue(IPropertySource source, Object setValue) {
		CommandBuilder cb = new CommandBuilder(null);
		IJavaObjectInstance splitpane = (IJavaObjectInstance) source.getEditableValue();
		EStructuralFeature sfLeftComponent =
			JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_LEFTCOMPONENT),
			sfRightComponent = JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_RIGHTCOMPONENT),
			sfTopComponent = JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_TOPCOMPONENT),
			sfBottomComponent = JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_BOTTOMCOMPONENT);
		IJavaInstance value = (IJavaInstance) setValue;
		String initString = value.getInitializationString();
		int orientation = HORIZONTAL_SPLIT; // default is HORIZONTAL_SPLIT
		if (initString.equals("0") //$NON-NLS-1$
			|| initString.equals("VERTICAL_SPLIT") //$NON-NLS-1$
			|| initString.equals("javax.swing.JSplitPane.VERTICAL_SPLIT")) //$NON-NLS-1$
			orientation = VERTICAL_SPLIT;
		else if (
			initString.equals("1") //$NON-NLS-1$
				|| initString.equals("HORIZONTAL_SPLIT") //$NON-NLS-1$
				|| initString.equals("javax.swing.JSplitPane.HORIZONTAL_SPLIT")) //$NON-NLS-1$
			orientation = HORIZONTAL_SPLIT;
		// Handle the case where the orientation is now vertical
		if (orientation == VERTICAL_SPLIT) {
			if (splitpane.refIsSet(sfLeftComponent)) {
				RefObject leftComp = (RefObject) ((RefObject) splitpane).refValue(sfLeftComponent);
				cb.cancelAttributeSetting(splitpane, sfLeftComponent);
				cb.applyAttributeSetting((RefObject) splitpane, sfTopComponent, leftComp);
			}
			if (splitpane.refIsSet(sfRightComponent)) {
				RefObject rightComp = (RefObject) ((RefObject) splitpane).refValue(sfRightComponent);
				cb.cancelAttributeSetting(splitpane, sfRightComponent);
				cb.applyAttributeSetting((RefObject) splitpane, sfBottomComponent, rightComp);
			}
			// Handle the case where the orientation is now horizontal
		} else if (orientation == HORIZONTAL_SPLIT) {
			if (splitpane.refIsSet(sfTopComponent)) {
				RefObject topComp = (RefObject) ((RefObject) splitpane).refValue(sfTopComponent);
				cb.cancelAttributeSetting(splitpane, sfTopComponent);
				cb.applyAttributeSetting((RefObject) splitpane, sfLeftComponent, topComp);
			}
			if (splitpane.refIsSet(sfBottomComponent)) {
				RefObject bottomComp = (RefObject) ((RefObject) splitpane).refValue(sfBottomComponent);
				cb.cancelAttributeSetting(splitpane, sfBottomComponent);
				cb.applyAttributeSetting((RefObject) splitpane, sfRightComponent, bottomComp);
			}
		}
		// apply the orientation value to the splitpane			
		cb.applyAttributeSetting(splitpane, (RefStructuralFeature) getTarget(), setValue);
		return cb.getCommand();
	}

	/*
	 * Reset the 'orientation' property. If any of the component are set with top or bottom,
	 * cancel the settings and set them with respective 'left' and 'right' settings.
	 */
	public Command resetValue(IPropertySource source) {
		CommandBuilder cb = new CommandBuilder(null);
		IJavaObjectInstance splitpane = (IJavaObjectInstance) source.getEditableValue();
		EStructuralFeature sfLeftComponent =
			JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_LEFTCOMPONENT),
			sfRightComponent = JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_RIGHTCOMPONENT),
			sfTopComponent = JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_TOPCOMPONENT),
			sfBottomComponent = JavaInstantiation.getSFeature(splitpane, JBCFConstants.SF_JSPLITPANE_BOTTOMCOMPONENT);
		if (splitpane.refIsSet(sfTopComponent)) {
			RefObject topComp = (RefObject) ((RefObject) splitpane).refValue(sfTopComponent);
			cb.cancelAttributeSetting(splitpane, sfTopComponent);
			cb.applyAttributeSetting((RefObject) splitpane, sfLeftComponent, topComp);
		}
		if (splitpane.refIsSet(sfBottomComponent)) {
			RefObject bottomComp = (RefObject) ((RefObject) splitpane).refValue(sfBottomComponent);
			cb.cancelAttributeSetting(splitpane, sfBottomComponent);
			cb.applyAttributeSetting((RefObject) splitpane, sfRightComponent, bottomComp);
		}
		RestoreDefaultPropertyValueCommand cmd = new RestoreDefaultPropertyValueCommand();
		cmd.setTarget(source);
		cmd.setPropertyId(getTarget());
		cb.append(cmd);
		return cb.getCommand();
	}

}