package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.beaninfo.PropertyDecorator;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.jbcf.BeanFeatureDecorator;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.IBeanTypeProxy;

public class JSplitPaneProxyAdapter extends ContainerProxyAdapter {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfLeftComponent, sfRightComponent, sfTopComponent, sfBottomComponent, sfDividerLocation;
	protected JSplitPaneManager splitPaneManager;
	protected boolean dividerSet;

	public JSplitPaneProxyAdapter(IBeanProxyDomain domain) {
		super(domain);

		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain.getEditDomain());
		sfLeftComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_LEFTCOMPONENT);
		sfRightComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_RIGHTCOMPONENT);
		sfBottomComponent =
			JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_BOTTOMCOMPONENT);
		sfTopComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_TOPCOMPONENT);
		sfDividerLocation = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_DIVIDERLOCATION);	
		
		// Real kludge, we need to force the default divider location to -1. If we don't then when we reset
		// it, it will use the bogus 60 that comes up after construction. This guy gets there because the
		// default buttons that splitpane inserts under the covers forces the divider to have an initial location
		// immediately after construction. And that is the value we see. Not the true default of -1.
		getOriginalSettingsTable().put(sfDividerLocation, domain.getProxyFactoryRegistry().getBeanProxyFactory().createBeanProxyWith(-1)		);
			
	}
	
	protected void applied(RefStructuralFeature as, Object newValue, int position) {
		super.applied(as, newValue, position);

		if ((as == sfLeftComponent
			|| as == sfRightComponent
			|| as == sfTopComponent
			|| as == sfBottomComponent) &&
			newValue != null) {
			IBeanProxyHost componentProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) newValue);
			IComponentProxyHost componentAdapter = (IComponentProxyHost) componentProxyHost;
			// Now that we've added it, set the parent component.
			componentAdapter.setParentComponentProxyHost((IComponentProxyHost) this);
		}
	}
	
	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#setupBeanProxy(IBeanProxy)
	 */
	protected void setupBeanProxy(IBeanProxy beanProxy) {
		super.setupBeanProxy(beanProxy);
		
		if (beanProxy != null) {
			if (splitPaneManager == null)
				splitPaneManager = new JSplitPaneManager();
			
			splitPaneManager.setJSplitPaneBeanProxy(this, beanProxy);
		}
		
	}

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#releaseBeanProxy()
	 */
	public void releaseBeanProxy() {
		super.releaseBeanProxy();
		
		if (splitPaneManager != null) {
			splitPaneManager.dispose();
			splitPaneManager = null;
		}			
	}

	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#applyBeanFeature(RefStructuralFeature, PropertyDecorator, BeanFeatureDecorator, IBeanProxy)
	 */
	protected void primApplyBeanFeature(
		RefStructuralFeature sf,
		PropertyDecorator propDecor,
		BeanFeatureDecorator featureDecor,
		IBeanProxy settingBeanProxy) throws Exception {
		// If it is divider, then set through the manager. 
		if (sf != sfDividerLocation || splitPaneManager == null) 
			super.primApplyBeanFeature(sf, propDecor, featureDecor, settingBeanProxy);
		else {
			splitPaneManager.setDividerLocation(settingBeanProxy);
			dividerSet = ((IIntegerBeanProxy) settingBeanProxy).intValue() != -1;
		}
	}

	/**
	 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#childInvalidated(IComponentProxyHost)
	 */
	public void childInvalidated(IComponentProxyHost childProxy) {
		if (!dividerSet && splitPaneManager != null)
			splitPaneManager.resetToPreferredSizes();
		super.childInvalidated(childProxy);
	}

}