package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.Arrays;
import java.util.List;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.properties.PropertySourceAdapter;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyHost;
import com.ibm.etools.propertysheet.*;
import com.ibm.etools.proxy.IBeanProxy;

/**
 * Property source adapter for the special features for components added to a JTabbedPane
 */
public class JTabComponentPropertySourceAdapter extends PropertySourceAdapter {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IPropertySource componentPS;
	protected List myDescriptors;

	/*
	 * Need to merge in the properties of the component.
	 * We will have to wrapper any ISourced or ICommand descriptors because
	 * those will have the wrong source passed into them. We need to intercept
	 * and have the correct source (the component) passed into them.
	 *
	 * 
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		RefObject component =
			(RefObject) getRefTarget().refValue(JavaInstantiation.getSFeature(getRefTarget().getResourceSet(), JBCFConstants.SF_JTABCOMPONENT_COMPONENT));
		componentPS = (IPropertySource) component.getAdapter(IPropertySource.class);
		IPropertyDescriptor[] theirs = componentPS.getPropertyDescriptors();

		IPropertyDescriptor[] wrappedTheirs = new IPropertyDescriptor[theirs.length];
		int wi = 0;
		for (int i = 0; i < theirs.length; i++) {
			IPropertyDescriptor pd = theirs[i];
			if (pd.getId() instanceof RefStructuralFeature) {
				// exclude bounds/size/location
				String fn = ((RefStructuralFeature) pd.getId()).refName();
				if ("bounds".equals(fn) || "size".equals(fn) || "location".equals(fn)) //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					continue;
			}
			if (pd instanceof ISourcedPropertyDescriptor
				|| pd instanceof ICommandPropertyDescriptor)
				wrappedTheirs[wi++] = new WrapperedPropertyDescriptor(componentPS, pd);
			else
				wrappedTheirs[wi++] = pd;
		}

		// Finally build the complete list.
		IPropertyDescriptor[] mine = super.getPropertyDescriptors();
		//Save the ID's for later compares.
		myDescriptors = new ArrayList(mine.length);
		for (int i = 0; i < mine.length; i++) {
			myDescriptors.add(mine[i].getId());
		}
		IPropertyDescriptor[] finalList = new IPropertyDescriptor[mine.length + wi];
		System.arraycopy(wrappedTheirs, 0, finalList, 0, wi);
		System.arraycopy(mine, 0, finalList, wi, mine.length);
		return finalList;
	}

	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public Object getPropertyValue(Object feature) {
		if (myDescriptors.contains(feature))
			return super.getPropertyValue(feature);

		return componentPS.getPropertyValue(feature);
	}

	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public boolean isPropertySet(Object feature) {
		if (myDescriptors.contains(feature))
			return super.isPropertySet(feature);

		return componentPS.isPropertySet(feature);
	}

	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public void resetPropertyValue(Object feature) {
		if (myDescriptors.contains(feature))
			super.resetPropertyValue(feature);
		else
			componentPS.resetPropertyValue(feature);
	}

	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public void setPropertyValue(Object feature, Object val) {
		if (myDescriptors.contains(feature))
			super.setPropertyValue(feature, val);
		else
			componentPS.setPropertyValue(feature, val);
	}
}