package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;

/**
 * Tree edit part for handling JTabbedPane in the Outline viewer
 * 
 * Since we can't really select the individual tabs on the JTabbedPane
 * from the Graphviewer, we depend on the tree editpart for selecting the
 * pages or components on those pages. 
 * 
 * When a page (tree editpart.. child of JTabbePane) is selected,
 * the corresponding JTabbedPaneGraphicalEditPart is selected and it 
 * notifies the JTabbedPanedProxyAdapter to bring the page to the front.
 * 
 * When this occurs, the previously graph viewed page figures are hidden and the 
 * currently selected page is made visible. If we didn't hide the editparts 
 * from the other pages, the figure rectangle outlines will "bleed" through 
 * and show on the top page.
 */
public class JTabbedPaneTreeEditPart extends ComponentTreeEditPart implements IEMFListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter containerAdapter;
	protected EStructuralFeature sfTabs, sfComponent;

	public JTabbedPaneTreeEditPart(Object model) {
		super(model);
	}
	public void activate() {
		((RefObject) getModel()).addAdapter(getModelAdapter());
		super.activate();
	}
	public void deactivate() {
		((RefObject) getModel()).removeAdapter(getModelAdapter());
		super.deactivate();
	}
	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		((ComponentTreeEditPart) ep).setPropertySource((IPropertySource) ((RefObject) model).refContainer().getAdapter(IPropertySource.class));	// This is the property source of the actual model which is part of the constraintComponent.
// keep the following for the future so we can show the tab title in the beans viewer.		
//		((ComponentTreeEditPart) ep).setLabelDecorator(new JTabbedPaneChildTreeLabelDecorator());
		return ep;
	}
	protected void createEditPolicies() {
		// The TreeContainerEditPolicy is the CDE one
		// We don't care about being a Container with components
		// We are just interested in showing the tabs (or pages) as children
		super.createEditPolicies();
		installEditPolicy(
			EditPolicy.TREE_CONTAINER_ROLE,
			new com.ibm.etools.cde.TreeContainerEditPolicy(new JTabbedPaneContainerPolicy(EditDomain.getEditDomain(this))));
	}
	protected Adapter getModelAdapter() {
		if (containerAdapter == null) {
			containerAdapter = new EMFListenerAdapter(this, false);
		}
		return containerAdapter;
	}
	protected List getModelChildren() {
		List jtabComponents = (List) ((RefObject) getModel()).refValue(sfTabs);
		ArrayList children = new ArrayList(jtabComponents.size());
		Iterator itr = jtabComponents.iterator();
		while (itr.hasNext()) {
			RefObject component = (RefObject) itr.next();
			children.add(component.refValue(sfComponent));
		}
		return children;
	}
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sfTabs)
			refreshChildren();
	}
	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		ResourceSet rset = ((RefObject) model).getResourceSet();
		sfTabs = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABBEDPANE_TABS);
		sfComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABCOMPONENT_COMPONENT);
	}
}