package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
/**
 * JTable has a relationship "columns" which holds its children which are javax.swing.table.TableColumn instances
 */
public class JTableTreeEditPart extends ComponentTreeEditPart implements IEMFListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter containerAdapter;
	protected EStructuralFeature sfColumns;
	public JTableTreeEditPart(Object model) {
		super(model);
	}
	public void activate() {
		super.activate();
		((RefObject) getModel()).addAdapter(getModelAdapter());
	}

	public void deactivate() {
		super.deactivate();
		((RefObject) getModel()).removeAdapter(getModelAdapter());
	}
	protected Adapter getModelAdapter() {
		if (containerAdapter == null) {
			containerAdapter = new EMFListenerAdapter(this, false);
		}
		return containerAdapter;
	}
	protected void createEditPolicies() {
		// The TreeContainerEditPolicy is the CDE one
		// We don't care about being a Container with components
		// We are just interested in showing the viewPortView as a child
		super.createEditPolicies();
		installEditPolicy(
			EditPolicy.TREE_CONTAINER_ROLE,
			new com.ibm.etools.cde.TreeContainerEditPolicy(
				new JTableContainerPolicy(EditDomain.getEditDomain(this))));
	}
	public List getModelChildren() {
		return (List) ((RefObject) getModel()).refValue(sfColumns);
	}
	public void notifyChanged(
		Notifier notifier,
		int eventType,
		RefObject sf,
		Object oldValue,
		Object newValue,
		int pos) {
		if (sf == sfColumns)
			refreshChildren();
	}
	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		return ep;
	}
	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		ResourceSet rset = ((RefObject) model).getResourceSet();
		sfColumns = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABLE_COLUMNS);
	}
}