package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFContainerPolicy;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;

/**
 * @author pwalker
 *
 * Container policy for JToolBar. Children can be components or Actions.
 */
public class JToolBarContainerPolicy extends EMFContainerPolicy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected JavaClass classComponent, classAction;

	/**
	 * Constructor for JToolBarContainerPolicy.
	 * @param domain
	 */
	public JToolBarContainerPolicy(EditDomain domain) {
		super(null, domain);
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain);
		classComponent = (JavaClass) Utilities.getJavaClass("java.awt.Component", rset); //$NON-NLS-1$
		classAction = (JavaClass) Utilities.getJavaClass("javax.swing.Action", rset); //$NON-NLS-1$
	}

	protected boolean isValidChild(Object component) {
		// Also need to verify that the component is valid 
		return classComponent.isInstance(component) || classAction.isInstance(component);
	}

	/**
	 * @see com.ibm.etools.cde.ContainerPolicy#setContainer(Object)
	 */
	public void setContainer(Object container) {
		super.setContainer(container);
		if (container != null) {
			JavaClass modelType = (JavaClass) ((RefObject) container).refMetaObject();
			containmentSF = (EStructuralFeature) modelType.metaObject("items"); //$NON-NLS-1$
		}
	}

}
