package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;

/**
 * GraphicalEditPart for javax.swing.JToolBar.
 */
public class JToolBarGraphicalEditPart extends ContainerGraphicalEditPart {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfItems;
	protected JavaClass classAction;

	public JToolBarGraphicalEditPart(Object model) {
		super(model);
	}

	/*
	 * Use a JToolBarLayoutPolicy which is a FlowLayout
	 */
	protected void createLayoutEditPolicy() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new JToolBarLayoutEditPolicy(this));
	}

	/*
	 * Get only Components... can't handle Actions right now allow though they
	 * will show up in the VM and be reflected back into the graph viewer. 
	 */
	public List getModelChildren() {
		List items = (List) ((RefObject) getModel()).refValue(sfItems);
		ArrayList children = new ArrayList();
		Iterator iter = items.iterator();
		while (iter.hasNext()) {
			IJavaInstance item = (IJavaInstance) iter.next();
			if (!classAction.isInstance(item))
				children.add(item);
		}
		return children;
	}

	public void notifyChanged(
		Notifier notifier,
		int eventType,
		RefObject sf,
		Object oldValue,
		Object newValue,
		int pos) {
		if (sf == sfItems)
			refreshChildren();
	}

	protected EditPart createChild(Object model) {
		EditDomain dom = EditDomain.getEditDomain(this);
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		if (ep instanceof ComponentGraphicalEditPart) {
			((ComponentGraphicalEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
			((ComponentGraphicalEditPart) ep).setTransparent(true);
		}
		return ep;
	}

	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		JavaClass modelType = (JavaClass) ((RefObject) model).refMetaObject();
		sfItems = (EStructuralFeature) modelType.metaObject("items"); //$NON-NLS-1$
		ResourceSet rset = ((IJavaObjectInstance) model).getResourceSet();
		classAction = (JavaClass) Utilities.getJavaClass("javax.swing.Action", rset); //$NON-NLS-1$
	}
}
