package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;

import com.ibm.etools.cde.emf.CDECreationTool;
import com.ibm.etools.common.command.*;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.requests.CreateRequest;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.*;

public class LabelCreationPolicy implements CDECreationTool.CreationPolicy , IExecutableExtension {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected String fLabelKey = "Label"; //$NON-NLS-1$
	
public Command getCommand(Command aCommand, final CreateRequest aCreateRequest){
	
	Command setLabelCommand = new AbstractCommand(){
		public void execute(){
			// Query the live label to see whether there is one or not
			Object newObject = aCreateRequest.getNewObject();
			IBeanProxy labelProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance)newObject);
			IMethodProxy getLabelMethodProxy = labelProxy.getTypeProxy().getMethodProxy("getText"); //$NON-NLS-1$
			IBeanProxy existingLabelProxy = getLabelMethodProxy.invokeCatchThrowableExceptions(labelProxy);
			// Create a new label string if there is no label proxy, or if there is one for an empty string
			if ( existingLabelProxy == null || ((IStringBeanProxy)existingLabelProxy).stringValue().trim().equals("")) { //$NON-NLS-1$
				// Thew new label will be "Label".  This is held externally
				// The key to use is LabelPolicy.text.xxx where xxx is a piece of inializationData
				String labelString = VisualMessages.getString("LabelPolicy.text." + fLabelKey); //$NON-NLS-1$
				RefObject refNewObject = (RefObject)newObject;
				ResourceSet resourceSet = refNewObject.getResourceSet();
				IJavaInstance newLabel = BeanUtilities.createJavaObject("java.lang.String" , resourceSet , "\"" + labelString + "\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				EStructuralFeature sf_label = (EStructuralFeature)refNewObject.refMetaObject().metaObject("text"); //$NON-NLS-1$
				((RefObject)newObject).refSetValue(sf_label,newLabel);
			}
		}
		// We don't really care about redo and undo for now
		public void redo(){
		}
		public boolean canExecute(){
		    return true;
		}
	};
	
	if ( aCommand instanceof CompoundCommand ) {
		((CompoundCommand)aCommand).append(setLabelCommand);
		return aCommand;
	} else {
		CompoundCommand result = new CompoundCommand(aCommand.getLabel());
		result.append(aCommand);
		result.append(setLabelCommand);
		return result;
	}
}
public void setInitializationData(IConfigurationElement ce, String pName, Object initData) {
	fLabelKey = (String)initData;
}
    /**
     * @see CreationPolicy#getDefaultSuperString(EClass)
     */
    public String getDefaultSuperString(EClass superClass) {
        return null;
    }

}
