package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.widgets.Composite;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaHelpers;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.IJBCFCellEditor;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.propertysheet.ObjectComboBoxCellEditor;
/**
 * The method createItems shows a list of available layout manager classes from which the 
 * user can pick one.
 */
public class LayoutManagerCellEditor extends ObjectComboBoxCellEditor implements IJBCFCellEditor, INeedData {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain fEditDomain;
	
	protected static String[] fItems = new String[]{
		VisualMessages.getString("Layout.NullLayout") , VisualMessages.getString("Layout.BorderLayout") ,VisualMessages.getString("Layout.BoxLayoutX_AXIS"), VisualMessages.getString("Layout.BoxLayoutY_AXIS"), VisualMessages.getString("Layout.CardLayout"), VisualMessages.getString("Layout.FlowLayout"), VisualMessages.getString("Layout.GridBagLayout"), VisualMessages.getString("Layout.GridLayout") //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$
	};
	protected static String[] fClassNames = new String[] {
		"","java.awt.BorderLayout", "javax.swing.BoxLayoutX_Axis", "javax.swing.BoxLayoutY_Axis", "java.awt.CardLayout","java.awt.FlowLayout","java.awt.GridBagLayout","java.awt.GridLayout" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$
	};
/**
 * This method shows a list of available layout manager classes from which the 
 * user can pick one.
 * For now the list is hard coded because there is no way yet ( until we get the VA2000 Java IDE ) 
 * to see all classes that implement a specific interface.
 */
public LayoutManagerCellEditor(Composite aComposite){
	super(aComposite,fItems);
}
/**
 * Return a MOF class that represents the constraint bean
 */
protected Object doGetObject(int index) {
	if (index == sNoSelection || index == 0)
		return null;
	String layoutManagerClassName = fClassNames[index];
	ResourceSet rset = JBCFEditDomainHelper.getResourceSet(fEditDomain);
	// If this is one of the BoxLayout's, we need force reflection
	// on the actual BoxLayout so it will find these special dummy classes.
	if (layoutManagerClassName.equals("javax.swing.BoxLayoutX_Axis") //$NON-NLS-1$
		|| layoutManagerClassName.equals("javax.swing.BoxLayoutY_Axis")) { //$NON-NLS-1$
		JavaHelpers javaClass = JavaClassImpl.reflect("javax.swing.BoxLayout", rset); //$NON-NLS-1$
		if (javaClass != null)
			 ((JavaClass) javaClass).getEDecorators();
	}
	JavaHelpers javaClass = JavaClassImpl.reflect(layoutManagerClassName, rset);
	ILayoutPolicyFactory factory =
		BeanAwtUtilities.getLayoutPolicyFactoryFromLayoutManger((EClassifier) javaClass, fEditDomain);
	return factory.getLayoutManagerInstance(javaClass, rset);
}

protected int doGetIndex(Object anObject){
	// The argument is an IJavaInstance.  Get its bean proxy and compare it against
	// the values stored by us
	if (anObject == null) {
		return 0;
	} else if (anObject instanceof IJavaObjectInstance) {
		String className = LayoutManagerLabelProvider.getQualifiedName((IJavaObjectInstance)anObject);
		// Look for this class name in our known list 
		for(int i=1 ; i<fClassNames.length ; i++){
			if (className.equals(fClassNames[i])){
				return i;
			}
		}
	}
	return sNoSelection;
}

public String getJavaInitializationString() {
	com.ibm.etools.cde.CDEHack.fixMe();	// Are there some managers that aren't default ctor'd. If so, we need a different way 
							// of getting the initialization string. This is only used for the customizer stuff.
	Object v = doGetValue();
	if (v == null)
		return "null"; //$NON-NLS-1$
	else {
		IJavaObjectInstance jv = (IJavaObjectInstance) v;
		return "new " + jv.getJavaType().getQualifiedName() + "()"; //$NON-NLS-1$ //$NON-NLS-2$
	}
}
public static String getDisplayName(String className){
	String dispName = null;
	for(int i=0;i<fClassNames.length;i++){
		if(fClassNames[i].equals(className)){
			dispName = new String(fItems[i]);
			break;
		}
	}
	if(dispName==null)
		return ""; //$NON-NLS-1$
	else
		return dispName;
}

public String isCorrectObject(Object anObject){
	return null;
}
public void setData(Object data){
	fEditDomain = (EditDomain) data;
}
}
