package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.jface.viewers.LabelProvider;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.proxy.IBeanProxy;
/**
 * Renderer for the layout property for a Container
 */
public class LayoutManagerLabelProvider extends LabelProvider implements INeedData {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain editDomain;
	
	public String getText(Object element) {
		IBeanProxy beanProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance) element, JBCFEditDomainHelper.getResourceSet(editDomain));		if (!beanProxy.isNullProxy()) {
			String qualifiedName = getQualifiedName(element);
			return LayoutManagerCellEditor.getDisplayName(qualifiedName);
		} else {
			return VisualMessages.getString("Layout.NullLayout"); //$NON-NLS-1$
		}
	}

	public static String getQualifiedName(Object element) {
		String qualifiedName = ""; //$NON-NLS-1$
		IBeanProxy beanProxy = BeanProxyUtilities.getBeanProxy((IJavaInstance) element);
		if (!beanProxy.isNullProxy()) {
			qualifiedName = beanProxy.getTypeProxy().getTypeName();
			// With a BoxLayout, there are two possible display names...
			// based on the axis. Need to get the initialization string.
			if (qualifiedName.equals("javax.swing.BoxLayout")) { //$NON-NLS-1$
				String initString = ((IJavaInstance) element).getInitializationString();
				int lastCommaIndex = initString.lastIndexOf(","); //$NON-NLS-1$
				if (lastCommaIndex != -1) {
					int lastParenthesisIndex = initString.lastIndexOf(")"); //$NON-NLS-1$
					if (lastParenthesisIndex != -1) {
						String axisString = initString.substring(lastCommaIndex + 1, lastParenthesisIndex).trim();
						axisString.trim();
						if (axisString.equals("0") //$NON-NLS-1$
							|| axisString.equals("X_AXIS") //$NON-NLS-1$
							|| axisString.equals("javax.swing.BoxLayout.X_AXIS")) //$NON-NLS-1$
							qualifiedName = "javax.swing.BoxLayoutX_Axis"; //$NON-NLS-1$
						else if (
							axisString.equals("1") //$NON-NLS-1$
								|| axisString.equals("Y_AXIS") //$NON-NLS-1$
								|| axisString.equals("javax.swing.BoxLayout.Y_AXIS")) //$NON-NLS-1$
							qualifiedName = "javax.swing.BoxLayoutY_Axis"; //$NON-NLS-1$
					}
				}
			}
		}
		return qualifiedName;
	}
	/**
	 * @see INeedData#setData(Object)
	 */
	public void setData(Object data) {
		editDomain = (EditDomain) data;
	}

}