package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.common.command.Command;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.commands.CommandBuilder;
import java.util.*;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
/**
 * Layout Policy helper. This can be used by all layout types if they wish.
 * Creation date: (11/10/00 11:55:27 AM)
 * @author: Peter Walker
 */
public abstract class LayoutPolicyHelper implements ILayoutPolicyHelper {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ContainerPolicy policy;

	protected EStructuralFeature sfConstraintConstraint;
	protected EClass classConstraintComponent;
	protected EFactory visualFact;
	
	public LayoutPolicyHelper(ContainerPolicy ep) {
		setContainerPolicy(ep);
	}
	
	public LayoutPolicyHelper() {
	}
	
	public void setContainerPolicy(ContainerPolicy policy) {
		this.policy = policy;
		if (policy != null) {
			// Eventually we will be set with a policy. At that time we can compute these.
			ResourceSet rset = JBCFEditDomainHelper.getResourceSet(policy.getEditDomain());
			sfConstraintConstraint = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_CONSTRAINT);
			
			classConstraintComponent = (EClass) rset.getObject(JBCFConstants.CLASS_CONTAINER_CONSTRAINTCOMPONENT);		
			visualFact = JBCFConstants.getFactory(classConstraintComponent);		
		}
	}
	
	protected IJavaObjectInstance getContainer() {
		return (IJavaObjectInstance) policy.getContainer();
	}
	
	public Command getCreateChildCommand(Object childComponent, Object constraint, Object position) {		
		RefObject constraintComponent = (RefObject) visualFact.create(classConstraintComponent);
		if (constraint != NO_CONSTRAINT_VALUE)
			constraintComponent.refSetValue(JavaInstantiation.getSFeature(getContainer(), JBCFConstants.SF_CONSTRAINT_CONSTRAINT), constraint != null ? convertConstraint(constraint) : null);	// Put the constraint into the constraint component.
		return policy.getCreateCommand(constraintComponent, childComponent, position);
	}
	
	public Command getAddChildrenCommand(List childrenComponents, List constraints, Object position) {
		ArrayList componentConstraints = new ArrayList(childrenComponents.size());
		Iterator itr = constraints.iterator();
		while (itr.hasNext()) {
			RefObject constraintComponent = (RefObject) visualFact.create(classConstraintComponent);
			componentConstraints.add(constraintComponent);
			Object constraint = itr.next();
			if (constraint != NO_CONSTRAINT_VALUE)
				constraintComponent.refSetValue(sfConstraintConstraint, constraint != null ? convertConstraint(constraint) : null);	// Put the constraint into the constraint component.
		}
		
		return policy.getAddCommand(componentConstraints, childrenComponents, position);
	}
	
	public Command getOrphanChildrenCommand(List children) {	
		// Now get the orphan command for the children.
		return policy.getOrphanChildrenCommand(children);
	}
	
	/**
	 * Apply the corresponding constraint to each child.
	 * The child is the component itself, it must be inside a ConstraintComponent.
	 */
	public Command getChangeConstraintCommand(List children, List constraints) {
		CommandBuilder cb = new CommandBuilder(null);
		
		Iterator childs = children.iterator();
		Iterator consts = constraints.iterator();
		while (childs.hasNext()) {
			RefObject constraintComponent = (RefObject) ((RefObject) childs.next()).refContainer();
			Object oconstraint = consts.next();
			if (oconstraint != NO_CONSTRAINT_VALUE) {
				IJavaObjectInstance constraint = oconstraint != null ? convertConstraint(oconstraint) : null;
				cb.applyAttributeSetting(constraintComponent, sfConstraintConstraint, constraint);
			} else
				cb.cancelAttributeSetting(constraintComponent, sfConstraintConstraint);
		}
		return !cb.getCompoundCommand().isEmpty() ? cb.getCommand() : null;
	}
	
	/*
	 * Convert the incoming constraint to the actual constraint.
	 * This is necessary because often the incoming constraint is not a java object.
	 * It does this for convienence. But we need to java object.
	 */
	protected abstract IJavaObjectInstance convertConstraint(Object constraint);
}


