package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.*;
import com.ibm.etools.propertysheet.*;
import org.eclipse.core.runtime.*;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.jface.viewers.*;
import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.gef.commands.*;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.cde.commands.ApplyAttributeSettingCommand;
import com.ibm.etools.cde.commands.CancelAttributeSettingCommand;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.cde.EditDomain;

public class LayoutPropertyDescriptor extends BeanPropertyDescriptorAdapter implements ICommandPropertyDescriptor {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
public ILabelProvider getLabelProvider() {
	Class labelProviderClass = null;
	String classNameAndData = "com.ibm.etools.jbcf.visual/com.ibm.etools.jbcf.visual.LayoutManagerLabelProvider"; //$NON-NLS-1$
	try {
		labelProviderClass = CDEPlugin.getClassFromString(classNameAndData);
		if (labelProviderClass != null) {
			return createLabelProviderInstance(labelProviderClass, classNameAndData, null, this);
		}
	} catch (ClassNotFoundException e) {
		// One specified, but incorrect, log it
		JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e), JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
	}
	return null;
}
public CellEditor createPropertyEditor(Composite parent) {
	Class editorClass = null;
	String classNameAndData = "com.ibm.etools.jbcf.visual/com.ibm.etools.jbcf.visual.LayoutManagerCellEditor"; //$NON-NLS-1$
	try {
		editorClass = CDEPlugin.getClassFromString(classNameAndData);
		if (editorClass != null) {
			return createCellEditorInstance(editorClass,parent, classNameAndData, null);
		}
	} catch (ClassNotFoundException e) {
		// One specified, but incorrect, log it
		JBCFPlugin.getPlugin().getMsgLogger().log(new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e), JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
	}
	return null;
}
/**
 * It's here that we create the command to apply the new value to the 'layout' property of the container and we also create 
 * multiple commands to update the container's components. 
 * We do this by getting the new layoutManager's factory's constaint converter and based on the layoutmanager, 
 * the converter will create the command(s) necessary to update the constraints for any components in the container. 
 * 
 * With this capability, this allows us to create constraints on the container's components when going from a FlowLayoutManager
 * to BorderLayoutManager and vice versa... although changing to a FlowLayoutManager actually removes the constraints.
 */
public Command setValue(IPropertySource source, Object setValue){
	// The target of the BeanPropertySourceAdapter is the container
	IJavaObjectInstance container = (IJavaObjectInstance) source.getEditableValue();
	IPropertySource oldLayout = (IPropertySource)source.getPropertyValue(getTarget());
	if (oldLayout == null || oldLayout.getEditableValue() != setValue) {

		// Get the layout factory's layout switcher and let it create the necessary commands to apply the new
		// layout property to the container and the constraint commands for each of the children.
		IBeanProxyHost containerProxyHost = BeanProxyUtilities.getBeanProxyHost(container);
		IBeanProxy containerProxy = containerProxyHost.getBeanProxy();
		EditDomain domain = containerProxyHost.getBeanProxyDomain().getEditDomain();
		EClassifier layoutManagerClass = null;
		if (setValue != null)
			layoutManagerClass = (EClassifier) ((IJavaObjectInstance)setValue).refMetaObject();
		ILayoutPolicyFactory layoutFactory = BeanAwtUtilities.getLayoutPolicyFactoryFromLayoutManger(layoutManagerClass, domain);
		if (layoutFactory != null) {
			ContainerPolicy cp = new ContainerPolicy(domain);
			cp.setContainer(container);
			ILayoutSwitcher switcher = layoutFactory.getLayoutSwitcher(cp);
			if (switcher != null) {
				return switcher.getCommand((RefStructuralFeature)getTarget(), (IJavaObjectInstance) setValue);
			}
		}
	}

	// Same value, or no switcher, so we just apply the new setting.	
	ApplyAttributeSettingCommand changeCmd = new ApplyAttributeSettingCommand();
	changeCmd.setTarget(container);
	changeCmd.setAttribute((RefStructuralFeature)getTarget());
	changeCmd.setAttributeSettingValue(setValue);
	return changeCmd;
}

public Command resetValue(IPropertySource source){
	// The target of the BeanPropertySourceAdapter is the container
	IJavaObjectInstance container = (IJavaObjectInstance) source.getEditableValue();

	// Get the layout factory's layout switcher and let it create the necessary commands to apply the new
	// layout property to the container and the constraint commands for each of the children.
	IBeanProxyHost containerProxyHost = BeanProxyUtilities.getBeanProxyHost(container);
	IBeanProxy containerProxy = containerProxyHost.getBeanProxy();
	EditDomain domain = containerProxyHost.getBeanProxyDomain().getEditDomain();
	IBeanProxy defaultLayoutManager = (IBeanProxy) containerProxyHost.getOriginalSettingsTable().get((RefStructuralFeature) getTarget());
	EClassifier layoutManagerClass = null;
	if (defaultLayoutManager != null)
		layoutManagerClass = (EClassifier) BeanProxyUtilities.getJavaType(defaultLayoutManager, container.getResourceSet());
	ILayoutPolicyFactory layoutFactory = BeanAwtUtilities.getLayoutPolicyFactoryFromLayoutManger(layoutManagerClass, domain);
	if (layoutFactory != null) {
		ContainerPolicy cp = new ContainerPolicy(domain);
		cp.setContainer(container);
		ILayoutSwitcher switcher = layoutFactory.getLayoutSwitcher(cp);
		if (switcher != null) {
			return switcher.getCancelCommand((RefStructuralFeature)getTarget(), defaultLayoutManager);
		}
	}

	// No switcher, so we just cancel the setting.	The constraints of any children will probably be bad.
	CancelAttributeSettingCommand changeCmd = new CancelAttributeSettingCommand();
	changeCmd.setTarget(container);
	changeCmd.setAttribute((RefStructuralFeature)getTarget());
	return changeCmd;
}

}