package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import java.util.*;

import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
/**
 * Base LayoutSwitcher class.
 */
public abstract class LayoutSwitcher implements ILayoutSwitcher {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ContainerPolicy policy;
	
	public LayoutSwitcher(ContainerPolicy policy) {
		this.policy = policy;
	}
	
	/**
	 * Get the command to orphan the children and then add them back at the 
	 * new calculated constraints.
	 */
	public Command getCommand(RefStructuralFeature sf, IJavaObjectInstance newManager) {
		IJavaObjectInstance containerBean = getContainerBean();
		
		CommandBuilder cb = new CommandBuilder("change layout"); //$NON-NLS-1$
		
		List constraintComponents = (List) containerBean.refValue(JavaInstantiation.getSFeature(containerBean, JBCFConstants.SF_CONTAINER_COMPONENTS));
		ArrayList children = new ArrayList(constraintComponents.size());
		Iterator itr = constraintComponents.iterator();
		EStructuralFeature sfConstraintComponent = JavaInstantiation.getSFeature(containerBean, JBCFConstants.SF_CONSTRAINT_COMPONENT);
		while(itr.hasNext()) {
			children.add(((RefObject) itr.next()).refValue(sfConstraintComponent));
		}
		
		// First orphan all of the children so that the old constraints are removed.		
		if (!children.isEmpty()) {
			// Get the layout policy helper class from the layout policy factory and
			// set it in the container helper policy.
			IBeanProxy containerProxy = BeanProxyUtilities.getBeanProxy(containerBean);			
			ILayoutPolicyFactory lpFactory = BeanAwtUtilities.getLayoutPolicyFactory(containerProxy, policy.getEditDomain());
			ILayoutPolicyHelper lpHelper = lpFactory.getLayoutPolicyHelper(policy);			
			cb.append(lpHelper.getOrphanChildrenCommand(children));
		}
		
		// Now apply new layout manager (it must be applied before adding the children back
		// in, or there will be refresh problems. It the children were added back in first,
		// they would have new kind of constraints, but it will still be the old layout manager.
		// Putting the manager first means it lays out no children when added, and then lays
		// out the children when they are added with the correct constraints).
		cb.applyAttributeSetting(containerBean, sf, newManager);
		
		// Now get the change constraint commands themselves to add the children at the right constraints.
		if (!children.isEmpty())
			cb.append(getChangeConstraintsCommand(children));
		return cb.getCommand();
	}
	
	/**
	 * Get the cancel command to orphan the children and then add them back at the 
	 * new calculated constraints.
	 */
	public Command getCancelCommand(RefStructuralFeature sf, IBeanProxy defaultManager) {
		IJavaObjectInstance containerBean = getContainerBean();
		
		CommandBuilder cb = new CommandBuilder("cancel layout"); //$NON-NLS-1$
		
		List constraintComponents = (List) containerBean.refValue(JavaInstantiation.getSFeature(containerBean, JBCFConstants.SF_CONTAINER_COMPONENTS));
		ArrayList children = new ArrayList(constraintComponents.size());
		Iterator itr = constraintComponents.iterator();
		EStructuralFeature sfConstraintComponent = JavaInstantiation.getSFeature(containerBean, JBCFConstants.SF_CONSTRAINT_COMPONENT);
		while(itr.hasNext()) {
			children.add(((RefObject) itr.next()).refValue(sfConstraintComponent));
		}
		
		// First orphan all of the children so that the old constraints are removed.		
		if (!children.isEmpty()) {
			// Get the layout policy helper class from the layout policy factory and
			// set it in the container helper policy.
			IBeanProxy containerProxy = BeanProxyUtilities.getBeanProxy(containerBean);			
			ILayoutPolicyFactory lpFactory = BeanAwtUtilities.getLayoutPolicyFactory(containerProxy, policy.getEditDomain());
			ILayoutPolicyHelper lpHelper = lpFactory.getLayoutPolicyHelper(policy);			
			cb.append(lpHelper.getOrphanChildrenCommand(children));
		}
		
		// Now cancel the layout manager (it must be canceled before adding the children back
		// in, or there will be refresh problems. If the children were added back in first,
		// they would have new kind of constraints, but it will still be the old layout manager.
		// Canceling the manager first means it lays out no children when added, and then lays
		// out the children when they are added with the correct constraints).
		cb.cancelAttributeSetting(containerBean, sf);
		
		// Now get the change constraint commands themselves to add the children at the right constraints.
		if (!children.isEmpty())
			cb.append(getChangeConstraintsCommand(children));
		return cb.getCommand();
	}	
	
	/**
	 * The container bean.
	 */
	protected IJavaObjectInstance getContainerBean() {
		return (IJavaObjectInstance) policy.getContainer();
	}
	
	/*
	 * Get the command to add the children back in the right constraints.
	 * This allows the switcher to determine the order to add them back and whether
	 * they all can be added back. 
	 * The children are the actual components, not the constraintComponents. The implementation
	 * should create new ConstraintComponents (usually through the helper policy of the
	 * new layout).
	 */
	protected abstract Command getChangeConstraintsCommand(List children);
}