package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.propertysheet.*;

/**
 * @author pwalker
 *
 * A IPropertySource used to wrapper the actual IPropertySource adapter in order to 
 * exclude the "bounds", "size", and "location" properties in order to hide them on the property sheet.
 */
public class NonBoundsBeanPropertySource implements IPropertySource {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IPropertySource fSourceAdapter;

	public NonBoundsBeanPropertySource(Notifier target) {
		super();
		fSourceAdapter = (IPropertySource) target.getAdapter(IPropertySource.class);
	}
	/*
	 * Need to merge in the properties of the component.
	 * We will have to wrapper any ISourced or ICommand descriptors because
	 * those will have the wrong source passed into them. We need to intercept
	 * and have the correct source (the component) passed into them.
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		IPropertyDescriptor[] theirs = fSourceAdapter.getPropertyDescriptors();
		IPropertyDescriptor[] wrappedTheirs = new IPropertyDescriptor[theirs.length];
		int wi = 0;
		for (int i = 0; i < theirs.length; i++) {
			IPropertyDescriptor pd = theirs[i];
			if (pd.getId() instanceof RefStructuralFeature) {
				// exclude bounds/size/location
				String fn = ((RefStructuralFeature) pd.getId()).refName();
				if ("bounds".equals(fn) || "size".equals(fn) || "location".equals(fn)) //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					continue;
			}
			if (pd instanceof ISourcedPropertyDescriptor || pd instanceof ICommandPropertyDescriptor)
				wrappedTheirs[wi++] = new WrapperedPropertyDescriptor(this, pd);
			else
				wrappedTheirs[wi++] = pd;
		}

		// Finally build the complete list.
		IPropertyDescriptor[] finalList = new IPropertyDescriptor[wi];
		System.arraycopy(wrappedTheirs, 0, finalList, 0, wi);
		return finalList;
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return fSourceAdapter.getEditableValue();
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(Object)
	 * Forward to the source adapter
	 */
	public Object getPropertyValue(Object id) {
		return fSourceAdapter.getPropertyValue(id);
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(Object)
	 * Forward to the source adapter
	 */
	public boolean isPropertySet(Object id) {
		return fSourceAdapter.isPropertySet(id);
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(Object)
	 * Forward to the source adapter
	 */
	public void resetPropertyValue(Object id) {
		fSourceAdapter.resetPropertyValue(id);
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(Object, Object)
	 * Forward to the source adapter
	 */
	public void setPropertyValue(Object id, Object value) {
		fSourceAdapter.setPropertyValue(id, value);
	}

}
