package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.common.command.Command;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.awt.IRectangleBeanProxy;
import com.ibm.etools.draw2d.geometry.Rectangle;
import java.util.*;
/**
 * This helper class is used when a container's layout is changed
 * to a null Layout. Each of the constraints for the container's children 
 * constraints need to be assigned a rectangle.
 *
 * Creation date: (10/11/00 12:13:19 PM)
 * @author: Peter Walker
 */
public class NullLayoutSwitcher extends LayoutSwitcher {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected NullLayoutPolicyHelper helper;
	
	/**
	 * NullLayoutConstraintConverter constructor comment.
	 */
	public NullLayoutSwitcher(ContainerPolicy cp) {
		super(cp);
		helper = new NullLayoutPolicyHelper(cp);
	}
	
	/**
	 * Returns a compound command containing the child constraint commands to position the
	 * children within a null Layout container.
	 * Since this is a null Layout, it doesn't matter what the previous constraints
	 * were for the children since we're going to create Rectangle constraints.
	 * In order to do this we must get the position coordinates from the live bean
	 * and create setting commands with a Bean rectangle constraint.
	 */
	protected Command getChangeConstraintsCommand(List children) {
		ArrayList constraints = new ArrayList(children.size());
		Iterator comps = children.iterator();
		while (comps.hasNext()) {
			IJavaObjectInstance comp = (IJavaObjectInstance) comps.next();
			IBeanProxy compProxy = BeanProxyUtilities.getBeanProxy(comp);
			
			IRectangleBeanProxy rectProxy = (IRectangleBeanProxy) BeanAwtUtilities.invoke_getBounds(compProxy);
			Rectangle newRect = new Rectangle(rectProxy.getX(), rectProxy.getY(), rectProxy.getWidth(), rectProxy.getHeight());
			constraints.add(new NullLayoutPolicyHelper.NullConstraint(newRect, true, true));
		}
		
		return helper.getAddChildrenCommand(children, constraints, null);	// Now let the helper add them back correctly.
	}
}
