package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.jface.action.IMenuManager;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.draw2d.IFigure;
import com.ibm.etools.draw2d.XYLayout;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.gef.GraphicalEditPart;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.visual.tab.IOrderManager;
import com.ibm.etools.jbcf.visual.tab.ITabRootEditPart;
import com.ibm.etools.jbcf.visual.tab.JTabOrderManager;
import com.ibm.etools.jbcf.visual.tab.ShowTabOrderAction;

/**
 * This is the graphical editpart for any RootPaneContainer, such as JFrame. There is
 * a Swing interface called RootPaneContainer, and this is the general edit part for it.
 */
public class RootPaneContainerGraphicalEditPart extends ComponentGraphicalEditPart implements IEMFListener  {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter containerAdapter;
	private EStructuralFeature sf_contentPane;
	
	public RootPaneContainerGraphicalEditPart(Object model) {
		super(model);
	}

	public void activate() {
		super.activate();
		((RefObject) getModel()).addAdapter(getModelAdapter());
		// The content pane of a RootPaneContainer doesn't have the RootPaneContainer as its
		// immediate parent. There is a RootPane in-between that we don't normally see.
		// However, for position information, we want it to look like it is a direct
		// child of the RootPaneContainer. To do this we ame the RootPaneContainer as the
		// parent component proxy host. 

		// This is to force in the implicit content pane and make it available as a mof setting.			
		getModelChildren();
	}
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sf_contentPane)
			refreshChildren();
	}
	public void deactivate() {
		super.deactivate();
		((RefObject) getModel()).removeAdapter(getModelAdapter());
	}

	protected Adapter getModelAdapter() {
		if (containerAdapter == null) {
			containerAdapter = new EMFListenerAdapter(this, false);
		}
		return containerAdapter;
	}
	/**
	 * Our logical child is the content pane.  We must create one for now if it is not there
	 */
	public List getModelChildren() {
		RefObject model = (RefObject) getModel();
		ArrayList result = new ArrayList();

		if (sf_contentPane != null) {
			if (model.refIsSet(sf_contentPane)) {
				// We have an explicit content pane, return it unless it is null
				// OR unless it doesn't have a beanProxy, i.e., is in error
				Object contentPane = model.refValue(sf_contentPane);
				if (contentPane != null && BeanProxyUtilities.getBeanProxy((IJavaInstance) contentPane) != null) {
					result.add(contentPane);
				}
			} else {
				// Create a JPanel with a BorderLayout and explicitly use this as the content pane.
				IJavaInstance contentPane =
					RootPaneContentPaneCreation.createContentPane(
						(IJavaObjectInstance) getModel(),
						EditDomain.getEditDomain(this));
				if (contentPane != null) {
					result.add(contentPane);
				}
			}
		}
			
		if (!result.isEmpty())
			return result;
		return Collections.EMPTY_LIST;
	}

	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.CONTAINER_ROLE, new DefaultContainerEditPolicy());
		installEditPolicy(VisualComponentsLayoutPolicy.LAYOUT_POLICY, new VisualComponentsLayoutPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, createLayoutEditPolicy());
	}
	
	protected EditPolicy createLayoutEditPolicy() {
		return new CDELayoutEditPolicy(new RootPaneContainerPolicy(EditDomain.getEditDomain(this)));
	}
	
	protected EditPart createChild(Object model) {
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		((ComponentGraphicalEditPart) ep).setTransparent(true);
		((ComponentGraphicalEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
		// So that it doesn't create an image, we subsume it here.
		// The component proxy host of the rootPane needs to know that we are its parent
		// otherwise it thinks that its immediate parent is ( which is a layout pane )
		// and the positions get all wrong
		IComponentProxyHost rootPaneContainerProxyAdapter =
			(IComponentProxyHost) BeanProxyUtilities.getBeanProxyHost((IJavaInstance) getModel());			
		IComponentProxyHost contentsPaneProxyAdapter =
			(IComponentProxyHost) BeanProxyUtilities.getBeanProxyHost(
				(IJavaInstance) ((RefObject) getModel()).refValue(sf_contentPane));
		contentsPaneProxyAdapter.setParentComponentProxyHost(rootPaneContainerProxyAdapter);
				
		return ep;
	}
	protected IFigure createFigure() {
		IFigure fig = super.createFigure();
		fig.setLayoutManager(new XYLayout());
		return fig;
	}
	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		JavaClass modelType = (JavaClass) ((RefObject) model).refMetaObject();
		sf_contentPane = (EStructuralFeature) modelType.metaObject("contentPane"); //$NON-NLS-1$
	}
}
