package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import com.ibm.etools.cde.CDEUtilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Adapter;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.BeanProxyUtilities;

/**
 * This is the tree editpart for any RootPaneContainer, such as JFrame. There is
 * a Swing interface called RootPaneContainer, and this is the general edit part for it.
 */
public class RootPaneContainerTreeEditPart extends ComponentTreeEditPart implements IEMFListener {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EMFListenerAdapter containerAdapter;
	private EStructuralFeature sf_contentPane;

	/**
	 * Constructor with the model argument
	 */
	public RootPaneContainerTreeEditPart(Object model) {
		super(model);
	}

	public void activate() {
		super.activate();
		((RefObject) getModel()).addAdapter(getModelAdapter());
	}

	public void deactivate() {
		super.deactivate();
		((RefObject) getModel()).removeAdapter(getModelAdapter());
	}

	protected Adapter getModelAdapter() {
		if (containerAdapter == null) {
			containerAdapter = new EMFListenerAdapter(this, false);
		}
		return containerAdapter;
	}

	protected EditPart createChild(Object model) {
		EditPart ep = CDEUtilities.getEditPartFactory(this).createEditPart(model);
		((ComponentTreeEditPart) ep).setPropertySource(new NonBoundsBeanPropertySource((RefObject)model));
		return ep;
	}

	/**
	 * Our children is the root pane.  We must create one for now if it is not there
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.TREE_CONTAINER_ROLE, new com.ibm.etools.cde.TreeContainerEditPolicy(createContainerEditPolicy()));
	}

	protected com.ibm.etools.cde.ContainerPolicy createContainerEditPolicy() {
		return new RootPaneContainerPolicy(EditDomain.getEditDomain(this));
	}

	public List getModelChildren() {
		RefObject model = (RefObject) getModel();
		ArrayList result = new ArrayList();

		if (sf_contentPane != null) {
			if (model.refIsSet(sf_contentPane)) {
				// We have an explicit content pane, return it.
				// unless it is null - This can occur if the user has the line ivjJFrame.setContentPane(null)
				Object contentPane = model.refValue(sf_contentPane);
				if (contentPane != null && BeanProxyUtilities.getBeanProxy((IJavaInstance) contentPane) != null) {
					result.add(contentPane);
				}
			} else {
				// Create a JPanel with a BorderLayout and explicitly use this as the content pane.
				IJavaInstance contentPane =
					RootPaneContentPaneCreation.createContentPane(
						(IJavaObjectInstance) getModel(),
						EditDomain.getEditDomain(this));
				if (contentPane != null) {
					result.add(contentPane);
				}
			}
		}
		if (!result.isEmpty())
			return result;
		return Collections.EMPTY_LIST;
	}

	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		if (sf == sf_contentPane)
			refreshChildren();
	}
	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		JavaClass modelType = (JavaClass) ((RefObject) model).refMetaObject();
		sf_contentPane = (EStructuralFeature) modelType.metaObject("contentPane"); //$NON-NLS-1$
	}
}