package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.properties.NameInCompositionPropertyDescriptor;
import com.ibm.etools.cdm.CDMPackage;
import com.ibm.etools.cdm.KeyedString;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.BeanUtilities;

/**
 * This class is used by the RootPane EditParts to create the
 * content pane if it doesn't have one. It makes sure everything
 * is done under the correct control.
 */
public class RootPaneContentPaneCreation {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static IJavaObjectInstance createContentPane(final IJavaObjectInstance rootPaneContainer, final EditDomain domain) {
		final IJavaObjectInstance[] contentPaneHolder = new IJavaObjectInstance[1];

		IModelChangeController controller = (IModelChangeController) domain.getData(IModelChangeController.MODEL_CHANGE_CONTROLLER_KEY);
		controller.run(new Runnable() {
			public void run() {
				ResourceSet rset = rootPaneContainer.getResourceSet();
					final IJavaObjectInstance contentPane = (IJavaObjectInstance) BeanUtilities.createJavaObject("javax.swing.JPanel", //$NON-NLS-1$
						rset, "new javax.swing.JPanel()"); //$NON-NLS-1$
				if (contentPane != null) {
						IJavaInstance layoutBean = BeanUtilities.createJavaObject("java.awt.BorderLayout", //$NON-NLS-1$
							rset, "new java.awt.BorderLayout()"); //$NON-NLS-1$
					if (layoutBean != null) {
						EStructuralFeature sf_containerLayout = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_LAYOUT);
						contentPane.refSetValue(sf_containerLayout, layoutBean);
					}
				}

				if (contentPane != null) {
					// We need to create an annotation for this poofed up contentpane
					KeyedString ks = ((CDMPackage) RefRegister.getPackage(CDMPackage.packageURI)).getCDMFactory().createKeyedString();
					ks.setKey(NameInCompositionPropertyDescriptor.NAME_IN_COMPOSITION_KEY);
					ks.setValue("jContentPane"); //$NON-NLS-1$
					Command cmd = AnnotationPolicy.applyAnnotationSetting(contentPane, ks, domain);
					if (cmd.canExecute())
						cmd.execute();
					// Now apply it to the root container model
					JavaClass modelType = (JavaClass) rootPaneContainer.refMetaObject();
					EStructuralFeature sf_contentPane = (EStructuralFeature) modelType.metaObject("contentPane"); //$NON-NLS-1$
					rootPaneContainer.refSetValue(sf_contentPane, contentPane);
				}
				
				contentPaneHolder[0] = contentPane;
			}
		});

		return contentPaneHolder[0];
	}

}