package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import com.ibm.etools.cde.CDELayoutEditPolicy;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.BeanProxyUtilities;

/**
 * @author pwalker
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class RootPaneJMenuBarContainerGraphicalEditPart extends RootPaneContainerGraphicalEditPart {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private EStructuralFeature sf_jmenubar;

	/**
	 * Constructor for RootPaneJMenuBarContainerGraphicalEditPart.
	 * @param model
	 */
	public RootPaneJMenuBarContainerGraphicalEditPart(Object model) {
		super(model);
	}
	
	protected EditPart createChild(Object model) {
		EditPart ep = super.createChild(model);
		// So that it doesn't create an image, we subsume it here.
		// The component proxy host of the rootPane needs to know that we are its parent
		// otherwise it thinks that its immediate parent is ( which is a layout pane )
		// and the positions get all wrong
		IComponentProxyHost rootPaneContainerProxyAdapter =
			(IComponentProxyHost) BeanProxyUtilities.getBeanProxyHost((IJavaInstance) getModel());			
		IComponentProxyHost jmenuBarProxyAdapter =
			(IComponentProxyHost) BeanProxyUtilities.getBeanProxyHost(
				(IJavaInstance) ((RefObject) getModel()).refValue(sf_jmenubar));
		if (jmenuBarProxyAdapter != null)
			jmenuBarProxyAdapter.setParentComponentProxyHost(rootPaneContainerProxyAdapter);

		return ep;
	}
	
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.LAYOUT_ROLE, createLayoutEditPolicy());
	}
	
	protected EditPolicy createLayoutEditPolicy() {
		return new CDELayoutEditPolicy(new RootPaneJMenuBarContainerPolicy(EditDomain.getEditDomain(this)));
	}
	
	/**
	 * Our logical child is the JMenuBar.
	 */
	public List getModelChildren() {
		RefObject model = (RefObject) getModel();
		List result = super.getModelChildren();

		// If there is a JMenuBar, include this in the list
		if (sf_jmenubar != null) {
			Object jmenuBar = model.refValue(sf_jmenubar);
			if (jmenuBar != null)
				result.add(jmenuBar);
		}

		return result;
	}

	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
		super.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
		if (sf == sf_jmenubar)
			refreshChildren();
	}

	public void setModel(Object model) {
		super.setModel(model);
		JavaClass modelType = (JavaClass) ((RefObject) model).refMetaObject();
		sf_jmenubar = (EStructuralFeature) modelType.metaObject("JMenuBar"); //$NON-NLS-1$
	}
}
