package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.List;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.emf.EMFContainerPolicy;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.java.JavaClass;

/**
 * @author pwalker
 *
 * Container policy for Swing containers that can have both a rootpane and JMenuBar
 * such as JFrame, JDialog, JApplet, and JInternalFrame.
 */
public class RootPaneJMenuBarContainerPolicy extends EMFContainerPolicy {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * Constructor for RootPaneJMenuBarContainerPolicy.
	 * @param containmentSF
	 * @param domain
	 */
	public RootPaneJMenuBarContainerPolicy(EditDomain domain) {
		super(null, domain);
	}

	/**
	 * @see com.ibm.etools.cde.ContainerPolicy#getDeleteDependentCommand(Object)
	 * If the container structural feature for this child is the JMenuBar SF,
	 * continue on and let the parent handle it.
	 */
	public Command getDeleteDependentCommand(Object child) {
		if ( ((RefObject)child).refContainerSF() == containmentSF )
			return super.getDeleteDependentCommand(child);
		else
			return null;
	}

	/**
	 * @see com.ibm.etools.cde.ContainerPolicy#getMoveChildrenCommand(List, Object)
	 */
	public Command getMoveChildrenCommand(List children, Object positionBeforeChild) {
		if (children != null && !children.isEmpty()) {
			if ( ((RefObject)children.get(0)).refContainerSF() == containmentSF )
				return super.getMoveChildrenCommand(children, positionBeforeChild);
		}
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @see com.ibm.etools.cde.ContainerPolicy#getOrphanChildrenCommand(List)
	 */
	public Command getOrphanChildrenCommand(List children) {
		if (children != null && !children.isEmpty()) {
			if ( ((RefObject)children.get(0)).refContainerSF() == containmentSF )
				return super.getOrphanChildrenCommand(children);
		}
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @see com.ibm.etools.cde.ContainerPolicy#setContainer(Object)
	 */
	public void setContainer(Object container) {
		super.setContainer(container);
		if (container != null) {
			JavaClass modelType = (JavaClass) ((RefObject) container).refMetaObject();
			containmentSF = (EStructuralFeature) modelType.metaObject("JMenuBar"); //$NON-NLS-1$
		}
	}
}
