package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.jbcf.BeanPropertyDescriptorAdapter;
import com.ibm.etools.propertysheet.ICommandPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.propertysheet.RestoreDefaultPropertyValueCommand;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.proxy.awt.IRectangleBeanProxy;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.jbcf.PointJavaClassCellEditor;
/**
 * Provide some specific overrides for Size property.
 */
public class SizePropertyDescriptor extends BeanPropertyDescriptorAdapter implements ICommandPropertyDescriptor {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public boolean areNullsInvalid() {
		return true;
	}
	
	public Command setValue(IPropertySource source, Object setValue) {
		// Set the size and unset the bounds, if set.
		// Unset the bounds first before applying size because
		// they interfere with each other.
		CommandBuilder cb = new CommandBuilder(null);
		IJavaObjectInstance comp = (IJavaObjectInstance) source.getEditableValue();
		EStructuralFeature sfComponentBounds = JavaInstantiation.getSFeature(comp, JBCFConstants.SF_COMPONENT_BOUNDS);
		if (comp.refIsSet(sfComponentBounds)) {
			cb.cancelAttributeSetting(comp, sfComponentBounds);
			EStructuralFeature sfComponentLocation = JavaInstantiation.getSFeature(comp, JBCFConstants.SF_COMPONENT_LOCATION);
			if (!comp.refIsSet(sfComponentLocation)) {
				// A little more difficult, need to get the location from the bounds
				// and SET Location with it so we don't loose that.
				IJavaInstance boundsObject = (IJavaInstance) comp.refValue(sfComponentBounds);
				IRectangleBeanProxy bounds = (IRectangleBeanProxy) BeanProxyUtilities.getBeanProxy(boundsObject);
				Object newLoc = BeanUtilities.createJavaObject("java.awt.Point", //$NON-NLS-1$
					comp.getResourceSet(),
					PointJavaClassCellEditor.getJavaInitializationString(bounds.getX(), bounds.getY()));
				cb.applyAttributeSetting(comp, sfComponentLocation, newLoc);
			}
		}
			
		cb.applyAttributeSetting(comp, (RefStructuralFeature) getTarget(), setValue);
		return cb.getCommand();
	}
	
	public Command resetValue(IPropertySource source) {
		RestoreDefaultPropertyValueCommand cmd  = new RestoreDefaultPropertyValueCommand();
		cmd.setTarget(source);
		cmd.setPropertyId(getTarget());
		return cmd;
	}

}


