package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.proxy.IMethodProxy;

public class TableColumnProxyAdapter extends BeanProxyAdapter {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	
	protected EStructuralFeature sfModelIndex, sfHeaderValue;
	protected IMethodProxy resetHeaderValueProxy;
	
	public TableColumnProxyAdapter(IBeanProxyDomain aDomain) {
		super(aDomain);

		sfModelIndex = JavaInstantiation.getSFeature(JBCFEditDomainHelper.getResourceSet(domain.getEditDomain()), JBCFConstants.SF_TABLECOLUMN_MODELINDEX);		
		sfHeaderValue = JavaInstantiation.getSFeature(JBCFEditDomainHelper.getResourceSet(domain.getEditDomain()), JBCFConstants.SF_TABLECOLUMN_HEADERVALUE);				
	}
	
	/**
	 * When we validate we need to revalidate the JTable as well to cause the image to
	 * refresh.
	 */
	public void validateBeanProxy() {
	
		RefBaseObject table = ((RefObject) getTarget()).refContainer();
		// If we are on the freeform then container will not be an instance of table.
		if (table instanceof IJavaObjectInstance) {
			IBeanProxyHost tableProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance) table);
			// The table is a Component so this should refresh the shell and the image
			tableProxyHost.revalidateBeanProxy();
		}
	
	}
	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#applied(RefStructuralFeature, Object, int)
	 */
	protected void applied(RefStructuralFeature sf, Object newValue, int position) {
		super.applied(sf, newValue, position);
		
		modelIndexChanged(sf);
	}

	protected void modelIndexChanged(RefStructuralFeature sf) {
		if (sf == sfModelIndex && getErrorStatus() != ERROR_SEVERE && !getRefTarget().refIsSet(sfHeaderValue)) {
			IJavaObjectInstance table = (IJavaObjectInstance) ((RefObject) getTarget()).refContainer();
			// Not sure if this would occur - perhaps we could be on the free form although this
			// is unlikely
			if (table != null) {
				IBeanProxyHost tableProxyHost = BeanProxyUtilities.getBeanProxyHost(table);
				if (tableProxyHost.getErrorStatus() != ERROR_SEVERE) {		
					// Ask the JTableManager to reset the header value to the default since we've changed the index and have no header value set.
					getResetHeaderValueProxy().invokeCatchThrowableExceptions(null, new IBeanProxy[] {tableProxyHost.getBeanProxy(), getBeanProxy()});
					tableProxyHost.revalidateBeanProxy();
				}
			}
		}
	}

	/**
	 * @see com.ibm.etools.jbcf.BeanProxyAdapter#canceled(RefStructuralFeature, Object, int)
	 */
	protected void canceled(RefStructuralFeature sf, Object oldValue, int position) {
		super.canceled(sf, oldValue, position);
		
		modelIndexChanged(sf);
	}
	
	protected IMethodProxy getResetHeaderValueProxy() {
		if (resetHeaderValueProxy == null) {
			resetHeaderValueProxy = getBeanProxy()
				.getProxyFactoryRegistry()
				.getBeanTypeProxyFactory()
				.getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.JTableManager")	//$NON-NLS-1$
				.getMethodProxy("resetHeaderValue", new String[] {"javax.swing.JTable", "javax.swing.table.TableColumn"});	//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		return resetHeaderValueProxy;
	}	

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#releaseBeanProxy()
	 */
	public void releaseBeanProxy() {
		super.releaseBeanProxy();
		
		if (resetHeaderValueProxy != null) {
			resetHeaderValueProxy.getProxyFactoryRegistry().releaseProxy(resetHeaderValueProxy);
			resetHeaderValueProxy = null;
		}		
	}

}