package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.Iterator;
import java.util.List;

import com.ibm.etools.gef.editpolicies.LayoutEditPolicy;
import com.ibm.etools.gef.editpolicies.NonResizableEditPolicy;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.gef.Request;
import com.ibm.etools.gef.requests.ChangeBoundsRequest;
import com.ibm.etools.gef.requests.GroupRequest;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.gef.requests.CreateRequest;
/**
 * The layout input policy for java.awt.Containers for 
 * layout managers that we don't know how to handle.
 * It allows add/delete, but add will not have a constraint,
 * and change constraint will be disabled.
 * This is because if we don't know how to handle the layout,
 * we don't know how to handle the constraint.
 *
 * This is true no matter whether it is LayoutManager or LayoutManager2.
 */
public class UnknownLayoutInputPolicy extends LayoutEditPolicy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ContainerPolicy containerPolicy;
	
	protected UnknownLayoutPolicyHelper fLayoutPolicyHelper;	
	
public UnknownLayoutInputPolicy(ContainerPolicy aPolicy) {
	containerPolicy = aPolicy;
	fLayoutPolicyHelper = new UnknownLayoutPolicyHelper(aPolicy);
}

public void activate() {
	super.activate();
	containerPolicy.setContainer(getHost().getModel());
}

public void deactivate() {
	super.deactivate();
	containerPolicy.setContainer(null);
}

public EditPolicy createChildEditPolicy(EditPart aChild){
	return new NonResizableEditPolicy();
}
/**
 * Add of children requested.
 */
protected Command getAddCommand(Request request) {
	ChangeBoundsRequest cbReq = (ChangeBoundsRequest) request;	
	List children = ContainerPolicy.getChildren(cbReq);
	List constraints = Collections.nCopies(children.size(), null);
	return fLayoutPolicyHelper.getAddChildrenCommand(children, constraints, null);
}

/**
 * A new child is being created in this container.
 */
protected Command getCreateCommand(CreateRequest request) {
	// Unknown means no constraint and add at end.
	return fLayoutPolicyHelper.getCreateChildCommand(request.getNewObject(), null, null);
}

/**
 * getDeleteDependantCommand method comment.
 */
protected Command getDeleteDependantCommand(Request request) {
	Command deleteContributionCmd = containerPolicy.getCommand(request);
	if (deleteContributionCmd == null)
		return UnexecutableCommand.INSTANCE;	// It can't be created.
	return deleteContributionCmd;
}
/**
 * getMoveChildCommand method comment.
 * We don't perform move/resize since we don't know how.
 */
protected Command getMoveChildrenCommand(Request request) {
	return null;
}
/**
 * getOrphanChildCommand. To orphan, we delete only the child. We don't
 * delete the subpart since the part is not going away, just somewhere's
 * else. So the subpart stays.
 */
protected Command getOrphanChildrenCommand(Request request) {
	return fLayoutPolicyHelper.getOrphanChildrenCommand(ContainerPolicy.getChildren((GroupRequest) request));
}

}
