package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.widgets.Display;

import com.ibm.etools.proxy.*;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.jbcf.IBeanProxyDomain;
/**
 * Windows need explicit disposing when the proxy is disposed
 */
public class WindowProxyAdapter extends ContainerProxyAdapter {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	
public WindowProxyAdapter(IBeanProxyDomain domain) {
	super(domain);
}
/**
 * Windows need to be positioned off screen and also made visible
 * even when the MOF model says they are mot
 */
public IBeanProxy instantiateOnFreeForm(IBeanProxy aFreeFormDialogHost){
	
	// Need to make sure location/visibility are good before we instantiate so that it won't flash on the screen.
	applyLocation(false, BeanAwtUtilities.getOffScreenLocation());
	applyVisibility(false, Boolean.TRUE);
	
	if (getBeanProxy() == null && getErrorStatus() != ERROR_SEVERE)
		instantiateBeanProxy();	// If not already instantiated, and not errors, try again. If already instantiated w/severe, don't waste time
	
	if (getErrorStatus() == ERROR_SEVERE)
		return null;	// It is bad, don't go on.
	
	// Having done this the frame on the target VM will now possibly be visible
	// We should attempt to restore focus to the IDE
	if (Display.getCurrent().getActiveShell() != null)
	    Display.getCurrent().getActiveShell().setFocus();
	
	return getBeanProxy();
	
}
/**
 * releaseBeanProxy: Get rid of the bean proxy being held.
 * This allows for recreation if needed.
 * java.awt.Window objects (e.g. Frame) require an explicit
 * dispose to be called to get rid of it. Will only do it
 * if we own it.
 */
public void releaseBeanProxy() {
	if (fOwnsProxy && getBeanProxy() != null && getBeanProxy().isValid()) {
		// Invoke a method to dispose of the window.
		BeanAwtUtilities.invoke_dispose(getBeanProxy());
	};
	super.releaseBeanProxy();
}

public void disposeOnFreeForm(IBeanProxy aFreeFormDialogHost){
	if (fOwnsProxy && getBeanProxy() != null && getBeanProxy().isValid()) {
		// Invoke a method to dispose of the window.
		BeanAwtUtilities.invoke_dispose(getBeanProxy());
	};	
}

/**
 * @see com.ibm.etools.jbcf.visual.IComponentProxyHost#childValidated(IComponentProxyHost)
 */
public void childValidated(IComponentProxyHost childProxy) {
	super.childValidated(childProxy);
	// See if we are not explicitly setting size. If we aren't, then
	// we need to do a pack.
	RefObject rtarget = getRefTarget();
	if (getBeanProxy() != null && !rtarget.refIsSet(sfComponentBounds) && !rtarget.refIsSet(sfComponentSize))
		BeanAwtUtilities.invoke_window_pack(getBeanProxy());		
}

}
