package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class BasicColorPropertyPage extends JPanel implements ListSelectionListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private static java.util.ResourceBundle resabtedit = java.util.ResourceBundle.getBundle("com/ibm/etools/jbcf/visual/beaninfo/vceedit");  //$NON-NLS-1$
	public static String[] colorNames =
		{resabtedit.getString("black"), resabtedit.getString("blue"), resabtedit.getString("cyan"), resabtedit.getString("darkGray"), resabtedit.getString("gray"), resabtedit.getString("green"), resabtedit.getString("lightGray"), //$NON-NLS-7$ //$NON-NLS-6$ //$NON-NLS-5$ //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("magenta"), resabtedit.getString("orange"), resabtedit.getString("pink"), resabtedit.getString("red"), resabtedit.getString("white"), resabtedit.getString("yellow")}; //$NON-NLS-6$ //$NON-NLS-5$ //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
	public static Color[] colorValues =
		{Color.black, Color.blue, Color.cyan, Color.darkGray, Color.gray, Color.green, Color.lightGray, 
		Color.magenta, Color.orange, Color.pink, Color.red, Color.white, Color.yellow};
	public static String[] initStrings =
		{"black", "blue", "cyan", "darkGray", "gray", "green", "lightGray",//$NON-NLS-7$//$NON-NLS-6$//$NON-NLS-5$//$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"magenta", "orange", "pink", "red", "white", "yellow"};//$NON-NLS-6$//$NON-NLS-5$//$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
	
	private JScrollPane scrollPane = null;
	private JList basicColorsList = null;
	protected transient PropertyChangeSupport fPropertyChange = new PropertyChangeSupport(this);

public BasicColorPropertyPage() {
	super();
	initialize();
}
/**
 * The addPropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
	fPropertyChange.addPropertyChangeListener(listener);
}
private void basicColorsListValueChanged(ListSelectionEvent arg1) {
	firePropertyChange("colorValue", null, getColorValue());//$NON-NLS-1$
}
private void fillColorList ( ) {
	DefaultListModel model = (DefaultListModel)getBasicColorsList().getModel();
	for(int i=0; i < colorNames.length; i++) {
		model.addElement(new Integer(i));
	}
	return;
}
/**
 * The firePropertyChange method was generated to support the propertyChange field.
 */
public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
	if ( fPropertyChange == null ) {
		fPropertyChange = new PropertyChangeSupport(this);
	}
	fPropertyChange.firePropertyChange(propertyName, oldValue, newValue);
}
private JList getBasicColorsList() {
	if (basicColorsList == null) {
		try {
			basicColorsList = new JList();
			basicColorsList.setName("BasicColorsList");//$NON-NLS-1$
			basicColorsList.setModel(new DefaultListModel());
			basicColorsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			basicColorsList.setCellRenderer(new ColorCellRenderer(colorNames,colorValues));
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return basicColorsList;
}
public Color getColorValue() {
	return (Color)getValue();
}
String getJavaInitializationString ( ) {
	int index = indexOfSelectedColor();
	return "java.awt.Color." + initStrings[index];//$NON-NLS-1$
}
	public static String getNameOfColor(int r, int g, int b){
		for(int i=0; i<colorValues.length; i++){
			if( (colorValues[i].getRed() == r) && (colorValues[i].getGreen() == g) && (colorValues[i].getBlue() == b) ){
				return(colorNames[i]);
			}
		}
		return null;
	}
	public static String getNameOfColor(Color c){
		for(int i=0; i<colorValues.length; i++){
			if( c.equals(colorValues[i]) ){
				return(colorNames[i]);
			}
		}
		return null;
	}
private JScrollPane getScrollPane() {
	if (scrollPane == null) {
		try {
			scrollPane = new JScrollPane();
			scrollPane.setName("ScrollPanel");//$NON-NLS-1$
			scrollPane.setViewportView(getBasicColorsList());
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	};
	return scrollPane;
}
Object getValue() {
	int index = indexOfSelectedColor();
	if (index < 0) {
		return null;
	} else {
		return colorValues[index];
	}
}
/**
 * Called whenever the part throws an exception.
 * @param exception java.lang.Throwable
 */
private void handleException(Throwable exception) {

	/* Uncomment the following lines to print uncaught exceptions to stdout */
	// System.out.println("--------- UNCAUGHT EXCEPTION ---------");
	// exception.printStackTrace(System.out);
}
private int indexOfSelectedColor() {
	return getBasicColorsList().getSelectedIndex();
}
/**
 * Initializes connections
 */
private void initConnections() {
	getBasicColorsList().addListSelectionListener(this);
}
/**
 * Initialize the class.
 */
private void initialize() {
	setName("BasicColorPropertyPage");//$NON-NLS-1$
	setLayout(new BorderLayout());
	setBackground(SystemColor.control);
	setSize(179, 181);
	this.add(getScrollPane());
	fillColorList();
	initConnections();
}
public static boolean isBasicColor(Color aColor) {
	for (int i=0; i<colorValues.length; i++) {
		if (aColor.equals(colorValues[i])) {
			return true;
		}
	}
	return false;
}
public void preselectColor(Color aColor) {
	for (int i=0; i<colorValues.length; i++) {
		if (aColor.equals(colorValues[i])) {
			getBasicColorsList().setSelectedIndex(i);
			getBasicColorsList().ensureIndexIsVisible(i);
			return;
		}
	}
	getBasicColorsList().setSelectedIndex(0);
	getBasicColorsList().ensureIndexIsVisible(0);
	return;
}
/**
 * The removePropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
	fPropertyChange.removePropertyChangeListener(listener);
}
public void setColorValue(Color colorValue) {
	// I may use this in the future in order to convert
	// the color value from another page into a System
	// color, but for now, do nothing.
	return;
}
public void valueChanged(ListSelectionEvent e) {
	if ((e.getSource() == getBasicColorsList()) ) {
		basicColorsListValueChanged(e);
	}
}
}
