package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.beans.PropertyChangeListener;

public class BevelBorderPropertyPage extends JPanel implements ItemListener,IBorderPropertyPage {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private JRadioButton raisedButton = null;
	private JRadioButton loweredButton = null;
	private ButtonGroup buttonGroup = null;
	private int bevelType = BevelBorder.RAISED;
	
public BevelBorderPropertyPage(){
	super();  
	initialize();
}

public String getDisplayName(){
	if ( getBevelType() == BevelBorder.LOWERED ) {
		return VisualBeanInfoMessages.getString("BevelBorder.Lowered.DisplayName"); //$NON-NLS-1$
	} else {
		return VisualBeanInfoMessages.getString("BevelBorder.Raised.DisplayName"); //$NON-NLS-1$
	}
}

public String getName(){
	return "BevelBorderPropertyPage"; //$NON-NLS-1$
}

public int getBevelType(){
	if (getRaisedButton().isSelected()){
		bevelType = BevelBorder.RAISED;
	}
	else if (getLoweredButton().isSelected()){
		bevelType = BevelBorder.LOWERED;
	}
	return bevelType;
}

public JRadioButton getRaisedButton(){
	if(raisedButton == null){
		try{
			raisedButton = new JRadioButton(VisualBeanInfoMessages.getString("BevelBorder.radio.Raised.Text"));  //$NON-NLS-1$
			raisedButton.setBackground(SystemColor.control);
			raisedButton.setName("Raised Bevel"); //$NON-NLS-1$
			raisedButton.addItemListener(this);
			
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return raisedButton;
}

public JRadioButton getLoweredButton() {
	if (loweredButton == null){
		try{
			loweredButton = new JRadioButton(VisualBeanInfoMessages.getString("BevelBorder.radio.Lowered.Text"));  //$NON-NLS-1$
			loweredButton.setBackground(SystemColor.control);
			loweredButton.setName("Lowered Bevel"); //$NON-NLS-1$
			loweredButton.addItemListener(this);
		}catch(Throwable e){
			e.printStackTrace();
		}
	
	}
	return loweredButton;
}

public ButtonGroup getButtonGroup(){
	if(buttonGroup == null){
		try{
			buttonGroup = new ButtonGroup();
			buttonGroup.add(getRaisedButton());
			buttonGroup.add(getLoweredButton());
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return buttonGroup;
}

public void initialize(){
	this.setName(getName());
	this.setBackground(SystemColor.control);
	this.setLayout(new GridLayout(8, 1, 0, 0));
	getButtonGroup();
	this.add(getRaisedButton(), getRaisedButton().getName());
	this.add(getLoweredButton(), getLoweredButton().getName());
	getRaisedButton().setSelected(true);
}

public Border getBorderValue(){
	Border aBorder = BorderFactory.createBevelBorder(getBevelType());
	return aBorder;
}

public void itemStateChanged(ItemEvent e){
	if(e.getSource() == getRaisedButton()){
		bevelType = BevelBorder.RAISED;
	}
	else if(e.getSource() == getLoweredButton()){
		bevelType = BevelBorder.LOWERED;
	}
	
	firePropertyChange("borderValueChanged",null,getBorderValue()); //$NON-NLS-1$
}

public String getJavaInitializationString(){		
	if (getBevelType() == BevelBorder.LOWERED){
		return "javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)"; //$NON-NLS-1$
	}
	
	//default is Raised BevelBorder
	return "javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)"; //$NON-NLS-1$
}

public boolean okToSetBorder(Border aBorder) {
	
	if ( aBorder instanceof BevelBorder ) {
		BevelBorder bevelBorder = (BevelBorder)aBorder;	
		bevelType = bevelBorder.getBevelType();
		if (bevelType == BevelBorder.RAISED){
			raisedButton.setSelected(true);
		}
		else if (bevelType == BevelBorder.LOWERED){
			loweredButton.setSelected(true);
		}
		return true;
	} 
	return false;
}


}