package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class BorderPropertyEditor extends JPanel implements ListSelectionListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private Border fBorder = null;
	private JLabel borderLabel = new JLabel(VisualBeanInfoMessages.getString("BorderPropertyEditor.Label.Text")); //$NON-NLS-1$
	private JList borderList = null;
	private JScrollPane borderPane = null;
	private JPanel detailPanel = null;
	private JPanel borderPreviewer = null;
	private boolean displayButtonRemoved = false;
	
	class BorderListModel extends Object{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		String listName;
		Class pageClass;
		Object page;
		BorderListModel(String aListName,Class aPageClass){
			listName = aListName;
			pageClass = aPageClass;
		}
		BorderListModel(String aListName,String aPageClassName){
			listName = aListName;
			try {
				pageClass = Class.forName(aPageClassName);
			} catch ( ClassNotFoundException exc ) {
				exc.printStackTrace();
			}
		}	
		
		String getListName(){
			return listName;
		}
		
		String getDisplayName(){
			if ( page == null ){
				return ""; //$NON-NLS-1$
			} else {
				return getPage().getDisplayName();
			}
		}
			
		IBorderPropertyPage getPage(){
			if ( page == null ) {
				try{
					page = pageClass.newInstance();
				}catch (Exception exc){
					exc.printStackTrace();
				}
			}
			return (IBorderPropertyPage)page;
		}
	};
	
	private BorderListModel nullBorderListModel = new BorderListModel("",NoBorderSelectedPropertyPage.class); //$NON-NLS-1$
	private BorderListModel[] fBorderModels = {
			nullBorderListModel,
			new BorderListModel(VisualBeanInfoMessages.getString("PropertyPage.Bevel"),BevelBorderPropertyPage.class), //$NON-NLS-1$
			//new BorderListModel("CompoundBorder", CompoundBorderPropertyPage.class),
			new BorderListModel(VisualBeanInfoMessages.getString("PropertyPage.Empty"),EmptyBorderPropertyPage.class), //$NON-NLS-1$
			new BorderListModel(VisualBeanInfoMessages.getString("PropertyPage.Etched"),EtchedBorderPropertyPage.class), //$NON-NLS-1$
			new BorderListModel(VisualBeanInfoMessages.getString("PropertyPage.Line"),LineBorderPropertyPage.class), //$NON-NLS-1$
			new BorderListModel(VisualBeanInfoMessages.getString("PropertyPage.Matte"),MatteBorderPropertyPage.class), //$NON-NLS-1$
			new BorderListModel(VisualBeanInfoMessages.getString("PropertyPage.Soft"),SoftBevelBorderPropertyPage.class), //$NON-NLS-1$
			new BorderListModel(VisualBeanInfoMessages.getString("PropertyPage.Titled"),TitledBorderPropertyPage.class), //$NON-NLS-1$
			
						
	};
	
	private JButton displayButton = new JButton(VisualBeanInfoMessages.getString("BorderPropertyEditor.DisplayButton.Text")); //$NON-NLS-1$
	private JLabel displayBorderName = new JLabel(" ");
	private BorderPagePropertyListener borderPagePropertyListener = new BorderPagePropertyListener();
	
	// An instance of this class is used to listen to each page which should signal
	// an event of name "border" and the new value of the border
	class BorderPagePropertyListener implements PropertyChangeListener{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		public void propertyChange(PropertyChangeEvent anEvent){
			if ( "borderValueChanged".equals(anEvent.getPropertyName())){ //$NON-NLS-1$
				setPreviewBorder( (Border) anEvent.getNewValue());
			}
		}
	}
		
	
public BorderPropertyEditor() {
	super();
	this.initialize();
}

public BorderPropertyEditor(Border aBorder) {
	super();
	this.initialize();
	setBorderValue(aBorder);
}

public Border getBorderValue() {
	if (getBorderList().getSelectedIndex() == -1){		
		IBorderPropertyPage aPage = nullBorderListModel.getPage();
		return aPage.getBorderValue();
	}
	BorderListModel aModel = (BorderListModel) (getBorderList().getSelectedValue());
	IBorderPropertyPage aPage = aModel.getPage();
	fBorder = aPage.getBorderValue();
	return fBorder;
}

public String getBorderName(){
	if (getBorderList().getSelectedIndex() < 0 ){		
		return nullBorderListModel.getDisplayName();
	}
	BorderListModel aModel = (BorderListModel) (getBorderList().getSelectedValue());
	String fBorderName = aModel.getDisplayName();
	if (fBorderName != null){
		return fBorderName;
	}
	return ""; //$NON-NLS-1$
}

public BorderListModel[] getBorderModelList(){
	return fBorderModels; 
}

public JList getBorderList(){
	if (borderList == null){
		try{
			borderList = new JList();
			DefaultListModel borderModel = new DefaultListModel();
			borderList.setModel(borderModel);
			String[] fBorderNames = new String[fBorderModels.length];
			for (int i = 0; i < fBorderModels.length; i++){
				borderModel.addElement(fBorderModels[i]);
				fBorderNames[i] = fBorderModels[i].getListName();
			}
			
			borderList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			borderList.setCellRenderer(new BorderCellRenderer(fBorderNames));
			
		}catch(Throwable ivjExc){
			handleException(ivjExc);
		}
		
	}
	return borderList;
		
}

public JScrollPane getBorderPane(){
	if (borderPane == null){
		try{
			borderPane = new JScrollPane(getBorderList());
			borderPane.setBackground(SystemColor.control);
		}catch(Throwable ivjExc){
			handleException(ivjExc);
		}
	}
	return borderPane;
}

public JPanel getBorderPreviewer(){
	if (borderPreviewer == null){
		try{
			borderPreviewer = new JPanel();
			borderPreviewer.setName("BorderPreviewer"); //$NON-NLS-1$
			borderPreviewer.setLayout(new BorderLayout());
			borderPreviewer.add(displayButton, "Center"); //$NON-NLS-1$
			displayButton.setBackground(Color.white);
		}catch(Throwable exc){
			handleException(exc);
		}
	}
	return borderPreviewer;
}

public JPanel getDetailPanel(){
	if (detailPanel == null){
		try{
			detailPanel = new JPanel();
			detailPanel.setLayout(new CardLayout());
			for (int i = 0; i < fBorderModels.length; i++){
				IBorderPropertyPage aPage = fBorderModels[i].getPage();
				detailPanel.add((Component)aPage, aPage.getName());
			}
			
		}catch(Throwable ivjExc){
			handleException(ivjExc);
		}
	}
	return detailPanel;
}

private void handleException(Throwable exception) {
	System.out.println("--------- UNCAUGHT EXCEPTION ---------"); //$NON-NLS-1$
	exception.printStackTrace(System.out);
}


private void initialize() {
	setName("BorderPropertyEditor"); //$NON-NLS-1$
	setLayout(new GridBagLayout());
	setBackground(SystemColor.control);
	
	
	borderLabel.setBackground(SystemColor.control);
	
	GridBagConstraints constraintsBorderLabel = new GridBagConstraints();
	constraintsBorderLabel.gridx = 0; constraintsBorderLabel.gridy = 0;
	constraintsBorderLabel.insets = new Insets(2, 2, 0, 2);
	constraintsBorderLabel.anchor = GridBagConstraints.WEST;
	this.add(borderLabel, constraintsBorderLabel);
	
	GridBagConstraints constraintsBorderPane = new GridBagConstraints();
	constraintsBorderPane.gridx = 0; constraintsBorderPane.gridy = 1;
	constraintsBorderPane.fill = GridBagConstraints.BOTH;
	constraintsBorderPane.insets = new Insets(2, 2, 2, 2);
	constraintsBorderPane.weightx = 1.0;
	constraintsBorderPane.weighty = 1.0;
	constraintsBorderPane.anchor = GridBagConstraints.WEST;
	this.add(getBorderPane(), constraintsBorderPane);

	GridBagConstraints constraintsDetailPanel = new GridBagConstraints();
	constraintsDetailPanel.gridx = 1; constraintsDetailPanel.gridy = 1;
	constraintsDetailPanel.fill = GridBagConstraints.BOTH;
	constraintsDetailPanel.insets = new Insets(2, 2, 2, 2);
	constraintsDetailPanel.weightx = 1.0;
	this.add(getDetailPanel(), constraintsDetailPanel);
	
	GridBagConstraints constraintsBorderPreviewer = new GridBagConstraints();
	constraintsBorderPreviewer.gridx = 2; constraintsBorderPreviewer.gridy = 1;
	constraintsBorderPreviewer.fill = GridBagConstraints.BOTH;
	constraintsBorderPreviewer.insets = new Insets(2, 2, 2, 2);
	constraintsBorderPreviewer.weightx = 1.0;
	this.add(getBorderPreviewer(), constraintsBorderPreviewer);
	
	GridBagConstraints constraintsDisplayBorderName = new GridBagConstraints();
	constraintsDisplayBorderName.gridx = 0; constraintsDisplayBorderName.gridy = 2;
	constraintsDisplayBorderName.gridwidth = GridBagConstraints.REMAINDER; 
	constraintsDisplayBorderName.anchor = GridBagConstraints.WEST;
	constraintsDisplayBorderName.insets = new Insets(2, 2, 2, 2);
	displayBorderName.setForeground(Color.blue);
	this.add(displayBorderName, constraintsDisplayBorderName);

	getBorderList().addListSelectionListener(this);
		
}

public void setBorderValue(Border aBorder) {
	fBorder = aBorder;
	setPreviewBorder(fBorder);
	
	//show appropriate detail pane
	boolean findRightPage = false;
	for (int i = 0; i < fBorderModels.length; i++){
		IBorderPropertyPage aPage = fBorderModels[i].getPage();
		findRightPage = aPage.okToSetBorder(aBorder);
		if (findRightPage){
			getBorderList().setSelectedIndex(i);
			aPage.addPropertyChangeListener(borderPagePropertyListener);
			switchToPage((Component)aPage);
			break;
		}
	}

	//if nothing matches in the list, show blank detail pane
	if (! findRightPage){
		getBorderList().clearSelection();
		IBorderPropertyPage aPage = nullBorderListModel.getPage();
		aPage.addPropertyChangeListener(borderPagePropertyListener);
		switchToPage((Component)aPage);	
	}
}

public String getBorderInitializationString(){
	if (getBorderList().getSelectedIndex() == -1){
		IBorderPropertyPage aPage = new NoBorderSelectedPropertyPage();
		return aPage.getJavaInitializationString();
	}
	
 	BorderListModel aModel = (BorderListModel) (getBorderList().getSelectedValue());
	IBorderPropertyPage aPage = aModel.getPage();
 	return aPage.getJavaInitializationString();
}

public void valueChanged(ListSelectionEvent e){
	
	if (e.getSource() == getBorderList()){	
		if (getBorderList().getSelectedIndex() < 0){
			IBorderPropertyPage nullPage = new NoBorderSelectedPropertyPage();
			nullPage.addPropertyChangeListener(borderPagePropertyListener);
			switchToPage((Component)nullPage);
			fBorder = nullPage.getBorderValue();			
			setPreviewBorder(fBorder);
		}
		else{	
			BorderListModel aModel = (BorderListModel) (getBorderList().getSelectedValue());
			IBorderPropertyPage aPage = aModel.getPage();
			aPage.addPropertyChangeListener(borderPagePropertyListener);
			switchToPage((Component)aPage);
			fBorder = aPage.getBorderValue();			
			setPreviewBorder(fBorder);
		}
	}
}

public void switchToPage(Component aPage){
	CardLayout layout = (CardLayout)getDetailPanel().getLayout();
	layout.show(getDetailPanel(), aPage.getName());
}

private void setPreviewBorder(Border aBorder) {
	if (aBorder == null) return;
	if (displayButtonRemoved) {
		// reset in case this was removed from a previous ClassCastException
		borderPreviewer.add(displayButton, BorderLayout.CENTER);
		displayButtonRemoved = false;
	}
	displayButton.setBorder(aBorder);
	displayBorderName.setText(aBorder.getClass().getName());
}

/**
 * @see java.awt.Component#paint(Graphics)
 * Need to override to capture the ClassCastExceptions that are thrown
 * when the border is a special inner class border used by the component
 * type and cannot be applied as a border to the display button (a JButton).
 */
public void paint(Graphics g) {
	try {
		super.paint(g);
	} catch (ClassCastException e) {
		displayButtonRemoved = true;
		borderPreviewer.remove(displayButton);
		borderPreviewer.setBackground(borderPane.getBackground());
		this.repaint();
	}
}

}