package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.beans.*;
import java.text.MessageFormat;

public class ColorEditor extends PropertyEditorSupport {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	ColorPropertyEditor customEditor = null;
	Object fColor = null;
/**
 * Provides Description for the BeanFeatureRenderer.update().
 * Overrides the default by PropertyEditorSupport
 * Creation date: (10/25/00 6:24:51 PM)
 * @return java.lang.String
 */
public String getAsText() {
	if (customEditor != null) {
		fColor = customEditor.getColorValue();
	}
	if ( fColor == null ) return ""; //$NON-NLS-1$
	String basicName = BasicColorPropertyPage.getNameOfColor((Color)fColor);
	String systemName = SystemColorPropertyPage.getNameOfColor((Color)fColor);

	// Replaced the below two - inorder to get accurate description based on
	// Objects rather than on the intensity values.
	//basicName = BasicColorPropertyPage.getNameOfColor(((Color)fColor).getRed(), ((Color)fColor).getGreen(), ((Color)fColor).getBlue());
	//systemName = SystemColorPropertyPage.getNameOfColor(((Color)fColor).getRed(), ((Color)fColor).getGreen(), ((Color)fColor).getBlue());

	if(basicName!=null){
		return MessageFormat.format(VisualBeanInfoMessages.getString("ColorEditor.Color(String)"), new Object[]{basicName}); //$NON-NLS-1$
	}else
		if(systemName!=null){
			return MessageFormat.format(VisualBeanInfoMessages.getString("ColorEditor.Color(String)"), new Object[]{systemName}); //$NON-NLS-1$
		}else{
			return MessageFormat.format(VisualBeanInfoMessages.getString("ColorEditor.Colot(int,int,int)"),  //$NON-NLS-1$
											new Object[]{	new Integer(((Color)fColor).getRed()), 
															new Integer(((Color)fColor).getGreen()), 
															new Integer(((Color)fColor).getBlue())});
		}
}


public Component getCustomEditor() {
	if (customEditor == null) {
		Color aColor = (Color)getValue();
		if (aColor == null) {
			customEditor = new ColorPropertyEditor();
		} else {
			customEditor = new ColorPropertyEditor(aColor);
		}
	}	
	return customEditor;
}
public String getJavaInitializationString() {
	if (customEditor != null) {
		return customEditor.getColorInitializationString();
	} else {
		java.awt.Color value = (java.awt.Color) (super.getValue());
		if (value != null) {
			int r = value.getRed();
			int g = value.getGreen();
			int b = value.getBlue();
			return "new java.awt.Color(" + r + "," + g + "," + b + ")"; //$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		}
		return "new java.awt.Color(0, 0, 0)"; //$NON-NLS-1$
	}
}
public Object getValue() {

	if (customEditor != null) {
		return customEditor.getColorValue();
	} else {
		return super.getValue();
	}	
}
public boolean isPaintable() {
	return true;
}
public void paintValue(java.awt.Graphics gfx, java.awt.Rectangle box) {
	if (fColor != null) {
		gfx.drawRect( box.x, box.y, box.width, box.height );
		gfx.setColor( (Color) fColor );
		gfx.fillRect( box.x, box.y, box.width, box.height );
	}
}
public void setAsText(String text) throws IllegalArgumentException {
	throw new IllegalArgumentException(text);
}
public void setValue(Object newValue) {

	fColor = newValue;
	if (customEditor != null) {
		customEditor.setColorValue((Color)newValue);
	}	
	super.setValue(newValue);

}
public boolean supportsCustomEditor() {
	return true;
}
}
