package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class ColorPropertyEditor extends JPanel implements ItemListener, PropertyChangeListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private static java.util.ResourceBundle resabtedit = java.util.ResourceBundle.getBundle("com/ibm/etools/jbcf/visual/beaninfo/vceedit");  //$NON-NLS-1$
	private Color colorValue;
	
	private JRadioButton basicChoice = null;
	private JRadioButton systemChoice = null;
	private JRadioButton rgbChoice = null;
	private ButtonGroup buttonGroup = null;

	private JPanel pageHolder = null;
	private JPanel pageSelector = null;
	private FlowLayout pageSelectorFlowLayout = null;

	private BasicColorPropertyPage basicColorPage = null;
	private SystemColorPropertyPage systemColorPage = null;
	private RGBColorPropertyPage rgbColorPage = null;

	private JPanel colorPreviewer = null;
public ColorPropertyEditor() {
	super();
	this.initialize();
}
public ColorPropertyEditor(Color aColor) {
	super();
	this.initialize();
	setColorValue(aColor);
}
private void basicChoiceStateChanged( ) {
	switchToPage(getBasicColorPage());
	setPreviewerColor(getBasicColorPage().getColorValue());
	return;
}
private JRadioButton getBasicChoice() {
	if (basicChoice == null) {
		try {
			basicChoice = new JRadioButton();
			basicChoice.setBackground(SystemColor.control);
			basicChoice.setName("Basic");//$NON-NLS-1$
			basicChoice.setText(resabtedit.getString("Basic")); //$NON-NLS-1$
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return basicChoice;
}
private BasicColorPropertyPage getBasicColorPage() {
	if (basicColorPage == null) {
		try {
			basicColorPage = new BasicColorPropertyPage();
			basicColorPage.setName("BasicColorPage");//$NON-NLS-1$
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return basicColorPage;
}
private ButtonGroup getButtonGroup() {
	if (buttonGroup == null) {
		try {
			buttonGroup = new ButtonGroup();
			buttonGroup.add(getBasicChoice());
			buttonGroup.add(getSystemChoice());
			buttonGroup.add(getRGBChoice());
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return buttonGroup;
}
public String getColorInitializationString() {
	if (getSystemColorPage().isVisible()) {
		return getSystemColorPage().getJavaInitializationString();
	} else if (getBasicColorPage().isVisible()) {
		return getBasicColorPage().getJavaInitializationString();
	} else {
		return getRGBColorPage().getJavaInitializationString();
	}
}
private JPanel getColorPreviewer() {
	if (colorPreviewer == null) {
		try {
			colorPreviewer = new JPanel();
			colorPreviewer.setName("ColorPreviewer");//$NON-NLS-1$
			colorPreviewer.setLayout(new FlowLayout());
			colorPreviewer.setBorder(new EtchedBorder(
				EtchedBorder.RAISED,
				SystemColor.controlShadow,
				SystemColor.controlLtHighlight));
			colorPreviewer.setBackground(SystemColor.control);
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return colorPreviewer;
}
public Color getColorValue() {
	if (getSystemColorPage().isVisible()) {
		return getSystemColorPage().getColorValue();
	} else if (getBasicColorPage().isVisible()) {
		return getBasicColorPage().getColorValue();
	} else {
		return getRGBColorPage().getColorValue();
	}
}
private JPanel getPageHolder() {
	if (pageHolder == null) {
		try {
			pageHolder = new JPanel();
			pageHolder.setName("PageHolder");//$NON-NLS-1$
			pageHolder.setLayout(new CardLayout());
			pageHolder.setBackground(SystemColor.control);
			pageHolder.add(getBasicColorPage(), getBasicColorPage().getName());
			pageHolder.add(getSystemColorPage(), getSystemColorPage().getName());
			pageHolder.add(getRGBColorPage(), getRGBColorPage().getName());
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return pageHolder;
}
private JPanel getPageSelector() {
	if (pageSelector == null) {
		try {
			pageSelector = new JPanel();
			pageSelector.setName("PageSelector");//$NON-NLS-1$
			pageSelector.setLayout(getPageSelectorFlowLayout());
			pageSelector.setBackground(SystemColor.control);
			pageSelector.add(getBasicChoice(), getBasicChoice().getName());
			pageSelector.add(getSystemChoice(), getSystemChoice().getName());
			pageSelector.add(getRGBChoice(), getRGBChoice().getName());
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return pageSelector;
}
private FlowLayout getPageSelectorFlowLayout() {
	FlowLayout pageSelectorFlowLayout = null;
	try {
		/* Create part */
		pageSelectorFlowLayout = new FlowLayout();
		pageSelectorFlowLayout.setAlignment(0);
	} catch (Throwable ivjExc) {
		handleException(ivjExc);
	}
	return pageSelectorFlowLayout;
}
private JRadioButton getRGBChoice() {
	if (rgbChoice == null) {
		try {
			rgbChoice = new JRadioButton();
			rgbChoice.setBackground(SystemColor.control);
			rgbChoice.setName("RGB");//$NON-NLS-1$
			rgbChoice.setText(resabtedit.getString("RGB")); //$NON-NLS-1$
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return rgbChoice;
}
private RGBColorPropertyPage getRGBColorPage() {
	if (rgbColorPage == null) {
		try {
			rgbColorPage = new RGBColorPropertyPage();
			rgbColorPage.setName("RGBColorPage");//$NON-NLS-1$
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return rgbColorPage;
}
private JRadioButton getSystemChoice() {
	if (systemChoice == null) {
		try {
			systemChoice = new JRadioButton();
			systemChoice.setBackground(SystemColor.control);
			systemChoice.setName("System");//$NON-NLS-1$
			systemChoice.setText(resabtedit.getString("System")); //$NON-NLS-1$
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return systemChoice;
}
private SystemColorPropertyPage getSystemColorPage() {
	if (systemColorPage == null) {
		try {
			systemColorPage = new SystemColorPropertyPage();
			systemColorPage.setName(resabtedit.getString("SystemColorPage")); //$NON-NLS-1$
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return systemColorPage;
}
/**
 * Called whenever the part throws an exception.
 * @param exception java.lang.Throwable
 */
private void handleException(Throwable exception) {

	/* Uncomment the following lines to print uncaught exceptions to stdout */
	System.out.println("--------- UNCAUGHT EXCEPTION ---------");//$NON-NLS-1$
	exception.printStackTrace(System.out);
}
/**
 * Initializes connections
 */
private void initConnections() {
	
	getBasicChoice().addItemListener(this);
	getSystemChoice().addItemListener(this);
	getRGBChoice().addItemListener(this);
	
	getButtonGroup();
	
	getBasicColorPage().addPropertyChangeListener(this);
	getSystemColorPage().addPropertyChangeListener(this);
	getRGBColorPage().addPropertyChangeListener(this);
}
/**
 * Initialize the class.
 */
private void initialize() {
	setName("ColorPropertyEditor");//$NON-NLS-1$
	setLayout(new GridBagLayout());
	setBackground(SystemColor.control);
	
	initConnections();

	GridBagConstraints constraintsPageSelector = new GridBagConstraints();
	constraintsPageSelector.gridx = 0; constraintsPageSelector.gridy = 0;
	constraintsPageSelector.gridwidth = 1; constraintsPageSelector.gridheight = 1;
	constraintsPageSelector.fill = GridBagConstraints.HORIZONTAL;
	constraintsPageSelector.anchor = GridBagConstraints.WEST;
	constraintsPageSelector.weightx = 0.0;
	constraintsPageSelector.weighty = 0.0;
	this.add(getPageSelector(), constraintsPageSelector);

	GridBagConstraints constraintsPageHolder = new GridBagConstraints();
	constraintsPageHolder.gridx = 0; constraintsPageHolder.gridy = 1;
	constraintsPageHolder.gridwidth = 1; constraintsPageHolder.gridheight = 1;
	constraintsPageHolder.fill = GridBagConstraints.BOTH;
	constraintsPageHolder.anchor = GridBagConstraints.CENTER;
	constraintsPageHolder.weightx = 0.0;
	constraintsPageHolder.weighty = 1.0;
	this.add(getPageHolder(), constraintsPageHolder);

	GridBagConstraints constraintsColorPreviewer = new GridBagConstraints();
	constraintsColorPreviewer.gridx = 1; constraintsColorPreviewer.gridy = 1;
	constraintsColorPreviewer.gridwidth = 1; constraintsColorPreviewer.gridheight = 1;
	constraintsColorPreviewer.fill = GridBagConstraints.HORIZONTAL;
	constraintsColorPreviewer.anchor = GridBagConstraints.CENTER;
	constraintsColorPreviewer.weightx = 0.0;
	constraintsColorPreviewer.weighty = 1.0;
	constraintsColorPreviewer.ipadx = 70; 	constraintsColorPreviewer.ipady = 80;
	constraintsColorPreviewer.insets = new Insets(50, 35, 50, 35);
	this.add(getColorPreviewer(), constraintsColorPreviewer);


}
/**
 * Method to handle events for the ItemListener interface.
 * @param e java.awt.event.ItemEvent
 */
public void itemStateChanged(ItemEvent e) {
	if ((e.getSource() == getBasicChoice()) ) {
		basicChoiceStateChanged();
	}
	if ((e.getSource() == getSystemChoice()) ) {
		systemChoiceStateChanged();
	}
	if ((e.getSource() == getRGBChoice()) ) {
		rgbChoiceStateChanged();
	}
}
/**
 * Method to handle events for the PropertyChangeListener interface.
 * @param evt java.beans.PropertyChangeEvent
 */
public void propertyChange(PropertyChangeEvent evt) {
	if (evt.getPropertyName().equals("colorValue")) {//$NON-NLS-1$
		if ((evt.getSource() == getBasicColorPage()) && (getBasicColorPage().isVisible())) {
			setPreviewerColor(getBasicColorPage().getColorValue());
		}
		if ((evt.getSource() == getSystemColorPage()) && (getSystemColorPage().isVisible())) {
			setPreviewerColor(getSystemColorPage().getColorValue());
		}
		if ((evt.getSource() == getRGBColorPage()) && (getRGBColorPage().isVisible())) {
			setPreviewerColor(getRGBColorPage().getColorValue());
		}
	}
}
private void rgbChoiceStateChanged( ) {
	switchToPage(getRGBColorPage());
	setPreviewerColor(getRGBColorPage().getColorValue());
	return;
}
public void setColorValue(Color newColor) {
	colorValue = newColor;
	if ( colorValue == null ) {
		getBasicChoice().setSelected(true);
		getBasicColorPage().requestDefaultFocus();
	} else if (newColor instanceof SystemColor) {
		getSystemColorPage().preselectColor(newColor);
		getSystemChoice().setSelected(true);
		getSystemColorPage().requestDefaultFocus();		
	} else if (BasicColorPropertyPage.isBasicColor(newColor)) {
		getBasicColorPage().preselectColor(newColor);
		getBasicChoice().setSelected(true);
		getBasicColorPage().requestDefaultFocus();		
	} else {
		getRGBColorPage().preselectColor(newColor);
		getRGBChoice().setSelected(true);
		getRGBColorPage().requestDefaultFocus();		
	};
	setPreviewerColor(newColor);
}
private void setPreviewerColor(Color aColor) {
	getColorPreviewer().setBackground(aColor);
	getColorPreviewer().repaint();
}
private void switchToPage(Component aPage) {
	CardLayout layout = (CardLayout)getPageHolder().getLayout();
	layout.show(getPageHolder(),aPage.getName());
	((JPanel)aPage).requestDefaultFocus();
	return;
}
private void systemChoiceStateChanged( ) {
	switchToPage(getSystemColorPage());
	setPreviewerColor(getSystemColorPage().getColorValue());
	return;
}

public Dimension getPreferredSize(){
	return new Dimension(450,250);
}

}
