package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import javax.swing.border.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

/**
 * Insert the type's description here.
 * Creation date: (11/30/00 10:39:34 AM)
 * @author: Sri
 */
public class ColorPropertyEditorJT extends JPanel implements PropertyChangeListener, ChangeListener{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private Color colorValue = null;
	private JTabbedPane select = null;
	private JPanel colorPreviewer = null;
	BasicColorPropertyPage basic = null;
	SystemColorPropertyPage system = null;
	RGBColorPropertyPage rgb = null;
/**
 * ColorPropertyEditorJT constructor comment.
 */
public ColorPropertyEditorJT() {
	super();
	initialize();
}
	public ColorPropertyEditorJT(Color aColor){
		super();
		this.initialize();
		setColorValue(aColor);
	}
public String getColorInitializationString() {
	if( select == null )return null;
	Component got = select.getSelectedComponent();
	if( got instanceof BasicColorPropertyPage ) return ((BasicColorPropertyPage)got).getJavaInitializationString();
	if( got instanceof SystemColorPropertyPage ) return ((SystemColorPropertyPage)got).getJavaInitializationString();
	if( got instanceof RGBColorPropertyPage ) return ((RGBColorPropertyPage)got).getJavaInitializationString();
	return null;
}
private JPanel getColorPreviewer() {
	if(colorPreviewer == null){
		colorPreviewer = new JPanel();
		colorPreviewer.setName("ColorPreviewer");//$NON-NLS-1$
		colorPreviewer.setLayout(new FlowLayout());
		colorPreviewer.setBorder(new EtchedBorder(
			EtchedBorder.RAISED,
			SystemColor.controlShadow,
			SystemColor.controlLtHighlight
		));
		//colorPreviewer.setBorder(new EmptyBorder(20,20,20,20));
		colorPreviewer.setBackground(colorValue);
	}
	return colorPreviewer;
}
/**
 * Insert the method's description here.
 * Creation date: (11/30/00 10:45:09 AM)
 * @return java.awt.Color
 */
public Color getColorValue() {
	if( select == null )return null;
	Component got = select.getSelectedComponent();
	if( got instanceof BasicColorPropertyPage ) return ((BasicColorPropertyPage)got).getColorValue();
	if( got instanceof SystemColorPropertyPage ) return ((SystemColorPropertyPage)got).getColorValue();
	if( got instanceof RGBColorPropertyPage ) return ((RGBColorPropertyPage)got).getColorValue();
	return null;
}
	private void initConnections(){
		basic.addPropertyChangeListener(this);
		system.addPropertyChangeListener(this);
		rgb.addPropertyChangeListener(this);
		select.addChangeListener(this);
	}
	private void initialize(){
		setName("ColorPropertyEditor"); //$NON-NLS-1$
		setLayout( new GridLayout(1,2) );
		setBackground(SystemColor.control);
		select = new JTabbedPane();
		basic = new BasicColorPropertyPage();
		system = new SystemColorPropertyPage();
		rgb = new RGBColorPropertyPage();
		select.addTab(VisualBeanInfoMessages.getString("ColorPropertyEditor.Tab.Basic"), basic); //$NON-NLS-1$
		select.addTab(VisualBeanInfoMessages.getString("ColorPropertyEditor.Tab.System"), system); //$NON-NLS-1$
		select.addTab(VisualBeanInfoMessages.getString("ColorPropertyEditor.Tab.RGB"), rgb); //$NON-NLS-1$
		add( select );
		add( getColorPreviewer() );
		initConnections();
	}
	public void propertyChange(PropertyChangeEvent evt){
		Object src = evt.getSource();
		if(  src instanceof BasicColorPropertyPage && select.getSelectedIndex()==0){
			getColorPreviewer().setBackground( ((BasicColorPropertyPage)src).getColorValue() );
		}
		if(  src instanceof SystemColorPropertyPage && select.getSelectedIndex()==1){
			getColorPreviewer().setBackground( ((SystemColorPropertyPage)src).getColorValue() );
		}
		if(  src instanceof RGBColorPropertyPage && select.getSelectedIndex()==2){
			getColorPreviewer().setBackground( ((RGBColorPropertyPage)src).getColorValue() );
		}
	}
/**
 * Insert the method's description here.
 * Creation date: (11/30/00 10:43:29 AM)
 * @param aColor java.awt.Color
 */
public void setColorValue(java.awt.Color aColor) {
	colorValue = aColor;
	getColorPreviewer().setBackground(aColor);
	if( select != null ){
		if( colorValue instanceof SystemColor ){
			select.setSelectedIndex(1);
			((SystemColorPropertyPage)select.getSelectedComponent()).preselectColor(colorValue);
		}else{
			if( BasicColorPropertyPage.isBasicColor(colorValue) ){
				select.setSelectedIndex(0);
				((BasicColorPropertyPage)select.getSelectedComponent()).preselectColor(colorValue);
			}else{
				select.setSelectedIndex(2);
				((RGBColorPropertyPage)select.getSelectedComponent()).preselectColor(colorValue);;
			}
		}
	}
}
	public void stateChanged(javax.swing.event.ChangeEvent evt){;
		getColorPreviewer().setBackground( colorValue );
	}
}
