package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.awt.event.*;
import java.text.MessageFormat;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;


public class EmptyBorderPropertyPage extends JPanel implements IBorderPropertyPage, DocumentListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private JLabel topLabel = new JLabel(VisualBeanInfoMessages.getString("Top")); //$NON-NLS-1$
	private JTextField topField = new JTextField();
	private JLabel leftLabel = new JLabel(VisualBeanInfoMessages.getString("Left")); //$NON-NLS-1$
	private JTextField leftField = new JTextField();
	private JLabel bottomLabel = new JLabel(VisualBeanInfoMessages.getString("Bottom")); //$NON-NLS-1$
	private JTextField bottomField = new JTextField();
	private JLabel rightLabel = new JLabel(VisualBeanInfoMessages.getString("Right")); //$NON-NLS-1$
	private JTextField rightField = new JTextField();
	
		                                             
public EmptyBorderPropertyPage(){
	super();  
	initialize();
}

public String getName(){
	return "EmptyBorderPropertyPage"; //$NON-NLS-1$
}

public void initialize(){
	this.setName("MatteBorderPropertyPage"); //$NON-NLS-1$
	this.setBackground(SystemColor.control);
	setLayout(new GridLayout(16, 1, 0, 0));
	add(topLabel);
	add(topField);
	topField.getDocument().addDocumentListener(this);
	add(leftLabel);
	add(leftField);
	leftField.getDocument().addDocumentListener(this);
	add(bottomLabel);
	add(bottomField);
	bottomField.getDocument().addDocumentListener(this);
	add(rightLabel);
	add(rightField);
	rightField.getDocument().addDocumentListener(this);
}
protected int getIntValue(String aString){
	try {
		return Integer.parseInt(aString);
	} catch ( NumberFormatException nfexc ) {
		return 0;
	}	
}

public int getTop(){
	return getIntValue(topField.getText());
}

public int getLeft(){
	return getIntValue(leftField.getText());
}

public int getBottom(){
	return getIntValue(bottomField.getText());
}

public int getRight(){
	return getIntValue(rightField.getText());
}

public Border getBorderValue(){
	Border aBorder = BorderFactory.createEmptyBorder(getTop(), getLeft(), getBottom(), getRight());
	return aBorder;
}

public void updateHandle(DocumentEvent e){
	if (e.getDocument() == topField.getDocument() || e.getDocument() == leftField.getDocument() || 
	e.getDocument() == bottomField.getDocument() || e.getDocument() == rightField.getDocument()){
		firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
	}
}

public void insertUpdate(DocumentEvent e){
	updateHandle(e);
}

public void removeUpdate(DocumentEvent e){
	updateHandle(e);
}

public void changedUpdate(DocumentEvent e){
	updateHandle(e);
}

public String getDisplayName(){
	return MessageFormat.format(VisualBeanInfoMessages.getString("EmptyBorder.Empty(top,left,bottom,right)"), //$NON-NLS-1$
								new Object[]{ 	new Integer(getTop()), 
												new Integer(getLeft()), 
												new Integer(getBottom()),
												new Integer(getRight())});
}

public String getJavaInitializationString(){
	return "javax.swing.BorderFactory.createEmptyBorder(" + getTop() + "," + getLeft() + //$NON-NLS-1$ //$NON-NLS-2$
	            "," + getBottom() + "," + getRight() + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
}

public boolean okToSetBorder(Border aBorder){
	if (aBorder instanceof EmptyBorder && !(aBorder instanceof MatteBorder)){
		Insets fInsets = ((EmptyBorder) aBorder).getBorderInsets();
		topField.setText(String.valueOf(fInsets.top));
		leftField.setText(String.valueOf(fInsets.left));
		bottomField.setText(String.valueOf(fInsets.bottom));
		rightField.setText(String.valueOf(fInsets.right));
		return true;
	}
	
	return false;
}

}
