package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class EtchedBorderPropertyPage extends JPanel implements IBorderPropertyPage, ItemListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private JRadioButton raisedButton = null;
	private JRadioButton loweredButton = null;
	private ButtonGroup buttonGroup = null;
	private int etchedType = EtchedBorder.RAISED;
	
public EtchedBorderPropertyPage(){
	super();  
	initialize();
}

public String getDisplayName(){
	if ( getEtchedType() == EtchedBorder.LOWERED ){
		return VisualBeanInfoMessages.getString("EtchedBorder.DisplayName.Lowered"); //$NON-NLS-1$
	} else {
		return VisualBeanInfoMessages.getString("EtchedBorder.DisplayName.Raised"); //$NON-NLS-1$
	}
}

public String getName(){
	return "EtchedBorderPropertyPage"; //$NON-NLS-1$
}

public int getEtchedType(){
	if (getRaisedButton().isSelected() == true){
		etchedType = EtchedBorder.RAISED;
	}
	else if (getLoweredButton().isSelected() == true){
		etchedType = EtchedBorder.LOWERED;
	}
	return etchedType;
}

public JRadioButton getRaisedButton(){
	if(raisedButton == null){
		try{
			raisedButton = new JRadioButton(VisualBeanInfoMessages.getString("EtchedBorder.radio.Raised.Text")); //$NON-NLS-1$
			raisedButton.setBackground(SystemColor.control);
			raisedButton.setName("Raised Etched"); //$NON-NLS-1$
			
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return raisedButton;
}

public JRadioButton getLoweredButton(){
	if (loweredButton == null){
		try{
			loweredButton = new JRadioButton(VisualBeanInfoMessages.getString("EtchedBorder.radio.Lowered.Text")); //$NON-NLS-1$
			loweredButton.setBackground(SystemColor.control);
			loweredButton.setName("Lowered Etched"); //$NON-NLS-1$
		}catch(Throwable e){
			e.printStackTrace();
		}
	
	}
	return loweredButton;
}

public ButtonGroup getButtonGroup(){
	if(buttonGroup == null){
		try{
			buttonGroup = new ButtonGroup();
			buttonGroup.add(getRaisedButton());
			buttonGroup.add(getLoweredButton());
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return buttonGroup;
}

public void initialize(){
	this.setName("EtchedBorderPropertyPage"); //$NON-NLS-1$
	this.setBackground(SystemColor.control);
	this.setLayout(new GridLayout(8, 1, 0, 0));
	getButtonGroup();
	this.add(getRaisedButton(), getRaisedButton().getName());
	this.add(getLoweredButton(), getLoweredButton().getName());
	getRaisedButton().addItemListener(this);
	getRaisedButton().setSelected(true);
	getLoweredButton().addItemListener(this);
}

public Border getBorderValue(){
	Border aBorder = BorderFactory.createEtchedBorder(getEtchedType());
	return aBorder;
}

public void itemStateChanged(ItemEvent e){
	if(e.getSource() == getRaisedButton()){
		etchedType = EtchedBorder.RAISED;
	}
	else if(e.getSource() == getLoweredButton()){
		etchedType = EtchedBorder.LOWERED;
	}
	firePropertyChange("borderValueChanged",null,getBorderValue()); //$NON-NLS-1$
}

public boolean okToSetBorder(Border aBorder){
	if (aBorder instanceof EtchedBorder){
		etchedType = ((EtchedBorder)aBorder).getEtchType();
		if (etchedType == EtchedBorder.RAISED){
			raisedButton.setSelected(true);
		}
		else if (etchedType == EtchedBorder.LOWERED){
			loweredButton.setSelected(true);
		}
		return true;
	}
	
	return false;
}

public String getJavaInitializationString(){
	if (getEtchedType() == EtchedBorder.LOWERED){
		return "javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED)"; //$NON-NLS-1$
	}
	
	//default is Raised EtchedBorder
	return "javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED)"; //$NON-NLS-1$
}


}