package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import java.text.MessageFormat;

public class LineBorderPropertyPage extends JPanel implements IBorderPropertyPage, DocumentListener, ListSelectionListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private static java.util.ResourceBundle resabtedit = java.util.ResourceBundle.getBundle("com/ibm/etools/jbcf/visual/beaninfo/vceedit");  //$NON-NLS-1$
	public static String[] colorNames =
		{resabtedit.getString("black"), resabtedit.getString("blue"), resabtedit.getString("cyan"), resabtedit.getString("darkGray"), resabtedit.getString("gray"), resabtedit.getString("green"), resabtedit.getString("lightGray"), //$NON-NLS-7$ //$NON-NLS-6$ //$NON-NLS-5$ //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("magenta"), resabtedit.getString("orange"), resabtedit.getString("pink"), resabtedit.getString("red"), resabtedit.getString("white"), resabtedit.getString("yellow")}; //$NON-NLS-6$ //$NON-NLS-5$ //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
	public static Color[] colorValues =
		{Color.black, Color.blue, Color.cyan, Color.darkGray, Color.gray, Color.green, Color.lightGray, 
		Color.magenta, Color.orange, Color.pink, Color.red, Color.white, Color.yellow};
	public static String[] initStrings =
		{"black", "blue", "cyan", "darkGray", "gray", "green", "lightGray",//$NON-NLS-7$//$NON-NLS-6$//$NON-NLS-5$//$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"magenta", "orange", "pink", "red", "white", "yellow"};//$NON-NLS-6$//$NON-NLS-5$//$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$

	private JLabel widthLabel = new JLabel(VisualBeanInfoMessages.getString("LineBorder.Width.Label.Text")); //$NON-NLS-1$
	private JTextField widthField = new JTextField();
	private JList colorList = null;
	private JScrollPane colorPane = null;
	private JLabel colorLabel = new JLabel(VisualBeanInfoMessages.getString("LineBorder.Color.Label.text")); //$NON-NLS-1$
		
public LineBorderPropertyPage(){
	super();
	initialize();
}

public String getName(){
	return "LineBorderPropertyPage"; //$NON-NLS-1$
}

public String getDisplayName(){
	int index = getColorList().getSelectedIndex();
	String color = ((index<0)||(index>initStrings.length))?"":initStrings[getColorList().getSelectedIndex()];//$NON-NLS-1$
	return MessageFormat.format(VisualBeanInfoMessages.getString("LineBorder.DisplayName(width,Color)"), new Object[]{ new Integer(getBorderWidth()), color}); //$NON-NLS-1$
}

protected String getBorderColorJavaInitializationString(){
	int i = getColorList().getSelectedIndex();
	if (i >= 0 && i <  initStrings.length){
		return "java.awt.Color." +  initStrings[i]; //$NON-NLS-1$
	}
	else return "java.awt.Color." +  initStrings[0]; //$NON-NLS-1$
}

public String getJavaInitializationString(){
	return "javax.swing.BorderFactory.createLineBorder(" + getBorderColorJavaInitializationString() + "," + getBorderWidth() + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
}

public JList getColorList(){
	if (colorList== null){
		try{
			colorList = new JList(colorValues);
			colorList.setCellRenderer(new ColorCellRenderer(colorNames, colorValues));
		}catch(Throwable ivjExc){
			handleException(ivjExc);
		}
	}
	return colorList;
}

public JScrollPane getColorPane(){
	if(colorPane == null){
		try{
			colorPane = new JScrollPane(getColorList());
		}catch(Throwable ivjExc){
			handleException(ivjExc);
		}
	}
	return colorPane;
}

public void handleException(Throwable exception){
	exception.printStackTrace();
}

public void initialize(){
	setName("LineBorderPropertypage"); //$NON-NLS-1$
	setLayout(new BorderLayout());
	setBackground(SystemColor.control);
	JPanel p1 = new JPanel();
	p1.setBackground(SystemColor.control);
	p1.setLayout(new BorderLayout());
	p1.add(widthLabel, "West"); //$NON-NLS-1$
	p1.add(widthField, "Center"); //$NON-NLS-1$
	widthField.getDocument().addDocumentListener(this);
	//widthField.setEditable(true);
	this.add(p1, "North"); //$NON-NLS-1$
	JPanel p2 = new JPanel();
	p2.setLayout(new BorderLayout());
	p2.add(colorLabel, "North"); //$NON-NLS-1$
	p2.setBackground(SystemColor.control);
	p2.add(getColorPane(), "Center"); //$NON-NLS-1$
	this.add(p2, "Center"); //$NON-NLS-1$
	
	getColorList().addListSelectionListener(this);
}

public int getBorderWidth(){
	try {
		return Integer.parseInt(widthField.getText());
	} catch ( NumberFormatException nfexc ) {
		return 0;
	}		
}

public Color getBorderColor(){
	int index = getColorList().getSelectedIndex();
	if (index < 0) {
		return colorValues[0];
	} else {
		return colorValues[index];
	}
}

public Border getBorderValue(){
	Border aLineBorder = BorderFactory.createLineBorder(getBorderColor(), getBorderWidth());
	return aLineBorder;
	
}

public void valueChanged(ListSelectionEvent e){
	if (e.getSource() == getColorList()){
		// Fire a property change event
		firePropertyChange("borderValueChanged",null,getBorderValue()); //$NON-NLS-1$
	}
}

public void updateHandle(DocumentEvent e){
	if (e.getDocument() == widthField.getDocument()){
		firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
	}
}

public void insertUpdate(DocumentEvent e){
	updateHandle(e);
}

public void removeUpdate(DocumentEvent e){
	updateHandle(e);
}

public void changedUpdate(DocumentEvent e){
	updateHandle(e);
}

public boolean okToSetBorder(Border aBorder){
	if (aBorder instanceof LineBorder){
		LineBorder fBorder = (LineBorder) aBorder;
		widthField.setText(String.valueOf(fBorder.getThickness()));
		for (int i = 0; i < colorValues.length; i++){
			if (colorValues[i].equals(fBorder.getLineColor())){
				colorList.setSelectedIndex(i);
			}
		}
		return true;
	}
	
	return false;
}

}

