package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.awt.event.*;
import java.text.MessageFormat;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class MatteBorderPropertyPage extends JPanel implements IBorderPropertyPage,DocumentListener, ListSelectionListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private static java.util.ResourceBundle resabtedit = java.util.ResourceBundle.getBundle("com/ibm/etools/jbcf/visual/beaninfo/vceedit");  //$NON-NLS-1$
	public static String[] colorNames =
		{resabtedit.getString("black"), resabtedit.getString("blue"), resabtedit.getString("cyan"), resabtedit.getString("darkGray"), resabtedit.getString("gray"), resabtedit.getString("green"), resabtedit.getString("lightGray"), //$NON-NLS-7$ //$NON-NLS-6$ //$NON-NLS-5$ //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("magenta"), resabtedit.getString("orange"), resabtedit.getString("pink"), resabtedit.getString("red"), resabtedit.getString("white"), resabtedit.getString("yellow")}; //$NON-NLS-6$ //$NON-NLS-5$ //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
	public static Color[] colorValues =
		{Color.black, Color.blue, Color.cyan, Color.darkGray, Color.gray, Color.green, Color.lightGray, 
		Color.magenta, Color.orange, Color.pink, Color.red, Color.white, Color.yellow};
	public static String[] initStrings =
		{"black", "blue", "cyan", "darkGray", "gray", "green", "lightGray",//$NON-NLS-7$//$NON-NLS-6$//$NON-NLS-5$//$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"magenta", "orange", "pink", "red", "white", "yellow"};//$NON-NLS-6$//$NON-NLS-5$//$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$

	private JLabel topLabel = new JLabel(VisualBeanInfoMessages.getString("Top")); //$NON-NLS-1$
	private JTextField topField = new JTextField();
	private JLabel leftLabel = new JLabel(VisualBeanInfoMessages.getString("Left")); //$NON-NLS-1$
	private JTextField leftField = new JTextField();
	private JLabel bottomLabel = new JLabel(VisualBeanInfoMessages.getString("Bottom")); //$NON-NLS-1$
	private JTextField bottomField = new JTextField();
	private JLabel rightLabel = new JLabel(VisualBeanInfoMessages.getString("Right")); //$NON-NLS-1$
	private JTextField rightField = new JTextField();
	
	private JList colorList = null;
	private JScrollPane colorPane = null;
	private JLabel colorLabel = new JLabel(VisualBeanInfoMessages.getString("MatteBorder.BorderColor")); //$NON-NLS-1$
	                                             
public MatteBorderPropertyPage(){
	super();  
	initialize();
}

public String getName(){
	return "MatteBorderPropertyPage"; //$NON-NLS-1$
}

public String getDisplayName(){
	
	String colorString = initStrings[getColorList().getSelectedIndex()];	
	return MessageFormat.format(VisualBeanInfoMessages.getString("MatteBorder.Matte(Color,top,left,bottom,right)"),  //$NON-NLS-1$
								new Object[]{colorString,
											 new Integer(getTop()),
											 new Integer(getLeft()),
											 new Integer(getBottom()),
											 new Integer(getRight())});
}

public JList getColorList(){
	if(colorList == null){
		try{
			colorList = new JList( colorValues);
			colorList.setCellRenderer(new ColorCellRenderer( colorNames,  colorValues));
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return colorList;
}

public JScrollPane getColorPane(){
	if (colorPane == null){
		try{
			colorPane = new JScrollPane(getColorList());
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return colorPane;
}

public void initialize(){
	this.setName("MatteBorderPropertyPage"); //$NON-NLS-1$
	this.setBackground(SystemColor.control);
	this.setLayout(new BorderLayout());
	
	JPanel p1 = new JPanel();
	p1.setBackground(SystemColor.control);
	p1.setLayout(new GridLayout(4, 2, 0, 0));
	p1.add(topLabel);
	p1.add(topField);
	p1.add(leftLabel);
	p1.add(leftField);
	p1.add(bottomLabel);
	p1.add(bottomField);
	p1.add(rightLabel);
	p1.add(rightField);
	this.add(p1, "North"); //$NON-NLS-1$
	
	topField.getDocument().addDocumentListener(this);
	leftField.getDocument().addDocumentListener(this);
	bottomField.getDocument().addDocumentListener(this);
	rightField.getDocument().addDocumentListener(this);
	
	JPanel p2 = new JPanel();
	p2.setLayout(new BorderLayout());
	p2.setBackground(SystemColor.control);
	p2.add(colorLabel, "North"); //$NON-NLS-1$
	p2.add(getColorPane(), "Center"); //$NON-NLS-1$
	this.add(p2, "Center"); //$NON-NLS-1$
	getColorList().addListSelectionListener(this);
}

protected int getIntValue(JTextField aTextField){
	try {
		return Integer.parseInt(aTextField.getText());
	} catch ( NumberFormatException nfexc ) {
		return 0;
	}	
}

public int getTop(){
	return getIntValue(topField);
}

public int getLeft(){
	return getIntValue(leftField);
}

public int getBottom(){
	return getIntValue(bottomField);
}

public int getRight(){
	return getIntValue(rightField);
}

public Color getColor(){
	int i = getColorList().getSelectedIndex();
	if (i >= 0 && i <  colorNames.length){
		return  colorValues[i];
	}
	else return  colorValues[0];
}

public Border getBorderValue(){
	Border aBorder = BorderFactory.createMatteBorder(getTop(), getLeft(), getBottom(), getRight(), getColor());
	return aBorder;
}

public void valueChanged(ListSelectionEvent e){
	if (e.getSource() == getColorList()){
		firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
	}
}

public void updateHandle(DocumentEvent e){
	if (e.getDocument() == topField.getDocument() || e.getDocument() == leftField.getDocument() || 
	e.getDocument() == bottomField.getDocument() || e.getDocument() == rightField.getDocument()){
		firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
	}
}

public void insertUpdate(DocumentEvent e){
	updateHandle(e);
}

public void removeUpdate(DocumentEvent e){
	updateHandle(e);
}

public void changedUpdate(DocumentEvent e){
	updateHandle(e);
}

public boolean okToSetBorder(Border aBorder){
	if (aBorder instanceof MatteBorder){
		MatteBorder fBorder = (MatteBorder) aBorder;
		Insets fInsets = fBorder.getBorderInsets();
		topField.setText(String.valueOf(fInsets.top));
		leftField.setText(String.valueOf(fInsets.left));
		bottomField.setText(String.valueOf(fInsets.bottom));
		rightField.setText(String.valueOf(fInsets.right));
	
		for (int i = 0; i <  colorValues.length; i++){
			if ( colorValues[i].equals(fBorder.getMatteColor())){
				getColorList().setSelectedIndex(i);
			}
		}
		return true;
	}
	return false;

}

protected String getBorderColorJavaInitializationString(){
	int i = getColorList().getSelectedIndex();
	if (i >= 0 && i <  initStrings.length){
		return "java.awt.Color." +  initStrings[i]; //$NON-NLS-1$
	}
	else return "java.awt.Color." +  initStrings[0]; //$NON-NLS-1$
}

public String getJavaInitializationString(){
	return "javax.swing.BorderFactory.createMatteBorder(" + getTop() + "," + getLeft() + "," + getBottom() + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			"," + getRight() + "," + getBorderColorJavaInitializationString() + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
}


}