package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.lang.Integer;


public class SoftBevelBorderPropertyPage extends JPanel implements IBorderPropertyPage, ItemListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private JRadioButton raisedButton = null;
	private JRadioButton loweredButton = null;
	private ButtonGroup buttonGroup = null;
	private int bevelType = SoftBevelBorder.RAISED;
	
public SoftBevelBorderPropertyPage(){
	super();  
	initialize();
}

public String getName(){
	return "SoftBevelBorderPropertyPage"; //$NON-NLS-1$
}

public int getBevelType(){
	if (getRaisedButton().isSelected() == true){
		bevelType = SoftBevelBorder.RAISED;
	}
	else if (getLoweredButton().isSelected() == true){
		bevelType = SoftBevelBorder.LOWERED;
	}
	return bevelType;
}

public JRadioButton getRaisedButton(){
	if(raisedButton == null){
		try{
			raisedButton = new JRadioButton(VisualBeanInfoMessages.getString("SoftBevelBorder.radio.Raised.Text")); //$NON-NLS-1$
			raisedButton.setBackground(SystemColor.control);
			raisedButton.setName("Raised Bevel"); //$NON-NLS-1$
			
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return raisedButton;
}

public JRadioButton getLoweredButton(){
	if (loweredButton == null){
		try{
			loweredButton = new JRadioButton(VisualBeanInfoMessages.getString("SoftBevelBorder.radio.Lowered.Text")); //$NON-NLS-1$
			loweredButton.setBackground(SystemColor.control);
			loweredButton.setName("Lowered Bevel"); //$NON-NLS-1$
		}catch(Throwable e){
			e.printStackTrace();
		}
	
	}
	return loweredButton;
}

public ButtonGroup getButtonGroup(){
	if(buttonGroup == null){
		try{
			buttonGroup = new ButtonGroup();
			buttonGroup.add(getRaisedButton());
			buttonGroup.add(getLoweredButton());
		}catch(Throwable e){
			e.printStackTrace();
		}
	}
	return buttonGroup;
}

public void initialize(){
	this.setName("BevelBorderPropertyPage"); //$NON-NLS-1$
	this.setBackground(SystemColor.control);
	this.setLayout(new GridLayout(8, 1, 0, 0));
	getButtonGroup();
	this.add(getRaisedButton(), getRaisedButton().getName());
	this.add(getLoweredButton(), getLoweredButton().getName());
	getRaisedButton().addItemListener(this);
	getRaisedButton().setSelected(true);
	getLoweredButton().addItemListener(this);
}

public Border getBorderValue(){
	Border aBorder = new SoftBevelBorder(bevelType);
	return aBorder;
}

public void itemStateChanged(ItemEvent e){
	if(e.getSource() == getRaisedButton()){
		bevelType = SoftBevelBorder.RAISED;
	}
	else if(e.getSource() == getLoweredButton()){
		bevelType = SoftBevelBorder.LOWERED;
	}
	
	firePropertyChange("borderValueChanged",null,getBorderValue()); //$NON-NLS-1$
}

public boolean okToSetBorder(Border aBorder){
	if (aBorder instanceof SoftBevelBorder){
		bevelType = ((SoftBevelBorder)aBorder).getBevelType();
		if (bevelType == SoftBevelBorder.RAISED){
			raisedButton.setSelected(true);
		}
		else if (bevelType == SoftBevelBorder.LOWERED){
			loweredButton.setSelected(true);
		}
		return true;
	}
	return false;

}

public String getDisplayName(){
	if ( getBevelType() == SoftBevelBorder.LOWERED ){
		return VisualBeanInfoMessages.getString("SoftBevelBorder.Lowered.DisplayName"); //$NON-NLS-1$
	} else {
		return VisualBeanInfoMessages.getString("SoftBevelBorder.Raised.DisplayName"); //$NON-NLS-1$
	}
}

public String getJavaInitializationString(){
	if (getBevelType() == SoftBevelBorder.LOWERED){
		return "new javax.swing.border.SoftBevelBorder(javax.swing.border.SoftBevelBorder.LOWERED)"; //$NON-NLS-1$
	}
	
	//default is Raised SoftBevelBorder
	return "new javax.swing.border.SoftBevelBorder(javax.swing.border.SoftBevelBorder.RAISED)"; //$NON-NLS-1$
} 

}
