package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class SystemColorPropertyPage extends JPanel implements ListSelectionListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private static java.util.ResourceBundle resabtedit = java.util.ResourceBundle.getBundle("com/ibm/etools/jbcf/visual/beaninfo/vceedit");  //$NON-NLS-1$
	static String[] colorNames =
		{resabtedit.getString("activeCaption"), resabtedit.getString("activeCaptionBorder"), resabtedit.getString("activeCaptionText"),  //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("control"), resabtedit.getString("controlDkShadow"), resabtedit.getString("controlHighlight"),  //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("controlLtHighlight"), resabtedit.getString("controlShadow"), resabtedit.getString("controlText"),  //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("desktop"), resabtedit.getString("inactiveCaption"), resabtedit.getString("inactiveCaptionBord"), //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("inactiveCaptionText"), resabtedit.getString("info"), resabtedit.getString("infoText"), resabtedit.getString("menu"),  //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("menuText"), resabtedit.getString("scrollbar"), resabtedit.getString("text"), resabtedit.getString("textHighlight"),  //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("textHighlightText"), resabtedit.getString("textInactiveText"), resabtedit.getString("textText"),  //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
		resabtedit.getString("window"), resabtedit.getString("windowBorder"), resabtedit.getString("windowText")}; //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$
	static SystemColor[] colorValues =
		{SystemColor.activeCaption, SystemColor.activeCaptionBorder, SystemColor.activeCaptionText, 
		SystemColor.control, SystemColor.controlDkShadow, SystemColor.controlHighlight, 
		SystemColor.controlLtHighlight, SystemColor.controlShadow, SystemColor.controlText, 
		SystemColor.desktop, SystemColor.inactiveCaption, SystemColor.inactiveCaptionBorder,
		SystemColor.inactiveCaptionText, SystemColor.info, SystemColor.infoText, SystemColor.menu, 
		SystemColor.menuText, SystemColor.scrollbar, SystemColor.text, SystemColor.textHighlight, 
		SystemColor.textHighlightText, SystemColor.textInactiveText, SystemColor.textText, 
		SystemColor.window, SystemColor.windowBorder, SystemColor.windowText};
	static String[] initStrings =
		{"activeCaption", "activeCaptionBorder", "activeCaptionText", //$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"control", "controlDkShadow", "controlHighlight", //$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"controlLtHighlight", "controlShadow", "controlText", //$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"desktop", "inactiveCaption", "inactiveCaptionBorder",//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"inactiveCaptionText", "info", "infoText", "menu", //$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"menuText", "scrollbar", "text", "textHighlight", //$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"textHighlightText", "textInactiveText", "textText", //$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		"window", "windowBorder", "windowText"};//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
	
	private JScrollPane scrollPane = null;
	private JList systemColorsList = null;
	protected transient PropertyChangeSupport propertyChange = new PropertyChangeSupport(this);
	
public SystemColorPropertyPage() {
	super();
	initialize();
}
/**
 * The addPropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
	propertyChange.addPropertyChangeListener(listener);
}
private void fillColorList ( ) {
	DefaultListModel model = (DefaultListModel)getSystemColorsList().getModel();
	for(int i=0; i < colorNames.length; i++) {
		model.addElement(new Integer(i));
	}
	return;
}
/**
 * The firePropertyChange method was generated to support the propertyChange field.
 */
public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
	if (propertyChange == null){
		propertyChange = new PropertyChangeSupport(this);
	}
	propertyChange.firePropertyChange(propertyName, oldValue, newValue);
}
public Color getColorValue() {
	return (Color)getValue();
}
String getJavaInitializationString ( ) {
	int index = indexOfSelectedColor();
	return "java.awt.SystemColor." + initStrings[index];//$NON-NLS-1$
}

public static String getNameOfColor(int r, int g, int b){
		for(int i=0; i<colorValues.length; i++){
			if( (colorValues[i].getRed() == r) && (colorValues[i].getGreen() == g) && (colorValues[i].getBlue() == b) ){
				return(colorNames[i]);
			}
		}
		return null;
	}
	
public static String getNameOfColor(Color c){
		for(int i=0; i<colorValues.length; i++){
			if( c.equals(colorValues[i]) ){
				return(colorNames[i]);
			}
		}
		return null;
	}
	
private JScrollPane getScrollPane() {
	if (scrollPane == null) {
		try {
			scrollPane = new JScrollPane();
			scrollPane.setName("ScrollPanel");//$NON-NLS-1$
			scrollPane.setViewportView(getSystemColorsList());
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	};
	return scrollPane;
}

private JList getSystemColorsList() {
	if (systemColorsList == null) {
		try {
			systemColorsList = new JList();
			systemColorsList.setName("SystemColorsList");//$NON-NLS-1$
			systemColorsList.setModel(new DefaultListModel());
			systemColorsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			systemColorsList.setCellRenderer(new ColorCellRenderer(colorNames,colorValues));
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return systemColorsList;
}
Object getValue() {
	int index = indexOfSelectedColor();
	if (index < 0) {
		return null;
	} else {
		return colorValues[index];
	}
}
/**
 * Called whenever the part throws an exception.
 * @param exception java.lang.Throwable
 */
private void handleException(Throwable exception) {

	/* Uncomment the following lines to print uncaught exceptions to stdout */
	// System.out.println("--------- UNCAUGHT EXCEPTION ---------");
	// exception.printStackTrace(System.out);
}
private int indexOfSelectedColor() {
	return getSystemColorsList().getSelectedIndex();
}
/**
 * Initializes connections
 */
private void initConnections() {
	getSystemColorsList().addListSelectionListener(this);
}
/**
 * Initialize the class.
 */
private void initialize() {
	setName("SystemColorPropertyPage");//$NON-NLS-1$
	setLayout(new BorderLayout());
	setBackground(SystemColor.control);
	setSize(179, 181);
	this.add(getScrollPane());
	fillColorList();
	initConnections();
}

public static boolean isSystemColor(Color aColor) {
	for (int i=0; i<colorValues.length; i++) {
		if (aColor.equals(colorValues[i])) {
			return true;
		}
	}
	return false;
}

public void preselectColor(Color aColor) {
	for (int i=0; i<colorValues.length; i++) {
		if (aColor.equals(colorValues[i])) {
			getSystemColorsList().setSelectedIndex(i);
			getSystemColorsList().ensureIndexIsVisible(i);
			return;
		}
	}
	getSystemColorsList().setSelectedIndex(0);
	getSystemColorsList().ensureIndexIsVisible(0);
	return;
}
/**
 * The removePropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
	propertyChange.removePropertyChangeListener(listener);
}
public void setColorValue(Color colorValue) {
	// I may use this in the future in order to convert
	// the color value from another page into a System
	// color, but for now, do nothing.
	return;
}
private void systemColorsListValueChanged(ListSelectionEvent arg1) {
	firePropertyChange("colorValue", null, getColorValue());//$NON-NLS-1$
}
public void valueChanged(ListSelectionEvent e) {
	if ((e.getSource() == getSystemColorsList()) ) {
		systemColorsListValueChanged(e);
	}
}
}
