package com.ibm.etools.jbcf.visual.beaninfo;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.*;
import java.awt.event.*;
import java.text.MessageFormat;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class TitledBorderPropertyPage extends JPanel implements IBorderPropertyPage, DocumentListener, ActionListener, ItemListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$

	// Change this variable to enable/disable editing of TitledBorder's Border property.
	private static final boolean SUB_BORDER_ENABLED = false;

	private int titleJustification = TitledBorder.DEFAULT_JUSTIFICATION;
	private int titlePosition = TitledBorder.DEFAULT_POSITION;

	private JLabel titleLabel = new JLabel(VisualBeanInfoMessages.getString("TitleBorder.Title")); //$NON-NLS-1$
	private JLabel titleJustificationLabel = new JLabel(VisualBeanInfoMessages.getString("TitledBorder.Title.TitleJustification")); //$NON-NLS-1$
	private JLabel titlePositionLabel = new JLabel(VisualBeanInfoMessages.getString("TitledBorder.Title.TitlePosition")); //$NON-NLS-1$
	
	private JLabel colorLabel = new JLabel(VisualBeanInfoMessages.getString("TitledBorder.TitleColor")); //$NON-NLS-1$

	// hacks for nls support
	// change for 5.1!
	private java.util.ResourceBundle fontResource = java.util.ResourceBundle.getBundle("com/ibm/etools/jbcf/visual/beaninfo/menucomponent");  //$NON-NLS-1$
	private JLabel fontLabel = new JLabel(fontResource.getString("fontSD")); //$NON-NLS-1$

	private JLabel borderLabel = new JLabel(VisualBeanInfoMessages.getString("TitledBorder.Border")); //$NON-NLS-1$

	private JTextField titleField = null;
	private JComboBox justificationCombo = null;
	private JComboBox positionCombo = null;
	private int[] justificationValues;
	private int[] positionValues;

	private Color titleColor = UIManager.getColor("TitledBorder.color"); //$NON-NLS-1$
	private Font titleFont = UIManager.getFont("TitledBorder.font"); //$NON-NLS-1$
	private Border titleBorder = null;

	private ColorEditor colorEditor = null;
	private FontEditor fontEditor = null;
	private BorderEditor borderEditor = null;

	private JPanel colorEditPanel = null;
	private JPanel fontEditPanel = null;
	private JPanel borderEditPanel = null;

	private JTextField colorField = null;
	private JTextField fontField = null;
	private JTextField borderField = null;

	private JButton colorButton = null;
	private JButton fontButton = null;
	private JButton borderButton = null;

	public TitledBorderPropertyPage() {
		super();
		initialize();
	}

	public String getName() {
		return "TitledBorderPropertyPage"; //$NON-NLS-1$
	}

	public JTextField getTitleField() {
		if (titleField == null) {
			titleField = new JTextField();
			titleField.addActionListener(this);
		}
		return titleField;
	}		

public String getTitleJustificationLabel( int constant ) {
	String label = constant + ""; //$NON-NLS-1$
	switch ( constant ) {
		case TitledBorder.DEFAULT_JUSTIFICATION : label = VisualBeanInfoMessages.getString("TitledBorder.radio.DefaultJustification.Text"); break; //$NON-NLS-1$
		case TitledBorder.LEFT                  : label = VisualBeanInfoMessages.getString("Left"); break; //$NON-NLS-1$
		case TitledBorder.CENTER                : label = VisualBeanInfoMessages.getString("Center"); break; //$NON-NLS-1$
		case TitledBorder.RIGHT                 : label = VisualBeanInfoMessages.getString("Right"); break; //$NON-NLS-1$
		case TitledBorder.LEADING               : label = VisualBeanInfoMessages.getString("Leading"); break; //$NON-NLS-1$
		case TitledBorder.TRAILING              : label = VisualBeanInfoMessages.getString("Trailing"); break; //$NON-NLS-1$
	}
	return label;
}

public String getTitleJustificationNamedConstant( int constant ) {
	String name = constant + ""; //$NON-NLS-1$
	switch ( constant ) {
		case TitledBorder.DEFAULT_JUSTIFICATION : name = "javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION"; break; //$NON-NLS-1$
		case TitledBorder.LEFT                  : name = "javax.swing.border.TitledBorder.LEFT"; break; //$NON-NLS-1$
		case TitledBorder.CENTER                : name = "javax.swing.border.TitledBorder.CENTER"; break; //$NON-NLS-1$
		case TitledBorder.RIGHT                 : name = "javax.swing.border.TitledBorder.RIGHT"; break; //$NON-NLS-1$
		case TitledBorder.LEADING               : name = "javax.swing.border.TitledBorder.LEADING"; break; //$NON-NLS-1$
		case TitledBorder.TRAILING              : name = "javax.swing.border.TitledBorder.TRAILING"; break; //$NON-NLS-1$
	}
	return name;
}

public int getTitleJustification(){
	return titleJustification;
}

public String getTitlePositionLabel( int constant ) {
	String label = constant + ""; //$NON-NLS-1$
	switch ( constant ) {
		case TitledBorder.DEFAULT_POSITION : label = VisualBeanInfoMessages.getString("TitledBorder.radio.DefaultPosition.Text"); break; //$NON-NLS-1$
		case TitledBorder.ABOVE_TOP        : label = VisualBeanInfoMessages.getString("TitledBorder.radio.AboveTop.Text"); break; //$NON-NLS-1$
		case TitledBorder.TOP              : label = VisualBeanInfoMessages.getString("Top"); break; //$NON-NLS-1$
		case TitledBorder.BELOW_TOP        : label = VisualBeanInfoMessages.getString("TitledBorder.radio.BelowTop.Text"); break; //$NON-NLS-1$
		case TitledBorder.ABOVE_BOTTOM     : label = VisualBeanInfoMessages.getString("TitledBorder.radio.AboveBottom.Text"); break; //$NON-NLS-1$
		case TitledBorder.BOTTOM           : label = VisualBeanInfoMessages.getString("Bottom"); break; //$NON-NLS-1$
		case TitledBorder.BELOW_BOTTOM     : label = VisualBeanInfoMessages.getString("TitledBorder.radio.BelowBottom.Text"); break; //$NON-NLS-1$
	}
    return label;
}

public String getTitlePositionNamedConstant( int constant ) {
	String name = constant + ""; //$NON-NLS-1$
	switch ( constant ) {
		case TitledBorder.DEFAULT_POSITION : name = "javax.swing.border.TitledBorder.DEFAULT_POSITION"; break; //$NON-NLS-1$
		case TitledBorder.ABOVE_TOP        : name = "javax.swing.border.TitledBorder.ABOVE_TOP"; break; //$NON-NLS-1$
		case TitledBorder.TOP              : name = "javax.swing.border.TitledBorder.TOP"; break; //$NON-NLS-1$
		case TitledBorder.BELOW_TOP        : name = "javax.swing.border.TitledBorder.BELOW_TOP"; break; //$NON-NLS-1$
		case TitledBorder.ABOVE_BOTTOM     : name = "javax.swing.border.TitledBorder.ABOVE_BOTTOM"; break; //$NON-NLS-1$
		case TitledBorder.BOTTOM           : name = "javax.swing.border.TitledBorder.BOTTOM"; break; //$NON-NLS-1$
		case TitledBorder.BELOW_BOTTOM     : name = "javax.swing.border.TitledBorder.BELOW_BOTTOM"; break; //$NON-NLS-1$
	}
    return name;
}
				

public int getTitlePosition() {
	return titlePosition;
}

public JComboBox getJustificationCombo() {
	if (justificationCombo == null) {
		justificationCombo = new JComboBox();
		justificationCombo.setBackground(SystemColor.control);
		justificationCombo.addItem(getTitleJustificationLabel(TitledBorder.DEFAULT_JUSTIFICATION));
		justificationCombo.addItem(getTitleJustificationLabel(TitledBorder.LEFT));
		justificationCombo.addItem(getTitleJustificationLabel(TitledBorder.CENTER));
		justificationCombo.addItem(getTitleJustificationLabel(TitledBorder.RIGHT));
		justificationCombo.addItem(getTitleJustificationLabel(TitledBorder.LEADING));
		justificationCombo.addItem(getTitleJustificationLabel(TitledBorder.TRAILING));

		justificationValues = new int[justificationCombo.getItemCount()];
		justificationValues[0] = TitledBorder.DEFAULT_JUSTIFICATION;
		justificationValues[1] = TitledBorder.LEFT;
		justificationValues[2] = TitledBorder.CENTER;
		justificationValues[3] = TitledBorder.RIGHT;
		justificationValues[4] = TitledBorder.LEADING;
		justificationValues[5] = TitledBorder.TRAILING;

		justificationCombo.addItemListener(this);
	}
	return justificationCombo;
}

public JComboBox getPositionCombo() {
	if (positionCombo == null) {
		positionCombo = new JComboBox();
		positionCombo.setBackground(SystemColor.control);
		positionCombo.addItem(getTitlePositionLabel(TitledBorder.DEFAULT_POSITION));
		positionCombo.addItem(getTitlePositionLabel(TitledBorder.ABOVE_TOP));
		positionCombo.addItem(getTitlePositionLabel(TitledBorder.TOP));
		positionCombo.addItem(getTitlePositionLabel(TitledBorder.BELOW_TOP));
		positionCombo.addItem(getTitlePositionLabel(TitledBorder.ABOVE_BOTTOM));
		positionCombo.addItem(getTitlePositionLabel(TitledBorder.BOTTOM));
		positionCombo.addItem(getTitlePositionLabel(TitledBorder.BELOW_BOTTOM));

		positionValues = new int[positionCombo.getItemCount()];
		positionValues[0] = TitledBorder.DEFAULT_POSITION;
		positionValues[1] = TitledBorder.ABOVE_TOP;
		positionValues[2] = TitledBorder.TOP;
		positionValues[3] = TitledBorder.BELOW_TOP;
		positionValues[4] = TitledBorder.ABOVE_BOTTOM;
		positionValues[5] = TitledBorder.BOTTOM;
		positionValues[6] = TitledBorder.BELOW_BOTTOM;

		positionCombo.addItemListener(this);
	}
	return positionCombo;
}

public FontEditor getFontEditor() {
	if (fontEditor == null) {
		fontEditor = new FontEditor();
		fontEditor.setValue(titleFont);
	}
	return fontEditor;
}

public JPanel getFontEditPanel() {
	if (fontEditPanel == null) {
		fontEditPanel = new JPanel();
		fontEditPanel.setBackground(SystemColor.control);
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
	    c.fill    = GridBagConstraints.HORIZONTAL;
	    c.anchor  = GridBagConstraints.NORTHWEST;
	    c.gridy = 0;
		fontEditPanel.setLayout( gb );

	    c.insets  = new Insets( 0, 0, 0, 5 );
        c.weightx = 1.0;
        c.gridx   = 0;	
		fontField = new JTextField();
		fontField.setText(getFontEditor().getAsText());
		fontField.setEditable(false);
		fontField.setBackground(SystemColor.control);
		gb.setConstraints( fontField, c );
		fontEditPanel.add(fontField);

	    c.insets  = new Insets( 0, 0, 0, 0 );
		c.weightx = 0;
		c.gridx = 1;
		fontButton = new JButton();
		fontButton.setText( "..." ); //$NON-NLS-1$
		fontButton.setMargin(new java.awt.Insets(0, 5, 0, 5));
		fontButton.setBackground(SystemColor.control);
		fontButton.addActionListener(this);
		gb.setConstraints( fontButton, c );
		fontEditPanel.add(fontButton);
	}
	return fontEditPanel;
}

public ColorEditor getColorEditor() {
	if (colorEditor == null) {
		colorEditor = new ColorEditor();
		colorEditor.setValue(titleColor);
	}
	return colorEditor;
}

public JPanel getColorEditPanel() {
	if (colorEditPanel == null) {
		colorEditPanel = new JPanel();
		colorEditPanel.setBackground(SystemColor.control);
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
	    c.fill    = GridBagConstraints.HORIZONTAL;
	    c.anchor  = GridBagConstraints.NORTHWEST;
	    c.gridy = 0;
		colorEditPanel.setLayout(gb);

	    c.insets  = new Insets( 0, 0, 0, 5 );
        c.weightx = 1.0;
        c.gridx   = 0;	
		colorField = new JTextField();
		colorField.setText(getColorEditor().getAsText());
		colorField.setEditable(false);
		colorField.setBackground(SystemColor.control);
		gb.setConstraints( colorField, c );
		colorEditPanel.add(colorField);

	    c.insets  = new Insets( 0, 0, 0, 0 );
		c.weightx = 0;
		c.gridx = 1;
		colorButton = new JButton();
		colorButton.setText( "..." ); //$NON-NLS-1$
		colorButton.setMargin(new java.awt.Insets(0, 5, 0, 5));
		colorButton.setBackground(SystemColor.control);
		colorButton.addActionListener(this);
		gb.setConstraints( colorButton, c );
		colorEditPanel.add(colorButton);
	}

	return colorEditPanel;
}

public BorderEditor getBorderEditor() {
	if (borderEditor == null) {
		borderEditor = new BorderEditor();
		borderEditor.setValue(titleBorder);
	}
	return borderEditor;
}

public JPanel getBorderEditPanel() {
	if (borderEditPanel == null) {
		borderEditPanel = new JPanel();
		borderEditPanel.setBackground(SystemColor.control);
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
	    c.fill    = GridBagConstraints.HORIZONTAL;
	    c.anchor  = GridBagConstraints.NORTHWEST;
	    c.gridy = 0;
        borderEditPanel.setLayout( gb );	    

	    c.insets  = new Insets( 0, 0, 0, 5 );
        c.weightx = 1.0;
        c.gridx   = 0;	
		borderField = new JTextField();
		borderField.setEditable(false);
		borderField.setBackground(SystemColor.control);
		gb.setConstraints( borderField, c );
		borderEditPanel.add(borderField);

	    c.insets  = new Insets( 0, 0, 0, 0 );
		c.weightx = 0;
		c.gridx = 1;
		borderButton = new JButton();
		borderButton.setText( "..." ); //$NON-NLS-1$
		borderButton.setMargin(new java.awt.Insets(0, 5, 0, 5));
		borderButton.setBackground(SystemColor.control);
		borderButton.addActionListener(this);
		gb.setConstraints( borderButton, c );
		borderEditPanel.add(borderButton);
	}

	return borderEditPanel;
}

public void initialize() {
	this.setName("TitledBorderPropertyPage"); //$NON-NLS-1$

	this.setBackground(SystemColor.control);
	Insets labelInsets = new Insets( 0, 5, 5, 5 );
	Insets compInsets = new Insets( 0, 5, 10, 5 );

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
    c.weightx = 1.0;
	c.fill    = GridBagConstraints.HORIZONTAL;
	c.anchor  = GridBagConstraints.NORTHWEST;
    c.gridx   = 0;

	this.setLayout(gb);

	c.gridy = 0;
	c.insets = labelInsets;
	gb.setConstraints( titleLabel, c );
	this.add(titleLabel);
	
	c.gridy = 1;
	c.insets = compInsets;
	gb.setConstraints( getTitleField(), c );	
	this.add(getTitleField());
	
	c.gridy = 2;
	c.insets = labelInsets;
	gb.setConstraints( titleJustificationLabel, c );
	this.add(titleJustificationLabel);
	
	c.gridy = 3;
	c.insets = compInsets;
	gb.setConstraints( getJustificationCombo(), c );
	this.add(getJustificationCombo());
	
	c.gridy = 4;
	c.insets = labelInsets;
	gb.setConstraints( titlePositionLabel, c );
	this.add(titlePositionLabel);
	
	c.gridy = 5;
	c.insets = compInsets;
	gb.setConstraints( getPositionCombo(), c );
	this.add(getPositionCombo());
	
	c.gridy = 6;
	c.insets = labelInsets;
	gb.setConstraints( fontLabel, c );
	this.add(fontLabel);
	
	c.gridy = 7;
	c.insets = compInsets;
	gb.setConstraints( getFontEditPanel(), c );
	this.add(getFontEditPanel());
	
	c.gridy = 8;
	c.insets = labelInsets;
	gb.setConstraints( colorLabel, c );
	this.add(colorLabel);
	
	c.gridy = 9;
	c.insets = compInsets;
	gb.setConstraints( getColorEditPanel(), c );
	this.add(getColorEditPanel());
	
	c.gridy = 10;
	c.insets = labelInsets;
	gb.setConstraints( borderLabel, c );
	this.add(borderLabel);
	
	c.gridy = 11;
	c.insets = compInsets;
	gb.setConstraints( getBorderEditPanel(), c );
	this.add(getBorderEditPanel());
	
	c.gridy = 12;
	c.weighty = 1.0;
	JPanel filler = new JPanel();
	filler.setBackground(SystemColor.control);
	gb.setConstraints( filler, c );
	this.add( filler );
	
	this.setMinimumSize(new Dimension(200, 350));
	if (!SUB_BORDER_ENABLED) {
		borderLabel.setVisible(false);
		getBorderEditPanel().setVisible(false);
	}
}

public SubEditorJDialog getFontChooserDialog() {
	SubEditorJDialog fontChooserDialog;
	Container parent = getTopLevelAncestor();
	if (parent instanceof Frame) {
		fontChooserDialog = new SubEditorJDialog((Frame) parent);
	} else if (parent instanceof Dialog) {
		fontChooserDialog = new SubEditorJDialog((Dialog) parent);
	} else {
		fontChooserDialog = new SubEditorJDialog((Frame) null);
	}
	fontChooserDialog.setPropertyEditor(getFontEditor().getCustomEditor());
	
	return fontChooserDialog;
}

public SubEditorJDialog getColorChooserDialog() {
	SubEditorJDialog colorChooserDialog = null;
	Container parent = getTopLevelAncestor();
	if (parent instanceof Frame) {
		colorChooserDialog = new SubEditorJDialog((Frame) parent);
	} else if (parent instanceof Dialog) {
		colorChooserDialog = new SubEditorJDialog((Dialog) parent);
	} else {
		colorChooserDialog = new SubEditorJDialog((Frame) null);
	}
	colorChooserDialog.setPropertyEditor(getColorEditor().getCustomEditor());

	return colorChooserDialog;
}

public SubEditorJDialog getBorderChooserDialog() {
	SubEditorJDialog borderChooserDialog = null;
	Container parent = getTopLevelAncestor();
	if (parent instanceof Frame) {
		borderChooserDialog = new SubEditorJDialog((Frame) parent);
	} else if (parent instanceof Dialog) {
		borderChooserDialog = new SubEditorJDialog((Dialog) parent);
	} else {
		borderChooserDialog = new SubEditorJDialog((Frame) null);
	}
	borderChooserDialog.setPropertyEditor(getBorderEditor().getCustomEditor());

	return borderChooserDialog;
}

public void updateFont() {
	titleFont = (Font) getFontEditor().getValue();
	fontField.setText(getFontEditor().getAsText());
	firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
}

public void updateColor() {
	titleColor = (Color) getColorEditor().getValue();
	colorField.setText(getColorEditor().getAsText());
	firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
}

public void updateBorder() {
	titleBorder = (Border) getBorderEditor().getValue();
	borderField.setText(getBorderEditor().getAsText());
	firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
}

public void actionPerformed(ActionEvent e) {
	Object source = e.getSource();
	int state;

	if (source == titleField) {
		firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
	} else if (source == fontButton) {
		state = getFontChooserDialog().showEditor();
		if (state == SubEditorJDialog.OK) {
			updateFont();
		}
	} else if (source == colorButton) {
		state = getColorChooserDialog().showEditor();
		if (state == SubEditorJDialog.OK) {
			updateColor();
		}
	} else if (source == borderButton) {
		state = getBorderChooserDialog().showEditor();
		if (state == SubEditorJDialog.OK) {
			updateBorder();
		}

	}
}

public void itemStateChanged(ItemEvent e) {
	if (e.getSource() == getJustificationCombo()) {
		int index = getJustificationCombo().getSelectedIndex();
		if (index >= 0 && index <= justificationValues.length) {
			titleJustification = justificationValues[index];
		}
	} else if (e.getSource() == getPositionCombo()) {
		int index = getPositionCombo().getSelectedIndex();
		if (index >= 0 && index <= positionValues.length) {
			titlePosition = positionValues[index];
		}
	}

	firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
}

public void updateHandle(DocumentEvent e) {
	if (e.getDocument() == titleField.getDocument()) {
		firePropertyChange("borderValueChanged", null, getBorderValue()); //$NON-NLS-1$
	}
}

public void insertUpdate(DocumentEvent e) {
	updateHandle(e);
}

public void removeUpdate(DocumentEvent e) {
	updateHandle(e);
}

public void changedUpdate(DocumentEvent e) {
	updateHandle(e);
}

public boolean okToSetBorder(Border aBorder) {
	if (aBorder instanceof TitledBorder) {
		TitledBorder fBorder = (TitledBorder) aBorder;
		titleField.setText(fBorder.getTitle());
		titleJustification = fBorder.getTitleJustification();
		titlePosition = fBorder.getTitlePosition();
		titleColor = fBorder.getTitleColor();
		titleFont = fBorder.getTitleFont();
		titleBorder = fBorder.getBorder();

		getJustificationCombo().setSelectedItem(getTitleJustificationLabel(titleJustification));
		getPositionCombo().setSelectedItem(getTitlePositionLabel(titlePosition));
		if (titleColor != null) {
			getColorEditor().setValue(titleColor);
			updateColor();
		}
		if (titleFont != null) {
			getFontEditor().setValue(titleFont);
			updateFont();
		}
		if (titleBorder != null) {
			getBorderEditor().setValue(titleBorder);
			updateBorder();
		}
		return true;
	}
	return false;
}

public Border getBorderValue() {
	Border aBorder = BorderFactory.createTitledBorder(titleBorder, getTitle(), titleJustification, titlePosition, titleFont, titleColor);
	return aBorder;
}

public String getTitle() {
	return getTitleField().getText();
}

public String getDisplayName() {
	return MessageFormat.format(VisualBeanInfoMessages.getString("TitledBorder.DisplayName.Title(TitleString,JustificationString,TitlePositionString)"), //$NON-NLS-1$
	                             new Object[] { getTitle(), getTitleJustificationLabel(getTitleJustification()), getTitlePositionLabel(getTitlePosition())});
}

public String getJavaInitializationString() {
	if ( getBorderEditor().getValue() == null ) {
        return "javax.swing.BorderFactory.createTitledBorder(null, \"" + getTitle() + "\", " + //$NON-NLS-1$ //$NON-NLS-2$
	        getTitleJustificationNamedConstant(getTitleJustification()) + ", " + getTitlePositionNamedConstant(getTitlePosition()) + ", " + //$NON-NLS-1$ //$NON-NLS-2$
		    getFontEditor().getJavaInitializationString() + ", " + getColorEditor().getJavaInitializationString() + ")"; //$NON-NLS-1$ //$NON-NLS-2$
	} else {
        return "javax.swing.BorderFactory.createTitledBorder(" + getBorderEditor().getJavaInitializationString() + ", \"" + getTitle() + "\", " + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	        getTitleJustificationNamedConstant(getTitleJustification()) + ", " + getTitlePositionNamedConstant(getTitlePosition()) + ", " + //$NON-NLS-1$ //$NON-NLS-2$
		    getFontEditor().getJavaInitializationString() + ", " + getColorEditor().getJavaInitializationString() + ")"; //$NON-NLS-1$ //$NON-NLS-2$
	}
}
}