package com.ibm.etools.jbcf.visual.common;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

/**
 * This contains constants needed to be shared
 * between the vm and the IDE for ImageData collection.
 */
public interface ImageDataConstants {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public final static int
		IMAGE_DATA_COLLECTION = 1,			// MsgId sent back with the image data collection input stream callback.
		IMAGE_GET_PREFERRED_DEPTH = 2,		// MsgId for requesting preferred depth.
		IMAGE_HAS_BEEN_ABORTED = 3;					// MsgId for image was aborted so ignore rest of data in the pipeline. The beanproxy parm will have the status int to use.
	
	// The stream for feeding image data back will be wrappered into a Data...Stream
	// so that some structured data may be sent.
	public final static byte
		CMD_DIM = 1,		// A new width and height, width/height will follow as integers.
		CMD_DIRECT = 2,	// A direct color palette, followed by a byte containing the depth.
		CMD_INDEXED = 3,	// An indexed color palette, followed by a byte containing the depth,
					//	an int containing the transparent pixel, an int containing the
					//	number of RGBs to follow, and array of bytes,
					//	representing the RGBs (R,G,B,R,G,B,...).
		CMD_BYTES = 4,		// Send bytes for a row. Followed by int x, int y. Following this will be repeat/nonrepeat blocks, until next command.
		CMD_INTS = 5,		// Send ints for a row. Followed by int x, int y. Following this will be repeat/nonrepeat blocks, until next command.
		CMD_REPEAT = 6,		// One pixel will be repeated. Followed by int numPixels, int/byte pixel (pixel to be repeated, depends on current cmd mode)
		CMD_NOREPEAT = 7,	// Pixel group. Followed by int numPixels, int/bype pixels (for numPixels, byte/int depends on current command mode).
		CMD_DONE = 8;		// The collection has ended. Followed by int status, shown below.
		
		
	// Completion codes
	public final static int
		/**
		 * An error was encountered while producing the image.
		 */
		IMAGE_ERROR = 1,
		
		/**
		 * One frame of the image is complete but there are more frames
		 * to be delivered.
		 */
		SINGLE_FRAME_DONE = 2,
		
		/**
		 * The image is complete and there are no more pixels or frames
		 * to be delivered.
		 */
		STATIC_IMAGE_DONE = 3,
		
		/**
		 * The image creation process was deliberately aborted.
		 */
		IMAGE_ABORTED = 4,
		
		/**
		 * Unknown status code from ImageProducer.
		 */
		UNKNOWN_STATUS = 5,
		
		/**
		 * Consume in progress.
		 */
		IN_PROGRESS = 6,
		
		/**
		 * Consume not in progress.
		 */
		NOT_IN_PROGRESS = -1;
		

}