package com.ibm.etools.jbcf.visual.tab;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;

import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.*;

/**
 * @author JoeWin
 */
public class TabOrderLineDecoration extends Shape implements RotatableDecoration {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Color fFrameColor = ColorConstants.red;
	protected int fIndex;
	protected int fNumChars; // Store the number of characters in the tab label
	protected int fR1 , fR2; // R1 and R2 are the two radix of the oval that is drawn as the decoration
	protected Point fOvalCorner;
	protected static Font fTabIndexFont = new Font(null, "ms sans serif", 8, SWT.NORMAL);//$NON-NLS-1$
	protected static FontMetrics fFontMetrics;
//	protected ArrowTip fArrowTip; // Use an arrow tip end decoration that knows how to draw itself
	protected boolean fDrawIndexShape = true;
	protected boolean fDrawArrowTip = true;
	protected Color fOvalBackGroundColor = ColorConstants.yellow;
	protected Color fOvalForegroundColor = ColorConstants.black;
	protected boolean fIsBeingSelected = false;
	protected Color fMoveOvalForegroundColor = ColorConstants.black;
	protected Color fMoveBackgroundColor = ColorConstants.orange;
	protected boolean fIsBeingMoved = false;
	protected int fCharWidth = -1;
	protected int fCharHeight = -1;
/**
 * Construct us with the index that we will draw
 */
public TabOrderLineDecoration(int anIndex){
	fIndex = anIndex;
//	fArrowTip = new ArrowTip( 0.55 , 8 , 6 );
	// Find the number of characters of the index
	fNumChars = ("" + fIndex).length();

}
/**
 * Construct us with the index that we will draw
 */
public TabOrderLineDecoration(int anIndex , boolean drawIndexShape , boolean drawArrowTip){
	this(anIndex);
	fDrawIndexShape = drawIndexShape;
	fDrawArrowTip = drawArrowTip;
}
/**
 * draw method comment.
 */
public void fillShape(Graphics g){

	// Get the original colors
	Color originalBackground = g.getBackgroundColor();
	Color originalForeground = g.getForegroundColor();
	
	int x1 = fLocation.x;
	int y1 = fLocation.y;

	if ( fDrawIndexShape ) {
		// A nice font is used for the text is held in a a static field
		Font originalFont = g.getFont();
		g.setFont(fTabIndexFont);
		// If the font metrics have not been stored already then record them
		if ( fFontMetrics == null ) fFontMetrics = g.getFontMetrics();
		fCharWidth = fFontMetrics.getAverageCharWidth() * fNumChars;
		fCharHeight = fFontMetrics.getHeight();
		System.out.println("--- Index = " + fIndex + " @" + x1 + y1 + fCharWidth + fCharHeight);		
		
		// The char height and charWidth are used to calculate the dimensions of an oval to be drawn
		// fR1 and fR2 are the two radix of the oval ( fR1 being horizontal and fR2 being vertical )
		fR1 = fCharWidth + 8;
		fR2 = fCharHeight + 6;
		fOvalCorner = new Point( x1 - fCharWidth/2 - 4 , y1 - fCharHeight/2 - 3);
		// Adjust the colors
		if(fIsBeingSelected){
			g.setForegroundColor( fOvalForegroundColor );
			g.setBackgroundColor( fOvalBackGroundColor );
		}else{
			g.setBackgroundColor( ColorConstants.yellow );
			g.setForegroundColor( ColorConstants.black );
		}
		if(fIsBeingMoved){
			g.setForegroundColor( fMoveOvalForegroundColor );
			g.setBackgroundColor( fMoveBackgroundColor );
		}
		// The original of the oval is the top LHS and not the center point so we must adjust to find a new origin that is the
		// mid point
		g.fillOval(new Rectangle(fOvalCorner.x , fOvalCorner.y  , fR1 , fR2));
		g.drawOval(new Rectangle(fOvalCorner.x , fOvalCorner.y , fR1 , fR2));
		if ( fIndex != -1 ) {
			g.drawText("" + fIndex , x1 - fCharWidth/2 , y1 - 7);
		}

		if ( fDrawArrowTip ) {
			// Draw the arrow tip in red, adjusting so that it is drawn towards the endpoint
			g.setBackgroundColor(fFrameColor);
			// Rather confusingly x1 and y1 are the end of the line and x2 and y2 are the start
//			Point endEdgeAdjustment = getEdgeAdjustment( x1 , y1 , x2 , y2 );
//			fArrowTip.draw( g , x1 + endEdgeAdjustment.x , y1 + endEdgeAdjustment.y , x2 , y2 );
		}
	
		// Restore the original fonts and colors
		g.setFont(originalFont);

	} else {
		// Draw the arrow tip in red
		if ( fDrawArrowTip ) {
			g.setBackgroundColor(fFrameColor);
//			fArrowTip.draw( g , x1  , y1 , x2 , y2 );
		}
	}

	g.setBackgroundColor( originalBackground );
	g.setForegroundColor( originalForeground );

}
/**
 * adjust the arguments so that they do not lie within our oval decoration.
 * For example if the points are 50,50 and 100,50 the line is a flat line.  If the oval has a width
 * of 5 then we should return a new point that is 95,50 which is where the line will now end
 * adjustStartToEdge is a twin to this method except that it moves the start rather than the end
 */
public Point getEdgeAdjustment( int endX , int endY , int startX , int startY ){

	// First find the angle of the line.  This is done by finding the angle to the start of the line
	// Note we do endX - startX and yet we do startY - endX.  This is because the y coordinate systems are opposite
	// between windows and geometry
	double lineAngle = Math.abs( Math.atan( (double) ( endY - startY ) / ( endX - startX ) ));
	// The line angle here is from the end to the start of the line.  We want the angle that this line represents
	// for the oval intersection.  If the line is coming in from the left ( i.e. endX > startX ) then we must take this
	// away from PI ( 180 degress )
	if ( endX > startX ) lineAngle = Math.PI - lineAngle;
	// Having found the angle of the line we need to find the point on the edge of our oval that lies at this angle
	// ( The reason we find the angle to the start of the line is that the elipse for adjusting an end is assumed to be
	// at the end of the line )
	// Find the point adjustment based on our oval's two radix
	Point circumferencePoint = ovalAngleToPoint( new Rectangle( 0 , 0 , fR1 , fR2 ) , lineAngle );
	// Having got the new circumference point we should adjust x and y by it
	// The x is the oval corner + the circumference point.x, the Y however has to deal with switching between
	// co-ordinate systems in windows where y is down and geometry where y is up
	// We return the adjustment so that both ends of the line can be adjusted
	int xAdjustment = circumferencePoint.x - fR1 / 2;
	int yAdjustment = circumferencePoint.y - fR2 / 2;
	// If the line is being drawn updarwds then the y adjustment is added and not subtracted
	if ( endY > startY ) yAdjustment = 0 - yAdjustment;
	return new Point(xAdjustment , yAdjustment);	
}
/**
 * fR2 holds the height of the elipse being drawn
 * If this has not been calculated then get it by using the static font metrics
 * This is because fR2 is only actually created when we are drawn
 */
public int getShapeHeight(){

	if ( fR2 != 0 ) {
		return fR2;
	} else if ( fFontMetrics != null ) {
		return fFontMetrics.getHeight() + 6;
	} else {
		// If we have no font metrics then assume the average character heightis 13
		return 13 + 6;
	}
}
/**
 * fR1 holds the width of the elipse being drawn
 * If this has not been calculated then get it by using the static font metrics
 * This is because fR1 is only actually created when we are drawn
 */
public int getShapeWidth(){

	if ( fR1 != 0 ) {
		return fR1;
	} else if ( fFontMetrics != null ) {
		return fFontMetrics.getAverageCharWidth() * fNumChars + 8;
	} else {
		// If we have no font metrics then assume the average character width is 5
		return 5 * fNumChars + 8;
	}
}
/**
 * Purpose is to highlight the endDecoration that is
 * selected, and is being moved around for
 * tabOrder changing.
 * Creation date: (10/13/00 3:01:03 PM)
 * @param value boolean
 */
public void isBeingMoved(boolean value) {
	fIsBeingMoved = value;
}
/**
 * This basically highlights the endDecoration of this
 * line, by setting the OvalColor, and frameColor to be
 * different when the line is being selected. It is 
 * called from the 
 * TabOrederLineConnectionViewObject::isBeingSelected() 
 * function.
 * Creation date: (10/11/00 2:44:52 PM)
 */
public void isBeingSelected(boolean value) {
	fIsBeingSelected = value;
	if(value == true){
		setOvalColor( ColorConstants.yellow , ColorConstants.blue );
		//setFrameColor( fBlue );
	}else{
		setOvalColor( ColorConstants.black, ColorConstants.yellow );
		//setFrameColor( fRed );
	}
}
/**
 * Set the frame color
 */
public void setFrameColor(Color aColor){
	fFrameColor = aColor;
}
/**
 * Insert the method's description here.
 * Creation date: (10/11/00 2:42:15 PM)
 * @param foreground com.ibm.swt.graphics.Color
 * @param background com.ibm.swt.graphics.Color
 */
protected void setOvalColor(Color foreground, Color background) {
	if(foreground != null){fOvalForegroundColor = foreground;}
	if(background != null){fOvalBackGroundColor = background;}
}
/**
 * Gets the point on an oval that corresponds to the given angle.
 */
static public Point ovalAngleToPoint(Rectangle r, double angle) {
	Point center = center(r);
	Point p = polarToPoint(angle, r.width/2, r.height/2);
	return new Point(center.x + p.x, center.y + p.y);
}
/**
 * Converts a polar to a point
 */
static public Point polarToPoint(double angle, double fx, double fy) {
	double si = Math.sin(angle);
	double co = Math.cos(angle);
    return new Point((int)(fx*co+0.5), (int)(fy*si+0.5));
}
static public Point center(Rectangle r) {
	return new Point(r.x + r.width /2, r.y + r.height/2);
}
protected Point fLocation;
public Rectangle getBounds(){
	return super.getBounds();
}
// Use this later to determine the arrow tip end
public void setReferencePoint(Point aPoint){

}
public void outlineShape(Graphics g){

}
}