package com.ibm.etools.jbcf.visual.tab;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import org.eclipse.swt.graphics.*;

import com.ibm.etools.gef.GraphicalEditPart;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.visual.ComponentGraphicalEditPart;
/**
 */
public abstract class TabOrderManager implements IOrderManager {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ITabRootEditPart fTabRootEditPart;
	protected List fTabOrderLines;
	protected List fTabOrderLineEPs;  // Collection of view objects for each tab order line.  Required for deactivation
	protected List fListenToBeans; 	// Collection of IBean objects that are contains that we listen to so we
										// know when to refresh tab order
	protected ComponentGraphicalEditPart fLastFocussableComponentEditPart;
	protected TabOrderLineConnectionEditPart fPreviousTabOrderLineConnectionEditPart;
	protected Hashtable fComponentVOtoTabOrderLineConnectionVO;
	protected int fComponentIndex;
//	protected TabOrderManagerMOFAdaptor fMOFAdaptor;
	protected static FontMetrics fFontMetrics;	// Store the font metrics used to draw the tab decoration
	protected boolean fDirty = false;
	

public TabOrderManager(ITabRootEditPart tabRootEditPart) {
	super();
	fTabRootEditPart = tabRootEditPart;
}
/**
 * Generate tab order line objects for each tabbable component
 */
public void activate() {

	initialize();
	generateTabOrderLines();

}	
protected void initialize(){

	// Clear the vectors
	fTabOrderLines = new ArrayList();
	fTabOrderLineEPs = new ArrayList();
	fListenToBeans = new ArrayList();
	// Reset the component index
	fLastFocussableComponentEditPart = null;
	fPreviousTabOrderLineConnectionEditPart = null;
	fComponentVOtoTabOrderLineConnectionVO =  new Hashtable();
	fComponentIndex = 1;

}
protected void generateTabOrderLines() {
	
	// Generate the tab order line from the frame down
	generateTabOrderLines( fTabRootEditPart.getBean() );

}
public void deactivate() {
	
	// Remove the tab lines as child view objects of the frame.  This will deactivate the figure for each one
	if(fTabOrderLineEPs != null){
		Iterator iter = fTabOrderLineEPs.iterator();
		while ( iter.hasNext() ) {
			GraphicalEditPart ep = (GraphicalEditPart) iter.next();
			fTabRootEditPart.removeAnnotation(ep);
		}
	}
	
	fTabOrderLines = null;
	fTabOrderLineEPs = null;	
	
}
/**
 * Generate tab order line objects for each tabbable component
 * Also for each tabbable component we need to find the view object that corresponds to it and
 * create an input policy for it that will deal with handling the input of the tab lines
 */
protected abstract void generateTabOrderLines(IJavaInstance containerComponent);
protected void createLastTabOrderLine( ComponentGraphicalEditPart anEditPart ) {
	
	anEditPart.toString();
	System.out.println("Last tab order line = " + anEditPart);
}
protected void createTabOrderLine(ComponentGraphicalEditPart fromComponentEditPart , ComponentGraphicalEditPart toComponentEditPart){

	// Increment the counter as to where this component is inside the tab ordering
	TabOrderLine line = primCreateTabOrderLine(fromComponentEditPart.getBean() , toComponentEditPart.getBean() );
	fComponentIndex ++;	
	fTabOrderLines.add(line);
	// The TabOrderLine is the model object.  We also need to create a view object and add it to the frame object
	// The view objects are collected up in a vector so we can remove them when we are deactivated
	TabOrderLineConnectionEditPart ep = createTabOrderLineConnectionEditPart(line,fromComponentEditPart,toComponentEditPart);
	// If we have a previous view object then set it
	if ( fPreviousTabOrderLineConnectionEditPart != null ) {
		ep.setPreviousEditPart(fPreviousTabOrderLineConnectionEditPart);
	}
	// The vo just created is now the most previous one
	fPreviousTabOrderLineConnectionEditPart = ep;
	ep.setSource( fromComponentEditPart );
	ep.setTarget( toComponentEditPart );
	fTabRootEditPart.addAnnotation(ep);
	fTabOrderLineEPs.add(ep);

}
/**
 * Create a view object for the argument
 */
protected TabOrderLineConnectionEditPart createTabOrderLineConnectionEditPart(TabOrderLine aLine, ComponentGraphicalEditPart source, ComponentGraphicalEditPart target){

	TabOrderLineConnectionEditPart result = new TabOrderLineConnectionEditPart(aLine,source,target);
	// If we have a prevoius view object then tell the tab order line connection view object
	// It will use the target connector of the previous VO as its source connector
	if ( fPreviousTabOrderLineConnectionEditPart != null ) {
		result.setPreviousEditPart(fPreviousTabOrderLineConnectionEditPart);
	}
	return result;

}
/**
 * Create a tab order line between the two arguments
 */
protected TabOrderLine primCreateTabOrderLine(IJavaInstance fromComponentBean , IJavaInstance toComponentBean){

	return new TabOrderLine(fromComponentBean , toComponentBean , this , fComponentIndex );

}	
}