package com.ibm.etools.jbcf.visual.vm;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.Component;
import java.awt.Container;

/**
 * This is the manager for a java.awt.Container. It allows
 * adding components before a particular component. 
 * It's purpose is to hide indexes and instead
 * work on components themselves. This is because the index of 
 * the component back in JBCF may not necessarily be the index
 * within the live component. This is because of components that
 * couldn't be instantiated due to errors, or if the container was
 * a subclass, there could be components already in the container.
 * 
 * Note: It is static. No need to have one for each container. The container
 * will always be passed in.
 */
public class ContainerManager {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	/**
	 * Return the index of the specified component. -1 if not found.
	 */
	public static int componentIndex(Container container, Component component) {
		Component[] components = container.getComponents();
		for (int i = 0; i < components.length; i++) {
			if (components[i] == component)
				return i;
		}
		
		return -1;
	}
	
	/**
	 * Add a component before the specified component. If the
	 * component is not found, then add at end.
	 */
	public static void addComponentBefore(Container container, Component addComponent, Component beforeComponent) {
		int pos = componentIndex(container, beforeComponent);
		container.add(addComponent, pos);
	}
	
	/**
	 * Add a component before the specified component with the given constraint. If the
	 * component is not found, then add at end.
	 */
	public static void addComponentBefore(Container container, Component addComponent, Object constraint, Component beforeComponent) {
		int pos = componentIndex(container, beforeComponent);
		container.add(addComponent, constraint, pos);
	}
	

}
