package com.ibm.etools.jbcf.wizard;

/*
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.*;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.*;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.part.ISetSelectionTarget;

import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.logging.util.Level;

/**
 * Page to create the JBCF Example class.
 */
public class CreatePage1 extends WizardNewFileCreationPage {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private IWorkbench workbench;
	private static int exampleCount = 1;
	private Button model1 = null;
	private Button model2 = null;
	private int modelSelected = 1;
	protected IConfigurationElement configElement;
	protected Map configData;

	private static final String REQUIRED_EXTENSION = ".java"; //$NON-NLS-1$
	public CreatePage1(IWorkbench aWorkbench, IStructuredSelection selection, IConfigurationElement configElement, Map configData) {
		super("sampleExamplePage1", selection); //$NON-NLS-1$
		if (configData != null) {
			String title = (String) configData.get("title"); //$NON-NLS-1$
			this.setTitle(title != null ? title : WizardMessages.getString("CreatePage1.CreatePage1.Title")); //$NON-NLS-1$
		}
		this.workbench = aWorkbench;
		this.configElement = configElement;
		this.configData = configData;
	}

	public void createControl(Composite parent) {
		super.createControl(parent);
		this.setFileName("Empty" + exampleCount + ".java"); //$NON-NLS-2$//$NON-NLS-1$
	}

	protected InputStream getInitialContents() {
		IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
		IResource folder = workspaceRoot.findMember(getContainerFullPath());
		IJavaElement pkg = JavaCore.create(folder);
		String packageName = null;
		if (pkg.getElementType() == IJavaElement.PACKAGE_FRAGMENT)
			packageName = pkg.getElementName();
		ByteArrayInputStream bais = null;
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			PrintStream ps = new PrintStream(baos);
			if (packageName != null) {
				ps.println("package " + packageName + ';'); //$NON-NLS-1$
				ps.println();
			}

			ps.println("public class " + getFileName().substring(0, getFileName().lastIndexOf('.')) + " {"); //$NON-NLS-1$ //$NON-NLS-2$
			ps.println('}');
			ps.close();
			bais = new ByteArrayInputStream(baos.toByteArray());
		} catch (Exception e) {
			JBCFPlugin.getPlugin().getMsgLogger().log(e, Level.WARNING);
		}
		return bais;
	}

	public boolean performFinish() {
		// add extension if non is provided 
		String fileName = getFileName();
		if (fileName != null && !fileName.endsWith(REQUIRED_EXTENSION)) {
			setFileName(fileName + REQUIRED_EXTENSION);
		}
		final IFile file = createNewFile();
		if (file != null) {
			++exampleCount; // Now that it is out there, need to update the counter for next time.
			// Need to setup classpath
			if (configData != null) {
				// First get the list of variable entries in sorted order.
				final TreeMap entries = new TreeMap();
				Iterator itr = configData.entrySet().iterator();
				while (itr.hasNext()) {
					Map.Entry mapentry = (Map.Entry) itr.next();
					// They are tested in this order because startswith("variable") would also pick up "variablesource"
					if (((String) mapentry.getKey()).startsWith("variablesource")) { //$NON-NLS-1$
						Integer i = new Integer(((String) mapentry.getKey()).substring("variablesource".length())); //$NON-NLS-1$
						if (i.intValue() > 0) {
							String[] entry = (String[]) entries.get(i);
							if (entry == null)
								entries.put(i, entry = new String[3]);
							entry[1] = (String) mapentry.getValue();
						}
					} else if (((String) mapentry.getKey()).startsWith("variableroot")) { //$NON-NLS-1$
						Integer i = new Integer(((String) mapentry.getKey()).substring("variableroot".length())); //$NON-NLS-1$
						if (i.intValue() > 0) {
							String[] entry = (String[]) entries.get(i);
							if (entry == null)
								entries.put(i, entry = new String[3]);
							entry[2] = (String) mapentry.getValue();
						}
					} else if (((String) mapentry.getKey()).startsWith("variable")) { //$NON-NLS-1$
						Integer i = new Integer(((String) mapentry.getKey()).substring("variable".length())); //$NON-NLS-1$
						if (i.intValue() > 0) {
							String[] entry = (String[]) entries.get(i);
							if (entry == null)
								entries.put(i, entry = new String[3]);
							entry[0] = (String) mapentry.getValue();
						}
					}
				}
				if (entries.size() > 0) {
					IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
					IProject project = workspaceRoot.getProject(getContainerFullPath().segment(0));
					final IJavaProject jproject = JavaCore.create(project);			
					WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
						protected void execute(IProgressMonitor monitor) throws CoreException, InterruptedException {
							try {
								monitor.beginTask(WizardMessages.getString("CreatePage1.CreatePage1.finish.begintask"), 2000); //$NON-NLS-1$
								IClasspathEntry[] raw, firstRaw = jproject.getRawClasspath();
								raw = firstRaw;
								Iterator itr = entries.values().iterator();
								while (itr.hasNext()) {
									String[] pathStrings = (String[]) itr.next();
									if (pathStrings[0] != null) {
										IPath var, varsource = null, varroot = null;
										var = new Path(pathStrings[0]);
										if (pathStrings[1] != null) {
											varsource = new Path(pathStrings[1]);
											if (pathStrings[2] != null)
												varroot = new Path(pathStrings[2]);
										}	
										IClasspathEntry pathEntry = JavaCore.newVariableEntry(var, varsource, varroot, true);
										raw = addVariableEntry(raw, pathEntry);
									}
								}
								if (raw != firstRaw) {
									// It actually changed, so put it back out.
									jproject.setRawClasspath(raw, monitor);
								}
							} finally {
								monitor.done();
							}
						}
			
					};			
					
					try {
						getContainer().run(true, true, op);
					} catch (InterruptedException e) {
						return false;
					} catch (InvocationTargetException e) {
						if (e.getTargetException() instanceof CoreException) {
							setErrorMessage(((CoreException) e.getTargetException()).getStatus().getMessage());
						}
						else {
							// CoreExceptions are handled above, but unexpected runtime exceptions and errors may still occur.
							setErrorMessage(e.getTargetException().getLocalizedMessage());
						}
						return false;
					}			
				}
			}


			// Now open the new guy.
			Shell shell = getShell();
			IWorkbenchPage page = getActivePage();
			if (shell == null || page == null) {
				return true;
			}
			final IWorkbenchPart focusPart = page.getActivePart();
			if (focusPart instanceof ISetSelectionTarget) {
				shell.getDisplay().asyncExec(new Runnable() {
					public void run() {
						ISelection selection = new StructuredSelection(file);
						((ISetSelectionTarget) focusPart).selectReveal(selection);
					}
				});
			}
			try {
				page.openEditor(file, "com.ibm.etools.jbcf.editorpart.JavaVisualEditor"); //$NON-NLS-1$
				return true;
			} catch (PartInitException e) {
				// A really bad error, but it is too late to set the error msg string, so we will pop up a msg dialog.
				ErrorDialog.openError(shell, WizardMessages.getString("CreatePage1.CreatePage1.finish.OpenError_ERROR_"), e.getMessage(), e.getStatus()); //$NON-NLS-1$
			}
		}
		return false;
	}

	protected IClasspathEntry[] addVariableEntry(IClasspathEntry[] raw, IClasspathEntry newEntry) {
		for (int i = 0; i < raw.length; i++) {
			if (newEntry.equals(raw[i])) {
				return raw;
			}
		}
		
		// Add it at the end.
		IClasspathEntry[] newRaw = new IClasspathEntry[raw.length + 1];
		System.arraycopy(raw, 0, newRaw, 0, raw.length);
		newRaw[raw.length] = newEntry;
		return newRaw;
	}

	private IWorkbenchPage getActivePage() {
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		if (window == null)
			return null;
		return window.getActivePage();
	}

	/**
	 * @see WizardNewFileCreationPage#validatePage
	 */
	protected boolean validatePage() {
		// check whether file with extension doesn't exist
		boolean valid = super.validatePage();
		if (!valid)
			return false;

		IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
		IPath containerPath = getContainerFullPath();
		IJavaProject jproject = null;
		if (containerPath != null && containerPath.segmentCount() > 0) {
			IProject project = workspaceRoot.getProject(containerPath.segment(0));
			try {
				if (!project.hasNature(JavaCore.NATURE_ID)) {
					setErrorMessage(WizardMessages.getString("CreatePage1.CreatePage1.validate.MustBeInJavaProj_ERROR_")); //$NON-NLS-1$
					return false;
				}
			} catch (CoreException e) {
				JBCFPlugin.getPlugin().getMsgLogger().log(e.getStatus(), Level.WARNING);
				return false;
			}
			jproject = JavaCore.create(project);
		} else
			return false;
		
		try {
			IResource folder = workspaceRoot.findMember(containerPath);
			IJavaElement pkg = JavaCore.create(folder);
			if (pkg == null || !jproject.isOnClasspath(pkg)) {
				setErrorMessage(WizardMessages.getString("CreatePage1.CreatePage1.validate.MustBeInJavaPkg_ERROR_")); //$NON-NLS-1$
				return false;
			}
		} catch (JavaModelException e) {
			JBCFPlugin.getPlugin().getMsgLogger().log(e.getStatus(), Level.WARNING);			
			return false;
		}

		String fileName = getFileName();
		if (fileName != null && !fileName.endsWith(REQUIRED_EXTENSION)) {
			fileName = fileName + REQUIRED_EXTENSION;
			IPath path = getContainerFullPath();

			if (path != null && workspaceRoot.exists(path.append(fileName))) {
				setErrorMessage(WizardMessages.getString("CreatePage1.CreatePage1.validate.FileExists_ERROR_")); //$NON-NLS-1$
				return false;
			}
		}
		return true;
	}
}