package com.ibm.etools.jbcf.wizard;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

/**
 * Creation Wizard for the JBCF plugins for users that wish to simple
 * beaninfo extensions. For creating a sample piece of code and for 
 * making sure the project contains the required jars and variables.
 * It will get this configuration info out of the plugin extension
 * that declares the wizard in the user's plugin.
 * 
 * It is instantiated from your plugin using the following format:
 * <pre>
 *     &lt;extension point = "org.eclipse.ui.newWizards"&gt;
 *       &lt;category
 *         id="<it>com.your.example.category</it>"
 *         name="<it>your example category name</it>"
 *         parentCategory="org.eclipse.ui.Examples">
 *       &lt;/category&gt;
 *       &lt;wizard
 *         id="<it>com.your.example.name</it>"
 *         name="<it>your example name</it>"
 *         category="org.eclipse.ui.Examples/<it>com.your.example.category</it>"
 *         icon="<it>your icon in your plugin</it>"
 *         &lt;class
 *           plugin="com.ibm.etools.jbcf"
 *           class="com.ibm.etools.jbcf.wizard.CreationWizard"&gt;
 *             &lt;parameter name="title" value="<it>title onf creation path</it>"/&gt;
 *             &lt;parameter name="variable<it>nn</it>" value="<it>your classpath variable entry</it>"/&gt;
 *             &lt;parameter name="variablesource<it>nn</it>" value="<it>your classpath variable entry for source for the entry code</it>"/&gt; 
 *             &lt;parameter name="variableroot<it>nn</it>" value="<it>your classpath variable entry for source root for the entry code</it>"/&gt;  * 
 *         &lt;/class&gt;
 *         &lt;description&gt;<it>your description goes here</it>&lt;/description&gt;
 *       &lt;/wizard&gt;
 *     &lt;/extension&gt;
 * </pre>
 * 
 * In the above "variable<it>nn</it> is a class path variable entry. There can be more than one and they will
 * be added to the end of the classpath for the project (if not already there) in the order of the <it>nn</it>
 * value. Note: If any all already there, they won't be moved and so may be out of order. The number can be any number greater than 0. It doesn't need to be two digits. The value will be added as a Variable
 * entry in the classpath. There can be more than one. Variable source and root are not required, it can be omitted for any one, but it should
 * have a matching <it>nn</it> value.
 */
public class CreationWizard extends Wizard implements INewWizard, IExecutableExtension {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private CreatePage1 examplePage = null;
	private IStructuredSelection selection;
	private IWorkbench workbench;

	/**
	 * Constructor for ExampleCreationWizard.
	 */
	public CreationWizard() {
		super();
		
	}

	public void addPages() {
		examplePage = new CreatePage1(workbench, selection, configElement, configData);
		addPage(examplePage);
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish() {
		return examplePage.performFinish();
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench;
		this.selection = selection;
	}

	protected Map configData;
	protected IConfigurationElement configElement;
	/**
	 * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(IConfigurationElement, String, Object)
	 */
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
		configElement = config;
		if (data instanceof Map)
			configData = (Map) data;

		String iconname = config.getAttribute("icon"); //$NON-NLS-1$
		if (iconname != null) {
			try {
				this.setDefaultPageImageDescriptor(ImageDescriptor.createFromURL(new URL(config.getDeclaringExtension().getDeclaringPluginDescriptor().getInstallURL(), iconname))); //$NON-NLS-1$
			} catch (MalformedURLException e) {
			}			
		}
	}

}
