package com.ibm.etools.jbcf.wizards;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.JavaPluginImages;
import org.eclipse.jdt.internal.ui.wizards.NewClassCreationWizard;
import org.eclipse.jdt.internal.ui.wizards.NewWizardMessages;

import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.jbcf.codegen.CodeGenPlugin;
import com.ibm.etools.jbcf.codegen.CodegenMessages;
import com.ibm.etools.jbcf.editorpart.JavaVisualEditorPart;

/**
 */
public class VisualClassExampleWizard extends NewClassCreationWizard implements IExecutableExtension  {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private VisualClassExampleWizardPage fPage;	
	private String fExampleClassName;
	
	public VisualClassExampleWizard() {
		super();
		setDefaultPageImageDescriptor(JBCFPlugin.getWizardTitleImageDescriptor());
		setDialogSettings(JavaPlugin.getDefault().getDialogSettings());
		setWindowTitle(CodegenMessages.getPluginPropertyString("example.label")); //$NON-NLS-1$
	}

	/*
	 * @see Wizard#createPages
	 */	
	public void addPages() {
		fPage= new VisualClassExampleWizardPage();
		addPage(fPage);
		fPage.init(getSelection());
		fPage.setSuperClass("javax.swing.JFrame",false); //$NON-NLS-1$
		fPage.setTypeName(fExampleClassName,false);
	}	

	/*
	 * @see Wizard#performFinish
	 */		
	public boolean performFinish() {
		if (finishPage(fPage.getRunnable())) {
			ICompilationUnit cu= fPage.getCreatedType().getCompilationUnit();
			if (cu.isWorkingCopy()) {
				cu= (ICompilationUnit)cu.getOriginalElement();
				// Now get the compilation unit from the ToDoList.java file in the examples folder
			}	
			try {
				IResource resource= cu.getUnderlyingResource();
				selectAndReveal(resource);
				openResource(resource);
			} catch (JavaModelException e) {
				JavaPlugin.log(e);
				// let pass, only reveal and open will fail
			}
			return true;
		}
		return false;		
	}		
	protected void openResource(IResource resource){
		JavaVisualEditorPart.openResource(resource,getShell().getDisplay());
	}
	
	public void setInitializationData(IConfigurationElement element,String string,Object object){
		if ( object instanceof String ) {
			fExampleClassName = (String)object;
		}
	}

}