package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.common.command.*;
import java.util.ListIterator;

/**
 *  This command will undo its commands in the same order applied.
 *
 *  However any Chained commands will undo in reverse order since
 *  that can't be controlled.
 */
public class ForwardUndoCompoundCommand extends CompoundCommand {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	public ForwardUndoCompoundCommand() {
		super();
	}

	public ForwardUndoCompoundCommand(String label) {
		super(label);
	}

	/**
	 * This calls {@link Command#undo} for each command in the list, in forward order.
	 */
	public void undo() {

		for (ListIterator commands = commandList.listIterator(); commands.hasNext();) {
			try {
				Command command = (Command) commands.next();
				command.undo();
			} catch (RuntimeException exception) {

				// Skip over the command that threw the exception.
				//
				commands.previous();

				// Iterate backwards over the undone commands to redo them.
				//
				while (commands.hasPrevious()) {
					Command command = (Command) commands.previous();
					command.redo();
				}

				throw exception;
			}
		}

	}

}