package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import org.eclipse.ui.views.properties.IPropertyDescriptor;
/**
 * This is an extension of IPropertyDescriptor which provides added
 * accessers for the AbstractPropertySheetEntry over the default
 * property descriptor.
 */
public interface IEToolsPropertyDescriptor extends IPropertyDescriptor {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	/**
	 * Should an entry for this property be expandable.
	 * In other words, if the first of the compatible descriptors 
	 * for a property sheet entry return true to this method,
	 * then children (sub-entries) to this entry will be
	 * created. The plus sign will be placed on the entry.
	 * Setting false is useful when the cell editor does complete 
	 * customization of the property and it is not desired
	 * to have sub-entries further customize it.
	 *
	 * Descriptors that are not IEToolsPropertyDescriptors will
	 * be treated as if this accesser returns true. This will
	 * preserve the default behavior.
	 */
	public boolean isExpandable();
	
	/**
	 * Should an entry for this property allow nulls.
	 * In other words, if the first of the compatible descriptors 
	 * for a property sheet entry return true to this method,
	 * then null settings won't be allowed. It it is false,
	 * then the cell editor and any applied validators will determine
	 * if null is invalid or not.
	 *
	 * Descriptors that are not IEToolsPropertyDescriptors will
	 * be treated as if this accesser returns true. This will
	 * preserve the default behavior.
	 */
	public boolean areNullsInvalid();	
	
	/**
	 *Is this property read only and not allowing editing
	 *In other words, if the first of the compatible descriptors
	 *for a property sheet entry return true to this method,
	 *and the property sheet doesn't want to show read only properties,
	 *No new entry is created and displayed for this property in the property sheet.
	 */
	public boolean isReadOnly();

}