package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import org.eclipse.jface.viewers.ICellEditorValidator;
/**
 * This is a special validator that can wrapper several validators. It allows
 * the combination of validators. This could be useful when several things
 * need to be checked for and don't want to code them all in one.
 * It allows access to the wrappered validators so that they can be touched,
 * such as they may be ISourced and so we need to set the ISourced onto all of them.
 * 
 * Note: The implementation could itself be a complete validator, but this won't
 * be returned in the list of validators.
 */

public interface IWrapperedValidator extends ICellEditorValidator {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	/**
	 * Return the array of validators.
	 * Note: It may return null, meaning no wrappered validators. This 
	 * could happen if the implementation does its own validatation. That
	 * way it may validate it and pass it on to any of its wrappered validators.
	 */
	public ICellEditorValidator[] getValidators();
	public void setValidators(ICellEditorValidator[] validators);

}