package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import org.eclipse.jface.viewers.ICellEditorValidator;
/**
 * This is a validator which can wrapper another validator that
 * says nulls are invalid. It will first check to see if it is
 * a null, and if it is, it will return an invalid message.
 * If it is not a null, then it will pass onto the wrappered
 * validator (if any) and let it determine if the value is valid.
 */
public class NoNullsValidator implements IWrapperedValidator {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected ICellEditorValidator[] fWrapperedValidators;
	
	public NoNullsValidator() {
	}

	public NoNullsValidator(ICellEditorValidator validator) {
		fWrapperedValidators = validator != null ? new ICellEditorValidator[] {validator} : null;
	}
	
	public NoNullsValidator(ICellEditorValidator[] validators) {
		fWrapperedValidators = validators;
	}
	
	public ICellEditorValidator[] getValidators() {
		return fWrapperedValidators;
	}
	
	public void setValidators(ICellEditorValidator[] validators) {
		fWrapperedValidators = validators;
	}
	
	public String isValid(Object value){
		if (value == null)
			return PropertysheetMessages.getString(PropertysheetMessages.NULL_INVALID);

		if (fWrapperedValidators != null) {			
			for (int i=0; i<fWrapperedValidators.length; i++) {
				if (fWrapperedValidators[i] != null) {
					String result = fWrapperedValidators[i].isValid(value);
					if (result != null && result.length() > 0)
						return result;	// One of them has stopped it
				}
			}
		}
		return null;
	}
	
}