package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import org.eclipse.jface.action.Action;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.ui.views.properties.IPropertySheetEntry;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import java.net.*;
import java.util.Iterator;
/**
 * This action is used to set the active entries to null.
 */
public class SetToNullAction extends EToolsPropertySheetAction {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected static ImageDescriptor NULL_RESET_IMAGE;
		
	public SetToNullAction(EToolsPropertySheetPage page) {
		super(page);
		if ( NULL_RESET_IMAGE == null ) {
			String url = "platform:/plugin/com.ibm.etools.propertysheet/images/nullreset16.gif"; //$NON-NLS-1$
			try {
				NULL_RESET_IMAGE = ImageDescriptor.createFromURL(new URL(url));
			} catch (MalformedURLException e) {
				NULL_RESET_IMAGE = ImageDescriptor.getMissingImageDescriptor();
			}		
		};
		setImageDescriptor(NULL_RESET_IMAGE);
		setText(PropertysheetMessages.getString(PropertysheetMessages.SET_NULLS_LABEL));
		setToolTipText(PropertysheetMessages.getString(PropertysheetMessages.SET_NULLS_TOOLTIP));		
	}		

	/**
	 * Call when selection has been changed. The EToolsProeprtySheetPage will do this for us.
	 */
	public void selectionChanged(ISelection selection) {
		boolean enable = false;
		IStructuredSelection selected = (IStructuredSelection) getSelection();
		if (!selected.isEmpty()) {
			IDescriptorPropertySheetEntry entry = (IDescriptorPropertySheetEntry) selected.getFirstElement();
			// Can't have nulls if the editor is not active or if the descriptor says no nulls.
			enable = !entry.areNullsInvalid();
		}
		setEnabled(enable);
	}

	/**
	 * @see Action
	 */
	public void run() {
		IStructuredSelection selected = (IStructuredSelection) getSelection();
		if (!selected.isEmpty()) {
			IDescriptorPropertySheetEntry entry = (IDescriptorPropertySheetEntry) selected.getFirstElement();
			entry.setToNull();
		}		
	}	
}