package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.ui.views.properties.IPropertySheetEntry;

/**
 * This action is used to toggle the property sheet viewer
 * to show or not show read only property values
 */

public class ShowReadOnlyAction extends EToolsPropertySheetAction {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	public ShowReadOnlyAction(EToolsPropertySheetPage page) {
		super(page);
		setText(PropertysheetMessages.getString(PropertysheetMessages.SHOW_READ_ONLY_LABEL));
		setChecked(false);
	}		

	public void run() {
		IPropertySheetEntry root = getRootEntry();
		if (root instanceof IDescriptorPropertySheetEntry) {
			propertySheet.deactivateCellEditor();	// Deactivate the current cell editor because we are rebuilding everything under there feet.
			IDescriptorPropertySheetEntry rootEntry = (IDescriptorPropertySheetEntry) root;
			rootEntry.setShowReadOnly(isChecked());
			rootEntry.refreshFromRoot();
		}		
	}
	
	public void setChecked(boolean value) {
		super.setChecked(value);
		valueChanged(value);
	}
	
	private void valueChanged(boolean on) {
		setToolTipText(on ? PropertysheetMessages.getString(PropertysheetMessages.SHOW_READ_ONLY_HIDE_TOOLTIP) : PropertysheetMessages.getString(PropertysheetMessages.SHOW_READ_ONLY_SHOW_TOOLTIP));
	}

}