package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import org.eclipse.jface.action.Action;
import org.eclipse.ui.views.properties.IPropertySheetEntry;
/**
 * This action is used to toggle the property
 * to show or not show whether an entry is the
 * default value. The standard way is to add an asterick in front of it.
 */
public class ShowSetValuesAction extends EToolsPropertySheetAction {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	public ShowSetValuesAction(EToolsPropertySheetPage page, boolean showSetValues) {
		super(page);
		setText(PropertysheetMessages.getString(PropertysheetMessages.SHOW_SET_VALUES_LABEL));
		setChecked(showSetValues);
	}		

	/**
	 * @see Action
	 */
	public void run() {
		IPropertySheetEntry root = getRootEntry();
		if (root instanceof IDescriptorPropertySheetEntry) {
			propertySheet.deactivateCellEditor();	// Deactivate the current cell editor because we are rebuilding everything under there feet.			
			IDescriptorPropertySheetEntry rootEntry = (IDescriptorPropertySheetEntry) root;
			rootEntry.setShowSetValues(isChecked());
			rootEntry.refreshFromRoot();
		}		
	}
	
	/**
	 * @see Action
	 */
	public void setChecked(boolean value) {
		super.setChecked(value);
		valueChanged(value);
	}
	
	private void valueChanged(boolean on) {
		setToolTipText(on ? PropertysheetMessages.getString(PropertysheetMessages.SHOW_NULLS_HIDE_TOOLTIP) : PropertysheetMessages.getString(PropertysheetMessages.SHOW_NULLS_SHOW_TOOLTIP));
	}

}