package com.ibm.etools.propertysheet;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.widgets.Composite;

/**
 * An text cell editor that allows non-strings to be
 * sent in without throwing an exception. It just
 * sets an error message.
 */
public class StringCellEditor extends TextCellEditor {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected boolean fWasNull = false;
		
	public StringCellEditor(Composite parent) {
		super(parent);
	}	
	
	/**
	 * Verify that the value is a string, and if it isn't 
	 * then set an error msg. If it is a string,
	 * let the super handle it.
	 */
	protected boolean isCorrect(Object value) {
		if (value == null || value instanceof String) {
			return super.isCorrect(value);
		} 
		
		setErrorMessage(PropertysheetMessages.getString(PropertysheetMessages.NOT_STRING));
		return false;
	}
	
	/**
	 * The value is being requested. If the original value
	 * was null, and if the new value is "", then return null.
	 */
	protected Object doGetValue() {
		Object v = super.doGetValue();
		if (fWasNull && "".equals(v)) //$NON-NLS-1$
			return null;
		else
			return v;
	}
	
	/**
	 * A value is being set, allow non-string with error msg
	 * instead of throwing an exception. (duh) Setting
	 * anything other than a string will set the value
	 * to "" so that something can be displayed. The isCorrect
	 * routine will take care of the validity of the value.
	 *
	 * @param value a text string (type <code>String</code>)
	 */
	protected void doSetValue(Object value) {
		fWasNull = value == null;
		if (value instanceof String)
			super.doSetValue(value);
		else
			super.doSetValue(""); //$NON-NLS-1$
	}
}


