package com.ibm.etools.proxy;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


/**
 * Bean Proxy interface. This is the
 * root interface for any bean proxy instance.
 * Creation date: (12/3/99 11:37:01 AM)
 * @author: Joe Winchester
 */
public interface IBeanProxy {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
/**
 * equals: Equal if:
 *         1) This proxy == (identity) to the other object
 *         2) Else if other is an IBeanProxy, then if
 *            equals on the server.
 *         3) If this is a constant proxy and the other is too or is a constant
 *            value (e.g. IStringBeanProxy.equals(String), then true if values are equals.
 */
public boolean equals(Object anObject);
  
/**
 * Return the registry this proxy was created with.
 */
public ProxyFactoryRegistry getProxyFactoryRegistry();
/**
 * Return the proxied class of the bean
 * Creation date: (12/3/99 11:37:01 AM)
 * @author: Joe Winchester
 */
public IBeanTypeProxy getTypeProxy();
/**
 * Answer if this proxy is wrappering a null.
 */
public boolean isNullProxy();
/**
 * Return a string representation of the bean itself
 * This could be done by finding the toString() method by a findMethod()
 * on the type proxy, etc... but toString() is so ubiquitous that it is
 * explicitly declared on the IBeanProxy interface
 * Creation date: (12/3/99 11:37:01 AM)
 * @author: Joe Winchester
 */
public String toBeanString();
/**
 * Is this bean still valid? It could be invalid because it
 * was specifically released but someone is still holding onto it.
 */
public boolean isValid();
}
