package com.ibm.etools.proxy;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
/**
 * This interface is used as a registration interface for a proxy vm implementation.
 * The implementation is called whenever it is required to startup a vm implementation.
 * This interface must be implemented by classes registered in the "com.ibm.etools.proxy.proxyImplementations"
 * extension point.
 */

public interface IRegistration {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	/**
	 * This will be called to start vm implementation. It will be called
	 * from the Proxy plugin startVM request from the VM that the project
	 * is configured to run.
	 *
	 * contribs: Contributors to use, may be null.
	 * attachAWT: Whether AWT should be attached to this implementation.
	 * project: Project to start on. This will not be null and it will be a JavaProject.
	 * vmName: The name to give to this implementation (this will show up in the debugger in the normal implementation case).
	 * pm: ProgressMonitor to use while starting the implementation.
	 */
	public ProxyFactoryRegistry startImplementation(IConfigurationContributor[] contribs, boolean attachAWT, IProject project, String vmName, IProgressMonitor pm) throws CoreException;
}