package com.ibm.etools.proxy.common.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.proxy.common.CommandException;
/**
 * This is an exception that is thrown when the processing
 * of data was invalid for the command. 
 * There will be a flag indicting error type, and a flag
 * indicating whether the connection is recoverable or not.
 * If not recoverable, then the connection needs to be closed.
 *
 */
public class UnexpectedCommandException extends CommandException {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	private final Object fErrorType;
	private final boolean fRecoverable;
	
	public UnexpectedCommandException(Object errorType, boolean recoverable) {
		fErrorType = errorType;
		fRecoverable = recoverable;
	}		
	
	public UnexpectedCommandException(Object errorType, boolean recoverable, Object data) {
		super(data);
		fErrorType = errorType;
		fRecoverable = recoverable;
	}
	
	public UnexpectedCommandException(Object errorType, boolean recoverable, String msg, Object data) {
		super(msg, data);
		fErrorType = errorType;
		fRecoverable = recoverable;
	}

	public Object getErrorType() {
		return fErrorType;
	}
	
	public boolean isRecoverable() {
		return fRecoverable;
	}
	
	public String getMessage() {
		String m = super.getMessage();
		if (fErrorType != null) {
			String dataMsg = getExceptionData() != null ? " Errordata="+getExceptionData().toString() : "";	//$NON-NLS-1$ //$NON-NLS-2$
			if (m == null)
				return "Errortype="+fErrorType.toString() + dataMsg; //$NON-NLS-1$
			else
				return m + ':' + "Errortype="+fErrorType.toString() + dataMsg; //$NON-NLS-1$
		} else
			return m;
	}
}