package com.ibm.etools.proxy.initParser.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.awt.Color;
import java.lang.reflect.Array;
import java.util.Dictionary;
import java.util.HashMap;

import com.ibm.etools.proxy.initParser.InitializationStringEvaluationException;
import com.ibm.etools.proxy.initParser.InitializationStringParser;

import org.eclipse.core.boot.IPlatformRunnable;

public class TestInitializationStringParser implements IPlatformRunnable {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected static int passed = 0;
	protected static int failed = 0;
	protected static int passedmaybe = 0;
	protected static int failedWithoutExceptions = 0;
	/**
	 * TestInitializationStringParser constructor comment.
	 */
	public TestInitializationStringParser() {
		super();
	}
	/**
	 * TestInitializationStringParser constructor comment.
	 */
	public static void main(String[] args) {
		TestInitializationStringParser test = new TestInitializationStringParser();

		try {
			test.run(args);
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		System.exit(0);	// Needed so that it terminates. We had created some windows, so they keep the process going.
	}

	/**
	 * @see IPlatformRunnable#run(Object)
	 */
	public Object run(Object args) throws Exception {

		passed = failed = passedmaybe = 0;
		System.out.println("-----------------------------------------------------------------------------");
		System.out.println("- OK		Perfect identity match == between parsed and expected result");
		System.out.println("- OK=		Not identity but .equals matches between parsed and expected");
		System.out.println("- OK?		Not .equals but both same class type and toString() matches");
		System.out.println("- OK??		Not .equals but both same class type and toString() does not match");
		System.out.println("- FAIL??	Parsed and Expected same classes but toString() does not match");
		System.out.println("- FAIL		Parsed and Expected are not of the same type");
		System.out.println("- FAIL**	Failure without exception thrown");
		System.out.println("- FAIL$$	Exception was expected Failure without exception thrown");		
		System.out.println("------------------- Begin tests ---------------------------------------------");
		needsCodingTests();		
		runBlockTests();
		runArrayConstructorTests();
		runMultiArgConstructorTests();
		runMultiArgStaticMethodTests();
		runMultiArgInstanceTests();
		runNumberTests();
		runCastTests();
		runExceptionTests();
		runLiteralTests();
		runStringTests();
		runCharTests();
		runJFCTests();
		runOverloadingTests();
		runBorderTests();		
		System.out.println("------------------- End tests -----------------------------------------------");
		System.out.println("- Failed= " + failed);
		System.out.println("- Passed= " + passed + " probable passes= " + passedmaybe);
		if ( failedWithoutExceptions > 0 ) {
			System.out.println("- Failed without throwing exceptions= " + failedWithoutExceptions);
			System.out.println("  These must be fixed to throw exceptions otherwise the BeanProxyAdapter processes them as a good value");
		}
		return null;
	}
	public static void needsCodingTests() {
		// Doing math is not coded yet.  We don't expect to encounter any strings like this as part of GA
//		testInitString("10 + 20", new Integer(10 + 20));
		// Inner classes need coding but we have a test to make sure we throw an explicit exception
		testInitString("new javax.swing.table.DefaultTableModel(){}",new javax.swing.table.DefaultTableModel(){},true);			
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get((float)12,(float)24,(float)50)",com.ibm.etools.proxy.initParser.test.NavigationParameters.get((float)12,(float)24,(float)50));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters((float)12,(float)24,(float)50)",new com.ibm.etools.proxy.initParser.test.NavigationParameters((float)12,(float)24,(float)50));						
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set((float)12,(float)24,(float)50)",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set((float)12,(float)24,(float)50));
	}
	public static void runBlockTests() {
		System.out.println("----- Test Block expressions ------------------------------------------");		
		testInitString(
			"(((new com.ibm.etools.proxy.initParser.test.NavigationParameters(3)).setElemAt(\"accountStatementDetails\",0)).setElemAt(new com.ibm.etools.proxy.initParser.test.NavigationParameters(\"OpnBalance\",\"OpnBalance\",null,false,false,true,50),1)).setElemAt(new com.ibm.etools.proxy.initParser.test.NavigationParameters(\"OpnDate\",\"OpnDate\",null,false,false,true,50),2);",
			(((new NavigationParameters(3)).setElemAt("accountStatementDetails",0)).setElemAt(new NavigationParameters("OpnBalance","OpnBalance",null,false,false,true,50),1)).setElemAt(new NavigationParameters("OpnDate","OpnDate",null,false,false,true,50),2));
		testInitString(
			"(((new com.ibm.etools.proxy.initParser.test.NavigationParameters(3)).setElemAt(\"accountStatementDetails\",0)).setElemAt(new com.ibm.etools.proxy.initParser.test.NavigationParameters(\"OpnBalance\",\"OpnBalance\",null,false,false,true,50),1))",
			(((new NavigationParameters(3)).setElemAt("accountStatementDetails",0)).setElemAt(new NavigationParameters("OpnBalance","OpnBalance",null,false,false,true,50),1)));
		testInitString(
			"((new com.ibm.etools.proxy.initParser.test.NavigationParameters(3)).setElemAt(new com.ibm.etools.proxy.initParser.test.NavigationParameters(\"OpnBalance\",\"OpnBalance\",null,false,false,true,50),1))",
			((new NavigationParameters(3)).setElemAt(new NavigationParameters("OpnBalance","OpnBalance",null,false,false,true,50),1)));
		testInitString(
			"((new com.ibm.etools.proxy.initParser.test.NavigationParameters(3)).setElemAt(\"accountStatementDetails\",0))",
			((new NavigationParameters(3)).setElemAt("accountStatementDetails",0)));			
		testInitString(
			"(((new String(\"Frog\")).replace('o','i')).replace('i','a')).replace('a','o')",
			(((new String("Frog")).replace('o','i')).replace('i','a')).replace('a','o'));
		testInitString(
			"((((new String(\"Frog\")).replace('o','i')).replace('i','a')).replace('a','o')).replace('o','e')",
			((((new String("Frog")).replace('o','i')).replace('i','a')).replace('a','o')).replace('o','e'));
		testInitString(
			"(new String(\"Frog\").toLowerCase()).toUpperCase()",
			(new String("Frog").toLowerCase()).toUpperCase()
			);
		testInitString(
			"((new String(\"Frog\").toLowerCase()).toUpperCase()).toLowerCase()",
			((new String("Frog").toLowerCase()).toUpperCase()).toLowerCase()
			);
		testInitString(
			"(((new String(\"Frog\").toLowerCase()).toUpperCase()).toLowerCase()).toUpperCase()",
			(((new String("Frog").toLowerCase()).toUpperCase()).toLowerCase()).toUpperCase()
			);
		testInitString(
			"(new String(\"Frogs\").concat(\"are\")).concat(\"cool\")",
			(new String("Frogs").concat("are")).concat("cool")
			);
		testInitString(
			"((new String(\"Frogs\").concat(\"are\")).concat(\"very\")).concat(\"cool\")",
			((new String("Frogs").concat("are")).concat("very")).concat("cool")
			);	
		testInitString(
			"(((new String(\"Frogs\").concat(\"just\")).concat(\"rule\")).concat(\"cool\")).concat(\"kingdom\")",
			(((new String("Frogs").concat("just")).concat("rule")).concat("cool")).concat("kingdom")
			);									
		testInitString(
			"((new String(\"Frogs\").concat(\"just\")).concat(new String(\"totally\")).concat(new String(\"Rock\")))",
			((new String("Frogs").concat("just")).concat(new String("totally")).concat(new String("Rock")))
			);												
	}
	public static void runArrayConstructorTests() {
//		// Test array construction
		System.out.println("----- Test Array expressions ------------------------------------------");
		testInitString("new String[2]",new String[2]);
		testInitString("new String[2][2]",new String[2][2]);		
		testInitString("new java.awt.Color[3]",new java.awt.Color[3]);
		testInitString("new java.awt.Color[3][4]",new java.awt.Color[3][4]);		
		testInitString("new int[3]", new int[3]);
		// Needs fixing
//		testInitString("new String[new Integer(3).intValue()]",new String[new Integer(3).intValue()]);		
		testInitString("new int[] {-2,3}",new int[]{-2,3});
		testInitString("new int[] {-2,3,}",new int[]{-2,3,});		
		testInitString("new double[] {(-2), 3.0 }",new double[] {(-2), 3.0 });
		testInitString("new String[] {\"1\",\"2\"}",new String[] {"1","2"});		
		testInitString("new String[] {\"Frog\",\"Dog\",\"Cow\"}",new String[] {"Frog","Dog","Cow"});				
		testInitString("new java.awt.Color[] { java.awt.Color.red , new java.awt.Color(0,0,0) }",new java.awt.Color[] { java.awt.Color.red , new java.awt.Color(0,0,0) });
		testInitString("new int[][] { { 2 , -3 } , { 4 , 5 } }",new int[][] { { 2 , -3 } , { 4 , 5 } });
		testInitString("new int[][] {{1,2,3},{4,5,6}}",new int[][]{{1,2,3},{4,5,6}});		
		// Much to my surprise, extra commands are valid syntax at the end of the arguments even without any following argument
		// the resolve that occurs by the Visual Editor actually inserts these, so we need tests to make sure we can deal with them correctly
		testInitString("new int[][] { {2 ,(-3),} , { 4 , 5,}}",new int[][] { { 2 , (-3),} , { 4 , 5,}});
		testInitString("new int[][] { {2 ,(-3),} , { 4 , 5,},}",new int[][] { { 2 , (-3),} , { 4 , 5,},});		
		testInitString("new double[][] { { 2 , 3 } , { 4 , 5 } }",new double[][] { { 2 , 3 } , { 4 , 5 } });		
		testInitString("new double[][]{{ 5.0, -2.0, 5.0, -1.0, -1.0, 5.0, -2.0, 5.0 }, { 5.0, -2.0, 5.0, -2.0, 5.0, -1.0, 5.0, -2.0, 5.0 }}",new double[][]{{ 5.0, -2.0, 5.0, -1.0, -1.0, 5.0, -2.0, 5.0 }, { 5.0, -2.0, 5.0, -2.0, 5.0, -1.0, 5.0, -2.0, 5.0 }});
		testInitString("new int[][] {{1,2,3},{3,4,5}}",new int[][] {{1,2,3},{3,4,5}});
		testInitString("new int[][] { { 1 , 2 } , { 3 , 4 } }",new int[][] {{1,2},{3,4}});		
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new int[][] {{1,2,3},{3,4,5}})",new com.ibm.etools.proxy.initParser.test.NavigationParameters(new int[][] {{1,2,3},{3,4,5}}));
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new int[][] {{1,2,3,},{3,4,5,}})",new com.ibm.etools.proxy.initParser.test.NavigationParameters(new int[][] {{1,2,3,},{3,4,5,}}));		
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new int[][] {{1,2,3,},{3,4,5,},})",new com.ibm.etools.proxy.initParser.test.NavigationParameters(new int[][] {{1,2,3,},{3,4,5,},}));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[]{1,2,3})",new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[]{1,2,3}));
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[]{1.0,2,-3.5})",new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[]{1.0,2,-3.5}));
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[]{-1,2,-0.5})",new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[]{-1,2,-0.5}));
//		This test is one that is similar to the type of constructor used by TableLayout for SWA
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[][]{{ 5.0, -2.0, 5.0, -1.0, -1.0, 5.0, -2.0, 5.0 }, { 5.0, -2.0, 5.0, -2.0, 5.0, -1.0, 5.0, -2.0, 5.0 }})",
						new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[][]{{ 5.0, -2.0, 5.0, -1.0, -1.0, 5.0, -2.0, 5.0 }, { 5.0, -2.0, 5.0, -2.0, 5.0, -1.0, 5.0, -2.0, 5.0 }}));
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[][]{{ 5.0, -2.0, 5.0, -1.0, -1.0, 5.0, -2.0, 5.0 , }, { 5.0, -2.0, 5.0, -2.0, 5.0, -1.0, 5.0, -2.0, 5.0 , } , })",
						new com.ibm.etools.proxy.initParser.test.NavigationParameters(new double[][]{{ 5.0, -2.0, 5.0, -1.0, -1.0, 5.0, -2.0, 5.0 , }, { 5.0, -2.0, 5.0, -2.0, 5.0, -1.0, 5.0, -2.0, 5.0 , } , }));
		// Needs fixing - We don't have dimensions great than 2 working yet
//		testInitString("new int[][][] { { { 1 , 2 } , { 3 , 4 } }, { { 5 , 6 } , { 7 , 8 } } }",new int[][][] { { { 1 , 2 } , { 3 , 4 } }, { { 5 , 6 } , { 7 , 8 } } });

	}	
	public static void runMultiArgStaticMethodTests(){
		System.out.println("----- Multiple argument static method calls ---------------------------");		
		// One tests fails here
		// Test static method calls with a number of arguments
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get(false, false)",com.ibm.etools.proxy.initParser.test.NavigationParameters.get(false,false));
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get(12, 24)",com.ibm.etools.proxy.initParser.test.NavigationParameters.get(12,24));		
		// JRW - This needs fixing		
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get((float)12,(float)24,(float)50)",com.ibm.etools.proxy.initParser.test.NavigationParameters.get((float)12,(float)24,(float)50));				
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get(12.5d, 24.5d)",com.ibm.etools.proxy.initParser.test.NavigationParameters.get(12.5d,24.5d));				
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get(\"a\", \"b\")",com.ibm.etools.proxy.initParser.test.NavigationParameters.get("a","b"));				
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get(java.awt.Color.red, java.awt.Color.red)",com.ibm.etools.proxy.initParser.test.NavigationParameters.get(java.awt.Color.red,java.awt.Color.red));				
		testInitString("com.ibm.etools.proxy.initParser.test.NavigationParameters.get(new java.awt.Color(255,0,0), new java.awt.Color(255,0,0))",com.ibm.etools.proxy.initParser.test.NavigationParameters.get(new java.awt.Color(255,0,0),new java.awt.Color(255,0,0)));										
	}
	public static void runMultiArgConstructorTests(){
		System.out.println("----- Multiple Argument constructors-----------------------------------");				
		// Test a number of constructors where there are multiple arguments - this has caused us problems in the past
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(false, false)",new com.ibm.etools.proxy.initParser.test.NavigationParameters(false,false));
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(12, 24)",new com.ibm.etools.proxy.initParser.test.NavigationParameters(12,24));		
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters((float)12,(float)24,(float)50)",new com.ibm.etools.proxy.initParser.test.NavigationParameters((float)12,(float)24,(float)50));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(12.5d, 24.5d)",new com.ibm.etools.proxy.initParser.test.NavigationParameters(12.5d,24.5d));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(\"a\", \"b\")",new com.ibm.etools.proxy.initParser.test.NavigationParameters("a","b"));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(java.awt.Color.red, java.awt.Color.red)",new com.ibm.etools.proxy.initParser.test.NavigationParameters(java.awt.Color.red,java.awt.Color.red));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(new java.awt.Color(255,0,0), new java.awt.Color(255,0,0))",new com.ibm.etools.proxy.initParser.test.NavigationParameters(new java.awt.Color(255,0,0),new java.awt.Color(255,0,0)));						
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters(\"processor\", 0, \"customerSearchClientOp\", 0, \"2343434\", \"3443234234\", \"depositRepFmt\", \"depositRepFmt\", \"\", \"\", 0, 0, 0, 0, false, false)",new com.ibm.etools.proxy.initParser.test.NavigationParameters("processor", 0, "customerSearchClientOp", 0, "2343434", "3443234234", "depositRepFmt", "depositRepFmt", "", "", 0, 0, 0, 0, false, false));		
	}
	public static void runMultiArgInstanceTests(){
		System.out.println("----- Multiple argument instance methods-------------------------------");		
		// Test a number of instance methods where there are multiple arguments - this has caused us problems in the past
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(false, false)",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(false,false));
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(12, 24)",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(12,24));		
		// JRW - This needs fixing				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set((float)12,(float)24,(float)50)",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set((float)12,(float)24,(float)50));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(12.5d, 24.5d)",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(12.5d,24.5d));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(\"a\", \"b\")",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set("a","b"));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(java.awt.Color.red, java.awt.Color.red)",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(java.awt.Color.red,java.awt.Color.red));				
		testInitString("new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(new java.awt.Color(255,0,0), new java.awt.Color(255,0,0))",new com.ibm.etools.proxy.initParser.test.NavigationParameters().set(new java.awt.Color(255,0,0),new java.awt.Color(255,0,0)));
	}	
	public static void runNumberTests() {
		System.out.println("----- Numeric expressions -- ------------------------------------------");				
		testInitString("10", new Integer(10));
		testInitString("10d", new Double(10d));
		testInitString("-10d", new Double(-10d));
		testInitString("10f", new Float(10f));
		testInitString("10l", new Long(10l));
		testInitString("10.75", new Double(10.75));
		testInitString("-10.75", new Double(-10.75));
		testInitString(" 10.75 ", new Double(10.75));
		testInitString("-10.75d", new Double(-10.75d));
	}
	public static void runCastTests() {
		System.out.println("----- Cast statements -------------------------------------------------");				
		// Test for cast expressions in numbers and spaces around numbers and casts
		testInitString("new Float((float)10)", new Float((float) 10));
		testInitString("new Float( (float) 10 )", new Float((float) 10));
		testInitString("new Float((float)10.3)", new Float((float) 10.3));
		testInitString("new Float((float)5.3)", new Float((float) 5.3));
		testInitString("new Float( (float) 10.3 )", new Float((float) 10.3));
		testInitString("(String)null", (String) null);	
		testInitString("(String)\"hi\"", (String) "hi");
		testInitString("new javax.swing.JLabel( (String) null)", new javax.swing.JLabel((String) null));			
		testInitString("(short)10", new Short((short) 10));	
		testInitString("(float)10.3", new Float((float)10.3));	
		// Casts statements with qualified expression used to fail		
		testInitString("(java.lang.String)com.ibm.etools.proxy.initParser.test.NavigationParameters.getReversed(\"Frog\")",(String)com.ibm.etools.proxy.initParser.test.NavigationParameters.getReversed("Frog"));
		testInitString("(String)com.ibm.etools.proxy.initParser.test.NavigationParameters.getReversed(\"Frog\")",(String)com.ibm.etools.proxy.initParser.test.NavigationParameters.getReversed("Frog"));
		testInitString("(java.awt.Color)com.ibm.etools.proxy.initParser.test.NavigationParameters.getColor(\"red\")",com.ibm.etools.proxy.initParser.test.NavigationParameters.getColor("red"));
		// Test where the method is typed to return Color and we cast to the more generailized subclass of SystemColor
		testInitString("(java.awt.SystemColor)com.ibm.etools.proxy.initParser.test.NavigationParameters.getColor(\"window\")",com.ibm.etools.proxy.initParser.test.NavigationParameters.getColor("window"));					
	}
	public static void runExceptionTests() {
		// Test for badly formed expressions throwing exceptions
		System.out.println("----- Exceptions expected for create 'Too complex' error messages -----");				
		testInitString("new Integer(\"3,3\")", null, true);
		testInitString("new ABCD(4)", null, true);
		testInitString("getMethod()",null,true);
		testInitString("unresolvedLiteral",null,true);
	}
	public static void runLiteralTests() {
		System.out.println("----- Java literals ---------------------------------------------------");		
		testInitString("\"Frog\"", "Frog");
		testInitString("null", null);
		testInitString("false", Boolean.FALSE);
		testInitString("true", Boolean.TRUE);
		testInitString("Boolean.TRUE", Boolean.TRUE);
	}
	public static void runStringTests() {
		// Perform some basic tests
		System.out.println("----- Strings containing escaped characters and tokens ----------------");		
		testInitString("\"Frog/123\"", "Frog/123");
		testInitString("String.valueOf(10)", "10");
		testInitString("\"Frog(123)prince\"", "Frog(123)prince");
		testInitString("\"Frog\\\"prince\\\"123\"", "Frog\"prince\"123");
		testInitString("\"Frog\\prince\\123\"", "Frog\\prince\\123");	
		testInitString("\"\\Frog\"", "\\Frog");	
		testInitString("\"Frog\".length()", new Integer(4));
	}
	public static void runCharTests() {
		// Perform some basic tests
		System.out.println("----- char and Character---- ------------------------------------------");		
		testInitString("'a'", new Character('a'));
		testInitString("new Character('a')", new Character('a'));
		testInitString("'\"'", new Character('\"'));
		testInitString("'\\''", new Character('\''));
		testInitString("'\\\\'", new Character('\\'));	
		testInitString("'asdf'", null, true);
	}	
	public static void runJFCTests() {
		System.out.println("----- Common expressions encountered by JFC JavaBeans -----------------");		
		testInitString("new java.awt.Dimension(1,1)", new java.awt.Dimension(1, 1));
		testInitString("new java.awt.Dimension(-1,-1)", new java.awt.Dimension(-1, -1));
		testInitString("java.awt.Color.cyan", java.awt.Color.cyan);
		testInitString("java.awt.Color.cyan.toString()", java.awt.Color.cyan.toString());
		testInitString("new Integer(\"50\")", new Integer(50));
		testInitString("new Integer(50)", new Integer(50));
		testInitString("new java.awt.FlowLayout()", new java.awt.FlowLayout());
		testInitString("new java.awt.Color(10,20,30)", new java.awt.Color(10, 20, 30));
        testInitString("new javax.swing.ImageIcon( \"C:/WINNT/Zapotec.bmp\")",new javax.swing.ImageIcon( "C:/WINNT/Zapotec.bmp" ));		
		testInitString(
			"new java.awt.Rectangle( 10 , 20 , 30 , 40 )",
			new java.awt.Rectangle(10, 20, 30, 40));
		testInitString("new java.util.Locale(\"fr\",\"FR\")", new java.util.Locale("fr", "FR"));
		testInitString("new java.util.Locale(\"fr\",\"FR\",\"\")", new java.util.Locale("fr", "FR", ""));
		//Locale constructor cannot take null as argument like this: new Java.util.Locale("fr", "FR", null)
		//so this is expected to fail
		testInitString("new java.util.Locale(\"fr\",\"FR\",null)", null, true);
		testInitString("new java.awt.Dialog(new java.awt.Frame())",new java.awt.Dialog(new java.awt.Frame()));
		testInitString("new Integer( 50 ).toString().length()", new Integer(2));
		testInitString("new java.util.ArrayList().size()", new Integer(new java.util.ArrayList().size()));		
		testInitString("new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR)",new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));		
		testInitString("new java.awt.Panel(null)", new java.awt.Panel(null));		

	}
	public static void runBorderTests(){

		System.out.println("----- javax.swing.Border tests ----------------------------------------");
		testInitString(
			"javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED)" ,
			 javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));		
		testInitString(
			"javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)",
			javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
		testInitString(
			"javax.swing.BorderFactory.createEmptyBorder()",
			javax.swing.BorderFactory.createEmptyBorder());
		testInitString(
			"javax.swing.BorderFactory.createMatteBorder(2, 2, 2, 2, java.awt.Color.black)",
			javax.swing.BorderFactory.createMatteBorder(2, 2, 2, 2, java.awt.Color.black));
		testInitString(
			"new javax.swing.border.SoftBevelBorder(0)",
			new javax.swing.border.SoftBevelBorder(0));
		testInitString(
			"javax.swing.BorderFactory.createEmptyBorder(2, 2, 2, 2)",
			javax.swing.BorderFactory.createEmptyBorder(2, 2, 2, 2));
		testInitString(
			"javax.swing.BorderFactory.createMatteBorder(2, 2, 2, 2, java.awt.Color.black)",
			javax.swing.BorderFactory.createMatteBorder(2, 2, 2, 2, java.awt.Color.black));
		testInitString(
			"javax.swing.BorderFactory.createLineBorder(java.awt.Color.black, 2)",
			javax.swing.BorderFactory.createLineBorder(java.awt.Color.black, 2));
		testInitString(
			"new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED)",
			new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
		testInitString(
			"javax.swing.BorderFactory.createTitledBorder(null,\"frog\",2,0)",
			javax.swing.BorderFactory.createTitledBorder(null, "frog", 2, 0));
			
		// This is an important test to make sure that messages arguments that contain messages are process correctly
		testInitString(
			"new javax.swing.border.CompoundBorder(javax.swing.BorderFactory.createEmptyBorder(),javax.swing.BorderFactory.createEmptyBorder())",
			new javax.swing.border.CompoundBorder(javax.swing.BorderFactory.createEmptyBorder(),javax.swing.BorderFactory.createEmptyBorder()));			

	}
	public static void runOverloadingTests() {
		// Test getting correct signature out of multiple valid ones.
		// and also test gratuitous use of paranetheses within expressions
		System.out.println("----- Overloaded method signatures ------------------------------------");		
		testInitString("(short) 3", new Short((short) 3));
		testInitString("(new String(\"Frog\")).length()",new Integer(4));				
		testInitString("((new String(\"Frog\"))).length()",new Integer(4));						
		testInitString("((new String(\"Frog\")).length())",new Integer(4));						
		testInitString("(((new String(\"Frog\")).length()))",new Integer(4));												
//		testInitString("new Test2().method()", new Integer(1));
//		testInitString("(new Test2()).method((short) 3)", new Integer(2));
	}

	public static void testInitString(String aString, Object expectedResult) {
		testInitString(aString, expectedResult, false);
	}

	/**
	 * TestInitializationStringParser constructor comment.
	 */
	public static void testInitString(String aString, Object expectedResult, boolean throwsException) {

		StringBuffer buffer = new StringBuffer();
		try {
			InitializationStringParser parser = InitializationStringParser.createParser(aString);
			Object result = parser.evaluate();
			if (throwsException) {
				buffer.append(
					"FAIL$$ It should of thrown an exception. Instead result="
						+ (result != null ? result.toString() : "null"));
				failed++;
				System.out.println(buffer.toString());				
			} else {
				if (result == expectedResult) {
					buffer.append("OK - " + aString + " == " + result);
					passed++;
				} else if (
					(result == null && expectedResult != null) || (expectedResult == null && result != null)) {
					buffer.append("FAIL** - ExpectedResult=" + expectedResult + " result=" + result);
					failed++;
					failedWithoutExceptions++;
				} else if (result.equals(expectedResult)) {
					buffer.append("OK= - " + aString + " = " + result);
					passed++;
				} else {
					// It may be that the equals expression is bad.  If so use the toString() to do a partial comparison
					if (result.getClass() == expectedResult.getClass()) {
						if (result.toString().equals(expectedResult.toString())) {
							buffer.append("OK?- " + aString + " ExpectedResult=" + expectedResult + " ActualResult=" + result);
							passed++;
						} else {
							// The toStrings do not match perfectly but the classes do.
							// Try and see how close the two strings are
							if ( expectedResult.getClass().isArray() && result.getClass().isArray()){
								Class expectedResultClass = expectedResult.getClass().getComponentType();
								Class resultClass = result.getClass().getComponentType();
								int resultLength = Array.getLength(result);
								int expectedLength = Array.getLength(expectedResult);
								if ( expectedLength == resultLength ){
									if ( resultClass == expectedResultClass ) {
										buffer.append("OK?- " + aString + " ExpectedArrayDimensions=" + expectedLength + ", ActualArrayLength=" + resultLength + " ExpectedComponentType=" + expectedResultClass + ", ActualComponentType=" + resultClass);
										passedmaybe++;																	
									} else {
										buffer.append("OK?- " + aString + " ExpectedArrayDimensions=" + expectedLength + ", ActualArrayLength=" + resultLength + " ExpectedComponentType=" + expectedResultClass + ", ActualComponentType=" + resultClass);
										passedmaybe++;																											
									}
								} else {
									buffer.append(
										"FAIL** - "
											+ aString
											+ " ExpectedResult="
											+ expectedResult
											+ " ActualResult="
											+ result
											+ " ExpectedClass="
											+ expectedResult.getClass()
											+ " ActualClass="
											+ result.getClass());
											failed++;
											failedWithoutExceptions++;		
								}								
							} else {
								buffer.append(
									"OK??- "
										+ aString
										+ " ExpectedResult="
										+ expectedResult
										+ " ActualResult="
										+ result
										+ " Class= "
										+ result.getClass());
								passedmaybe++;
							}
						}
					} else {
						buffer.append(
							"FAIL** - "
								+ aString
								+ " ExpectedResult="
								+ expectedResult
								+ " ActualResult="
								+ result
								+ " ExpectedClass="
								+ expectedResult.getClass()
								+ " ActualClass="
								+ result.getClass());
						failed++;
						failedWithoutExceptions++;
					}
				}
				buffer.append(" --> is Primitive? " + parser.isPrimitive());
				System.out.println(buffer.toString());
			}
		} catch (InitializationStringEvaluationException e) {
			if (throwsException) {
				System.out.println("OK - Exception was expected for + \"" + aString + "\"");
				passed++;
			} else {
				System.out.println(
					"FAIL - "
						+ aString
						+ " Exception thrown= "
						+ e.getOriginalException().getClass()
						+ " : "
						+ e.getOriginalException().getMessage());
				failed++;
			}
		}
	}

}