package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.common.remote.*;
import com.ibm.etools.proxy.common.CommandException;

import java.io.IOException;
/**
 * Interface for the Remote VM Connection object.
 */

public interface IREMConnection {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	/**
	 * Did this construct correctly.
	 * This is needed because an exception could be thrown in the ctor and
	 * that's not a good thing to do.
	 */
	public boolean isConnected();
	
	/**
	 * Terminate the server.
	 * WARNING: This is only here so that factory can to it. Termination should ONLY
	 * be done by the Factory not any one else. 
	 */
	public void terminateServer();
			
	/**
	 * Close the connection.
	 */
	public void close();
	
	/**
	 * Get the class information. Null return if not found.
	 */
	public Commands.GetClassReturn getClass(String className) throws CommandException;

	/**
	 * Get the class information from the id. 
	 */
	 public Commands.GetClassIDReturn getClassFromID(int classID) throws CommandException;

	

	/**
	 * Get object data from an id.
	 */	
	public void getObjectData(int objectID, Commands.ValueObject valueReturn) throws CommandException;
		
	/**
	 * Get a new instance using the initialization string.
	 */
	public void getNewInstance(int classID, String initString, Commands.ValueObject newInstance) throws CommandException;
		
	/**
	 * Invoke the method call.
	 * The parms valueObject must represent an Object[] (either through ARRAY_IDS or OBJECT)
	 */	
	public void invokeMethod(int methodID, Commands.ValueObject invokeOn, Commands.ValueObject parms, Commands.ValueObject returnValue) throws CommandException;
		
	/**
	 * Release the id. It is no longer needed on the client.
	 */
	public void releaseID(int id);
	
}