package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.util.List;

import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.Plugin;

import com.ibm.etools.proxy.IClasspathContributionController;
import com.ibm.etools.proxy.ProxyPlugin;

/**
 * The plugin class for the com.ibm.etools.proxy plugin.
 */

public class ProxyRemotePlugin extends Plugin {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	private static ProxyRemotePlugin PROXY_REMOTE_PLUGIN = null;

	// Debug options
	public static final String DEBUG_VM = "/debug/debugvm", // Bring up debugger on remote vm's. //$NON-NLS-1$
		IO_CONSOLE = "/debug/ioconsole", // IO Through console (system in and out) for asking questions. //$NON-NLS-1$
		DEBUG_VM_TRACEOUT = "/debug/vmtraceout"; // Trace the output from the remote vm's. //$NON-NLS-1$

	public ProxyRemotePlugin(IPluginDescriptor pluginDescriptor) {
		super(pluginDescriptor);
		PROXY_REMOTE_PLUGIN = this;
	}

	/**
	 * Accessor method to get the singleton plugin.
	 */
	public static ProxyRemotePlugin getPlugin() {
		return PROXY_REMOTE_PLUGIN;
	}


	private REMRegistryController pluginRegistryController;
	
	/**
	 * Return the registry controller.
	 */
	public REMRegistryController getRegistryController() {
		return pluginRegistryController != null ? pluginRegistryController : (pluginRegistryController = new REMRegistryController());
	}
	
	/**
	 * Shuts down this plug-in and discards all plug-in state.
	 *
	 * In this case, terminate all of the active registries so that they can be shutdown.
	 * Don't want them hanging around after termination of the desktop.
	 *
	 * @exception CoreException if this method fails to shut down
	 *   this plug-in 
	 */
	public void shutdown() {
		getRegistryController().shutdown();
	}	
	
	/**
	 * Method to update any class paths with any
	 * paths that need to be added to a VM. In this case, it is
	 * the remotevm.jar and the remotecommon.jar that need to be added. The first jar contains
	 * the code necessary to run the remote vm and listen for commands. The second jar contains
	 * the common code that is required to be shared on both the IDE side and remote side. It
	 * is the interface code so that they both talk the same language.
	 * support.
	 */
	public void updateClassPaths(List classPaths, IClasspathContributionController controller) {
		ProxyPlugin proxyPlugin = ProxyPlugin.getPlugin();
		controller.contributeClasspath(proxyPlugin.localizeFromPlugin(this, "remotevm.jar"), classPaths, -1); //$NON-NLS-1$
		controller.contributeClasspath(proxyPlugin.localizeFromPlugin(this, "runtime/remotecommon.jar"), classPaths, -1); //$NON-NLS-1$
	}
}