package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.common.remote.Commands;
import java.math.BigDecimal;
/**
 * Remote Implementation of IBeanTypeProxy for BigDecimal.
 * Creation date: (2/6/00 9:01:40 AM)
 * @author: Joe Winchester
 */
final class REMBigDecimalBeanTypeProxy extends REMAbstractNumberBeanTypeProxy {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
// Some values are cache'd for speed
	private final INumberBeanProxy zeroProxy;
	private final static BigDecimal zeroValue = new BigDecimal(0D);

REMBigDecimalBeanTypeProxy(REMProxyFactoryRegistry aRegistry, IBeanTypeProxy numberType) {
	super(aRegistry, new Integer(Commands.BIG_DECIMAL_CLASS), BigDecimal.class.getName(), numberType);
	
	// Create the cached values
	zeroProxy = new REMBigDecimalBeanProxy(aRegistry, zeroValue);	
}

/**
 * Helper to create a bean proxy
 * Package protected because everyone should go through the factory API
 * that is defined as part of IBeanProxyFactory
 */
INumberBeanProxy createBigDecimalBeanProxy(BigDecimal aBigDecimal) {
	return (!zeroValue.equals(aBigDecimal)) ?
		new REMBigDecimalBeanProxy(fRegistry, aBigDecimal) : zeroProxy;
}

/**
 * Generic way to create an BigDecimal proxy, however, the Number must be a BigDecimal.
 */
protected INumberBeanProxy createNumberBeanProxy(Number aNumber) {
	return createBigDecimalBeanProxy((BigDecimal) aNumber);
}

/**
 * newInstance method. Number's don't have a default ctor, so the standard way won't work.
 * Return the zero proxy for this case.
 */
public IBeanProxy newInstance() {
	return zeroProxy;
}

/**
 * Create a new bean proxy the ValueObject passed in.
 * We will assume the object is correct type.
 */
public IBeanProxy newBeanProxy(Commands.ValueObject value) {
	return createBigDecimalBeanProxy((BigDecimal) value.anObject);
}	

}


