package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.common.remote.Commands;
import com.ibm.etools.proxy.*;
/**
 * Remote Implementation of IBooleanBeanProxy for Boolean.TYPE.
 * Creation date: (2/6/00 9:02:54 AM)
 * @author: Joe Winchester
 */
final class REMBooleanTypeBeanProxy extends REMConstantBeanProxy implements IBooleanBeanProxy {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	private final boolean fBoolean;

/**
 * Store the boolean value so that we can return the booleanValue faster
 * without the need for going across the line. It is package protected because it is created in
 * special way, so no one else should create these.
 * @param aBean java.lang.Object
 */
REMBooleanTypeBeanProxy(REMProxyFactoryRegistry aRegistry, boolean value) {
	super(aRegistry);
	fBoolean = value;
}

/**
 * equals: Equal if:
 *         1) This proxy == (identity) to the other object
 *         2) Else if other is an IBeanProxy and not a constant one, then if
 *            equals on the server.
 *         3) If this is a constant proxy and the other is too or is a constant
 *            value (e.g. IStringBeanProxy.equals(String), then true if values are equals.
 */
public boolean equals(Object anObject) {
	if (this == anObject)
		return true;	// Identity
	if (anObject instanceof REMBooleanTypeBeanProxy)
		return fBoolean == ((REMBooleanTypeBeanProxy) anObject).booleanValue();
	if (anObject instanceof Boolean)
		return fBoolean == ((Boolean) anObject).booleanValue();
	return false;
}

/**
 * Return the boolean value
 */
public boolean booleanValue() {
	return fBoolean;
}
/**
 * Return the java.lang.Boolean value
 */
public Boolean getBooleanValue() {
	return fBoolean ? Boolean.TRUE : Boolean.FALSE;
}

/**
 * Return the toString of the bean itself.
 */
public String toBeanString() {
	return fBoolean ? Boolean.TRUE.toString() : Boolean.FALSE.toString();
}

/**
 * Get the beantype
 */
public IBeanTypeProxy getTypeProxy() {
	return ((REMStandardBeanTypeProxyFactory) fFactory.getBeanTypeProxyFactory()).booleanType;
}

/**
 * Render the bean into value object.
 */
public void renderBean(Commands.ValueObject value) {
	value.set(fBoolean);
}
}


