package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.common.remote.Commands;
/**
 * IDE Implementation of ICharacterBeanProxy for the Character class.
 * Creation date: (2/6/00 9:02:54 AM)
 * @author: Joe Winchester
 */
final class REMCharacterClassBeanProxy extends REMConstantBeanProxy implements ICharacterBeanProxy {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected Character fCharValue;

/**
 * As well as storing the bean store the char value so that we can return the charValue faster
 * without the need for repeated re-casting. It is package protected because it is created in
 * special way, so no one else should create these.
 * @param aBean java.lang.Object
 */
REMCharacterClassBeanProxy(REMProxyFactoryRegistry aRegistry, Character value) {
	super(aRegistry);
	fCharValue = value;
}

/**
 * equals: Equal if:
 *         1) This proxy == (identity) to the other object
 *         2) Else if other is an IBeanProxy and not a constant one, then if
 *            equals on the server.
 *         3) If this is a constant proxy and the other is too or is a constant
 *            value (e.g. IStringBeanProxy.equals(String), then true if values are equals.
 */
public boolean equals(Object anObject) {
	if (this == anObject)
		return true;	// Identity
	if (anObject instanceof REMCharacterClassBeanProxy)
		return fCharValue.charValue() == ((REMCharacterClassBeanProxy) anObject).charValue();
	if (anObject instanceof Character)
		return fCharValue.charValue() == ((Character) anObject).charValue();
	return false;
}

/**
 * Return the char value
 */
public char charValue() {
	return fCharValue.charValue();
}
/**
 * Return the java.lang.Character value
 */
public Character characterValue() {
	return fCharValue;
}

/**
 * Answer the toString() of the value.
 */
public String toBeanString() {
	return fCharValue.toString();
}

/**
 * Get the beantype
 */
public IBeanTypeProxy getTypeProxy() {
	return ((REMStandardBeanTypeProxyFactory) fFactory.getBeanTypeProxyFactory()).characterClass;
}

/**
 * Render the bean into value object.
 */
public void renderBean(Commands.ValueObject value) {
	value.set(fCharValue);
}
}


