package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.common.remote.Commands;
import com.ibm.etools.proxy.*;
/**
 * Remote implementation of ICharacterBeanProxy.
 * Creation date: (2/6/00 9:02:54 AM)
 * @author: Joe Winchester
 */
final class REMCharacterTypeBeanProxy extends REMConstantBeanProxy implements ICharacterBeanProxy {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected char fChar;

REMCharacterTypeBeanProxy(REMProxyFactoryRegistry aRegistry, char value) {
	super(aRegistry);
	fChar = value;
}

/**
 * equals: Equal if:
 *         1) This proxy == (identity) to the other object
 *         2) Else if other is an IBeanProxy and not a constant one, then if
 *            equals on the server.
 *         3) If this is a constant proxy and the other is too or is a constant
 *            value (e.g. IStringBeanProxy.equals(String), then true if values are equals.
 */
public boolean equals(Object anObject) {
	if (this == anObject)
		return true;	// Identity
	if (anObject instanceof REMCharacterTypeBeanProxy)
		return fChar == ((REMCharacterTypeBeanProxy) anObject).charValue();
	if (anObject instanceof Character)
		return fChar == ((Character) anObject).charValue();
	return false;
}

/**
 * Return the char value
 */
public char charValue() {
	return fChar;
}
/**
 * Return the java.lang.Character value
 */
public Character characterValue() {
	return new Character(fChar);
}

/**
 * Answer the toString() of the value.
 */
public String toBeanString() {
	return String.valueOf(fChar);
}

/**
 * Get the beantype
 */
public IBeanTypeProxy getTypeProxy() {
	return ((REMStandardBeanTypeProxyFactory) fFactory.getBeanTypeProxyFactory()).charType;
}

/**
 * Render the bean into value object.
 */
public void renderBean(Commands.ValueObject value) {
	value.set(fChar);
}
}


