package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import org.eclipse.core.runtime.*;
import com.ibm.etools.logging.util.Level;
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.common.remote.*;
/**
 * Proxy for a constructor.
 */

final class REMConstructorProxy extends REMAbstractBeanProxy implements IConstructorProxy {


	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
REMConstructorProxy(REMProxyFactoryRegistry aRegistry, Integer anID){
	super(aRegistry, anID);
}

/**
 * Using this constructor (assuming it takes no arguments) create a new instance.
 * Exception will be raised if this constructor takes arguments.
 */
public IBeanProxy newInstance() throws ThrowableProxy {
	return newInstance(null);
}

/**
 * The type proxy is constant proxy out of the method factory.
 */
public IBeanTypeProxy getTypeProxy() {
	return ((REMMethodProxyFactory) fFactory.getMethodProxyFactory()).ctorType;
}

/**
 * Using this constructor (with the arguments) create a new instance.
 * Exception will be raised if arguments don't match the constructor.
 */
public IBeanProxy newInstance(IBeanProxy[] creationArguments) throws ThrowableProxy {
	IREMMethodProxy newInstanceMethod = (IREMMethodProxy) REMStandardBeanProxyConstants.getConstants(fFactory).getConstructorNewInstance();
	
	// Constructor.newInstance takes one parm (Object[]) where [0] is an Object[] of the arguments to the constructor.
	Object[] parms = null;
	if (creationArguments != null)
		parms = new Object[] {new TransmitableArray(Commands.OBJECT_CLASS, creationArguments)};
		
	return newInstanceMethod.invokeWithParms(this, parms);
}
public IBeanProxy newInstanceCatchThrowableExceptions(){
	try {
		return newInstance();
	} catch ( ThrowableProxy e ) {
		ProxyPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, ProxyRemotePlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, "", e));		 //$NON-NLS-1$
		fFactory.releaseProxy(e);	// Since it's no longer needed, get rid of now instead of GC time.
		return null;		
	}
}
public IBeanProxy newInstanceCatchThrowableExceptions(IBeanProxy[] args){
	try {
		return newInstance(args);
	} catch ( ThrowableProxy e ) {
		ProxyPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, ProxyRemotePlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, "", e));		 //$NON-NLS-1$
		fFactory.releaseProxy(e);	// Since it's no longer needed, get rid of now instead of GC time.
		return null;		
	}
}
}