package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
 
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.common.remote.Commands;

/**
 * A special bean type proxy. It is for those bean types that had an
 * instantiation error while being initialized. It is so we have
 * the message. All other actions against it will fail.
 * @author richkulp
 */
public class REMInitErrorBeanTypeProxy implements IREMBeanTypeProxy {

	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected final String initializationError;
	protected final REMProxyFactoryRegistry registry;
	protected final String classname;
	
	/**
	 * Constructor for REMInitErrorBeanTypeProxy.
	 */
	public REMInitErrorBeanTypeProxy(REMProxyFactoryRegistry registry, String initializationError, String classname) {
		super();
		this.registry = registry;
		this.initializationError = initializationError;
		this.classname = classname;
	}

	/**
	 * @see com.ibm.etools.proxy.remote.IREMBeanTypeProxy#newBeanProxy(Integer)
	 */
	public IREMBeanProxy newBeanProxy(Integer anID) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.remote.IREMBeanTypeProxy#newBeanTypeForClass(Integer, String, boolean)
	 */
	public IREMBeanTypeProxy newBeanTypeForClass(Integer anID, String aClassname, boolean anAbstract) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getConstructorProxy(String[])
	 */
	public IConstructorProxy getConstructorProxy(String[] argumentClassNames) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getConstructorProxy(IBeanTypeProxy[])
	 */
	public IConstructorProxy getConstructorProxy(IBeanTypeProxy[] argumentTypes) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getFieldProxy(String)
	 */
	public IFieldProxy getFieldProxy(String fieldName) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getMethodProxy(String)
	 */
	public IMethodProxy getMethodProxy(String methodName) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getMethodProxy(String, String[])
	 */
	public IMethodProxy getMethodProxy(String methodName, String[] argumentClassNames) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getMethodProxy(String, String)
	 */
	public IMethodProxy getMethodProxy(String methodName, String argumentClassName) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getMethodProxy(String, IBeanTypeProxy[])
	 */
	public IMethodProxy getMethodProxy(String methodName, IBeanTypeProxy[] argumentTypes) {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getNullConstructorProxy()
	 */
	public IConstructorProxy getNullConstructorProxy() {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanProxy#getProxyFactoryRegistry()
	 */
	public ProxyFactoryRegistry getProxyFactoryRegistry() {
		return registry;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getSuperBeanTypeProxy()
	 */
	public IBeanTypeProxy getSuperBeanTypeProxy() {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getTypeName()
	 */
	public String getTypeName() {
		return classname;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getFormalTypeName()
	 */
	public String getFormalTypeName() {
		return getTypeName();
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#isArray()
	 */
	public boolean isArray() {
		return false;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#isInterface()
	 */
	public boolean isInterface() {
		return false;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#isPrimitive()
	 */
	public boolean isPrimitive() {
		return false;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#isKindOf(IBeanTypeProxy)
	 */
	public boolean isKindOf(IBeanTypeProxy aBeanProxyType) {
		return false;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#newInstance()
	 */
	public IBeanProxy newInstance() throws ThrowableProxy {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#newInstance(String)
	 */
	public IBeanProxy newInstance(String initializationString) throws ThrowableProxy, ClassCastException, InstantiationException {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanTypeProxy#getInitializationError()
	 */
	public String getInitializationError() {
		return initializationError;
	}

	/**
	 * @see com.ibm.etools.proxy.remote.IREMBeanProxy#getID()
	 */
	public Integer getID() {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.remote.IREMBeanProxy#release()
	 */
	public void release() {
	}

	/**
	 * @see com.ibm.etools.proxy.remote.IREMBeanProxy#renderBean(ValueObject)
	 */
	public void renderBean(Commands.ValueObject renderInto) {
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanProxy#getTypeProxy()
	 */
	public IBeanTypeProxy getTypeProxy() {
		return null;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanProxy#isNullProxy()
	 */
	public boolean isNullProxy() {
		return false;
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanProxy#toBeanString()
	 */
	public String toBeanString() {
		return ""; //$NON-NLS-1$
	}

	/**
	 * @see com.ibm.etools.proxy.IBeanProxy#isValid()
	 */
	public boolean isValid() {
		return false;
	}

}
