package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.common.remote.Commands;
import com.ibm.etools.proxy.*;
/**
 * Remote Implementation of IIntegerBeanProxy for the primitive "int".
 * It can answer all of the primitive number types (int, long, short, byte, double, float).
 * Creation date: (2/6/00 9:02:54 AM)
 * @author: Joe Winchester
 */
class REMIntegerTypeBeanProxy extends REMConstantBeanProxy implements IIntegerBeanProxy {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	private final int fInt;

/**
 * It is package protected because they are created
 * in a special way and no one else should create them.
 * @param aBean java.lang.Object
 */
REMIntegerTypeBeanProxy(REMProxyFactoryRegistry aRegistry, int anInt) {
	super(aRegistry);
	fInt = anInt;
}
/**
 * equals: Equal if:
 *         1) This proxy == (identity) to the other object
 *         2) Else if other is an IBeanProxy and not a constant one, then if
 *            equals on the server.
 *         3) If this is a constant proxy and the other is too or is a constant
 *            value (e.g. IStringBeanProxy.equals(String), then true if values are equals.
 */
public boolean equals(Object anObject) {
	if (this == anObject)
		return true;	// Identity
	if (anObject instanceof REMIntegerTypeBeanProxy)
		return fInt == ((REMIntegerTypeBeanProxy) anObject).intValue();
	if (anObject instanceof Integer)
		return fInt == ((Integer) anObject).intValue();
	return false;
}
/**
 * byteValue method comment.
 */
public byte byteValue() {
	return (byte) fInt;
}
/**
 * doubleValue method comment.
 */
public double doubleValue() {
	return (double) fInt;
}
/**
 * floatValue method comment.
 */
public float floatValue() {
	return (float) fInt;
}
/**
 * Return the int value
 */
public int intValue() {
	return fInt;
}
/**
 * longValue method comment.
 */
public long longValue() {
	return (long) fInt;
}
/**
 * numberValue method comment.
 */
public Number numberValue() {
	return new Integer(fInt);
}
/**
 * shortValue method comment.
 */
public short shortValue() {
	return (short) fInt;
}

/**
 * The bean's toString.
 */
public String toBeanString() {
	return String.valueOf(fInt);
}

/**
 * Get the beantype
 */
public IBeanTypeProxy getTypeProxy() {
	return ((REMStandardBeanTypeProxyFactory) fFactory.getBeanTypeProxyFactory()).intType;
}

/**
 * Render the bean into value object.
 */
public void renderBean(Commands.ValueObject value) {
	value.set(fInt);
}
}


