package com.ibm.etools.proxy.remote;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.proxy.common.remote.Commands;
import com.ibm.etools.proxy.*;
/**
 * Proxy for "null" value.
 */

final class REMNullBeanProxy extends REMConstantBeanProxy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	REMNullBeanProxy(REMProxyFactoryRegistry aFactory) {
		super(aFactory);
	}
	/**
	 * equals: Equal if:
	 *         1) This proxy == (identity) to the other object
	 *         2) Else if other is an IBeanProxy and not a constant one, then if
	 *            equals on the server.
	 *         3) If this is a constant proxy and the other is too or is a constant
	 *            value (e.g. IStringBeanProxy.equals(String), then true if values are equals.
	 * Since there is only one NullProxy then only if identity or anObject is null.
	 */
	public boolean equals(Object anObject) {
		return (anObject == null || this == anObject);	// Identity or anObject is null

	}		
	public boolean isNullProxy() {
		return true;
	}
	
	public String toBeanString() {
		return "null"; //$NON-NLS-1$
	}
	
	/**
	 * Get the beantype
	 */
	public IBeanTypeProxy getTypeProxy() {
		return ((REMStandardBeanTypeProxyFactory) fFactory.getBeanTypeProxyFactory()).voidType;
	}
	
	/**
	 * Render the bean into value object.
	 */
	public void renderBean(Commands.ValueObject value) {
		value.set();
	}	

}