package com.ibm.etools.proxy.remote.awt;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.proxy.remote.*;
import com.ibm.etools.proxy.IStandardBeanTypeProxyFactory;
import com.ibm.etools.proxy.IBeanTypeProxy;
/**
 * BeanType factory standard AWT bean types.
 * This is package protected because it shouldn't be
 * referenced outside the package. It should only be accessed through
 * the interface.
 */
class REMStandardAWTBeanTypeProxyFactory implements IREMBeanTypeProxyFactory {

	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	static final String BEAN_TYPE_FACTORY_KEY = "java.awt"; //$NON-NLS-1$
	
	protected final REMProxyFactoryRegistry fFactoryRegistry;
	
	REMStandardAWTBeanTypeProxyFactory(REMProxyFactoryRegistry aRegistry) {
		fFactoryRegistry = aRegistry;	
		fFactoryRegistry.registerBeanTypeProxyFactory(BEAN_TYPE_FACTORY_KEY, this);
	}
	
	/**
	 * We don't pre-cache any types. We need to have the
	 * id from the server to create types.
	 */
	public IREMBeanTypeProxy getExtensionBeanTypeProxy(String className){
		return null;
	}
	
	/**
	 * Create the correct beantype from the class and id passed in.
	 */
	public IREMBeanTypeProxy getExtensionBeanTypeProxy(String className, Integer classID, IBeanTypeProxy superType){
		
		if ("java.awt.Dimension".equals(className)) //$NON-NLS-1$
			return new REMDimensionBeanTypeProxy(fFactoryRegistry, classID, className, superType);
		else if ("java.awt.Point".equals(className)) //$NON-NLS-1$
			return new REMPointBeanTypeProxy(fFactoryRegistry, classID, className, superType);			
		else if ("java.awt.Rectangle".equals(className)) //$NON-NLS-1$
			return new REMRectangleBeanTypeProxy(fFactoryRegistry, classID, className, superType);
		else 
			return null;
	}
	
	/**
	 * Terminate this factory. Since it doesn't hold onto anything other than the registry,
	 * and nothing will be holding onto this factory, nothing needs to be done. It will be GC'd.
	 */
	public void terminateFactory() {
	}	

}