package com.ibm.etools.proxy.remote.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import org.eclipse.core.runtime.*;
import com.ibm.etools.proxy.remote.*;
import com.ibm.etools.proxy.*;
import org.eclipse.core.boot.IPlatformRunnable;
import org.eclipse.core.resources.*;
import java.net.URL;
import com.ibm.etools.proxy.test.JavaProjectUtil;
/**
 * This is an application extension to allow test of the remote vm within the
 * workbench environment.
 *
 * The path to the rt.jar to use is set by the argument "-jdk fully-qualified-rt.jar". The default will use
 * rt.jar that started the application.
 *
 * This is only a test class to see if things are set up correctly.
 * Because this should be run only under the supervision or request
 * of an IBM tech support, it is not NLSed.
 */
public class TestRemoteVM implements IPlatformRunnable {

	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
public Object run(Object p1) throws CoreException {	
	System.out.println("-- Initializing the test data --"); //$NON-NLS-1$
	final IProject[] project = new IProject[] {JavaProjectUtil.getProject("Test Remote Proxy")}; //$NON-NLS-1$
	try {
	// Delete/create/populate within a runnable so that the build is done only after all changes have been made.
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				// First create/recreate the project.
				if (project[0] != null)
					project[0].delete(true, true, monitor);	// Get rid of it, we want a clean one for testing.
				
				try {
					// Now import the requested files.
					JavaProjectUtil.importZipURLs(new Path("/Test Remote Proxy"), new URL[] {new URL(ProxyRemotePlugin.getPlugin().getDescriptor().getInstallURL(), "testdata/testproject-src.jar")}, monitor); //$NON-NLS-1$ //$NON-NLS-2$
					project[0] = JavaProjectUtil.getProject("Test Remote Proxy"); //$NON-NLS-1$
				} catch (java.net.MalformedURLException e) {
				}
			}
		}, null);
		System.out.println("-- Data initialized --"); //$NON-NLS-1$
	
		System.out.println("-- Starting the remote vm --"); //$NON-NLS-1$
		REMProxyFactoryRegistry registry = (REMProxyFactoryRegistry) ProxyPlugin.getPlugin().startImplementation(project[0], "Test", null, new NullProgressMonitor() ); //$NON-NLS-1$
		if (registry != null) {
			try {
				com.ibm.etools.proxy.test.TestProxy.testProxy(registry);
				testProjectAccess(registry);
			} finally {
				registry.terminateRegistry();
			}
		} else
			System.out.println("*** No remote VM was created for some reason. ***"); //$NON-NLS-1$
	} finally {
		if (project[0] != null)
			project[0].delete(true, false, null);	// Get rid of the project and the files themselves.
		ResourcesPlugin.getWorkspace().save(true, null);	// Save the current state of the workspace.
	}
	return null;
}

private void testProjectAccess(ProxyFactoryRegistry registry) {
	IStandardBeanTypeProxyFactory proxyTypeFactory = registry.getBeanTypeProxyFactory();
	IStandardBeanProxyFactory proxyFactory = registry.getBeanProxyFactory();

	System.out.println("--------- Starting Workspace Project Access tests ---------"); //$NON-NLS-1$
	try {	
		IBeanTypeProxy testClassType = proxyTypeFactory.getBeanTypeProxy("com.ibm.etools.proxy.remote.testing.TestClass"); //$NON-NLS-1$
		if (testClassType == null)
			System.out.println("Failed. TestClass should of been found."); //$NON-NLS-1$
		else {
			IBeanProxy testClass = testClassType.newInstance();
			if (testClass == null)
				System.out.println("Failed. TestClass should of been instantiated."); //$NON-NLS-1$
			else {
				IMethodProxy testMethod = testClassType.getMethodProxy("getTestString"); //$NON-NLS-1$
				if (testMethod == null)
					System.out.println("Failed. getTestString method should of been found."); //$NON-NLS-1$
				else {
					IStringBeanProxy aString = (IStringBeanProxy) testMethod.invoke(testClass);
					if (aString == null)
						System.out.println("Failed. getTestString should not of returned null."); //$NON-NLS-1$
					else
						if (!("TESTSTRING".equals(aString.stringValue()))) //$NON-NLS-1$
							System.out.println("Failed. string should of been \"TESTSTRING\" not \""+aString.stringValue()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
	} catch (Throwable e) {
		System.out.println("Failed. Unexpected exception thrown."); //$NON-NLS-1$
		e.printStackTrace();
	}
	System.out.println("--------- Stopping Workspace Project Access tests ---------");	 //$NON-NLS-1$
}	

}