package com.ibm.etools.proxy.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.io.File;
import java.io.FilenameFilter;
import java.net.URL;
import java.util.zip.ZipFile;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.ui.dialogs.IOverwriteQuery;
import org.eclipse.ui.wizards.datatransfer.ImportOperation;
import org.eclipse.ui.wizards.datatransfer.ZipFileStructureProvider;
/**
 * The purpose of this class is to be a utility for manipulating/populating java projects.
 *  1) Get a java project as one of the resources.
 *  2) Import zip files into workspace, and create a project from the zip file. The zipfile needs to contain a .project file.
 */

public class JavaProjectUtil {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	
	public static IProject getProject(String projectName) throws CoreException {
		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
		return (project.exists()) ? project: null;
	}
	
	
	/**
	 * Import the specified zipfiles (the entire contents of each), given by URLs, into a project.
	 * Returns true if worked, false if any failed or op cancelled.
	 * The URLs must be either FILE type or they must be EclipseURLConnection types.
	 * The projectPath is relative to the workspace root.
	 * 
	 * NOTE: If the project doesn't exist, it will exist by the time this is finished. So don't try
	 * to create the project later.
	 */
	public static boolean importZipURLs(IPath projectPath, URL[] zipFileURLs, IProgressMonitor monitor) {
if (false && System.getenv("Really should redesign this so that I do the import so that we don't need UI for ImportOperation.") != null) ;		 //$NON-NLS-1$
		for (int i=0; i<zipFileURLs.length; i++) {
			try {
				String zipFileName = Platform.asLocalURL(zipFileURLs[i]).getFile();
				ZipFileStructureProvider zipProvider = new ZipFileStructureProvider(new ZipFile(zipFileName));
				try {
					ImportOperation op = new ImportOperation(
						projectPath,
						zipProvider.getRoot(),
						zipProvider,
						new IOverwriteQuery() {
							public String queryOverwrite(String pathString) {
								return IOverwriteQuery.ALL;
							}
						});
					op.setCreateContainerStructure(true);
					op.setOverwriteResources(true);
					op.run(monitor);
				} catch (InterruptedException e) {						
					return false;
				} catch (java.lang.reflect.InvocationTargetException e) {
					return false;
				} finally {
					try {
						zipProvider.getZipFile().close();
					} catch (java.io.IOException e) {
					}
				}
			} catch (java.io.IOException e) {
				return false;
			}
		}
		return true;		
	}
	

	/**
	 * Create a project from the path. The path will be a complete project, including a .project file.
	 * The path should be from the workspace root.
	 */
	public static IProject createProject(IWorkspace workspace, IPath projectPath, final IProgressMonitor pm) throws CoreException {
		IProjectDescription newDescription = null;
		
		File projectDir = workspace.getRoot().getLocation().append(projectPath).toFile();
		File projectFile = null;
		if (projectDir.exists() && projectDir.isDirectory()) {
			File files[] = projectDir.listFiles(new FilenameFilter() {
				/**
				 * @see java.io.FilenameFilter#accept(File, String)
				 */
				public boolean accept(File dir, String name) {
					return name.equals(IProjectDescription.DESCRIPTION_FILE_NAME);
				}
			});
			if (files == null || files.length == 0)
				return null;
			projectFile = files[0];
		}


		IPath projectFilePath = new Path(projectFile.getPath());
		try {
			newDescription = workspace.loadProjectDescription(projectFilePath);
		} catch (CoreException exception) {
			//no good couldn't get the name
			exception.printStackTrace();
			return null;
		}
		
		// create the new project operation
		final IProject project = workspace.getRoot().getProject(newDescription.getName());		
		final IProjectDescription description = newDescription;
		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {		
			public void run(IProgressMonitor monitor) throws CoreException {
				project.create(description, new SubProgressMonitor(monitor, 1000));
				project.open(new SubProgressMonitor(monitor, 1000));				
			}
		}, pm);
		
		return project;
	}
		
}