package com.ibm.etools.proxy.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


/**
 * Test the standard AWT proxy support to see that everything works correctly.
 * we're testing the interface to it. To test other VM's
 * you would send in the appropriate registry.
 *
 * This is only a test class to see if things are set up correctly.
 * Because this should be run only under the supervision or request
 * of an IBM tech support, it is not NLSed.
 */
 
import com.ibm.etools.proxy.*;
import com.ibm.etools.proxy.awt.*;

public class TestAwtProxy {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
public static void testAwtProxy(ProxyFactoryRegistry registry) {
	IStandardBeanTypeProxyFactory proxyTypeFactory = registry.getBeanTypeProxyFactory();
	IStandardBeanProxyFactory proxyFactory = registry.getBeanProxyFactory();
	
	JavaStandardAwtBeanConstants jc = JavaStandardAwtBeanConstants.getConstants(registry);
	if (!jc.isAWTAvailable()) {
		System.out.println("**** AWT is not available in the VM. It was specified that it should be. Cannot run the awt tests."); //$NON-NLS-1$
		return;
	}

	System.out.println("--------- Starting standard awt tests ---------"); //$NON-NLS-1$
	try {		
		IBeanTypeProxy colorType = proxyTypeFactory.getBeanTypeProxy("java.awt.Color"); //$NON-NLS-1$
		if (colorType == null)
			System.out.println("Failed test 1.1"); //$NON-NLS-1$
		else {
			IBeanProxy colorProxy = colorType.newInstance("java.awt.Color.cyan"); //$NON-NLS-1$
			if (colorProxy == null)
				System.out.println("Failed test 1.2"); //$NON-NLS-1$
			else {
				if (!"java.awt.Color[r=0,g=255,b=255]".equals(colorProxy.toBeanString())) //$NON-NLS-1$
					System.out.println("Failed test 1.3: toString=\""+colorProxy.toBeanString()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		
		IStandardAwtBeanProxyFactory awtProxyFactory = (IStandardAwtBeanProxyFactory) registry.getBeanProxyFactoryExtension(IStandardAwtBeanProxyFactory.REGISTRY_KEY);
		if (awtProxyFactory == null)
			System.out.println("Failed test 2.1"); //$NON-NLS-1$
		else {
			// Test the creation through the helpers
			IDimensionBeanProxy dimBean = awtProxyFactory.createDimensionBeanProxyWith(5,6);
			if (dimBean == null)
				System.out.println("Failed test 2.2"); //$NON-NLS-1$
			else {
				if (!"java.awt.Dimension[width=5,height=6]".equals(dimBean.toBeanString())) //$NON-NLS-1$
					System.out.println("Failed test 2.3: toString=\""+dimBean.toBeanString()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
			}
			IPointBeanProxy pointBean = awtProxyFactory.createPointBeanProxyWith(5,6);
			if (pointBean == null)
				System.out.println("Failed test 2.4"); //$NON-NLS-1$
			else {
				if (!"java.awt.Point[x=5,y=6]".equals(pointBean.toBeanString())) //$NON-NLS-1$
					System.out.println("Failed test 2.5: toString=\""+pointBean.toBeanString()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
			}
			IRectangleBeanProxy rectBean = awtProxyFactory.createBeanProxyWith(5,6,7,8);
			if (rectBean == null)
				System.out.println("Failed test 2.6"); //$NON-NLS-1$
			else {
				if (!"java.awt.Rectangle[x=5,y=6,width=7,height=8]".equals(rectBean.toBeanString())) //$NON-NLS-1$
					System.out.println("Failed test 2.7: toString=\""+rectBean.toBeanString()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
			
		// Test the creation through init strings.
		IBeanTypeProxy dimType = proxyTypeFactory.getBeanTypeProxy("java.awt.Dimension"); //$NON-NLS-1$
		if (dimType == null)
			System.out.println("Failed test 3.1"); //$NON-NLS-1$
		else {
			IDimensionBeanProxy dimProxy = (IDimensionBeanProxy) dimType.newInstance("new java.awt.Dimension(7,8)"); //$NON-NLS-1$
			if (dimProxy == null)
				System.out.println("Failed test 3.2"); //$NON-NLS-1$
			else
				if (!"java.awt.Dimension[width=7,height=8]".equals(dimProxy.toBeanString())) //$NON-NLS-1$
					System.out.println("Failed test 3.3: toString=\""+dimProxy.toBeanString()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
				else {
					// Test the Proxy special methods. 
					int width = dimProxy.getWidth();
					if (width != 7)
						System.out.println("Failed test 3.4"); //$NON-NLS-1$
				}
		}
		IBeanTypeProxy pointType = proxyTypeFactory.getBeanTypeProxy("java.awt.Point"); //$NON-NLS-1$
		if (pointType == null)
			System.out.println("Failed test 4.1"); //$NON-NLS-1$
		else {
			IPointBeanProxy pointProxy = (IPointBeanProxy) pointType.newInstance("new java.awt.Point(9,8)"); //$NON-NLS-1$
			if (pointProxy == null)
				System.out.println("Failed test 4.2"); //$NON-NLS-1$
			else {
				if (!"java.awt.Point[x=9,y=8]".equals(pointProxy.toBeanString())) //$NON-NLS-1$
					System.out.println("Failed test 4.3: toString=\""+pointProxy.toBeanString()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
			}		
		}
		IBeanTypeProxy rectType = proxyTypeFactory.getBeanTypeProxy("java.awt.Rectangle"); //$NON-NLS-1$
		if (rectType == null)
			System.out.println("Failed test 5.1"); //$NON-NLS-1$
		else {
			IRectangleBeanProxy rectProxy = (IRectangleBeanProxy) rectType.newInstance("new java.awt.Rectangle(7,8,9,1)"); //$NON-NLS-1$
			if (rectProxy == null)
				System.out.println("Failed test 5.2"); //$NON-NLS-1$
			else {
				if (!"java.awt.Rectangle[x=7,y=8,width=9,height=1]".equals(rectProxy.toBeanString())) //$NON-NLS-1$
					System.out.println("Failed test 5.3: toString=\""+rectProxy.toBeanString()+"\""); //$NON-NLS-1$ //$NON-NLS-2$
			}		
		}
	} catch (ThrowableProxy e) {
		System.out.println("-------- Exception on Remote VM occurred."); //$NON-NLS-1$
		e.printStackTrace();
	} catch (InstantiationException e) {
		e.printStackTrace();
	}
			
	System.out.println("--------- Ending standard awt tests ---------");	 //$NON-NLS-1$
		
}

}