package com.ibm.etools.proxy.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.proxy.*;
import java.io.InputStream;
import java.io.IOException;
/**
 * This is the callback for the test case of the stream.
 */
public class TestCallbackStream implements ICallback {

	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	Object tester;	// Notified when the test is finished.
	
	public TestCallbackStream(Object o) {
		tester = o;
	}
	
	public Object calledBack(int msgID, IBeanProxy parm){
		System.out.println("Failed Callback Test. Shouldn't have gotton to parm"); //$NON-NLS-1$
		stopTest();
		return null;
	}
	public Object calledBack(int msgID, Object[] parms){
		System.out.println("Failed Callback Test. Shouldn't have gotton to multi-parms"); //$NON-NLS-1$
		stopTest();		
		return null;
	}
	public void calledBackStream(int msgID, InputStream in){
		// Read from the stream for 30000 bytes. Verify that
		// we get an incrementing stream, i.e. 0, 1, ..., 127, -127, ...
		
		for (int i=0; i<30000; i++) {
			try {
				int next = in.read();
				if (next == -1) {
					System.out.println("Failed Callback Stream test. Stream ended too soon."); //$NON-NLS-1$
					stopTest();
					return;
				}
				
				if (((byte) next) != ((byte) i)) {
					System.out.println("Failed Callback Stream test. Did not get correct next byte at:" + i); //$NON-NLS-1$
					stopTest();
					return;
				}
			} catch (IOException e) {
				e.printStackTrace();
				stopTest();				
				return;
			}
		}
		
		try {
			if (in.read() != -1) {
				System.out.println("Failed Callback Stream test. Too much data sent."); //$NON-NLS-1$
				stopTest();				
				return;
			}
			System.out.println("Successful Callback Stream test."); //$NON-NLS-1$
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		stopTest();
	}
	
	void stopTest() {
		synchronized(tester) {
			tester.notify();
		}
	}
		

}