package com.ibm.etools.proxy.test;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


/**
 * Test the proxy support to see that everything works correctly.
 * we're testing the interface to it. To test other VM's
 * you would send in the appropriate registry.
 *
 * This is only a test class to see if things are set up correctly.
 * Because this should be run only under the supervision or request
 * of an IBM tech support, it is not NLSed.
 */
 
import com.ibm.etools.proxy.*;

public class TestProxy {

	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
public static void testProxy(ProxyFactoryRegistry registry) {
	IStandardBeanTypeProxyFactory proxyTypeFactory = registry.getBeanTypeProxyFactory();
	IStandardBeanProxyFactory proxyFactory = registry.getBeanProxyFactory();

	System.out.println("--------- Starting standard tests ---------"); //$NON-NLS-1$
	try {	
		IBeanTypeProxy objectType = proxyTypeFactory.getBeanTypeProxy("java.lang.Object"); //$NON-NLS-1$
		if (objectType == null || objectType.isArray())
			System.out.println("Failed test 1.1"); //$NON-NLS-1$
		else if (!objectType.getTypeName().equals("java.lang.Object")) //$NON-NLS-1$
			System.out.println("Failed test 1.2"); //$NON-NLS-1$
			
		IBeanTypeProxy integerType = proxyTypeFactory.getBeanTypeProxy("java.lang.Integer"); //$NON-NLS-1$
		if (integerType == null)
			System.out.println("Failed test 2.1"); //$NON-NLS-1$
		else if (!integerType.getTypeName().equals("java.lang.Integer")) //$NON-NLS-1$
			System.out.println("Failed test 2.2");		 //$NON-NLS-1$
		else if (!integerType.isKindOf(objectType))
			System.out.println("Failed test 2.3"); //$NON-NLS-1$
		else {
			IMethodProxy mthd = integerType.getMethodProxy("valueOf", "java.lang.String"); //$NON-NLS-1$ //$NON-NLS-2$
			if (mthd == null)
				System.out.println("Failed test 2.4"); //$NON-NLS-1$
			else {
				// See if we can invoke methods.
				IIntegerBeanProxy anInt = (IIntegerBeanProxy) mthd.invoke(proxyFactory.getNullBeanProxy(), proxyFactory.createBeanProxyWith("5")); //$NON-NLS-1$
				if (anInt == null)
					System.out.println("Failed test 2.5"); //$NON-NLS-1$
				else
					if (anInt.intValue() != 5)
						System.out.println("Failed test 2.6"); //$NON-NLS-1$
						
				// See if invoke with bad type throws the ExceptionProxy.
				try {
					mthd.invoke(proxyFactory.getNullBeanProxy(), proxyFactory.createBeanProxyWith(5));
					System.out.println("Failed test 2.6a"); //$NON-NLS-1$
				} catch (ThrowableProxy e) {
					// We should of gotton the exception. See if it is of the correct type.
					if (!e.getTypeProxy().getTypeName().equals("java.lang.IllegalArgumentException")) //$NON-NLS-1$
						System.out.println("Failed test 2.6b"); //$NON-NLS-1$
				}						
						
				// See if we can create it from an initialization string.
				anInt = (IIntegerBeanProxy) integerType.newInstance("new java.lang.Integer(6)"); //$NON-NLS-1$
				if (anInt == null)
					System.out.println("Failed test 2.7"); //$NON-NLS-1$
				else
					if (anInt.intValue() != 6)
						System.out.println("Failed test 2.8");  //$NON-NLS-1$
				// See if string initialization works.
				IStringBeanProxy aString = (IStringBeanProxy) proxyTypeFactory.getBeanTypeProxy("java.lang.String").newInstance("\"abcd\""); //$NON-NLS-1$ //$NON-NLS-2$
				if (aString == null)
					System.out.println("Failed test 2.7a"); //$NON-NLS-1$
				else
					if (!"abcd".equals(aString.stringValue())) //$NON-NLS-1$
						System.out.println("Failed test 2.8a");  //$NON-NLS-1$
						
				// See if we can create it from a constructor.
				IConstructorProxy ctor = integerType.getConstructorProxy(new String[] {"java.lang.String"}); //$NON-NLS-1$
				if (ctor == null)
					System.out.println("Failed test 2.9"); //$NON-NLS-1$
				else {
					anInt = (IIntegerBeanProxy) ctor.newInstance(new IBeanProxy[] {proxyFactory.createBeanProxyWith("0")}); //$NON-NLS-1$
					if (anInt.intValue() != 0)
						System.out.println("Failed test 2.10"); //$NON-NLS-1$
						
					// Zero is cached, let's see if that is what we got above.
					IIntegerBeanProxy aZero = proxyFactory.createBeanProxyWith(new Integer(0));
					if (anInt != aZero)
						System.out.println("Failed test 2.11"); //$NON-NLS-1$
				}
			}
			
			// See that we get a primitive back when the method return type is a primitive.
			IMethodProxy mthdPrim = integerType.getMethodProxy("intValue"); //$NON-NLS-1$
			IIntegerBeanProxy anInt = (IIntegerBeanProxy) mthdPrim.invoke(proxyFactory.createBeanProxyWith(5));
			if (!anInt.getTypeProxy().getTypeName().equals("int")) //$NON-NLS-1$
				System.out.println("Failed test 3"); //$NON-NLS-1$
			IFieldProxy fieldPrim = integerType.getFieldProxy("MIN_VALUE"); //$NON-NLS-1$
			anInt = (IIntegerBeanProxy) fieldPrim.get(null);
			if (!anInt.getTypeProxy().getTypeName().equals("int")) //$NON-NLS-1$
				System.out.println("Failed test 4");		 //$NON-NLS-1$
		}
		

	
		IArrayBeanTypeProxy arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("[I"); //$NON-NLS-1$
		if (arrayType == null || !arrayType.isArray())
			System.out.println("Failed test 5.1"); //$NON-NLS-1$
		else {
			if (!arrayType.getTypeName().equals("[I")) //$NON-NLS-1$
				System.out.println("Failed test 5.2"); //$NON-NLS-1$
			else {
				if (arrayType.getComponentType() != proxyTypeFactory.getBeanTypeProxy("int")) //$NON-NLS-1$
					System.out.println("Failed test 5.3: "+arrayType.getComponentType()); //$NON-NLS-1$
			}
		}
		
		arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("[Ljava.lang.Integer;"); //$NON-NLS-1$
		if (arrayType == null || !arrayType.isArray())
			System.out.println("Failed test 6.1"); //$NON-NLS-1$
		else {
			if (!arrayType.getTypeName().equals("[Ljava.lang.Integer;")) //$NON-NLS-1$
				System.out.println("Failed test 6.2"); //$NON-NLS-1$
			else {
				if (arrayType.getComponentType() != proxyTypeFactory.getBeanTypeProxy("java.lang.Integer")) //$NON-NLS-1$
					System.out.println("Failed test 6.3: "+arrayType.getComponentType()); //$NON-NLS-1$
			}
		}
		
		arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("[[I"); //$NON-NLS-1$
		if (arrayType == null || !arrayType.isArray())
			System.out.println("Failed test 7.1"); //$NON-NLS-1$
		else {
			if (!arrayType.getTypeName().equals("[[I")) //$NON-NLS-1$
				System.out.println("Failed test 7.2"); //$NON-NLS-1$
			else {
				if (arrayType.getComponentType() != proxyTypeFactory.getBeanTypeProxy("[I")) //$NON-NLS-1$
					System.out.println("Failed test 7.3: "+arrayType.getComponentType()); //$NON-NLS-1$
			}
		}
		
		arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("int", 2); //$NON-NLS-1$
		if (arrayType == null || !arrayType.isArray())
			System.out.println("Failed test 8.1"); //$NON-NLS-1$
		else {
			if (!arrayType.getTypeName().equals("[[I")) //$NON-NLS-1$
				System.out.println("Failed test 7.2"); //$NON-NLS-1$
		}
		
		arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("java.lang.Integer", 2); //$NON-NLS-1$
		if (arrayType == null || !arrayType.isArray())
			System.out.println("Failed test 8.1"); //$NON-NLS-1$
		else {
			if (!arrayType.getTypeName().equals("[[Ljava.lang.Integer;")) //$NON-NLS-1$
				System.out.println("Failed test 8.2"); //$NON-NLS-1$
		}
		
		arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("[Ljava.lang.Integer;", 2); //$NON-NLS-1$
		if (arrayType == null || !arrayType.isArray())
			System.out.println("Failed test 9.1"); //$NON-NLS-1$
		else {
			if (!arrayType.getTypeName().equals("[[[Ljava.lang.Integer;")) //$NON-NLS-1$
				System.out.println("Failed test 9.2"); //$NON-NLS-1$
		}			
	
		IArrayBeanProxy arrayProxy = proxyFactory.createBeanProxyWith(integerType, new int[] {3, 4});
		if (arrayProxy == null)
			System.out.println("Failed test 10.1"); //$NON-NLS-1$
		else if (!arrayProxy.getTypeProxy().getTypeName().equals("[[Ljava.lang.Integer;")) //$NON-NLS-1$
			System.out.println("Failed test 10.2: "+arrayProxy.getTypeProxy().getTypeName()); //$NON-NLS-1$
		else {
			if (arrayProxy.getLength() != 3)
				System.out.println("Failed test 10.3"); //$NON-NLS-1$
			IArrayBeanProxy idx1 = (IArrayBeanProxy) arrayProxy.get(0);
			if (idx1 == null || idx1.getLength() != 4)
				System.out.println("Failed test 10.4"); //$NON-NLS-1$
		}
	
		arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("java.lang.Integer", 1); //$NON-NLS-1$
		arrayProxy = proxyFactory.createBeanProxyWith(arrayType, new int[] {1});
		if (arrayProxy == null)
			System.out.println("Failed test 11.1"); //$NON-NLS-1$
		else if (!arrayProxy.getTypeProxy().getTypeName().equals("[Ljava.lang.Integer;")) //$NON-NLS-1$
			System.out.println("Failed test 11.2: "+arrayProxy.getTypeProxy().getTypeName()); //$NON-NLS-1$
		else if (arrayProxy.getLength() != 1)
			System.out.println("Failed test 11.3"); //$NON-NLS-1$
	
		arrayProxy = proxyFactory.createBeanProxyWith(arrayType, new int[] {2 ,3});
		if (arrayProxy == null)
			System.out.println("Failed test 12.1"); //$NON-NLS-1$
		else if (!arrayProxy.getTypeProxy().getTypeName().equals("[[Ljava.lang.Integer;")) //$NON-NLS-1$
			System.out.println("Failed test 12.2: "+arrayProxy.getTypeProxy().getTypeName()); //$NON-NLS-1$
		else {
			if (arrayProxy.getLength() != 2)
				System.out.println("Failed test 12.3"); //$NON-NLS-1$
			IArrayBeanProxy idx1 = (IArrayBeanProxy) arrayProxy.get(0);
			if (idx1 == null || idx1.getLength() != 3)
				System.out.println("Failed test 12.4"); //$NON-NLS-1$
			IBeanProxy idx2 = arrayProxy.get(new int [] {1,2});
			if (idx2 != null)
				System.out.println("Failed test 12.5"); //$NON-NLS-1$
			IBeanProxy anInt = proxyFactory.createBeanProxyWith(new java.lang.Integer(23));
			// Test set into array idx1, but retrieve from arrayProxy. Tests
			//   1) Set with one index
			//   2) Get with multi index
			//   3) And that the same array is set that exists within the first array.
			idx1.set(anInt, 2);
			IBeanProxy idx3 = arrayProxy.get(new int[] {0, 2});
			if (!idx3.getTypeProxy().getTypeName().equals("java.lang.Integer")) //$NON-NLS-1$
				System.out.println("Failed test 12.6"); //$NON-NLS-1$
			if (!anInt.equals(idx3))
				System.out.println("Failed test 12.7"); //$NON-NLS-1$
			// Inverse test of above, multi set, single get of same array.
			arrayProxy.set(anInt, new int[] {0,1});
			if (!anInt.equals(idx1.get(1)))
				System.out.println("Failed test 12.8"); //$NON-NLS-1$
		}		
			
		arrayProxy = proxyFactory.createBeanProxyWith(arrayType, null);
		if (arrayProxy == null)
			System.out.println("Failed test 13.1"); //$NON-NLS-1$
		else if (!arrayProxy.getTypeProxy().getTypeName().equals("[Ljava.lang.Integer;")) //$NON-NLS-1$
			System.out.println("Failed test 13.2: "+arrayProxy.getTypeProxy().getTypeName()); //$NON-NLS-1$
		else if (arrayProxy.getLength() != 0)
			System.out.println("Failed test 13.3");		 //$NON-NLS-1$
	
		arrayProxy = proxyFactory.createBeanProxyWith(proxyTypeFactory.getBeanTypeProxy("java.lang.Integer", 2), null); //$NON-NLS-1$
		if (arrayProxy == null)
			System.out.println("Failed test 14.1"); //$NON-NLS-1$
		else if (!arrayProxy.getTypeProxy().getTypeName().equals("[[Ljava.lang.Integer;")) //$NON-NLS-1$
			System.out.println("Failed test 14.2: "+arrayProxy.getTypeProxy().getTypeName()); //$NON-NLS-1$
		else if (arrayProxy.getLength() != 0)
			System.out.println("Failed test 14.3");		 //$NON-NLS-1$
			
			
		arrayProxy = proxyFactory.createBeanProxyWith(proxyTypeFactory.getBeanTypeProxy("java.lang.Integer", 2), 3); //$NON-NLS-1$
		if (arrayProxy == null)
			System.out.println("Failed test 15.1"); //$NON-NLS-1$
		else if (!arrayProxy.getTypeProxy().getTypeName().equals("[[Ljava.lang.Integer;")) //$NON-NLS-1$
			System.out.println("Failed test 15.2: "+arrayProxy.getTypeProxy().getTypeName()); //$NON-NLS-1$
		else {
			if (arrayProxy.getLength() != 3)
				System.out.println("Failed test 15.3"); //$NON-NLS-1$
			IArrayBeanProxy idx1 = (IArrayBeanProxy) arrayProxy.get(0);
			if (idx1 != null)
				System.out.println("Failed test 15.4"); //$NON-NLS-1$
		}		
			
			
			
		// Make sure that if the final component type is an
		// int (primitive) that we get a primitive proxy back and not an Integer object.		
		arrayType = (IArrayBeanTypeProxy) proxyTypeFactory.getBeanTypeProxy("int", 1); //$NON-NLS-1$
		arrayProxy = proxyFactory.createBeanProxyWith(arrayType, 3);
		arrayProxy.set(proxyFactory.createBeanProxyWith(44), 1);
		IBeanProxy idx1 = arrayProxy.get(1);
		if (!idx1.getTypeProxy().getTypeName().equals("int")) //$NON-NLS-1$
			System.out.println("Fails test 16"); //$NON-NLS-1$
		
		// Now test the Collection interface.
		IBeanTypeProxy vectorType = proxyTypeFactory.getBeanTypeProxy("java.util.Vector"); //$NON-NLS-1$
		IBeanProxy vectorProxy = vectorType.newInstance();
		CollectionBeanProxyWrapper col = new CollectionBeanProxyWrapper(vectorProxy);
		if (!col.add(proxyFactory.createBeanProxyWith(33)))
			System.out.println("Failed test 17.1"); //$NON-NLS-1$
		if (col.size() != 1)
			System.out.println("Failed test 17.2"); //$NON-NLS-1$
		IArrayBeanProxy arrayOut = col.toArray();
		if (((IIntegerBeanProxy) arrayOut.get(0)).intValue() != 33)
			System.out.println("Failed test 17.3"); //$NON-NLS-1$
		arrayOut = proxyFactory.createBeanProxyWith(proxyTypeFactory.getBeanTypeProxy("java.lang.Integer"), 2); //$NON-NLS-1$
		IArrayBeanProxy arrayOut2 = col.toArray(arrayOut);
		if (((IIntegerBeanProxy) arrayOut2.get(0)).intValue() != 33)
			System.out.println("Failed test 17.3"); //$NON-NLS-1$
		// It should of been the exact same array (arrayOut == arrayOut2 under the proxies.
		if (((IIntegerBeanProxy) arrayOut.get(0)).intValue() != 33)
			System.out.println("Failed test 17.4"); //$NON-NLS-1$
					
		// See if the iterator interface works.
		IteratorBeanProxyWrapper itrProxy = col.iterator();
		if (itrProxy == null)
			System.out.println("Failed test 17.5"); //$NON-NLS-1$
		int iterations = 0;
		while (itrProxy.hasNext()) {
			iterations++;
			IBeanProxy itrValue = itrProxy.next();
			if (((IIntegerBeanProxy) itrValue).intValue() != 33)
				System.out.println("Failed test 17.6"); //$NON-NLS-1$
		}
		if (iterations != 1)
			System.out.println("Failed test 17.7");	// Shouldn't of been more than one entry. //$NON-NLS-1$
	
		try {
			// Test the setting with wrong type throws an exception
			col = new CollectionBeanProxyWrapper(arrayProxy);
			System.out.println("Failed test 18"); //$NON-NLS-1$
		} catch (ClassCastException e) {
			// If here, then the test was passed
		}
		
		// See if the list inteface works.
		ListBeanProxyWrapper list = new ListBeanProxyWrapper(vectorProxy);
		list.add(0, proxyFactory.createBeanProxyWith(true));
		if (!((IBooleanBeanProxy) list.get(0)).booleanValue())
			System.out.println("Failed test 18.1"); //$NON-NLS-1$
		
		// See if the ListIterator interface works
		ListIteratorBeanProxyWrapper lItrProxy = list.listIterator();
		if (lItrProxy == null)
			System.out.println("Failed test 19.1"); //$NON-NLS-1$
		if (!lItrProxy.hasNext())
			System.out.println("Failed test 19.2"); //$NON-NLS-1$
		IBooleanBeanProxy bool1 = (IBooleanBeanProxy) lItrProxy.next();
		if (!bool1.booleanValue())
			System.out.println("Failed test 19.3"); //$NON-NLS-1$
		IIntegerBeanProxy int1 = (IIntegerBeanProxy) lItrProxy.next();
		if (int1.intValue() != 33)
			System.out.println("Failed test 19.4"); //$NON-NLS-1$
		if (lItrProxy.hasNext())
			System.out.println("Failed test 19.5"); //$NON-NLS-1$
		if (lItrProxy.previousIndex() != 1)
			System.out.println("Failed test 19.6"); //$NON-NLS-1$
			
		// Test able to access beantype proxy and invoke methods on it. This isn't
		// the approved way of getting methods, but it tests that method proxies
		// are created correctly if returned from an invoke method.
		IBeanTypeProxy classTypeProxy = proxyTypeFactory.getBeanTypeProxy("java.lang.Class"); //$NON-NLS-1$
		IMethodProxy getMethodMethod = classTypeProxy.getMethodProxy("getMethod", new String[] {"java.lang.String", "[Ljava.lang.Class;"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		IBeanProxy method = getMethodMethod.invoke(integerType, new IBeanProxy[] {
			proxyFactory.createBeanProxyWith("byteValue"), proxyFactory.getNullBeanProxy()}); //$NON-NLS-1$
		IBeanProxy byteValue = ((IMethodProxy) method).invoke(proxyFactory.createBeanProxyWith(new Integer(254)));
		if (byteValue == null)
			System.out.println("Failed test 20.1"); //$NON-NLS-1$
		else {
			if (!byteValue.getTypeProxy().getTypeName().equals("byte")) //$NON-NLS-1$
				System.out.println("Failed test 20.2"); //$NON-NLS-1$
			else if (((INumberBeanProxy) byteValue).byteValue() !=(byte)254)
				System.out.println("Failed test 20.3"); //$NON-NLS-1$
		}	
		
		System.out.println("--- Starting the callback test ---"); //$NON-NLS-1$
		IBeanTypeProxy callbackType = proxyTypeFactory.getBeanTypeProxy("com.ibm.etools.proxy.remote.test.TestCallback"); //$NON-NLS-1$
		if (callbackType != null) {
			Object testObject = new Object();
			IBeanProxy callbackProxy = callbackType.newInstance();
			TestCallback cb = new TestCallback(testObject);
			registry.getCallbackRegistry().registerCallback(callbackProxy, cb);
			IMethodProxy start = callbackType.getMethodProxy("start"); //$NON-NLS-1$
			IMethodProxy stop = callbackType.getMethodProxy("stop");							 //$NON-NLS-1$
			synchronized(testObject) {
				start.invokeCatchThrowableExceptions(callbackProxy);
				try {
					testObject.wait(30000);	// Should be done by 30 seconds.
				} catch (InterruptedException e) {
				}
			}				
			stop.invokeCatchThrowableExceptions(callbackProxy);
			registry.getCallbackRegistry().deregisterCallback(callbackProxy);				
			System.out.println("If there were no callback messages, then the test failed."); //$NON-NLS-1$
			System.out.println("If there is anything in the .log file, then the test failed.");			 //$NON-NLS-1$
		} else
			System.out.println("Failed callback test. Callback class not found."); //$NON-NLS-1$
			
		System.out.println("--- Starting the callback stream test ---"); //$NON-NLS-1$
		callbackType = proxyTypeFactory.getBeanTypeProxy("com.ibm.etools.proxy.remote.test.TestCallbackStream"); //$NON-NLS-1$
		if (callbackType != null) {
			Object testObject = new Object();
			IBeanProxy callbackProxy = callbackType.newInstance();
			TestCallbackStream cb = new TestCallbackStream(testObject);
			registry.getCallbackRegistry().registerCallback(callbackProxy, cb);
			IMethodProxy start = callbackType.getMethodProxy("start");							 //$NON-NLS-1$
			synchronized(testObject) {			
				start.invokeCatchThrowableExceptions(callbackProxy);
			try {
					testObject.wait(30000);	// Should be done by 30 seconds.
				} catch (InterruptedException e) {
				}	
			}		
			registry.getCallbackRegistry().deregisterCallback(callbackProxy);				
			System.out.println("If there were no callback messages, then the test failed."); //$NON-NLS-1$
			System.out.println("If there is anything in the .log file, then the test failed.");			 //$NON-NLS-1$
		} else
			System.out.println("Failed callback stream test. Callback class not found."); //$NON-NLS-1$


	} catch (ThrowableProxy e) {
		System.out.println("-------- Exception on Remote VM occurred."); //$NON-NLS-1$
		e.printStackTrace();
	} catch (InstantiationException e) {
		e.printStackTrace();
	}
			
	System.out.println("--------- Ending standard tests ---------"); //$NON-NLS-1$
	
	if (com.ibm.etools.proxy.awt.JavaStandardAwtBeanConstants.getConstants(registry).isAWTRegistered()) {
		TestAwtProxy.testAwtProxy(registry);
	}
}

}